@@ -123,6 +123,7 @@ DHTDBImpl
 		SimpleTimer.addPeriodicEvent(
 			"DHTDB:op",
 			original_republish_interval,
+			true, // absolute, we don't want effective time changes (computer suspend/resume) to shift these
 			new TimerEventPerformer()
 			{
 				public void
@@ -149,6 +150,7 @@ DHTDBImpl
 		SimpleTimer.addPeriodicEvent(
 				"DHTDB:cp",
 				cache_republish_interval + 10000 - (int)(Math.random()*20000),
+				true,	// absolute, we don't want effective time changes (computer suspend/resume) to shift these
 				new TimerEventPerformer()
 				{
 					public void
@@ -393,8 +395,6 @@ DHTDBImpl
 			
 			for (int i=0;i<values.length;i++){
 				
-				DHTTransportValue	t_value = values[i];
-								
 					// last check, verify that the contact is who they say they are, only for non-forwards
 					// as cache forwards are only accepted if they are "close enough" and we can't 
 					// rely on their identify due to the way that cache republish works (it doesn't
@@ -1396,7 +1396,7 @@ DHTDBImpl
 		
 		ip_filter.ban( 
 				contact.getAddress().getAddress().getHostAddress(),
-				"DHT: Sender stored excessive entries at this node (" + reason + ")" );		
+				"DHT: Sender stored excessive entries at this node (" + reason + ")", false );		
 	}
 	
 	protected void
