@@ -1,7 +1,7 @@
 /*
  * Created on 03-Aug-2004
  * Created by Paul Gardner
- * Copyright (C) 2004 Aelitis, All Rights Reserved.
+ * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * 
- * AELITIS, SARL au capital de 30,000 euros
+ * AELITIS, SAS au capital de 46,603.30 euros
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  *
  */
@@ -27,14 +27,26 @@ package com.aelitis.azureus.core.diskmanager.cache;
  *
  */
 
+import java.io.File;
+
 import org.gudy.azureus2.core3.torrent.TOTorrentFile;
 
 public interface 
 CacheFileOwner 
 {
+	public static final int	CACHE_MODE_NORMAL			= 1;
+	public static final int	CACHE_MODE_NO_CACHE			= 2;
+	public static final int	CACHE_MODE_EXPERIMENTAL		= 3;
+	
 	public String
 	getCacheFileOwnerName();
 	
 	public TOTorrentFile
 	getCacheFileTorrentFile();
+	
+	public File
+	getCacheFileControlFileDir( );
+	
+	public int
+	getCacheMode();
 }
