@@ -30,10 +30,13 @@ package com.aelitis.azureus.core.diskmanager.cache.impl;
 import java.io.File;
 import java.util.*;
 
-import org.gudy.azureus2.core3.torrent.*;
+import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.logging.LogEvent;
+import org.gudy.azureus2.core3.logging.LogIDs;
+import org.gudy.azureus2.core3.logging.Logger;
+import org.gudy.azureus2.core3.torrent.TOTorrent;
+import org.gudy.azureus2.core3.torrent.TOTorrentFile;
 import org.gudy.azureus2.core3.util.*;
-import org.gudy.azureus2.core3.config.*;
-import org.gudy.azureus2.core3.logging.*;
 
 import com.aelitis.azureus.core.diskmanager.cache.*;
 import com.aelitis.azureus.core.diskmanager.file.*;
@@ -83,9 +86,8 @@ CacheFileManagerImpl
 	
 	protected CacheFileManagerStatsImpl	stats;
 	
-		// copy on update semantics
-	
-	protected Map	torrent_to_cache_file_map	= new HashMap();
+
+	protected Map	torrent_to_cache_file_map	= new LightHashMap();
 	
 	protected long				cache_bytes_written;
 	protected long				cache_bytes_read;
@@ -217,10 +219,9 @@ CacheFileManagerImpl
 							return( owner.getCacheFileTorrentFile());
 						}
 						public File
-						getControlFile(
-							String	name )
+						getControlFileDir( )
 						{
-							return( owner.getCacheFileControlFile( name ));
+							return( owner.getCacheFileControlFileDir( ));
 						}
 					}, file,
 					type==CacheFile.CT_LINEAR?FMFile.FT_LINEAR:FMFile.FT_COMPACT );
@@ -229,7 +230,13 @@ CacheFileManagerImpl
 			
 			CacheFile	cf;
 			
-			if (( tf != null && tf.getLength() < cache_files_not_smaller_than  ) || !cache_enabled ){ 
+			int	cache_mode = owner.getCacheMode();
+			
+			if ( cache_mode == CacheFileOwner.CACHE_MODE_EXPERIMENTAL ){
+				
+				cf = new CacheFileWithoutCacheMT( this, fm_file, tf );
+
+			}else if (( tf != null && tf.getLength() < cache_files_not_smaller_than  ) || !cache_enabled || cache_mode == CacheFileOwner.CACHE_MODE_NO_CACHE ){ 
 				
 				cf = new CacheFileWithoutCache( this, fm_file, tf );
 				
@@ -249,13 +256,8 @@ CacheFileManagerImpl
 					updated_cache_files.put( cf, null );
 										
 					if ( tf != null ){
-		
-									
-						Map	new_map = new HashMap( torrent_to_cache_file_map );
-								
-						new_map.put( tf, cf );
-				
-						torrent_to_cache_file_map	= new_map;
+
+						torrent_to_cache_file_map.put( tf, cf );
 					}	
 				}finally{
 					
@@ -798,76 +800,97 @@ CacheFileManagerImpl
 			try{
 				this_mon.enter();
 						
-				Map	new_map = new HashMap( torrent_to_cache_file_map );
-				
-				new_map.remove( tf );
-	
-				torrent_to_cache_file_map	= new_map;
+				torrent_to_cache_file_map.remove( tf );
 				
 			}finally{
-				
 				this_mon.exit();
 			}
 		}
 	}
 	
-	protected long
-	getBytesInCache(
-		TOTorrent		torrent,
-		int				piece_number,
-		int				offset,
-		long			length )
-	{
-			// copied on update, grab local ref to access
-		
-		Map	map = torrent_to_cache_file_map;
-		
+    protected boolean[] getBytesInCache(TOTorrent torrent, long[] absoluteOffsets, long[] lengths)
+    {
+    	//sanity checks
+    	if(absoluteOffsets.length != lengths.length)
+    		throw new IllegalArgumentException("Offsets/Lengths mismatch");
+    	long prevEnding = 0;
+    	for(int i = 0;i<lengths.length;i++)
+    	{
+    		if(absoluteOffsets[i]<prevEnding || lengths[i] <= 0 )
+    			throw new IllegalArgumentException("Offsets/Lengths are not in ascending order");
+    		prevEnding = absoluteOffsets[i]+lengths[i];
+    	}
+    	
+    	
 		TOTorrentFile[]	files = torrent.getFiles();
+		long[] fileOffsets = new long[files.length]; 
+	
+		boolean[] results = new boolean[absoluteOffsets.length];
+		Arrays.fill(results, true); // assume everything to be cached, then check for the opposite 
 		
-		long	piece_size = torrent.getPieceLength();
+		final long first = absoluteOffsets[0];
+		final long last = absoluteOffsets[absoluteOffsets.length-1]+lengths[lengths.length-1];
+		long fileOffset = 0;
+		int firstFile = -1;
+		boolean lockAcquired = false;
 		
-		long	target_start 	= piece_number*piece_size + offset;
-		long	target_end		= target_start + length;
+		Map localCacheMap = new LightHashMap();
 		
-		long	pos = 0;
-		
-		long	result	= 0;
-		
-		for (int i=0;i<files.length;i++){
-			
-			TOTorrentFile	tf = files[i];
-			
-			long	len = tf.getLength();
-			
-			long	this_start 	= pos;
-			
-			pos	+= len;
-			
-			long	this_end	= pos;
+		try {
+			for(int i=0;i<files.length;i++)
+			{
+				TOTorrentFile	tf = files[i];
+				long length = tf.getLength();
+				fileOffsets[i] = fileOffset;
+				if(firstFile == -1 && fileOffset <= first && first < fileOffset+length)
+				{
+					firstFile = i;
+					this_mon.enter();
+					lockAcquired = true;
+				}
 				
-			if ( this_end <= target_start ){
-				
-				continue;
-			}
-			
-			if ( target_end <= this_start ){
-				
-				break;
-			}
-			
-			long	bit_start	= target_start>this_start?target_start:this_start;
-			long	bit_end		= target_end<this_end?target_end:this_end;
-			
-			CacheFileWithCache	cache_file = (CacheFileWithCache)map.get( tf );
-			
-			if ( cache_file != null ){
-				
-				result	+= cache_file.getBytesInCache( bit_start - this_start, bit_end - bit_start );
+				if(fileOffset > last)
+					break;
+
+				if(lockAcquired)
+				{
+					CacheFileWithCache	cache_file = (CacheFileWithCache)torrent_to_cache_file_map.get( tf );
+					localCacheMap.put(tf, cache_file);
+				}
+
+				fileOffset += length;
 			}
+		} finally {
+			if(lockAcquired)
+				this_mon.exit();
 		}
 		
-		return( result );
-	}
+
+		
+		for (int i=firstFile;-1 < i && i <files.length;i++){
+			TOTorrentFile	tf = files[i];
+			CacheFileWithCache	cache_file = (CacheFileWithCache)localCacheMap.get( tf );
+			
+			long length = tf.getLength();
+			
+			fileOffset = fileOffsets[i];
+			if(fileOffset > last)
+				break;
+			
+			if(cache_file != null)
+				cache_file.getBytesInCache(results,absoluteOffsets,lengths);
+			else // we have no cache file and thus no cache entries
+				for(int j=0;j<results.length;j++) // check if any chunks fall into this non-file
+					if((absoluteOffsets[j] < fileOffset+length && absoluteOffsets[j] > fileOffset) || (absoluteOffsets[j]+lengths[j] < fileOffset+length &&  absoluteOffsets[j]+lengths[j] > fileOffset))
+						results[j] = false; // no file -> no cache entry
+		}
+		
+		if(!lockAcquired) // never found a matching torrentfile
+			Arrays.fill(results, false);
+		
+
+    	return results;
+    }
 	
 	protected void
 	rethrow(
@@ -895,54 +918,59 @@ CacheFileManagerImpl
 		try{
 			writer.indent();
 			
+			Iterator it;
+			
+				// grab a copy to avoid potential deadlock as we never take the manager monitor
+				// and then the file's own monitor, always the other way around
+			
 			try{
 				this_mon.enter();
 
-				writer.println( "Entries = " + cache_entries.size());
-				
-				Iterator it = cache_entries.keySet().iterator();
-				
-				Set	files = new HashSet();
-				
-				while( it.hasNext()){
-					
-					CacheEntry	entry = (CacheEntry)it.next();
-					
-					CacheFileWithCache file = entry.getFile();
-					
-					if( !files.contains( file )){
-						
-						files.add( file );
-						
-						TOTorrentFile torrentFile = file.getTorrentFile();
-						String fileLength = "";
-						try {
-							fileLength = "" + file.getLength();
-						} catch (Exception e) {
-							if (torrentFile != null)
-								fileLength = "" + torrentFile.getLength();
-						}
-						String	hash = "<unknown>";
-						
-						try{
-							if (torrentFile != null)
-								hash = ByteFormatter.encodeString( torrentFile.getTorrent().getHash());
-							
-						}catch( Throwable e ){
-						}
-						
-						String name = file.getName();
+				it = new ArrayList( cache_entries.keySet()).iterator();
 
-						writer.println("File: " + Debug.secretFileName(name) + ", size "
-								+ fileLength + ", torrent " + hash + ", access = "
-								+ file.getAccessMode());
-					}
-				}
-				
 			}finally{
 				
 				this_mon.exit();
 			}
+			
+			writer.println( "Entries = " + cache_entries.size());
+							
+			Set	files = new HashSet();
+			
+			while( it.hasNext()){
+				
+				CacheEntry	entry = (CacheEntry)it.next();
+				
+				CacheFileWithCache file = entry.getFile();
+				
+				if( !files.contains( file )){
+					
+					files.add( file );
+					
+					TOTorrentFile torrentFile = file.getTorrentFile();
+					String fileLength = "";
+					try {
+						fileLength = "" + file.getLength();
+					} catch (Exception e) {
+						if (torrentFile != null)
+							fileLength = "" + torrentFile.getLength();
+					}
+					String	hash = "<unknown>";
+					
+					try{
+						if (torrentFile != null)
+							hash = ByteFormatter.encodeString( torrentFile.getTorrent().getHash());
+						
+					}catch( Throwable e ){
+					}
+					
+					String name = file.getName();
+
+					writer.println("File: " + Debug.secretFileName(name) + ", size "
+							+ fileLength + ", torrent " + hash + ", access = "
+							+ file.getAccessMode());
+				}
+			}
 		}finally{
 			
 			writer.exdent();
