@@ -30,13 +30,18 @@ package com.aelitis.azureus.core.diskmanager.cache.impl;
 import java.io.File;
 import java.util.*;
 
-import org.gudy.azureus2.core3.util.*;
-import org.gudy.azureus2.core3.torrent.*;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
-import org.gudy.azureus2.core3.logging.*;
+import org.gudy.azureus2.core3.logging.LogEvent;
+import org.gudy.azureus2.core3.logging.LogIDs;
+import org.gudy.azureus2.core3.logging.Logger;
+import org.gudy.azureus2.core3.torrent.TOTorrent;
+import org.gudy.azureus2.core3.torrent.TOTorrentFile;
+import org.gudy.azureus2.core3.util.*;
 
-import com.aelitis.azureus.core.diskmanager.cache.*;
-import com.aelitis.azureus.core.diskmanager.file.*;
+import com.aelitis.azureus.core.diskmanager.cache.CacheFile;
+import com.aelitis.azureus.core.diskmanager.cache.CacheFileManagerException;
+import com.aelitis.azureus.core.diskmanager.file.FMFile;
+import com.aelitis.azureus.core.diskmanager.file.FMFileManagerException;
 
 public class 
 CacheFileWithCache 
@@ -65,11 +70,9 @@ CacheFileWithCache
 				long	offset2 = o2.getFilePosition();
 				int		length2	= o2.getLength();
 				
-				if (	offset1 + length1 <= offset2 ||
-						offset2 + length2 <= offset1 ){
-					
+				if (offset1 + length1 <= offset2 || offset2 + length2 <= offset1 ||  length1 == 0 || length2 == 0)
+				{				
 				}else{
-					
 					Debug.out( "Overlapping cache entries - " + o1.getString() + "/" + o2.getString());
 				}
 		   	
@@ -102,7 +105,7 @@ CacheFileWithCache
 	protected TOTorrent      			torrent;
 	protected long						file_offset_in_torrent;
 	
-	protected long[]					read_history		= new long[ READAHEAD_HISTORY ];
+	protected long[]					read_history; // lazy allocation
 	protected int						read_history_next	= 0;
 	
 	protected TreeSet					cache			= new TreeSet(comparator);
@@ -138,8 +141,6 @@ CacheFileWithCache
 		manager		= _manager;
 		file		= _file;
 				
-		Arrays.fill( read_history, -1 );
-		
 		if ( _torrent_file != null ){
 			
 			torrent_file	= _torrent_file;
@@ -255,7 +256,7 @@ CacheFileWithCache
 	
 		try{
 			if ( manager.isCacheEnabled()){
-			
+				
 				if (TRACE)
 					Logger.log(new LogEvent(torrent, LOGID, "readCache: " + getName()
 							+ ", " + file_position + " - "
@@ -282,6 +283,13 @@ CacheFileWithCache
 				try{
 						
 					this_mon.enter();
+					
+					
+					if(read_history == null)
+					{
+						read_history = new long[ READAHEAD_HISTORY ];
+						Arrays.fill( read_history, -1 );
+					}
 	
 						// record the position of the byte *following* the end of this read
 					
@@ -412,7 +420,8 @@ CacheFileWithCache
 							boolean	do_read_ahead	= 
 										i == 0 &&		// first time round
 										!recursive &&
-										!disable_read_cache && 
+										!disable_read_cache &&
+										read_history != null &&
 										manager.isReadCacheEnabled() &&
 										read_length <  current_read_ahead_size &&
 										file_position + current_read_ahead_size <= file.getLength();
@@ -1166,54 +1175,98 @@ CacheFileWithCache
 		flushOldDirtyData( oldest_dirty_time, -1 );
 	}
 	
-	protected long
+	protected void
 	getBytesInCache(
-		long	offset,
-		long	length )
+		boolean[] toModify,
+		long[]	absoluteOffsets,
+		long[]	lengths )
 	{
-		try{
-			this_mon.enter();
-			
-			long	result	= 0;
-			
-			Iterator	it = cache.iterator();
-			
-			long	start_pos	= offset;
-			long	end_pos		= offset + length;
-			
-			while( it.hasNext()){
-			
-				CacheEntry	entry = (CacheEntry)it.next();
-				
-				long	this_start 		= entry.getFilePosition();
-				int		entry_length	= entry.getLength();
-				
-				long	this_end	= this_start + entry_length;
-				
-				if ( this_end <= start_pos ){
-					
-					continue;
-				}
-				
-				if ( end_pos <= this_start ){
-					
-					break;
-				}
-				
-				long	bit_start	= start_pos<this_start?this_start:start_pos;
-				long	bit_end		= end_pos>=this_end?this_end:end_pos;
 
-				result	+= bit_end - bit_start;
+		final long baseOffset = file_offset_in_torrent;
+
+		int i = 0;
+
+		long first = absoluteOffsets[0];
+		long last = absoluteOffsets[absoluteOffsets.length-1]+lengths[lengths.length-1];
+		
+		// chunk might span file boundaries
+		long lastEnd = Math.max(absoluteOffsets[0],baseOffset);
+		while(absoluteOffsets[i]+lengths[i] < baseOffset)
+			i++;
+
+		boolean doSkipping = true;
+
+		if ( !this_mon.enter(250)){
+
+			Debug.outNoStack( "Failed to lock stats, abandoning" );
+			
+			return;
+		}
+
+		try{
+			Iterator it = cache.subSet(new CacheEntry(first-1), new CacheEntry(last)).iterator();
+			
+			//cache.subSet(new CacheEntry())
+
+			while(it.hasNext())
+			{
+				CacheEntry	entry = (CacheEntry)it.next();
+				long startPos = entry.getFilePosition()+baseOffset;
+				long endPos = startPos+entry.getLength();
+				// the following check ensures that we are within the interesting region
+				if(startPos < first)
+					continue; // skip forward until we reach a chunk
+				
+				// perform skipping from the previous round
+				if(doSkipping)
+					while(i < absoluteOffsets.length && absoluteOffsets[i] < startPos)
+					{
+						toModify[i] = false;
+						i++;
+					}
+
+				if(i >= absoluteOffsets.length)
+					break;
+
+				doSkipping = false;
+
+				if(startPos >= absoluteOffsets[i] && endPos >= absoluteOffsets[i]+lengths[i])
+				{ // chunk completely falls into cache entry -> don't invalidate
+					i++;
+					doSkipping = true;
+				} else if(startPos >= lastEnd)
+				{ // chunk spans multiple cache entries AND we skipped -> invalidate
+					doSkipping = true;
+				} else if(startPos >= absoluteOffsets[i]+lengths[i]) 
+				{  // end of a spanning chunk -> don't invalidate
+					i++;
+					doSkipping = true;
+				}
+
+				if(endPos > last)
+					break;
+
+				lastEnd = endPos;
 			}
-			
-			return( result );
-			
-		}finally{
-			
+
+		} finally {
 			this_mon.exit();
 		}
+
+		if(doSkipping) // we fell through the loop but there's still cleanup to do
+			while(i<absoluteOffsets.length)
+			{
+				if(absoluteOffsets[i]+lengths[i] < baseOffset || absoluteOffsets[i] > baseOffset+torrent_file.getLength())
+				{
+					i++;
+					continue;
+				}
+				toModify[i] = false;
+				i++;
+			}
 	}
 	
+	
 		// support methods
 	
 	protected void
@@ -1336,23 +1389,6 @@ CacheFileWithCache
 	{
 		return( file.getStorageType()==FMFile.FT_COMPACT?CT_COMPACT:CT_LINEAR );
 	}
-	
-	public void
-	ensureOpen(
-		String	reason )
-
-		throws CacheFileManagerException
-	{
-		try{
-				// no cache flush required here
-			
-			file.ensureOpen( reason );
-			
-		}catch( FMFileManagerException e ){
-			
-			manager.rethrow(this,e);
-		}	
-	}
 
 	public long
 	getLength()
@@ -1374,7 +1410,7 @@ CacheFileWithCache
 				try{
 					this_mon.enter();
 			
-					long	physical_size = file.getLength();
+					long	physical_size = file.exists() ?  file.getLength() : 0;
 
 					Iterator	it = cache.iterator();
 					
@@ -1406,7 +1442,7 @@ CacheFileWithCache
 				}
 			}else{
 				
-				return( file.getLength());
+				return( file.exists() ? file.getLength() : 0);
 			}
 						
 		}catch( FMFileManagerException e ){
@@ -1427,7 +1463,7 @@ CacheFileWithCache
 				// we can optimise this if the file's already big enough as cache entries can
 				// only make it bigger
 			
-			long	physical_length = file.getLength();
+			long	physical_length = file.exists() ? file.getLength() : 0;
 			
 			long	res = physical_length - compare_to;
 			
@@ -1649,6 +1685,12 @@ CacheFileWithCache
 		}
 	}
 	
+	public boolean
+	isOpen()
+	{
+		return( file.isOpen());
+	}
+	
 	public void
 	delete()
 	
