@@ -128,8 +128,19 @@ FMFile
 	
 		throws FMFileManagerException;
 	
+	public boolean
+	isOpen();
+	
 	public void
 	delete()
 	
 		throws FMFileManagerException;
+	
+	public FMFile
+	createClone()
+	
+		throws FMFileManagerException;
+	
+	public boolean
+	isClone();
 }
