@@ -27,6 +27,7 @@ import java.util.*;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.config.impl.TransferSpeedValidator;
+import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.global.GlobalManager;
 import org.gudy.azureus2.core3.global.GlobalManagerAdapter;
 import org.gudy.azureus2.core3.global.GlobalManagerFactory;
@@ -37,10 +38,10 @@ import org.gudy.azureus2.core3.ipfilter.IpFilterManager;
 import org.gudy.azureus2.core3.logging.*;
 import org.gudy.azureus2.core3.ipfilter.*;
 import org.gudy.azureus2.core3.security.SESecurityManager;
+import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncer;
+import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncerResponse;
 import org.gudy.azureus2.core3.tracker.host.*;
 import org.gudy.azureus2.core3.util.*;
-import org.gudy.azureus2.platform.PlatformManager;
-import org.gudy.azureus2.platform.PlatformManagerCapabilities;
 import org.gudy.azureus2.platform.PlatformManagerFactory;
 import org.gudy.azureus2.platform.PlatformManagerListener;
 import org.gudy.azureus2.plugins.*;
@@ -53,6 +54,9 @@ import com.aelitis.azureus.core.instancemanager.AZInstanceManagerFactory;
 import com.aelitis.azureus.core.nat.NATTraverser;
 import com.aelitis.azureus.core.networkmanager.NetworkManager;
 import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
+import com.aelitis.azureus.core.networkmanager.admin.NetworkAdminNetworkInterface;
+import com.aelitis.azureus.core.networkmanager.admin.NetworkAdminNetworkInterfaceAddress;
+import com.aelitis.azureus.core.networkmanager.admin.NetworkAdminPropertyChangeListener;
 import com.aelitis.azureus.core.peermanager.PeerManager;
 import com.aelitis.azureus.core.peermanager.download.session.TorrentSessionManager;
 import com.aelitis.azureus.core.peermanager.nat.PeerNATTraverser;
@@ -62,7 +66,9 @@ import com.aelitis.azureus.core.speedmanager.SpeedManager;
 import com.aelitis.azureus.core.speedmanager.SpeedManagerAdapter;
 import com.aelitis.azureus.core.speedmanager.SpeedManagerFactory;
 import com.aelitis.azureus.core.update.AzureusRestarterFactory;
+import com.aelitis.azureus.launcher.classloading.PrimaryClassloader;
 import com.aelitis.azureus.plugins.dht.DHTPlugin;
+import com.aelitis.azureus.plugins.tracker.dht.DHTTrackerPlugin;
 
 /**
  * @author parg
@@ -77,6 +83,9 @@ AzureusCoreImpl
 	protected static AzureusCore		singleton;
 	protected static AEMonitor			class_mon	= new AEMonitor( "AzureusCore:class" );
 	
+	private static final String DM_ANNOUNCE_KEY	= "AzureusCore:announce_key";
+	private static final boolean LOAD_PLUGINS_IN_OTHER_THREAD = true;
+	
 	public static AzureusCore
 	create()
 	
@@ -126,8 +135,13 @@ AzureusCoreImpl
 	private CryptoManager		crypto_manager;
 	private NATTraverser		nat_traverser;
 	
+	private long create_time = SystemTime.getCurrentTime();
+
+
 	private boolean				started;
 	private boolean				stopped;
+	private boolean				restarting;
+	
 	private List				listeners				= new ArrayList();
 	private List				lifecycle_listeners		= new ArrayList();
 	private List				operation_listeners		= new ArrayList();
@@ -138,9 +152,14 @@ AzureusCoreImpl
 
 	private AzureusCoreOperation	initialisation_op = createOperation( AzureusCoreOperation.OP_INITIALISATION );
 	
+	public static boolean SUPPRESS_CLASSLOADER_ERRORS = false;
+	
 	protected
 	AzureusCoreImpl()
 	{
+		if(!SUPPRESS_CLASSLOADER_ERRORS && !(this.getClass().getClassLoader() instanceof PrimaryClassloader))
+			System.out.println("###\nWarning: Core not instantiated through a PrimaryClassloader, this can lead to restricted functionality or bugs in future versions\n###");
+		
 		COConfigurationManager.initialise();
 		
 		MessageText.loadBundle();
@@ -151,7 +170,7 @@ AzureusCoreImpl
 		
 		AETemporaryFileHandler.startup();
     
-		AEThread.setOurThread();
+		AEThread2.setOurThread();
 		
 			// set up a backwards pointer from config -> app dir so we can derive one from the other. It'll get saved on closedown, no need to do so now
 				
@@ -173,6 +192,22 @@ AzureusCoreImpl
 						}
 						
 						stop();
+						
+					}else if ( type == ET_SUSPEND ){
+						
+						if (Logger.isEnabled()){
+							Logger.log(new LogEvent(LOGID, "Platform manager requested suspend"));
+						}
+						
+						COConfigurationManager.save();
+						
+					}else if ( type == ET_RESUME ){
+		
+						if (Logger.isEnabled()){
+							Logger.log(new LogEvent(LOGID, "Platform manager requested resume"));
+						}
+						
+						announceAll( true );
 					}
 				}
 			});
@@ -197,13 +232,14 @@ AzureusCoreImpl
 						private boolean setting_limits;
 						
 						public int
-						getCurrentProtocolUploadSpeed()
+						getCurrentProtocolUploadSpeed(
+							int	average_period )
 						{
 							if ( global_manager != null ){
 								
 								GlobalManagerStats stats = global_manager.getStats();
 								
-								return( stats.getProtocolSendRateNoLAN());
+								return( stats.getProtocolSendRateNoLAN( average_period ));
 								
 							}else{
 								
@@ -212,20 +248,46 @@ AzureusCoreImpl
 						}
 						
 						public int
-						getCurrentDataUploadSpeed()
+						getCurrentDataUploadSpeed(
+							int	average_period )
 						{
 							if ( global_manager != null ){
 								
 								GlobalManagerStats stats = global_manager.getStats();
 								
-								return( stats.getDataSendRateNoLAN());
+								return( stats.getDataSendRateNoLAN( average_period ));
 								
 							}else{
 								
 								return(0);
 							}
 						}
-						public int
+
+                        public int
+                        getCurrentProtocolDownloadSpeed(
+                        	int	average_period )
+                        {
+                            if( global_manager != null ){
+                                GlobalManagerStats stats = global_manager.getStats();
+                                return (stats.getProtocolReceiveRateNoLAN( average_period ) );
+                            }else{
+                                return(0);
+                            }
+                        }
+
+                        public int
+                        getCurrentDataDownloadSpeed(
+                        	int	average_period )
+                        {
+                            if( global_manager != null ){
+                                GlobalManagerStats stats = global_manager.getStats();
+                                return (stats.getDataReceiveRateNoLAN( average_period ) );
+                            }else{
+                                return(0);
+                            }
+                        }
+                        
+                        public int
 						getCurrentUploadLimit()
 						{
 							String key = TransferSpeedValidator.getActiveUploadParameter( global_manager );
@@ -272,14 +334,14 @@ AzureusCoreImpl
 						public int
 						getCurrentDownloadLimit()
 						{
-							return( COConfigurationManager.getIntParameter( TransferSpeedValidator.getDownloadParameter()) * 1024 );
+							return( TransferSpeedValidator.getGlobalDownloadRateLimitBytesPerSecond());
 						}
 						
 						public void
 						setCurrentDownloadLimit(
 							int		bytes_per_second )
 						{
-							COConfigurationManager.setParameter( TransferSpeedValidator.getDownloadParameter(), bytes_per_second/1024 );
+							TransferSpeedValidator.setGlobalDownloadRateLimitBytesPerSecond( bytes_per_second );
 						}
 						
 						public Object
@@ -355,6 +417,57 @@ AzureusCoreImpl
 				});
 	}
 	
+	protected void
+	announceAll(
+		boolean	force )
+	{
+		Logger.log(	new LogEvent(LOGID, "Updating trackers" ));
+
+		GlobalManager gm = getGlobalManager();
+		
+		if ( gm != null ){
+			
+			List	downloads = gm.getDownloadManagers();
+			
+			long now	= SystemTime.getCurrentTime();
+
+			for (int i=0;i<downloads.size();i++){
+				
+				DownloadManager	dm = (DownloadManager)downloads.get(i);
+				
+				Long	last_announce_l = (Long)dm.getData( DM_ANNOUNCE_KEY );
+				
+				long	last_announce	= last_announce_l==null?create_time:last_announce_l.longValue();
+				
+				TRTrackerAnnouncer an = dm.getTrackerClient();
+				
+				if ( an != null ){
+					
+					TRTrackerAnnouncerResponse last_announce_response = an.getLastResponse();
+					
+					if ( 	now - last_announce > 15*60*1000 ||
+							last_announce_response == null ||
+							last_announce_response.getStatus() == TRTrackerAnnouncerResponse.ST_OFFLINE ||
+							force ){
+	
+						dm.setData( DM_ANNOUNCE_KEY, new Long( now ));
+						
+						Logger.log(	new LogEvent(LOGID, "    updating tracker for " + dm.getDisplayName()));
+	
+						dm.requestTrackerAnnounce( true );
+					}
+				}
+			}
+		}
+		
+	    PluginInterface dht_tracker_pi = getPluginManager().getPluginInterfaceByClass( DHTTrackerPlugin.class );
+
+	    if ( dht_tracker_pi != null ){
+	    	
+	    	((DHTTrackerPlugin)dht_tracker_pi.getPlugin()).announceAll();
+	    }
+	}
+	
 	public LocaleUtil
 	getLocaleUtil()
 	{
@@ -366,7 +479,7 @@ AzureusCoreImpl
 	
 		throws AzureusCoreException
 	{
-		AEThread.setOurThread();
+		AEThread2.setOurThread();
 		
 		try{
 			this_mon.enter();
@@ -388,13 +501,29 @@ AzureusCoreImpl
 			this_mon.exit();
 		}
 		
-		if (Logger.isEnabled())
-			Logger.log(new LogEvent(LOGID, "Loading of Plugins starts"));
 
-		pi.loadPlugins(this, false);
+		// run plugin loading in parallel to the global manager loading
+		AEThread2 pluginload = new AEThread2("PluginLoader",true)
+		{
+			public void run() {
+				if (Logger.isEnabled())
+					Logger.log(new LogEvent(LOGID, "Loading of Plugins starts"));
+				pi.loadPlugins(AzureusCoreImpl.this, false);
+				if (Logger.isEnabled())
+					Logger.log(new LogEvent(LOGID, "Loading of Plugins complete"));
+			}
+		};
 		
-		if (Logger.isEnabled())
-			Logger.log(new LogEvent(LOGID, "Loading of Plugins complete"));
+		if (LOAD_PLUGINS_IN_OTHER_THREAD) {
+			pluginload.start();
+		}
+		else {
+			pluginload.run();
+		}
+
+
+		
+
 
 		// Disable async loading of existing torrents, because there are many things
 		// (like hosting) that require all the torrents to be loaded.  While we
@@ -423,6 +552,11 @@ AzureusCoreImpl
 					}
 				}, 0);
 
+		// wait until plugin loading is done
+		if (LOAD_PLUGINS_IN_OTHER_THREAD) {
+			pluginload.join();
+		}
+		
 		triggerLifeCycleComponentCreated(global_manager);
 
 		pi.initialisePlugins();
@@ -459,15 +593,16 @@ AzureusCoreImpl
 											{
 												public void 
 												seedingStatusChanged( 
-													boolean seeding_only_mode )
+													boolean seeding_only_mode,
+													boolean	b )
 												{
 													checkConfig();
 												}
 											});
 									
 									COConfigurationManager.addAndFireParameterListeners(
-										new String[]{	TransferSpeedValidator.AUTO_UPLOAD_CONFIGKEY,
-														TransferSpeedValidator.AUTO_UPLOAD_SEEDING_CONFIGKEY },
+										new String[]{	TransferSpeedValidator.AUTO_UPLOAD_ENABLED_CONFIGKEY,
+														TransferSpeedValidator.AUTO_UPLOAD_SEEDING_ENABLED_CONFIGKEY },
 										new ParameterListener()
 										{
 											public void 
@@ -495,18 +630,37 @@ AzureusCoreImpl
 		}catch( Throwable e ){
 		}
 		
-	    new AEThread("Plugin Init Complete")
+	    new AEThread2("Plugin Init Complete", false )
 	       {
 	        	public void
-	        	runSupport()
+	        	run()
 	        	{
+	        		for (int i=0;i<lifecycle_listeners.size();i++){
+	        			
+	        			try{
+	        				AzureusCoreLifecycleListener listener = (AzureusCoreLifecycleListener)lifecycle_listeners.get(i);
+	        				
+	        				if ( !listener.requiresPluginInitCompleteBeforeStartedEvent()){
+	        				
+	        					listener.started( AzureusCoreImpl.this );
+	        				}
+	        			}catch( Throwable e ){
+	        				
+	        				Debug.printStackTrace(e);
+	        			}
+	        		}
+	        		
 	        		pi.initialisationComplete();
 	        		
 	        		for (int i=0;i<lifecycle_listeners.size();i++){
 	        			
 	        			try{
-	        				((AzureusCoreLifecycleListener)lifecycle_listeners.get(i)).started( AzureusCoreImpl.this );
+	        				AzureusCoreLifecycleListener listener = (AzureusCoreLifecycleListener)lifecycle_listeners.get(i);
 	        				
+	        				if ( listener.requiresPluginInitCompleteBeforeStartedEvent()){
+	        				
+	        					listener.started( AzureusCoreImpl.this );
+	        				}				
 	        			}catch( Throwable e ){
 	        				
 	        				Debug.printStackTrace(e);
@@ -532,13 +686,85 @@ AzureusCoreImpl
 			AzureusCoreImpl.this.stop();
 	     }
 	   });	
+	   	   
+	   AEDiagnostics.checkDumpsAndNatives();
 	   
-	   checkBadNatives();
+	   NetworkAdmin na = NetworkAdmin.getSingleton();
 	   
-	   NetworkAdmin.getSingleton().runInitialChecks();
+	   na.runInitialChecks();
+	   
+	   na.addPropertyChangeListener(
+			   new NetworkAdminPropertyChangeListener()
+			   {
+				   private String	last_as;
+				   
+				   public void
+				   propertyChanged(
+						   String		property )
+				   {
+					   NetworkAdmin na = NetworkAdmin.getSingleton();
+
+					   if ( property.equals( NetworkAdmin.PR_NETWORK_INTERFACES )){
+
+						   boolean	found_usable = false;
+						   
+						   NetworkAdminNetworkInterface[] intf = na.getInterfaces();
+						   
+						   for (int i=0;i<intf.length;i++){
+							   
+							   NetworkAdminNetworkInterfaceAddress[] addresses = intf[i].getAddresses();
+							   
+							   for (int j=0;j<addresses.length;j++){
+								   
+								   if ( !addresses[j].isLoopback()){
+									   
+									   found_usable = true;
+								   }
+							   }
+						   }
+						   
+						   		// ignore event if nothing usable
+						   
+						   if ( !found_usable ){
+							   
+							   return;
+						   }
+						   
+						   Logger.log(	new LogEvent(LOGID, "Network interfaces have changed (new=" + na.getNetworkInterfacesAsString() + ")"));
+
+						   announceAll( false );
+						   
+					   }else if ( property.equals( NetworkAdmin.PR_AS )){
+						   
+						   String	as = na.getCurrentASN().getAS();
+						   
+						   if ( last_as == null ){
+							   
+							   last_as = as;
+							   
+						   }else if ( !as.equals( last_as )){
+							   
+							   Logger.log(	new LogEvent(LOGID, "AS has changed (new=" + as + ")" ));
+
+							   last_as = as;
+							   
+							   announceAll( false );
+						   }
+					   }
+				   }
+			   });
 	}
 	
-	public void triggerLifeCycleComponentCreated(AzureusCoreComponent component) {
+	public boolean
+	isStarted()
+	{
+		return( started );
+	}
+	
+	public void 
+	triggerLifeCycleComponentCreated(
+		AzureusCoreComponent component )
+	{
 		for (int i = 0; i < lifecycle_listeners.size(); i++) {
 
 			try{
@@ -551,60 +777,6 @@ AzureusCoreImpl
 			}
 		}
 	}
-  
-	protected void
-	checkBadNatives()
-	{
-		PlatformManager	p_man = PlatformManagerFactory.getPlatformManager();
-		
-		if ( 	p_man.getPlatformType() == PlatformManager.PT_WINDOWS &&
-				p_man.hasCapability( PlatformManagerCapabilities.TestNativeAvailability )){
-		
-	
-			String[]	dlls = { 	"niphk", 
-									"nvappfilter", 
-									"netdog", 
-									"vlsp", 
-									"imon", 
-									"sarah", 
-									"MxAVLsp", 
-									"mclsp", 
-									"radhslib", 
-									"winsflt",
-									"nl_lsp",
-									"AxShlex"
-						};
-			
-			for (int i=0;i<dlls.length;i++){
-				
-				String	dll = dlls[i];
-				
-				if ( !COConfigurationManager.getBooleanParameter( "platform.win32.dll_found." + dll, false )){
-							
-					try{
-						if ( p_man.testNativeAvailability( dll + ".dll" )){
-							
-							COConfigurationManager.setParameter( "platform.win32.dll_found." + dll, true );
-
-							String	detail = MessageText.getString( "platform.win32.baddll." + dll );
-							
-							Logger.logTextResource(
-									new LogAlert(
-											LogAlert.REPEATABLE, 
-											LogAlert.AT_WARNING,
-											"platform.win32.baddll.info" ),	
-									new String[]{ dll + ".dll", detail });
-						}
-			
-					}catch( Throwable e ){
-						
-						Debug.printStackTrace(e);
-					}
-				}
-			}
-		}
-	}
- 
 	
 	private void
 	runNonDaemon(
@@ -622,10 +794,10 @@ AzureusCoreImpl
 			
 			final Throwable[]	error = {null};
 			
-			new AEThread( "AzureusCore:runNonDaemon" )
+			new AEThread2( "AzureusCore:runNonDaemon", false )
 			{
 				public void
-				runSupport()
+				run()
 				{
 					try{
 			
@@ -734,6 +906,9 @@ AzureusCoreImpl
 		}
 		
 		try{
+			if (Logger.isEnabled())
+				Logger.log(new LogEvent(LOGID, "Invoking synchronous 'stopping' listeners"));
+
 			for (int i=0;i<sync_listeners.size();i++){		
 				try{
 					((AzureusCoreLifecycleListener)sync_listeners.get(i)).stopping( this );
@@ -744,6 +919,9 @@ AzureusCoreImpl
 				}
 			}
 			
+			if (Logger.isEnabled())
+				Logger.log(new LogEvent(LOGID, "Invoking asynchronous 'stopping' listeners"));
+
 				// in case something hangs during listener notification (e.g. version check server is down
 				// and the instance manager tries to obtain external address) we limit overall dispatch
 				// time to 10 seconds
@@ -764,8 +942,14 @@ AzureusCoreImpl
 					10*1000 );
 	
 			
+			if (Logger.isEnabled())
+				Logger.log(new LogEvent(LOGID, "Stopping global manager"));
+
 			global_manager.stopGlobalManager();
-				
+			
+			if (Logger.isEnabled())
+				Logger.log(new LogEvent(LOGID, "Invoking synchronous 'stopped' listeners"));
+
 			for (int i=0;i<sync_listeners.size();i++){		
 				try{
 					((AzureusCoreLifecycleListener)sync_listeners.get(i)).stopped( this );
@@ -776,6 +960,9 @@ AzureusCoreImpl
 				}
 			}
 			
+			if (Logger.isEnabled())
+				Logger.log(new LogEvent(LOGID, "Invoking asynchronous 'stopped' listeners"));
+
 			ListenerManager.dispatchWithTimeout(
 					async_listeners,
 					new ListenerManagerDispatcher()
@@ -791,6 +978,9 @@ AzureusCoreImpl
 					},
 					10*1000 );
 				
+			if (Logger.isEnabled())
+				Logger.log(new LogEvent(LOGID, "Waiting for quiescence"));
+
 			NonDaemonTaskRunner.waitUntilIdle();
 			
 				// shut down diags - this marks the shutdown as tidy and saves the config
@@ -819,12 +1009,12 @@ AzureusCoreImpl
 					
 					final Thread	t = threads[i];
 					
-					if ( t != null && t != Thread.currentThread() && !t.isDaemon() && !AEThread.isOurThread( t )){
+					if ( t != null && t != Thread.currentThread() && !t.isDaemon() && !AEThread2.isOurThread( t )){
 						
-						new AEThread( "VMKiller", true )
+						new AEThread2( "VMKiller", true )
 						{
 							public void
-							runSupport()
+							run()
 							{
 								try{
 									Thread.sleep(10*1000);
@@ -891,6 +1081,8 @@ AzureusCoreImpl
 
 				checkRestartSupported();
 
+				restarting = true;
+				
 				stopSupport(false);
 
 				if (Logger.isEnabled())
@@ -931,6 +1123,12 @@ AzureusCoreImpl
         });
 	}
 	
+	public boolean
+	isRestarting()
+	{
+		return( restarting );
+	}
+	
 	public void
 	checkRestartSupported()
 	
