@@ -22,8 +22,6 @@
 
 package com.aelitis.azureus.core.instancemanager.impl;
 
-import java.io.BufferedInputStream;
-import java.io.ByteArrayInputStream;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.net.NetworkInterface;
@@ -37,18 +35,22 @@ import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.logging.LogEvent;
 import org.gudy.azureus2.core3.logging.LogIDs;
 import org.gudy.azureus2.core3.logging.Logger;
+import org.gudy.azureus2.core3.peer.PEPeerSource;
 import org.gudy.azureus2.core3.torrent.TOTorrent;
 import org.gudy.azureus2.core3.util.AEMonitor;
+import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.AESemaphore;
 import org.gudy.azureus2.core3.util.AEThread;
 import org.gudy.azureus2.core3.util.BDecoder;
 import org.gudy.azureus2.core3.util.BEncoder;
 import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.core3.util.DelayedEvent;
 import org.gudy.azureus2.core3.util.SHA1Simple;
 import org.gudy.azureus2.core3.util.SimpleTimer;
 import org.gudy.azureus2.core3.util.SystemTime;
 import org.gudy.azureus2.core3.util.TimerEvent;
 import org.gudy.azureus2.core3.util.TimerEventPerformer;
+import org.gudy.azureus2.plugins.PluginInterface;
 import org.gudy.azureus2.plugins.download.Download;
 import org.gudy.azureus2.pluginsimpl.local.download.DownloadManagerImpl;
 
@@ -59,6 +61,7 @@ import com.aelitis.azureus.core.instancemanager.AZInstanceManager;
 import com.aelitis.azureus.core.instancemanager.AZInstanceManagerListener;
 import com.aelitis.azureus.core.instancemanager.AZInstanceTracked;
 import com.aelitis.azureus.core.util.NetUtils;
+import com.aelitis.azureus.plugins.dht.DHTPlugin;
 import com.aelitis.net.udp.mc.MCGroup;
 import com.aelitis.net.udp.mc.MCGroupAdapter;
 import com.aelitis.net.udp.mc.MCGroupFactory;
@@ -261,6 +264,26 @@ AZInstanceManagerImpl
 		}.start();
 	}
 	
+	public long 
+	getClockSkew() 
+	{
+		try{
+		    PluginInterface dht_pi = core.getPluginManager().getPluginInterfaceByClass( DHTPlugin.class );
+	    		    
+		    if ( dht_pi != null ){
+	    	
+		    	DHTPlugin dht = (DHTPlugin)dht_pi.getPlugin();
+		    	
+		    	return( dht.getClockSkew());
+		    }
+		}catch( Throwable e ){
+			
+			Debug.printStackTrace(e);
+		}
+		
+	    return( 0 );
+	}
+	
 	public void
 	trace(
 		String	str )
@@ -407,7 +430,7 @@ AZInstanceManagerImpl
 		int					length )
 	{
 		try{
-			Map	map = BDecoder.decode(new BufferedInputStream(new ByteArrayInputStream( data, 0, length )));
+			Map	map = BDecoder.decode( data, 0, length );
 			
 			long	version = ((Long)map.get( "ver" )).longValue();
 			long	type	= ((Long)map.get( "type" )).longValue();
@@ -421,66 +444,69 @@ AZInstanceManagerImpl
 			
 			AZOtherInstanceImpl	instance = AZOtherInstanceImpl.decode( originator_address, (Map)map.get( "orig" ));
 			
-			if ( type == MT_ALIVE ){
+			if ( instance != null ){
 				
-				checkAdd( instance );
-				
-			}else if ( type == MT_BYE ){
-				
-				checkRemove( instance );
-				
-			}else{
-				
-				checkAdd( instance );
-				
-				Map	body = (Map)map.get( "body" );
-				
-				if ( type == MT_REQUEST ){
+				if ( type == MT_ALIVE ){
 					
-					String	originator_id	= instance.getID();
+					checkAdd( instance );
 					
-					if ( !originator_id.equals( my_instance.getID())){
+				}else if ( type == MT_BYE ){
+					
+					checkRemove( instance );
+					
+				}else{
+					
+					checkAdd( instance );
+					
+					Map	body = (Map)map.get( "body" );
+					
+					if ( type == MT_REQUEST ){
 						
-						Map	reply = requestReceived( instance, body );
-					
-						if ( reply != null ){
+						String	originator_id	= instance.getID();
 						
-							reply.put( "oid", originator_id.getBytes());
-							reply.put( "rid", body.get( "rid" ));
+						if ( !originator_id.equals( my_instance.getID())){
 							
-							sendMessage( MT_REPLY, reply, originator );
-						}
-					}
-				}else if ( 	type == MT_REPLY ){
-					
-					String	originator_id	= new String((byte[])body.get( "oid" ));
-					
-					if ( originator_id.equals( my_instance.getID())){
+							Map	reply = requestReceived( instance, body );
 						
-						long req_id = ((Long)body.get("rid")).longValue();
-						
-						try{
-							this_mon.enter();
+							if ( reply != null ){
 							
-							for (int i=0;i<requests.size();i++){
+								reply.put( "oid", originator_id.getBytes());
+								reply.put( "rid", body.get( "rid" ));
 								
-								request	req = (request)requests.get(i);
-								
-								if ( req.getID() == req_id ){
-									
-									req.addReply( instance, body );
-								}
+								sendMessage( MT_REPLY, reply, originator );
 							}
-						}finally{
+						}
+					}else if ( 	type == MT_REPLY ){
+						
+						String	originator_id	= new String((byte[])body.get( "oid" ));
+						
+						if ( originator_id.equals( my_instance.getID())){
 							
-							this_mon.exit();
+							long req_id = ((Long)body.get("rid")).longValue();
+							
+							try{
+								this_mon.enter();
+								
+								for (int i=0;i<requests.size();i++){
+									
+									request	req = (request)requests.get(i);
+									
+									if ( req.getID() == req_id ){
+										
+										req.addReply( instance, body );
+									}
+								}
+							}finally{
+								
+								this_mon.exit();
+							}
 						}
 					}
 				}
 			}
 		}catch( Throwable e ){
 			
-			Debug.printStackTrace(e);
+			Debug.out( "Invalid packet received from " + originator, e );
 		}
 	}
 	
@@ -549,6 +575,11 @@ AZInstanceManagerImpl
 				return( null );
 			}
 			
+			if ( !matching_dm.getDownloadState().isPeerSourceEnabled( PEPeerSource.PS_PLUGIN )){
+				
+				return( null );
+			}
+			
 			int	dm_state = matching_dm.getState();
 			
 			if ( dm_state == DownloadManager.STATE_ERROR || dm_state == DownloadManager.STATE_STOPPED ){
@@ -910,6 +941,19 @@ AZInstanceManagerImpl
 			return( true );
 		}
 		
+		if ( explicit_peers.size() > 0 ){
+			
+			Iterator	it = explicit_peers.iterator();
+			
+			while( it.hasNext()){
+
+				if (((InetSocketAddress)it.next()).getAddress().equals( address )){
+					
+					return( true );
+				}
+			}
+		}
+		
 		return( false );
 	}
 	
@@ -991,7 +1035,7 @@ AZInstanceManagerImpl
 		InetAddress			explicit_address,
 		boolean				force_send_alive )
 	{
-		InetSocketAddress	sad = new InetSocketAddress( explicit_address, MC_GROUP_PORT );
+		final InetSocketAddress	sad = new InetSocketAddress( explicit_address, MC_GROUP_PORT );
 		
 		boolean	new_peer = false;
 		
@@ -1017,7 +1061,22 @@ AZInstanceManagerImpl
 		
 		if ( force_send_alive || new_peer ){
 			
-			sendAlive( sad );
+				// take this off the current thread as there are potential deadlock issues
+				// regarding this during initialisation as sending the event attempts to
+				// get the external address, this may hit DHT and the current thread
+				// maybe initialising the DHT...
+			
+			new DelayedEvent(
+					"AZInstanceManagerImpl:delaySendAlive", 
+					0,
+					new AERunnable()
+					{
+						public void 
+						runSupport()
+						{
+							sendAlive( sad );
+						}
+					});
 		}
 		
 		return( new_peer );
