@@ -37,6 +37,7 @@ import com.aelitis.azureus.core.networkmanager.impl.http.HTTPNetworkManager;
 import com.aelitis.azureus.core.networkmanager.impl.tcp.TCPNetworkManager;
 import com.aelitis.azureus.core.networkmanager.impl.udp.UDPNetworkManager;
 import com.aelitis.azureus.core.peermanager.messaging.*;
+import com.aelitis.azureus.core.util.FeatureAvailability;
 
 
 
@@ -50,6 +51,7 @@ public class NetworkManager {
   private static final NetworkManager instance = new NetworkManager();
 
   private static int max_download_rate_bps;
+  private static int external_max_download_rate_bps;
   
   private static int max_upload_rate_bps_normal;
   private static int max_upload_rate_bps_seeding_only;
@@ -65,13 +67,17 @@ public class NetworkManager {
   public static boolean 	REQUIRE_CRYPTO_HANDSHAKE;
   public static boolean 	INCOMING_HANDSHAKE_FALLBACK_ALLOWED;	
   public static boolean 	OUTGOING_HANDSHAKE_FALLBACK_ALLOWED;
+  public static boolean 	INCOMING_CRYPTO_ALLOWED;	
+
+  private static boolean	USE_REQUEST_LIMITING;
 	
 
   static {
   	COConfigurationManager.addAndFireParameterListeners(
   			new String[]{ "network.transport.encrypted.require",
     									"network.transport.encrypted.fallback.incoming",
-    									"network.transport.encrypted.fallback.outgoing",
+       									"network.transport.encrypted.fallback.outgoing",
+       									"network.transport.encrypted.allow.incoming",
     									"LAN Speed Enabled",
     									"Max Upload Speed KBs",
     									"Max LAN Upload Speed KBs",
@@ -80,13 +86,17 @@ public class NetworkManager {
     									"Max Download Speed KBs",
     									"Max LAN Download Speed KBs",
     									"network.tcp.mtu.size",
-  										"network.udp.mtu.size" },
+  										"network.udp.mtu.size",
+  										"Use Request Limiting"},
   										
     		new ParameterListener()	{
     			 public void  parameterChanged(	String ignore ) {
-    				 REQUIRE_CRYPTO_HANDSHAKE				= COConfigurationManager.getBooleanParameter( "network.transport.encrypted.require");
-    				 INCOMING_HANDSHAKE_FALLBACK_ALLOWED	= COConfigurationManager.getBooleanParameter( "network.transport.encrypted.fallback.incoming");
-    				 OUTGOING_HANDSHAKE_FALLBACK_ALLOWED	= COConfigurationManager.getBooleanParameter( "network.transport.encrypted.fallback.outgoing");
+    				 REQUIRE_CRYPTO_HANDSHAKE				= COConfigurationManager.getBooleanParameter("network.transport.encrypted.require");
+    				 INCOMING_HANDSHAKE_FALLBACK_ALLOWED	= COConfigurationManager.getBooleanParameter("network.transport.encrypted.fallback.incoming");
+       				 OUTGOING_HANDSHAKE_FALLBACK_ALLOWED	= COConfigurationManager.getBooleanParameter("network.transport.encrypted.fallback.outgoing");
+       				 INCOMING_CRYPTO_ALLOWED				= COConfigurationManager.getBooleanParameter("network.transport.encrypted.allow.incoming");
+  
+    				 USE_REQUEST_LIMITING					= COConfigurationManager.getBooleanParameter("Use Request Limiting");
     				 
     				 max_upload_rate_bps_normal = COConfigurationManager.getIntParameter( "Max Upload Speed KBs" ) * 1024;
     				 if( max_upload_rate_bps_normal < 1024 )  max_upload_rate_bps_normal = UNLIMITED_RATE;
@@ -96,15 +106,19 @@ public class NetworkManager {
     				 if( max_lan_upload_rate_bps < 1024 )  max_lan_upload_rate_bps = UNLIMITED_RATE;
     				 if( max_lan_upload_rate_bps > UNLIMITED_RATE )  max_lan_upload_rate_bps = UNLIMITED_RATE;
     	        
-    				 max_upload_rate_bps_seeding_only = COConfigurationManager.getIntParameter( "Max Upload Speed Seeding KBs" ) * 1024;
+    				  
+    				 max_upload_rate_bps_seeding_only = COConfigurationManager.getIntParameter( "Max Upload Speed Seeding KBs" ) * 1024; 
     				 if( max_upload_rate_bps_seeding_only < 1024 )  max_upload_rate_bps_seeding_only = UNLIMITED_RATE;
     				 if( max_upload_rate_bps_seeding_only > UNLIMITED_RATE )  max_upload_rate_bps_seeding_only = UNLIMITED_RATE;
     	        
     				 seeding_only_mode_allowed = COConfigurationManager.getBooleanParameter( "enable.seedingonly.upload.rate" );
+    			
     				 
-    				 max_download_rate_bps = COConfigurationManager.getIntParameter( "Max Download Speed KBs" ) * 1024;
-    				 if( max_download_rate_bps < 1024 )  max_download_rate_bps = UNLIMITED_RATE;
-    				 if( max_download_rate_bps > UNLIMITED_RATE )  max_download_rate_bps = UNLIMITED_RATE;
+    				 external_max_download_rate_bps = max_download_rate_bps = (int)(COConfigurationManager.getIntParameter( "Max Download Speed KBs" ) * 1024); // leave 5KiB/s room for the request limiting  
+    				 if( max_download_rate_bps < 1024 || max_download_rate_bps > UNLIMITED_RATE)
+    					 max_download_rate_bps = UNLIMITED_RATE;
+    				 else if(USE_REQUEST_LIMITING && FeatureAvailability.isRequestLimitingEnabled())
+    					 max_download_rate_bps += Math.max(max_download_rate_bps * 0.1, 5*1024);
     	        
     				 lan_rate_enabled = COConfigurationManager.getBooleanParameter("LAN Speed Enabled");
     				 max_lan_download_rate_bps = COConfigurationManager.getIntParameter( "Max LAN Download Speed KBs" ) * 1024;
@@ -135,11 +149,12 @@ public class NetworkManager {
   private final TransferProcessor lan_download_processor = new TransferProcessor( TransferProcessor.TYPE_DOWNLOAD, new LimitedRateGroup(){
     public int getRateLimitBytesPerSecond() {  return max_lan_download_rate_bps;  }
   });
-  
-  
-  private final LimitedRateGroup unlimited_rate_group = new LimitedRateGroup() {
-  	public int getRateLimitBytesPerSecond() {  return 0;  }
-  };
+     
+  public static boolean
+  isLANRateEnabled()
+  {
+	  return( lan_rate_enabled );
+  }
   
   private NetworkManagerStats	stats = new NetworkManagerStats();
   
@@ -187,13 +202,37 @@ public class NetworkManager {
     return max_upload_rate_bps_seeding_only;
   }
   
+  /**
+   * This method is for display purposes only, the internal rate limiting is 10% higher than returned by this method!
+   */
   public static int getMaxDownloadRateBPS() {
     if( max_download_rate_bps == UNLIMITED_RATE )  return 0;
-    return max_download_rate_bps; 
+    return external_max_download_rate_bps; 
   }
   
+  public static final int CRYPTO_OVERRIDE_NONE			= 0;
+  public static final int CRYPTO_OVERRIDE_REQUIRED		= 1;
+  public static final int CRYPTO_OVERRIDE_NOT_REQUIRED	= 2;
   
   
+  public static boolean
+  getCryptoRequired(
+	int	override_level )
+  {
+	  if ( override_level == CRYPTO_OVERRIDE_NONE ){
+	    
+		  return( REQUIRE_CRYPTO_HANDSHAKE );
+		  
+	  }else if ( override_level == CRYPTO_OVERRIDE_REQUIRED ){
+	    		
+		  return( true );
+		  
+	  }else{
+		  
+		  return( false );
+	  }
+  }
+  
   public void initialize() {
 	HTTPNetworkManager.getSingleton();  
 	   
@@ -203,7 +242,7 @@ public class NetworkManager {
       public void destroyInitiated(){}
       public void destroyed(){}
 
-      public void seedingStatusChanged( boolean seeding_only ) {
+      public void seedingStatusChanged( boolean seeding_only, boolean b ) {
         seeding_only_mode = seeding_only;
         refreshRates();
       }
@@ -257,6 +296,15 @@ public class NetworkManager {
     });
   }
  
+  public NetworkConnection
+  bindTransport(
+	Transport				transport,
+	MessageStreamEncoder	encoder,
+	MessageStreamDecoder	decoder )
+  {
+	  return( NetworkConnectionFactory.create( transport, encoder, decoder ));
+  }
+		
   
   /**
    * Cancel a request for inbound connection routing.
@@ -317,14 +365,17 @@ public class NetworkManager {
    * @param upload_group upload rate limit group
    * @param download_group download rate limit group
    */
-  public void startTransferProcessing( NetworkConnectionBase peer_connection, LimitedRateGroup upload_group, LimitedRateGroup download_group ) {
+  public void 
+  startTransferProcessing( 
+	NetworkConnectionBase 	peer_connection )
+  {
   	if( peer_connection.isLANLocal() && lan_rate_enabled ) {
-  		lan_upload_processor.registerPeerConnection( peer_connection, unlimited_rate_group );
-  		lan_download_processor.registerPeerConnection( peer_connection, unlimited_rate_group );
+  		lan_upload_processor.registerPeerConnection( peer_connection, true );
+  		lan_download_processor.registerPeerConnection( peer_connection, false );
   	}
   	else {
-  		upload_processor.registerPeerConnection( peer_connection, upload_group );
-  		download_processor.registerPeerConnection( peer_connection, download_group );
+  		upload_processor.registerPeerConnection( peer_connection, true );
+  		download_processor.registerPeerConnection( peer_connection, false );
   	}
   }
   
@@ -375,7 +426,56 @@ public class NetworkManager {
   	}
   }
   
+  public void
+  addRateLimiter(
+	NetworkConnectionBase 	peer_connection,
+	LimitedRateGroup		group,
+	boolean					upload )
+  {
+	  if ( upload ){
+		  if ( lan_upload_processor.isRegistered( peer_connection )){
+			  
+		  		lan_upload_processor.addRateLimiter( peer_connection, group );
+	
+		  }else{
+		  		upload_processor.addRateLimiter( peer_connection, group );
+		  } 
+	  }else{
+		  if ( lan_download_processor.isRegistered( peer_connection )){
+			  
+			  	lan_download_processor.addRateLimiter( peer_connection, group );
+	
+		  }else{
+		  		download_processor.addRateLimiter( peer_connection, group );
+		  } 
+	  }
+  }
     
+  public void
+  removeRateLimiter(
+	NetworkConnectionBase 	peer_connection,
+	LimitedRateGroup		group,
+	boolean					upload )
+  {
+	  if ( upload ){
+		  if ( lan_upload_processor.isRegistered( peer_connection )){
+			  
+		  		lan_upload_processor.removeRateLimiter( peer_connection, group );
+	
+		  }else{
+		  		upload_processor.removeRateLimiter( peer_connection, group );
+		  } 
+	  }else{
+		  if ( lan_download_processor.isRegistered( peer_connection )){
+			  
+			  	lan_download_processor.removeRateLimiter( peer_connection, group );
+	
+		  }else{
+		  		download_processor.removeRateLimiter( peer_connection, group );
+		  } 
+	  }
+  }
+  
   public NetworkManagerStats
   getStats()
   {
