@@ -28,6 +28,7 @@ import java.util.*;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
+import org.gudy.azureus2.core3.disk.DiskManager;
 import org.gudy.azureus2.core3.logging.LogEvent;
 import org.gudy.azureus2.core3.logging.LogIDs;
 import org.gudy.azureus2.core3.logging.Logger;
@@ -39,6 +40,7 @@ import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.DirectByteBuffer;
 import org.gudy.azureus2.core3.util.SimpleTimer;
 import org.gudy.azureus2.core3.util.SystemTime;
+import org.gudy.azureus2.core3.util.TimeFormatter;
 import org.gudy.azureus2.core3.util.TimerEvent;
 import org.gudy.azureus2.core3.util.TimerEventPerformer;
 
@@ -53,6 +55,8 @@ import com.aelitis.azureus.core.peermanager.messaging.bittorrent.BTHave;
 import com.aelitis.azureus.core.peermanager.messaging.bittorrent.BTInterested;
 import com.aelitis.azureus.core.peermanager.messaging.bittorrent.BTPiece;
 import com.aelitis.azureus.core.peermanager.messaging.bittorrent.BTRequest;
+import com.aelitis.azureus.core.util.CopyOnWriteList;
+import com.aelitis.azureus.core.util.HTTPUtils;
 
 public abstract class 
 HTTPNetworkConnection 
@@ -63,6 +67,12 @@ HTTPNetworkConnection
 	
 	protected static final String	NL			= "\r\n";
 
+	private static final String	HDR_SERVER 				= "Server: " + Constants.AZUREUS_NAME + " " + Constants.AZUREUS_VERSION + NL;
+	private static final String HDR_KEEP_ALIVE_TIMEOUT 	= "Keep-Alive: timeout=30" + NL;
+	private static final String HDR_CACHE_CONTROL		= "Cache-Control: public, max-age=86400" + NL;
+
+	private static final String	DEFAULT_CONTENT_TYPE	= HTTPUtils.guessContentTypeFromFileType(null);
+	
 	private static int        max_read_block_size;
 
 	static{
@@ -201,14 +211,13 @@ HTTPNetworkConnection
 	private HTTPNetworkManager	manager;
 	private NetworkConnection	connection;
 	private PEPeerTransport		peer;
-	private String				url;
 	
 	private HTTPMessageDecoder	decoder;
 	private HTTPMessageEncoder	encoder;
 	
 	private boolean			sent_handshake	= false;
 	
-	private byte[]	peer_id	= PeerUtils.createPeerID();
+	private byte[]	peer_id	= PeerUtils.createWebSeedPeerID();
 
 	private boolean	choked	= true;
 	
@@ -224,18 +233,33 @@ HTTPNetworkConnection
 	
 	private boolean	closing;
 	private boolean	destroyed;
+		
+	private String	last_modified_date;
+	private String	content_type	= DEFAULT_CONTENT_TYPE;
+	
+	private CopyOnWriteList	request_listeners = null;
 	
 	protected
 	HTTPNetworkConnection(
 		HTTPNetworkManager		_manager,
 		NetworkConnection		_connection,
-		PEPeerTransport			_peer,
-		String					_url )
+		PEPeerTransport			_peer )
 	{
-		manager		= _manager;
-		connection	= _connection;
-		peer		= _peer;
-		url			= _url;
+		manager			= _manager;
+		connection		= _connection;
+		peer			= _peer;
+		
+		DiskManager dm = peer.getManager().getDiskManager();
+		
+		long	last_modified = 0;
+		
+		try{
+			last_modified = dm.getFiles()[0].getFile(true).lastModified();
+						
+		}catch( Throwable e ){
+		}
+		
+		last_modified_date = TimeFormatter.getHTTPDate( last_modified );
 		
 		network_connection_key = new networkConnectionKey();
 			
@@ -263,14 +287,17 @@ HTTPNetworkConnection
 			}
 		}
 		
-		decoder.setConnection( this );
+			// note that the decoder can synchronously call-back if is preloaded with a header
+			// here...
+		
 		encoder.setConnection( this );
+		decoder.setConnection( this );
 	}
 	
 	protected boolean
 	isSeed()
 	{
-		if ( !peer.getControl().isSeeding()){
+		if ( ( !peer.getControl().isSeeding()) || peer.getControl().getHiddenBytes() > 0 ){
 			
 			if (Logger.isEnabled()){
 				Logger.log(new LogEvent(peer,LOGID, "Download is not seeding" ));
@@ -284,6 +311,13 @@ HTTPNetworkConnection
 		return( true );
 	}
 	
+	protected void
+	setContentType(
+		String	ct )
+	{
+		content_type	= ct;
+	}
+	
 	protected HTTPNetworkManager
 	getManager()
 	{
@@ -339,7 +373,7 @@ HTTPNetworkConnection
 	protected RawMessage
 	encodeBitField()
 	{
-		decoder.addMessage( new BTInterested());
+		decoder.addMessage( new BTInterested((byte)1));
 		
 		return( null );
 	}
@@ -359,7 +393,8 @@ HTTPNetworkConnection
 	
 	protected abstract void
 	decodeHeader(
-		String		header )
+		HTTPMessageDecoder		decoder,
+		String					header )
 	
 		throws IOException;
 
@@ -367,16 +402,51 @@ HTTPNetworkConnection
 	encodeHeader(
 		httpRequest	request )
 	{
-		String	res = 
-			"HTTP/1.1 " + (request.isPartialContent()?"206 Partial Content":"200 OK" ) + NL + 
-				"Content-Type: application/octet-stream" + NL +
-				"Server: " + Constants.AZUREUS_NAME + " " + Constants.AZUREUS_VERSION + NL +
-				"Connection: " + ( request.keepAlive()?"Keep-Alive":"Close" ) + NL +
-				(request.keepAlive()?("Keep-Alive: timeout=30" + NL) :"" ) +
-				"Content-Length: " + request.getTotalLength() + NL +
-				NL;
-							
-		return( res );
+		String	current_date = TimeFormatter.getHTTPDate( SystemTime.getCurrentTime());
+		
+		StringBuffer	res = new StringBuffer(256);
+		
+		res.append( "HTTP/1.1 " );
+		res.append( request.isPartialContent()?"206 Partial Content":"200 OK" );
+			res.append( NL );
+
+		res.append( "Content-Type: " );
+		res.append( content_type );
+	 		res.append( NL );		
+	 		
+		res.append( "Date: " );
+		res.append( current_date );
+		 	res.append( NL );
+		 	
+		res.append( "Last-Modified: " );
+		res.append( last_modified_date );
+			res.append( NL );
+
+		res.append( HDR_CACHE_CONTROL );
+		
+			// not sure about ETag. I was going to use the torrent hash but I don't understand the link
+			// between URL, range requests and ETags. Do we need to generate different ETags for each
+			// webseed piece request URL or can we use the torrent hash and rely on the fact that the
+			// URL changes? Are range-requests irrelevant as far as ETags go - I'd like to think so...
+		
+		res.append( HDR_SERVER );
+		
+		res.append( "Connection: " );
+		res.append( request.keepAlive()?"Keep-Alive":"Close" );
+			res.append( NL );
+		
+		if ( request.keepAlive()){
+				
+			res.append( HDR_KEEP_ALIVE_TIMEOUT );
+		}
+		
+		res.append( "Content-Length: " );
+		res.append( request.getTotalLength());
+		res.append( NL );
+		
+		res.append( NL );
+						
+		return( res.toString());
 	}
 	
 	protected void
@@ -393,13 +463,13 @@ HTTPNetworkConnection
 			
 			sent_handshake	= true;
 			
-			decoder.addMessage( new BTHandshake( control.getHash(), peer_id, false ));
+			decoder.addMessage( new BTHandshake( control.getHash(), peer_id, false, (byte)1 ));
 			
 			byte[]	bits = new byte[(control.getPieces().length +7) /8];
 			
 			DirectByteBuffer buffer = new DirectByteBuffer( ByteBuffer.wrap( bits ));
 			
-			decoder.addMessage( new BTBitfield( buffer ));
+			decoder.addMessage( new BTBitfield( buffer, (byte)1 ));
 		}
 		
 		synchronized( outstanding_requests ){
@@ -448,7 +518,8 @@ HTTPNetworkConnection
 					new BTRequest( 
 							this_piece_number, 
 							offset_in_piece, 
-							request_size ),
+							request_size,
+							(byte)1),
 					http_request );
 					
 				if ( request_size == length ){
@@ -502,7 +573,7 @@ HTTPNetworkConnection
 		}
 	}
 	
-	protected RawMessage
+	protected RawMessage[]
 	encodePiece(
 		Message		message )
 	{
@@ -518,7 +589,7 @@ HTTPNetworkConnection
 
 			if ( destroyed ){
 				
-				return( getEmptyRawMessage( message ));
+				return( new RawMessage[]{ getEmptyRawMessage( message )});
 			}
 		
 			for (int i=0;i<outstanding_requests.size();i++){
@@ -566,12 +637,12 @@ HTTPNetworkConnection
 			
 			Debug.out( "request not matched" );
 			
-			return( getEmptyRawMessage( message ));
+			return( new RawMessage[]{ getEmptyRawMessage( message )});
 		}
 		
 		if ( ready_requests.size() == 0 ){
 			
-			return( getEmptyRawMessage( message ));
+			return( new RawMessage[]{ getEmptyRawMessage( message )});
 		}
 		
 		try{
@@ -584,29 +655,30 @@ HTTPNetworkConnection
 		pendingRequest req	= (pendingRequest)ready_requests.get(0);
 		
 		DirectByteBuffer[]	buffers;
-		int					buffer_index	= 0;
 		
 		httpRequest	http_request = req.getHTTPRequest();
 		
-		buffers = new DirectByteBuffer[ ready_requests.size() + 1 ];
+		RawMessage[]	raw_messages = new RawMessage[ ready_requests.size()];
+		
+		for (int i=0;i<raw_messages.length;i++){
+			
+			buffers = new DirectByteBuffer[ 2 ];
 
-		if ( !http_request.hasSentFirstReply()){
+			if ( !http_request.hasSentFirstReply()){
 		
-			http_request.setSentFirstReply();
+				http_request.setSentFirstReply();
 						
-			String	header = encodeHeader( http_request );
+				String	header = encodeHeader( http_request );
 			
-			buffers[buffer_index++] = new DirectByteBuffer( ByteBuffer.wrap( header.getBytes()));
+				buffers[0] = new DirectByteBuffer( ByteBuffer.wrap( header.getBytes()));
 			
-		}else{
+			}else{
 			
-				// we have to do this as core code assumes buffer entry 0 is protocol
-			
-			buffers[buffer_index++] = new DirectByteBuffer( ByteBuffer.allocate(0));
-		}
-		
-		for (int i=0;i<ready_requests.size();i++){
+					// we have to do this as core code assumes buffer entry 0 is protocol
 			
+				buffers[0] = new DirectByteBuffer( ByteBuffer.allocate(0));
+			}
+					
 			req	= (pendingRequest)ready_requests.get(i);
 
 			BTPiece	this_piece = req.getBTPiece();
@@ -620,18 +692,33 @@ HTTPNetworkConnection
 				
 				piece_map.set( piece_number );
 				
-				decoder.addMessage( new BTHave( piece_number ));
+				decoder.addMessage( new BTHave( piece_number, (byte)1 ));
 			}
 			
-			buffers[buffer_index++] = this_piece.getPieceData();
-		}
-		
-		return(	new RawMessageImpl( 
-						message, 
+			buffers[1] = this_piece.getPieceData();
+			
+			req.logQueued();
+			
+			if ( request_listeners != null ){
+				
+				Iterator it = request_listeners.iterator();
+			
+				while( it.hasNext()){
+					
+					((requestListener)it.next()).requestComplete( req );
+				}
+			}
+			
+			raw_messages[i] = 
+				new RawMessageImpl( 
+						this_piece, 
 						buffers,
 						RawMessage.PRIORITY_HIGH, 
 						true, 
-						new Message[0] ));
+						new Message[0] );
+		}
+		
+		return( raw_messages );
 	}
 	
 	protected int
@@ -789,11 +876,128 @@ HTTPNetworkConnection
 			    	int byte_count )
 			    {	
 			    }
+			    
+			    public void flush(){}
 			});
 		
 		getConnection().getOutgoingMessageQueue().addMessage( http_message, false );
 	}
 	
+	protected void
+	flushRequests(
+		final flushListener		l )
+	{
+		boolean	sync_fire = false;
+		
+		synchronized( outstanding_requests ){
+
+			final int request_count = outstanding_requests.size();
+						
+			if ( request_count == 0 ){
+				
+				sync_fire = true;
+				
+			}else{
+				
+				if ( request_listeners == null ){
+					
+					request_listeners = new CopyOnWriteList();
+				}
+				
+				request_listeners.add(
+					new requestListener()
+					{
+						int	num_to_go = request_count;
+						
+						public void
+						requestComplete(
+							pendingRequest r )
+						{
+							num_to_go--;
+							
+							if ( num_to_go == 0 ){
+								
+								request_listeners.remove( this );
+								
+								flushRequestsSupport( l );
+							}
+						}
+					});
+			}
+		}
+		
+		if ( sync_fire ){
+			
+			flushRequestsSupport( l );
+		}
+	}
+	
+	protected void
+	flushRequestsSupport(
+		final flushListener		l )
+	{
+		OutgoingMessageQueue omq = getConnection().getOutgoingMessageQueue();
+		
+		final Message	http_message = new HTTPMessage( new byte[0] );
+		
+		omq.registerQueueListener(
+			new OutgoingMessageQueue.MessageQueueListener()
+			{
+				public boolean 
+				messageAdded( 
+					Message message )
+				{	
+					return( true );
+				}
+				   
+				public void 
+				messageQueued( 
+					Message message )
+				{			
+				}
+				    
+				public void 
+				messageRemoved( 
+					Message message )
+				{
+				}
+				    
+				public void 
+				messageSent( 
+					Message message )
+				{
+					if ( message == http_message ){
+						
+						l.flushed();
+					}
+				}
+				    
+			    public void 
+			    protocolBytesSent( 
+			    	int byte_count )
+			    {	
+			    }
+				 
+			    public void 
+			    dataBytesSent( 
+			    	int byte_count )
+			    {	
+			    }
+			    
+			    public void flush(){}
+			});
+		
+		omq.addMessage( http_message, false );
+		
+			// if after adding the message there's no bytes on the queue then we need to trigger an
+			// immediate flushed event as the queue won't get processed (0 bytes on it...)
+		
+		if ( omq.getTotalSize() == 0 ){
+			
+			l.flushed();
+		}
+	}
+	
 	protected class
 	httpRequest
 	{
@@ -832,7 +1036,7 @@ HTTPNetworkConnection
 				total_length += lengths[i];
 			}
 		}
-		
+				
 		protected boolean
 		isPartialContent()
 		{
@@ -889,6 +1093,21 @@ HTTPNetworkConnection
 		}
 	}
 	
+	protected interface
+	flushListener
+	{
+		public void
+		flushed();
+	}
+	
+	protected interface
+	requestListener
+	{
+		public void
+		requestComplete(
+			pendingRequest	request );
+	}
+	
 	protected class
 	pendingRequest
 	{
@@ -899,7 +1118,7 @@ HTTPNetworkConnection
 		private httpRequest	http_request;
 		
 		private BTPiece	bt_piece;
-		
+				
 		protected
 		pendingRequest(
 			BTRequest		_request,
@@ -910,6 +1129,19 @@ HTTPNetworkConnection
 			length	= _request.getLength();
 			
 			http_request	= _http_request;
+			
+			/*
+			if ( peer.getIp().equals( "64.71.5.2")){
+				
+				TimeFormatter.milliTrace( 
+						"http_req_create: " +
+							piece + "/" + start + 
+							" [hr=" + http_requests.size() + 
+							",cr=" + choked_requests.size() + 
+							",or=" + outstanding_requests.size() + 
+							",d=" + decoder.getQueueSize() + "]" );
+			}
+			*/
 		}
 		
 		protected int
@@ -947,6 +1179,36 @@ HTTPNetworkConnection
 			BTPiece	_bt_piece )
 		{
 			bt_piece	= _bt_piece;
+			
+			/*
+			if ( peer.getIp().equals( "64.71.5.2")){
+
+				TimeFormatter.milliTrace( 
+					"http_req_data: " +
+						piece + "/" + start + 
+						" [hr=" + http_requests.size() + 
+						",cr=" + choked_requests.size() + 
+						",or=" + outstanding_requests.size() + 
+						",d=" + decoder.getQueueSize() + "]" );
+			}
+			*/
+		}
+		
+		protected void
+		logQueued()
+		{
+			/*
+			if ( peer.getIp().equals( "64.71.5.2")){
+
+				TimeFormatter.milliTrace( 
+					"http_req_out: " +
+						piece + "/" + start + 
+						" [hr=" + http_requests.size() + 
+						",cr=" + choked_requests.size() + 
+						",or=" + outstanding_requests.size() + 
+						",d=" + decoder.getQueueSize() + "]" );
+			}
+			*/
 		}
 	}
 	
