@@ -31,6 +31,7 @@ import java.net.InetSocketAddress;
 import java.util.Properties;
 
 
+import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.FileUtil;
 import org.gudy.azureus2.core3.util.SystemTime;
@@ -40,6 +41,7 @@ import org.gudy.azureus2.plugins.logging.LoggerChannel;
 import org.gudy.azureus2.plugins.peers.Peer;
 import org.gudy.azureus2.plugins.peers.PeerManager;
 import org.gudy.azureus2.plugins.ui.config.ActionParameter;
+import org.gudy.azureus2.plugins.ui.config.BooleanParameter;
 import org.gudy.azureus2.plugins.utils.UTTimerEvent;
 import org.gudy.azureus2.plugins.utils.UTTimerEventPerformer;
 
@@ -82,8 +84,8 @@ import com.aelitis.azureus.plugins.dht.impl.DHTPluginStorageManager;
 public class 
 DHTPluginImpl
 {
-	private static final String	SEED_ADDRESS	= "dht.aelitis.com";
-	private static final String	SEED_IP			= "85.31.105.2";	// fallback in case DNS resolution fails
+	private static final String	SEED_ADDRESS_V4	= Constants.DHT_SEED_ADDRESS_V4;
+	private static final String	SEED_ADDRESS_V6	= Constants.DHT_SEED_ADDRESS_V6;
 	private static final int	SEED_PORT		= 6881;
 		
 	private static final long	MIN_ROOT_SEED_IMPORT_PERIOD	= 8*60*60*1000;
@@ -94,15 +96,16 @@ DHTPluginImpl
 	private int					status;
 	private String				status_text;
 	
-	private ActionParameter		reseed;
+	private ActionParameter		reseed_param;
+	private BooleanParameter	warn_user_param;
 	
 	private DHT					dht;
 	private int					port;
 	private byte				protocol_version;
-	private int					network;		
+	private int					network;	
+	private boolean				v6;
 	private DHTTransportUDP		transport;
-	private long				integrated_time;
-		
+
 	private DHTPluginStorageManager storage_manager;
 
 	private long				last_root_seed_import_time;
@@ -116,11 +119,14 @@ DHTPluginImpl
 	DHTPluginImpl(
 		PluginInterface			_plugin_interface,
 		DHTNATPuncherAdapter	_nat_adapter,
+		DHTPluginImplAdapter	_adapter,
 		byte					_protocol_version,
 		int						_network,
+		boolean					_v6,
 		String					_ip,
 		int						_port,
 		ActionParameter			_reseed,
+		BooleanParameter		_warn_user_param,
 		boolean					_logging,
 		LoggerChannel			_log,
 		DHTLogger				_dht_log )
@@ -128,17 +134,21 @@ DHTPluginImpl
 		plugin_interface	= _plugin_interface;
 		protocol_version	= _protocol_version;
 		network				= _network;
+		v6					= _v6;
 		port				= _port;
-		reseed				= _reseed;
+		reseed_param		= _reseed;
+		warn_user_param		= _warn_user_param;
 		log					= _log;
 		dht_log				= _dht_log;
 		
+		final DHTPluginImplAdapter	adapter = _adapter;
+		
 		try{
 			storage_manager = new DHTPluginStorageManager( network, dht_log, getDataDir( _network ));
 			
 			final PluginConfig conf = plugin_interface.getPluginconfig();
 			
-			int	send_delay = conf.getPluginIntParameter( "dht.senddelay", 50 );
+			int	send_delay = conf.getPluginIntParameter( "dht.senddelay", 25 );
 			int	recv_delay	= conf.getPluginIntParameter( "dht.recvdelay", 25 );
 			
 			boolean	bootstrap	= conf.getPluginBooleanParameter( "dht.bootstrapnode", false );
@@ -151,12 +161,13 @@ DHTPluginImpl
 				DHTTransportFactory.createUDP( 
 						_protocol_version,
 						_network,
+						_v6,
 						_ip,
 						storage_manager.getMostRecentAddress(),
 						_port, 
 						4,
 						2,
-						20000, 	// udp timeout - tried less but a significant number of 
+						10000, 	// udp timeout - tried less but a significant number of 
 								// premature timeouts occurred
 						send_delay, recv_delay, 
 						bootstrap,
@@ -171,6 +182,11 @@ DHTPluginImpl
 						DHTTransportContact	local_contact )
 					{
 						storage_manager.localContactChanged( local_contact );
+						
+						if ( adapter != null ){
+							
+							adapter.localContactChanged( getLocalAddress());
+						}
 					}
 					
 					public void
@@ -290,7 +306,7 @@ DHTPluginImpl
 
 					int	warned_port = plugin_interface.getPluginconfig().getPluginIntParameter( "udp_warned_port", 0 );
 					
-					if ( warned_port == port  ){
+					if ( warned_port == port || !warn_user_param.getValue() ){
 						
 						log.log( msg );
 						
@@ -364,6 +380,12 @@ DHTPluginImpl
 		}
 	}
 	
+	public long
+	getClockSkew()
+	{
+		return( transport.getStats().getSkewAverage());
+	}
+	
 	public void
 	logStats()
 	{
@@ -425,7 +447,7 @@ DHTPluginImpl
 		DHTTransportContact	remove_afterwards )
 	{
 		try{
-			reseed.setEnabled( false );						
+			reseed_param.setEnabled( false );						
 
 			log.log( "DHT " + (first?"":"re-") + "integration starts" );
 		
@@ -441,16 +463,14 @@ DHTPluginImpl
 			}
 			
 			long	end = SystemTime.getCurrentTime();
-	
-			integrated_time	= end;
-			
+				
 			log.log( "DHT " + (first?"":"re-") + "integration complete: elapsed = " + (end-start));
 			
 			dht.print();
 			
 		}finally{
 			
-			reseed.setEnabled( true );						
+			reseed_param.setEnabled( true );						
 		}
 	}
 	
@@ -480,7 +500,7 @@ DHTPluginImpl
 					// only try boostrapping off connected peers on the main network as it is unlikely
 					// any of them are running CVS and hence the boostrap will fail
 				
-				if ( network == DHT.NW_MAIN ){
+				if ( network == DHT.NW_MAIN || network == DHT.NW_MAIN_V6 ){
 					
 						// first look for peers to directly import
 					
@@ -614,19 +634,13 @@ outer:
 	getSeedAddress()
 	{
 		try{
-			return( InetAddress.getByName( SEED_ADDRESS ));
+			return( InetAddress.getByName( v6?SEED_ADDRESS_V6:SEED_ADDRESS_V4 ));
 			
 		}catch( Throwable e ){
+				
+			Debug.printStackTrace( e );
 			
-			try{
-				return( InetAddress.getByName( SEED_IP ));
-				
-			}catch( Throwable f ){
-				
-				log.log(f);
-				
-				return( null );
-			}
+			return( null );
 		}
 	}
 	
@@ -709,7 +723,7 @@ outer:
 						
 							if ( listener != null ){
 								
-								listener.complete( timeout );
+								listener.complete( key, timeout );
 							}
 						}
 					});
@@ -805,7 +819,7 @@ outer:
 							
 							if ( listener != null ){
 								
-								listener.complete( _timeout );
+								listener.complete( key, _timeout );
 							}
 						}
 					});
@@ -876,7 +890,87 @@ outer:
 							
 								if ( listener != null ){
 								
-									listener.complete( timeout );
+									listener.complete( key, timeout );
+								}
+							}			
+						});
+	}
+	
+	public void
+	remove(
+		final DHTPluginContact[]			targets,
+		final byte[]						key,
+		final String						description,
+		final DHTPluginOperationListener	listener )
+	{
+		DHTTransportContact[]	t_contacts = new DHTTransportContact[ targets.length ];
+		
+		for (int i=0;i<targets.length;i++){
+			
+			t_contacts[i] = ((DHTPluginContactImpl)targets[i]).getContact();
+		}
+		
+		dht.remove( 	t_contacts,
+						key,
+						description,
+						new DHTOperationListener()
+						{
+							public void
+							searching(
+								DHTTransportContact	contact,
+								int					level,
+								int					active_searches )
+							{
+								String	indent = "";
+								
+								for (int i=0;i<level;i++){
+									
+									indent += "  ";
+								}
+								
+								// log.log( indent + "Remove: level = " + level + ", active = " + active_searches + ", contact = " + contact.getString());
+							}
+							
+							public void
+							found(
+								DHTTransportContact	contact )
+							{
+							}
+
+							public void
+							diversified()
+							{
+							}
+							
+							public void
+							read(
+								DHTTransportContact	contact,
+								DHTTransportValue	value )
+							{
+								// log.log( "Remove: read " + value.getString() + " from " + contact.getString());
+							}
+							
+							public void
+							wrote(
+								DHTTransportContact	contact,
+								DHTTransportValue	value )
+							{
+								// log.log( "Remove: wrote " + value.getString() + " to " + contact.getString());
+								if ( listener != null ){
+									
+									listener.valueWritten( new DHTPluginContactImpl( DHTPluginImpl.this, contact ), mapValue( value ));
+								}
+							}
+							
+							public void
+							complete(
+								boolean				timeout )
+							{
+								// log.log( "Remove: complete, timeout = " + timeout );
+							
+								if ( listener != null ){
+								
+									listener.complete( key, timeout );
 								}
 							}			
 						});
