@@ -41,15 +41,7 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
-import org.gudy.azureus2.core3.util.AEMonitor;
-import org.gudy.azureus2.core3.util.BDecoder;
-import org.gudy.azureus2.core3.util.BEncoder;
-import org.gudy.azureus2.core3.util.ByteArrayHashMap;
-import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.core3.util.DisplayFormatters;
-import org.gudy.azureus2.core3.util.HashWrapper;
-import org.gudy.azureus2.core3.util.SHA1Simple;
-import org.gudy.azureus2.core3.util.SystemTime;
+import org.gudy.azureus2.core3.util.*;
 
 import com.aelitis.azureus.core.dht.DHT;
 import com.aelitis.azureus.core.dht.DHTLogger;
@@ -137,7 +129,7 @@ DHTPluginStorageManager
 		log			= _log;
 		data_dir	= _data_dir;
 		
-		data_dir.mkdirs();
+		FileUtil.mkdirs(data_dir);
 		
 		readRecentAddresses();
 		
@@ -202,9 +194,15 @@ DHTPluginStorageManager
 			boolean	ok = false;
 			
 			try{
-				dos = new DataOutputStream( new FileOutputStream( saving ));
+				FileOutputStream fos = new FileOutputStream( saving );
+					
+				dos = new DataOutputStream(fos);
 					
 				dht.exportState( dos, 32 );
+					
+				dos.flush();
+					
+				fos.getFD().sync();
 			
 				ok	= true;
 				
@@ -410,6 +408,10 @@ DHTPluginStorageManager
 						
 					os.write( data );
 				
+					os.flush();
+				
+					os.getFD().sync();
+				
 					os.close();
 				
 					ok	= true;
@@ -1460,6 +1462,43 @@ DHTPluginStorageManager
 		}
 	}
 	
+	public void
+	setStorageForKey(
+		String	key,
+		byte[]	data )
+	{
+		try{
+			storage_mon.enter();
+			
+			Map	map = readMapFromFile( "general" );
+			
+			map.put( key, data );
+			
+			writeMapToFile( map, "general" );
+			
+		}finally{
+			
+			storage_mon.exit();
+		}
+	}
+	
+	public byte[]
+	getStorageForKey(
+		String	key )
+	{
+		try{
+			storage_mon.enter();
+			
+			Map	map = readMapFromFile( "general" );
+			
+			return((byte[])map.get( key ));
+			
+		}finally{
+			
+			storage_mon.exit();
+		}
+	}
+	
 	protected static class
 	keyBlock
 		implements DHTStorageBlock
