@@ -24,14 +24,17 @@ package com.aelitis.azureus.plugins.extseed.impl;
 
 import java.util.*;
 
+import org.gudy.azureus2.core3.config.impl.TransferSpeedValidator;
 import org.gudy.azureus2.core3.util.AESemaphore;
 import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.core3.util.SystemTime;
 import org.gudy.azureus2.plugins.PluginInterface;
 import org.gudy.azureus2.plugins.clientid.ClientIDGenerator;
 import org.gudy.azureus2.plugins.download.Download;
 import org.gudy.azureus2.plugins.peers.Peer;
 import org.gudy.azureus2.plugins.peers.PeerManager;
 import org.gudy.azureus2.plugins.peers.PeerReadRequest;
+import org.gudy.azureus2.plugins.peers.PeerStats;
 import org.gudy.azureus2.plugins.peers.Piece;
 import org.gudy.azureus2.plugins.torrent.Torrent;
 import org.gudy.azureus2.plugins.utils.Monitor;
@@ -39,15 +42,20 @@ import org.gudy.azureus2.plugins.utils.PooledByteBuffer;
 import org.gudy.azureus2.plugins.utils.Semaphore;
 
 import com.aelitis.azureus.plugins.extseed.ExternalSeedException;
+import com.aelitis.azureus.plugins.extseed.ExternalSeedPeer;
 import com.aelitis.azureus.plugins.extseed.ExternalSeedPlugin;
 import com.aelitis.azureus.plugins.extseed.ExternalSeedReader;
 import com.aelitis.azureus.plugins.extseed.ExternalSeedReaderListener;
+import com.aelitis.azureus.plugins.extseed.util.ExternalSeedHTTPDownloaderListener;
 
 public abstract class 
 ExternalSeedReaderImpl 
 	implements ExternalSeedReader
 {
-	protected static final int	RECONNECT_DEFAULT = 30*1000;
+	public static final int	RECONNECT_DEFAULT 			= 30*1000;
+	public static final int INITIAL_DELAY				= 30*1000;
+	public static final int STALLED_DOWNLOAD_SPEED		= 20*1024;
+	public static final int STALLED_PEER_SPEED			= 5*1024;
 	
 
 	private ExternalSeedPlugin	plugin;
@@ -191,9 +199,15 @@ ExternalSeedReaderImpl
 	
 	protected void
 	setReconnectDelay(
-		int	delay )
+		int			delay,
+		boolean		reset_failures )
 	{
 		reconnect_delay = delay;
+		
+		if ( reset_failures ){
+			
+			consec_failures = 0;
+		}
 	}
 	
 	protected boolean
@@ -202,7 +216,7 @@ ExternalSeedReaderImpl
 		Peer		peer,
 		long		time_since_start )
 	{
-		boolean	early_days = time_since_start < 30000;
+		boolean	early_days = time_since_start < INITIAL_DELAY;
 		
 		try{
 
@@ -249,8 +263,65 @@ ExternalSeedReaderImpl
 				// now the more interesting stuff
 			
 			if ( transient_seed ){
+						
+					// kick any existing peers that are running too slowly if the download appears
+					// to be stalled		
+				
+				Peer[]	existing_peers = peer_manager.getPeers( getIP());
+						
+				int	existing_peer_count = existing_peers.length;
+				
+				int	global_limit	= TransferSpeedValidator.getGlobalDownloadRateLimitBytesPerSecond();
+				
+				if ( global_limit > 0 ){
+				
+						// if we have a global limit in force and we are near it then no point in
+						// activating 
+					
+					int current_down = plugin.getGlobalDownloadRateBytesPerSec();
+					
+					if ( global_limit - current_down < 5*1024 ){
+						
+						return( false );
+					}
+				}
+				
+				int	download_limit  = peer_manager.getDownloadRateLimitBytesPerSecond();
+						
+				if ( global_limit > 0 && global_limit < download_limit ){
+					
+					download_limit = global_limit;
+				}
+				
+				if ( 	( download_limit == 0 || download_limit > STALLED_DOWNLOAD_SPEED + 5*1024 ) &&
+						peer_manager.getStats().getDownloadAverage() < STALLED_DOWNLOAD_SPEED ){
+					
+					for (int i=0;i<existing_peers.length;i++){
+					
+						Peer	existing_peer = existing_peers[i];
+						
+							// no point in booting ourselves!
+						
+						if ( existing_peer instanceof ExternalSeedPeer ){
+							
+							continue;
+						}
+						
+						PeerStats stats = existing_peer.getStats();
+						
+						if ( stats.getTimeSinceConnectionEstablished() > INITIAL_DELAY ){
+							
+							if ( stats.getDownloadAverage() < STALLED_PEER_SPEED ){
 								
-				if ( peer_manager.getPeers( getIP()).length == 0 ){
+								existing_peer.close( "Replacing slow peer with web-seed", false, false );
+								
+								existing_peer_count--;
+							}
+						}
+					}
+				}
+				
+				if ( existing_peer_count == 0 ){
 					
 					// check to see if we have pending connections to the same address 
 								
@@ -377,7 +448,7 @@ ExternalSeedReaderImpl
 				
 				if ( active ){
 					
-					if ( now - peer_manager_change_time > 30000 && readyToDeactivate( peer_manager, peer )){
+					if ( now - peer_manager_change_time > INITIAL_DELAY && readyToDeactivate( peer_manager, peer )){
 													
 						setActive( false );			
 					}
@@ -806,11 +877,110 @@ ExternalSeedReaderImpl
 		
 		return( requests.size());
 	}
-
-	protected abstract void
+	    
+	public byte[]
+   	read(
+   		int			piece_number,
+   		int			piece_offset,
+   		int			length,
+   		final int	timeout )
+   	
+   		throws ExternalSeedException
+   	{
+   		final byte[] 	result = new byte[ length ];
+   		
+   		ExternalSeedHTTPDownloaderListener listener =
+   			new ExternalSeedHTTPDownloaderListener()
+   			{
+   				private int		bp;
+   				private long	start_time = SystemTime.getCurrentTime();
+   				
+   				public byte[]
+   	        	getBuffer()
+   	        	
+   	        		throws ExternalSeedException
+   	        	{
+   					return( result );
+   	        	}
+   	        	
+   	        	public void
+   	        	setBufferPosition(
+   	        		int	position )
+   	        	{
+   	        		bp = position;
+   	        	}
+   	        	
+   	        	public int
+   	        	getBufferPosition()
+   	        	{
+   	        		return( bp );
+   	        	}
+   	        	
+   	        	public int
+   	        	getBufferLength()
+   	        	{
+   	        		return( result.length );
+   	        	}
+   	        	
+   	        	public int
+   	        	getPermittedBytes()
+   	        	
+   	        		throws ExternalSeedException
+   	        	{
+   	        		return( result.length );
+   	        	}
+   	        	
+   	        	public int
+   	        	getPermittedTime()
+   	        	{
+   	        		if ( timeout == 0 ){
+   	        			
+   	        			return( 0 );
+   	        		}
+   	        		
+   	        		int	rem = timeout - (int)( SystemTime.getCurrentTime() - start_time );
+   	        		
+   	        		if ( rem <= 0 ){
+   	        			
+   	        			return( -1 );
+   	        		}
+   	        		
+   	        		return( rem );
+   	        	}
+   	        	
+   	        	public void
+   	        	reportBytesRead(
+   	        		int		num )
+   	        	{        		
+   	        	}
+   	        	
+   	        	public void
+   	        	done()
+   	        	{        		
+   	        	}
+   			};
+   				
+   		readData( piece_number, piece_offset, length, listener );
+   		
+   		return( result );
+   	}
+	
+	protected void
 	readData(
 		ExternalSeedReaderRequest	request )
 	
+		throws ExternalSeedException
+	{	
+		readData( request.getStartPieceNumber(), request.getStartPieceOffset(), request.getLength(), request );
+	}
+	
+	protected abstract void
+	readData(
+		int									piece_number,
+		int									piece_offset,
+		int									length,
+		ExternalSeedHTTPDownloaderListener	listener )
+	
 		throws ExternalSeedException;
 	
 	protected void
