@@ -69,6 +69,13 @@ DownloadRemoveRulesPlugin
 	
 	protected BooleanParameter 	remove_update_torrents; 
 
+	public static void
+	load(
+		PluginInterface		plugin_interface )
+	{
+		plugin_interface.getPluginProperties().setProperty( "plugin.version", 	"1.0" );
+		plugin_interface.getPluginProperties().setProperty( "plugin.name", "Download Remove Rules" );
+	}
 	
 	public void
 	initialize(
@@ -78,9 +85,6 @@ DownloadRemoveRulesPlugin
 		
 		HostNameToIPResolver.addResolverRequest( AELITIS_TRACKER, this );
 		
-		plugin_interface.getPluginProperties().setProperty( "plugin.version", 	"1.0" );
-		plugin_interface.getPluginProperties().setProperty( "plugin.name", "Download Remove Rules" );
-
 		log = plugin_interface.getLogger().getChannel("DLRemRules");
 
 		BasicPluginConfigModel	config = plugin_interface.getUIManager().createBasicPluginConfigModel( "torrents", "download.removerules.name" );
@@ -362,15 +366,17 @@ DownloadRemoveRulesPlugin
 							try{
 								download.remove();
 								
-								String msg = 
-									plugin_interface.getUtilities().getLocaleUtilities().getLocalisedMessageText(
-										"download.removerules.removed.ok",
-										new String[]{ download.getName() });
-									
-								log.logAlert( 
-									LoggerChannel.LT_INFORMATION,
-									msg );
-							
+								String msg = plugin_interface.getUtilities().getLocaleUtilities().getLocalisedMessageText(
+										"download.removerules.removed.ok", new String[] {
+											download.getName()
+										});
+
+								if (download.getFlag(Download.FLAG_LOW_NOISE)) {
+									log.log(download.getTorrent(), LoggerChannel.LT_INFORMATION,
+											msg);
+								} else {
+									log.logAlert(LoggerChannel.LT_INFORMATION, msg);
+								}
 							}catch( Throwable e ){
 								
 								log.logAlert( "Automatic removal of download '" + download.getName() + "' failed", e );
