@@ -32,6 +32,7 @@ import org.gudy.azureus2.plugins.*;
 import org.gudy.azureus2.plugins.logging.*;
 import org.gudy.azureus2.plugins.torrent.*;
 import org.gudy.azureus2.plugins.tracker.*;
+import org.gudy.azureus2.plugins.utils.DelayedTask;
 import org.gudy.azureus2.plugins.sharing.*;
 import org.gudy.azureus2.plugins.download.*;
 
@@ -56,15 +57,20 @@ ShareHosterPlugin
 		
 	protected boolean			initialised	= false;
 	
+	public static void
+	load(
+		PluginInterface		plugin_interface )
+	{
+		plugin_interface.getPluginProperties().setProperty( "plugin.version", 	"1.0" );
+		plugin_interface.getPluginProperties().setProperty( "plugin.name", 		"Share Hoster" );
+	}
+	
 	public void 
 	initialize(
 		PluginInterface _plugin_interface )
 	{
 		plugin_interface = _plugin_interface;
 		
-		plugin_interface.getPluginProperties().setProperty( "plugin.version", 	"1.0" );
-		plugin_interface.getPluginProperties().setProperty( "plugin.name", 		"Share Hoster" );
-
 		log	= plugin_interface.getLogger().getChannel("ShareHosterPlugin");
 		
 		log.log( LoggerChannel.LT_INFORMATION, "ShareHosterPlugin: initialisation starts");
@@ -75,19 +81,16 @@ ShareHosterPlugin
 	public void
 	initializationComplete()
 	{
-			// take this process off the main thread as we don't want share-recreation to hang
-			// initialisation
-		
-		plugin_interface.getUtilities().createThread(
-			"ShareHosterImplugin::init",
-			new Runnable()
+		final DelayedTask dt = plugin_interface.getUtilities().createDelayedTask(new Runnable()
 			{
-				public void
-				run()
+				public void 
+				run() 
 				{
 					initialise();
 				}
 			});
+		
+		dt.queue();
 	}
 	
 	protected void
