@@ -19,7 +19,8 @@
  */
 package com.aelitis.azureus.plugins.startstoprules.defaultplugin;
 
-import java.util.Map;
+import java.util.Iterator;
+import java.util.List;
 
 import org.gudy.azureus2.core3.config.COConfigurationListener;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
@@ -27,7 +28,6 @@ import org.gudy.azureus2.core3.util.AEMonitor;
 import org.gudy.azureus2.core3.util.SystemTime;
 import org.gudy.azureus2.plugins.PluginConfig;
 import org.gudy.azureus2.plugins.download.Download;
-import org.gudy.azureus2.plugins.download.DownloadScrapeResult;
 import org.gudy.azureus2.plugins.download.DownloadStats;
 import org.gudy.azureus2.plugins.logging.LoggerChannel;
 
@@ -180,6 +180,16 @@ public class DefaultRankCalculator implements Comparable {
 			"StartStopRules:downloadData");
 
 	private final StartStopRulesDefaultPlugin rules;
+	
+	
+	
+	// state-caches for sorting
+	
+	int lastModifiedScrapeResultPeers = 0;
+	int lastModifiedScrapeResultSeeds = 0;
+	int lastModifiedShareRatio = 0;
+	// modified by a listener in StartStopRulesDefaultPlugin
+	boolean lastScrapeResultOk = false;
 
 	/**
 	 * Default Initializer
@@ -269,9 +279,9 @@ public class DefaultRankCalculator implements Comparable {
 		boolean aIsComplete = dlData.dl.isComplete();
 		boolean bIsComplete = dl.isComplete();
 		if (aIsComplete && !bIsComplete)
-			return 1;
-		if (!aIsComplete && bIsComplete)
 			return -1;
+		if (!aIsComplete && bIsComplete)
+			return 1;
 
 		// Test FP
 		if (dlData.bIsFirstPriority && !bIsFirstPriority)
@@ -290,8 +300,8 @@ public class DefaultRankCalculator implements Comparable {
 
 		if (iRankType != StartStopRulesDefaultPlugin.RANK_TIMED) {
 			// Test Large/Small Swarm pref
-			int numPeersThem = rules.calcPeersNoUs(dlData.dl);
-			int numPeersUs = rules.calcPeersNoUs(dl);
+			int numPeersThem = dlData.lastModifiedScrapeResultPeers;
+			int numPeersUs = lastModifiedScrapeResultPeers;
 			if (bPreferLargerSwarms)
 				value = numPeersThem - numPeersUs;
 			else
@@ -300,9 +310,7 @@ public class DefaultRankCalculator implements Comparable {
 				return value;
 
 			// Test Share Ratio
-			int shareRatioUs = dl.getStats().getShareRatio();
-			int shareRatioThem = dlData.dl.getStats().getShareRatio();
-			value = shareRatioUs - shareRatioThem;
+			value = lastModifiedShareRatio - dlData.lastModifiedShareRatio;
 			if (value != 0)
 				return value;
 		}
@@ -317,7 +325,7 @@ public class DefaultRankCalculator implements Comparable {
 	
 	public boolean isForceActive() {
 		DownloadStats stats = dl.getStats();
-		return System.currentTimeMillis() - stats.getTimeStarted() <= FORCE_ACTIVE_FOR;
+		return SystemTime.getCurrentTime() - stats.getTimeStarted() <= FORCE_ACTIVE_FOR;
 	}
 
 	/**
@@ -335,14 +343,14 @@ public class DefaultRankCalculator implements Comparable {
 		// - Must be above speed threshold, or started less than 30s ago
 		if (state != Download.ST_DOWNLOADING) {
 			bIsActive = false;
-		} else if (System.currentTimeMillis() - stats.getTimeStarted() <= FORCE_ACTIVE_FOR) {
+		} else if (SystemTime.getCurrentTime() - stats.getTimeStarted() <= FORCE_ACTIVE_FOR) {
 			bIsActive = true;
 		} else {
 			// activity based on DL Average
 			bIsActive = (stats.getDownloadAverage() >= minSpeedForActiveDL);
 
 			if (bActivelyDownloading != bIsActive) {
-				long now = System.currentTimeMillis();
+				long now = SystemTime.getCurrentTime();
 				// Change
 				if (lDLActivelyChangedOn == -1) {
 					// Start Timer
@@ -385,7 +393,7 @@ public class DefaultRankCalculator implements Comparable {
 		// when bAutoStart0Peers
 		if (iRankType == StartStopRulesDefaultPlugin.RANK_TIMED
 				&& !isFirstPriority()
-				&& !(bAutoStart0Peers && rules.calcPeersNoUs(dl) == 0 && scrapeResultOk(dl))) {
+				&& !(bAutoStart0Peers && rules.calcPeersNoUs(dl) == 0 && lastScrapeResultOk)) {
 			bIsActive = (state == Download.ST_SEEDING);
 
 		} else if (state != Download.ST_SEEDING
@@ -393,13 +401,13 @@ public class DefaultRankCalculator implements Comparable {
 			// Not active if we aren't seeding
 			// Not active if we are AutoStarting 0 Peers, and peer count == 0
 			bIsActive = false;
-		} else if (System.currentTimeMillis() - stats.getTimeStarted() <= FORCE_ACTIVE_FOR) {
+		} else if (SystemTime.getCurrentTime() - stats.getTimeStarted() <= FORCE_ACTIVE_FOR) {
 			bIsActive = true;
 		} else {
 			bIsActive = (stats.getUploadAverage() >= minSpeedForActiveSeeding);
 
 			if (bActivelySeeding != bIsActive) {
-				long now = System.currentTimeMillis();
+				long now = SystemTime.getCurrentTime();
 				// Change
 				if (lCDActivelyChangedOn == -1) {
 					// Start Timer
@@ -453,13 +461,12 @@ public class DefaultRankCalculator implements Comparable {
 
 			// here we are seeding
 
-			int shareRatio = stats.getShareRatio();
+			lastModifiedShareRatio = stats.getShareRatio();
+			lastModifiedScrapeResultPeers = rules.calcPeersNoUs(dl);
+			lastModifiedScrapeResultSeeds = rules.calcSeedsNoUs(dl);
 
-			int numPeers = rules.calcPeersNoUs(dl);
-			int numSeeds = rules.calcSeedsNoUs(dl);
-
-			boolean bScrapeResultsOk = (numPeers > 0 || numSeeds > 0
-					|| scrapeResultOk(dl)) && (numPeers >= 0 && numSeeds >= 0);
+			boolean bScrapeResultsOk = (lastModifiedScrapeResultPeers > 0 || lastModifiedScrapeResultSeeds > 0
+					|| lastScrapeResultOk) && (lastModifiedScrapeResultPeers >= 0 && lastModifiedScrapeResultSeeds >= 0);
 
 			if (!isFirstPriority()) {
 				// Check Ignore Rules
@@ -467,26 +474,26 @@ public class DefaultRankCalculator implements Comparable {
 				// (we don't want leechers circumventing the 0.5 rule)
 
 				//0 means unlimited
-				if (iIgnoreShareRatio != 0 && shareRatio >= iIgnoreShareRatio
-						&& (numSeeds >= iIgnoreShareRatio_SeedStart || !scrapeResultOk(dl))
-						&& shareRatio != -1) {
+				if (iIgnoreShareRatio != 0 && lastModifiedShareRatio >= iIgnoreShareRatio
+						&& (lastModifiedScrapeResultSeeds >= iIgnoreShareRatio_SeedStart || !bScrapeResultsOk)
+						&& lastModifiedShareRatio != -1) {
 					
 					if (rules.bDebugLog)
-						sExplainSR += "  shareratio met: shareRatio(" + shareRatio
+						sExplainSR += "  shareratio met: shareRatio(" + lastModifiedShareRatio
 								+ ") >= " + iIgnoreShareRatio + "\n";
 
 					dl.setSeedingRank(SR_SHARERATIOMET);
 					return SR_SHARERATIOMET;
 				} else if (rules.bDebugLog && iIgnoreShareRatio != 0
-						&& shareRatio >= iIgnoreShareRatio) {
+						&& lastModifiedShareRatio >= iIgnoreShareRatio) {
 					sExplainSR += "  shareratio NOT met: ";
-					if (numSeeds >= iIgnoreShareRatio_SeedStart) 
-						sExplainSR += numSeeds + " below seed threshold of "
+					if (lastModifiedScrapeResultSeeds >= iIgnoreShareRatio_SeedStart) 
+						sExplainSR += lastModifiedScrapeResultSeeds + " below seed threshold of "
 								+ iIgnoreShareRatio_SeedStart;
 					sExplainSR += "\n";
 				}
 
-				if (numPeers == 0 && bScrapeResultsOk) {
+				if (lastModifiedScrapeResultPeers == 0 && bScrapeResultsOk) {
 					// If both bIgnore0Peers and bFirstPriorityIgnore0Peer are on,
 					// we won't know which one it is at this point.
 					// We have to use the normal SR_0PEERS in case it isn't FP
@@ -505,7 +512,7 @@ public class DefaultRankCalculator implements Comparable {
 //						dl.setSeedingRank(SR_FP0PEERS);
 //						return SR_FP0PEERS;
 //					}
-				} else if (rules.bDebugLog && numPeers == 0) {
+				} else if (rules.bDebugLog && lastModifiedScrapeResultPeers == 0) {
 					sExplainSR += "  0 Peer Ignore rule NOT applied: Scrape invalid\n";
 				}
 
@@ -522,10 +529,10 @@ public class DefaultRankCalculator implements Comparable {
 //				}
 
 				//0 means disabled
-				if ((iIgnoreSeedCount != 0) && (numSeeds >= iIgnoreSeedCount)) {
+				if ((iIgnoreSeedCount != 0) && (lastModifiedScrapeResultSeeds >= iIgnoreSeedCount)) {
 					if (rules.bDebugLog)
 						sExplainSR += "  SeedCount Ignore rule met.  numSeeds("
-								+ numSeeds + " >= iIgnoreSeedCount(" + iIgnoreSeedCount + ")\n"; 
+								+ lastModifiedScrapeResultSeeds + " >= iIgnoreSeedCount(" + iIgnoreSeedCount + ")\n"; 
 
 					dl.setSeedingRank(SR_NUMSEEDSMET);
 					return SR_NUMSEEDSMET;
@@ -534,10 +541,10 @@ public class DefaultRankCalculator implements Comparable {
 				// Ignore when P:S ratio met
 				// (More Peers for each Seed than specified in Config)
 				//0 means never stop
-				if (iIgnoreRatioPeers != 0 && numSeeds != 0) {
-					float ratio = (float) numPeers / numSeeds;
+				if (iIgnoreRatioPeers != 0 && lastModifiedScrapeResultSeeds != 0) {
+					float ratio = (float) lastModifiedScrapeResultPeers / lastModifiedScrapeResultSeeds;
 					if (ratio <= iIgnoreRatioPeers
-							&& numSeeds >= iIgnoreRatioPeers_SeedStart) {
+							&& lastModifiedScrapeResultSeeds >= iIgnoreRatioPeers_SeedStart) {
 
 						if (rules.bDebugLog)
 							sExplainSR += "  P:S Ignore rule met.  ratio(" + ratio
@@ -619,22 +626,30 @@ public class DefaultRankCalculator implements Comparable {
 
 			// SeedCount and SPRatio require Scrape Results..
 			if (bScrapeResultsOk) {
-				if ((iRankType == StartStopRulesDefaultPlugin.RANK_SEEDCOUNT)
-						&& (iRankTypeSeedFallback == 0 || iRankTypeSeedFallback > numSeeds)) {
-					if (numSeeds < 10000)
-						newSR = 10000 - numSeeds;
+				if ( iRankType == StartStopRulesDefaultPlugin.RANK_PEERCOUNT )
+				{
+					if(lastModifiedScrapeResultPeers > lastModifiedScrapeResultSeeds * 10)
+						newSR = 100 * lastModifiedScrapeResultPeers * 10;
+					else
+						newSR = 100 * lastModifiedScrapeResultPeers * lastModifiedScrapeResultPeers/(lastModifiedScrapeResultSeeds+1);
+				}				
+				else if ((iRankType == StartStopRulesDefaultPlugin.RANK_SEEDCOUNT)
+						&& (iRankTypeSeedFallback == 0 || iRankTypeSeedFallback > lastModifiedScrapeResultSeeds))
+				{
+					if (lastModifiedScrapeResultSeeds < 10000)
+						newSR = 10000 - lastModifiedScrapeResultSeeds;
 					else
 						newSR = 1;
 					// shift over to make way for fallback
 					newSR *= SEEDONLY_SHIFT;
 
 				} else { // iRankType == RANK_SPRATIO or we are falling back
-					if (numPeers != 0) {
-						if (numSeeds == 0) {
-							if (numPeers >= minPeersToBoostNoSeeds)
+					if (lastModifiedScrapeResultPeers != 0) {
+						if (lastModifiedScrapeResultSeeds == 0) {
+							if (lastModifiedScrapeResultPeers >= minPeersToBoostNoSeeds)
 								newSR += SPRATIO_BASE_LIMIT;
 						} else { // numSeeds != 0 && numPeers != 0
-							float x = (float) numSeeds / numPeers;
+							float x = (float) lastModifiedScrapeResultSeeds / lastModifiedScrapeResultPeers;
 							newSR += SPRATIO_BASE_LIMIT / ((x + 1) * (x + 1));
 						}
 					}
@@ -698,24 +713,26 @@ public class DefaultRankCalculator implements Comparable {
 			return false;
 		}
 		
-		int numPeers = rules.calcPeersNoUs(dl);
-		int numSeeds = rules.calcSeedsNoUs(dl);
-
-		// FP while our content doesn't have another seed
-		if (dl.getState() == Download.ST_SEEDING) {
-			Map contentMap = dl.getMapAttribute(rules.torrentAttributeContent);
-			if (contentMap != null && contentMap.containsKey("ourContent")) {
-				if (((Long) contentMap.get("ourContent")).longValue() == 1) {
-					if (dl.getStats().getAvailability() < 2 && numSeeds == 0) {
-						return true;
-					}
-				}
+		List listeners = rules.getFPListeners();
+		StringBuffer fp_listener_debug = null;
+		if (rules.bDebugLog && !listeners.isEmpty()) {fp_listener_debug = new StringBuffer();}
+		for (Iterator iter = listeners.iterator(); iter.hasNext();) {
+			StartStopRulesFPListener l = (StartStopRulesFPListener) iter.next();
+			boolean result = l.isFirstPriority(dl, lastModifiedScrapeResultSeeds, lastModifiedScrapeResultPeers, fp_listener_debug);
+			if (fp_listener_debug != null && fp_listener_debug.length() > 0) {
+				char last_ch = fp_listener_debug.charAt(fp_listener_debug.length()-1);
+				if (last_ch != '\n') {fp_listener_debug.append('\n');}
+				sExplainFP += fp_listener_debug;
+				fp_listener_debug.setLength(0);
+			}
+			if (result) {
+				return true;
 			}
 		}
 
 		// FP doesn't apply when S:P >= set SPratio (SPratio = 0 means ignore)
-		if (numPeers > 0 && numSeeds > 0
-				&& (numSeeds / numPeers) >= iFirstPriorityIgnoreSPRatio
+		if (lastModifiedScrapeResultPeers > 0 && lastModifiedScrapeResultSeeds > 0
+				&& (lastModifiedScrapeResultSeeds / lastModifiedScrapeResultPeers) >= iFirstPriorityIgnoreSPRatio
 				&& iFirstPriorityIgnoreSPRatio != 0) {
 			if (rules.bDebugLog)
 				sExplainFP += "Not FP: S:P >= " + iFirstPriorityIgnoreSPRatio + ":1\n";
@@ -723,7 +740,7 @@ public class DefaultRankCalculator implements Comparable {
 		}
 
 		//not FP if no peers  //Nolar, 2105 - Gouss, 2203
-		if (numPeers == 0 && scrapeResultOk(dl) && bFirstPriorityIgnore0Peer) {
+		if (lastModifiedScrapeResultPeers == 0 && lastScrapeResultOk && bFirstPriorityIgnore0Peer) {
 			if (rules.bDebugLog)
 				sExplainFP += "Not FP: 0 peers\n";
 			return false;
@@ -829,7 +846,7 @@ public class DefaultRankCalculator implements Comparable {
 			int numSeeds = rules.calcSeedsNoUs(dl);
 
 			if (iIgnoreShareRatio != 0 && shareRatio >= iIgnoreShareRatio
-					&& (numSeeds >= iIgnoreShareRatio_SeedStart || !scrapeResultOk(dl))
+					&& (numSeeds >= iIgnoreShareRatio_SeedStart || !lastScrapeResultOk)
 					&& shareRatio != -1) {
 				return true;
 			}
@@ -851,9 +868,4 @@ public class DefaultRankCalculator implements Comparable {
 
 		return false;
 	}
-
-	private boolean scrapeResultOk(Download download) {
-		DownloadScrapeResult sr = download.getLastScrapeResult();
-		return (sr.getResponseType() == DownloadScrapeResult.RT_SUCCESS);
-	}
 }
