@@ -26,15 +26,13 @@ import java.util.*;
 import org.gudy.azureus2.core3.config.COConfigurationListener;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.util.*;
-import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;
-
-import com.aelitis.azureus.plugins.startstoprules.defaultplugin.ui.swt.StartStopRulesDefaultPluginSWTUI;
-
-import org.gudy.azureus2.plugins.*;
+import org.gudy.azureus2.plugins.Plugin;
+import org.gudy.azureus2.plugins.PluginConfig;
+import org.gudy.azureus2.plugins.PluginInterface;
+import org.gudy.azureus2.plugins.PluginListener;
 import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.plugins.download.*;
 import org.gudy.azureus2.plugins.logging.LoggerChannel;
-import org.gudy.azureus2.plugins.torrent.TorrentAttribute;
 import org.gudy.azureus2.plugins.ui.UIInstance;
 import org.gudy.azureus2.plugins.ui.UIManager;
 import org.gudy.azureus2.plugins.ui.UIManagerListener;
@@ -42,7 +40,14 @@ import org.gudy.azureus2.plugins.ui.config.ConfigSection;
 import org.gudy.azureus2.plugins.ui.menus.MenuItem;
 import org.gudy.azureus2.plugins.ui.menus.MenuItemListener;
 import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;
-import org.gudy.azureus2.plugins.ui.tables.*;
+import org.gudy.azureus2.plugins.ui.tables.TableColumn;
+import org.gudy.azureus2.plugins.ui.tables.TableContextMenuItem;
+import org.gudy.azureus2.plugins.ui.tables.TableManager;
+import org.gudy.azureus2.plugins.ui.tables.TableRow;
+import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;
+
+import com.aelitis.azureus.core.util.CopyOnWriteList;
+import com.aelitis.azureus.plugins.startstoprules.defaultplugin.ui.swt.StartStopRulesDefaultPluginSWTUI;
 
 /** Handles Starting and Stopping of torrents.
  *
@@ -77,6 +82,9 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 
 	/** Rank completed torrents using a timed rotation of minTimeAlive */
 	public static final int RANK_TIMED = 3;
+	
+	/** Rank completed torrents using the peers count, weighted by the seeds to peers ratio */
+	public static final int RANK_PEERCOUNT = 4;
 
 	/** 
 	 * Force at least one check every period of time (in ms).  
@@ -93,13 +101,15 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 	 * Interval in ms between checks to see if the {@link #somethingChanged} 
 	 * flag changed 
 	 */
-	private static final int PROCESS_CHECK_PERIOD = 500;
+	private static final int PROCESS_CHECK_PERIOD = 120;
 
 	/** Wait xx ms before starting completed torrents (so scrapes can come in) */
 	private static final int MIN_SEEDING_STARTUP_WAIT = 20000;
 
 	/** Wait at least xx ms for first scrape, before starting completed torrents */
 	private static final int MIN_FIRST_SCRAPE_WAIT = 90000;
+	
+	private static final float IGNORE_SLOT_THRESHOLD_FACTOR = 0.9f;
 
 	// Core/Plugin classes
 	private AEMonitor this_mon = new AEMonitor("StartStopRules");
@@ -117,12 +127,19 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 
 	/** Map to relate downloadData to a Download */
 	private static Map downloadDataMap = AEMonitor.getSynchronisedMap(new HashMap());
+	/**
+	 * this is used to reduce the number of comperator invocations
+	 * by keeping a mostly sorted copy around, must be nulled whenever the map is changed
+	 */
+	private volatile DefaultRankCalculator[] sortedArrayCache;
 
 	private volatile boolean closingDown;
 
 	private volatile boolean somethingChanged;
 
-	private Set ranksToRecalc = new HashSet();
+	private Set ranksToRecalc = new LightHashSet();
+	
+	private AEMonitor ranksToRecalc_mon = new AEMonitor("ranksToRecalc");
 
 	/** When rules class started.  Used for initial waiting logic */
 	private long startedOn;
@@ -147,6 +164,10 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 	private boolean _maxActiveWhenSeedingEnabled;
 
 	private int _maxActiveWhenSeeding;
+	
+	private int globalDownloadLimit;
+	private int globalUploadLimit;
+	private int globalUploadWhenSeedingLimit;
 
 	private int maxDownloads;
 
@@ -157,8 +178,8 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 	private long minTimeAlive;
 
 	private boolean bAutoStart0Peers;
-
-	private int iMaxUploadSpeed;
+	
+	private boolean bStopOnceBandwidthMet = false;
 
 	private static boolean bAlreadyInitialized = false;
 
@@ -169,9 +190,17 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 	private TableContextMenuItem debugMenuItem = null;
 
 	private boolean bSWTUI = false;
-
-	TorrentAttribute torrentAttributeContent = null;
-
+	
+	private CopyOnWriteList listenersFP = new CopyOnWriteList();
+	
+	public static void
+	load(
+		PluginInterface		plugin_interface )
+	{
+		plugin_interface.getPluginProperties().setProperty("plugin.version", "1.0");
+		plugin_interface.getPluginProperties().setProperty("plugin.name", "Start/Stop Rules");
+	}
+	
 	public void initialize(PluginInterface _plugin_interface) {
 		if (bAlreadyInitialized) {
 			System.err.println("StartStopRulesDefaultPlugin Already initialized!!");
@@ -184,12 +213,8 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		startedOn = SystemTime.getCurrentTime();
 
 		pi = _plugin_interface;
-		torrentAttributeContent = pi.getTorrentManager().getPluginAttribute(
-				TorrentAttribute.TA_CONTENT_MAP);
 		download_manager = pi.getDownloadManager();
 
-		pi.getPluginProperties().setProperty("plugin.version", "1.0");
-		pi.getPluginProperties().setProperty("plugin.name", "Start/Stop Rules");
 		pi.getPluginconfig().setPluginConfigKeyPrefix("");
 
 		// Create a configModel for StartStopRules
@@ -278,7 +303,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		configModel.addIntParameter2(
 				"StartStopManager_iRankType",
 				"ConfigView.label.seeding.rankType",
-				com.aelitis.azureus.plugins.startstoprules.defaultplugin.StartStopRulesDefaultPlugin.RANK_SPRATIO);
+				StartStopRulesDefaultPlugin.RANK_SPRATIO);
 		configModel.addIntParameter2("StartStopManager_iRankTypeSeedFallback",
 				"ConfigView.label.seeding.rankType.seed.fallback", 0);
 
@@ -330,6 +355,10 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 				"StartStopManager_bMaxActiveTorrentsWhenSeedingEnabled",
 				"ConfigView.label.queue.maxactivetorrentswhenseeding", false);
 
+		configModel.addBooleanParameter2(
+				"StartStopManager_bStopOnceBandwidthMet", 
+				"ConfigView.label.queue.stoponcebandwidthmet", true);
+				 
 		// first Priority subsection
 		// ---------
 		configModel.addIntParameter2("StartStopManager_iFirstPriority_Type",
@@ -422,29 +451,25 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 	 */
 	private class ChangeFlagCheckerTask implements TimerEventPerformer
 	{
-		long last_process_time = 0;
+		final long FORCE_CHECK_CYCLES = FORCE_CHECK_PERIOD / PROCESS_CHECK_PERIOD;
+
+		long cycleNo = 0;
 
 		public void perform(TimerEvent event) {
-			if (closingDown)
+			if (closingDown) {
 				return;
+			}
 
-			long now = SystemTime.getCurrentTime();
-
-			if (now < last_process_time
-					|| now - last_process_time >= FORCE_CHECK_PERIOD) {
-
+			cycleNo++;
+			if (cycleNo > FORCE_CHECK_CYCLES) {
 				somethingChanged = true;
 			}
 
 			if (somethingChanged) {
-
 				try {
-					last_process_time = now;
-
+					cycleNo = 0;
 					process();
-
 				} catch (Exception e) {
-
 					Debug.printStackTrace(e);
 				}
 			}
@@ -488,6 +513,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 	{
 		public void scrapeResult(DownloadScrapeResult result) {
 			Download dl = result.getDownload();
+			DefaultRankCalculator dlData = (DefaultRankCalculator) downloadDataMap.get(dl);
 
 			// Skip if error (which happens when listener is first added and the
 			// torrent isn't scraped yet)
@@ -495,11 +521,13 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 				if (bDebugLog)
 					log.log(dl.getTorrent(), LoggerChannel.LT_INFORMATION,
 							"Ignored somethingChanged: new scrapeResult (RT_ERROR)");
+				if (dlData != null)
+					dlData.lastScrapeResultOk = false;
 				return;
 			}
 
-			DefaultRankCalculator dlData = (DefaultRankCalculator) downloadDataMap.get(dl);
 			if (dlData != null) {
+				dlData.lastScrapeResultOk = true;
 				requestProcessCycle(dlData);
 				if (bDebugLog)
 					log.log(dl.getTorrent(), LoggerChannel.LT_INFORMATION,
@@ -565,6 +593,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			} else {
 				dlData = new DefaultRankCalculator(StartStopRulesDefaultPlugin.this,
 						download);
+				sortedArrayCache = null;
 				downloadDataMap.put(download, dlData);
 				download.addListener(download_listener);
 				download.addTrackerListener(download_tracker_listener, false);
@@ -586,6 +615,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			download.removeActivationListener(download_activation_listener);
 
 			if (downloadDataMap.containsKey(download)) {
+				sortedArrayCache = null;
 				downloadDataMap.remove(download);
 			}
 
@@ -692,7 +722,15 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			bDebugLog = plugin_config.getBooleanParameter("StartStopManager_bDebugLog");
 
 			bAutoStart0Peers = plugin_config.getBooleanParameter("StartStopManager_bAutoStart0Peers");
-			iMaxUploadSpeed = plugin_config.getIntParameter("Max Upload Speed KBs", 0);
+			
+			
+			globalDownloadLimit = plugin_config.getIntParameter("Max Download Speed KBs", 0);
+			globalUploadLimit = plugin_config.getIntParameter("Max Upload Speed KBs", 0);
+			globalUploadWhenSeedingLimit = plugin_config.getBooleanParameter("enable.seedingonly.upload.rate") ? plugin_config.getIntParameter("Max Upload Speed Seeding KBs", 0) : globalUploadLimit;
+			
+			bStopOnceBandwidthMet = plugin_config.getBooleanParameter("StartStopManager_bStopOnceBandwidthMet");
+			
+			
 
 			boolean move_top = plugin_config.getBooleanParameter("StartStopManager_bNewSeedsMoveTop");
 			plugin_config.setBooleanParameter(
@@ -745,7 +783,14 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			for (int i = 0; i < dlDataArray.length; i++) {
 				dlDataArray[i].getDownloadObject().setSeedingRank(0);
 			}
-			ranksToRecalc.addAll(allDownloads);
+			try {
+				ranksToRecalc_mon.enter();
+
+				ranksToRecalc.addAll(allDownloads);
+				
+			} finally {
+				ranksToRecalc_mon.exit();
+			}
 			requestProcessCycle(null);
 
 			if (bDebugLog) {
@@ -916,6 +961,11 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		int maxActive;
 
 		int maxTorrents;
+		
+		public int maxUploadSpeed()
+		{
+			return downloading == 0 ? globalUploadWhenSeedingLimit : globalUploadLimit;
+		}
 
 		/**
 		 * Default Constructor
@@ -1031,7 +1081,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 
 			if (maxActive == 0) {
 				maxTorrents = 9999;
-			} else if (iMaxUploadSpeed == 0) {
+			} else if (maxUploadSpeed() == 0) {
 				maxTorrents = maxActive + 4;
 			} else {
 				// Don't allow more "seeding/downloading" torrents than there is enough
@@ -1043,7 +1093,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 				// each torrent to have potentially 3kbps.
 				if (minSpeedPerActive < 3)
 					minSpeedPerActive = 3;
-				maxTorrents = (iMaxUploadSpeed / minSpeedPerActive);
+				maxTorrents = (maxUploadSpeed() / minSpeedPerActive);
 				// Allow user to do stupid things like have more slots than their 
 				// upload speed can handle
 				if (maxTorrents < maxActive)
@@ -1063,7 +1113,11 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		int numWaitingOrSeeding; // Running Count
 
 		int numWaitingOrDLing; // Running Count
-
+		
+		long accumulatedDownloadSpeed;
+		
+		long accumulatedUploadSpeed;
+		
 		/**
 		 * store whether there's a torrent higher in the list that is queued
 		 * We don't want to start a torrent lower in the list if there's a higherQueued
@@ -1103,8 +1157,15 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			now = SystemTime.getCurrentTime();
 
 			somethingChanged = false;
-			Object[] recalcArray = ranksToRecalc.toArray();
-			ranksToRecalc.clear();
+			Object[] recalcArray;
+			try {
+				ranksToRecalc_mon.enter();
+
+				recalcArray = ranksToRecalc.toArray();
+				ranksToRecalc.clear();
+			} finally {
+				ranksToRecalc_mon.exit();
+			}
 			for (int i = 0; i < recalcArray.length; i++) {
 				DefaultRankCalculator rankObj = (DefaultRankCalculator) recalcArray[i];
 				if (bDebugLog) {
@@ -1123,8 +1184,11 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			}
 
 			// pull the data into a local array, so we don't have to lock/synchronize
-			DefaultRankCalculator[] dlDataArray;
-			dlDataArray = (DefaultRankCalculator[]) downloadDataMap.values().toArray(
+			DefaultRankCalculator[] dlDataArray;			
+			if(sortedArrayCache != null && sortedArrayCache.length == downloadDataMap.size())
+				dlDataArray = sortedArrayCache;
+			else
+				dlDataArray = sortedArrayCache = (DefaultRankCalculator[]) downloadDataMap.values().toArray(
 					new DefaultRankCalculator[downloadDataMap.size()]);
 
 			TotalsStats totals = new TotalsStats(dlDataArray);
@@ -1144,7 +1208,8 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 					"tIncQd=" + totals.incompleteQueued,
 					"mxCdrs=" + totals.maxSeeders,
 					"tFP=" + totals.firstPriority,
-					"maxT=" + totals.maxTorrents
+					"maxT=" + totals.maxTorrents,
+					"maxA=" + totals.maxActive,
 				};
 			}
 
@@ -1175,6 +1240,9 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 				if ((download.getState() == Download.ST_WAITING)) {
 					try {
 						download.initialize();
+
+						String s = "initialize: state is waiting";
+						log.log(download.getTorrent(), LoggerChannel.LT_INFORMATION, s);
 					} catch (Exception ignore) {
 						/*ignore*/
 					}
@@ -1195,6 +1263,31 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 						|| state == Download.ST_ERROR) {
 					continue;
 				}
+				
+				if (download.isForceStart()) {
+					if (state == Download.ST_STOPPED || state == Download.ST_QUEUED) {
+						try {
+							download.restart();
+							String s = "restart: isForceStart";
+							log.log(download.getTorrent(), LoggerChannel.LT_INFORMATION, s);
+							dlData.sTrace += s + "\n";
+						} catch (DownloadException e) {
+						}
+
+						state = download.getState();
+					}
+
+					if (state == Download.ST_READY) {
+						try {
+							download.start();
+							String s = "Start: isForceStart";
+							log.log(download.getTorrent(), LoggerChannel.LT_INFORMATION, s);
+							dlData.sTrace += s + "\n";
+						} catch (DownloadException e) {
+							/* ignore */
+						}
+					}
+				}
 
 				// Handle incomplete DLs
 				if (!download.isComplete()) {
@@ -1217,7 +1310,8 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 					"tIncQd=" + totals.incompleteQueued,
 					"mxCdrs=" + totals.maxSeeders,
 					"tFP=" + totals.firstPriority,
-					"maxT=" + totals.maxTorrents
+					"maxT=" + totals.maxTorrents,
+					"maxA=" + totals.maxActive,
 				};
 				printDebugChanges("<<process() ", mainDebugEntries, mainDebugEntries2,
 						"", "", true, null);
@@ -1286,6 +1380,27 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		if (maxDLs < minDownloads) {
 			maxDLs = minDownloads;
 		}
+		
+		boolean bActivelyDownloading = dlData.getActivelyDownloading();
+		boolean globalDownLimitReached;
+		boolean globalRateAdjustedActivelyDownloading;
+		boolean fakedActively;
+		if (bStopOnceBandwidthMet) {
+  		boolean isRunning = download.getState() == Download.ST_DOWNLOADING;
+  		globalDownLimitReached = globalDownloadLimit > 0 && vars.accumulatedDownloadSpeed/1024 > globalDownloadLimit * IGNORE_SLOT_THRESHOLD_FACTOR;
+  		globalRateAdjustedActivelyDownloading = bActivelyDownloading || (isRunning && globalDownLimitReached);
+  		fakedActively = globalRateAdjustedActivelyDownloading && !bActivelyDownloading;
+  		if(fakedActively)
+  		{
+  			totals.activelyDLing++;
+  			totals.maxSeeders = calcMaxSeeders(totals.activelyDLing + totals.waitingToDL);
+  		}
+		} else {
+			globalDownLimitReached = false;
+			globalRateAdjustedActivelyDownloading = bActivelyDownloading;
+			fakedActively = false;
+		}
+			
 
 		if (bDebugLog) {
 			String s = ">> DL state=" + sStates.charAt(download.getState())
@@ -1293,7 +1408,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 					+ ";numW8tngorDLing=" + vars.numWaitingOrDLing + ";maxCDrs="
 					+ totals.maxSeeders + ";forced=" + boolDebug(download.isForceStart())
 					+ ";actvDLs=" + totals.activelyDLing + ";maxDLs=" + maxDLs
-					+ ";ActDLing=" + boolDebug(dlData.getActivelyDownloading())
+					+ ";ActDLing=" + boolDebug(bActivelyDownloading) + ";globDwnRchd=" + boolDebug(globalDownLimitReached)
 					+ ";hgherQd=" + boolDebug(vars.higherDLtoStart) + ";isCmplt="
 					+ boolDebug(download.isComplete());
 			log.log(download.getTorrent(), LoggerChannel.LT_INFORMATION, s);
@@ -1302,7 +1417,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 
 		// must use fresh getActivelyDownloading() in case state changed to 
 		// downloading
-		if ((state == Download.ST_DOWNLOADING && dlData.getActivelyDownloading())
+		if ((state == Download.ST_DOWNLOADING && globalRateAdjustedActivelyDownloading)
 				|| state == Download.ST_READY || state == Download.ST_WAITING
 				|| state == Download.ST_PREPARING) {
 			vars.numWaitingOrDLing++;
@@ -1311,45 +1426,46 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		if (state == Download.ST_READY || state == Download.ST_DOWNLOADING
 				|| state == Download.ST_WAITING) {
 
-			boolean bActivelyDownloading = dlData.getActivelyDownloading();
-
 			// Stop torrent if over limit
 			boolean bOverLimit = vars.numWaitingOrDLing > maxDLs
 					|| (vars.numWaitingOrDLing >= maxDLs && vars.higherDLtoStart);
 
 			boolean bDownloading = state == Download.ST_DOWNLOADING;
 
-			if ((maxDownloads != 0)
-					&& bOverLimit
-					&& (bActivelyDownloading || !bDownloading || (bDownloading
-							&& totals.maxActive != 0 && !bActivelyDownloading && totals.activelyCDing
-							+ totals.activelyDLing >= totals.maxActive))) {
-				try {
-					if (bDebugLog) {
-						String s = "   stopAndQueue: " + vars.numWaitingOrDLing
-								+ " waiting or downloading, when limit is " + maxDLs + "("
-								+ maxDownloads + ")";
-						if (vars.higherDLtoStart) {
+			if ((maxDownloads != 0) && bOverLimit &&
+					(globalRateAdjustedActivelyDownloading || !bDownloading ||
+						(bDownloading && totals.maxActive != 0 && !globalRateAdjustedActivelyDownloading &&	totals.activelyCDing + totals.activelyDLing >= totals.maxActive)
+					)
+				)
+			{
+				try
+				{
+					if (bDebugLog)
+					{
+						String s = "   stopAndQueue: " + vars.numWaitingOrDLing + " waiting or downloading, when limit is " + maxDLs + "(" + maxDownloads + ")";
+						if (vars.higherDLtoStart)
+						{
 							s += " and higher DL is starting";
 						}
 						log.log(download.getTorrent(), LoggerChannel.LT_INFORMATION, s);
 						dlData.sTrace += s + "\n";
 					}
 					download.stopAndQueue();
-
 					// reduce counts
 					vars.numWaitingOrDLing--;
-					if (state == Download.ST_DOWNLOADING) {
+					if (state == Download.ST_DOWNLOADING)
+					{
 						totals.downloading--;
-						if (bActivelyDownloading)
+						if (bActivelyDownloading || fakedActively)
 							totals.activelyDLing--;
-					} else {
+					} else
+					{
 						totals.waitingToDL--;
 					}
-					totals.maxSeeders = calcMaxSeeders(totals.activelyDLing
-							+ totals.waitingToDL);
-				} catch (Exception ignore) {
-					/*ignore*/
+					totals.maxSeeders = calcMaxSeeders(totals.activelyDLing + totals.waitingToDL);
+				} catch (Exception ignore)
+				{
+					/* ignore */
 				}
 
 				state = download.getState();
@@ -1419,7 +1535,13 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			}
 		}
 
+		int oldState = state;
 		state = download.getState();
+
+		if (oldState != state) {
+			somethingChanged = true;
+		}
+
 		if (download.getSeedingRank() >= 0
 				&& (state == Download.ST_QUEUED || state == Download.ST_READY
 						|| state == Download.ST_WAITING || state == Download.ST_PREPARING)) {
@@ -1437,6 +1559,11 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			log.log(download.getTorrent(), LoggerChannel.LT_INFORMATION, s);
 			dlData.sTrace += s + "\n";
 		}
+		
+		if (bStopOnceBandwidthMet) {
+			vars.accumulatedDownloadSpeed += download.getStats().getDownloadAverage();
+			vars.accumulatedUploadSpeed += download.getStats().getUploadAverage();
+		}
 	}
 
 	/**
@@ -1469,7 +1596,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		//    c) other
 		// 7) Seeding Torrent changes to Queued.  Go to step 1.
 
-		int numPeers = calcPeersNoUs(download);
+		int numPeers = dlData.lastModifiedScrapeResultPeers;
 		boolean isFP = false;
 
 		if (bDebugLog) {
@@ -1489,7 +1616,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 		}
 
 		try {
-			boolean bScrapeOk = scrapeResultOk(download);
+			boolean bScrapeOk = dlData.lastScrapeResultOk;
 
 			// Ignore rules and other auto-starting rules do not apply when 
 			// bAutoStart0Peers and peers == 0. So, handle starting 0 peers 
@@ -1536,7 +1663,45 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 				isFP = dlData.isFirstPriority();
 			}
 
-			boolean bActivelySeeding = dlData.getActivelySeeding();
+		boolean bActivelySeeding = dlData.getActivelySeeding();
+		boolean globalDownLimitReached;
+		boolean globalUpLimitReached;
+		boolean globalRateAdjustedActivelySeeding;
+		boolean fakedActively;
+		if (bStopOnceBandwidthMet) {
+			boolean isRunning = download.getState() == Download.ST_SEEDING;
+			globalUpLimitReached = totals.maxUploadSpeed() > 0 && vars.accumulatedUploadSpeed/1024 > totals.maxUploadSpeed() * IGNORE_SLOT_THRESHOLD_FACTOR;
+			globalDownLimitReached = globalDownloadLimit > 0 && vars.accumulatedDownloadSpeed/1024 > globalDownloadLimit * IGNORE_SLOT_THRESHOLD_FACTOR;
+			globalRateAdjustedActivelySeeding = bActivelySeeding || (isRunning && (globalUpLimitReached || globalDownLimitReached));
+			fakedActively = globalRateAdjustedActivelySeeding && !bActivelySeeding;
+			if(fakedActively)
+				totals.activelyCDing++;
+		} else {
+			globalUpLimitReached = false;
+			globalRateAdjustedActivelySeeding = bActivelySeeding;
+			globalDownLimitReached = false;
+			fakedActively = false;
+		}
+				
+			
+			// Is it OK to set this download to a queued state?
+			// It is if:
+			//   1) It is either READY or SEEDING; and
+			//   2) It is either one of the following; and
+			//        a) Not a first priority torrent; or
+			//        b) There is a limit to the number of active torrents, and the number of
+			//           waiting and seeding torrents is already higher (or equal to) the number
+			//           of maximum allowed active torrents (taking away the number of minimum
+			//           required downloads).
+			//
+			//           So I understand that to mean - it isn't first priority and leaving
+			//           this torrent running would mean that there aren't enough slots to
+			//           fulfil the minimum downloads requirement, because there are so many
+			//           torrents seeding (or waiting to seed) already. Or, in the case there
+			//           is no minimum downloads requirement - it's just overrun the maximum
+			//           active torrents count.
+			//
+			//   3) It hasn't been force started.
 			boolean okToQueue = (state == Download.ST_READY || state == Download.ST_SEEDING)
 					&& (!isFP || (isFP && ((totals.maxActive != 0 && vars.numWaitingOrSeeding >= totals.maxActive
 							- minDownloads))))
@@ -1561,7 +1726,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 					(state == Download.ST_READY || state == Download.ST_WAITING
 							|| state == Download.ST_PREPARING ||
 					// Forced Start torrents are pre-included in count
-					(state == Download.ST_SEEDING && bActivelySeeding && !download.isForceStart()))) {
+					(state == Download.ST_SEEDING && globalRateAdjustedActivelySeeding && !download.isForceStart()))) {
 				vars.numWaitingOrSeeding++;
 				if (bDebugLog)
 					sDebugLine += "\n  Torrent is waiting or seeding";
@@ -1630,7 +1795,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 					}
 					state = download.getState();
 					totals.activelyCDing++;
-					bActivelySeeding = true;
+					globalRateAdjustedActivelySeeding = bActivelySeeding = true;
 					vars.numWaitingOrSeeding++;
 				} else if (okToQueue) {
 					// In between switching from STATE_WAITING and STATE_READY,
@@ -1654,7 +1819,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 					// (Actively Seeding OR StartingUp OR Seeding a non-active download) 
 					okToStop = !download.isChecking()
 							&& (bOverLimit || rank < DefaultRankCalculator.SR_IGNORED_LESS_THAN)
-							&& (bActivelySeeding || !bSeeding || (!bActivelySeeding && bSeeding));
+							&& (globalRateAdjustedActivelySeeding || !bSeeding || (!globalRateAdjustedActivelySeeding && bSeeding));
 
 					if (bDebugLog) {
 						if (okToStop) {
@@ -1701,11 +1866,17 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 						vars.bStopAndQueued = true;
 						// okToQueue only allows READY and SEEDING state.. and in both cases
 						// we have to reduce counts
-						if (bActivelySeeding) {
+						if (bActivelySeeding || fakedActively) {
 							totals.activelyCDing--;
 							bActivelySeeding = false;
-							vars.numWaitingOrSeeding--;
 						}
+						if(globalRateAdjustedActivelySeeding)
+						{
+							vars.numWaitingOrSeeding--;
+							globalRateAdjustedActivelySeeding = false;
+						}
+							
+						
 						// force stop allows READY states in here, so adjust counts 
 						if (state == Download.ST_READY)
 							totals.waitingToSeed--;
@@ -1742,7 +1913,7 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 							|| state == Download.ST_WAITING || state == Download.ST_PREPARING)) {
 				vars.higherCDtoStart = true;
 			}
-
+			
 		} finally {
 			if (bDebugLog) {
 				String[] debugEntries2 = new String[] {
@@ -1761,6 +1932,11 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 						true, dlData);
 			}
 		}
+		
+		
+		if (bStopOnceBandwidthMet) {
+			vars.accumulatedUploadSpeed += download.getStats().getUploadAverage();
+		}
 	}
 
 	private String boolDebug(boolean b) {
@@ -1772,14 +1948,14 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			boolean bAlwaysPrintNoChangeLine, DefaultRankCalculator dlData) {
 		boolean bAnyChanged = false;
 		String sDebugLineNoChange = sPrefixFirstLine;
-		String sDebugLineOld = "";
-		String sDebugLineNew = "";
+		StringBuffer sDebugLineOld = new StringBuffer(120);
+		StringBuffer sDebugLineNew = new StringBuffer(120);
 		for (int j = 0; j < oldEntries.length; j++) {
 			if (oldEntries[j].equals(newEntries[j]))
 				sDebugLineNoChange += oldEntries[j] + ";";
 			else {
-				sDebugLineOld += oldEntries[j] + ";";
-				sDebugLineNew += newEntries[j] + ";";
+				sDebugLineOld.append(oldEntries[j]);sDebugLineOld.append(";");
+				sDebugLineNew.append(newEntries[j]);sDebugLineNew.append(";");
 				bAnyChanged = true;
 			}
 		}
@@ -1889,14 +2065,14 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 	public void requestProcessCycle(DefaultRankCalculator rankToRecalc) {
 		if (rankToRecalc != null) {
 			try {
-				this_mon.enter();
+				ranksToRecalc_mon.enter();
 
 				ranksToRecalc.add(rankToRecalc);
 			} finally {
-				this_mon.exit();
+				ranksToRecalc_mon.exit();
 			}
 		}
-
+		
 		if (somethingChanged) {
 			processMergeCount++;
 		} else {
@@ -1941,5 +2117,17 @@ public class StartStopRulesDefaultPlugin implements Plugin,
 			writer.exdent();
 		}
 	}
+	
+	public void addListener(StartStopRulesFPListener listener) {
+		listenersFP.add(listener);
+	}
+	
+	public void removeListener(StartStopRulesFPListener listener) {
+		listenersFP.remove(listener);
+	}
+	
+	public List getFPListeners() {
+		return listenersFP.getList();
+	}
 } // class
 
