@@ -54,6 +54,10 @@ public class ConfigSectionSeedingFirstPriority implements UISWTConfigSection {
 
   public void configSectionDelete() {
   }
+  
+	public int maxUserMode() {
+		return 0;
+	}
 
   public Composite configSectionCreate(Composite parent) {
     // Seeding Automation Setup
