@@ -19,22 +19,40 @@
  */
 package com.aelitis.azureus.ui.swt.skin;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Date;
+import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.browser.Browser;
 import org.eclipse.swt.custom.SashForm;
-import org.eclipse.swt.graphics.*;
+import org.eclipse.swt.events.DisposeEvent;
+import org.eclipse.swt.events.DisposeListener;
+import org.eclipse.swt.graphics.Color;
+import org.eclipse.swt.graphics.Cursor;
+import org.eclipse.swt.graphics.Image;
+import org.eclipse.swt.graphics.Point;
+import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.layout.FormAttachment;
 import org.eclipse.swt.layout.FormData;
 import org.eclipse.swt.layout.FormLayout;
-import org.eclipse.swt.widgets.*;
-
+import org.eclipse.swt.widgets.Canvas;
+import org.eclipse.swt.widgets.Composite;
+import org.eclipse.swt.widgets.Control;
+import org.eclipse.swt.widgets.Display;
+import org.eclipse.swt.widgets.Event;
+import org.eclipse.swt.widgets.Group;
+import org.eclipse.swt.widgets.Listener;
+import org.eclipse.swt.widgets.Sash;
+import org.eclipse.swt.widgets.Shell;
 import org.gudy.azureus2.core3.util.AEMonitor;
 import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.Debug;
 
+import com.aelitis.azureus.ui.IUIIntializer;
 import com.aelitis.azureus.ui.skin.SkinProperties;
 import com.aelitis.azureus.ui.swt.utils.ImageLoader;
 import com.aelitis.azureus.ui.swt.utils.ImageLoaderFactory;
@@ -46,7 +64,7 @@ import com.aelitis.azureus.ui.swt.utils.ImageLoaderFactory;
  */
 public class SWTSkin
 {
-	static boolean DEBUGLAYOUT = false; //System.getProperty("debuglayout") != null;
+	static boolean DEBUGLAYOUT = System.getProperty("debuglayout") != null;
 
 	private Map mapImageLoaders = new HashMap();
 
@@ -56,13 +74,13 @@ public class SWTSkin
 
 	private Listener ontopPaintListener;
 
-	// Key = Skin Object ID; Value = Control
+	// Key = Skin Object ID; Value = Array of SWTSkinObject
 	private HashMap mapIDsToControls = new HashMap();
 
-	// Key = TabSet ID; Value = List of Control
+	// Key = TabSet ID; Value = SWTSkinTabSet
 	private HashMap mapTabSetToControls = new HashMap();
 
-	// Key = Widget ID; Value = Control
+	// Key = Widget ID; Value = Array of SWTSkinObject
 	private HashMap mapPublicViewIDsToControls = new HashMap();
 
 	private HashMap mapPublicViewIDsToListeners = new HashMap();
@@ -76,12 +94,30 @@ public class SWTSkin
 
 	private boolean bLayoutComplete = false;
 
+	private List listenersLayoutComplete = new ArrayList();
+
+	private final ClassLoader classLoader;
+	
+	private boolean ourSkinProperties = false;
+
 	/**
 	 * 
 	 */
 	public SWTSkin() {
-		skinProperties = new SWTSkinPropertiesImpl();
-		ImageLoaderFactory.createInstance(Display.getDefault(), skinProperties);
+		this.classLoader = SWTSkin.class.getClassLoader();
+		ourSkinProperties = true;
+		init(new SWTSkinPropertiesImpl());
+	}
+
+	public SWTSkin(ClassLoader classLoader, String skinPath, String mainSkinFile) {
+		this.classLoader = classLoader;
+		ourSkinProperties = true;
+		init(new SWTSkinPropertiesImpl(classLoader, skinPath, mainSkinFile));
+	}
+		
+	private void init(SWTSkinProperties skinProperties) {
+		this.skinProperties = skinProperties;
+		ImageLoaderFactory.createInstance(classLoader, Display.getDefault(), skinProperties);
 
 		ontopPaintListener = new Listener() {
 			public void handleEvent(Event event) {
@@ -125,7 +161,7 @@ public class SWTSkin
 			return loader;
 		}
 
-		loader = new ImageLoader(Display.getDefault(), properties);
+		loader = new ImageLoader(classLoader, Display.getDefault(), properties);
 		mapImageLoaders.put(properties, loader);
 
 		return loader;
@@ -154,7 +190,7 @@ public class SWTSkin
 			boolean bAlreadyPresent = false;
 			for (int i = 0; i < existingObjects.length; i++) {
 				//System.out.println(".." + existingObjects[i]);
-				if (existingObjects[i].equals(object)) {
+				if (existingObjects[i] != null && existingObjects[i].equals(object)) {
 					bAlreadyPresent = true;
 					System.err.println("already present: " + key + "; " + object
 							+ "; existing: " + existingObjects[i]);
@@ -173,7 +209,8 @@ public class SWTSkin
 				//						+ (length + 1));
 			}
 		} else {
-			arrayMap.put(key, new SWTSkinObject[] { object
+			arrayMap.put(key, new SWTSkinObject[] {
+				object
 			});
 		}
 	}
@@ -234,6 +271,26 @@ public class SWTSkin
 		}
 	}
 
+	/**
+	 * Get all the SWTSkinObject of a specific type
+	 * 
+	 * @param sTypeRegEx Type of skin objects to find (container, image, etc)
+	 * @return
+	 *
+	 * @since 3.0.1.3
+	 */
+	public SWTSkinObject[] getSkinObjectsByType(String sTypeRegEx) {
+		List list = new ArrayList();
+		for (Iterator iter = mapIDsToControls.keySet().iterator(); iter.hasNext();) {
+			String sID = (String) iter.next();
+			SWTSkinObject so = getSkinObjectByID(sID);
+			if (so != null && so.getType().matches(sTypeRegEx)) {
+				list.add(so);
+			}
+		}
+		return (SWTSkinObject[]) list.toArray(new SWTSkinObject[0]);
+	}
+
 	public SWTSkinObject getSkinObjectByID(String sID) {
 		SWTSkinObject[] objects = (SWTSkinObject[]) mapIDsToControls.get(sID);
 		if (objects == null) {
@@ -275,9 +332,15 @@ public class SWTSkin
 		return (SWTSkinTabSet) mapTabSetToControls.get(sID);
 	}
 
-	public void activateTab(SWTSkinObject skinObjectInTab) {
+	public SWTSkinObjectTab activateTab(SWTSkinObject skinObjectInTab) {
 		if (skinObjectInTab == null) {
-			return;
+			return null;
+		}
+		
+		if (skinObjectInTab instanceof SWTSkinObjectTab) {
+			SWTSkinObjectTab tab = (SWTSkinObjectTab)skinObjectInTab;
+			tab.getTabset().setActiveTab(tab);
+			return tab;
 		}
 
 		for (Iterator iter = mapTabSetToControls.values().iterator(); iter.hasNext();) {
@@ -294,12 +357,13 @@ public class SWTSkin
 					if (hasSkinObject(object, skinObjectInTab)) {
 						//System.out.println("FOUND");
 						tabset.setActiveTab(tab);
-						return;
+						return tab;
 					}
 				}
 			}
 		}
-		System.out.println("NOT FOUND");
+		System.out.println("NOT FOUND" + skinObjectInTab);
+		return null;
 	}
 
 	private boolean hasSkinObject(SWTSkinObject start, SWTSkinObject skinObject) {
@@ -308,8 +372,9 @@ public class SWTSkin
 			for (int i = 0; i < children.length; i++) {
 				SWTSkinObject object = children[i];
 				//System.out.println("  check " + object + " in " + start + " for " + skinObject);
-				if (hasSkinObject(object, skinObject))
+				if (hasSkinObject(object, skinObject)) {
 					return true;
+				}
 			}
 		}
 		//System.out.println("  check " + start + " for " + skinObject);
@@ -322,23 +387,57 @@ public class SWTSkin
 		return getTabSet(sTabSetID);
 	}
 
-	public boolean setActiveTab(String sTabSetID, String sTabID) {
+	public boolean setActiveTab(String sTabSetID, String sTabViewID) {
 		SWTSkinTabSet tabSet = getTabSet(sTabSetID);
 		if (tabSet == null) {
 			System.err.println(sTabSetID);
 			return false;
 		}
 
-		return tabSet.setActiveTab(sTabID);
+		return tabSet.setActiveTab(sTabViewID);
 	}
 
+	/**
+	 * @param shell2
+	 * @param string
+	 * @param uiInitializer
+	 *
+	 * @since 3.0.5.3
+	 */
 	public void initialize(Shell shell, String startID) {
+		initialize(shell, startID, null);
+	}
+
+	public void initialize(Shell shell, String startID,
+			IUIIntializer uiInitializer) {
 
 		this.shell = shell;
 		FormLayout layout = new FormLayout();
 		shell.setLayout(layout);
 		shell.setBackgroundMode(SWT.INHERIT_DEFAULT);
+		
+		shell.addDisposeListener(new DisposeListener() {
+			public void widgetDisposed(DisposeEvent e) {
+				for (Iterator iter = mapImageLoaders.values().iterator(); iter.hasNext();) {
+					ImageLoader loader = (ImageLoader) iter.next();
+					loader.unLoadImages();
+				}
+				if (ourSkinProperties) {
+					//skinProperties.dispose();
+				}
+			}
+		});
+		
+		Color bg = skinProperties.getColor(startID + ".color");
+		if (bg != null) {
+			shell.setBackground(bg);
+		}
 
+		Color fg = skinProperties.getColor(startID + ".fgcolor");
+		if (fg != null) {
+			shell.setForeground(fg);
+		}
+		
 		String[] sMainGroups = skinProperties.getStringArray(startID + ".widgets");
 		if (sMainGroups == null) {
 			System.out.println("NO " + startID + ".widgets!!");
@@ -351,6 +450,10 @@ public class SWTSkin
 			if (DEBUGLAYOUT) {
 				System.out.println("Container: " + sID);
 			}
+			
+			if (uiInitializer != null) {
+				uiInitializer.increaseProgress();
+			}
 
 			linkIDtoParent(skinProperties, sID, sID, null, false, true);
 		}
@@ -376,7 +479,6 @@ public class SWTSkin
 	}
 
 	public void layout() {
-		DEBUGLAYOUT = false;
 		if (DEBUGLAYOUT) {
 			System.out.println("==== Start Apply Layout");
 		}
@@ -394,15 +496,17 @@ public class SWTSkin
 			}
 		}
 
-		for (Iterator iter = mapTabSetToControls.values().iterator(); iter.hasNext();) {
-			SWTSkinTabSet tabset = (SWTSkinTabSet) iter.next();
-			tabset.clean();
-		}
-
 		// Disabled due to Browser flickering
 		//addPaintListenerToAll(shell);
 
 		bLayoutComplete = true;
+
+		Object[] listeners = listenersLayoutComplete.toArray();
+		for (int i = 0; i < listeners.length; i++) {
+			SWTSkinLayoutCompleteListener l = (SWTSkinLayoutCompleteListener) listeners[i];
+			l.skinLayoutCompleted();
+		}
+
 		if (DEBUGLAYOUT) {
 			System.out.println("==== End Apply Layout");
 		}
@@ -537,7 +641,8 @@ public class SWTSkin
 					// Object, Offset, Alignment
 					String sWidget = sParams[0];
 
-					SWTSkinObject configSkinObject = getSkinObjectByID(sWidget);
+					SWTSkinObject configSkinObject = getSkinObjectByID(sWidget,
+							skinObject.getParent());
 					int iNextPos;
 					if (configSkinObject != null) {
 						control = configSkinObject.getControl();
@@ -635,6 +740,13 @@ public class SWTSkin
 				newFormData.height);
 		newFormData.width = properties.getIntValue(sConfigID + ".width",
 				newFormData.width);
+		if (!skinObject.getDefaultVisibility()) {
+			if (newFormData.width != 0 && newFormData.height != 0) {
+				controlToLayout.setData("oldSize", new Point(newFormData.width, newFormData.height));
+			}
+			newFormData.width = 0;
+			newFormData.height = 0;
+		}
 		controlToLayout.setLayoutData(newFormData);
 	}
 
@@ -780,7 +892,6 @@ public class SWTSkin
 	private SWTSkinObject createMySash(SWTSkinProperties properties,
 			final String sID, String sConfigID, String[] typeParams,
 			SWTSkinObject parentSkinObject, final boolean bVertical) {
-
 		SWTSkinObject skinObject = new SWTSkinObjectSash(this, properties, sID,
 				sConfigID, typeParams, parentSkinObject, bVertical);
 		addToControlMap(skinObject);
@@ -878,6 +989,21 @@ public class SWTSkin
 
 		return skinObject;
 	}
+	
+	private SWTSkinObject createSlider(SWTSkinProperties properties,
+			String sID, String sConfigID, String[] typeParams,
+			SWTSkinObject parentSkinObject) {
+		SWTSkinObject skinObject = new SWTSkinObjectSlider(this, properties, sID,
+				sConfigID, typeParams, parentSkinObject);
+		addToControlMap(skinObject);
+
+		if (bLayoutComplete) {
+			attachControl(skinObject);
+		}
+
+		return skinObject;
+	}
+	
 
 	public Shell getShell() {
 		return shell;
@@ -943,16 +1069,66 @@ public class SWTSkin
 		return handCursorListener;
 	}
 
+	/**
+	 * Create a skin object based off an existing config "template"
+	 *  
+	 * @param sID ID of new skin object
+	 * @param sConfigID config id to use to create new skin object 
+	 * @param parentSkinObject location to place new skin object in
+	 * 
+	 * @return new skin object
+	 */
 	public SWTSkinObject createSkinObject(String sID, String sConfigID,
 			SWTSkinObject parentSkinObject) {
 		SWTSkinObject skinObject = linkIDtoParent(skinProperties, sID, sConfigID,
 				parentSkinObject, true, true);
-		if (bLayoutComplete) {
-			((Composite) skinObject.getControl()).getParent().layout(true);
+		if (bLayoutComplete && skinObject != null) {
+			Control control = skinObject.getParent().getControl();
+			if (control instanceof Composite) {
+				((Composite) control).layout(true);
+			}
 		}
 		return skinObject;
 	}
 
+	public void addSkinObject(SWTSkinObject skinObject) {
+		String sViewID = skinObject.getViewID();
+		if (sViewID != null) {
+			setSkinObjectViewID(skinObject, sViewID);
+		}
+
+		attachControl(skinObject);
+	}
+
+	/**
+	 * @param skinObject
+	 *
+	 * @since 3.0.1.3
+	 */
+	public void removeSkinObject(SWTSkinObject skinObject) {
+		skinObject.triggerListeners(SWTSkinObjectListener.EVENT_DESTROY);
+
+		SWTSkinObject[] objects = (SWTSkinObject[]) mapIDsToControls.get(skinObject.getSkinObjectID());
+		if (objects != null) {
+			for (int i = 0; i < objects.length; i++) {
+				if (objects[i] == skinObject) {
+					objects[i] = null;
+				}
+			}
+		}
+
+		objects = (SWTSkinObject[]) mapPublicViewIDsToControls.get(skinObject.getViewID());
+		if (objects != null) {
+			for (int i = 0; i < objects.length; i++) {
+				if (objects[i] == skinObject) {
+					objects[i] = null;
+				}
+			}
+		}
+
+		skinObject.dispose();
+	}
+
 	private SWTSkinObject linkIDtoParent(SWTSkinProperties properties,
 			String sID, String sConfigID, SWTSkinObject parentSkinObject,
 			boolean bForceCreate, boolean bAddView) {
@@ -964,7 +1140,7 @@ public class SWTSkin
 				sType = sTypeParams[0];
 				bForceCreate = true;
 			} else {
-				// best guess
+				// no type, use best guess
 				sType = null;
 
 				String sImageLoc = properties.getStringValue(sConfigID);
@@ -989,7 +1165,8 @@ public class SWTSkin
 					return null;
 				}
 
-				sTypeParams = new String[] { sType
+				sTypeParams = new String[] {
+					sType
 				};
 			}
 
@@ -1024,21 +1201,24 @@ public class SWTSkin
 			} else if (sType.equals("clone")) {
 				skinObject = createClone(properties, sID, sConfigID, sTypeParams,
 						parentSkinObject);
+			} else if (sType.equals("slider")) {
+				skinObject = createSlider(properties, sID, sConfigID, sTypeParams,
+						parentSkinObject);
 			} else if (sType.equals("hidden")) {
 				return null;
 			} else if (sType.equals("browser")) {
 				skinObject = createBrowser(properties, sID, sConfigID, parentSkinObject);
-			} else {
+			}  else if (sType.equals("separator")) {
+				skinObject = createSeparator(properties, sID, sConfigID, sTypeParams,
+						parentSkinObject);
+			}else {
 				System.err.println(sConfigID + ": Invalid type of " + sType);
 			}
 
 			if (bAddView) {
-				String sViewID = properties.getStringValue(sConfigID + ".view");
+				String sViewID = skinObject.getViewID();
 				if (sViewID != null) {
 					setSkinObjectViewID(skinObject, sViewID);
-					if (skinObject instanceof SWTSkinObjectBasic) {
-						((SWTSkinObjectBasic) skinObject).setViewID(sViewID);
-					}
 				}
 			}
 
@@ -1080,8 +1260,8 @@ public class SWTSkin
 			System.err.println("XXXXXXXX " + sID + " has no config ID.."
 					+ Debug.getStackTrace(false, false));
 		}
-
-		String[] sCloneParams;
+		
+				String[] sCloneParams;
 		if (typeParams.length > 1) {
 			int size = typeParams.length - 1;
 			sCloneParams = new String[size];
@@ -1222,6 +1402,22 @@ public class SWTSkin
 		return skinObject;
 	}
 
+	
+	private SWTSkinObject createSeparator(SWTSkinProperties properties,
+			String sID, String sConfigID, String[] typeParams,
+			SWTSkinObject parentSkinObject) {
+		SWTSkinObject skinObject = new SWTSkinObjectSeparator(this, properties, sID,
+				sConfigID, parentSkinObject);
+		addToControlMap(skinObject);
+
+		if (bLayoutComplete) {
+			attachControl(skinObject);
+		}
+
+		return skinObject;
+	}
+	
+	
 	public SWTSkinProperties getSkinProperties() {
 		return skinProperties;
 	}
@@ -1264,6 +1460,16 @@ public class SWTSkin
 		}
 	}
 
+	public boolean isLayoutComplete() {
+		return bLayoutComplete;
+	}
+
+	public void addListener(SWTSkinLayoutCompleteListener l) {
+		if (!listenersLayoutComplete.contains(l)) {
+			listenersLayoutComplete.add(l);
+		}
+	}
+
 	public static void main(String[] args) {
 		java.util.Date d = new java.util.Date();
 		long t = d.getTime();
@@ -1275,5 +1481,6 @@ public class SWTSkin
 		Date then = new Date(t);
 
 		System.out.println(d + ";" + then);
+
 	}
 }
