@@ -24,9 +24,12 @@ package com.aelitis.azureus.ui.swt.skin;
  * @created Aug 4, 2006
  *
  */
-public interface SWTSkinObjectText extends SWTSkinObject
+public interface SWTSkinObjectText
+	extends SWTSkinObject
 {
 	public void setText(String text);
 
 	public void setTextID(String id);
+	
+	public void setTextID(String id, String[] params);
 }
