@@ -22,7 +22,6 @@
 
 package com.aelitis.net.magneturi.impl;
 
-import java.awt.image.BufferedImage;
 import java.io.BufferedReader;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
@@ -40,12 +39,12 @@ import java.net.URL;
 import java.net.URLDecoder;
 import java.util.*;
 
-import javax.imageio.ImageIO;
-
+import org.bouncycastle.util.encoders.Base64;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.logging.*;
 import org.gudy.azureus2.core3.util.*;
 
+import com.aelitis.azureus.core.util.png.PNG;
 import com.aelitis.net.magneturi.MagnetURIHandler;
 import com.aelitis.net.magneturi.MagnetURIHandlerListener;
 import com.aelitis.net.magneturi.MagnetURIHandlerProgressListener;
@@ -69,6 +68,8 @@ MagnetURIHandlerImpl
 	private static final int				DOWNLOAD_TIMEOUT	= 3*60*1000;
 	
 	protected static final String	NL			= "\015\012";
+	
+	private final static boolean DEBUG = false;
 
 	public static MagnetURIHandler
 	getSingleton()
@@ -164,7 +165,28 @@ MagnetURIHandlerImpl
 										        	BufferedReader br = new BufferedReader(new InputStreamReader(sck.getInputStream(),Constants.DEFAULT_ENCODING));
 										        	
 										        	String line = br.readLine();
-					
+
+										        	if (DEBUG) {
+										        		System.out.println("=====");
+										        		System.out.println("Traffic Class: "
+										        				+ sck.getTrafficClass());
+										        		System.out.println("OS: " + sck.getOutputStream());
+										        		System.out.println("isBound? " + sck.isBound()
+										        				+ "; isClosed=" + sck.isClosed() + "; isConn="
+										        				+ sck.isConnected() + ";isIShutD "
+										        				+ sck.isInputShutdown() + ";isOShutD "
+										        				+ sck.isOutputShutdown());
+										        		System.out.println("- - - -");
+										        		System.out.println(line);
+
+										        		while (br.ready()) {
+										        			String extraline = br.readLine();
+										        			System.out.println(extraline);
+										        		}
+										        		System.out.println("=====");
+										        	}
+
+
 										        	if ( line != null ){
 										        		
 											        	if ( line.toUpperCase().startsWith( "GET " )){
@@ -178,7 +200,7 @@ MagnetURIHandlerImpl
 											        		
 											        		line = line.substring( 0, pos );
 											        		
-											        		close_socket = process( line, sck.getOutputStream() );
+											        		close_socket = process( line, br, sck.getOutputStream() );
 											        		
 											        	}else{
 											        		
@@ -193,6 +215,7 @@ MagnetURIHandlerImpl
 											       				+ "'" + address + "': no data read"));
 									        		
 										        	}
+										        	
 											   }else{
 												   
 											      	Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING,
@@ -250,6 +273,7 @@ MagnetURIHandlerImpl
 	protected boolean
 	process(
 		String			get,
+		BufferedReader	is,
 		OutputStream	os )
 	
 		throws IOException
@@ -266,6 +290,9 @@ MagnetURIHandlerImpl
 		if ( pos != -1 ){
 					
 			StringTokenizer	tok = new StringTokenizer( get.substring( pos+1 ), "&" );
+			if (DEBUG) {
+				System.out.println("params:" + get.substring( pos+1 ));
+			}
 			
 			while( tok.hasMoreTokens()){
 				
@@ -601,30 +628,130 @@ MagnetURIHandlerImpl
 				
 				Integer	info = (Integer)info_map.get( name );
 				
+				int	value = Integer.MIN_VALUE;
+				
 				if ( info != null ){					
 					
-					ByteArrayOutputStream	baos = new ByteArrayOutputStream();
+					value = info.intValue();
 					
-					int	width 	= info.intValue();
-					int	height	= 1;
+				}else{
 					
-					BufferedImage	image = new BufferedImage( width, height, BufferedImage.TYPE_INT_RGB );
+					HashMap paramsCopy = new HashMap();
+					paramsCopy.putAll(params);
+
+					for (int i=0;i<listeners.size() && value == Integer.MIN_VALUE;i++){
+						
+						value = ((MagnetURIHandlerListener)listeners.get(i)).get( name, paramsCopy );
+					}
+				}
+								
+				if ( value == Integer.MIN_VALUE ){
 					
-					for (int i=0;i<width;i++){
-						for (int j=0;j<height;j++){
-							int	red 	= i+j;
-							int	green 	= i*255/width;
-							int	blue	= j*255/height;
+						// no value, see if we have a default
+					
+					String	def_str = (String)params.get( "default" );
+
+					if ( def_str != null ){
+						
+						try{
+							value = Integer.parseInt( def_str );
 							
-							image.setRGB(i,j,((red<<16)&0xff0000)|((green<<8)&0xff00)|((blue)&0xff));
+						}catch( Throwable e ){
+							
+							Debug.printStackTrace( e );
 						}
 					}
-					ImageIO.write( image, "JPG", baos );
+				}else{
+					
+						// have a value, see if we have a max
+					
+					String	max_str = (String)params.get( "max" );
 
-					byte[]	data = baos.toByteArray();
-											
-					writeReply( os, "image/jpeg", data );
+					if ( max_str != null ){
 						
+						try{
+							int	max = Integer.parseInt( max_str );
+							
+							if ( value > max ){
+								
+								value = max;
+							}
+						}catch( Throwable e ){
+							
+							Debug.printStackTrace(e);
+						}
+					}
+				}
+				
+				if ( value != Integer.MIN_VALUE ){
+	
+					if ( value < 0 ){
+						
+						value = 0;
+					}
+					
+						// need to trim if too large
+					
+					if ( value > 1024 * 1024 ){
+						
+						value = 1024 * 1024;	
+					}
+										
+						// see if we need to div/mod for clients that don't support huge images
+						// e.g. http://localhost:45100/getinfo?name=Plugin.azupnpav.content_port&mod=8
+					
+					int	width 	= value;
+					int	height	= 1;
+
+						// divmod -> encode div+1 as width, mod+1 as height
+					
+					String	div_mod = (String)params.get( "divmod" );
+					
+					if ( div_mod != null ){
+						
+						int	n = Integer.parseInt( div_mod );
+						
+						width 	= ( value / n ) + 1;
+						height	= ( value % n ) + 1;
+						
+					}else{
+						
+						String	div = (String)params.get( "div" );
+						
+						if ( div != null ){
+							
+							width = value / Integer.parseInt( div );
+							
+						}else{
+							
+							String	mod = (String)params.get( "mod" );
+							
+							if ( mod != null ){
+								
+								width = value % Integer.parseInt( mod );
+							}
+						}
+					}
+
+					String	img_type = (String)params.get( "img_type" );
+					
+					if ( img_type != null && img_type.equals( "png" )){
+						
+						byte[]	data = PNG.getPNGBytesForSize( width, height );
+												
+						writeReply( os, "image/png", data );
+						
+					}else{
+						
+						ByteArrayOutputStream	baos = new ByteArrayOutputStream();				
+
+						writeImage( baos, width, height );
+						
+						byte[]	data = baos.toByteArray();
+												
+						writeReply( os, "image/bmp", data );
+					}
+					
 					return( true );
 				}
 			}
@@ -633,11 +760,135 @@ MagnetURIHandlerImpl
 			
 			return( true );
 			
+		}else if ( get.startsWith( "/setinfo?" )){
+
+			String name 	= (String)params.get( "name" );
+			
+			HashMap paramsCopy = new HashMap();
+			paramsCopy.putAll(params);
+
+			if ( name != null ){
+
+				boolean	result = false;
+				
+				for (int i=0;i<listeners.size() && !result;i++){
+					
+					result = ((MagnetURIHandlerListener)listeners.get(i)).set( name, paramsCopy );
+				}
+				
+				int	width 	= result?20:10;
+				int height 	= result?20:10;
+				
+				String	img_type = (String)params.get( "img_type" );
+
+				if ( img_type != null && img_type.equals( "png" )){
+					
+					byte[]	data = PNG.getPNGBytesForSize( width, height );
+					
+					writeReply( os, "image/png", data );
+
+				}else{
+					
+					ByteArrayOutputStream	baos = new ByteArrayOutputStream();
+	
+					writeImage( baos, width, height);
+					
+					byte[]	data = baos.toByteArray();
+											
+					writeReply( os, "image/bmp", data );
+				}
+				
+				return( true );
+			}
+		}else if ( get.equals( "/browserheaders.js" )){
+
+			String	headers_str = "";
+			
+			while( true ){
+				
+				String	header = is.readLine();
+				
+				if ( header == null ){
+					
+					break;
+				}
+				
+				header = header.trim();
+				
+				if ( header.length() == 0 ){
+					
+					break;
+				}
+				
+				headers_str += (headers_str.length()==0?"":"\n") + header;
+			}
+			
+			String script = "var headers = \"" + new String( Base64.encode( headers_str.getBytes( "UTF-8" ))) + "\";";
+
+			
+			writeReply( os, "application/x-javascript", script );
 		}
 		
 		return( true );
 	}
 	
+	/**
+	 * @param os
+	 * @param width
+	 * @param height
+	 *
+	 * @since 3.0.2.1
+	 */
+	private void writeImage(OutputStream os, int width, int height) {
+		
+			// DON'T CHANGE ANY OF THIS WITHOUT (AT LEAST) CHANGING THE JWS LAUNCHER CODE
+			// AS IT MANUALLY DECODES THE BMP TO DETERMINE ITS SIZE!!!!
+		
+		int rowWidth = width / 8;
+		if ((rowWidth % 4) != 0) {
+			rowWidth = ((rowWidth / 4) + 1) * 4;
+		}
+		int imageSize = rowWidth * height;
+		int fileSize = 54 + imageSize;
+		try {
+			os.write(new byte[] {
+				'B',
+				'M'
+			});
+			write4Bytes(os, fileSize);
+			write4Bytes(os, 0);
+			write4Bytes(os, 54); // data pos
+
+			write4Bytes(os, 40); // header size
+			write4Bytes(os, width);
+			write4Bytes(os, height);
+			write4Bytes(os, (1 << 16) + 1); // 1 plane and 1 bpp color
+			write4Bytes(os, 0);
+			write4Bytes(os, imageSize);
+			write4Bytes(os, 0);
+			write4Bytes(os, 0);
+			write4Bytes(os, 0);
+			write4Bytes(os, 0);
+
+			byte[] data = new byte[imageSize];
+			os.write(data);
+
+		} catch (IOException e) {
+			Debug.out(e);
+		}
+	}
+	
+	private void write4Bytes(OutputStream os, long l) {
+		try {
+			os.write((int) (l & 0xFF));
+			os.write((int) ((l >> 8) & 0xFF));
+			os.write((int) ((l >> 16) & 0xFF));
+			os.write((int) ((l >> 24) & 0xFF));
+		} catch (IOException e) {
+			Debug.out(e);
+		}
+	}
+
 	protected String
 	getMessageText(
 		String	resource )
@@ -695,7 +946,7 @@ MagnetURIHandlerImpl
 		pw.print( "Content-Length: " + content.length + NL + NL );
 		
 		pw.flush();
-		
+
 		os.write( content );
 
 	}
