@@ -67,7 +67,7 @@ PRUDPPacketHandlerImpl
 	private PRUDPPacketHandlerStatsImpl	stats = new PRUDPPacketHandlerStatsImpl( this );
 	
 	
-	private Map			requests = new HashMap();
+	private Map			requests = new LightHashMap();
 	private AEMonitor	requests_mon	= new AEMonitor( "PRUDPPH:req" );
 	
 	
@@ -114,7 +114,7 @@ PRUDPPacketHandlerImpl
 		port				= _port;
 		explicit_bind_ip	= _bind_ip;
 		
-		default_bind_ip = NetworkAdmin.getSingleton().getDefaultBindAddress();
+		default_bind_ip = NetworkAdmin.getSingleton().getSingleHomedServiceBindAddress();
 		
 		calcBind();
 		
@@ -274,6 +274,8 @@ PRUDPPacketHandlerImpl
 	receiveLoop(
 		AESemaphore	init_sem )
 	{
+		long	last_socket_close_time = 0;
+		
 		NetworkAdminPropertyChangeListener prop_listener = 
 			new NetworkAdminPropertyChangeListener()
 	    	{
@@ -283,7 +285,7 @@ PRUDPPacketHandlerImpl
 	    		{
 	    			if ( property == NetworkAdmin.PR_DEFAULT_BIND_ADDRESS ){
 	    				
-	    				setDefaultBindAddress( NetworkAdmin.getSingleton().getDefaultBindAddress());
+	    				setDefaultBindAddress( NetworkAdmin.getSingleton().getSingleHomedServiceBindAddress());
 	    			}
 	    		}
 	    	};
@@ -392,14 +394,41 @@ PRUDPPacketHandlerImpl
 					
 					}catch( SocketTimeoutException e ){
 											
-					}catch( Throwable e ){
+					}catch( Throwable e ){							
+						
+							// on vista we get periodic socket closures
+						
+						String	message = e.getMessage();
+						
+						if ( 	socket.isClosed() || 
+								( message != null && message.toLowerCase().indexOf( "socket closed" ) != -1 )){
 							
+							long	now = SystemTime.getCurrentTime();
+							
+								// can't guarantee there aren't situations where we get into a screaming
+								// closed loop so guard against this somewhat
+								
+							if ( now - last_socket_close_time < 500 ){
+								
+								Thread.sleep( 250 );
+							}
+							
+							last_socket_close_time = now;
+							
+							if (Logger.isEnabled())
+								Logger.log(new LogEvent(LOGID,
+										"PRUDPPacketReceiver: recycled UDP port " + port + " after close: ok=" + successful_accepts ));
+
+							break;
+						}
+
 						failed_accepts++;
 						
 						if (Logger.isEnabled())
 							Logger.log(new LogEvent(LOGID,
-									"PRUDPPacketReceiver: receive failed on port " + port, e)); 
+									"PRUDPPacketReceiver: receive failed on port " + port + ": ok=" + successful_accepts + ", fails=" + failed_accepts, e)); 
 	
+					
 						if (( failed_accepts > 100 && successful_accepts == 0 ) || failed_accepts > 1000 ){						
 			
 							Logger.logTextResource(new LogAlert(LogAlert.UNREPEATABLE,
@@ -774,6 +803,22 @@ PRUDPPacketHandlerImpl
 		return( request.getReply());
 	}
 	
+	public PRUDPPacket
+	sendAndReceive(
+		PasswordAuthentication	auth,
+		PRUDPPacket				request_packet,
+		InetSocketAddress		destination_address,
+		long					timeout,
+		int						priority )
+	
+		throws PRUDPPacketHandlerException
+	{
+		PRUDPPacketHandlerRequestImpl	request = 
+			sendAndReceive( auth, request_packet, destination_address, null, timeout, priority );
+		
+		return( request.getReply());
+	}
+	
 	public void
 	sendAndReceive(
 		PRUDPPacket					request_packet,
@@ -1095,7 +1140,7 @@ PRUDPPacketHandlerImpl
 	
 		throws PRUDPPacketHandlerException
 	{
-		if ( socket == null ){
+		if ( socket == null || socket.isClosed()){
 			
 			if ( init_error != null ){
 				
@@ -1202,6 +1247,16 @@ PRUDPPacketHandlerImpl
 	
 		throws PRUDPPacketHandlerException
 	{
+		if ( socket == null || socket.isClosed()){
+			
+			if ( init_error != null ){
+				
+				throw( new PRUDPPacketHandlerException( "Transport unavailable", init_error ));
+			}
+			
+			throw( new PRUDPPacketHandlerException( "Transport unavailable" ));
+		}
+		
 		try{
 			checkTargetAddress( target );
 			
