@@ -1,7 +1,7 @@
 /*
  * Created on 14-Jun-2004
  * Created by Paul Gardner
- * Copyright (C) 2004 Aelitis, All Rights Reserved.
+ * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * 
- * AELITIS, SARL au capital de 30,000 euros
+ * AELITIS, SAS au capital de 46,603.30 euros
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  *
  */
@@ -30,7 +30,17 @@ package com.aelitis.net.upnp;
 public interface 
 UPnPLogListener 
 {
+	public static final int	TYPE_ALWAYS					= 1;
+	public static final int	TYPE_ONCE_PER_SESSION		= 2;
+	public static final int	TYPE_ONCE_EVER				= 3;
+	
 	public void
 	log(
 		String		str );
+	
+	public void
+	logAlert(
+		String		str,
+		boolean		error,
+		int			type );	
 }
