@@ -1,7 +1,7 @@
 /*
  * Created on 14-Jun-2004
  * Created by Paul Gardner
- * Copyright (C) 2004 Aelitis, All Rights Reserved.
+ * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * 
- * AELITIS, SARL au capital de 30,000 euros
+ * AELITIS, SAS au capital de 46,603.30 euros
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  *
  */
@@ -31,10 +31,12 @@ import java.util.*;
 import java.net.*;
 import java.io.*;
 
+import com.aelitis.azureus.core.util.Java15Utils;
+
 import org.gudy.azureus2.core3.util.AEMonitor;
+import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.plugins.PluginInterface;
-import org.gudy.azureus2.plugins.logging.*;
+import org.gudy.azureus2.core3.util.ThreadPool;
 
 import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
 import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderAdapter;
@@ -43,26 +45,24 @@ import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderFact
 import org.gudy.azureus2.plugins.utils.xml.simpleparser.SimpleXMLParserDocument;
 import org.gudy.azureus2.plugins.utils.xml.simpleparser.SimpleXMLParserDocumentException;
 
+import com.aelitis.azureus.core.util.HTTPUtils;
 import com.aelitis.net.upnp.*;
 import com.aelitis.net.upnp.impl.device.*;
-import com.aelitis.net.upnp.services.UPnPWANConnectionPortMapping;
-import com.aelitis.net.upnp.services.UPnPWANIPConnection;
 
 public class 
 UPnPImpl
 	extends 	ResourceDownloaderAdapter
-	implements 	UPnP, SSDPListener
-{
-	public static final boolean	USE_HTTP_CONNECTION		= true;
-	
+	implements 	UPnP, SSDPIGDListener
+{	
 	public static final String	NL	= "\r\n";
 	
-	protected static UPnPImpl	singleton;
-	protected static AEMonitor	class_mon 	= new AEMonitor( "UPnP:class" );
+	private static UPnPImpl	singleton;
+	private static AEMonitor	class_mon 	= new AEMonitor( "UPnP:class" );
 	
 	public static UPnP
 	getSingleton(
-		PluginInterface		plugin_interface )
+		UPnPAdapter		adapter,
+		String[]		selected_interfaces )
 	
 		throws UPnPException
 	{
@@ -71,7 +71,7 @@ UPnPImpl
 		
 			if ( singleton == null ){
 				
-				singleton = new UPnPImpl( plugin_interface );
+				singleton = new UPnPImpl( adapter, selected_interfaces );
 			}
 			
 			return( singleton );
@@ -82,33 +82,38 @@ UPnPImpl
 		}
 	}
 	
-	protected PluginInterface		plugin_interface;
-	protected LoggerChannel			log;
-	protected SSDP					ssdp;
+	private UPnPAdapter				adapter;
+	private SSDPIGD					ssdp;
 	
-	protected Map		root_locations	= new HashMap();
+	private Map			root_locations	= new HashMap();
 	
-	protected List		log_listeners	= new ArrayList();
-	protected List		log_history		= new ArrayList();
+	private List		log_listeners		= new ArrayList();
+	private List		log_history			= new ArrayList();
+	private List		log_alert_history	= new ArrayList();
 	
-	protected List		rd_listeners		= new ArrayList();
-	protected AEMonitor	rd_listeners_mon 	= new AEMonitor( "UPnP:L" );
+	private List		rd_listeners		= new ArrayList();
+	private AEMonitor	rd_listeners_mon 	= new AEMonitor( "UPnP:L" );
 
-	protected int		trace_index		= 0;
+	private int		http_calls_ok	= 0;
+	private int		direct_calls_ok	= 0;
+	
+	private int		trace_index		= 0;
+	
+	private ThreadPool	device_dispatcher	 = new ThreadPool("UPnPDispatcher", 1, true );
+	private Set			device_dispatcher_pending	= new HashSet();
 	
 	protected AEMonitor	this_mon 	= new AEMonitor( "UPnP" );
 
 	protected
 	UPnPImpl(
-		PluginInterface		_plugin_interface )
+		UPnPAdapter		_adapter,
+		String[]		_selected_interfaces )
 	
 		throws UPnPException
 	{
-		plugin_interface	= _plugin_interface;
+		adapter	= _adapter;
 		
-		log		= plugin_interface.getLogger().getChannel("UPnP Core");
-		
-		ssdp = SSDPFactory.create( this );
+		ssdp = SSDPIGDFactory.create( this, _selected_interfaces );
 		
 		ssdp.addListener(this);
 		
@@ -117,70 +122,168 @@ UPnPImpl
 	
 	public void
 	rootDiscovered(
-		InetAddress	local_address,
-		URL			location )
+		final NetworkInterface		network_interface,
+		final InetAddress			local_address,
+		final String				usn,
+		final URL					location )
 
 	{
-		UPnPRootDeviceImpl root_device = (UPnPRootDeviceImpl)root_locations.get( location.getHost());
+		
+			// we need to take this operation off the main thread as it can take some time. This is a single
+			// concurrency queued thread pool so things get done serially in the right order
+		
+		try{
+			rd_listeners_mon.enter();
+
+			if ( device_dispatcher_pending.contains( usn )){
 			
-		if ( root_device != null ){
-	
-				// device is still there with same IP, however 
-				// 1) port of UPnP device might have changed (it does on mine when enabling/disabling UPnP)
-				// 2) our local IP might have changed (DHCP reassignment)
-			
-			if ( 	root_device.getLocation().equals( location ) &&
-					root_device.getLocalAddress().equals( local_address )){
-				
-					// everythings the same, nothing to do
+				// System.out.println( "UPnP: skipping discovery of " + usn + " as already pending (queue=" + device_dispatcher_pending.size() + ")" );
 				
 				return;
 			}
 			
-				// something changed, resetablish everything
+			if ( device_dispatcher_pending.size() > 512 ){
+				
+				Debug.out( "Device dispatcher queue is full - dropping discovery of " + usn + "/" + location );
+			}
 			
-			root_locations.remove( location.getHost());
+			device_dispatcher_pending.add( usn );
+	
+		}finally{
 			
-			root_device.destroy( true );
+			rd_listeners_mon.exit();
 		}
 		
-		log( "UPnP: root discovered = " + location + ", local = " + local_address.toString() );
-		
-		try{
-			root_device = new UPnPRootDeviceImpl( this, local_address, location );
-		
-			List	listeners;
-			
-			try{
-				rd_listeners_mon.enter();
-				
-				root_locations.put( location.getHost(), root_device );
+		device_dispatcher.run(
+			new AERunnable()
+			{
+				public void
+				runSupport()
+				{
+					final UPnPRootDeviceImpl old_root_device;
+					
+					try{
+						rd_listeners_mon.enter();
 
-				listeners = new ArrayList( rd_listeners );
+						old_root_device = (UPnPRootDeviceImpl)root_locations.get( usn );
+						
+						device_dispatcher_pending.remove( usn );
+						
+					}finally{
+						
+						rd_listeners_mon.exit();
+					}
+					
+					if ( old_root_device != null ){
 				
-			}finally{
-				
-				rd_listeners_mon.exit();
-			}
-			
-			for (int i=0;i<listeners.size();i++){
-				
-				((UPnPListener)listeners.get(i)).rootDeviceFound( root_device );
-				
-			}
+							// we remember one route to the device - if the network interfaces change
+							// we do a full reset so we don't need to deal with that here
+						
+						if ( !old_root_device.getNetworkInterface().getName().equals( network_interface.getName())){
+							
+							return;
+						}
+						
+							// check that the device's location is the same
+												
+						if ( old_root_device.getLocation().equals( location )){		
+
+							return;
+						}
+					}
+					
+					if ( old_root_device != null ){
+						
+							// something changed, resetablish everything
+						
+						try{
+								// not the best "atomic" code here but it'll do as the code that adds roots (this)
+								// is single threaded via the dispatcher
+							
+							rd_listeners_mon.enter();
+
+							root_locations.remove( usn );
+						
+						}finally{
+							
+							rd_listeners_mon.exit();
+						}
+						
+						old_root_device.destroy( true );
+					}
 		
-		}catch( UPnPException e ){
+					List	listeners;
+					
+					try{
+						rd_listeners_mon.enter();
+						
+						listeners = new ArrayList( rd_listeners );
+						
+					}finally{
+						
+						rd_listeners_mon.exit();
+					}
+					
+					for (int i=0;i<listeners.size();i++){
+						
+						try{
+							if ( !((UPnPListener)listeners.get(i)).deviceDiscovered( usn, location )){
+								
+								return;
+							}
+							
+						}catch( Throwable e ){
+							
+							Debug.printStackTrace(e);
+						}
+					}				
+
+					log( "UPnP: root discovered: usn=" + usn + ", location=" + location + ", ni=" + network_interface.getName() + ",local=" + local_address.toString() );
+					
+					try{
+						UPnPRootDeviceImpl new_root_device = new UPnPRootDeviceImpl( UPnPImpl.this, network_interface, local_address, usn, location );
+											
+						try{
+							rd_listeners_mon.enter();
+							
+							root_locations.put( usn, new_root_device );
+								
+							listeners = new ArrayList( rd_listeners );
+
+						}finally{
+							
+							rd_listeners_mon.exit();
+						}
 			
-			log( e.toString());
-		}
+						for (int i=0;i<listeners.size();i++){
+							
+							try{
+								((UPnPListener)listeners.get(i)).rootDeviceFound( new_root_device );
+								
+							}catch( Throwable e ){
+								
+								Debug.printStackTrace(e);
+							}
+						}
+					
+					}catch( UPnPException e ){
+						
+						String	message = e.getMessage();
+						
+						String msg = message==null?Debug.getNestedExceptionMessageAndStack( e ):message;
+												
+						adapter.log( msg );
+					}
+				}
+			});
 	}
 	
 	public void
 	rootAlive(
+		String		usn,
 		URL			location )
-
 	{
-		UPnPRootDeviceImpl root_device = (UPnPRootDeviceImpl)root_locations.get( location.getHost());
+		UPnPRootDeviceImpl root_device = (UPnPRootDeviceImpl)root_locations.get( usn );
 			
 		if ( root_device == null ){
 			
@@ -190,29 +293,46 @@ UPnPImpl
 	
 	public void
 	rootLost(
-		InetAddress	local_address,
-		URL			location )
+		final InetAddress	local_address,
+		final String		usn )
 	{
-		UPnPRootDeviceImpl	root_device;
-	
-		try{
-			rd_listeners_mon.enter();
-
-			root_device = (UPnPRootDeviceImpl)root_locations.remove( location.getHost());
-			
-		}finally{
-			
-			rd_listeners_mon.exit();
-		}
+			// we need to take this operation off the main thread as it can take some time
 		
-		if ( root_device == null ){
+		device_dispatcher.run(
+			new AERunnable()
+			{
+				public void
+				runSupport()
+				{
+					UPnPRootDeviceImpl	root_device	= null;
+									
+					try{
+						rd_listeners_mon.enter();
 			
-			return;
-		}
-		
-		log( "UPnP: root lost = " + location +", local = " + local_address.toString() );
+						root_device = (UPnPRootDeviceImpl)root_locations.remove( usn );
+				
+					}finally{
+						
+						rd_listeners_mon.exit();
+					}
+					
+					if ( root_device == null ){
+						
+						return;
+					}
+									
+					log( "UPnP: root lost: usn=" + usn + ", location=" + root_device.getLocation() + ", ni=" + root_device.getNetworkInterface().getName() + ",local=" + root_device.getLocalAddress().toString());
+				
+					root_device.destroy( false );
+				}
+			});
+	}
 	
-		root_device.destroy( false );
+	public void
+	interfaceChanged(
+		NetworkInterface	network_interface )
+	{
+		reset();
 	}
 	
 	public void
@@ -285,6 +405,8 @@ UPnPImpl
 			
 			LineNumberReader	lnr = new LineNumberReader( new InputStreamReader( is, "UTF-8" ));
 			
+			Set	ignore_map = null;
+			
 			while( true ){
 				
 				String	line = lnr.readLine();
@@ -294,14 +416,44 @@ UPnPImpl
 					break;
 				}
 				
-				data.append( line.trim() + "\n" );	
+					// remove any obviously invalid characters - I've seen some routers generate stuff like
+					// 0x18 which stuffs the xml parser with "invalid unicode character"
+				
+				for (int i=0;i<line.length();i++){
+					
+					char	c = line.charAt(i);
+				
+					if ( c < 0x20 && c != '\r' && c != '\t' ){
+						
+						data.append( ' ' );
+						
+						if ( ignore_map == null ){
+							
+							ignore_map = new HashSet();
+						}
+						
+						Character	cha = new Character(c);
+						
+						if ( !ignore_map.contains( cha )){
+						
+							ignore_map.add( cha );
+							
+							adapter.trace( "    ignoring character(s) " + (int)c + " in xml response" );
+						}
+					}else{
+						
+						data.append( c );
+					}
+				}
+				
+				data.append( "\n" );				
 			}
 				
 			String	data_str = data.toString();
 			
-			log.log( "UPnP:Response:" + data_str );
+			adapter.trace( "UPnP:Response:" + data_str );
 			
-			return( plugin_interface.getUtilities().getSimpleXMLParserDocumentFactory().create( data_str ));
+			return( adapter.parseXML( data_str ));
 			
 		}catch( Throwable e ){
 			
@@ -314,7 +466,7 @@ UPnPImpl
 				
 			}catch( Throwable f ){
 				
-				Debug.printStackTrace( f );
+				adapter.log(f);
 			}
 			
 			if ( e instanceof SimpleXMLParserDocumentException ){
@@ -328,24 +480,69 @@ UPnPImpl
 	
 	public SimpleXMLParserDocument
 	downloadXML(
-		URL		url )
+		URL				url )
+	
+		throws UPnPException
+	{
+		return( downloadXML( null, url ));
+	}
+	
+	public SimpleXMLParserDocument
+	downloadXML(
+		UPnPDeviceImpl	device,
+		URL				url )
 	
 		throws UPnPException
 	{
-		ResourceDownloaderFactory rdf = plugin_interface.getUtilities().getResourceDownloaderFactory();
-		
-		ResourceDownloader rd = rdf.getRetryDownloader( rdf.create( url ), 3 );
-		
-		rd.addListener( this );
-		
 		try{
+				// some devices have borked relative urls, work around
+			
+			if ( device != null ){
+				
+				device.restoreRelativeBaseURL();
+			}
+			
+			return( downloadXMLSupport( device, url ));
+			
+		}catch( UPnPException e ){
+			
+			if ( device != null ){
+				
+				device.clearRelativeBaseURL();
+				
+				return( downloadXMLSupport( device, url ));
+			}
+			
+			throw( e );
+		}
+	}
+	
+	protected SimpleXMLParserDocument
+	downloadXMLSupport(
+		UPnPDeviceImpl	device,
+		URL				url )
+	
+		throws UPnPException
+	{
+		try{
+			ResourceDownloaderFactory rdf = adapter.getResourceDownloaderFactory();
+				
+			ResourceDownloader rd = rdf.getRetryDownloader( rdf.create( url, true ), 3 );
+				
+			rd.addListener( this );
+				
 			InputStream	data = rd.download();
-			
-			return( parseXML( data ));
-			
+					
+			try{
+				return( parseXML( data ));
+					
+			}finally{
+					
+				data.close();
+			}
 		}catch( Throwable e ){
-			
-			log( e );
+				
+			adapter.log( Debug.getNestedExceptionMessageAndStack(e));
 
 			if (e instanceof UPnPException ){
 				
@@ -356,6 +553,22 @@ UPnPImpl
 		}	
 	}
 	
+	protected boolean
+	forceDirect()
+	{
+		String	http_proxy 	= System.getProperty( "http.proxyHost" );
+		String	socks_proxy = System.getProperty( "socksProxyHost" );
+
+			// extremely unlikely we want to proxy upnp requests
+		
+		boolean force_direct = 	( http_proxy != null && http_proxy.trim().length() > 0 ) ||
+								( socks_proxy != null && socks_proxy.trim().length() > 0 );
+
+		return( force_direct );
+	}
+	
+
+	
 	public SimpleXMLParserDocument
 	performSOAPRequest(
 		UPnPService		service,
@@ -364,127 +577,152 @@ UPnPImpl
 	
 		throws SimpleXMLParserDocumentException, UPnPException, IOException
 	{
-		log.log( "UPnP:Request:" + request );
+		SimpleXMLParserDocument	res;
+				
+		if ( service.getDirectInvocations() || forceDirect()){
+			
+			res = performSOAPRequest( service, soap_action, request, false );
 
+		}else{
+			
+			try{
+				res =  performSOAPRequest( service, soap_action, request, true );
+					
+				http_calls_ok++;
+				
+			}catch( IOException e ){
+				
+				res = performSOAPRequest( service, soap_action, request, false );
+				
+				direct_calls_ok++;
+				
+				if ( direct_calls_ok == 1 ){
+					
+					log( "Invocation via http connection failed (" + e.getMessage() + ") but socket connection succeeded" );
+				}
+			}
+		}
+		
+		return( res );
+	}
+	
+	/**
+	 * The use_http_connection flag is set to false sometimes to avoid using
+	 * the URLConnection library for some dopey UPnP routers.
+	 */
+	public SimpleXMLParserDocument
+	performSOAPRequest(
+		UPnPService		service,
+		String			soap_action,
+		String			request,
+		boolean			use_http_connection)
+	
+		throws SimpleXMLParserDocumentException, UPnPException, IOException
+	{
 		URL	control = service.getControlURL();
 		
-		if ( USE_HTTP_CONNECTION ){
-			
-			HttpURLConnection	con = (HttpURLConnection)control.openConnection();
-			
+		adapter.trace( "UPnP:Request: -> " + control + "," + request );
+
+		if (use_http_connection) {
+			HttpURLConnection	con = (HttpURLConnection)Java15Utils.openConnectionForceNoProxy(control);
+				
 			con.setRequestProperty( "SOAPAction", "\""+ soap_action + "\"");
-			
+				
 			con.setRequestProperty( "Content-Type", "text/xml; charset=\"utf-8\"" );
-			
+				
 			con.setRequestProperty( "User-Agent", "Azureus (UPnP/1.0)" );
-			
+				
 			con.setRequestMethod( "POST" );
-			
+				
 			con.setDoInput( true );
 			con.setDoOutput( true );
-			
+				
 			OutputStream	os = con.getOutputStream();
-			
+				
 			PrintWriter	pw = new PrintWriter( new OutputStreamWriter(os, "UTF-8" ));
-						
+							
 			pw.println( request );
-			
+				
 			pw.flush();
-	
+		
 			con.connect();
-			
-			if ( con.getResponseCode() == 405 ){
 				
-					// gotta retry with M-POST method
-								
+			if ( con.getResponseCode() == 405 || con.getResponseCode() == 500 ){
+					
+				// gotta retry with M-POST method
+									
 				con = (HttpURLConnection)control.openConnection();
-				
+					
 				con.setRequestProperty( "Content-Type", "text/xml; charset=\"utf-8\"" );
-				
+					
 				con.setRequestMethod( "M-POST" );
-				
+					
 				con.setRequestProperty( "MAN", "\"http://schemas.xmlsoap.org/soap/envelope/\"; ns=01" );
-	
+		
 				con.setRequestProperty( "01-SOAPACTION", "\""+ soap_action + "\"");
-				
+					
 				con.setDoInput( true );
 				con.setDoOutput( true );
-				
+					
 				os = con.getOutputStream();
-				
+					
 				pw = new PrintWriter( new OutputStreamWriter(os, "UTF-8" ));
-							
+								
 				pw.println( request );
-				
+					
 				pw.flush();
-	
+		
 				con.connect();
-			
+				
 				return( parseXML(con.getInputStream()));	
-				
+					
 			}else{
-				
+					
 				return( parseXML(con.getInputStream()));
 			}
-		}else{
-	
+		}
+		else {
 			Socket	socket = new Socket(control.getHost(), control.getPort());
 			
-			PrintWriter	pw = new PrintWriter(new OutputStreamWriter( socket.getOutputStream(), "UTF8" ));
-		
-			String	url_target = control.toString();
-			
-			int	p1 	= url_target.indexOf( "://" ) + 3;
-			p1		= url_target.indexOf( "/", p1 );
-			
-			url_target = url_target.substring( p1 );
-			
-			pw.print( "POST " + url_target + " HTTP/1.1" + NL );
-			pw.print( "Content-Type: text/xml; charset=\"utf-8\"" + NL );
-			pw.print( "SOAPAction: \"" + soap_action + "\"" + NL );
-			pw.print( "User-Agent: Azureus (UPnP/1.0)" + NL );
-			pw.print( "Host: " + control.getHost() + NL );
-			pw.print( "Content-Length: " + request.getBytes( "UTF8" ).length + NL );
-			pw.print( "Connection: Keep-Alive" + NL );
-			pw.print( "Pragma: no-cache" + NL + NL );
+			try{
+				PrintWriter	pw = new PrintWriter(new OutputStreamWriter( socket.getOutputStream(), "UTF8" ));
 
-			pw.print( request );
+				String	url_target = control.toString();
 			
-			pw.flush();
-			
-			InputStream	is = socket.getInputStream();
-			
-			String	reply_header = "";
-			
-			while(true){
+				int	p1 	= url_target.indexOf( "://" ) + 3;
+				p1		= url_target.indexOf( "/", p1 );
 				
-				byte[]	buffer = new byte[1];
+				url_target = url_target.substring( p1 );
 				
-				if ( is.read( buffer ) <= 0 ){
+				pw.print( "POST " + url_target + " HTTP/1.1" + NL );
+				pw.print( "Content-Type: text/xml; charset=\"utf-8\"" + NL );
+				pw.print( "SOAPAction: \"" + soap_action + "\"" + NL );
+				pw.print( "User-Agent: Azureus (UPnP/1.0)" + NL );
+				pw.print( "Host: " + control.getHost() + NL );
+				pw.print( "Content-Length: " + request.getBytes( "UTF8" ).length + NL );
+				pw.print( "Connection: Keep-Alive" + NL );
+				pw.print( "Pragma: no-cache" + NL + NL );
+	
+				pw.print( request );
+				
+				pw.flush();
+				
+				InputStream	is = HTTPUtils.decodeChunkedEncoding( socket.getInputStream());
+				
+				return( parseXML( is ));
+				
+			}finally{
+
+				try{
+					socket.close();
 					
-					throw( new IOException( "Premature end of input stream" ));
-				}
-				
-				reply_header += (char)buffer[0];
-				
-				if ( reply_header.endsWith( NL+NL )){
+				}catch( Throwable e ){
 					
-					break;
-				}
-			}
-			
-			p1 = reply_header.indexOf( NL );
-			
-			String	first_line = reply_header.substring( 0, p1 ).trim();
-			
-			if ( first_line.indexOf( "200" ) == -1 ){
-				
-				throw( new IOException( "HTTP request failed:" + first_line ));
-			}
-			
-			return( parseXML( is ));
-		}
-	}
+					Debug.printStackTrace(e);
+				} // end catch
+			} // end finally
+		} // end else
+	} // end method
 	
 	protected File
 	getTraceFile()
@@ -499,17 +737,17 @@ UPnPImpl
 				trace_index = 1;
 			}
 			
-			return( new File( plugin_interface.getUtilities().getAzureusUserDir(), "upnp_trace" + trace_index + ".log" ));
+			return( new File( adapter.getTraceDir(), "upnp_trace" + trace_index + ".log" ));
 		}finally{
 			
 			this_mon.exit();
 		}
 	}
 	
-	public PluginInterface
-	getPluginInterface()
+	public UPnPAdapter
+	getAdapter()
 	{
-		return( plugin_interface );
+		return( adapter );
 	}
 	
 	public void
@@ -564,15 +802,47 @@ UPnPImpl
 	}
 	
 	public void
-	addLogListener(
-		UPnPLogListener	l )
+	logAlert(
+		String	str,
+		boolean	error,
+		int		type )
 	{
-		List	old_logs;
+		List	old_listeners;
 		
 		try{
 			this_mon.enter();
 
-			old_logs = new ArrayList(log_history);
+			old_listeners = new ArrayList(log_listeners);
+
+			log_alert_history.add(new Object[]{ str, new Boolean( error ), new Integer( type )});
+			
+			if ( log_alert_history.size() > 32 ){
+				
+				log_alert_history.remove(0);
+			}
+		}finally{
+			
+			this_mon.exit();
+		}
+		
+		for (int i=0;i<old_listeners.size();i++){
+	
+			((UPnPLogListener)old_listeners.get(i)).logAlert( str, error, type );
+		}
+	}
+	
+	public void
+	addLogListener(
+		UPnPLogListener	l )
+	{
+		List	old_logs;
+		List	old_alerts;
+		
+		try{
+			this_mon.enter();
+
+			old_logs 	= new ArrayList(log_history);
+			old_alerts 	= new ArrayList(log_alert_history);
 
 			log_listeners.add( l );
 		}finally{
@@ -584,6 +854,13 @@ UPnPImpl
 			
 			l.log((String)old_logs.get(i));
 		}
+		
+		for (int i=0;i<old_alerts.size();i++){
+			
+			Object[]	entry = (Object[])old_alerts.get(i);
+			
+			l.logAlert((String)entry[0], ((Boolean)entry[1]).booleanValue(), ((Integer)entry[2]).intValue());
+		}
 	}
 		
 	public void
@@ -613,7 +890,19 @@ UPnPImpl
 		
 		for (int i=0;i<old_locations.size();i++){
 			
-			l.rootDeviceFound(((UPnPRootDevice)old_locations.get(i)));
+			UPnPRootDevice	device = (UPnPRootDevice)old_locations.get(i);
+			
+			try{
+				
+				if ( l.deviceDiscovered( device.getUSN(), device.getLocation())){
+					
+					l.rootDeviceFound(device);
+				}
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
+			}
 		}
 	}
 		
@@ -621,18 +910,36 @@ UPnPImpl
 	removeRootDeviceListener(
 		UPnPListener	l )
 	{
-		rd_listeners.remove( l );
+		try{
+			this_mon.enter();
+
+			rd_listeners.remove( l );
+			
+		}finally{
+			
+			this_mon.exit();
+		}
 	}
+	
+	/*
 	public static void
 	main(
 		String[]		args )
 	{
 		try{
-			UPnP	upnp = UPnPFactory.getSingleton(null);	// won't work with null ....
+			UPnP	upnp = UPnPFactory.getSingleton(null,null);	// won't work with null ....
 				
 			upnp.addRootDeviceListener(
 					new UPnPListener()
 					{
+						public boolean
+						deviceDiscovered(
+							String		USN,
+							URL			location )
+						{
+							return( true );
+						}
+						
 						public void
 						rootDeviceFound(
 							UPnPRootDevice		device )
@@ -642,7 +949,7 @@ UPnPImpl
 								
 							}catch( Throwable e ){
 								
-								Debug.printStackTrace( e );
+								e.printStackTrace();
 							}
 						}						
 					});
@@ -656,13 +963,22 @@ UPnPImpl
 					{
 						System.out.println( str );
 					}
+					public void
+					logAlert(
+						String	str,
+						boolean	error,
+						int		type )
+					{
+						System.out.println( str );
+					}
+
 				});
 			
 			Thread.sleep(20000);
 			
 		}catch( Throwable e ){
 			
-			Debug.printStackTrace( e );
+			e.printStackTrace();
 		}
 	}
 	
@@ -734,4 +1050,5 @@ UPnPImpl
 			}
 		}
 	}
+	*/
 }
