@@ -25,9 +25,12 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.io.IOException;
+import java.net.URL;
 
 import org.gudy.azureus2.core3.config.impl.*;
 import org.gudy.azureus2.core3.util.Constants;
+import org.gudy.azureus2.core3.util.SystemProperties;
+import org.gudy.azureus2.core3.util.protocol.AzURLStreamHandlerFactory;
 
 public class 
 COConfigurationManager 
@@ -49,6 +52,8 @@ COConfigurationManager
 	    CONFIG_CACHE_SIZE_MAX_MB = size;
 	}
 	
+	public static final boolean	ENABLE_MULTIPLE_UDP_PORTS	= false;
+	
 	private static boolean	pre_initialised;
 	
 	public static synchronized void
@@ -59,6 +64,7 @@ COConfigurationManager
 			pre_initialised	= true;
 		
 			try{
+				/*
 			  	String	handlers = System.getProperty( "java.protocol.handler.pkgs" );
 			  	
 			  	if ( handlers == null ){
@@ -71,13 +77,23 @@ COConfigurationManager
 			  	}
 			  	
 			  	System.setProperty( "java.protocol.handler.pkgs", handlers );
+			  	*/
+				
+				URL.setURLStreamHandlerFactory(new AzURLStreamHandlerFactory());
 			  	 	
 			  		// DNS cache timeouts
 			  	
 			  	System.setProperty("sun.net.inetaddr.ttl", "60");
 			  	System.setProperty("networkaddress.cache.ttl", "60");
+			  	
+			  	
+			  	// flick AWT into headless mode, which is supposed to make it more lightweight
+			  	// don't do this as it borks (for example) swing based plugins, java webstart installer,
+			  	// swing webui plugin, ....
+			  	
+			  	// System.setProperty("java.awt.headless", "true");
 		      
-			  		// defaults, overridden later if needed
+		  		// defaults, overridden later if needed
 		      
 			  	System.setProperty( "sun.net.client.defaultConnectTimeout", "120000" );	
 			  	System.setProperty(	"sun.net.client.defaultReadTimeout", "60000" ); 
@@ -88,6 +104,9 @@ COConfigurationManager
 			    if( Constants.isOSX ) {
 			        System.setProperty( "java.nio.preferSelect", "true" );
 			    }
+			    
+			    SystemProperties.determineApplicationName();
+			    
 			}catch( Throwable e ){
 				
 					// can happen in applet
@@ -97,23 +116,29 @@ COConfigurationManager
 		}
 	}
 	
-	public static void
+	public static ConfigurationManager
 	initialise()
 	{
 		preInitialise();
 		
-		ConfigurationManager.getInstance();
+		return ConfigurationManager.getInstance();
 	}
 	
-	public static void
+	public static ConfigurationManager
 	initialiseFromMap(
 		Map		data )
 	{
 		preInitialise();
 		
-		ConfigurationManager.getInstance(data);
+		return ConfigurationManager.getInstance(data);
 	}
 
+	public static final boolean
+	isNewInstall()
+	{
+		return( ConfigurationManager.getInstance().isNewInstall());
+	}
+	
 	public static String
 	getStringParameter(
 		String		_name )
@@ -135,6 +160,12 @@ COConfigurationManager
 		return ConfigurationManager.getInstance().setParameter( parameter, value );
 	}
 
+	public static boolean
+	verifyParameter(String parameter, String value) 
+	{
+		return ConfigurationManager.getInstance().verifyParameter( parameter, value );
+	}
+	
 	public static boolean
 	getBooleanParameter(
 		String		_name )
@@ -142,6 +173,9 @@ COConfigurationManager
 		return( ConfigurationManager.getInstance().getBooleanParameter( _name ));
 	}
 	
+	/**
+	 * @deprecated You should set ConfigurationDefaults, and use {@link #getBooleanParameter(String)} 
+	 */
 	public static boolean
 	getBooleanParameter(
 		String		_name,
@@ -163,6 +197,15 @@ COConfigurationManager
 		return( ConfigurationManager.getInstance().getIntParameter( _name ));
 	}
 	
+	
+	/**
+	 * Only use this for internal values, NOT for ones that the user can sensibly change. In this
+	 * case add the key to the configuration defaults and use the above method
+	 * @param _name
+	 * @param _def
+	 * @return
+	 */
+	
 	public static int
 	getIntParameter(
 		String		_name,
@@ -177,6 +220,39 @@ COConfigurationManager
 		return ConfigurationManager.getInstance().setParameter( parameter, value );
 	}
 	
+	public static boolean 
+	setParameter(String parameter, long value) 
+	{
+		return ConfigurationManager.getInstance().setParameter( parameter, value );
+	}
+	
+	public static long
+	getLongParameter(
+		String		_name )
+	{
+		return( ConfigurationManager.getInstance().getLongParameter( _name ));
+	}
+	
+	/**
+	 * Only use this for internal values, NOT for ones that the user can sensibly change. In this
+	 * case add the key to the configuration defaults and use the above method
+	 * @param _name
+	 * @param _def
+	 * @return
+	 */
+	
+	public static long
+	getLongParameter(
+		String		_name,
+		long		_def )
+	{
+		return( ConfigurationManager.getInstance().getLongParameter( _name, _def ));
+	}
+	
+	public static byte[] getByteParameter(String _name) {
+		return( ConfigurationManager.getInstance().getByteParameter(_name));
+	}
+
 	public static byte[]
 	getByteParameter(
 		String		_name,
@@ -220,7 +296,10 @@ COConfigurationManager
 	{
 		return ConfigurationManager.getInstance().setRGBParameter( parameter, red, green, blue);
 	}
-
+	
+	public static boolean setRGBParameter(String parameter, int[] rgb, boolean override) {
+		return ConfigurationManager.getInstance().setRGBParameter(parameter, rgb, override);
+	}
 
 	public static float
 	getFloatParameter(
@@ -229,6 +308,13 @@ COConfigurationManager
 		return( ConfigurationManager.getInstance().getFloatParameter( _name ));
 	}
 	
+	public static float
+	getFloatParameter(
+		String		_name,
+		float		_def )
+	{
+		return( ConfigurationManager.getInstance().getFloatParameter( _name, _def ));
+	}
 	public static boolean 
 	setParameter(String parameter, float value)
 	{
@@ -268,12 +354,34 @@ COConfigurationManager
 		return( ConfigurationManager.getInstance().getMapParameter( parameter, def ));
 	}
 	
+	  /**
+	   * Returns true if a parameter with the given name exists.
+	   * @param key The name of the parameter to check.
+	   * @param explicit If <tt>true</tt>, we only check for a value which is
+	   *     definitely stored explicitly, <tt>false</tt> means that we'll also
+	   *     check against configuration defaults too.  
+	   */
+	public static boolean hasParameter(String parameter, boolean explicit) {
+		return ConfigurationManager.getInstance().hasParameter(parameter, explicit);
+	}
+	
 	public static void
 	save()
 	{
 		ConfigurationManager.getInstance().save();
 	}
 	
+		/**
+		 * Mark as needing a save but not immediately - use when potentially needing a large number of saves that aren't
+		 * absolutely required to be immediately persisted
+		 */
+	
+	public static void
+	setDirty()
+	{
+		ConfigurationManager.getInstance().setDirty();
+	}
+	
 	public static void
 	addListener(
 		COConfigurationListener		listener )
@@ -287,6 +395,20 @@ COConfigurationManager
 	  ConfigurationManager.getInstance().addParameterListener(parameter, listener);
 	}
 
+	/**
+	 * @param strings
+	 * @param parameterListener
+	 *
+	 * @since 3.0.1.5
+	 */
+	public static void addParameterListener(String[] ids,
+			ParameterListener listener) {
+		ConfigurationManager instance = ConfigurationManager.getInstance();
+		for (int i = 0; i < ids.length; i++) {
+		  instance.addParameterListener(ids[i], listener);
+		}
+	}
+
 	public static void
 	addAndFireParameterListener(String parameter, ParameterListener listener)
 	{
@@ -323,6 +445,24 @@ COConfigurationManager
   	return ConfigurationDefaults.getInstance().getAllowedParameters();
   }
   
+  public static Set
+  getDefinedParameters()
+  {
+  	return ConfigurationManager.getInstance().getDefinedParameters();
+  }
+  
+  	/**
+  	 * raw parameter access
+  	 * @param name
+  	 * @return
+  	 */
+  public static Object
+  getParameter(
+	String	name )
+  {
+  	return ConfigurationManager.getInstance().getParameter(name);
+  }
+		 
   	/**
   	 * checks if a default is defined for the named parameter
   	 * @param parameter
@@ -363,6 +503,10 @@ COConfigurationManager
   	ConfigurationDefaults.getInstance().addParameter( parameter, _default );
   }
   
+  public static void setFloatDefault(String parameter, float _default) {
+	  ConfigurationDefaults.getInstance().addParameter( parameter, _default );
+  }
+  
   public static void
   setIntDefault(
   	String	parameter,
@@ -370,6 +514,14 @@ COConfigurationManager
   {
   	ConfigurationDefaults.getInstance().addParameter( parameter, _default );
   }
+
+  public static void
+  setLongDefault(
+  	String	parameter,
+	long	_default )
+  {
+  	ConfigurationDefaults.getInstance().addParameter( parameter, _default );
+  }
   
   public static void
   setStringDefault(
@@ -396,5 +548,18 @@ COConfigurationManager
   
   public static boolean removeParameter(String parameter) {
 		return ConfigurationManager.getInstance().removeParameter(parameter);
-	}
+  }
+  
+  public static boolean removeRGBParameter(String parameter) {
+		return ConfigurationManager.getInstance().removeRGBParameter(parameter);
+  }
+  
+  public interface
+  ParameterVerifier
+  {
+	  public boolean
+	  verify(
+		String	parameter,
+		Object	value );
+  }
 }
