@@ -34,7 +34,11 @@ import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncer;
 import org.gudy.azureus2.core3.tracker.host.TRHost;
 import org.gudy.azureus2.core3.tracker.server.TRTrackerServer;
 import org.gudy.azureus2.core3.util.*;
-
+import com.aelitis.azureus.core.speedmanager.impl.SpeedManagerImpl;
+import com.aelitis.azureus.core.speedmanager.impl.v2.SpeedLimitConfidence;
+import com.aelitis.azureus.core.speedmanager.impl.v2.SpeedLimitMonitor;
+import com.aelitis.azureus.core.speedmanager.impl.v2.SpeedManagerAlgorithmProviderV2;
+import com.aelitis.azureus.core.speedmanager.impl.v2.SMConst;
 
 /**
  *
@@ -49,7 +53,7 @@ import org.gudy.azureus2.core3.util.*;
  *   GUI_ (eventually) for GUI specific things
  *   Core_ (eventually) for core specific things
  * - Second is some identifing term. It starts with a small letter denoting
- *   the vaiable type.
+ *   the variable type.
  *   b Boolean
  *   i Integer
  *   s String
@@ -70,7 +74,7 @@ public class ConfigurationDefaults {
   private static ConfigurationDefaults configdefaults;
   private static AEMonitor				class_mon	= new AEMonitor( "ConfigDef");
   
-  private HashMap def = null;
+  private Map def = null;
   
   public static final int def_int = 0;
   public static final long def_long = 0;
@@ -89,7 +93,21 @@ public class ConfigurationDefaults {
   	
 	    if(configdefaults == null){
 	    
-	      configdefaults = new ConfigurationDefaults();
+	    	try{
+	    		configdefaults = new ConfigurationDefaults();
+	    		
+	    	}catch( Throwable e ){
+	    		
+	    			// this is here for when we are just using a few of the Azureus classes and
+	    			// we can live with no defaults (e.g. swing webui). If we initialise
+	    			// the normal config-defaults fully this pulls in all sorts of unwanted
+	    			// classes (platform manager for example). Also, don't using Debug/Logging
+	    			// to record this fact!
+	    		
+	    		System.out.println( "Falling back to default defaults as environment is restricted" );
+	    		
+	    		configdefaults = new ConfigurationDefaults( new HashMap());
+	    	}
 	    }
 	    
 	    return configdefaults;
@@ -114,6 +132,7 @@ public class ConfigurationDefaults {
     
     def.put("TCP.Listen.Port", new Long( 6881 ));
     def.put("TCP.Listen.Port.Enable", TRUE );
+    def.put("TCP.Listen.Port.Override", "");
     def.put("UDP.Listen.Port", new Long( 6881 ));
     def.put("UDP.Listen.Port.Enable", TRUE );
     def.put("UDP.NonData.Listen.Port", new Long( 6881 ));	// two effective enablers for this, dht + tracker udp client
@@ -122,6 +141,8 @@ public class ConfigurationDefaults {
     def.put("HTTP.Data.Listen.Port.Override", ZERO);
     def.put("HTTP.Data.Listen.Port.Enable", FALSE );
     
+    def.put("IPV6 Prefer Addresses",FALSE);
+    	
     def.put("max active torrents", new Long(4));
     def.put("max downloads", new Long(4));
     def.put("min downloads", ONE);
@@ -142,11 +163,15 @@ public class ConfigurationDefaults {
     def.put( "enable.seedingonly.maxuploads", FALSE );
     def.put( "max.uploads.when.busy.inc.min.secs", new Long( 30 ));
     def.put( "Max Download Speed KBs", ZERO );
+    def.put( "Use Request Limiting", TRUE );
+    def.put( "Use Request Limiting Priorities", TRUE );
     def.put( "Max Upload Speed KBs", ZERO);
     def.put( "Max Upload Speed Seeding KBs", ZERO );
     def.put( "enable.seedingonly.upload.rate", FALSE );
+    def.put( "Max Seeds Per Torrent", ZERO);
     
-    def.put( "Auto Upload Speed Enabled", FALSE );
+    //def.put( "Auto Upload Speed Enabled", FALSE );
+    def.put( TransferSpeedValidator.AUTO_UPLOAD_ENABLED_CONFIGKEY, FALSE ); //"Auto Upload Speed Enabled"
     def.put( "Auto Upload Speed Seeding Enabled", FALSE );
     def.put( "AutoSpeed Available", FALSE );	// informative read-only parameter
     def.put( "AutoSpeed Min Upload KBs", ZERO );
@@ -157,6 +182,7 @@ public class ConfigurationDefaults {
     def.put( "AutoSpeed Download Adj Enable", FALSE );
     def.put( "AutoSpeed Download Adj Ratio", "1.0" );
     def.put( "AutoSpeed Latency Factor", new Long(50));
+    def.put( "AutoSpeed Forced Min KBs", new Long(4));
     def.put( "Auto Upload Speed Debug Enabled", FALSE );
     
     def.put( "ASN Autocheck Performed Time", ZERO );
@@ -180,8 +206,11 @@ public class ConfigurationDefaults {
     def.put("Ip Filter Allow",FALSE);
     def.put("Ip Filter Enable Banning", TRUE);
     def.put("Ip Filter Ban Block Limit", new Long(4));
+    def.put("Ip Filter Ban Discard Ratio", "5.0" );
+    def.put("Ip Filter Ban Discard Min KB", new Long(128));
     def.put("Ip Filter Banning Persistent", TRUE);
     def.put("Ip Filter Enable Description Cache", TRUE);
+    def.put("Ip Filter Autoload File", "");
     def.put("Allow Same IP Peers",FALSE);
     def.put("Use Super Seeding",FALSE);
     def.put("Pause Downloads On Exit", FALSE );
@@ -200,12 +229,15 @@ public class ConfigurationDefaults {
     def.put("update.start",TRUE);
     def.put("update.periodic",TRUE);
     def.put("update.opendialog",TRUE);
-    
+    def.put("update.autodownload", FALSE);
+
+    def.put("Send Version Info", TRUE);
     
     // Logging
     def.put("Logger.Enabled", FALSE);  //logging in general
     def.put("Logging Enable", FALSE);  //file logging
     def.put("Logging Dir", "");
+    def.put("Logging Timestamp", "HH:mm:ss.SSS");
     def.put("Logging Max Size", new Long(5));
     int[] logComponents = { 0, 1, 2, 4 };
     for (int i = 0; i < logComponents.length; i++)
@@ -253,6 +285,7 @@ public class ConfigurationDefaults {
     def.put( "Watch Torrent Folder Path", "" );
     def.put( "Prioritize First Piece", FALSE );
     def.put( "Prioritize Most Completed Files", FALSE );
+    def.put( "Piece Picker Request Hint Enabled", TRUE );
     def.put( "Use Lazy Bitfield", FALSE );
     def.put( "Zero New", FALSE );
     def.put( "XFS Allocation", FALSE );
@@ -264,6 +297,8 @@ public class ConfigurationDefaults {
     def.put( "Popup File Finished", FALSE);
     def.put( "Popup Download Added", FALSE);
     def.put( "Show Timestamp For Alerts", FALSE);
+    def.put( "Use Message Box For Popups", FALSE);
+    def.put( "Suppress Alerts", FALSE);
     
     //default torrent directory option
     def.put( "Save Torrent Files", TRUE );
@@ -271,7 +306,9 @@ public class ConfigurationDefaults {
 
   
     def.put( "Bind IP", "" );
+    def.put( "Enforce Bind IP", FALSE);
     def.put( "Stats Export Peer Details", FALSE );
+    def.put( "Stats Export File Details", FALSE );
     def.put( "Stats XSL File", "" );
     def.put( "Stats Enable", FALSE );
     def.put( "Stats Period", new Long(StatsWriterPeriodic.DEFAULT_SLEEP_PERIOD) );
@@ -290,6 +327,9 @@ public class ConfigurationDefaults {
     def.put( "Tracker Client Read Timeout", new Long(60));
 	def.put( "Tracker Client Send OS and Java Version", TRUE);
 	def.put( "Tracker Client Show Warnings", TRUE);
+	def.put( "Tracker Client Min Announce Interval", ZERO);
+	def.put( "Tracker Client Numwant Limit", new Long(100));
+	def.put( "Tracker Client No Port Announce", FALSE);
 	
     def.put( "Tracker Public Enable", FALSE );
     def.put( "Tracker Log Enable", FALSE );
@@ -312,6 +352,7 @@ public class ConfigurationDefaults {
     def.put( "Tracker Announce Cache Min Peers", new Long( TRTrackerServer.DEFAULT_ANNOUNCE_CACHE_PEER_THRESHOLD ) );
     def.put( "Tracker Poll Interval Min", new Long( TRTrackerServer.DEFAULT_MIN_RETRY_DELAY) );
     def.put( "Tracker Poll Interval Max", new Long( TRTrackerServer.DEFAULT_MAX_RETRY_DELAY) );
+    def.put( "Tracker Poll Seed Interval Mult", new Long(1));
     def.put( "Tracker Scrape Retry Percentage", new Long( TRTrackerServer.DEFAULT_SCRAPE_RETRY_PERCENTAGE ) );
     def.put( "Tracker Password Enable Web", FALSE );
     def.put( "Tracker Password Web HTTPS Only", FALSE);
@@ -328,6 +369,7 @@ public class ConfigurationDefaults {
     def.put( "Tracker Max POST Time Multiplier", ONE);
     def.put( "Tracker Max Threads", new Long( 48 ));
     def.put( "Tracker TCP NonBlocking", FALSE);
+    def.put( "Tracker TCP NonBlocking Restrict Request Types", TRUE);
     def.put( "Tracker TCP NonBlocking Conc Max", new Long(2048));
     def.put( "Tracker Client Scrape Enable", TRUE);
     def.put( "Tracker Client Scrape Stopped Enable", TRUE);
@@ -354,6 +396,7 @@ public class ConfigurationDefaults {
     def.put( "config.style.useUnitsRateBits", FALSE );
     def.put( "config.style.separateProtDataStats", FALSE );
     def.put( "config.style.dataStatsOnly", FALSE );
+    def.put( "config.style.doNotUseGB", FALSE );
     
     def.put( "Save Torrent Backup", FALSE );
     
@@ -382,7 +425,7 @@ public class ConfigurationDefaults {
     def.put( "diskmanager.friendly.hashchecking", FALSE );
     def.put( "diskmanager.hashchecking.smallestfirst", TRUE );    
     def.put( "Default Start Torrents Stopped", FALSE);
-    def.put( "Server Enable UDP", TRUE);
+    def.put( "Server Enable UDP", TRUE); // this actually means the UDP tracker client
     def.put( "diskmanager.perf.cache.enable", TRUE);
     def.put( "diskmanager.perf.cache.enable.read", FALSE);
     def.put( "diskmanager.perf.cache.enable.write", TRUE);
@@ -401,24 +444,34 @@ public class ConfigurationDefaults {
     def.put( "diskmanager.perf.write.aggregate.request.limit", ZERO );
     def.put( "diskmanager.perf.write.aggregate.byte.limit", ZERO );
     def.put( "diskmanager.perf.checking.read.priority", FALSE );
+    def.put( "diskmanager.perf.checking.fully.async", FALSE );
     def.put( "diskmanager.perf.queue.torrent.bias", TRUE );
     
+    	// Peer control
+    
+    def.put( "peercontrol.udp.fallback.connect.fail", TRUE );
+    def.put( "peercontrol.udp.fallback.connect.drop", TRUE );
+    def.put( "peercontrol.udp.probe.enable", FALSE );
+    def.put( "peercontrol.hide.piece", FALSE );
+    def.put( "peercontrol.scheduler.use.priorities", TRUE );
+    	
     def.put( "File.truncate.if.too.large", FALSE);
     def.put( "Enable System Tray", TRUE);
     def.put( "config.style.table.defaultSortOrder", ZERO);
     def.put( "Ignore.peer.ports", "0" );
     def.put( "Security.JAR.tools.dir", "" );
-    def.put( "network.max.simultaneous.connect.attempts", new Long( 8 ));
+    def.put( "network.max.simultaneous.connect.attempts", new Long( Constants.isWindows?8:24 ));
     def.put( "network.tcp.mtu.size", new Long(1500) );
     def.put( "network.udp.mtu.size", new Long(1500) );
     def.put( "network.tcp.socket.SO_SNDBUF", ZERO );
     def.put( "network.tcp.socket.SO_RCVBUF", ZERO );
-    def.put( "network.tcp.socket.IPTOS", "" );
+    def.put( "network.tcp.socket.IPDiffServ", "" );
     def.put( "network.tcp.read.select.time", new Long(25));
     def.put( "network.tcp.write.select.time", new Long(25));
     def.put( "network.control.write.idle.time", new Long(50));
     def.put( "network.control.write.aggressive", FALSE );
     def.put( "network.control.read.idle.time", new Long(50));
+    def.put( "network.control.read.aggressive", FALSE );
     def.put( "peermanager.schedule.time", new Long(100));
     def.put( "confirm_torrent_removal", FALSE );
     def.put( "add_torrents_silently", FALSE );
@@ -430,6 +483,9 @@ public class ConfigurationDefaults {
     def.put( "Play File Finished Announcement", FALSE);
     def.put( "Play File Finished Announcement Text", "File Complete");
     
+    def.put( "filechannel.rt.buffer.millis", new Long( 60*1000 ));
+    def.put( "filechannel.rt.buffer.pieces", new Long( 5 ));
+    
     def.put( "BT Request Max Block Size", new Long(65536));
     def.put( "network.tcp.enable_safe_selector_mode", FALSE );
     def.put( "network.tcp.safe_selector_mode.chunk_size", new Long( 60 ));
@@ -439,9 +495,14 @@ public class ConfigurationDefaults {
     def.put( "network.transport.encrypted.fallback.outgoing", FALSE );
     def.put( "network.transport.encrypted.fallback.incoming", FALSE );
     def.put( "network.transport.encrypted.use.crypto.port", FALSE );
+    def.put( "network.transport.encrypted.allow.incoming", TRUE );	// should always be TRUE, here for testing only
     
     def.put( "network.bind.local.port", ZERO );
     
+    def.put( "crypto.keys.system.managed", FALSE );
+    
+    def.put( "peer.nat.traversal.request.conc.max", new Long(3));
+
     // Memory settings
     
     def.put( "memory.slice.limit.multiplier", new Long(1));
@@ -451,7 +512,7 @@ public class ConfigurationDefaults {
     def.put( "Completed Files Directory", "" );
     def.put( "Move Only When In Default Save Dir", TRUE );
     def.put( "Move Torrent When Done", TRUE );
-    def.put( "File.move.subdir_is_default", FALSE );
+    def.put( "File.move.subdir_is_default", TRUE );
     
     // This gets permanently set by GlobalManagerImpl to false once Azureus has initialised.
     // This is intended to be used once as part of an upgrade.
@@ -464,8 +525,42 @@ public class ConfigurationDefaults {
     def.put( "File.move.download.removed.move_torrent", TRUE );
     def.put( "File.move.download.removed.move_partial", FALSE );
     
-    def.put("FilesView.separate_rename_and_retarget", FALSE);
+    def.put( "File.delete.include_files_outside_save_dir", FALSE);
+    
+    def.put("FilesView.show.full.path", FALSE );
+    
+    def.put("MyTorrentsView.menu.show_parent_folder_enabled", FALSE);
+    def.put("FileBrowse.usePathFinder", FALSE);
+    	
+    //temp section for SpeedManagerAlgorithmProviderV2
+    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DOWNLOAD_MAX_LIMIT, new Long(SMConst.START_DOWNLOAD_RATE_MAX) );
+    def.put(SpeedManagerAlgorithmProviderV2.SETTING_UPLOAD_MAX_LIMIT, new Long(SMConst.START_UPLOAD_RATE_MAX) );
 
+    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_GOOD_SET_POINT, new Long(50) );
+    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_GOOD_TOLERANCE, new Long(100) );
+    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_BAD_SET_POINT, new Long(900) );
+    def.put(SpeedManagerAlgorithmProviderV2.SETTING_DHT_BAD_TOLERANCE, new Long(500) );
+
+    	//**** NOTE! This default can be overridden in ConfigurationChecker 
+    def.put(SpeedManagerImpl.CONFIG_VERSION, new Long(2) );	// 1 == classic, 2 == beta
+    
+    def.put( SpeedLimitMonitor.DOWNLOAD_CONF_LIMIT_SETTING, SpeedLimitConfidence.NONE.getString() );
+    def.put( SpeedLimitMonitor.UPLOAD_CONF_LIMIT_SETTING, SpeedLimitConfidence.NONE.getString() );
+    def.put( SpeedLimitMonitor.UPLOAD_CHOKE_PING_COUNT, new Long(1) );
+
+    //default V2 algorithm seeding and download mode usage, stored as an Int
+    def.put( SpeedLimitMonitor.USED_UPLOAD_CAPACITY_SEEDING_MODE, new Long(90) );  
+    def.put( SpeedLimitMonitor.USED_UPLOAD_CAPACITY_DOWNLOAD_MODE, new Long(60) );
+
+    def.put( SpeedManagerAlgorithmProviderV2.SETTING_WAIT_AFTER_ADJUST, TRUE );
+    def.put( SpeedManagerAlgorithmProviderV2.SETTING_INTERVALS_BETWEEN_ADJUST, new Long(2) );
+}
+  
+  protected 
+  ConfigurationDefaults(
+	Map	_def )
+  {
+	  def = _def;
   }
   
   protected void
@@ -525,6 +620,11 @@ public class ConfigurationDefaults {
 	  return (byte[])def.get(p);
   }
   
+  public boolean getBooleanParameter(String p) throws ConfigurationParameterNotFoundException {
+  	checkParameterExists(p);
+  	return ((Long)def.get(p)).equals(TRUE);
+  }
+  
   public boolean hasParameter(String p) {
 	  return def.containsKey(p);
   }
@@ -559,6 +659,10 @@ public class ConfigurationDefaults {
     def.put(sKey, lParameter);
   }
   
+  public void addParameter(String sKey, long lParameter) {
+	    def.put(sKey, new Long(lParameter));
+  }
+  
   public void addParameter(String sKey, float fParameter) {
 	  def.put(sKey, new Float(fParameter));
   }
@@ -648,11 +752,15 @@ public class ConfigurationDefaults {
 		  
 		  	// see http://www.faqs.org/rfcs/rfc3696.html
 		  
+		  	// we need to include "~" here for I2P generated 'URLs'
+		  
 		  for (int i=0;i<value.length();i++){
 			  
 			  char	c = value.charAt(i);
 			  
-			  if ( Character.isLetterOrDigit( c ) || c == '.' || c == '-' ){
+			  	// : for IPv6
+			  
+			  if ( Character.isLetterOrDigit( c ) || c == '.' || c == '-' || c == ':' || c == '~' ){
 				  
 			  }else{
 				  
