@@ -27,6 +27,7 @@ import java.io.File;
 import org.gudy.azureus2.core3.disk.impl.piecemapper.DMPieceList;
 import org.gudy.azureus2.core3.torrent.TOTorrent;
 import org.gudy.azureus2.core3.util.DirectByteBuffer;
+import org.gudy.azureus2.core3.util.IndentWriter;
 
 /**
 * @author MjrTom
@@ -114,6 +115,10 @@ DiskManager
 		DiskManagerReadRequest 			request, 
 		DiskManagerReadRequestListener 	listener );
 
+	public boolean
+	hasOutstandingReadRequestForPiece(
+		int		piece_number );
+	
 		/**
 		 * Create a request to check a particular piece
 		 * @param pieceNumber	-1 for a complete recheck request
@@ -138,6 +143,10 @@ DiskManager
 		DiskManagerCheckRequest			request,
 		DiskManagerCheckRequestListener	listener );
 	
+	public boolean
+	hasOutstandingCheckRequestForPiece(
+		int		piece_number );
+	
 		/**
 		 * recheck the entire torrent asynchronously, reporting each piece to the listener
 		 * @param listener
@@ -150,6 +159,10 @@ DiskManager
 		DiskManagerCheckRequestListener	listener );
 	
 	public void
+	setPieceCheckingEnabled(
+		boolean		enabled );
+			
+	public void
     saveResumeData(
     	boolean interim_save )
 		
@@ -163,6 +176,7 @@ DiskManager
 	getNbPieces();
 
 	public DiskManagerFileInfo[] getFiles();
+	public DiskManagerFileInfoSet getFileSet();
 	public DiskManagerPiece getPiece(int PieceNumber);
 
 	public DMPieceList getPieceList(int pieceNumber);
@@ -176,9 +190,10 @@ DiskManager
 	public int
 	getPieceLength();
 	
-	public int 
-	getLastPieceLength();
-
+	public int
+	getPieceLength(
+		int	piece_number );
+	
 	public long
 	getRemaining();
 	
@@ -196,10 +211,8 @@ DiskManager
 
     public void
     downloadRemoved();
-
-	public void
-	moveDataFiles(
-		File	new_parent_dir );
+	
+	public void moveDataFiles(File new_parent_dir, String dl_name);
 	
 		/**
 		 * returns -1 if no recheck in progress, percentage complete in 1000 notation otherwise
@@ -218,7 +231,8 @@ DiskManager
 		 */
 	
 	public boolean 
-	checkBlockConsistency(
+	checkBlockConsistencyForWrite(
+		String				originator,
 		int 				pieceNumber, 
 		int 				offset, 
 		DirectByteBuffer 	data );
@@ -232,13 +246,19 @@ DiskManager
 		 */
 	
 	public boolean 
-	checkBlockConsistency(
+	checkBlockConsistencyForRead(
 		String	originator,
 		int 	pieceNumber, 
 		int	 	offset, 
 		int 	length );
 		
-   
+	public boolean 
+	checkBlockConsistencyForHint(
+		String	originator,
+		int 	pieceNumber, 
+		int	 	offset, 
+		int 	length );
+	
 	public TOTorrent
 	getTorrent();
 	
@@ -267,15 +287,12 @@ DiskManager
 	 * @return true if the pieceNumber is Needed and not Done
 	 */
 	public boolean isInteresting(int pieceNumber);
+	
 	public boolean isDone(int pieceNumber);
-/*
-    public boolean isRequestable(int pieceNumber);
-	public boolean calcNeeded(int pieceNumber);
-	public void clearNeeded(int pieceNumber);
-	public long getLastWriteTime(int pieceNumber);
-	public long getNbBlocks(int pieceNumber);
-	public long getNbWritten(int pieceNumber);
-	public boolean isRequested(int pieceNumber);
-	public void setRequested(int pieceNumber);
-*/
+
+	public int getCacheMode();
+	
+	public void
+	generateEvidence(
+		IndentWriter		writer );
 }
\ No newline at end of file
