@@ -22,7 +22,6 @@ package org.gudy.azureus2.core3.disk;
 import java.io.File;
 import java.io.IOException;
 
-import org.gudy.azureus2.core3.disk.DiskManager;
 import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.torrent.TOTorrentFile;
 import org.gudy.azureus2.core3.util.DirectByteBuffer;
@@ -46,11 +45,20 @@ DiskManagerFileInfo
 	
 	public void setSkipped(boolean b);
 	 
-	
+	/**
+	 * Relink the file to the destination given - this method deals with if the file
+	 * is part of a simple torrent or not (so it may set the download name to keep it
+	 * in sync). If you just want a simple relink, use setLinkAtomic.
+	 * 
+	 * @param link_destination
+	 * @return
+	 */
 	public boolean
 	setLink(
 		File	link_destination );
 	
+	public boolean setLinkAtomic(File link_destination);
+	
 		// gets the current link, null if none
 	
 	public File
@@ -61,9 +69,7 @@ DiskManagerFileInfo
 		 * @param type	one of ST_LINEAR or ST_COMPACT
 		 */
 	
-	public boolean
-	setStorageType(
-		int		type );
+	public boolean setStorageType(int type );
 	
 	public int
 	getStorageType();
