@@ -165,7 +165,7 @@ DMWriterImpl
 	public boolean 
 	zeroFile( 
 		DiskManagerFileInfoImpl file, 
-		long 					length ) 
+		long 					length ) throws DiskManagerException 
 	{
 		CacheFile	cache_file = file.getCacheFile();
 		
@@ -246,6 +246,11 @@ DMWriterImpl
 									{
 										return( -1 );
 									}
+									
+									public void 
+									requestExecuted(long bytes) 
+									{							
+									}
 								});
            
 						sem.reserve();
@@ -280,7 +285,7 @@ DMWriterImpl
 			
 			Debug.printStackTrace( e );
 			
-			return( false );
+			throw new DiskManagerException(e);			
 		}
 			
 		return true;
@@ -638,6 +643,11 @@ DMWriterImpl
 									{
 										return( -1 );
 									}
+									
+									public void 
+									requestExecuted(long bytes) 
+									{					
+									}
 								});
 							
 							sem.reserve();
@@ -717,6 +727,11 @@ DMWriterImpl
 					{
 						return( -1 );
 					}
+					
+					public void 
+					requestExecuted(long bytes) 
+					{							
+					}
 				};
 				
 			disk_access.queueWriteRequest(
@@ -758,6 +773,11 @@ DMWriterImpl
 			return( -1 );
 		}
 		
+		public void 
+		requestExecuted(long bytes) 
+		{							
+		}
+		
 		protected void
 		failed(
 			Throwable			cause )
