@@ -23,15 +23,22 @@ package org.gudy.azureus2.core3.download;
 
 import java.io.File;
 
-import org.gudy.azureus2.core3.global.*;
-import org.gudy.azureus2.core3.disk.*;
-import org.gudy.azureus2.core3.peer.*;
-import org.gudy.azureus2.core3.torrent.*;
-import org.gudy.azureus2.core3.tracker.client.*;
+import org.gudy.azureus2.core3.disk.DiskManager;
+import org.gudy.azureus2.core3.disk.DiskManagerFileInfo;
+import org.gudy.azureus2.core3.disk.DiskManagerFileInfoSet;
+import org.gudy.azureus2.core3.global.GlobalManager;
+import org.gudy.azureus2.core3.peer.PEPeer;
+import org.gudy.azureus2.core3.peer.PEPeerManager;
+import org.gudy.azureus2.core3.peer.PEPiece;
+import org.gudy.azureus2.core3.torrent.TOTorrent;
+import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncer;
+import org.gudy.azureus2.core3.tracker.client.TRTrackerScraperResponse;
 import org.gudy.azureus2.core3.util.IndentWriter;
 import org.gudy.azureus2.plugins.download.DownloadAnnounceResult;
 import org.gudy.azureus2.plugins.download.DownloadScrapeResult;
 
+import com.aelitis.azureus.core.networkmanager.LimitedRateGroup;
+
 public interface
 DownloadManager
 {
@@ -100,6 +107,13 @@ DownloadManager
 
     public void forceRecheck();
 
+		/**
+		 * @param l
+		 *
+		 * @since 3.0.0.7
+		 */
+		void forceRecheck(ForceRecheckListener l);
+
         /**
          * Reset the file download state to totally undownloaded. Download must be stopped
          * @param file
@@ -117,6 +131,16 @@ DownloadManager
     public void
     recheckFile(
         DiskManagerFileInfo     file );
+    	
+    	/**
+    	 * Use with care - introduced to support speed tests whereby we want to avoid checking the
+    	 * virtual torrent used for the test
+    	 * @param enabled
+    	 */
+    
+    public void
+    setPieceCheckingEnabled(
+    	boolean	enabled );
 
   /**
    * Stop the download manager, and do any file/torrent removals.
@@ -148,6 +172,8 @@ DownloadManager
 
     public DiskManagerFileInfo[]
     getDiskManagerFileInfo();
+    
+    public DiskManagerFileInfoSet getDiskManagerFileInfoSet();
 
     public PEPeerManager
     getPeerManager();
@@ -190,12 +216,20 @@ DownloadManager
     public long
     getSize();
 
+    /**
+     * This includes the full path to the torrent file.
+     * @return
+     */
     public String
     getTorrentFileName();
 
     public void
     setTorrentFileName(String string);
 
+    /**
+     * Returns the file location where we save the torrent, this includes the parent
+     * directory and the actual file name.
+     */
     public File
     getAbsoluteSaveLocation();
 
@@ -211,6 +245,12 @@ DownloadManager
     setTorrentSaveDir(
         String sPath );
 
+    /**
+     * changes the save directory. Only call this if you know what you are doing!!!!
+     * @param sPath
+     */
+    public void setTorrentSaveDir(String parent_dir, String dl_name);
+
     public boolean isForceStart();
 
     public void setForceStart(boolean forceStart);
@@ -338,11 +378,20 @@ DownloadManager
     public void
     saveDownload();
 
-      /** To retreive arbitrary objects against this object. */
+      /** 
+       * To retreive arbitrary objects against this object. 
+       * @deprecated Use getUserData */
+    
     public Object getData (String key);
-      /** To store arbitrary objects against this object. */
+      /** To store arbitrary objects against this object.
+       *  @deprecated @deprecated Use setUserData
+       *  */
+    
     public void setData (String key, Object value);
 
+    public Object getUserData (Object key);
+    /** To store arbitrary objects against this object. */
+ 	public void setUserData (Object key, Object value);
 
       /**
        * Determine whether disk allocation has already been done.
@@ -386,6 +435,13 @@ DownloadManager
     public int
     getEffectiveUploadRateLimitBytesPerSecond();
 
+    public void
+    setCryptoLevel(
+    	int	level );
+    
+    public int
+    getCryptoLevel();
+    
         /**
          * Move data files to new location. Torrent must be in stopped/error state
          * @param new_parent_dir
@@ -408,6 +464,16 @@ DownloadManager
      * @throws DownloadManagerException
      */
     public void renameDownload(String new_name) throws DownloadManagerException;
+    
+    /**
+     * Move the files and rename a download in one go.
+     * 
+     * For convenience - either argument can be null, but not both.
+     * 
+     * @see #moveDataFiles(File)
+     * @see #renameDownload(String)
+     */
+    public void moveDataFiles(File new_parent_dir, String new_name) throws DownloadManagerException;
 
         /**
          * Move torrent file to new location. Download must be stopped/error
@@ -449,14 +515,12 @@ DownloadManager
      * location of the torrent file or download.
      * 
      * @since 2.5.0.2
-     * @param for_moving Indicates whether you want this information for the purposes
-     *     of moving the download or not.
      * @author amc1
      * @return An array of type <tt>File</tt> of size 2, first element containing the
      *     calculated location for the download's data files, and the second element
      *     containing the location for the download's torrent file.
      */
-    public File[] calculateDefaultPaths(boolean for_moving);
+    public File[] calculateDefaultPaths();
     
     /**
      * Returns <tt>true</tt> if the download is being saved to one of the default
@@ -487,14 +551,16 @@ DownloadManager
         DownloadScrapeResult    result );
 
   /**
-   * Is advanced AZ messaging enabled for this download.
+   * Is extended messaging enabled for this download (meaning AzMP and LTEP support).
    * @return true if enabled, false if disabled
    */
-    public boolean isAZMessagingEnabled();
+    public boolean isExtendedMessagingEnabled();
 
   /**
-   * Enable or disable advanced AZ messaging for this download.
+   * Enable or disable extended messaging messaging for this download.
    * @param enable true to enabled, false to disabled
+   * 
+   * @deprecated Want to get rid of this if possible.
    */
     public void setAZMessagingEnabled( boolean enable );
 
@@ -507,6 +573,9 @@ DownloadManager
     destroy(
     	boolean	is_duplicate );
 
+    public boolean
+    isDestroyed();
+    
     public PEPiece[]
     getCurrentPieces();
 
@@ -521,6 +590,16 @@ DownloadManager
 	public boolean
 	seedPieceRecheck();
 	
+	public void
+	addRateLimiter(
+		LimitedRateGroup	group,
+		boolean				upload );
+	
+	public void
+	removeRateLimiter(
+		LimitedRateGroup	group,
+		boolean				upload );
+	
     public void
     addListener(
             DownloadManagerListener listener );
@@ -529,6 +608,8 @@ DownloadManager
     removeListener(
             DownloadManagerListener listener );
 
+    	// tracker listeners
+    
     public void
     addTrackerListener(
         DownloadManagerTrackerListener  listener );
@@ -537,6 +618,8 @@ DownloadManager
     removeTrackerListener(
         DownloadManagerTrackerListener  listener );
 
+    	// peer listeners
+    
     public void
     addPeerListener(
         DownloadManagerPeerListener listener );
@@ -550,6 +633,23 @@ DownloadManager
     removePeerListener(
         DownloadManagerPeerListener listener );
 
+		// piece listeners
+
+    public void
+    addPieceListener(
+    		DownloadManagerPieceListener listener );
+
+    public void
+    addPieceListener(
+        DownloadManagerPieceListener listener,
+        boolean bDispatchForExisting );
+
+    public void
+    removePieceListener(
+    		DownloadManagerPieceListener listener );
+
+    	// disk listeners
+    
     public void
     addDiskListener(
         DownloadManagerDiskListener listener );
@@ -573,4 +673,36 @@ DownloadManager
     public void
     generateEvidence(
         IndentWriter        writer );
+    
+    public int[] getStorageType(DiskManagerFileInfo[] infos);
+    
+    /**
+     * @since 3.0.5.1
+     */
+    public boolean canMoveDataFiles();
+    
+    /**
+     * Renames the save file, torrent file and the displayed name.
+     */
+    public void rename(String new_name) throws DownloadManagerException;
+
+    /**
+     * @since 3.0.5.1
+     */
+    public void renameTorrent(String new_name) throws DownloadManagerException;
+
+    /**
+     * @since 3.0.5.1
+     */
+    public void moveTorrentFile(File parent_dir, String new_name) throws DownloadManagerException;
+    
+    /**
+     * Sets the torrent file name (will perform a move of the torrent file) - 
+     * this shouldn't be used unless you know what you are doing (it requires
+     * the download to be stopped!) - it is safer to use moveTorrentFile instead.
+     * 
+     * @since 3.0.5.3
+     */
+    public void setTorrentFile(File new_parent_dir, String new_name) throws DownloadManagerException;
+
 }
\ No newline at end of file
