@@ -30,6 +30,8 @@ import org.gudy.azureus2.core3.tracker.client.*;
 import org.gudy.azureus2.core3.util.HashWrapper;
 import org.gudy.azureus2.core3.download.*;
 
+import org.gudy.azureus2.plugins.dht.mainline.MainlineDHTProvider;
+
 /**
  * The GlobalManager contains a list of all the downloads
  * (DownloadManager objects) that Azureus controls. 
@@ -62,6 +64,28 @@ public interface GlobalManager extends AzureusCoreComponent {
 	public DownloadManager addDownloadManager(String fileName, byte[]	optionalHash,
 			String savePath, int initialState, boolean persistent);
 
+	
+	/**
+	 * Create and add a Download Manager to the global list
+	 * 
+	 * @param fileName location and name of torrent file
+	 * @param savePath path to write the data to
+	 * @param safeFile subdirectory or filename to write the data to
+	 * @param initialState Initial state of download. See DownloadManager.STATE_*
+	 * @param persistent Whether the download should be treated as persistent download
+	 * @param for_seeding Whether the manager should assume the torrent is 
+	 *                     already complete and ready for seeding. 
+	 * 
+	 * @return The Downloadmanger based on the supplied information.<br>
+	 *          May return an existing DownloadManager if torrent was already
+	 *          in GlobalManager. 
+	 */
+	public DownloadManager addDownloadManager(String fileName,
+			byte[] optionalHash, String savePath, String saveFile,
+			int initialState, boolean persistent, boolean for_seeding, 
+			DownloadManagerInitialisationAdapter adapter );
+	
+	
 	/**
 	 * Create and add a Download Manager to the global list
 	 * 
@@ -101,7 +125,8 @@ public interface GlobalManager extends AzureusCoreComponent {
 	 * @param dm DownloadManager to check
 	 * @throws GlobalManagerDownloadRemovalVetoException
 	 */
-	public void canDownloadManagerBeRemoved(DownloadManager dm)
+	public void canDownloadManagerBeRemoved(DownloadManager dm,
+			boolean remove_torrent, boolean remove_data)
 			throws GlobalManagerDownloadRemovalVetoException;
 
 	/**
@@ -209,8 +234,16 @@ public interface GlobalManager extends AzureusCoreComponent {
 	 * 
 	 * @return  Seeding Only State
 	 */
+	
 	public boolean isSeedingOnly();
 
+	/**
+	 * As for isSeedingOnly but includes queued seeds
+	 * @return
+	 */
+	
+	public boolean isPotentiallySeedingOnly();
+
 	/**
 	 * Retrieve the index of a DownloadManager within the GlobalManager
 	 * list retrieved via {@link #getDownloadManagers()}.
@@ -347,4 +380,30 @@ public interface GlobalManager extends AzureusCoreComponent {
 	 * @param trigger
 	 */
 	void addListener(GlobalManagerListener listener, boolean trigger);
+
+	/**
+	 * @param manager
+	 * @param remove_torrent
+	 * @param remove_data
+	 * @throws GlobalManagerDownloadRemovalVetoException
+	 *
+	 * @since 3.0.1.7
+	 */
+	void removeDownloadManager(DownloadManager manager, boolean remove_torrent,
+			boolean remove_data) throws GlobalManagerDownloadRemovalVetoException;
+	
+	/**
+	 * Calling this method doesn't <i>prepare</i> Azureus to be DHT-ready, it is
+	 * only used to store, or remove, a DHT provider - so that it can be globally
+	 * accessible. See the DHT manager classes in the plugin API for a better way
+	 * to register a provider.
+	 * 
+	 * @since 3.0.4.3
+	 */
+	void setMainlineDHTProvider(MainlineDHTProvider provider);
+
+	/**
+	 * @since 3.0.4.3
+	 */
+	MainlineDHTProvider getMainlineDHTProvider();
 }
\ No newline at end of file
