@@ -29,7 +29,6 @@ package org.gudy.azureus2.core3.global.impl;
 import java.io.File;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
-import java.net.NetworkInterface;
 import java.net.URL;
 import java.util.*;
 
@@ -53,9 +52,11 @@ import org.gudy.azureus2.core3.util.*;
 
 import com.aelitis.azureus.core.AzureusCore;
 import com.aelitis.azureus.core.helpers.TorrentFolderWatcher;
+import com.aelitis.azureus.core.peermanager.control.PeerControlSchedulerFactory;
 import com.aelitis.azureus.core.util.CopyOnWriteList;
 
 import org.gudy.azureus2.plugins.network.ConnectionManager;
+import org.gudy.azureus2.plugins.dht.mainline.MainlineDHTProvider;
 
 
 /**
@@ -109,7 +110,9 @@ public class GlobalManagerImpl
                     
 				}else if ( type == LDT_SEEDING_ONLY ){
                     
-                    target.seedingStatusChanged( ((Boolean)value).booleanValue() );
+					boolean	[] temp = (boolean[])value;
+					
+                    target.seedingStatusChanged( temp[0], temp[1] );
                 }
 			}
 		});
@@ -133,7 +136,11 @@ public class GlobalManagerImpl
 				{					
 					GlobalManagerDownloadWillBeRemovedListener	target = (GlobalManagerDownloadWillBeRemovedListener)_listener;
 					
-					target.downloadWillBeRemoved((DownloadManager)value);
+					DownloadManager dm = (DownloadManager) ((Object[])value)[0];
+					boolean remove_torrent = ((Boolean) ((Object[])value)[1]).booleanValue();
+					boolean remove_data = ((Boolean) ((Object[])value)[2]).booleanValue();
+					
+					target.downloadWillBeRemoved(dm, remove_torrent, remove_data);
 				}
 			});
 	
@@ -171,18 +178,17 @@ public class GlobalManagerImpl
 	private volatile boolean	destroyed;
 	private volatile boolean 	needsSaving = false;
   
-	private boolean seeding_only_mode = false;
+	private boolean seeding_only_mode 				= false;
+	private boolean potentially_seeding_only_mode	= false;
+	
 	private FrequencyLimitedDispatcher	check_seeding_only_state_dispatcher = 
 		new FrequencyLimitedDispatcher(
 			new AERunnable(){ public void runSupport(){ checkSeedingOnlyStateSupport(); }}, 5000 );
 	
-	
+	private boolean	force_start_non_seed_exists;
 	private int 	nat_status				= ConnectionManager.NAT_UNKNOWN;
 	private boolean	nat_status_probably_ok;
-	
-	private Set		old_network_interfaces;
-	private long	last_network_change;
-	
+		
    private CopyOnWriteList	dm_adapters = new CopyOnWriteList();
 
    /** delay loading of torrents */
@@ -192,14 +198,17 @@ public class GlobalManagerImpl
    /** Monitor to block adding torrents while loading existing torrent list */
    AESemaphore loadingSem = new AESemaphore("Loading Torrents");
 
+   AEMonitor addingDM_monitor = new AEMonitor("addingDM");
+   /** List of torrents being added, but not added to the GM list yet */ 
+   List addingDMs = new ArrayList();
 	
-	
+   private MainlineDHTProvider provider = null;
+   
    public class Checker extends AEThread {
     int loopFactor;
     private static final int waitTime = 10*1000;
     // 5 minutes save resume data interval (default)
     private int saveResumeLoopCount = 5*60*1000 / waitTime;
-    private int netCheckLoopCount	= 60*1000 / waitTime;
     private int natCheckLoopCount	= 30*1000 / waitTime;
     private int seedPieceCheckCount	= 30*1000 / waitTime;
            
@@ -233,12 +242,7 @@ public class GlobalManagerImpl
 	          	
 	        	saveDownloads( true );
 	        }
-	        
-	        if ((loopFactor % netCheckLoopCount == 0)) {
-          	
-	        	checkNetwork();
-	        }
-	        
+	        	        
 	        if ((loopFactor % natCheckLoopCount == 0)) {
 	          	
 	        	computeNATStatus();
@@ -248,6 +252,10 @@ public class GlobalManagerImpl
 	        		// Doing this via listeners is too much effort
 	        	
 		        checkSeedingOnlyState();
+		        
+		        	// double check consistency
+		        
+		        checkForceStart( false );
 	        }
 	        	
 	        if ((loopFactor % seedPieceCheckCount == 0)) {
@@ -267,14 +275,6 @@ public class GlobalManagerImpl
 		            /*
 		             * seeding rules have been moved to StartStopRulesDefaultPlugin
 		             */
-	              
-	         		// Handle forced starts here
-         	
-	        	if (	manager.getState() == DownloadManager.STATE_READY &&
-	        			manager.isForceStart()) {
-            	
-	        		manager.startDownload();
-	           }
 	        }        
 
       	}catch( Throwable e ){
@@ -502,10 +502,16 @@ public class GlobalManagerImpl
     		public void
     		announceDetailsChanged()
     		{	
-				Logger.log(new LogEvent(LOGID,
-						"Announce details have changed, updating trackers"));
+				Logger.log( new LogEvent(LOGID, "Announce details have changed, updating trackers" ));
 
-    			trackerAnnounceAll();
+				List	managers = managers_cow;
+				
+				for (int i=0;i<managers.size();i++){
+					
+					DownloadManager	manager = (DownloadManager)managers.get(i);
+					
+					manager.requestTrackerAnnounce( true );
+				}
     		}
     	});
   }
@@ -551,18 +557,33 @@ public class GlobalManagerImpl
 	 	return addDownloadManager(fileName, optionalHash, savePath, initialState,
 				persistent, false, null);
 	}
+
+	  public DownloadManager 
+	  addDownloadManager(
+	  		String torrent_file_name, 
+	  		byte[] optionalHash,
+			String savePath,
+			int initialState, 
+			boolean persistent, 
+			boolean for_seeding,
+			DownloadManagerInitialisationAdapter _adapter )
+	  {
+		  return addDownloadManager(torrent_file_name, optionalHash, savePath, null, initialState, persistent, for_seeding, _adapter);
+	  }
+	
 	
   /**
-   * @return true, if the download was added
-   *
-   * @author Rene Leonhardt
-   */
+	 * @return true, if the download was added
+	 * 
+	 * @author Rene Leonhardt
+	 */
 	
   public DownloadManager 
   addDownloadManager(
   		String torrent_file_name, 
   		byte[] optionalHash,
-		String savePath, 
+		String savePath,
+		String saveFile,
 		int initialState, 
 		boolean persistent, 
 		boolean for_seeding,
@@ -573,7 +594,7 @@ public class GlobalManagerImpl
 
 		// wait for "load existing" to complete
 		loadingSem.reserve(60 * 1000);
-
+		
 		DownloadManagerInitialisationAdapter adapter = getDMAdapter(_adapter);
 
 		/* to recover the initial state for non-persistent downloads the simplest way is to do it here
@@ -615,6 +636,9 @@ public class GlobalManagerImpl
 
 		File torrentDir = null;
 		File fDest = null;
+		HashWrapper hash = null;
+		boolean deleteDest = false;
+		boolean removeFromAddingDM = false;
 
 		try {
 			File f = new File(torrent_file_name);
@@ -632,11 +656,54 @@ public class GlobalManagerImpl
 			fDest = TorrentUtils.copyTorrentFileToSaveDir(f, persistent);
 
 			String fName = fDest.getCanonicalPath();
+			
+			try {
+				// Check if we already have the torrent loaded or loading
+
+				if (optionalHash != null) {
+					hash = new HashWrapper(optionalHash);
+				} else {
+					// This does not trigger locale decoding :)
+					TOTorrent torrent = TorrentUtils.readFromFile(fDest, false);
+					hash = torrent.getHashWrapper();
+				}
+				
+				if (hash != null) {
+					removeFromAddingDM = true;
+
+					// loaded check
+					DownloadManager existingDM = getDownloadManager(hash);
+					if (existingDM != null) {
+						deleteDest = true;
+						return existingDM;
+					}
+
+  				try {
+  					// loading check
+  					addingDM_monitor.enter();
+  					
+  					if (addingDMs.contains(hash)) {
+  						removeFromAddingDM = false;
+  						deleteDest = true;
+  						return null;
+  					}
+
+  					addingDMs.add(hash);
+  				} finally {
+  					addingDM_monitor.exit();
+  				}
+				}
+
+				
+			} catch (Exception e) {
+				// ignore any error.. let it bork later in case old code relies
+				// on it borking later
+			}
 
 			// now do the creation!
 
 			DownloadManager new_manager = DownloadManagerFactory.create(this,
-					optionalHash, fName, savePath, initialState, persistent, for_seeding,
+					optionalHash, fName, savePath, saveFile, initialState, persistent, for_seeding,
 					file_priorities, adapter);
 
 			manager = addDownloadManager(new_manager, true, true);
@@ -645,25 +712,43 @@ public class GlobalManagerImpl
 			// this torrent exists and the new one isn't needed (yuck)
 
 			if (manager == null || manager != new_manager) {
-				fDest.delete();
-				File backupFile = new File(fName + ".bak");
-				if (backupFile.exists())
-					backupFile.delete();
+				deleteDest = true;
 			}
 		} catch (IOException e) {
 			System.out.println("DownloadManager::addDownloadManager: fails - td = "
 					+ torrentDir + ", fd = " + fDest);
 			Debug.printStackTrace(e);
 			manager = DownloadManagerFactory.create(this, optionalHash,
-					torrent_file_name, savePath, initialState, persistent, for_seeding,
+					torrent_file_name, savePath, saveFile, initialState, persistent, for_seeding,
 					file_priorities, adapter);
 			manager = addDownloadManager(manager, true, true);
 		} catch (Exception e) {
 			// get here on duplicate files, no need to treat as error
 			manager = DownloadManagerFactory.create(this, optionalHash,
-					torrent_file_name, savePath, initialState, persistent, for_seeding,
+					torrent_file_name, savePath, saveFile, initialState, persistent, for_seeding,
 					file_priorities, adapter);
 			manager = addDownloadManager(manager, true, true);
+		} finally {
+			if (deleteDest) {
+  			fDest.delete();
+  			File backupFile;
+				try {
+					backupFile = new File(fDest.getCanonicalPath() + ".bak");
+	  			if (backupFile.exists())
+	  				backupFile.delete();
+				} catch (IOException e) {
+				}
+			}
+
+			if (removeFromAddingDM && hash != null) {
+  			try {
+  				addingDM_monitor.enter();
+  				
+  				addingDMs.remove(hash);
+  			} finally {
+  				addingDM_monitor.exit();
+  			}
+			}
 		}
 
 		if (needsFixup && manager != null) {
@@ -972,6 +1057,9 @@ public class GlobalManagerImpl
   }
     
   public DownloadManager getDownloadManager(TOTorrent torrent) {
+  	if (torrent == null) {
+  		return null;
+  	}
     try {
       return getDownloadManager(torrent.getHashWrapper());
     } catch (TOTorrentException e) {
@@ -987,22 +1075,43 @@ public class GlobalManagerImpl
   
   public void 
   canDownloadManagerBeRemoved(
-  	DownloadManager manager) 
+  	DownloadManager manager,
+  	boolean remove_torrent, boolean remove_data) 
   
   	throws GlobalManagerDownloadRemovalVetoException
   {
   	try{
-  		removal_listeners.dispatchWithException( LDT_MANAGER_WBR, manager );
+  		removal_listeners.dispatchWithException(LDT_MANAGER_WBR, new Object[] {
+				manager,
+				new Boolean(remove_torrent),
+				new Boolean(remove_data)
+			});
   		
   	}catch( Throwable e ){
-  		
-  		throw((GlobalManagerDownloadRemovalVetoException)e);
+  		if (e instanceof GlobalManagerDownloadRemovalVetoException) {
+    		throw((GlobalManagerDownloadRemovalVetoException)e);
+  		}
+			GlobalManagerDownloadRemovalVetoException gmv = new GlobalManagerDownloadRemovalVetoException("Error running veto check");
+			gmv.initCause(e);
+			throw gmv;
   	}
   }
   
+
   public void 
   removeDownloadManager(
-  	DownloadManager manager) 
+  	DownloadManager manager)
+  
+  	throws GlobalManagerDownloadRemovalVetoException
+  {
+  	removeDownloadManager(manager, false, false);
+  }
+
+  public void 
+  removeDownloadManager(
+  	DownloadManager manager,
+		boolean	remove_torrent,
+		boolean	remove_data )
   
   	throws GlobalManagerDownloadRemovalVetoException
   {
@@ -1013,7 +1122,9 @@ public class GlobalManagerImpl
 		  return;
 	  }
 	  
-  	canDownloadManagerBeRemoved( manager );
+  	canDownloadManagerBeRemoved( manager, remove_torrent, remove_data );
+  	
+  	manager.stopIt(DownloadManager.STATE_STOPPED, remove_torrent, remove_data);
   	
     try{
     	managers_mon.enter();
@@ -1128,6 +1239,12 @@ public class GlobalManagerImpl
     				{	
     					return( null );
     				}
+    				
+    				public String
+    				getName()
+    				{
+    					return( "Stopping global manager" );
+    				}
     			});
 	}catch( Throwable e ){
 		Debug.printStackTrace( e );
@@ -1190,8 +1307,8 @@ public class GlobalManagerImpl
       
       long	now = SystemTime.getCurrentTime();
       
-	  if(progress_listener != null &&  SystemTime.getCurrentTime() - lastListenerUpdate > 100) {
-		  lastListenerUpdate = SystemTime.getCurrentTime();
+	  if(progress_listener != null &&  now - lastListenerUpdate > 100) {
+		  lastListenerUpdate = now;
 
 		  int	currentDownload = i+1;
 		  
@@ -1540,11 +1657,19 @@ public class GlobalManagerImpl
 					  if(progress_listener != null &&  SystemTime.getCurrentTime() - lastListenerUpdate > 100) {
 						  lastListenerUpdate = SystemTime.getCurrentTime();
 	
+						  String shortFileName = fileName;
+						  try {
+							  File f = new File(fileName);
+							  shortFileName = f.getName();
+						  } catch (Exception e) {
+							// TODO: handle exception
+						}
+						  
 						  progress_listener.reportPercent(100 * currentDownload / nbDownloads);
 						  progress_listener.reportCurrentTask(MessageText.getString("splash.loadingTorrent") 
 								  + " " + currentDownload + " "
 								  + MessageText.getString("splash.of") + " " + nbDownloads
-								  + " : " + fileName );
+								  + " : " + shortFileName );
 					  }
 	
 					  //migration from using a single savePath to a separate dir and file entry
@@ -2169,24 +2294,70 @@ public class GlobalManagerImpl
 	DownloadManager 	manager, 
 	int 				new_state ) 
   {
-    needsSaving = true;  //make sure we update 'downloads.config' on state changes
-    
-    	//run seeding-only-mode check
-    
-   PEPeerManager	pm_manager = manager.getPeerManager();
-    
-    if ( 	new_state == DownloadManager.STATE_DOWNLOADING && 
-    		pm_manager != null &&
-    		pm_manager.hasDownloadablePiece()){
-    	
-    	//the new state is downloading, so can skip the full check
-    	
-    	setSeedingOnlyState( false );
-    	
-    }else{
-    	
-    	checkSeedingOnlyState();
-    }
+	  needsSaving = true;  //make sure we update 'downloads.config' on state changes
+
+	  //run seeding-only-mode check
+
+	  PEPeerManager	pm_manager = manager.getPeerManager();
+
+	  if ( 	new_state == DownloadManager.STATE_DOWNLOADING && 
+			  pm_manager != null &&
+			  pm_manager.hasDownloadablePiece()){
+
+		  //the new state is downloading, so can skip the full check
+
+		  setSeedingOnlyState( false, false );
+
+	  }else{
+
+		  checkSeedingOnlyState();
+	  }
+	  	  		  
+	  checkForceStart( manager.isForceStart() && new_state == DownloadManager.STATE_DOWNLOADING );
+  }
+  
+  protected void
+  checkForceStart(
+	 boolean	known_to_exist )
+  {
+	  boolean	exists;
+	  
+	  if ( known_to_exist ){
+		  
+		  exists	= true;
+		  
+	  }else{
+		  
+		  exists	= false;
+		  
+		  if ( force_start_non_seed_exists ){
+			  
+			  List managers = managers_cow;
+			  
+			  for( int i=0; i < managers.size(); i++ ) {
+			    	  
+				  DownloadManager dm = (DownloadManager)managers.get( i );
+	
+				  if ( dm.isForceStart() && dm.getState() == DownloadManager.STATE_DOWNLOADING  ){
+					  
+					  exists = true;
+					  
+					  break;
+				  }
+			  }
+		  }
+	  }
+	  
+	  if ( exists != force_start_non_seed_exists ){
+		  
+		  force_start_non_seed_exists = exists;
+		  
+		  Logger.log(new LogEvent(LOGID, "Force start download " + (force_start_non_seed_exists?"exists":"doesn't exist") + ", modifying download weighting" ));
+		  
+		  //System.out.println( "force_start_exists->" + force_start_non_seed_exists );
+		  
+		  PeerControlSchedulerFactory.getSingleton().overrideWeightedPriorities( force_start_non_seed_exists  );
+	  }
   }
   
   protected void
@@ -2198,8 +2369,9 @@ public class GlobalManagerImpl
   protected void
   checkSeedingOnlyStateSupport()
   {
-    boolean seeding = false;
-    	
+    boolean seeding 			= false;
+    boolean	potentially_seeding	= false;
+    
     List managers = managers_cow;
     
     for( int i=0; i < managers.size(); i++ ) {
@@ -2208,15 +2380,23 @@ public class GlobalManagerImpl
 
         PEPeerManager pm = dm.getPeerManager();
         
+        int	state = dm.getState();
+
         if ( dm.getDiskManager() == null || pm == null ){
         	
-        		// download not running, not interesting
+        		// download not running
+        	
+        	if ( state == DownloadManager.STATE_QUEUED ){
+        		 
+        		if ( dm.isDownloadComplete( false )){
+        			
+        			potentially_seeding = true;
+        		}
+        	}
         	
         	continue;
         }
-        
-        int	state = dm.getState();
-        
+                
         if ( state == DownloadManager.STATE_DOWNLOADING ){
         	
         	if (!pm.hasDownloadablePiece()){
@@ -2227,7 +2407,8 @@ public class GlobalManagerImpl
         		
         	}else{
         		
-        		seeding = false;
+        		seeding 			= false;
+        		potentially_seeding	= false;
         		
         		break;
         	}
@@ -2237,18 +2418,33 @@ public class GlobalManagerImpl
         }
     }
     
-    setSeedingOnlyState( seeding );
+    if ( seeding ){
+    	
+    	potentially_seeding = true;
+    }
+    
+    setSeedingOnlyState( seeding, potentially_seeding );
   }
   
   
   protected void
   setSeedingOnlyState(
-		boolean	seeding )
+		boolean		seeding,
+		boolean		potentially_seeding )
   { 
-    if( seeding != seeding_only_mode ) {
-      seeding_only_mode = seeding;
-      listeners.dispatch( LDT_SEEDING_ONLY, new Boolean( seeding_only_mode ) );
-    }
+	  synchronized( this ){
+		  
+		  if ( 	seeding 			!= seeding_only_mode ||
+	    		potentially_seeding != potentially_seeding_only_mode ){
+		    	
+		      seeding_only_mode 			= seeding;
+		      potentially_seeding_only_mode	= potentially_seeding;
+		      
+		      // System.out.println( "dispatching " + seeding_only_mode + "/" + potentially_seeding_only_mode );
+		      
+		      listeners.dispatch( LDT_SEEDING_ONLY, new boolean[]{ seeding_only_mode, potentially_seeding_only_mode });
+		  }
+	  }
   }
 		
   public boolean
@@ -2257,6 +2453,11 @@ public class GlobalManagerImpl
 	  return( seeding_only_mode );
   }
   
+  public boolean
+  isPotentiallySeedingOnly()
+  {
+	  return( potentially_seeding_only_mode );
+  }
   
 	public long 
 	getTotalSwarmsPeerRate(
@@ -2346,79 +2547,6 @@ public class GlobalManagerImpl
 		return( nat_status );
 	}
 	
-	protected void
-	checkNetwork()
-	{
-		try{
-			Enumeration 	nis = NetworkInterface.getNetworkInterfaces();
-		
-			Set	network_interfaces = new HashSet();
-			
-			boolean	changed	= false;
-			
-			while( nis.hasMoreElements()){
-				
-				Object	 ni = nis.nextElement();
-				
-					// NetworkInterface's "equals" method is based on ni name + addresses
-				
-				if ( old_network_interfaces != null && !old_network_interfaces.contains( ni )){
-					
-					changed	= true;
-				}
-				
-				network_interfaces.add( ni );
-			}
-						
-			if ( changed ){
-				
-				long now	= SystemTime.getCurrentTime();
-				
-					// don't pick up "change" on first time through
-				
-				if ( last_network_change > 0 ){
-					
-					if ( now - last_network_change > 30*60*1000 ){
-					
-						Logger.log(new LogEvent(LOGID,
-								"Network interfaces have changed, updating trackers"));
-						
-						trackerAnnounceAll();
-					}else{
-						
-						Logger.log(new LogEvent(LOGID, "Network interfaces have changed, "
-								+ "not updating trackers as too soon after previous change"));
-	
-					}
-				}
-				
-				last_network_change	= now;
-			}
-			
-			old_network_interfaces	= network_interfaces;
-			
-		}catch( Throwable e ){
-		}
-	}
-	 
-	protected void
-	trackerAnnounceAll()
-	{
-		List	managers = managers_cow;
-		
-		for (int i=0;i<managers.size();i++){
-			
-			DownloadManager	manager = (DownloadManager)managers.get(i);
-			
-			TRTrackerAnnouncer	announcer = manager.getTrackerClient();
-			
-			if ( announcer != null ){
-				
-				announcer.update( true );
-			}
-		}
-	}
-	
 	protected void
 	seedPieceRecheck()
 	{
@@ -2647,4 +2775,12 @@ public class GlobalManagerImpl
 			e.printStackTrace();
 		}
 	}
+	
+	public void setMainlineDHTProvider(MainlineDHTProvider provider) {
+		this.provider = provider;
+	}
+	
+	public MainlineDHTProvider getMainlineDHTProvider() {
+		return this.provider;
+	}
 }
