@@ -19,21 +19,27 @@
  */
 package org.gudy.azureus2.core3.internat;
 
-import org.gudy.azureus2.core3.config.COConfigurationManager;
-import org.gudy.azureus2.core3.logging.*;
-import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.core3.util.FileUtil;
-import org.gudy.azureus2.core3.util.SystemProperties;
-import org.gudy.azureus2.core3.util.Constants;
-
 import java.io.File;
 import java.io.FilenameFilter;
 import java.net.URI;
 import java.net.URL;
 import java.net.URLClassLoader;
+import java.security.AccessController;
 import java.util.*;
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
+
+import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.logging.LogAlert;
+import org.gudy.azureus2.core3.logging.Logger;
+import org.gudy.azureus2.core3.util.Constants;
+import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.core3.util.FileUtil;
+import org.gudy.azureus2.core3.util.SystemProperties;
+
+import sun.security.action.GetPropertyAction;
 
 /**
  * @author Arbeiten
@@ -47,18 +53,34 @@ public class MessageText {
   private static Locale LOCALE_CURRENT = LOCALE_DEFAULT;
   private static final String BUNDLE_NAME = "org.gudy.azureus2.internat.MessagesBundle"; //$NON-NLS-1$
   private static Map pluginLocalizationPaths = new HashMap();
-  private static ResourceBundle RESOURCE_BUNDLE;
+  private static Collection pluginResourceBundles = new ArrayList();
+  private static IntegratedResourceBundle RESOURCE_BUNDLE;
   private static Set			platform_specific_keys	= new HashSet();
+	private static final Pattern PAT_PARAM_ALPHA = Pattern.compile("\\{([^0-9].+?)\\}");
+
 
   private static int bundle_fail_count	= 0;
   
+  private static List listeners = new ArrayList();
+  
+  // preload default language w/o plugins
   static{
-	  setResourceBundle( getResourceBundle(BUNDLE_NAME, LOCALE_DEFAULT, MessageText.class.getClassLoader()));
+	  setResourceBundle( new IntegratedResourceBundle( getResourceBundle( BUNDLE_NAME, LOCALE_DEFAULT, MessageText.class.getClassLoader()), pluginLocalizationPaths ));
+  }
+
+  	// grab a reference to the default bundle
+  
+  private static IntegratedResourceBundle DEFAULT_BUNDLE = RESOURCE_BUNDLE;
+  
+
+  public static void loadBundle() {
+	  loadBundle(false);
   }
   
-  public static void loadBundle() {
-		String savedLocaleString = COConfigurationManager
-				.getStringParameter("locale");
+  public static void loadBundle(boolean forceReload) {
+	  Locale	old_locale = getCurrentLocale();
+	  
+		String savedLocaleString = COConfigurationManager.getStringParameter("locale");
 
 		Locale savedLocale;
 		String[] savedLocaleStrings = savedLocaleString.split("_", 3);
@@ -81,14 +103,51 @@ public class MessageText {
 				savedLocale = Locale.getDefault();
 			}
 		}
-		MessageText.changeLocale(savedLocale);
+		MessageText.changeLocale(savedLocale,forceReload);
 
 		COConfigurationManager
 				.setParameter("locale.set.complete.count", COConfigurationManager
 						.getIntParameter("locale.set.complete.count") + 1);
 
+		Locale	new_locale = getCurrentLocale();
+		
+		if ( !old_locale.equals( new_locale ) || forceReload){
+			
+			for (int i=0;i<listeners.size();i++){
+				
+				try{
+					((MessageTextListener)listeners.get(i)).localeChanged( old_locale, new_locale);
+					
+				}catch( Throwable e ){
+					
+					Debug.printStackTrace(e);
+				}
+			}
+		}
 	}
   
+  	public static void
+  	addListener(
+  		MessageTextListener	listener )
+  	{
+  		listeners.add( listener );
+  	}
+ 	public static void
+  	addAndFireListener(
+  		MessageTextListener	listener )
+  	{
+  		listeners.add( listener );
+  		
+  		listener.localeChanged( getCurrentLocale(), getCurrentLocale());
+  	}
+  	
+  	public static void
+  	removeListener(
+  		MessageTextListener	listener )
+  	{
+  		listeners.remove( listener );
+  	}
+  	
   static ResourceBundle
   getResourceBundle(
 	String		name,
@@ -96,7 +155,8 @@ public class MessageText {
 	ClassLoader	cl )
   {
 	  try{
-		  return( ResourceBundle.getBundle(name, loc, cl ));
+		 
+		 return( ResourceBundle.getBundle(name, loc, cl ));
 		  
 	  }catch( Throwable e ){
 		  
@@ -108,7 +168,7 @@ public class MessageText {
 			  
 			  Logger.log(new LogAlert(LogAlert.REPEATABLE, LogAlert.AT_ERROR,
 						"Failed to load resource bundle. One possible cause is "
-								+ "that you have installed Azureus into a directory "
+								+ "that you have installed " + Constants.APP_NAME + " into a directory "
 								+ "with a '!' in it. If so, please remove the '!'."));
 		  }
 		  
@@ -136,35 +196,31 @@ public class MessageText {
 	  } 
   }
   
-  private static ResourceBundle DEFAULT_BUNDLE = RESOURCE_BUNDLE;
-  
   private static void
   setResourceBundle(
-	  ResourceBundle	bundle )
+	  IntegratedResourceBundle	bundle )
   {
 	  RESOURCE_BUNDLE	= bundle;
 	  
-	  Enumeration	keys = RESOURCE_BUNDLE.getKeys();
+	  Iterator	keys = RESOURCE_BUNDLE.getKeysLight();
 	  
 	  String	platform_suffix = getPlatformSuffix();
 	  
-	  platform_specific_keys.clear();
+	  Set platformKeys = new HashSet();
 	  
-	  while( keys.hasMoreElements()){
-		  
-		  String	key = (String)keys.nextElement();
-		  
-		  if ( key.endsWith( platform_suffix )){
-			  			  
-			  platform_specific_keys.add( key );
-		  }
+	  while( keys.hasNext()){
+		  String	key = (String)keys.next();
+		  if ( key.endsWith( platform_suffix ))
+			  platformKeys.add( key );
 	  }
+	  
+	  platform_specific_keys = platformKeys;
   }
   
  
   public static boolean keyExists(String key) {
     try {
-      RESOURCE_BUNDLE.getString(key);
+    	getResourceBundleString(key);
       return true;
     } catch (MissingResourceException e) {
       return false;
@@ -202,7 +258,7 @@ public class MessageText {
     
 	  try {
       
-		  return RESOURCE_BUNDLE.getString( target_key );
+		  return getResourceBundleString( target_key );
     
 	  }catch (MissingResourceException e) {
 		  
@@ -226,8 +282,8 @@ public class MessageText {
 
 	  try {
 	 
-		  return RESOURCE_BUNDLE.getString( target_key );
-		  
+		  return getResourceBundleString( target_key );
+
 	  } catch (MissingResourceException e) {
 		  
 	      return getPlatformNeutralString(key);
@@ -236,19 +292,49 @@ public class MessageText {
 
   public static String getPlatformNeutralString(String key) {
     try {
-      return RESOURCE_BUNDLE.getString(key);
+      return getResourceBundleString(key);
     } catch (MissingResourceException e) {
+    	// we support the usage of non-resource strings as long as they are wrapped in !
+      if ( key.startsWith("!") && key.endsWith( "!" )){
+    	  return( key.substring(1,key.length()-1 ));
+      }
       return '!' + key + '!';
     }
   }
 
   public static String getPlatformNeutralString(String key, String sDefault) {
     try {
-      return RESOURCE_BUNDLE.getString(key);
+      return getResourceBundleString(key);
     } catch (MissingResourceException e) {
       return sDefault;
     }
   }
+  
+  private static String getResourceBundleString(String key) {
+  	if (key == null) {
+  		return "";
+  	}
+
+		String value = RESOURCE_BUNDLE.getString(key);
+
+		// Replace {*} with a lookup of *
+		if (value != null && value.indexOf('}') > 0) {
+			Matcher matcher = PAT_PARAM_ALPHA.matcher(value);
+			while (matcher.find()) {
+				key = matcher.group(1);
+		    try {
+		    	String text = getResourceBundleString(key);
+					if (text != null) {
+						value = value.replaceAll("\\Q{" + key + "}\\E", text);
+					}
+		    } catch (MissingResourceException e) {
+		    	// ignore error
+		    }
+			}
+		}
+
+		return value;
+	}
 
   /**
    * Gets the localization key suffix for the running platform
@@ -318,6 +404,10 @@ public class MessageText {
   {
   	String	res = getString(key);
   	
+  	if (params == null) {
+  		return res;
+  	}
+  	
   	for(int i=0;i<params.length;i++){
   		
   		String	from_str 	= "%" + (i+1);
@@ -363,6 +453,10 @@ public class MessageText {
     try {
       return DEFAULT_BUNDLE.getString(key);
     } catch (MissingResourceException e) {
+    	// we support the usage of non-resource strings as long as they are wrapped in !
+    	if ( key.startsWith("!") && key.endsWith( "!" )){
+    		return( key.substring(1,key.length()-1 ));
+    	}
       return '!' + key + '!';
     }
   }
@@ -460,7 +554,6 @@ public class MessageText {
   	foundLocalesList.add( LOCALE_ENGLISH );
 
     Iterator val = bundleSet.iterator();
-    int i = 0;
     while (val.hasNext()) {
       String sBundle = (String)val.next();
       
@@ -511,14 +604,18 @@ public class MessageText {
   }
 
   private static boolean changeLocale(Locale newLocale, boolean force) {
-    if (!LOCALE_CURRENT.equals(newLocale) || force) {
+	// set locale for startup (will override immediately it on locale change anyway)
+	Locale.setDefault(newLocale);
+		
+    if (!isCurrentLocale(newLocale) || force) {
       Locale.setDefault(LOCALE_DEFAULT);
       ResourceBundle newResourceBundle = null;
       String bundleFolder = BUNDLE_NAME.replace('.', '/');
       final String prefix = BUNDLE_NAME.substring(BUNDLE_NAME.lastIndexOf('.') + 1);
       final String extension = ".properties";
 
-      
+      if(newLocale.equals(LOCALE_ENGLISH))
+    	  newLocale = LOCALE_DEFAULT;
       
       try {
         File userBundleFile = new File(SystemProperties.getUserPath());
@@ -561,9 +658,9 @@ public class MessageText {
                                                       new URLClassLoader(urls));
         // do more searches if getBundle failed, or if the language is not the 
         // same and the user wanted a specific country
-        if (newResourceBundle == null || 
-            (!newResourceBundle.getLocale().getLanguage().equals(newLocale.getLanguage()) &&
+        if ((!newResourceBundle.getLocale().getLanguage().equals(newLocale.getLanguage()) &&
              !newLocale.getCountry().equals(""))) {
+        	
           Locale foundLocale = newResourceBundle.getLocale();
           System.out.println("changeLocale: "+ 
                              (foundLocale.toString().equals("") ? "*Default Language*" : foundLocale.getDisplayLanguage()) +
@@ -595,25 +692,24 @@ public class MessageText {
       	Debug.printStackTrace( e );
       }
 
-      if (newResourceBundle != null) {
-        //
-        if (!newLocale.toString().equals("en") && 
-            !newResourceBundle.getLocale().equals(newLocale))
-        {
-          String sNewLanguage = newResourceBundle.getLocale().getDisplayName();
-          if (sNewLanguage == null || sNewLanguage.trim().equals(""))
-            sNewLanguage = "English (default)";
-          System.out.println("changeLocale: no message properties for Locale '"+ 
-                             newLocale.getDisplayName() +
-                             "' (" + newLocale + "), using '" + sNewLanguage + "'");
-        }
-        newLocale = newResourceBundle.getLocale();
-        Locale.setDefault(newLocale);
-        LOCALE_CURRENT = newLocale;
-		setResourceBundle( new IntegratedResourceBundle(newResourceBundle, pluginLocalizationPaths));
-        return true;
-      } else
-        return false;
+      if (newResourceBundle != null)
+			{
+				if (!newLocale.equals(LOCALE_DEFAULT) && !newResourceBundle.getLocale().equals(newLocale))
+				{
+					String sNewLanguage = newResourceBundle.getLocale().getDisplayName();
+					if (sNewLanguage == null || sNewLanguage.trim().equals(""))
+						sNewLanguage = "English (default)";
+					System.out.println("changeLocale: no message properties for Locale '" + newLocale.getDisplayName() + "' (" + newLocale + "), using '" + sNewLanguage + "'");
+				}
+				newLocale = newResourceBundle.getLocale();
+				Locale.setDefault(newLocale.equals(LOCALE_DEFAULT) ? LOCALE_ENGLISH : newLocale);
+				LOCALE_CURRENT = newLocale;
+				setResourceBundle(new IntegratedResourceBundle(newResourceBundle, pluginLocalizationPaths));
+				if(newLocale.equals(LOCALE_DEFAULT))
+					DEFAULT_BUNDLE = RESOURCE_BUNDLE;
+				return true;
+			} else
+				return false;
     }
     return false;
   }
@@ -628,13 +724,131 @@ public class MessageText {
   //         - extending ResourceBundle
   //         - override handleGetObject, store in hashtable
   //         - function to add another ResourceBundle, adds to hashtable
-  public static boolean integratePluginMessages(String localizationPath,ClassLoader classLoader) {
-    boolean integratedSuccessfully = false;
-    if (null != localizationPath && localizationPath.length() != 0 && !pluginLocalizationPaths.containsKey(localizationPath)) {
-      pluginLocalizationPaths.put(localizationPath,classLoader);
-	  setResourceBundle( new IntegratedResourceBundle(RESOURCE_BUNDLE, pluginLocalizationPaths));
-      integratedSuccessfully = true;
+  public static boolean integratePluginMessages(String localizationPath, ClassLoader classLoader) {
+		boolean integratedSuccessfully = false;
+		if (null != localizationPath && localizationPath.length() != 0 && !pluginLocalizationPaths.containsKey(localizationPath))
+		{
+			synchronized (pluginLocalizationPaths)
+			{
+				pluginLocalizationPaths.put(localizationPath, classLoader);
+			}
+			
+			RESOURCE_BUNDLE.addPluginBundle(localizationPath, classLoader);
+			setResourceBundle(RESOURCE_BUNDLE);
+
+			integratedSuccessfully = true;
+		}
+		return integratedSuccessfully;
+	}
+  
+  public static boolean integratePluginMessages(ResourceBundle bundle) {
+		synchronized (pluginResourceBundles)
+		{
+			pluginResourceBundles.add(bundle);
+		}
+		
+		RESOURCE_BUNDLE.addResourceMessages(bundle);
+		setResourceBundle(RESOURCE_BUNDLE);
+
+		return true;
+	}
+  
+/**
+ * Reverts Locale back to default, and removes the config settin. 
+ * Notifications of change should be done by the caller.
+ */
+  public static void revertToDefaultLocale() {
+  	// Aside from the last 2 lines, this is Sun's code that is run
+  	// at startup to determine the locale.  Too bad they didn't provide
+  	// a way to call this code explicitly..
+    String language, region, country, variant;
+    language = (String) AccessController.doPrivileged(
+                    new GetPropertyAction("user.language", "en"));
+    // for compatibility, check for old user.region property
+    region = (String) AccessController.doPrivileged(
+                    new GetPropertyAction("user.region"));
+    if (region != null) {
+        // region can be of form country, country_variant, or _variant
+        int i = region.indexOf('_');
+        if (i >= 0) {
+            country = region.substring(0, i);
+            variant = region.substring(i + 1);
+        } else {
+            country = region;
+            variant = "";
+        }
+    } else {
+        country = (String) AccessController.doPrivileged(
+                        new GetPropertyAction("user.country", ""));
+        variant = (String) AccessController.doPrivileged(
+                        new GetPropertyAction("user.variant", ""));
     }
-    return integratedSuccessfully;
+    changeLocale(new Locale(language, country, variant));
+    COConfigurationManager.removeParameter("locale");
   }
+  
+  public static interface
+  MessageTextListener
+  {
+	  public void
+	  localeChanged(
+		Locale	old_locale,
+		Locale	new_locale );
+  }
+  
+	/**
+	 * Sometime a localization key has 2 different returned values: one for Classic UI and another for the Vuze UI
+	 * This method will attempt to locate the given key (with the prefix v3. ) if applicable, if found
+	 * then it will return the key with the prepended prefix.  Otherwise it will return the key as given
+	 * @param localizationKey
+	 * @return
+	 */
+  public static String resolveLocalizationKey(String localizationKey) {
+		if (null == localizationKey) {
+			return null;
+		}
+
+		if (true == "az3".equalsIgnoreCase(COConfigurationManager.getStringParameter("ui"))) {
+			String v3Key = null;
+			if (false == localizationKey.startsWith("v3.")) {
+				v3Key = "v3." + localizationKey;
+			} else {
+				v3Key = localizationKey;
+			}
+
+			if (true == MessageText.keyExists(v3Key)) {
+				return v3Key;
+			}
+		}
+
+		return localizationKey;
+	}
+
+	/**
+	 * Sometime a accelerator key has 2 different returned values: one for Classic UI and another for the Vuze UI
+	 * This method will attempt to locate the given key (with the prefix v3. ) if applicable, if found
+	 * then it will return the key with the prepended prefix.  Otherwise it will return the key as given
+	 * @param acceleratorKey
+	 * @return
+	 */
+	public static String resolveAcceleratorKey(String acceleratorKey) {
+		if (null == acceleratorKey) {
+			return null;
+		}
+
+		if (true == "az3".equalsIgnoreCase(COConfigurationManager.getStringParameter("ui"))) {
+			String v3Key = null;
+			if (false == acceleratorKey.startsWith("v3.")) {
+				v3Key = "v3." + acceleratorKey;
+			} else {
+				v3Key = acceleratorKey;
+			}
+
+			if (true == MessageText.keyExists(v3Key + ".keybinding")) {
+				return v3Key;
+			}
+		}
+
+		return acceleratorKey;
+	}
 }
