@@ -26,19 +26,22 @@ package org.gudy.azureus2.core3.ipfilter.impl;
  *
  */
 
-import java.io.BufferedInputStream;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
+import java.io.*;
+import java.net.Inet6Address;
 import java.net.InetAddress;
 import java.util.*;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.ipfilter.*;
-import org.gudy.azureus2.core3.logging.*;
+import org.gudy.azureus2.core3.logging.LogEvent;
+import org.gudy.azureus2.core3.logging.LogIDs;
+import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.tracker.protocol.PRHelpers;
 import org.gudy.azureus2.core3.util.*;
 
+import com.aelitis.azureus.core.util.CopyOnWriteList;
+
 public class 
 IpFilterImpl 
 	implements IpFilter
@@ -68,6 +71,11 @@ IpFilterImpl
   
 	private List	listeners = new ArrayList();
 	
+	private CopyOnWriteList	external_handlers = new CopyOnWriteList();
+	
+	FrequencyLimitedDispatcher blockedListChangedDispatcher;
+
+	private IpFilterAutoLoaderImpl ipFilterAutoLoader;
 	
 	private IpFilterImpl() 
 	{
@@ -77,7 +85,24 @@ IpFilterImpl
 	  
 	  ipsBlocked = new LinkedList();
 	  
-	  try{
+		blockedListChangedDispatcher = new FrequencyLimitedDispatcher(
+				new AERunnable() {
+					public void runSupport() {
+						Object[] listenersArray = listeners.toArray();
+						for (int i = 0; i < listenersArray.length; i++) {
+							try {
+								IPFilterListener l = (IPFilterListener) listenersArray[i];
+								l.IPBlockedListChanged(IpFilterImpl.this);
+							} catch (Exception e) {
+								Debug.out(e);
+							}
+						}
+					}
+				}, 10000);
+
+		ipFilterAutoLoader = new IpFilterAutoLoaderImpl(this);
+		
+		try{
 		  loadBannedIPs();
 		  
 	  }catch( Throwable e ){
@@ -86,12 +111,21 @@ IpFilterImpl
 	  }
 	  try{
 	  	
-	  	loadFilters();
+	  	loadFilters(true, true);
 	  	
 	  }catch( Exception e ){
 	  	
 	  	Debug.printStackTrace( e );
 	  }
+	  
+	  COConfigurationManager.addParameterListener(new String[] {
+			"Ip Filter Allow",
+			"Ip Filter Enabled"
+		}, new ParameterListener() {
+			public void parameterChanged(String parameterName) {
+				markAsUpToDate();
+			}
+		});
 	}
   
 	public static IpFilter getInstance() {
@@ -118,7 +152,16 @@ IpFilterImpl
 	reload()
 		throws Exception
 	{
-		loadFilters();
+		reload(true);
+	}
+	
+	public void
+	reload(boolean allowAsyncDownloading)
+		throws Exception
+	{
+		range_manager.clearAllEntries();
+		markAsUpToDate();
+		loadFilters(allowAsyncDownloading, false);
 	}
 	
 	public void 
@@ -176,9 +219,17 @@ IpFilterImpl
 	}
   
 	private void 
-	loadFilters() 
+	loadFilters(boolean allowAsyncDownloading, boolean loadOldWhileAsyncDownloading) 
 		throws Exception
 	{
+		long startTime = System.currentTimeMillis();
+		ipFilterAutoLoader.loadOtherFilters(allowAsyncDownloading, loadOldWhileAsyncDownloading);
+		
+		if (getNbRanges() > 0) {
+			Logger.log(new LogEvent(LOGID, (System.currentTimeMillis() - startTime)
+					+ "ms for " + getNbRanges() + ". now loading norm"));
+		}
+
 		try{
 			class_mon.enter();
 		
@@ -237,6 +288,8 @@ IpFilterImpl
 			
 			class_mon.exit();
 		}
+		Logger.log(new LogEvent(LOGID, (System.currentTimeMillis() - startTime)
+				+ "ms to load all IP Filters"));
 	}
   
 	protected void
@@ -351,29 +404,34 @@ IpFilterImpl
 	}
 	protected boolean
 	isInRange(
-		IpRangeImpl	range,
+		IpRange		range,
 		String		address )
 	{
 		return( range_manager.isInRange( range, address ));
 	}
   
-  public boolean isInRange(String ipAddress) {
-    return isInRange( ipAddress, "" );
+  public boolean 
+  isInRange(
+	  String ipAddress) 
+  {
+    return isInRange( ipAddress, "", null );
   }
   
   
 	public boolean 
 	isInRange(
-		String ipAddress, 
-		String torrent_name) 
+		String 	ipAddress, 
+		String 	torrent_name,
+		byte[]	torrent_hash )
 	{
-		return( isInRange( ipAddress, torrent_name, true ));
+		return( isInRange( ipAddress, torrent_name, torrent_hash, true ));
 	}
 	
 	public boolean 
 	isInRange(
 		String ipAddress, 
 		String torrent_name,
+		byte[] torrent_hash,
 		boolean	loggable ) 
 	{
 		//In all cases, block banned ip addresses
@@ -396,21 +454,35 @@ IpFilterImpl
 		  return( false );
 	  }
 	  
-	  //never block lan local addresses
+	  	// don't currently support IPv6
+	  
+	  if ( ipAddress.indexOf( ":" ) != -1 ){
+		  
+		  return( false );
+	  }
+	  
+	  	//never block lan local addresses
+	  
 	  if( AddressUtils.isLANLocalAddress( ipAddress ) != AddressUtils.LAN_LOCAL_NO ) {
 	  	return false;
 	  }
-	  
-	  
-	  if(!COConfigurationManager.getBooleanParameter("Ip Filter Enabled",true)){
-		  
-	    return false;
-	  }
-	  
+	  	  
 	  boolean allow = COConfigurationManager.getBooleanParameter("Ip Filter Allow");
 	  
 	  IpRange	match = (IpRange)range_manager.isInRange( ipAddress );
 
+	  if ( match == null || allow ){
+		  
+		  IpRange explict_deny = checkExternalHandlers( torrent_hash, ipAddress );
+		  
+		  if ( explict_deny != null ){
+			  
+			  match	= explict_deny;
+			  
+			  allow = false;
+		  }
+	  }
+	  
 	  if(match != null) {
 	    if(!allow) {
 	    	
@@ -422,13 +494,22 @@ IpFilterImpl
 			  return( false );
 		  }
 		  
-	      addBlockedIP( new BlockedIpImpl(ipAddress,match, torrent_name, loggable), loggable );
+	      if ( addBlockedIP( new BlockedIpImpl( ipAddress, match, torrent_name, loggable), torrent_hash, loggable )){
 	      
-	      if (Logger.isEnabled())
-					Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocked : "
+		      if (Logger.isEnabled())
+						Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocked : "
+								+ ipAddress + ", in range : " + match));
+		      
+		      return true;
+		      
+	      }else{
+	    	  
+		      if (Logger.isEnabled())
+					Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocking Denied : "
 							+ ipAddress + ", in range : " + match));
 	      
-	      return true;
+		      return false;
+	      }
 	    }
       
 	    return false;  
@@ -442,13 +523,22 @@ IpFilterImpl
 		  return( false );
 		}
 		  
-	    addBlockedIP( new BlockedIpImpl(ipAddress,null, torrent_name, loggable), loggable );
+	    if ( addBlockedIP( new BlockedIpImpl(ipAddress,null, torrent_name, loggable), torrent_hash, loggable )){
 	    
-	    if (Logger.isEnabled())
-				Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocked : "
+		    if (Logger.isEnabled())
+					Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocked : "
+							+ ipAddress + ", not in any range"));
+		    
+		    return true;
+		    
+	    }else{
+	    	
+		    if (Logger.isEnabled())
+				Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocking Denied : "
 						+ ipAddress + ", not in any range"));
 	    
-	    return true;
+		    return false;
+	    }
 	  }
 	  
 	  return false;
@@ -459,6 +549,7 @@ IpFilterImpl
 	isInRange(
 		InetAddress ipAddress, 
 		String 		torrent_name,
+		byte[] 		torrent_hash,
 		boolean		loggable ) 
 	{
 		//In all cases, block banned ip addresses
@@ -480,32 +571,63 @@ IpFilterImpl
 		  return( false );
 	  }
 	  
-	  //never block lan local addresses
+	  	// don't currently support IPv6
+	  
+	  if ( ipAddress instanceof Inet6Address ){
+		  
+		  return( false );
+	  }
+	  
+	  	//never block lan local addresses
+	  
 	  if( AddressUtils.isLANLocalAddress( ipAddress ) != AddressUtils.LAN_LOCAL_NO ) {
+		  
 	  	return false;
 	  }
-	  
-	  
-	  if(!COConfigurationManager.getBooleanParameter("Ip Filter Enabled",true)){
-		  
-	    return false;
-	  }
-	  
+	  	  
 	  boolean allow = COConfigurationManager.getBooleanParameter("Ip Filter Allow");
 	  
 	  IpRange	match = (IpRange)range_manager.isInRange( ipAddress );
 
-	  if(match != null) {
+	  if ( match == null || allow ){
+		  
+		  	// get here if 
+		  	// 		match -> deny and we didn't match
+		  	//		match -> allow and we did match
+		  
+		  IpRange explicit_deny = checkExternalHandlers( torrent_hash, ipAddress );
+		  
+		  if ( explicit_deny != null ){
+			  
+			  	// turn this into a denial
+			  
+			  match = explicit_deny;
+			  
+			  allow = false;
+		  }
+	  }
+	  
+	  if ( match != null ){
 		  
 	    if(!allow) {
 	    			  
-	      addBlockedIP( new BlockedIpImpl(ipAddress.getHostAddress(),match, torrent_name, loggable), loggable );
+	      if ( addBlockedIP( new BlockedIpImpl(ipAddress.getHostAddress(),match, torrent_name, loggable), torrent_hash, loggable )){
 	      
-	      if (Logger.isEnabled())
-					Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocked : "
-							+ ipAddress + ", in range : " + match));
-	      
-	      return true;
+		      if (Logger.isEnabled())
+						Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocked : "
+								+ ipAddress + ", in range : " + match));
+		      
+		      return true;
+		      
+	      }else{
+		      
+		      if (Logger.isEnabled())
+						Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocking Denied: "
+								+ ipAddress + ", in range : " + match));
+		      
+		      return false;
+
+	      }
 	    }
       
 	    return false;  
@@ -514,43 +636,116 @@ IpFilterImpl
 	
 	  if( allow ){  
 		  
-	    addBlockedIP( new BlockedIpImpl(ipAddress.getHostAddress(),null, torrent_name, loggable), loggable );
+	    if ( addBlockedIP( new BlockedIpImpl(ipAddress.getHostAddress(),null, torrent_name, loggable), torrent_hash, loggable )){
 	    
-	    if (Logger.isEnabled())
-				Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocked : "
+		    if (Logger.isEnabled())
+					Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocked : "
+							+ ipAddress + ", not in any range"));
+		    
+		    return true;
+	    }else{
+	    	
+		    if (Logger.isEnabled())
+				Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Ip Blocking Denied : "
 						+ ipAddress + ", not in any range"));
 	    
-	    return true;
+		    return false;
+	    }
 	  }
 	  
 	  return false;
 	}
 	
-	
+	protected IpRange
+	checkExternalHandlers(
+		byte[]	torrent_hash,
+		String	address )
+	{
+		if ( external_handlers.size() > 0 ){
+					
+			Iterator it = external_handlers.iterator();
+			
+			while( it.hasNext()){
+				
+				if (((IpFilterExternalHandler)it.next()).isBlocked( torrent_hash, address )){
+					
+					return( new IpRangeImpl( "External handler", address, address, true ));
+				}
+			}
+		}
+		
+		return( null );
+	}
   
-  private void 
-  addBlockedIP( 
-	BlockedIp 	ip,
-	boolean		loggable ) 
-  {
-    try{  class_mon.enter();
-   
-      ipsBlocked.addLast( ip );
-      
-      num_ips_blocked++;
-      
-      if ( loggable ){
-    	  
-    	  num_ips_blocked_loggable++;
-      }
-      
-      if( ipsBlocked.size() > MAX_BLOCKS_TO_REMEMBER ) {  //only "remember" the last few blocks occurrences
-    	  
-        ipsBlocked.removeFirst();
-      }
-    }
-    finally{  class_mon.exit();  }
-  }
+	protected IpRange
+	checkExternalHandlers(
+		byte[]		torrent_hash,
+		InetAddress	address )
+	{
+		if ( external_handlers.size() > 0 ){
+					
+			Iterator it = external_handlers.iterator();
+			
+			while( it.hasNext()){
+				
+				if (((IpFilterExternalHandler)it.next()).isBlocked( torrent_hash, address )){
+					
+					String	ip = address.getHostAddress();
+					
+					return( new IpRangeImpl( "External handler", ip, ip, true ));
+				}
+			}
+		}
+		
+		return( null );
+	}
+	
+	private boolean 
+	addBlockedIP( 
+		BlockedIp 	ip,
+		byte[]		torrent_hash,
+		boolean		loggable ) 
+	{
+		if ( torrent_hash != null ){
+			
+			List	listeners_ref = listeners;
+	
+			for (int j=0;j<listeners_ref.size();j++){
+	
+				try{
+					if ( !((IPFilterListener)listeners_ref.get(j)).canIPBeBlocked( ip.getBlockedIp(), torrent_hash )){
+	
+						return( false );
+					}
+	
+				}catch( Throwable e ){
+	
+					Debug.printStackTrace(e);
+				}
+			}
+		}
+		
+		try{  class_mon.enter();
+
+			ipsBlocked.addLast( ip );
+	
+			num_ips_blocked++;
+	
+			if ( loggable ){
+	
+				num_ips_blocked_loggable++;
+			}
+	
+			if( ipsBlocked.size() > MAX_BLOCKS_TO_REMEMBER ) {  //only "remember" the last few blocks occurrences
+	
+				ipsBlocked.removeFirst();
+			}
+		}finally{
+			class_mon.exit();  
+		}
+		
+		return( true );
+	}
   
   
   
@@ -701,7 +896,7 @@ IpFilterImpl
 	
 	protected void
 	setValidOrNot(
-		IpRangeImpl		range,
+		IpRange		range,
 		boolean			valid )
 	{
 		try{
@@ -744,9 +939,31 @@ IpFilterImpl
 	
 	public boolean 
 	ban(
-		String 	ipAddress,
-		String	torrent_name ) 
+		String 		ipAddress,
+		String		torrent_name,
+		boolean		manual ) 
 	{
+			// always allow manual bans through
+		
+		if ( !manual ){
+			
+			List	listeners_ref = listeners;
+			
+			for (int j=0;j<listeners_ref.size();j++){
+				
+				try{
+					if ( !((IPFilterListener)listeners_ref.get(j)).canIPBeBanned( ipAddress )){
+						
+						return( false );
+					}
+					
+				}catch( Throwable e ){
+					
+					Debug.printStackTrace(e);
+				}
+			}
+		}
+		
 		boolean	block_ban = false;
 		
 		List	new_bans = new ArrayList();
@@ -820,9 +1037,9 @@ IpFilterImpl
 			
 			class_mon.exit();
 		}
-		
+			
 		List	listeners_ref = listeners;
-		
+
 		for (int i=0;i<new_bans.size();i++){
 			
 			BannedIp entry	= (BannedIp)new_bans.get(i);
@@ -892,9 +1109,10 @@ IpFilterImpl
 			
 			Integer	i_address = new Integer( address );
 			
-			bannedIps.remove(i_address);
+			if ( bannedIps.remove(i_address) != null ){
 			
-			saveBannedIPs();
+				saveBannedIPs();
+			}
 			
 		}finally{
 			
@@ -902,6 +1120,68 @@ IpFilterImpl
 		}
 	}
 	
+	public void
+	unban(String ipAddress, boolean block)
+	{
+		if ( block ){
+	
+			int	address = range_manager.addressToInt( ipAddress );	
+				
+			long	l_address = address;
+			
+	    	if ( l_address < 0 ){
+	     		
+				l_address += 0x100000000L;
+	     	}
+			
+			long	start 	= l_address & 0xffffff00;
+			long	end		= start+256;
+		
+			boolean	hit = false;
+			
+			try{
+				class_mon.enter();
+
+				for (long i=start;i<end;i++){
+					
+					Integer	a = new Integer((int)i);
+	
+					if ( bannedIps.remove(a) != null ){
+						
+						hit = true;
+					}
+				}
+				
+				if ( hit ){
+					
+					saveBannedIPs();
+				}
+			}finally{
+				
+				class_mon.exit();
+			}
+			
+		}else{
+			
+			try{
+				class_mon.enter();
+			
+				int	address = range_manager.addressToInt( ipAddress );
+				
+				Integer	i_address = new Integer( address );
+				
+				if ( bannedIps.remove(i_address) != null ){
+				
+					saveBannedIPs();
+				}
+				
+			}finally{
+				
+				class_mon.exit();
+			}
+		}
+	}
+	
 	
 	public BlockedIp[] 
 	getBlockedIps() 
@@ -940,7 +1220,7 @@ IpFilterImpl
 	public boolean
 	isEnabled()
 	{
-		return( COConfigurationManager.getBooleanParameter("Ip Filter Enabled",true));	
+		return( COConfigurationManager.getBooleanParameter("Ip Filter Enabled"));	
 	}
 
 	public void
@@ -953,7 +1233,9 @@ IpFilterImpl
 	public void
 	markAsUpToDate()
 	{
-	  	last_update_time	= SystemTime.getCurrentTime();		
+	  	last_update_time	= SystemTime.getCurrentTime();
+	  	
+	  	blockedListChangedDispatcher.dispatch();
 	}
 
 	public long
@@ -1006,15 +1288,29 @@ IpFilterImpl
 		}
 	}
 	
+	public void
+	addExternalHandler(
+		IpFilterExternalHandler h )
+	{
+		external_handlers.add( h );
+	}
+	
+	public void
+	removeExternalHandler(
+		IpFilterExternalHandler h )
+	{
+		external_handlers.remove( h );
+	}
+	
 	public static void
 	main(
 		String[]	args )
 	{
 		IpFilterImpl	filter = new IpFilterImpl();
 		
-		filter.ban( "255.1.1.1", "parp" );
-		filter.ban( "255.1.1.2", "parp" );
-		filter.ban( "255.1.2.2", "parp" );
+		filter.ban( "255.1.1.1", "parp", true );
+		filter.ban( "255.1.1.2", "parp", true );
+		filter.ban( "255.1.2.2", "parp", true );
 		
 		System.out.println( "is banned:" + filter.isBanned( "255.1.1.4" ));
 	}
