@@ -28,6 +28,7 @@ package org.gudy.azureus2.core3.peer;
 
 import org.gudy.azureus2.plugins.network.Connection;
 
+import com.aelitis.azureus.core.networkmanager.LimitedRateGroup;
 import com.aelitis.azureus.core.peermanager.messaging.Message;
 import com.aelitis.azureus.core.peermanager.piecepicker.util.BitFlags;
 
@@ -48,12 +49,20 @@ PEPeer
 	public final static int CLOSING      	= 40;
 	public final static int DISCONNECTED 	= 50;
 	
+	
 	// these should be maintained to match above list ordinals /10
 	// if they don't than some debug info won't work right (not too big of a deal)
 	public final static String[] StateNames = { "Twinkle",
 		"Connecting", "Handshaking", "Transfering", "Closing", "Disconnected"
 	};
 	
+	public static final int MESSAGING_BT_ONLY = 1;
+	public static final int MESSAGING_AZMP = 2;
+	public static final int MESSAGING_LTEP = 3;
+	//used for plugins, such as webseeds
+	public static final int MESSAGING_EXTERN = 4;
+	
+	
   /**
    * Add peer listener.
    * @param listener
@@ -156,6 +165,13 @@ PEPeer
 	public boolean isDownloadPossible();
 	
 	public boolean isSeed();
+	
+	/**
+	 * @return true if the peer is only uploading and can't use the data we have
+	 *         to offer (we're seeding) or we can't use the data he has to offer
+	 *         (we're downloading)
+	 */
+	public boolean isRelativeSeed();
  
 	public boolean isSnubbed();
 	
@@ -165,6 +181,8 @@ PEPeer
  	
 	public boolean isIncoming();
 
+	public boolean hasReceivedBitField();
+	
   /**
    * Get the peer's torrent completion percentage in thousand-notation,
    * i.e. 53.7% is returned as the value 0537.
@@ -191,13 +209,30 @@ PEPeer
 	public int getConsecutiveNoRequestCount();
 	public void setConsecutiveNoRequestCount( int num );
 
-  
+	public void setUploadRateLimitBytesPerSecond( int bytes );
+	public void setDownloadRateLimitBytesPerSecond( int bytes );
+	public int getUploadRateLimitBytesPerSecond();
+	public int getDownloadRateLimitBytesPerSecond();
+	
+	public void
+	addRateLimiter(
+		LimitedRateGroup	limiter,
+		boolean				upload );
+	
+	public void
+	removeRateLimiter(
+		LimitedRateGroup	limiter,
+		boolean				upload );
+	
   /** To retreive arbitrary objects against a peer. */
   public Object getData (String key);
   /** To store arbitrary objects against a peer. */
   public void setData (String key, Object value);
   
-  
+  public Object getUserData (Object key);
+  /** To store arbitrary objects against a peer. */
+  public void setUserData (Object key, Object value);
+
   /**
    * Get the connection that backs this peer.
    * @return connection
@@ -212,6 +247,13 @@ PEPeer
    */
   public boolean supportsMessaging();
   
+  /**
+   * @Return the handshaked messaging type, {@link PEPeer} constants
+   */ 
+  public int getMessagingMode();
+
+  
+  
   /**
    * Returns name of encryption used by the peer
    * @return
@@ -238,6 +280,13 @@ PEPeer
   public int getIncomingRequestCount();
   public int getOutgoingRequestCount();
   
+  /**
+   * amount of data queued for delivery to peer
+   * @return
+   */
+  
+  public int getOutboundDataQueueSize();
+  
   /**
    * get a list of piece numbers the peer has requested
    * @return list of Long() representing the piece number requested, in order
@@ -257,6 +306,9 @@ PEPeer
   public int
   getPercentDoneOfCurrentOutgoingRequest();
   
+  public long
+  getBytesRemaining();
+  
   /**
    * Get the time since this connection was first established.
    * NOTE: This method will always return 0 at any time before
@@ -269,4 +321,42 @@ PEPeer
 	public void setLastPiece(int i);
 	public int getLastPiece();
 	
+	public boolean
+	isLANLocal();
+	
+		/**
+		 * Send a request hint to the peer. 
+		 * @param piece_number
+		 * @param offset
+		 * @param length
+		 * @param life
+		 * @return true if sent, false otherwise
+		 */
+	
+	public boolean
+	sendRequestHint(
+		int		piece_number,
+		int		offset,
+		int		length,
+		int		life );
+	
+		/**
+		 * Get current request hint for a given piece for this peer. 
+		 * @return null if no hint int[]{ piece_number, offset, length } if hint found
+		 */
+	
+	public int[]
+	getRequestHint();
+	        		 
+	public void
+	clearRequestHint();
+	
+	public void
+	setHaveAggregationEnabled(
+		boolean		enabled );
+	
+	public byte[] getHandshakeReservedBytes();
+	
+	public String getClientNameFromPeerID();
+	public String getClientNameFromExtensionHandshake();
 }
\ No newline at end of file
