@@ -28,7 +28,7 @@ package org.gudy.azureus2.core3.peer;
  *
  */
 
-import java.util.List;
+import java.util.*;
 
 import org.gudy.azureus2.core3.disk.*;
 import org.gudy.azureus2.core3.peer.impl.PEPeerTransport;
@@ -46,10 +46,11 @@ import com.aelitis.azureus.core.peermanager.piecepicker.PiecePicker;
 public interface 
 PEPeerManager 
 {
-	
 	public DiskManager getDiskManager();
 	public PiecePicker getPiecePicker();
 	
+	public PEPeerManagerAdapter	getAdapter();
+	
 	public void
 	start();
 		
@@ -76,6 +77,8 @@ PEPeerManager
 
 	public float getMinAvailability();
 
+	public long getAvailWentBadTime();
+	
 	public boolean hasDownloadablePiece();
 	
     
@@ -103,6 +106,8 @@ PEPeerManager
 		
 	public long getRemaining();
 	
+	public long getHiddenBytes();
+	
 	public long getETA();
 
 	public String getElapsedTime();
@@ -130,16 +135,25 @@ PEPeerManager
   public boolean
   isSuperSeedMode();
   
+  public boolean
+  canToggleSuperSeedMode();
+  
+  public void
+  setSuperSeedMode( boolean on );
+  
   public boolean
   seedPieceRecheck();
   
-  public int getNbRemoteConnections();
+  public int getNbRemoteConnectionsExcludingUDP();
   
   public long getLastRemoteConnectionTime();
   
   public int
   getMaxNewConnectionsAllowed();
   
+  public boolean
+  hasPotentialConnections();
+  
   /**
    * Data bytes received.
    * @param l
@@ -205,9 +219,11 @@ PEPeerManager
 		String 		ip_address, 
 		int 		tcp_port,
 		int			udp_port,
-		boolean 	use_crypto );
+		boolean 	use_crypto,
+		Map			user_data );
   
 	
+	
 	public void
 	removePeer(
 		PEPeer	peer );
@@ -233,10 +249,18 @@ PEPeerManager
 	requestCanceled(
 		DiskManagerReadRequest	item );
 		
+	public boolean
+	requestExists(
+		String			peer_ip,
+		int				piece_number,
+		int				offset,
+		int				length );
+	
 	public boolean 
 	validatePieceReply(
-		int 		pieceNumber, 
-		int 		offset, 
+		PEPeerTransport		originator,
+		int 				pieceNumber, 
+		int 				offset, 
 		DirectByteBuffer 	data );
 	
 	public void 
@@ -261,7 +285,7 @@ PEPeerManager
    * Notify the manager that the given peer connection has been closed.
    * @param peer closed
    */
-  public void peerConnectionClosed( PEPeerTransport peer, boolean connect_failed );
+  public void peerConnectionClosed( PEPeerTransport peer, boolean connect_failed, boolean network_failed );
   
   
   
@@ -293,6 +317,9 @@ PEPeerManager
    */
   public LimitedRateGroup getDownloadLimitedRateGroup();
   
+  public int getUploadRateLimitBytesPerSecond();
+  
+  public int getDownloadRateLimitBytesPerSecond();
   
   /** To retreive arbitrary objects against this object. */
   public Object getData (String key);
@@ -320,6 +347,27 @@ PEPeerManager
 	 */
 	public PEPeerTransport getTransportFromAddress(String peer);
 	
+	public boolean
+	getPreferUDP();
+	
+	public void
+	setPreferUDP(
+		boolean	prefer );
+	
+	public void
+	addRateLimiter(
+		LimitedRateGroup	group,
+		boolean				upload );
+	
+	public void
+	removeRateLimiter(
+		LimitedRateGroup	group,
+		boolean				upload );
+	
+	public boolean
+	isPeerSourceEnabled(
+		String	peer_source );
+	
 	public void
 	generateEvidence(
 		IndentWriter		writer );
