@@ -42,6 +42,13 @@ PEPeerControl
 		int 			offset, 
 		int 			length );
 
+	public boolean
+	validateHintRequest(
+		PEPeerTransport	originator,
+		int 			pieceNumber, 
+		int 			offset, 
+		int 			length );
+	
 	public void 
 	havePiece(
 		int pieceNumber,
@@ -54,7 +61,7 @@ PEPeerControl
 	    int pieceNumber);
 	
 	public boolean
-	isAZMessagingEnabled();
+	isExtendedMessagingEnabled();
   
 	public boolean
 	isPeerExchangeEnabled();
@@ -63,6 +70,9 @@ PEPeerControl
 	getSecrets(
 		int	crypto_level );
 	
+	public int
+	getHiddenPiece();
+	
 	public void addPeerTransport( PEPeerTransport transport );
 	
 	public int
@@ -70,7 +80,8 @@ PEPeerControl
     
     public boolean 
     doOptimisticDisconnect(
-    	boolean pending_lan_local_peer );
+    	boolean pending_lan_local_peer,
+    	boolean	force );
     
 	public int getNbActivePieces();
 
@@ -81,4 +92,11 @@ PEPeerControl
 	public void decNbPeersSnubbed();
 	public void setNbPeersSnubbed(int n);
 	public int getNbPeersSnubbed();
+	
+	public void
+	badPieceReported(
+		PEPeerTransport		originator,
+		int					piece_number );
+	
+	public boolean isRTA();
 }
\ No newline at end of file
