@@ -238,7 +238,7 @@ TOTorrentDeserialiseImpl
 				// decode the stuff
 			
 			Iterator root_it = meta_data.keySet().iterator();
-			
+	
 			while( root_it.hasNext()){
 				
 				String	key = (String)root_it.next();
@@ -259,7 +259,7 @@ TOTorrentDeserialiseImpl
 						
 						try{
 						
-							setAnnounceURL( new URL( announce_url ));
+							setAnnounceURL( new URL(announce_url));
 							
 						}catch( MalformedURLException e ){
 							
@@ -270,7 +270,7 @@ TOTorrentDeserialiseImpl
 							
 							try{
 								
-								setAnnounceURL( new URL( announce_url ));
+								setAnnounceURL(new URL(announce_url));
 									
 							}catch( MalformedURLException f ){
 									
@@ -287,7 +287,8 @@ TOTorrentDeserialiseImpl
 					
 					Object	ann_list = meta_data.get( TK_ANNOUNCE_LIST );
 					
-					if( ann_list instanceof List ) {   //some malformed torrents have this key as a zero-sized string instead of a zero-sized list
+					if ( ann_list instanceof List ){   //some malformed torrents have this key as a zero-sized string instead of a zero-sized list
+						
 						announce_list = (List)ann_list;
 					}
 					
@@ -304,55 +305,63 @@ TOTorrentDeserialiseImpl
             
 						for (int i=0;i<announce_list.size();i++){
 							
-							List	set = (List)announce_list.get(i);
+							Object temp = announce_list.get(i);
+							
+							if ( temp instanceof List ){
 								
-							Vector urls = new Vector();
-								
-							for (int j=0;j<set.size();j++){
-								
-								String url_str = readStringFromMetaData((byte[])set.get(j));
+								List	set = (List)temp;
 									
-								url_str=url_str.replaceAll( " ", "" );
-                  
-					                	//check to see if the announce url is somewhere in the announce-list
-																		
-					            try{
-					            	urls.add( new URL( url_str ));		
-						    
-					            	if ( url_str.equalsIgnoreCase( announce_url )) {
-					                	
-					            		announce_url_found = true;
-					            	}
-						
-					            }catch( MalformedURLException e ){
+								Vector urls = new Vector();
 									
-					            	if ( url_str.indexOf( "://" ) == -1 ){
-											
-					            		url_str = "http:/" + (url_str.startsWith("/")?"":"/") + url_str;
-									}
-							         
-									try{
-						           		urls.add( new URL( url_str ));		
-						          
-						           		if ( url_str.equalsIgnoreCase( announce_url )) {
+								for (int j=0;j<set.size();j++){
+									
+									String url_str = readStringFromMetaData((byte[])set.get(j));
+										
+									url_str = url_str.replaceAll( " ", "" );
+	                  
+						                	//check to see if the announce url is somewhere in the announce-list
+																			
+						            try{
+						            	urls.add( new URL( StringInterner.intern(url_str) ));		
+							    
+						            	if ( url_str.equalsIgnoreCase( announce_url )) {
 						                	
 						            		announce_url_found = true;
 						            	}
-						       
-									}catch( MalformedURLException f ){
-				
-										Debug.printStackTrace( f );
-									} 
+							
+						            }catch( MalformedURLException e ){
+										
+						            	if ( url_str.indexOf( "://" ) == -1 ){
+												
+						            		url_str = "http:/" + (url_str.startsWith("/")?"":"/") + url_str;
+										}
+								         
+										try{
+							           		urls.add( new URL( StringInterner.intern(url_str) ));		
+							          
+							           		if ( url_str.equalsIgnoreCase( announce_url )) {
+							                	
+							            		announce_url_found = true;
+							            	}
+							       
+										}catch( MalformedURLException f ){
+					
+											Debug.printStackTrace( f );
+										} 
+									}
 								}
-							}
 							
-							if ( urls.size() > 0 ){
-							
-								URL[]	url_array = new URL[urls.size()];
+								if ( urls.size() > 0 ){
 								
-								urls.copyInto( url_array );
+									URL[]	url_array = new URL[urls.size()];
+									
+									urls.copyInto( url_array );
+									
+									addTorrentAnnounceURLSet( url_array );
+								}
+							}else{
 								
-								addTorrentAnnounceURLSet( url_array );
+								Debug.out( "Torrent has invalid url-list entry (" + temp + ") - ignoring" );
 							}
 						}
             
@@ -362,7 +371,7 @@ TOTorrentDeserialiseImpl
 			            if ( !announce_url_found && announce_url != null && announce_url.length() > 0) {
 			              try {
 			              	Vector urls = new Vector();
-			              	urls.add( new URL( announce_url ));
+			              	urls.add( new URL( StringInterner.intern(announce_url) ));
 			              	URL[] url_array = new URL[ urls.size() ];
 			              	urls.copyInto( url_array );
 			              	addTorrentAnnounceURLSet( url_array );
@@ -512,9 +521,9 @@ TOTorrentDeserialiseImpl
 					total_length += len;
 					
 						// preserve any non-standard attributes
-						
+											
 					Iterator file_it = file_map.keySet().iterator();
-					
+
 					while( file_it.hasNext()){
 						
 						String	key = (String)file_it.next();
@@ -563,8 +572,8 @@ TOTorrentDeserialiseImpl
 			
 				// extract and additional info elements
 				
-			Iterator	info_it = info.keySet().iterator();
-			
+			Iterator info_it = info.keySet().iterator();
+
 			while( info_it.hasNext()){
 
 				String	key = (String)info_it.next();
@@ -590,7 +599,7 @@ TOTorrentDeserialiseImpl
 				throw((TOTorrentException)e);	
 			}
 			
-			throw( new TOTorrentException( "Decode fails '" + e.toString() + "'",
+			throw( new TOTorrentException( "Torrent decode fails '" + Debug.getNestedExceptionMessageAndStack(e) + "'",
 											TOTorrentException.RT_DECODE_FAILS, e ));
 		}
 	}
@@ -616,10 +625,8 @@ TOTorrentDeserialiseImpl
 		Map			map )
 	{
 		System.out.println( indent + name + "{map}" );
-		
-		Set	keys = map.keySet();
-		
-		Iterator it = keys.iterator();
+				
+		Iterator it = map.keySet().iterator();
 		
 		while( it.hasNext()){
 			
