@@ -57,8 +57,11 @@ TOTorrentImpl
 	protected static final String TK_PATH_UTF8			= "path.utf-8";
 	protected static final String TK_COMMENT_UTF8		= "comment.utf-8";
 	
+	protected static final String TK_WEBSEED_BT			= "httpseeds";
+	protected static final String TK_WEBSEED_GR			= "url-list";
+	
 	protected static final List	TK_ADDITIONAL_OK_ATTRS = 
-		Arrays.asList(new String[]{TK_COMMENT_UTF8, AZUREUS_PROPERTIES });
+		Arrays.asList(new String[]{ TK_COMMENT_UTF8, AZUREUS_PROPERTIES, TK_WEBSEED_BT, TK_WEBSEED_GR });
 	
 	private byte[]							torrent_name;
 	private byte[]							torrent_name_utf8;
@@ -80,8 +83,8 @@ TOTorrentImpl
 	private long				creation_date;
 	private byte[]				created_by;
 	
-	private Map					additional_properties 		= new HashMap();
-	private Map					additional_info_properties	= new HashMap();
+	private Map					additional_properties 		= new LightHashMap(4);
+	private Map					additional_info_properties	= new LightHashMap(4);
 	
 	private boolean				created;
 	private boolean				serialising;
@@ -550,7 +553,7 @@ TOTorrentImpl
 		if (s0.equals(s1))
 			return false;
 		
-		announce_url	= newURL;
+		announce_url	= StringInterner.internURL(newURL);
 		return true;
 	}
 
@@ -844,7 +847,7 @@ TOTorrentImpl
 		String		name,
 		Object		value )
 	{
-		if ( name instanceof String ){
+		if ( value instanceof String ){
 			
 			setAdditionalStringProperty(name,(String)value);
 			
