@@ -58,7 +58,10 @@ import org.gudy.azureus2.pluginsimpl.local.clientid.ClientIDManagerImpl;
 
 import com.aelitis.azureus.core.dht.netcoords.DHTNetworkPosition;
 import com.aelitis.azureus.core.dht.netcoords.DHTNetworkPositionManager;
+import com.aelitis.azureus.core.networkmanager.NetworkManager;
+import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
 import com.aelitis.azureus.core.networkmanager.impl.udp.UDPNetworkManager;
+import com.aelitis.azureus.core.peermanager.utils.PeerClassifier;
 import com.aelitis.net.udp.uc.PRUDPPacket;
 import com.aelitis.net.udp.uc.PRUDPPacketHandler;
 import com.aelitis.net.udp.uc.PRUDPPacketHandlerException;
@@ -87,7 +90,22 @@ TRTrackerBTAnnouncerImpl
 	
     static{
 	  	PRUDPTrackerCodecs.registerCodecs();
-	}
+	  	COConfigurationManager.addAndFireParameterListeners(
+	  		new String[] {
+	  			"Tracker Client Min Announce Interval",
+	  			"Tracker Client Numwant Limit",
+	  			"Server Enable UDP"
+	  			},
+	  		new ParameterListener()
+	  		{
+	  			public void parameterChanged(final String parameterName)
+	  			{
+	  				userMinInterval = COConfigurationManager.getIntParameter("Tracker Client Min Announce Interval");
+	  				userMaxNumwant = COConfigurationManager.getIntParameter("Tracker Client Numwant Limit");
+	  				udpAnnounceEnabled = COConfigurationManager.getBooleanParameter("Server Enable UDP");
+	  			}	  		
+	  		});
+    }
 	
 	private static AEMonitor 	class_mon 			= new AEMonitor( "TRTrackerBTAnnouncer:class" );
 	private static Map			tracker_report_map	= new HashMap();
@@ -100,12 +118,16 @@ TRTrackerBTAnnouncerImpl
 	
 	protected int				tracker_state 			= TS_INITIALISED;
 	private String				tracker_status_str		= "";
-	private TRTrackerAnnouncerResponse	last_response			= null;
+	private TRTrackerAnnouncerResponseImpl	last_response			= null;
 	private long				last_update_time_secs;
 	private long				current_time_to_wait_secs;
 	private boolean				manual_control;
   
 	private long min_interval = 0;
+	private static int userMinInterval = 0;
+	private static int userMaxNumwant = 100;
+	private static boolean udpAnnounceEnabled = true;
+	
   
 	private int  failure_added_time = 0;
     private long failure_time_last_updated = 0;
@@ -141,6 +163,11 @@ TRTrackerBTAnnouncerImpl
 	private static final int	   	key_id_length	= 8;
 	private int						key_udp;
 	
+	
+	
+	private int announceCount;
+	private byte autoUDPprobeEvery = 1;
+	
   
 	private String tracker_id = "";
   
@@ -196,7 +223,7 @@ TRTrackerBTAnnouncerImpl
 	try{
 	    tracker_peer_id = ClientIDManagerImpl.getSingleton().generatePeerID( torrent, true );
 	
-	    if ( COConfigurationManager.getBooleanParameter("Tracker Separate Peer IDs", false)){
+	    if ( COConfigurationManager.getBooleanParameter("Tracker Separate Peer IDs")){
 	    	
 	    	data_peer_id = ClientIDManagerImpl.getSingleton().generatePeerID( torrent, false );
 	    	
@@ -628,7 +655,7 @@ TRTrackerBTAnnouncerImpl
 			
 			tracker_status_str = MessageText.getString("PeerManager.status.checking") + "..."; //$NON-NLS-1$ //$NON-NLS-2$      
 		
-			TRTrackerAnnouncerResponse	response = null;
+			TRTrackerAnnouncerResponseImpl	response = null;
 			
 			if ( stopped ){
 				
@@ -716,8 +743,17 @@ TRTrackerBTAnnouncerImpl
 					tracker_state	= TS_INITIALISED;
 					
 				}else{
-	    	       	        	
-					tracker_status_str = MessageText.getString("PeerManager.status.ok"); //set the status      //$NON-NLS-1$
+	    	       	    
+					if ( announce_data_provider.isPeerSourceEnabled( PEPeerSource.PS_BT_TRACKER )){
+						
+						tracker_status_str = MessageText.getString("PeerManager.status.ok");
+
+					}else{
+						
+						tracker_status_str = MessageText.getString("PeerManager.status.ps_disabled");
+						
+						response.setPeers(new TRTrackerAnnouncerResponsePeerImpl[0]);
+					}
 				}
 
 				String	reason = response.getAdditionalInfo();
@@ -759,7 +795,7 @@ TRTrackerBTAnnouncerImpl
 		}
 	}
 	
-	protected TRTrackerAnnouncerResponse startSupport() {
+	protected TRTrackerAnnouncerResponseImpl startSupport() {
 		if (Logger.isEnabled())
 			Logger.log(new LogEvent(torrent, LOGID, "Tracker Announcer is sending "
 					+ "a start Request"));
@@ -767,7 +803,7 @@ TRTrackerBTAnnouncerImpl
     return (update("started"));
   }
 
-  protected TRTrackerAnnouncerResponse completeSupport() {
+  protected TRTrackerAnnouncerResponseImpl completeSupport() {
   	if (Logger.isEnabled())
 			Logger.log(new LogEvent(torrent, LOGID, "Tracker Announcer is sending "
 					+ "a completed Request"));
@@ -775,7 +811,7 @@ TRTrackerBTAnnouncerImpl
 		return (update("completed"));
   }
 
-  protected TRTrackerAnnouncerResponse stopSupport() {
+  protected TRTrackerAnnouncerResponseImpl stopSupport() {
   	if (Logger.isEnabled())
 			Logger.log(new LogEvent(torrent, LOGID, "Tracker Announcer is sending "
 					+ "a stopped Request"));
@@ -783,7 +819,7 @@ TRTrackerBTAnnouncerImpl
     return (update("stopped"));
   }
 
-  protected TRTrackerAnnouncerResponse updateSupport() {
+  protected TRTrackerAnnouncerResponseImpl updateSupport() {
   	if (Logger.isEnabled())
 			Logger.log(new LogEvent(torrent, LOGID, "Tracker Announcer is sending "
 					+ "an update Request"));
@@ -791,7 +827,7 @@ TRTrackerBTAnnouncerImpl
     return update("");
   }
   
-  	private TRTrackerAnnouncerResponse 
+  	private TRTrackerAnnouncerResponseImpl
 	update(
 		String evt )
   	{
@@ -1010,7 +1046,7 @@ TRTrackerBTAnnouncerImpl
 	  int	num_want = calculateNumWant() * 4;
 
 
-      TRTrackerAnnouncerResponsePeer[]	cached_peers = getPeersFromCache(num_want);
+      TRTrackerAnnouncerResponsePeer[]	cached_peers = getPeersFromCache(num_want);      
       
       if ( cached_peers.length > 0 ){
 
@@ -1029,8 +1065,7 @@ TRTrackerBTAnnouncerImpl
   
   		throws Exception
 	{
- 		
-   		// set context in case authentication dialog is required
+   			// set context in case authentication dialog is required
     	
     	TorrentUtils.setTLSTorrentHash( torrent_hash );
     	
@@ -1043,31 +1078,72 @@ TRTrackerBTAnnouncerImpl
 			try{  
 				String	protocol = reqUrl.getProtocol();
 				
-				if (Logger.isEnabled())
+				if (Logger.isEnabled()){
 					Logger.log(new LogEvent(torrent, LOGID,
 							"Tracker Announcer is Requesting: " + reqUrl));
-		  
-		  		ByteArrayOutputStream message = new ByteArrayOutputStream();
-		  				
-		  		if ( protocol.equalsIgnoreCase("udp")){
-		  			
-		  			failure_reason = announceUDP( reqUrl, message );
-		  			
-		  		}else{
-		  			
-		  			failure_reason = announceHTTP( tracker_url, reqUrl, message );
-		  			
-		  		}
-					// if we've got some kind of response then return it
-				
-				if ( message.size() > 0 ){
-			
-					return( message.toByteArray());
-					
 				}
+				
+		  		ByteArrayOutputStream message = new ByteArrayOutputStream();
+		  		
+		  		
+		  		URL udpAnnounceURL = null;
+		  		
+		  		boolean	udp_probe = false;
+		  		
+		  		if (protocol.equalsIgnoreCase("udp") && udpAnnounceEnabled)
+				{
+					udpAnnounceURL = reqUrl;
+				} else if (protocol.equalsIgnoreCase("http") && !az_tracker
+					&& announceCount % autoUDPprobeEvery == 0 && udpAnnounceEnabled)
+				{
+					// if we don't know this tracker supports UDP then don't probe on
+					// first announce as we don't want a large delay on torrent startup
+					// also if we are stopping we don't want to initiate a probe as
+					// we want the stop instruction to get to tracker if possible
+					if ((stopped || announceCount == 0) && !TRTrackerUtils.isUDPProbeOK(reqUrl))
+					{
+						// skip probe
+					} else
+					{
+						udpAnnounceURL = new URL(reqUrl.toString().replaceFirst("^http", "udp"));
+						udp_probe = true;
+					}
+				}
+		  				
+		  		if (udpAnnounceURL != null)
+				{
+					failure_reason = announceUDP(reqUrl, message);
+					if ((failure_reason != null || message.size() == 0) && udp_probe)
+					{
+						// automatic UDP probe failed, use HTTP again
+						udpAnnounceURL = null;
+						if (autoUDPprobeEvery < 16)
+							autoUDPprobeEvery <<= 1;
+						else // unregister in case the tracker somehow changed its capabilities
+							TRTrackerUtils.setUDPProbeResult(reqUrl, false);
+						if (Logger.isEnabled())
+							Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_INFORMATION, "redirection of http announce [" + tracker_url[0] + "] to udp failed, will retry in " + autoUDPprobeEvery + " announces"));
+						
+					} else if (failure_reason == null && udp_probe)
+					{
+						TRTrackerUtils.setUDPProbeResult(reqUrl, true);
+						if (Logger.isEnabled())
+							Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_INFORMATION, "redirection of http announce [" + tracker_url[0] + "] to udp successful"));
+						autoUDPprobeEvery = 1;
+					}
+				}
+		  		
+		  		announceCount++;
+		  		
+		  		if ( udpAnnounceURL == null)
+		  			failure_reason = announceHTTP( tracker_url, reqUrl, message );
+
+		  		// if we've got some kind of response then return it
+				if ( message.size() > 0 )
+					return( message.toByteArray());
+
 					
 				if ( failure_reason == null ){
-				
 					failure_reason = "No data received from tracker";
 				}
 				
@@ -1110,7 +1186,7 @@ TRTrackerBTAnnouncerImpl
 		
 			if (Logger.isEnabled())
 				Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR,
-						"Exception while processing the Tracker Request : "
+						"Exception while processing the Tracker Request for " + reqUrl + ": "
 								+ failure_reason));
 			
 			throw( new Exception( failure_reason));
@@ -1556,7 +1632,7 @@ TRTrackerBTAnnouncerImpl
 		 					
 		 				}
 		 			
-		 					failure_reason = ((PRUDPPacketReplyError)reply).getMessage();
+		 				failure_reason = ((PRUDPPacketReplyError)reply).getMessage();
 		 				
 		 			}else{
 		 				
@@ -1668,12 +1744,19 @@ TRTrackerBTAnnouncerImpl
   	request.append(info_hash);
   	request.append(tracker_peer_id_str);
   	
-	String	port_details = TRTrackerUtils.getPortsForURL();
+	String	port_details = 
+		announce_data_provider.getCryptoLevel()==NetworkManager.CRYPTO_OVERRIDE_REQUIRED?
+				TRTrackerUtils.getPortsForURLFullCrypto():
+				TRTrackerUtils.getPortsForURL();
 
   	request.append(port_details);
   	request.append("&uploaded=").append(announce_data_provider.getTotalSent());
   	request.append("&downloaded=").append(announce_data_provider.getTotalReceived());
   	request.append("&left=").append(announce_data_provider.getRemaining());
+  	
+  		// 3017: added at request of tracker admins who want to be able to monitor swarm poisoning
+  	
+  	request.append("&corrupt=").append(announce_data_provider.getFailedHashCheck());
 	
     
     //TrackerID extension
@@ -1700,7 +1783,7 @@ TRTrackerBTAnnouncerImpl
     	
       //calculate how many peers we should ask for
     	
-      int numwant = calculateNumWant();
+      int numwant = Math.min(calculateNumWant(),userMaxNumwant);
 
 
       request.append("&numwant=").append(numwant);
@@ -1855,6 +1938,13 @@ TRTrackerBTAnnouncerImpl
 			request.append( "&azup=" + up );
 		}
 		
+		String as = NetworkAdmin.getSingleton().getCurrentASN().getAS();
+		
+	    if ( as.length() > 0 ){
+	    	
+	    	request.append( "&azas=" + URLEncoder.encode( as, "UTF8" ));
+	    }
+	    
 		DHTNetworkPosition	best_position = DHTNetworkPositionManager.getBestLocalPosition();
 		
 		if ( best_position != null ){
@@ -1877,6 +1967,11 @@ TRTrackerBTAnnouncerImpl
   protected int
   calculateNumWant()
   {
+	if ( !announce_data_provider.isPeerSourceEnabled( PEPeerSource.PS_BT_TRACKER )){
+		
+		return( 0 );
+	}
+
     int MAX_PEERS = 100;
     
     int maxAllowed = announce_data_provider.getMaxNewConnectionsAllowed();
@@ -1989,7 +2084,7 @@ TRTrackerBTAnnouncerImpl
 		boolean	shuffle )
 	{
 		try{
-			trackerUrlLists = new ArrayList();
+			trackerUrlLists = new ArrayList(1);
 	  
 				//This entry is present on multi-tracker torrents
 	  
@@ -2088,7 +2183,34 @@ TRTrackerBTAnnouncerImpl
 					   //parse the metadata
 				
 	 			try{
-	 				Map metaData = BDecoder.decode(data); //$NON-NLS-1$
+	 				Map metaData = BDecoder.decode(data);
+	 				
+ 					// obey any peers source restrictions
+	 				
+	 				Object o = metaData.get( "az_ps" );
+
+	 				if ( o instanceof List ){
+
+	 					List peer_sources = (List)o;
+
+	 					List	x = new ArrayList();
+	 					
+	 					for (int i=0;i<peer_sources.size();i++){
+	 						
+	 						Object o1 = peer_sources.get(i);
+	 						
+	 						if ( o1 instanceof byte[] ){
+	 							
+	 							x.add( new String((byte[])o1));
+	 						}
+	 					}
+	 					
+	 					String[]	y = new String[x.size()];
+	 					
+	 					x.toArray( y );
+	 					
+	 					announce_data_provider.setPeerSources( y );
+	 				}
 						
 	 					// handle any user warnings in the response
 					
@@ -2128,8 +2250,8 @@ TRTrackerBTAnnouncerImpl
 								}
 								
 								if ( log_it ){
-		 							Logger.logTextResource(new LogAlert(LogAlert.UNREPEATABLE,
-											LogAlert.AT_WARNING,
+		 							Logger.logTextResource(new LogAlert(torrent, 
+		 									LogAlert.UNREPEATABLE, LogAlert.AT_WARNING,
 											"TrackerClient.announce.warningmessage"), new String[] {
 											announce_data_provider.getName(), warning_message });
 		 						}
@@ -2144,7 +2266,7 @@ TRTrackerBTAnnouncerImpl
 										
 					try {
 						time_to_wait = ((Long) metaData.get("interval")).longValue();
-
+						
 						Long raw_min_interval = (Long) metaData.get("min interval");
 
 						if (Logger.isEnabled()) {
@@ -2160,7 +2282,7 @@ TRTrackerBTAnnouncerImpl
 
 						if (raw_min_interval != null) {
 							min_interval = raw_min_interval.longValue();
-
+							
 							// ignore useless values
 							// Note: Many trackers set min_interval and interval the same.
 							if (min_interval < 1) {
@@ -2189,6 +2311,18 @@ TRTrackerBTAnnouncerImpl
 								min_interval = 0;
 							}
 						}
+						
+						if(userMinInterval != 0)
+						{
+							time_to_wait = Math.max(userMinInterval, time_to_wait);
+							min_interval = Math.max(min_interval, userMinInterval);
+							if (Logger.isEnabled()) {
+								Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_INFORMATION,
+										"Overriding with user settings: 'interval' = " + time_to_wait
+												+ "; 'min interval' = " + min_interval));
+							}
+						}
+
 
 						// roll back 10 seconds to make sure we announce before the tracker
 						// times us out.  This is done after min_interval in order not to 
@@ -2287,7 +2421,19 @@ TRTrackerBTAnnouncerImpl
 	
 					    		if ( l_rtt != null ){
 					    			
-					    			total_rtt 	+= l_rtt.longValue();
+					    			long rtt = l_rtt.longValue();
+					    			
+					    			if ( rtt <= 0 ){
+					    				
+					    					// invalid, remove
+					    				
+					    				peer.remove( "r" );
+					    				
+					    			}else{
+					    				
+					    				total_rtt 	+= rtt;
+					    			}
+					    			
 					    			rtt_count++; 
 					    		}
 							}
@@ -2420,13 +2566,34 @@ TRTrackerBTAnnouncerImpl
 							try{
 								byte[]		ip_bytes = (byte[])peer.get("i");
 	
-					    		int	ip1 = 0xff & ip_bytes[0];
-					    		int	ip2 = 0xff & ip_bytes[1];
-					    		int	ip3 = 0xff & ip_bytes[2];
-					    		int	ip4 = 0xff & ip_bytes[3];
-	
-					    		String	ip 			= ip1 + "." + ip2 + "." + ip3 + "." + ip4;
-					    		
+								String	ip;
+								
+								if ( ip_bytes.length == 4 ){
+									
+						    		int	ip1 = 0xff & ip_bytes[0];
+						    		int	ip2 = 0xff & ip_bytes[1];
+						    		int	ip3 = 0xff & ip_bytes[2];
+						    		int	ip4 = 0xff & ip_bytes[3];
+		
+						    		ip 	= ip1 + "." + ip2 + "." + ip3 + "." + ip4;
+								}else{
+								
+									StringBuffer sb = new StringBuffer(39);
+									
+									for ( int j=0; j<16; j+=2 ){
+										
+										sb.append(	
+									    	Integer.toHexString(((ip_bytes[j]<<8) & 0xff00) | (ip_bytes[j+1]&0x00ff)));
+									    
+									    if (j < 14 ){
+									    	
+									       sb.append( ":" );
+									    }
+									}
+
+									ip = sb.toString();
+								}
+								
 					    		byte[]	tcp_bytes	= (byte[])peer.get("t");
 					    		
 					    		int		tcp_port 	= ((tcp_bytes[0]&0xff) << 8 ) + (tcp_bytes[1]&0xff );
@@ -2473,6 +2640,13 @@ TRTrackerBTAnnouncerImpl
 					    								    		
 					    		Long	l_up_speed = (Long)peer.get( "s" );
 					    		
+					    		boolean	biased = peer.containsKey("b");
+					    		
+					    		if ( biased ){
+					    			
+					    			PeerClassifier.setAzureusIP( ip );
+					    		}
+					    		
 								TRTrackerAnnouncerResponsePeerImpl new_peer = 
 									new TRTrackerAnnouncerResponsePeerImpl( 
 										PEPeerSource.PS_BT_TRACKER, 
@@ -2496,8 +2670,6 @@ TRTrackerBTAnnouncerImpl
 						    			extra = ",rtt=" + l_rtt;
 						    		}
 						    		
-						    		boolean	biased = peer.containsKey( "b" );
-
 						    		if ( biased ){
 						    			
 						    			extra += ",biased";
@@ -2716,10 +2888,48 @@ TRTrackerBTAnnouncerImpl
 
 				    		valid_meta_peers.add( peer );
 				    	}
-				    }else{
-						
+				    }else if ( meta_peers_peek instanceof Map ){
+
+				    		// some trackers incorrectly return an empty Map when no peers available
+				    	
+				    	if (((Map)meta_peers_peek).size() != 0 ){
+				    	
+							throw( new IOException( "peers missing from response" ));
+				    	}
+				    }else if (!metaData.containsKey("peers6")){ 
+				    	// we got nothing useful under peers and no peers6 either
 						throw( new IOException( "peers missing from response" ));
 				    }
+			    	
+			    	
+			    	final byte[] v6peers = (byte[])metaData.get("peers6");
+				    if (v6peers != null)
+					{
+				    	// 16 bytes for v6 + 2 bytes for port
+						final int entry_size = 18;
+
+						
+						final byte[] rawAddr = new byte[16]; 
+						
+						for (int i = 0; i <= v6peers.length - entry_size; i += entry_size)
+						{
+							System.arraycopy(v6peers, i, rawAddr, 0, 16);
+							String ip = InetAddress.getByAddress(rawAddr).getHostAddress();
+							int tcp_port = rawAddr[i+16] * 256 + rawAddr[i+17];
+							if (tcp_port < 0 || tcp_port > 65535)
+							{
+								if (Logger.isEnabled())
+									Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR, "Invalid compactv6 peer port given: " + ip + ": " + tcp_port));
+								continue;
+							}
+							byte[] peer_peer_id = getAnonymousPeerId(ip, tcp_port);
+							short protocol = DownloadAnnounceResultPeer.PROTOCOL_NORMAL;
+							TRTrackerAnnouncerResponsePeerImpl peer = new TRTrackerAnnouncerResponsePeerImpl(PEPeerSource.PS_BT_TRACKER, peer_peer_id, ip, tcp_port, 0, 0, protocol, TRTrackerAnnouncer.AZ_TRACKER_VERSION_1, (short) 0);
+							if (Logger.isEnabled())
+								Logger.log(new LogEvent(torrent, LOGID, "COMPACTv6 PEER: " + peer.getString()));
+							valid_meta_peers.add(peer);
+						}
+					}
 				    
 				    
 					TRTrackerAnnouncerResponsePeerImpl[] peers=new TRTrackerAnnouncerResponsePeerImpl[valid_meta_peers.size()];
@@ -2781,6 +2991,8 @@ TRTrackerBTAnnouncerImpl
 															+ " == " + incomplete + ". " : "") }));
 						} else {
 
+							resp.setScrapeResult( complete, incomplete );
+
 							TRTrackerScraper scraper = TRTrackerScraperFactory.getSingleton();
 
 							if (scraper != null) {
@@ -2812,17 +3024,21 @@ TRTrackerBTAnnouncerImpl
 						// decode could fail if the tracker's returned, say, an HTTP response
 						// indicating server overload
 	 				 				
-	 				String	trace_data = new String(data);
+	 				String	trace_data;
+	 				
+	 				if ( data.length <= 150 ){
+	 					
+	 					trace_data = new String(data);
+	 					
+	 				}else{
+	 					
+	 					trace_data = new String(data,0,150) + "...";
+	 				}
 	 				
 	 				if (Logger.isEnabled())
 						Logger.log(new LogEvent(torrent, LOGID, LogEvent.LT_ERROR,
 								"TRTrackerAnnouncer::invalid reply: " + trace_data));
 	 				
-	 				if ( trace_data.length() > 150 ){
-	 					
-	 					trace_data = trace_data.substring(0,150) + "...";
-	 				}
-	 				
 	 				failure_reason = "invalid reply: " + trace_data;
 	 			}	 				
 	 		}catch( Throwable e ){
@@ -2989,36 +3205,70 @@ TRTrackerBTAnnouncerImpl
 		}else{
 			DownloadAnnounceResultPeer[]	ext_peers = result.getPeers();
 			
-			TRTrackerAnnouncerResponsePeerImpl[] peers = new TRTrackerAnnouncerResponsePeerImpl[ext_peers.length];
+			List	l_peers = new ArrayList(ext_peers.length);
 				
+			boolean ps_enabled = announce_data_provider.isPeerSourceEnabled( PEPeerSource.PS_BT_TRACKER );
+						
 			for (int i=0;i<ext_peers.length;i++){
 				
 				DownloadAnnounceResultPeer	ext_peer = ext_peers[i];
 				
-				int		http_port	= 0;
-				byte	az_version 	= TRTrackerAnnouncer.AZ_TRACKER_VERSION_1;
+				String	ps = ext_peer.getSource();
 				
-				peers[i] = new TRTrackerAnnouncerResponsePeerImpl( 
-								ext_peer.getSource(),
-								ext_peer.getPeerID(),
-								ext_peer.getAddress(), 
-								ext_peer.getPort(),
-								ext_peer.getUDPPort(),
-								http_port,
-								ext_peer.getProtocol(),
-								az_version,
-								(short)0 );
+					// filter out any disabled peers
 				
-				if (Logger.isEnabled())
-					Logger.log(new LogEvent(torrent, LOGID, "EXTERNAL PEER: " + peers[i].getString())); 
-
+				if ( !ps_enabled && ps.equals( PEPeerSource.PS_BT_TRACKER )){
+										
+					continue;
+					
+				}else{
+					
+					int		http_port	= 0;
+					byte	az_version 	= TRTrackerAnnouncer.AZ_TRACKER_VERSION_1;
+					
+					TRTrackerAnnouncerResponsePeerImpl p = 
+						new TRTrackerAnnouncerResponsePeerImpl( 
+									ext_peer.getSource(),
+									ext_peer.getPeerID(),
+									ext_peer.getAddress(), 
+									ext_peer.getPort(),
+									ext_peer.getUDPPort(),
+									http_port,
+									ext_peer.getProtocol(),
+									az_version,
+									(short)0 );
+					
+					l_peers.add( p );
+				
+					if (Logger.isEnabled())
+						Logger.log(new LogEvent(torrent, LOGID, "EXTERNAL PEER: " + p.getString())); 
+				}
 			}
 			
+			TRTrackerAnnouncerResponsePeerImpl[] peers = new TRTrackerAnnouncerResponsePeerImpl[ l_peers.size()];
+
+			l_peers.toArray( peers );
+			
 			addToTrackerCache( peers);
 		
-			status = MessageText.getString("PeerManager.status.ok");
+			if ( ps_enabled || peers.length > 0 || ext_peers.length == 0 ){
+				
+				status = MessageText.getString("PeerManager.status.ok");
 
-			response = new TRTrackerAnnouncerResponseImpl( result.getURL(), torrent_hash, TRTrackerAnnouncerResponse.ST_ONLINE, result.getTimeToWait(), peers );
+			}else{
+				
+				status = MessageText.getString("PeerManager.status.ps_disabled");
+				
+				peers = new TRTrackerAnnouncerResponsePeerImpl[0];
+			}
+			
+			response = 
+				new TRTrackerAnnouncerResponseImpl( 
+						result.getURL(), 
+						torrent_hash, 
+						TRTrackerAnnouncerResponse.ST_ONLINE, 
+						result.getTimeToWait(), 
+						peers );
 		}
 		
 			// only make the user aware of the status if the underlying announce is
@@ -3032,4 +3282,40 @@ TRTrackerBTAnnouncerImpl
 		
 		listeners.dispatch( LDT_TRACKER_RESPONSE, response );
 	}
+	
+	public void 
+	generateEvidence(
+		IndentWriter writer )
+	{
+		writer.println( "BT announce:" );
+		
+		try{
+			writer.indent();
+			
+			writer.println( "state: " + tracker_state + ", in_progress=" + update_in_progress );
+			
+			writer.println( "current: " + (lastUsedUrl==null?"null":lastUsedUrl.toString()));
+			
+			writer.println( "last: " + (last_response==null?"null":last_response.getString()));
+			
+			writer.println( "last_update_secs: " + last_update_time_secs );
+			
+			writer.println( "secs_to_wait: " + current_time_to_wait_secs  + (manual_control?" - manual":""));
+			
+			writer.println( "min_interval: " + min_interval );
+			
+			writer.println( "min_interval_override: " + min_interval_override );
+			
+			writer.println( "rd: last_override=" + rd_last_override + ",percentage=" + rd_override_percentage );
+			
+			writer.println( "event: " + ( current_timer_event==null?"null":current_timer_event.getString()));
+			
+			writer.println( "stopped: " + stopped + ", for_q=" + stopped_for_queue );
+			
+			writer.println( "complete: " + completed + ", reported=" + complete_reported );
+			
+		}finally{
+			writer.exdent();
+		}
+	}
 }
\ No newline at end of file
