@@ -36,6 +36,7 @@ import java.util.zip.GZIPInputStream;
 import javax.net.ssl.*;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.logging.*;
 import org.gudy.azureus2.core3.security.SESecurityManager;
@@ -72,29 +73,38 @@ public class TrackerStatus {
   private final static String SS = "Scrape.status.";
   private final static String SSErr = "Scrape.status.error.";
 
-  private final static int FAULTY_SCRAPE_RETRY_INTERVAL = 60 * 10 * 1000;
-  private final static int NOHASH_RETRY_INTERVAL = 1000 * 60 * 60 * 3; // 3 hrs
-  
-  /**
-   * When scraping a single hash, also scrape other hashes that are going to
-   * be scraped within this range.
-   */
-  private final static int GROUP_SCRAPES_MS 	= 60 * 15 * 1000;
-  private final static int GROUP_SCRAPES_LIMIT	= 20;
-
-  
-  static{
-  	PRUDPTrackerCodecs.registerCodecs();
-  }
-  
-  private static List	logged_invalid_urls	= new ArrayList();
-  
-  private static ThreadPool	thread_pool = new ThreadPool( "TrackerStatus", 10, true );	// queue when full rather than block
-  
-  private final URL		tracker_url;
-  private boolean		az_tracker;
-  
-  private String 	scrapeURL = null;
+  	private final static int		FAULTY_SCRAPE_RETRY_INTERVAL	= 60 * 10 * 1000;
+	private final static int		NOHASH_RETRY_INTERVAL			= 1000 * 60 * 60 * 3;							// 3 hrs
+	
+	/**
+	 * When scraping a single hash, also scrape other hashes that are going to
+	 * be scraped within this range.
+	 */
+	private final static int		GROUP_SCRAPES_MS				= 60 * 15 * 1000;
+	private final static int		GROUP_SCRAPES_LIMIT				= 20;
+	
+	static
+	{
+		PRUDPTrackerCodecs.registerCodecs();
+	  	COConfigurationManager.addAndFireParameterListener("Server Enable UDP", new ParameterListener()
+		{
+			public void parameterChanged(final String parameterName) {
+				udpScrapeEnabled = COConfigurationManager.getBooleanParameter("Server Enable UDP");
+			}
+		});
+	}
+	
+	private static boolean udpScrapeEnabled = true; 
+	
+	private byte					autoUDPscrapeEvery				= 1;
+	private int						scrapeCount;
+	
+	private static List				logged_invalid_urls				= new ArrayList();
+	private static ThreadPool		thread_pool						= new ThreadPool("TrackerStatus", 10, true);	// queue when full rather than block
+	
+	private final URL				tracker_url;
+	private boolean					az_tracker;
+	private String					scrapeURL						= null;
  
   /** key = Torrent hash.  values = TRTrackerScraperResponseImpl */
   private HashMap 					hashes;
@@ -218,15 +228,16 @@ public class TrackerStatus {
    		
 	    		response = (TRTrackerScraperResponseImpl)hashes.get( hash );
 		    
-	    		if (response == null) {
-	    			
-	    			response = addHash(hash);
-	    		}
 	    	}finally{
 	    	
 	    		hashes_mon.exit();
 	    	}
 	
+    		if ( response == null ){
+    			
+    			response = addHash(hash);
+    		}
+    		
 	    	long lMainNextScrapeStartTime = response.getNextScrapeStartTime();
 	
 	    	if( !force && lMainNextScrapeStartTime > SystemTime.getCurrentTime() ) {
@@ -350,6 +361,8 @@ public class TrackerStatus {
 			boolean disable_stopped_scrapes = !COConfigurationManager
 					.getBooleanParameter("Tracker Client Scrape Stopped Enable");
 
+			byte[]	scrape_reply = null;
+			
 			try {
 				// if URL already includes a query component then just append our
 				// params
@@ -363,6 +376,8 @@ public class TrackerStatus {
 
 				String flags = "";
 				
+				List hashesForUDP = new ArrayList();
+				
 				for (int i = 0; i < responses.size(); i++) {
 					TRTrackerScraperResponseImpl response = (TRTrackerScraperResponseImpl) responses.get(i);
 
@@ -433,6 +448,10 @@ public class TrackerStatus {
 						
 						one_of_the_responses = response;
 						one_of_the_hashes = hash;
+						
+						// 28 + 16 + 70*20 -> IPv4/udp packet size of 1444 , that should go through most lines unfragmented
+						if(hashesForUDP.size() < 70)
+							hashesForUDP.add(hash);
 					}
 				} // for responses
 
@@ -465,20 +484,46 @@ public class TrackerStatus {
 
 				URL	redirect_url = null;
 				
-				if (reqUrl.getProtocol().equalsIgnoreCase("udp")) {
-
-					// TODO: support multi hash scrapes on UDP
-
-					scrapeUDP(reqUrl, message, one_of_the_hashes, one_of_the_responses);
-
-					bSingleHashScrapes = true;
-
-				} else {
-					
-					redirect_url = scrapeHTTP(reqUrl, message);
-				}
-
-				Map map = BDecoder.decode(message.toByteArray());
+				String protocol = reqUrl.getProtocol();
+				
+		  		URL udpScrapeURL = null;
+		  		
+		  		if(protocol.equalsIgnoreCase("udp") && udpScrapeEnabled)
+		  			udpScrapeURL = reqUrl;
+		  		else if(protocol.equalsIgnoreCase("http") && !az_tracker && scrapeCount % autoUDPscrapeEvery == 0 && udpScrapeEnabled)
+		  			udpScrapeURL = new URL(reqUrl.toString().replaceFirst("^http", "udp"));
+		  			
+		  		
+		  		if ( udpScrapeURL != null){
+		  			
+		  			boolean success = scrapeUDP( reqUrl, message, hashesForUDP );
+		  			
+		  			if((!success || message.size() == 0) && !protocol.equalsIgnoreCase("udp"))
+		  			{ // automatic UDP probe failed, use HTTP again
+		  				udpScrapeURL = null;
+		  				message.reset();
+		  				if(autoUDPscrapeEvery < 16)
+		  					autoUDPscrapeEvery <<= 1;
+						if (Logger.isEnabled())
+							Logger.log(new LogEvent(LOGID, LogEvent.LT_INFORMATION, "redirection of http scrape ["+scrapeURL+"] to udp failed, will retry in "+autoUDPscrapeEvery+" scrapes"));
+		  			} else if(success && !protocol.equalsIgnoreCase("udp"))
+		  			{
+						if (Logger.isEnabled())
+							Logger.log(new LogEvent(LOGID, LogEvent.LT_INFORMATION, "redirection of http scrape ["+scrapeURL+"] to udp successful"));
+		  				autoUDPscrapeEvery = 1;
+		  				TRTrackerUtils.setUDPProbeResult( reqUrl, true );
+		  			}
+		  				
+		  		}
+		  		
+		  		scrapeCount++;
+		  		
+		  		if(udpScrapeURL == null)
+		  			redirect_url = scrapeHTTP(reqUrl, message);
+				
+				scrape_reply = message.toByteArray();
+				
+				Map map = BDecoder.decode( scrape_reply );
 								
 				boolean	this_is_az_tracker = map.get( "aztracker" ) != null;
 				
@@ -489,7 +534,7 @@ public class TrackerStatus {
 					TRTrackerUtils.setAZTracker( tracker_url, az_tracker );
 				}
 				
-				Map mapFiles = map == null ? null : (Map) map.get("files");
+				Map mapFiles = (Map) map.get("files");
 
 				if (Logger.isEnabled())
 					Logger.log(new LogEvent(LOGID, "Response from scrape interface "
@@ -520,8 +565,9 @@ public class TrackerStatus {
 								.get("min_request_interval");
 						if (longScrapeValue != null)
 							iMinRequestInterval = longScrapeValue.intValue();
-						// Tracker owners wamt this log entry
-						Logger.log(new LogEvent(LOGID,
+						// Tracker owners want this log entry
+						if (Logger.isEnabled())
+							Logger.log(new LogEvent(LOGID,
 								"Received min_request_interval of " + iMinRequestInterval));
 					}
 				}
@@ -683,9 +729,11 @@ public class TrackerStatus {
 						// retrieve values
 						int seeds = ((Long) scrapeMap.get("complete")).intValue();
 						int peers = ((Long) scrapeMap.get("incomplete")).intValue();
+						Long comp = (Long) scrapeMap.get("downloaded");
+						int completed = comp == null ? -1 : comp.intValue();
 
 						// make sure we dont use invalid replies
-						if (seeds < 0 || peers < 0) {
+						if (seeds < 0 || peers < 0 || completed < -1) {
 							if (Logger.isEnabled()) {
 								HashWrapper hash = response.getHash();
 								Logger.log(new LogEvent(TorrentUtils.getDownloadManager(hash),
@@ -719,7 +767,10 @@ public class TrackerStatus {
 													+ " == " + seeds + ". " : "")
 											+ (peers < 0 ? MessageText
 													.getString("MyTorrentsView.peers")
-													+ " == " + peers + ". " : ""));
+													+ " == " + peers + ". " : "")
+											+ (completed < 0 ? MessageText
+													.getString("MyTorrentsView.completed")
+													+ " == " + completed + ". " : ""));
 
 							scraper.scrapeReceived(response);
 
@@ -737,6 +788,7 @@ public class TrackerStatus {
 						response.setScrapeStartTime(scrapeStartTime);
 						response.setSeeds(seeds);
 						response.setPeers(peers);
+						response.setCompleted(completed);
 						response.setStatus(TRTrackerScraperResponse.ST_ONLINE,
 								MessageText.getString(SS + "ok"));
 
@@ -801,6 +853,8 @@ public class TrackerStatus {
 				setAllError(e);
 			} catch (UnknownHostException e) {
 				setAllError(e);
+			} catch (BEncodingException e) {
+				setAllError(e);
 			} catch (Exception e) {
 				
 				// for apache we can get error 414 - URL too long. simplest solution
@@ -831,6 +885,22 @@ public class TrackerStatus {
 
 				String msg = Debug.getNestedExceptionMessage(e);
 
+				if ( scrape_reply != null ){
+
+	 				String	trace_data;
+	 				
+	 				if ( scrape_reply.length <= 150 ){
+	 					
+	 					trace_data = new String(scrape_reply);
+	 					
+	 				}else{
+	 					
+	 					trace_data = new String(scrape_reply,0,150) + "...";
+	 				}
+	 				
+	 				msg += " [" + trace_data + "]";
+				}
+				
 				for (int i = 0; i < responses.size(); i++) {
 					TRTrackerScraperResponseImpl response = (TRTrackerScraperResponseImpl) responses
 							.get(i);
@@ -873,6 +943,11 @@ public class TrackerStatus {
 		}
 
 		String msg = e.getLocalizedMessage();
+		
+		if(e instanceof BEncodingException)
+			if(msg.indexOf("html") != -1)
+				msg = "could not decode response, appears to be a website instead of tracker scrape: "+msg.replace('\n', ' ');
+			else msg = "bencoded response malformed:"+msg;
 
 		for (int i = 0; i < values.length; i++) {
 			TRTrackerScraperResponseImpl response = (TRTrackerScraperResponseImpl) values[i];
@@ -887,8 +962,8 @@ public class TrackerStatus {
 
 			response.setNextScrapeStartTime(SystemTime.getCurrentTime()
 					+ FAULTY_SCRAPE_RETRY_INTERVAL);
-			response.setStatus(TRTrackerScraperResponse.ST_ERROR,
-					MessageText.getString(SS + "error") + msg + " (IO)");
+			response.setStatus(TRTrackerScraperResponse.ST_ERROR,StringInterner.intern(
+					MessageText.getString(SS + "error") + msg + " (IO)"));
 			// notifiy listeners
 			scraper.scrapeReceived(response);
 		}
@@ -1059,54 +1134,43 @@ public class TrackerStatus {
 	  return( redirect_url );
   }
   
-  protected void
-  scrapeUDP(
-  	URL								reqUrl,
-	ByteArrayOutputStream			message,
-	HashWrapper						hash,
-	TRTrackerScraperResponseImpl	current_response )
-  
-  		throws Exception
-  {
-  		/* reduce network traffic by only scraping UDP when the torrent isn't running as
-  		 * UDP version 2 contains scrape data in the announce response
-  		 */
+  protected boolean scrapeUDP(URL reqUrl, ByteArrayOutputStream message, List hashes) throws Exception {
+		Map rootMap = new HashMap();
+		Map files = new ByteEncodedKeyHashMap();
+		rootMap.put("files", files);
+		
+		
+		/*
+		 * reduce network traffic by only scraping UDP when the torrent isn't
+		 * running as UDP version 2 contains scrape data in the announce
+		 * response
+		 */
+		
+		/* removed implementation for the time being
+		 
+		for (Iterator it = hashes.iterator(); it.hasNext();)
+		{
+			HashWrapper hash = (HashWrapper) it.next();
+			if (PRUDPPacketTracker.VERSION == 2 && scraper.isTorrentDownloading(hash))
+			{
+				if (Logger.isEnabled())
+					Logger.log(new LogEvent(TorrentUtils.getDownloadManager(hash), LOGID, LogEvent.LT_WARNING, "Scrape of " + reqUrl + " skipped as torrent running and " + "therefore scrape data available in " + "announce replies"));
+				// easiest approach here is to brew up a response that looks like the current one
+				Map file = new HashMap();
+				byte[] resp_hash = hash.getBytes();
+				// System.out.println("got hash:" + ByteFormatter.nicePrint( resp_hash, true ));
+				files.put(new String(resp_hash, Constants.BYTE_ENCODING), file);
+				file.put("complete", new Long(current_response.getSeeds()));
+				file.put("downloaded", new Long(-1)); // unknown
+				file.put("incomplete", new Long(current_response.getPeers()));
+				byte[] data = BEncoder.encode(rootMap);
+				message.write(data);
+				return true;
+			}
+		}
+		
+		*/
   	
-  	if ( 	PRUDPPacketTracker.VERSION == 2 &&
-  			scraper.isTorrentDownloading( hash )){
-  	
-  		if (Logger.isEnabled())
-				Logger.log(new LogEvent(TorrentUtils.getDownloadManager(hash), LOGID,
-						LogEvent.LT_WARNING, "Scrape of " + reqUrl
-								+ " skipped as torrent running and "
-								+ "therefore scrape data available in " + "announce replies"));
-
-			// easiest approach here is to brew up a response that looks like the current one
-		
-		Map	map = new HashMap();
-
-		Map	files = new ByteEncodedKeyHashMap();
-		
-		map.put( "files", files );
-									
-		Map	file = new HashMap();
-			
-		byte[]	resp_hash = hash.getBytes();
-		
-		// System.out.println("got hash:" + ByteFormatter.nicePrint( resp_hash, true ));
-	
-		files.put( new String(resp_hash, Constants.BYTE_ENCODING), file );
-		
-		file.put( "complete", new Long( current_response.getSeeds()));
-		file.put( "downloaded", new Long(-1));	// unknown
-		file.put( "incomplete", new Long(current_response.getPeers()));
-		
-		byte[] data = BEncoder.encode( map );
-		
-		message.write( data );
-		
-  		return;
-  	}
   	
 	reqUrl = TRTrackerUtils.adjustURLForHosting( reqUrl );
 
@@ -1140,7 +1204,7 @@ public class TrackerStatus {
 					
 					long	my_connection = connect_reply.getConnectionId();
 					
-					PRUDPPacketRequestScrape scrape_request = new PRUDPPacketRequestScrape( my_connection, hash.getBytes() );
+					PRUDPPacketRequestScrape scrape_request = new PRUDPPacketRequestScrape( my_connection, hashes );
 									
 					reply = handler.sendAndReceive( auth, scrape_request, destination );
 					
@@ -1151,8 +1215,6 @@ public class TrackerStatus {
 						if ( PRUDPPacketTracker.VERSION == 1 ){
 							PRUDPPacketReplyScrape	scrape_reply = (PRUDPPacketReplyScrape)reply;
 							
-							Map	map = new HashMap();
-							
 							/*
 							int	interval = scrape_reply.getInterval();
 							
@@ -1167,10 +1229,7 @@ public class TrackerStatus {
 							int[]		downloaded 		= scrape_reply.getDownloaded();
 							int[]		incomplete 		= scrape_reply.getIncomplete();
 							
-							Map	files = new ByteEncodedKeyHashMap();
-							
-							map.put( "files", files );
-							
+					
 							for (int i=0;i<reply_hashes.length;i++){
 								
 								Map	file = new HashMap();
@@ -1186,15 +1245,14 @@ public class TrackerStatus {
 								file.put( "incomplete", new Long(incomplete[i]));
 							}
 							
-							byte[] data = BEncoder.encode( map );
+							byte[] data = BEncoder.encode( rootMap );
 							
 							message.write( data );
 							
-							return;
+							return true;
 						}else{
 							PRUDPPacketReplyScrape2	scrape_reply = (PRUDPPacketReplyScrape2)reply;
 							
-							Map	map = new HashMap();
 							
 							/*
 							int	interval = scrape_reply.getInterval();
@@ -1209,36 +1267,32 @@ public class TrackerStatus {
 							int[]		downloaded 	= scrape_reply.getDownloaded();
 							int[]		incomplete 	= scrape_reply.getIncomplete();
 							
-							Map	files = new ByteEncodedKeyHashMap();
-							
-							map.put( "files", files );
-														
-							Map	file = new HashMap();
-								
-							byte[]	resp_hash = hash.getBytes();
+							int i=0;
+							for(Iterator it = hashes.iterator();it.hasNext() && i < complete.length;i++)
+							{
+								HashWrapper hash = (HashWrapper)it.next();
+								Map file = new HashMap();
+								file.put( "complete", new Long(complete[i]));
+								file.put( "downloaded", new Long(downloaded[i]));
+								file.put( "incomplete", new Long(incomplete[i]));
+								files.put( new String(hash.getBytes(), Constants.BYTE_ENCODING), file );
+							}
 							
 							// System.out.println("got hash:" + ByteFormatter.nicePrint( resp_hash, true ));
-						
-							files.put( new String(resp_hash, Constants.BYTE_ENCODING), file );
 							
-							file.put( "complete", new Long(complete[0]));
-							file.put( "downloaded", new Long(downloaded[0]));
-							file.put( "incomplete", new Long(incomplete[0]));
-							
-							byte[] data = BEncoder.encode( map );
+							byte[] data = BEncoder.encode( rootMap );
 							
 							message.write( data );
 							
-							return;
+							return true;
 						}
 					}else{
 						
 						failure_reason = ((PRUDPPacketReplyError)reply).getMessage();
 						
 						if (Logger.isEnabled())
-								Logger.log(new LogEvent(TorrentUtils.getDownloadManager(hash),
-										LOGID, LogEvent.LT_ERROR,
-										"Response from scrape interface : " + failure_reason));
+								Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR,
+										"Response from scrape interface "+ reqUrl +" : " + failure_reason));
 						
 						break;
 					}
@@ -1247,8 +1301,7 @@ public class TrackerStatus {
 					failure_reason = ((PRUDPPacketReplyError)reply).getMessage();
 					
 					if (Logger.isEnabled())
-							Logger.log(new LogEvent(TorrentUtils.getDownloadManager(hash), LOGID,
-									LogEvent.LT_ERROR, "Response from scrape interface : "
+							Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR, "Response from scrape interface "+ reqUrl +" : "
 											+ ((PRUDPPacketReplyError) reply).getMessage()));
 				
 					break;
@@ -1267,15 +1320,14 @@ public class TrackerStatus {
 		
 		if ( failure_reason != null ){
 			
-			Map	map = new HashMap();
-			
-			map.put( "failure reason", failure_reason.getBytes());
-			
-			byte[] data = BEncoder.encode( map );
+			rootMap.put( "failure reason", failure_reason.getBytes());
+			rootMap.remove("files");
 			
+			byte[] data = BEncoder.encode( rootMap );
 			message.write( data );
-
 		}
+		
+		return false;
 	}finally{
 		if ( auth != null ){
 			
@@ -1308,28 +1360,40 @@ public class TrackerStatus {
   
 
   protected TRTrackerScraperResponseImpl addHash(HashWrapper hash) {
-    TRTrackerScraperResponseImpl response = new TRTrackerBTScraperResponseImpl(this, hash);
-    if (scrapeURL == null)  {
-      response.setStatus(TRTrackerScraperResponse.ST_ERROR,
-                         MessageText.getString(SS + "error") + 
-                         MessageText.getString(SSErr + "badURL"));
-    } else {
-      response.setStatus(TRTrackerScraperResponse.ST_INITIALIZING,
-                         MessageText.getString(SS + "initializing"));
-    }
-    
-    response.setNextScrapeStartTime(checker.getNextScrapeCheckOn());
+ 
+	TRTrackerScraperResponseImpl response;
+	  
   	try{
   		hashes_mon.enter();
   	
-  		hashes.put( hash, response);
-      
+  		response = (TRTrackerScraperResponseImpl)hashes.get( hash );
+  		
+  		if ( response == null ){
+  			
+	  	    response = new TRTrackerBTScraperResponseImpl(this, hash);
+	  	    
+	  	    if ( scrapeURL == null ){
+	  	    	
+	  	      response.setStatus(TRTrackerScraperResponse.ST_ERROR,
+	  	                         MessageText.getString(SS + "error") + 
+	  	                         MessageText.getString(SSErr + "badURL"));
+	  	    }else{
+	  	    	
+	  	      response.setStatus(TRTrackerScraperResponse.ST_INITIALIZING,
+	  	                         MessageText.getString(SS + "initializing"));
+	  	    }
+	  	    
+	  	    response.setNextScrapeStartTime(checker.getNextScrapeCheckOn());
+	  	    
+	  		hashes.put( hash, response);
+  		}
   	}finally{
   		
   		hashes_mon.exit();
   	}
 
-    //notifiy listeners
+  		//notifiy listeners
+  	
     scraper.scrapeReceived( response );
 
   	return response;
