@@ -334,7 +334,7 @@ TRHostExternalTorrent
 		String		name,
 		Object		value )
 	{
-		if ( name instanceof String ){
+		if ( value instanceof String ){
 			
 			setAdditionalStringProperty(name,(String)value);
 			
