@@ -421,7 +421,7 @@ TRHostImpl
 					protocol = TRTrackerServerFactory.PR_DHT;
 				}
 				
-				boolean force_external = COConfigurationManager.getBooleanParameter("Tracker Port Force External", false );
+				boolean force_external = COConfigurationManager.getBooleanParameter("Tracker Port Force External");
 				
 				port = announce_url.getPort();
 				
@@ -911,6 +911,7 @@ TRHostImpl
 		
 	public boolean
 	permitted(
+		String		originator,
 		byte[]		hash,
 		boolean		explicit  )
 	{
@@ -963,7 +964,7 @@ TRHostImpl
 			int port = COConfigurationManager.getIntParameter("Tracker Port", TRHost.DEFAULT_PORT );
 	
 			try{
-				TOTorrent	external_torrent = new TRHostExternalTorrent(hash, new URL( "http://" + tracker_ip + ":" + port + "/announce"));
+				TOTorrent	external_torrent = new TRHostExternalTorrent(hash, new URL( "http://" + UrlUtils.convertIPV6Host(tracker_ip) + ":" + port + "/announce"));
 			
 				addTorrent( external_torrent, state, true, false, date_added );	
 				
@@ -1071,6 +1072,13 @@ TRHostImpl
 		server.addRequestListener(this);
 	}
 	
+	public void
+	serverDestroyed(
+		TRTrackerServer	server )
+	{
+		server.removeRequestListener(this);
+	}
+	
 	public void
 	preProcess(
 		TRTrackerServerRequest	request )
