@@ -40,11 +40,11 @@ public abstract class
 TRTrackerServerImpl 
 	implements 	TRTrackerServer
 {
-	public static  int RETRY_MINIMUM_SECS			= 60;
-	public static  int RETRY_MINIMUM_MILLIS			= RETRY_MINIMUM_SECS*1000;
-	public static  int CLIENT_TIMEOUT_MULTIPLIER	= 3;
+	public static final int RETRY_MINIMUM_SECS			= 60;
+	public static final int RETRY_MINIMUM_MILLIS		= RETRY_MINIMUM_SECS*1000;
+	public static final int CLIENT_TIMEOUT_MULTIPLIER	= 3;
 	
-	public static int TIMEOUT_CHECK 				= RETRY_MINIMUM_MILLIS*CLIENT_TIMEOUT_MULTIPLIER;
+	public static final int TIMEOUT_CHECK 				= RETRY_MINIMUM_MILLIS*CLIENT_TIMEOUT_MULTIPLIER;
 	
 	public static int		max_peers_to_send			= 0;
 	public static boolean	send_peer_ids				= true;
@@ -54,6 +54,7 @@ TRTrackerServerImpl
 	public static int		max_seed_retention			= 0;
 	public static int		seed_limit					= 0;
 	public static boolean	full_scrape_enable			= true;
+	public static boolean	restrict_non_blocking_requests	= true;
 	
 	public static boolean	all_networks_permitted		= true;
 	public static String[]	permitted_networks			= {};
@@ -66,6 +67,8 @@ TRTrackerServerImpl
 	
 	private static Map		torrent_map = new HashMap(); 
 	
+	private static Map		link_map	= new HashMap();
+	
 	protected AEMonitor class_mon 	= new AEMonitor( "TRTrackerServer:class" );
 
 
@@ -130,6 +133,8 @@ TRTrackerServerImpl
 		redirect_on_not_found = COConfigurationManager.getStringParameter( "Tracker Server Not Found Redirect" ).trim();
 		
 		support_experimental_extensions = COConfigurationManager.getBooleanParameter( "Tracker Server Support Experimental Extensions" );
+		
+		restrict_non_blocking_requests = COConfigurationManager.getBooleanParameter( "Tracker TCP NonBlocking Restrict Request Types" );
 	}
 	
 	protected static boolean
@@ -205,6 +210,7 @@ TRTrackerServerImpl
 	private long		current_total_clients;
 	
 	private int		current_min_poll_interval;
+	private int		current_min_seed_announce_mult;
 	
 	private TRTrackerServerStatsImpl	stats = new TRTrackerServerStatsImpl( this );
 		
@@ -218,6 +224,7 @@ TRTrackerServerImpl
 	private boolean	compact_enabled;
 	private boolean	key_enabled;
 	
+	private boolean	enabled	= true;
 
 	protected Vector	listeners 			= new Vector();
 	private List		auth_listeners		= new ArrayList();
@@ -263,6 +270,8 @@ TRTrackerServerImpl
 			current_min_poll_interval = RETRY_MINIMUM_SECS;
 		}
 		
+		current_min_seed_announce_mult = COConfigurationManager.getIntParameter( "Tracker Poll Seed Interval Mult" );
+		
 		current_announce_retry_interval = current_min_poll_interval;		
 
 		int	scrape_percentage 		= COConfigurationManager.getIntParameter("Tracker Scrape Retry Percentage", DEFAULT_SCRAPE_RETRY_PERCENTAGE );		
@@ -315,6 +324,51 @@ TRTrackerServerImpl
 		return( is_ready );
 	}
 	
+	public void
+	setEnabled(
+		boolean	e )
+	{
+		enabled	= e;
+	}
+	
+	public boolean
+	isEnabled()
+	{
+		return( enabled );
+	}
+	
+	public TRTrackerServerTorrent
+	addLink(
+		String					link,
+		TRTrackerServerTorrent	target )
+	{
+		try{
+			class_mon.enter();
+			
+			return((TRTrackerServerTorrent)link_map.put( link, target ));
+			
+		}finally{
+			
+			class_mon.exit();
+		}
+	}
+	
+	public void
+	removeLink(
+		String					link,
+		TRTrackerServerTorrent	target )
+	{
+		try{
+			class_mon.enter();
+			
+			link_map.remove( link );
+			
+		}finally{
+			
+			class_mon.exit();
+		}	
+	}
+	
 	public void
 	setBiasedPeers(
 		Set		peers )
@@ -494,6 +548,12 @@ TRTrackerServerImpl
 		return( res );
 	}
 	
+	public long
+	getSeedAnnounceIntervalMultiplier()
+	{
+		return( current_min_seed_announce_mult );
+	}
+	
 	public long
 	getScrapeRetryInterval(
 		TRTrackerServerTorrentImpl	torrent )
@@ -529,57 +589,77 @@ TRTrackerServerImpl
 	
 	public void
 	updateStats(
+		int							request_type,					
 		TRTrackerServerTorrentImpl	torrent,
 		int							bytes_in,
 		int							bytes_out )
 	{
-		try{
-			class_mon.enter();
+			// non-synced, but doesn't really matter too much and we don't want to bring all parallel
+			// requests down to a single monitor here
 		
-			stats.update( bytes_in, bytes_out );
+		stats.update( request_type, bytes_in, bytes_out );
+		
+		if ( torrent != null ){
+
+			torrent.updateXferStats( bytes_in, bytes_out );
+
+		}else{
+
+			int	num = torrent_map.size();
+
+				// this gets too expensive to do when we have a lot of torrents so just ignore
+				// stats if so
 			
-			if ( torrent != null ){
-				
-				torrent.updateXferStats( bytes_in, bytes_out );
-				
-			}else{
-				
-				int	num = torrent_map.size();
-				
-				if ( num > 0 ){
+			if ( num < 256 ){
 				
+				try{
+					class_mon.enter();
+	
+	
+					if ( num > 0 ){
+	
 						// full scrape or error - spread the reported bytes across the torrents
-				
-					int	ave_in	= bytes_in/num;
-					int	ave_out	= bytes_out/num;
-					
-					int	rem_in 	= bytes_in-(ave_in*num);
-					int rem_out	= bytes_out-(ave_out*num);
-					
-					Iterator	it = torrent_map.values().iterator();
-				
-					while(it.hasNext()){
-									
-						TRTrackerServerTorrentImpl	this_torrent = (TRTrackerServerTorrentImpl)it.next();
-						
-						if ( it.hasNext()){
-							
-							this_torrent.updateXferStats( ave_in, ave_out );
-							
-						}else{
-							
-							this_torrent.updateXferStats( ave_in+rem_in, ave_out+rem_out );
-							
+	
+						int	ave_in	= bytes_in/num;
+						int	ave_out	= bytes_out/num;
+	
+						int	rem_in 	= bytes_in-(ave_in*num);
+						int rem_out	= bytes_out-(ave_out*num);
+	
+						Iterator	it = torrent_map.values().iterator();
+	
+						while(it.hasNext()){
+	
+							TRTrackerServerTorrentImpl	this_torrent = (TRTrackerServerTorrentImpl)it.next();
+	
+							if ( it.hasNext()){
+	
+								this_torrent.updateXferStats( ave_in, ave_out );
+	
+							}else{
+	
+								this_torrent.updateXferStats( ave_in+rem_in, ave_out+rem_out );
+	
+							}
 						}
 					}
+				}finally{
+	
+					class_mon.exit();
 				}
 			}
-		}finally{
-			
-			class_mon.exit();
 		}
 	}
 	
+	protected void
+	updateTime(
+		int		request_type,
+		long	time )
+	{
+		stats.updateTime( request_type, time );
+	}
+		
+	
 	protected void
 	timerLoop()
 	{
@@ -682,16 +762,18 @@ TRTrackerServerImpl
 	
 	public TRTrackerServerTorrent
 	permit(
+		String		_originator,
 		byte[]		_hash,
 		boolean		_explicit )
 	
 		throws TRTrackerServerException
 	{
-		return( permit( _hash, _explicit, true ));
+		return( permit( _originator, _hash, _explicit, true ));
 	}
 	
 	public TRTrackerServerTorrent
 	permit(
+		String		_originator,
 		byte[]		_hash,
 		boolean		_explicit,
 		boolean		_enabled )
@@ -720,7 +802,7 @@ TRTrackerServerImpl
 			
 			for (int i=0;i<listeners.size();i++){
 				
-				if ( !((TRTrackerServerListener)listeners.elementAt(i)).permitted( _hash, _explicit )){
+				if ( !((TRTrackerServerListener)listeners.elementAt(i)).permitted( _originator, _hash, _explicit )){
 					
 					throw( new TRTrackerServerException( "operation denied"));			
 				}
@@ -800,6 +882,21 @@ TRTrackerServerImpl
 		}
 	}
 	
+	public TRTrackerServerTorrentImpl
+	getTorrent(
+		String		link )
+	{
+		try{
+			class_mon.enter();
+		
+			return((TRTrackerServerTorrentImpl)link_map.get( link ));
+			
+		}finally{
+			
+			class_mon.exit();
+		}
+	}
+	
 	public TRTrackerServerTorrentImpl[]
 	getTorrents()
 	{
@@ -1032,8 +1129,17 @@ TRTrackerServerImpl
 		request_listeners.removeElement(l);
 	}
 	
+	public void
+	close()
+	{
+		TRTrackerServerFactoryImpl.close( this );
+	}
+	
+	protected abstract void
+	closeSupport();
+	
 	protected void
-	destroy()
+	destroySupport()
 	{
 		destroyed	= true;
 		
