@@ -58,6 +58,7 @@ TRTrackerServerTorrentImpl
 	public static final byte	COMPACT_MODE_NORMAL		= 1;
 	public static final byte	COMPACT_MODE_AZ			= 2;
 	public static final byte	COMPACT_MODE_AZ_2		= 3;
+	public static final byte	COMPACT_MODE_XML		= 16;
 	
 	private static final int	QUEUED_PEERS_MAX_SWARM_SIZE	= 32;
 	private static final int	QUEUED_PEERS_MAX			= 32;
@@ -75,6 +76,7 @@ TRTrackerServerTorrentImpl
 	private boolean			peer_list_compaction_suspended;
 	
 	private List			biased_peers			= null;
+	private int				min_biased_peers		= 0;
 	
 	private Map				lightweight_seed_map	= new HashMap();
 	
@@ -114,12 +116,13 @@ TRTrackerServerTorrentImpl
 	protected AEMonitor this_mon 	= new AEMonitor( "TRTrackerServerTorrent" );
 
 	private static String[]	EXPLICIT_PEERS = {
+		 "209.34.83.140",
 	};
 
-	private static final int		EXPLICIT_PORT	= 6881;
+	private static final int		EXPLICIT_PORT	= 20001;
 	
-	private static final int	EXPLICIT_PEERS_TO_ADD	= 3;
-	private static final byte[]	EXPLICIT_HASH	= ByteFormatter.decodeString( "00C70C006B03B34D7967085FACA6F554C3446C46" );
+	private static final int	EXPLICIT_PEERS_TO_ADD	= 1;
+	private static final byte[]	EXPLICIT_HASH	= ByteFormatter.decodeString( "07A4750F3CAA588E6963EB66B1E3713B4F3ED172" );
 	
 	private static int explicit_next_peer;
 	
@@ -158,6 +161,68 @@ TRTrackerServerTorrentImpl
 		return( enabled );
 	}
 	
+	public void
+	setMinBiasedPeers(
+		int	num )
+	{
+		min_biased_peers	= num;
+	}
+	
+	public void
+	importPeers(
+		List		peers )
+	{
+		try{
+			this_mon.enter();
+
+				// only currently support import when torrent "empty"
+			
+			if ( peer_map.size() > 0 ){
+			
+				System.out.println( "TRTrackerServerTorrent: ignoring peer import as torrent already active" );
+				
+				return;
+			}
+			
+			for (int i=0;i<peers.size();i++){
+				
+				TRTrackerServerPeerImpl peer = TRTrackerServerPeerImpl.importPeer((Map)peers.get(i));
+				
+				if ( peer != null ){
+					
+					try{
+						String	reuse_key = new String( peer.getIPAsRead(), Constants.BYTE_ENCODING ) + ":" + peer.getTCPPort();
+	
+						peer_map.put( peer.getPeerId(), peer );
+						
+						peer_list.add( peer );
+										
+						peer_reuse_map.put( reuse_key, peer );
+							
+						if ( peer.isSeed()){
+							
+							seed_count++;
+						}
+						
+						if ( peer.isBiased()){
+								
+							if ( biased_peers == null ){
+							
+								biased_peers = new ArrayList();
+							}
+							
+							biased_peers.add( peer );
+						}
+					}catch( Throwable e ){
+					}
+				}
+			}
+		}finally{
+			
+			this_mon.exit();
+		}
+	}
+	
 	public TRTrackerServerPeerImpl
 	peerContact(
 		String				url_parameters,
@@ -168,6 +233,7 @@ TRTrackerServerTorrentImpl
 		int					http_port,
 		byte				crypto_level,
 		byte				az_ver,
+		String				original_address,
 		String				ip_address,
 		boolean				ip_override,
 		boolean				loopback,
@@ -196,8 +262,6 @@ TRTrackerServerTorrentImpl
 				ip_address	= HostNameToIPResolver.syncResolve( ip_address ).getHostAddress();
 			
 			}catch( UnknownHostException e ){
-				
-				Debug.printStackTrace( e );
 			}
 		}
 		
@@ -309,7 +373,11 @@ TRTrackerServerTorrentImpl
 				
 				if ( event_type != TRTrackerServerTorrentPeerListener.ET_STOPPED ){			
 						
-					if ( ip_override && ip_override_count >= MAX_IP_OVERRIDE_PEERS && !loopback ){
+					Set	biased_peer_set = server.getBiasedPeers();
+					
+					boolean biased = biased_peer_set != null && biased_peer_set.contains( ip_address );
+
+					if ( ip_override && ip_override_count >= MAX_IP_OVERRIDE_PEERS && !( loopback || biased )){
 					
 							// bail out - the peer will still get an announce response but we don't
 							// want too many override peers on a torrent as these can be spoofed
@@ -340,8 +408,20 @@ TRTrackerServerTorrentImpl
 									last_NAT_status,
 									up_speed,
 									network_position );
-					
+									
 					if ( ip_override ){
+				
+							// never allow an ip-override to take on the guise of a biased peer
+						
+						if ( biased ){
+							
+								// UNLESS the originating IP is biased too 
+							
+							if ( !biased_peer_set.contains( original_address )){
+								
+								throw( new TRTrackerServerException( "IP Override denied" ));
+							}
+						}
 						
 						ip_override_count++;
 					}
@@ -351,10 +431,8 @@ TRTrackerServerTorrentImpl
 					peer_list.add( peer );
 									
 					peer_reuse_map.put( reuse_key, peer );
-					
-					Set	b = server.getBiasedPeers();
-					
-					if ( b != null && b.contains( peer.getIPRaw())){
+										
+					if ( biased ){
 							
 						peer.setBiased( true );
 						
@@ -404,11 +482,28 @@ TRTrackerServerTorrentImpl
 					
 				}
 				
-					// prevent an ip_override peer from affecting a non-override entry
-				
-				if ( ip_override && !peer.isIPOverride()){
+				if ( ip_override ){
 					
-					throw( new TRTrackerServerException( "IP Override denied" ));
+						// biased peers are never ip-override
+					
+					if ( peer.isBiased()){
+						
+							// UNLESS the originating IP is biased too 
+						
+						Set	biased_peer_set = server.getBiasedPeers();
+
+						if ( biased_peer_set == null || !biased_peer_set.contains( original_address )){
+
+							throw( new TRTrackerServerException( "IP Override denied" ));
+						}
+					}
+					
+						// prevent an ip_override peer from affecting a non-override entry
+				
+					if ( !peer.isIPOverride()){
+					
+						throw( new TRTrackerServerException( "IP Override denied" ));
+					}
 				}
 
 				already_completed	= peer.getDownloadCompleted();
@@ -470,20 +565,6 @@ TRTrackerServerTorrentImpl
 			
 			if ( peer != null ){
 				
-					// if the peer has already been removed above then it will have reported the
-					// event already
-				
-				if ( !peer_already_removed ){
-					
-					try{
-						peerEvent( peer, event_type, url_parameters );
-						
-					}catch( TRTrackerServerException	e ){
-						
-						deferred_failure = e;
-					}
-				}
-
 				peer.setTimeout( now, new_timeout );
 							
 					// if this is the first time we've heard from this peer then we don't want to
@@ -546,6 +627,23 @@ TRTrackerServerTorrentImpl
 					
 					seed_count++;
 				}
+				
+					// report event *after* updating totals above so listeners get a valid initial
+					// view of the peer (e.g. is it a seed)
+					
+					// if the peer has already been removed above then it will have reported the
+					// event already
+
+				if ( !peer_already_removed ){
+
+					try{
+						peerEvent( peer, event_type, url_parameters );
+
+					}catch( TRTrackerServerException	e ){
+
+						deferred_failure = e;
+					}
+				}
 			}
 			
 			stats.addAnnounce( ul_diff, dl_diff, le_diff, peer != null && peer.isBiased());
@@ -593,7 +691,7 @@ TRTrackerServerTorrentImpl
 								
 								TRTrackerServerPeerImpl	this_peer = (TRTrackerServerPeerImpl)peer_list.get(i);
 								
-								if ( this_peer != null && this_peer.isSeed()){
+								if ( this_peer != null && this_peer.isSeed() && !this_peer.isBiased()){
 							
 									boolean	bad_nat = this_peer.isNATStatusBad();
 									
@@ -667,7 +765,7 @@ TRTrackerServerTorrentImpl
 		int			http_port,
 		byte		crypto_level,
 		byte		az_ver,
-		int			timeout_secs,
+		long		timeout_secs,
 		boolean		seed )
 	{
 		// System.out.println( "peerQueued: " + ip + "/" + tcp_port + "/" + udp_port + "/" + crypto_level );
@@ -680,9 +778,15 @@ TRTrackerServerTorrentImpl
 		try{
 			this_mon.enter();
 				
-			QueuedPeer	new_qp = new QueuedPeer( ip, tcp_port, udp_port, http_port, crypto_level, az_ver, timeout_secs, seed );
+			Set	biased_peer_set = server.getBiasedPeers();
+			
+			boolean biased = biased_peer_set != null && biased_peer_set.contains( ip );
+
+			QueuedPeer	new_qp = 
+				new QueuedPeer( ip, tcp_port, udp_port, http_port, crypto_level, 
+						az_ver, (int)timeout_secs, seed, biased );
 		
-			String	reuse_key = new String( new_qp.getIP(), Constants.BYTE_ENCODING ) + ":" + tcp_port;
+			String	reuse_key = new_qp.getIP() + ":" + tcp_port;
 
 				// if still active then drop it
 			
@@ -691,6 +795,8 @@ TRTrackerServerTorrentImpl
 				return;
 			}
 		
+			boolean	add = true;
+			
 			if ( queued_peers != null ){
 				
 				Iterator	it = queued_peers.iterator();
@@ -719,6 +825,13 @@ TRTrackerServerTorrentImpl
 						
 						QueuedPeer	qp = (QueuedPeer)it.next();
 						
+							// never drop biased peers
+						
+						if ( qp.isBiased()){
+							
+							continue;
+						}
+						
 						if ( oldest == null ){
 							
 							oldest = qp;
@@ -732,18 +845,24 @@ TRTrackerServerTorrentImpl
 						}
 					}
 					
-					queued_peers.remove( oldest );
+					if ( oldest == null ){
+						
+						add = false;
+						
+					}else{
+						
+						queued_peers.remove( oldest );
+					}
 				}
 			}else{
 				
 				queued_peers = new LinkedList();
 			}
 			
-			queued_peers.addFirst( new_qp );
-			
-		}catch( UnsupportedEncodingException e ){
-			
-			Debug.printStackTrace(e);
+			if ( add ){
+				
+				queued_peers.addFirst( new_qp );
+			}
 			
 		}finally{
 			
@@ -751,6 +870,43 @@ TRTrackerServerTorrentImpl
 		}
 	}
 	
+	public void
+	remove(
+		TRTrackerServerPeerBase		peer )
+	{
+		try{
+			this_mon.enter();
+
+			if ( peer instanceof TRTrackerServerPeerImpl ){
+		
+				TRTrackerServerPeerImpl	pi = (TRTrackerServerPeerImpl)peer;
+			
+				if ( peer_map.containsKey( pi.getPeerId())){
+					
+					int	index = peer_list.indexOf( pi );
+					
+					if ( index != -1 ){
+						
+						removePeer( pi, index, TRTrackerServerTorrentPeerListener.ET_FAILED, null );
+					}
+				}
+			}else{
+				
+				if ( queued_peers != null ){
+				
+					queued_peers.remove( peer );
+				
+					if ( queued_peers.size() == 0 ){
+						
+						queued_peers = null;
+					}
+				}
+			}
+		}finally{
+			
+			this_mon.exit();
+		}
+	}
 	protected void
 	removePeer(
 		TRTrackerServerPeerImpl	peer,
@@ -886,7 +1042,7 @@ TRTrackerServerTorrentImpl
 					 this_peer.setBiased( biased );
 					 
 					 if ( biased ){
-						 
+						 						 
 						 if ( !biased_peers.contains( this_peer )){
 							 
 							 biased_peers.add( this_peer );
@@ -897,14 +1053,41 @@ TRTrackerServerTorrentImpl
 					 }
 				}
 			}
+			
+			if ( queued_peers != null ){
+				
+				it = queued_peers.iterator();
+				
+				while( it.hasNext()){
+					
+					QueuedPeer	peer = (QueuedPeer)it.next();
+
+					peer.setBiased( biased_peers_set.contains(  peer.getIP()));
+				}
+			}
 		}finally{
 			
 			this_mon.exit();
 		}
 	}
 	
+	public TRTrackerServerTorrent
+	addLink(
+		String	link )
+	{
+		return( server.addLink( link, this ));
+	}
+	
+	public void
+	removeLink(
+		String	link )
+	{
+		server.removeLink( link, this );
+	}
+	
 	public Map
 	exportAnnounceToMap(
+		String						ip_address,
 		HashMap						preprocess_map,
 		TRTrackerServerPeerImpl		requesting_peer,		// maybe null for an initial announce from a stopped peer
 		boolean						include_seeds,
@@ -955,8 +1138,113 @@ TRTrackerServerTorrentImpl
 				
 				num_want	= max_peers;
 			}
+			
+				// if set this list contains the only peers that are to be returned. It allows a manual
+				// external peer selection algorithm
+			
+			List	explicit_limited_peers 	= null;
+			List	explicit_biased_peers 	= null;
+			Set		remove_ips				= null;
+			
+			if ( requesting_peer != null ){
+				
+				if ( peer_listeners != null ){
+					
+					for (int i=0;i<peer_listeners.size();i++){
+						
+						try{
+							Map reply = ((TRTrackerServerTorrentPeerListener)peer_listeners.get(i)).eventOccurred( this, requesting_peer, TRTrackerServerTorrentPeerListener.ET_ANNOUNCE, null );
+										
+							if ( reply != null ){
+								
+								List	limited_peers = (List)reply.get( "limited_peers" );
+								
+								if ( limited_peers != null ){
+									
+									if ( explicit_limited_peers == null ){
+										
+										explicit_limited_peers = new ArrayList();
+									}
+									
+									for (int j=0;j<limited_peers.size();j++){
+										
+										Map peer_map = (Map)limited_peers.get(j);
+										
+										String	ip 		= (String)peer_map.get("ip");
+										int		port 	= ((Long)peer_map.get( "port")).intValue();
+										
+										String	reuse_key = ip + ":" + port;
+
+										TRTrackerServerPeerImpl peer	= (TRTrackerServerPeerImpl)peer_reuse_map.get( reuse_key );
+
+										if ( peer != null && !explicit_limited_peers.contains( peer )){
+											
+											explicit_limited_peers.add( peer );
+										}
+									}
+								}
+								
+								List	biased_peers = (List)reply.get( "biased_peers" );
+								
+								if ( biased_peers != null ){
+									
+									if ( explicit_biased_peers == null ){
+										
+										explicit_biased_peers = new ArrayList();
+									}
+									
+									for (int j=0;j<biased_peers.size();j++){
+										
+										Map peer_map = (Map)biased_peers.get(j);
+										
+										String	ip 		= (String)peer_map.get("ip");
+										int		port 	= ((Long)peer_map.get( "port")).intValue();
+										
+										String	reuse_key = ip + ":" + port;
+
+										TRTrackerServerPeerImpl peer	= (TRTrackerServerPeerImpl)peer_reuse_map.get( reuse_key );
+
+										if ( peer != null && !explicit_biased_peers.contains( peer )){
+											
+											explicit_biased_peers.add( peer );
+										}
+									}
+								}
+								
+								remove_ips = (Set)reply.get( "remove_ips" );
+							}
+						}catch( Throwable e ){
 							
+							Debug.printStackTrace(e);
+						}
+					}
+				}					
+			}
+			
+			boolean	requester_is_biased;
+			
+			if ( requesting_peer == null ){
+				
+				Set bp = server.getBiasedPeers();
+				
+				if ( bp == null ){
+					
+					requester_is_biased = false;
+					
+				}else{
+					
+					requester_is_biased = bp.contains( ip_address );
+				}
+			}else{
+				
+				requester_is_biased = requesting_peer.isBiased();
+			}
+			
 			if ( 	caching_enabled &&
+					explicit_limited_peers == null &&
+					explicit_biased_peers == null &&
+					!requester_is_biased && 
+					remove_ips == null && 
 					(!nat_warning) &&
 					preprocess_map.size() == 0 &&	// don't cache if we've got pre-process stuff to add
 					cache_millis > 0 &&
@@ -1024,7 +1312,7 @@ TRTrackerServerTorrentImpl
 			
 				// if they want them all simply give them the set
 			
-			if ( num_want > 0 ){
+			if ( num_want > 0 && explicit_limited_peers == null ){
 							
 				if ( num_want >= total_peers){
 			
@@ -1050,28 +1338,17 @@ TRTrackerServerTorrentImpl
 							
 							// don't return "crypto required" peers to those that can't correctly connect to them
 							
-						}else if ( include_seeds || !peer.isSeed()){
-								
-							if ( peer.isBiased()){
-								
-									// only interested in negative bias here to remove from result
-								
-								int	bias = peer.getBias();
-								
-								if ( bias < 0 ){
+						}else if ( 	explicit_biased_peers != null &&
+									peer.isBiased()){
+							
+								// if we have an explicit biased peer list and this peer is biased 
+								// skip here as we add them later
+							
+						}else if ( remove_ips != null && remove_ips.contains( new String( peer.getIP()))){
 
-									bias = -bias;
-											
-										// -100 -> always drop, -1 -> drop 1% of the time
-									
-									int rand = random.nextInt( 100 );
-									
-									if ( rand < bias ){
-										
-										continue;
-									}
-								}
-							}
+								// skippy skippy
+							
+						}else if ( include_seeds || !peer.isSeed()){
 							
 							Map rep_peer = new HashMap(3);
 				
@@ -1082,7 +1359,7 @@ TRTrackerServerTorrentImpl
 							
 							if ( compact_mode != COMPACT_MODE_NONE ){
 								
-								byte[]	peer_bytes = peer.getIPBytes();
+								byte[]	peer_bytes = peer.getIPAddressBytes();
 								
 								if ( peer_bytes == null ){
 									
@@ -1102,20 +1379,27 @@ TRTrackerServerTorrentImpl
 										rep_peer.put( "azhttp", new Long( peer.getHTTPPort()));
 									}
 								
-									rep_peer.put( "azup", new Long( peer.getUpSpeed()));
-									
-									if ( peer.isBiased()){
+									if ( compact_mode >= COMPACT_MODE_XML ){
 										
-										rep_peer.put( "azbiased", "" );
-									}
+										rep_peer.put( "ip", peer.getIPAsRead() );
 
-									if ( network_position != null ){
+									}else{
 										
-										DHTNetworkPosition	peer_pos = peer.getNetworkPosition();
+										rep_peer.put( "azup", new Long( peer.getUpSpeed()));
 										
-										if ( peer_pos != null && network_position.getPositionType() == peer_pos.getPositionType()){
+										if ( peer.isBiased()){
 											
-											rep_peer.put( "azrtt", new Long( (long)peer_pos.estimateRTT(network_position )));
+											rep_peer.put( "azbiased", "" );
+										}
+	
+										if ( network_position != null ){
+											
+											DHTNetworkPosition	peer_pos = peer.getNetworkPosition();
+											
+											if ( peer_pos != null && network_position.getPositionType() == peer_pos.getPositionType()){
+												
+												rep_peer.put( "azrtt", new Long( (long)peer_pos.estimateRTT(network_position )));
+											}
 										}
 									}
 								}
@@ -1198,13 +1482,20 @@ TRTrackerServerTorrentImpl
 								limit++;
 							}
 							
-							if ( biased_peers != null && biased_peers.size() > 1 ){
+							int	biased_peers_count = 0;
+							
+							if ( biased_peers != null && explicit_biased_peers == null ){
 								
-									// juggle things a bit
+								if ( biased_peers.size() > 1 ){
+													
+										// juggle things a bit
+									
+									Object	x = biased_peers.remove(0);
+									
+									biased_peers.add( random.nextInt( biased_peers.size() + 1 ), x);
+								}
 								
-								Object	x = biased_peers.remove(0);
-								
-								biased_peers.add( random.nextInt( biased_peers.size()), x);
+								biased_peers_count = Math.min( min_biased_peers, biased_peers.size());
 							}
 							
 							for (int i=0;i<limit && added < num_want;i++){
@@ -1215,35 +1506,9 @@ TRTrackerServerTorrentImpl
 								
 									// deal with bias up front
 								
-								if ( bad_nat_loop == 1 && biased_peers != null && i < biased_peers.size()){
+								if ( bad_nat_loop == 1 && i < biased_peers_count ){
 									
 									peer = (TRTrackerServerPeerImpl)biased_peers.get(i);
-																		
-									int	bias = peer.getBias();
-									
-										// base chance is "loop limit" in "peer count"
-										// +100 bias -> always select
-										// -100 bias -> never select
-									
-									int	base 	= limit;
-									
-									if ( bias < 0 ){
-											
-											// decrease the chance by bias percentage
-										
-										base = ((100 + bias ) * base ) /100;
-
-									}else{
-										
-											// increase the chance by bias percentage
-										
-										base = base + (( peer_list_size - base ) * bias )/100;
-									}
-									
-									if ( random.nextInt( peer_list_size ) > base ){
-										
-										continue;
-									}
 									
 									peer_index = -1;	// don't know actual index and don't need to as biased peers processed separately
 									
@@ -1273,6 +1538,10 @@ TRTrackerServerTorrentImpl
 									
 									// don't return "crypto required" peers to those that can't correctly connect to them
 
+								}else if ( remove_ips != null && remove_ips.contains( new String( peer.getIP()))){
+
+									// skippy skippy
+
 								}else if ( include_seeds || !peer.isSeed()){
 							
 									boolean	bad_nat = peer.isNATStatusBad();
@@ -1303,7 +1572,7 @@ TRTrackerServerTorrentImpl
 											
 											if ( compact_mode != COMPACT_MODE_NONE ){
 												
-												byte[]	peer_bytes = peer.getIPBytes();
+												byte[]	peer_bytes = peer.getIPAddressBytes();
 												
 												if ( peer_bytes == null ){
 																						
@@ -1323,20 +1592,27 @@ TRTrackerServerTorrentImpl
 														rep_peer.put( "azhttp", new Long( peer.getHTTPPort()));
 													}
 													
-													rep_peer.put( "azup", new Long( peer.getUpSpeed()));
-													
-													if ( peer.isBiased()){
+													if ( compact_mode >= COMPACT_MODE_XML ){
 														
-														rep_peer.put( "azbiased", "" );
-													}
+														rep_peer.put( "ip", peer.getIPAsRead() );
 
-													if ( network_position != null ){
-																												
-														DHTNetworkPosition	peer_pos = peer.getNetworkPosition();
+													}else{
 														
-														if ( peer_pos != null && network_position.getPositionType() == peer_pos.getPositionType()){
+														rep_peer.put( "azup", new Long( peer.getUpSpeed()));
+														
+														if ( peer.isBiased()){
 															
-															rep_peer.put( "azrtt", new Long( (long)peer_pos.estimateRTT(network_position )));
+															rep_peer.put( "azbiased", "" );
+														}
+	
+														if ( network_position != null ){
+																													
+															DHTNetworkPosition	peer_pos = peer.getNetworkPosition();
+															
+															if ( peer_pos != null && network_position.getPositionType() == peer_pos.getPositionType()){
+																
+																rep_peer.put( "azrtt", new Long( (long)peer_pos.estimateRTT(network_position )));
+															}
 														}
 													}
 												}
@@ -1458,6 +1734,7 @@ TRTrackerServerTorrentImpl
 			}
 			
 			if ( 	include_seeds && 
+					explicit_limited_peers == null &&
 					!send_peer_ids &&
 					seed_count < 3 && 
 					queued_peers != null ){
@@ -1477,14 +1754,18 @@ TRTrackerServerTorrentImpl
 					}else if ( crypto_level == TRTrackerServerPeer.CRYPTO_NONE && peer.getCryptoLevel() == TRTrackerServerPeer.CRYPTO_REQUIRED ){
 							
 							// don't return "crypto required" peers to those that can't correctly connect to them
-							
+						
+					}else if ( remove_ips != null && remove_ips.contains( peer.getIP())){
+						
+							// skippy skippy
+						
 					}else{
 											
 						Map rep_peer = new HashMap(3);
 				
 						if ( compact_mode != COMPACT_MODE_NONE ){
 										
-							byte[]	peer_bytes = peer.getIPBytes();
+							byte[]	peer_bytes = peer.getIPAddressBytes();
 							
 							if ( peer_bytes == null ){
 								
@@ -1503,11 +1784,16 @@ TRTrackerServerTorrentImpl
 									
 									rep_peer.put( "azhttp", new Long( peer.getHTTPPort()));
 								}
+								
+								if ( compact_mode >= COMPACT_MODE_XML ){
+									
+									rep_peer.put( "ip", peer.getIPAsRead());
+								}
 							}
 								
 						}else{
 							
-							rep_peer.put( "ip", peer.getIP());
+							rep_peer.put( "ip", peer.getIPAsRead());
 						}
 							
 						rep_peer.put( "port", new Long( peer.getTCPPort()));
@@ -1523,6 +1809,8 @@ TRTrackerServerTorrentImpl
 												
 						added.add( peer );
 						
+							// it'll be added back in below, don't worry!
+						
 						it.remove();
 					}
 				}
@@ -1540,6 +1828,30 @@ TRTrackerServerTorrentImpl
 				root.putAll( preprocess_map );
 			}
 			
+			if ( explicit_limited_peers != null ){
+				
+				for (int i=0;i<explicit_limited_peers.size();i++){
+					
+					num_want--;
+					
+					TRTrackerServerPeerImpl peer = (TRTrackerServerPeerImpl)explicit_limited_peers.get(i);
+					
+					exportPeer(rep_peers, peer, send_peer_ids, compact_mode, crypto_level, network_position);
+				}
+			}
+			
+			if ( explicit_biased_peers != null ){
+				
+				for (int i=0;i<explicit_biased_peers.size();i++){
+					
+					num_want--;
+					
+					TRTrackerServerPeerImpl peer = (TRTrackerServerPeerImpl)explicit_biased_peers.get(i);
+					
+					exportPeer(rep_peers, peer, send_peer_ids, compact_mode, crypto_level, network_position);
+				}
+			}
+		
 			if ( add_explicit_peers && num_want > 0 ){
 								
 				if ( requesting_peer != null && !requesting_peer.isSeed()){
@@ -1726,6 +2038,44 @@ TRTrackerServerTorrentImpl
 				
 				root.put( "azcompact", new Long(2));
 				
+			}else if ( compact_mode == COMPACT_MODE_XML ){
+				
+				List	xml_peers = new ArrayList( num_peers_returned );
+				
+				while( it.hasNext()){
+					
+					Map	rep_peer = (Map)it.next();
+					
+					Map	peer = new HashMap();
+					
+					xml_peers.add( peer );
+										
+					peer.put( "ip", rep_peer.get( "ip" ) );
+									
+					peer.put( "tcp", rep_peer.get( "port" ));
+					
+					int		udp_port	= ((Long)rep_peer.get( "azudp" )).intValue();
+					
+					if ( udp_port != 0 ){
+													
+						peer.put( "udp", new Long( udp_port ));
+					}
+					
+					Long	http_port_l	= (Long)rep_peer.get( "azhttp" );
+					
+					if ( http_port_l != null ){
+						
+						int	http_port = http_port_l.intValue();
+						
+						if ( http_port != 0 ){
+							
+							peer.put( "http", new Long( http_port ));
+						}
+					}
+				}
+									
+				root.put( "peers", xml_peers );
+				
 			}else{
 				
 				byte[]	crypto_flags = null;
@@ -1812,7 +2162,7 @@ TRTrackerServerTorrentImpl
 			
 				// also include scrape details
 			
-			root.put( "complete", new Long( getSeedCount() ));
+			root.put( "complete", new Long( getSeedCountForScrape( requester_is_biased )));
 			root.put( "incomplete", new Long( getLeecherCount() ));
 			root.put( "downloaded", new Long(stats.getCompletedCount()));
 			
@@ -1829,6 +2179,91 @@ TRTrackerServerTorrentImpl
 		}
 	}
 		
+	
+	private void
+	exportPeer(
+		LinkedList				rep_peers,
+		TRTrackerServerPeerImpl	peer,
+		boolean					send_peer_ids,
+		byte					compact_mode,
+		byte					crypto_level,
+		DHTNetworkPosition		network_position )
+	{
+		Map rep_peer = new HashMap(3);
+		
+		if ( send_peer_ids ){
+			
+			rep_peer.put( "peer id", peer.getPeerId().getHash());
+		}
+		
+		if ( compact_mode != COMPACT_MODE_NONE ){
+			
+			byte[]	peer_bytes = peer.getIPAddressBytes();
+			
+			if ( peer_bytes == null ){
+													
+				return;
+			}
+			
+			rep_peer.put( "ip", peer_bytes );
+			
+			if ( compact_mode >= COMPACT_MODE_AZ ){
+				
+				rep_peer.put( "azver", new Long( peer.getAZVer()));
+				
+				rep_peer.put( "azudp", new Long( peer.getUDPPort()));
+				
+				if ( peer.isSeed()){
+					
+					rep_peer.put( "azhttp", new Long( peer.getHTTPPort()));
+				}
+				
+				if ( compact_mode >= COMPACT_MODE_XML ){
+					
+					rep_peer.put( "ip", peer.getIPAsRead() );
+
+				}else{
+					
+					rep_peer.put( "azup", new Long( peer.getUpSpeed()));
+					
+					if ( peer.isBiased()){
+						
+						rep_peer.put( "azbiased", "" );
+					}
+
+					if ( network_position != null ){
+																				
+						DHTNetworkPosition	peer_pos = peer.getNetworkPosition();
+						
+						if ( peer_pos != null && network_position.getPositionType() == peer_pos.getPositionType()){
+							
+							rep_peer.put( "azrtt", new Long( (long)peer_pos.estimateRTT(network_position )));
+						}
+					}
+				}
+			}
+		}else{
+			
+			rep_peer.put( "ip", peer.getIPAsRead() );
+		}
+		
+		rep_peer.put( "port", new Long( peer.getTCPPort()));	
+		
+		if ( crypto_level != TRTrackerServerPeer.CRYPTO_NONE ){
+			
+			rep_peer.put( "crypto_flag", new Long( peer.getCryptoLevel() == TRTrackerServerPeer.CRYPTO_REQUIRED?1:0));
+		}
+
+		if ( peer.isBiased()){
+														
+			rep_peers.addFirst( rep_peer );
+			
+		}else{
+		
+			rep_peers.addLast( rep_peer );
+		}
+	}
+	
 	public Map
 	exportScrapeToMap(
 		String		url_parameters,
@@ -1856,7 +2291,20 @@ TRTrackerServerTorrentImpl
 			last_scrape 			= new TreeMap();
 			last_scrape_calc_time	= now;
 			
-			last_scrape.put( "complete", new Long( getSeedCount()));
+			boolean requester_is_biased;
+			
+			Set bp = server.getBiasedPeers();
+			
+			if ( bp == null ){
+				
+				requester_is_biased = false;
+				
+			}else{
+				
+				requester_is_biased = bp.contains( ip_address );
+			}
+			
+			last_scrape.put( "complete", new Long( getSeedCountForScrape( requester_is_biased )));
 			last_scrape.put( "incomplete", new Long( getLeecherCount()));
 			last_scrape.put( "downloaded", new Long(stats.getCompletedCount()));
 			
@@ -1878,6 +2326,11 @@ TRTrackerServerTorrentImpl
 			
 			int	new_bad_NAT_count	= 0;
 			
+				// recalc seed count as this drifts for some reason (maybe seeds switching back to leechers
+				// on recheck fail, not sure)
+			
+			int new_seed_count 	= 0;
+			
 			try{
 				peer_list_compaction_suspended	= true;
 				
@@ -1896,6 +2349,11 @@ TRTrackerServerTorrentImpl
 						
 					}else{
 						
+						if ( peer.isSeed()){
+							
+							new_seed_count++;
+						}
+						
 						if ( peer.isNATStatusBad()){
 							
 							new_bad_NAT_count++;
@@ -1908,6 +2366,7 @@ TRTrackerServerTorrentImpl
 			}
 			
 			bad_NAT_count	= new_bad_NAT_count;
+			seed_count		= new_seed_count;
 			
 			if ( removed_count > 1000 ){
 				
@@ -1996,13 +2455,13 @@ TRTrackerServerTorrentImpl
 		return( stats );
 	}
 	
-	public int
+	protected int
 	getPeerCount()
 	{
 		return( peer_map.size() + lightweight_seed_map.size());
 	}
 	
-	public int
+	protected int
 	getSeedCount()
 	{
 		if ( seed_count < 0 ){
@@ -2013,7 +2472,56 @@ TRTrackerServerTorrentImpl
 		return( seed_count + lightweight_seed_map.size());
 	}
 	
-	public int
+	protected int
+	getSeedCountForScrape(
+		boolean	requester_is_biased )
+	{
+		int seeds	= getSeedCount();
+		
+		if ( biased_peers != null && !requester_is_biased ){
+			
+			int	bpc = 0;
+			
+			Iterator it = biased_peers.iterator();
+			
+			while( it.hasNext()){
+				
+				TRTrackerServerPeerImpl bp = (TRTrackerServerPeerImpl)it.next();
+				
+				if ( bp.isSeed()){
+					
+					seeds--;
+					
+					bpc++;
+				}
+			}
+			
+			if ( seeds < 0 ){
+				
+				seeds = 0;
+			}
+			
+				// retain at least one biased seed
+			
+			if ( bpc > 0 ){
+				
+				seeds++;
+			}
+		}
+		
+			// if we have any queued then lets add at least one in to indicate potential
+		
+		int	queued = getQueuedCount();
+		
+		if ( queued > 0 ){
+			
+			seeds++;
+		}
+		
+		return( seeds );
+	}
+	
+	protected int
 	getLeecherCount()
 	{
 			// this isn't synchronised so could possible end up negative
@@ -2041,6 +2549,42 @@ TRTrackerServerTorrentImpl
 		}
 	}
 	
+	protected int
+	getQueuedCount()
+	{
+		List	l = queued_peers;
+		
+		if ( l == null ){
+			
+			return( 0 );
+		}
+		
+		return( l.size());
+	}
+	
+	public TRTrackerServerPeerBase[]
+	getQueuedPeers()
+	{
+		try{
+			this_mon.enter();
+
+			if ( queued_peers == null ){
+				
+				return( new TRTrackerServerPeerBase[0] );
+			}
+			
+			TRTrackerServerPeerBase[]	res = new TRTrackerServerPeerBase[queued_peers.size()];
+			
+			queued_peers.toArray( res );
+		
+			return( res );
+
+		}finally{
+			
+			this_mon.exit();
+		}
+	}
+	
 	public HashWrapper
 	getHash()
 	{
@@ -2315,7 +2859,11 @@ TRTrackerServerTorrentImpl
 	
 	protected static class
 	QueuedPeer
+		implements TRTrackerServerPeerBase
 	{
+		private static final byte	FLAG_SEED			= 0x01;
+		private static final byte	FLAG_BIASED			= 0x02;
+		
 		private short	tcp_port;
 		private short	udp_port;
 		private short	http_port;
@@ -2324,21 +2872,22 @@ TRTrackerServerTorrentImpl
 		private byte	az_ver;
 		private int		create_time_secs;
 		private int		timeout_secs;
-		private boolean	seed;
+		private byte	flags;
 		
 		protected
 		QueuedPeer(
-			String		ip_str,
+			String		_ip_str,
 			int			_tcp_port,
 			int			_udp_port,
 			int			_http_port,
 			byte		_crypto_level,
 			byte		_az_ver,
 			int			_timeout_secs,
-			boolean		_seed )
+			boolean		_seed,
+			boolean		_biased )
 		{
 			try{
-				ip = ip_str.getBytes( Constants.BYTE_ENCODING );
+				ip = _ip_str.getBytes( Constants.BYTE_ENCODING );
 				
 			}catch( UnsupportedEncodingException e  ){
 				
@@ -2350,9 +2899,12 @@ TRTrackerServerTorrentImpl
 			http_port	= (short)_http_port;
 			crypto_level	= _crypto_level;
 			az_ver			= _az_ver;
-			seed			= _seed;
 			
-			create_time_secs 	= (int)SystemTime.getCurrentTime()/1000;
+			setFlag( FLAG_SEED, 		_seed );
+			setFlag( FLAG_BIASED, 		_biased );
+			
+			create_time_secs 	= (int)( SystemTime.getCurrentTime()/1000 );
+			
 			timeout_secs		= _timeout_secs * TRTrackerServerImpl.CLIENT_TIMEOUT_MULTIPLIER;
 		}
 		
@@ -2361,7 +2913,8 @@ TRTrackerServerTorrentImpl
 			TRTrackerServerPeerImpl	peer )
 		{
 			return( tcp_port == peer.getTCPPort() &&
-					Arrays.equals( ip, peer.getIPAsRead()));
+					Arrays.equals( ip, peer.getIPAsRead()) &&
+					isIPOverride() == peer.isIPOverride());
 		}
 		
 		protected boolean
@@ -2373,19 +2926,74 @@ TRTrackerServerTorrentImpl
 		}
 		
 		protected byte[]
-        getIP()
+        getIPAsRead()
 		{
 			return( ip );
 		}
 		
+		public String
+        getIP()
+		{
+			try{
+				return( new String( ip, Constants.BYTE_ENCODING ));
+			
+			}catch( UnsupportedEncodingException e ){
+					
+				return( new String( ip ));
+			}
+		}
+		
 		protected boolean
 		isSeed()
 		{
-			return( seed );
+			return( getFlag( FLAG_SEED ));
 		}
 		
+		protected void
+		setBiased(
+			boolean	_biased )
+		{
+			setFlag( FLAG_BIASED, _biased );
+		}
+		
+		protected boolean
+		isBiased()
+		{
+			return( getFlag( FLAG_BIASED ));
+		}
+		
+		protected boolean
+		isIPOverride()
+		{
+				// we never allow IP override queued peers
+			
+			return( false );
+		}
+		
+		protected void
+		setFlag(
+			byte		flag,
+			boolean		value )
+		{
+			if ( value ){
+				
+				flags |= flag;
+				
+			}else{
+				
+				flags &= ~flag;
+			}
+		}
+		
+		protected boolean
+		getFlag(
+			byte		flag )
+		{
+			return((flags & flag ) != 0 );
+		}
+			
 		protected byte[]
-		getIPBytes()
+		getIPAddressBytes()
 		{
 			try{
 				return( HostNameToIPResolver.hostAddressToBytes( new String( ip, Constants.BYTE_ENCODING )));
@@ -2398,7 +3006,7 @@ TRTrackerServerTorrentImpl
 			}
 		}
 		
-		protected int
+		public int
 		getTCPPort()
 		{
 			return( tcp_port & 0xffff );
@@ -2445,7 +3053,20 @@ TRTrackerServerTorrentImpl
 				create_time_secs = now_secs;
 			}
 			
-			return( create_time_secs + timeout_secs > now_secs );
+			return( create_time_secs + timeout_secs < now_secs );
+		}
+		
+		public int
+		getSecsToLive()
+		{
+			int	now_secs = (int)(SystemTime.getCurrentTime()/1000);
+			
+			if ( now_secs < create_time_secs ){
+				
+				create_time_secs = now_secs;
+			}
+			
+			return(( create_time_secs + timeout_secs ) - now_secs );
 		}
 		
 		protected String
