@@ -35,6 +35,7 @@ import org.bouncycastle.util.encoders.Base64;
 
 import com.aelitis.azureus.core.dht.netcoords.DHTNetworkPosition;
 import com.aelitis.azureus.core.dht.netcoords.DHTNetworkPositionManager;
+import com.aelitis.azureus.core.util.HTTPUtils;
 
 public abstract class 
 TRTrackerServerProcessorTCP
@@ -54,7 +55,14 @@ TRTrackerServerProcessorTCP
 		"Server: " + Constants.AZUREUS_NAME + " " + Constants.AZUREUS_VERSION + NL +
 		"Connection: close" + NL +
 		"Content-Length: ").getBytes();
-		
+	
+	protected static final byte[]	HTTP_RESPONSE_XML_START = (
+			"HTTP/1.1 200 OK" + NL + 
+			"Content-Type: text/xml; charset=\"utf-8\"" + NL +
+			"Server: " + Constants.AZUREUS_NAME + " " + Constants.AZUREUS_VERSION + NL +
+			"Connection: close" + NL +
+			"Content-Length: ").getBytes();
+
 	protected static final byte[]	HTTP_RESPONSE_END_GZIP 		= (NL + "Content-Encoding: gzip" + NL + NL).getBytes();
 	protected static final byte[]	HTTP_RESPONSE_END_NOGZIP 	= (NL + NL).getBytes();
 	
@@ -70,7 +78,7 @@ TRTrackerServerProcessorTCP
 	{
 		server	= _server;
 		
-		server_url = (server.isSSL()?"https":"http") + "://" + server.getHost() + ":" + server.getPort();
+		server_url = (server.isSSL()?"https":"http") + "://" + UrlUtils.convertIPV6Host(server.getHost()) + ":" + server.getPort();
 	}	
 
 	protected boolean
@@ -105,7 +113,9 @@ TRTrackerServerProcessorTCP
 		throws IOException
 	{
 		String	str = url_path;
-				
+			
+		int	request_type	= TRTrackerServerRequest.RT_UNKNOWN;
+
 		try{
 			Map	root = null;
 				
@@ -113,9 +123,10 @@ TRTrackerServerProcessorTCP
 			
 			boolean	gzip_reply = false;
 			
-			try{
-				int	request_type;
+			boolean		xml_output		= false;
 			
+			
+			try{
 				if ( str.startsWith( "/announce?" )){
 					
 					request_type	= TRTrackerServerRequest.RT_ANNOUNCE;
@@ -134,6 +145,12 @@ TRTrackerServerProcessorTCP
 					
 					str = "";
 				
+				}else if ( str.startsWith( "/query?" )){
+
+					request_type	= TRTrackerServerRequest.RT_QUERY;
+					
+					str = str.substring(7);
+
 				}else{
 					
 					String	redirect = TRTrackerServerImpl.redirect_on_not_found;
@@ -171,7 +188,7 @@ TRTrackerServerProcessorTCP
 						
 					}else{
 						
-						os.write( ("HTTP/1.1 404 Not Found" + NL + NL ).getBytes() );
+						os.write( ("HTTP/1.1 404 Not Found" + NL + NL + "Page not found." + NL).getBytes() );
 					}
 					
 					os.flush();
@@ -212,50 +229,8 @@ TRTrackerServerProcessorTCP
 						if ( enc_pos != -1 ){
 							
 							String	accept_encoding = lowercase_input_header.substring(enc_pos+16,e_pos);
-														
-							int gzip_index = accept_encoding.indexOf("gzip");
-							
-							if ( gzip_index != -1 ){
-								
-								gzip_reply	= true;
-								
-								if ( accept_encoding.length() - gzip_index >= 8 ){
-								
-										// gzip;q=0
-										// look to see if there's a q=0 (or 0.0) disabling gzip
-	
-									char[]	chars = accept_encoding.toCharArray();
-									
-									boolean	q_value = false;
-																	
-									for (int i=gzip_index+4;i<chars.length;i++){
-										
-										char	c = chars[i];
-										
-										if ( c == ',' ){
-											
-											break;
-											
-										}else if ( c == '=' ){
-											
-											q_value		= true;
-											gzip_reply	= false;
-											
-										}else{
-											
-											if ( q_value ){
-												
-												if ( c != ' ' && c != '0' && c != '.' ){
-													
-													gzip_reply	= true;
-													
-													break;
-												}
-											}
-										}
-									}
-								}
-							}
+						
+							gzip_reply = HTTPUtils.canGZIP( accept_encoding );
 						}
 					}
 				}
@@ -266,6 +241,7 @@ TRTrackerServerProcessorTCP
 					
 				byte[]		hash		= null;
 				List		hash_list	= null;
+				String		link		= null;
 				
 				HashWrapper	peer_id		= null;
 				int			tcp_port	= 0;
@@ -286,6 +262,7 @@ TRTrackerServerProcessorTCP
 				boolean		stop_to_queue	= false;
 				String		scrape_flags	= null;
 				int			up_speed		= 0;
+				boolean		hide			= false;
 				
 				DHTNetworkPosition	network_position = null;
 				
@@ -375,18 +352,26 @@ TRTrackerServerProcessorTCP
 							
 					}else if ( lhs.equals( "ip" )){
 							
-						if ( AENetworkClassifier.categoriseAddress( rhs ) == AENetworkClassifier.AT_PUBLIC ){
-	
-								// only accept public resolved addresses
+						// System.out.println( "override: " + real_ip_address + " -> " + rhs + " [" + input_header + "]" );
+						
+						if ( !HostNameToIPResolver.isNonDNSName( rhs )){
 							
 							for (int i=0;i<rhs.length();i++){
 								
 								char	c = rhs.charAt(i);
 								
-								if ( c != '.' && !Character.isDigit( c )){
+								if ( c != '.' && c != ':' && !Character.isDigit( c )){
 									
 									throw( new Exception( "IP override address must be resolved by the client" ));
-								}
+								}	
+							}
+							
+							try{
+								rhs	= HostNameToIPResolver.syncResolve( rhs ).getHostAddress();
+							
+							}catch( UnknownHostException e ){
+								
+								throw( new Exception( "IP override address must be resolved by the client" ));
 							}
 						}
 						
@@ -444,9 +429,24 @@ TRTrackerServerProcessorTCP
 						stop_to_queue	= true;
 												
 					}else if ( lhs.equals( "azsf" )){
-					
+						
 						scrape_flags = rhs;
 						
+					}else if ( lhs.equals( "link" )){
+						
+						link = rhs;
+					
+					}else if ( lhs.equals( "outform" )){
+
+						if ( rhs.equals( "xml" )){
+							
+							xml_output	= true;
+						}
+						
+					}else if ( lhs.equals( "hide" )){
+						
+						hide 	= Integer.parseInt( rhs ) == 1;
+
 					}else if ( TRTrackerServerImpl.supportsExtensions()){
 						
 						if ( lhs.equals( "aznp" )){
@@ -469,6 +469,19 @@ TRTrackerServerProcessorTCP
 					}
 				}
 				
+					// let them hide!
+					// this is also useful if an az client wants to just hide themselves on 
+					// particular torrents (to prevent inward connections) as they can just
+					// add a tracker-extension to append this option
+				
+				if ( hide ){
+					
+					tcp_port 	= 0;
+					crypto_port	= 0;
+					http_port	= 0;
+					udp_port	= 0;
+				}
+				
 				if ( crypto_level == TRTrackerServerPeer.CRYPTO_REQUIRED ){
 					
 					if ( crypto_port != 0 ){
@@ -493,7 +506,11 @@ TRTrackerServerProcessorTCP
 					// >= so that if this tracker is "old" and sees a version 3+ it replies with the
 					// best it can - version 2
 				
-				if ( az_ver >= 2 ){
+				if ( xml_output ){
+					
+					compact_mode = TRTrackerServerTorrentImpl.COMPACT_MODE_XML;
+					
+				}else if ( az_ver >= 2 ){
 					
 					compact_mode = TRTrackerServerTorrentImpl.COMPACT_MODE_AZ_2;
 				}
@@ -506,7 +523,7 @@ TRTrackerServerProcessorTCP
 							server, str,
 							root_out, peer_out,
 							request_type,
-							hashes, scrape_flags,
+							hashes, link, scrape_flags,
 							peer_id, no_peer_id, compact_mode, key, 
 							event, stop_to_queue,
 							tcp_port&0xffff, udp_port&0xffff, http_port&0xffff,
@@ -547,6 +564,9 @@ TRTrackerServerProcessorTCP
 				
 			}catch( Exception e ){
 				
+				String	warning_message = null;
+				
+				Map	error_entries		= null;
 				
 				if ( e instanceof TRTrackerServerException ){
 					
@@ -554,6 +574,8 @@ TRTrackerServerProcessorTCP
 					
 					int	reason = tr_excep.getResponseCode();
 					
+					error_entries = tr_excep.getErrorEntries();
+					
 					if ( reason != -1 ){
 						
 						String	resp = "HTTP/1.1 " + reason + " " + tr_excep.getResponseText() + NL;
@@ -572,14 +594,33 @@ TRTrackerServerProcessorTCP
 							resp += key + ": " + value + NL;
 						}
 
+						byte[]	payload = null;
+						
+						if ( error_entries != null ){
+							
+							payload = BEncoder.encode( error_entries );
+							
+							resp += "Content-Length: " + payload.length + NL;
+						}
+						
 						resp += NL;
 
 						os.write( resp.getBytes());
 						
+						if ( payload != null ){
+							
+							os.write( payload );
+						}
+						
 						os.flush();
 
 						return;
 					}
+					
+					if ( tr_excep.isUserMessage()){
+						
+						warning_message = tr_excep.getMessage();
+					}					
 				}else if ( e instanceof NullPointerException ){
 					
 					e.printStackTrace();
@@ -597,25 +638,84 @@ TRTrackerServerProcessorTCP
 				}
 					
 				root	= new HashMap();
-				
+								
 				root.put( "failure reason", message );
+				
+				if ( warning_message != null ){
+					
+					root.put( "warning message", warning_message );
+				}
+				
+				if ( error_entries != null ){
+					
+					root.putAll( error_entries );
+				}
 			}
 		
 			setTaskState( "writing response" );
 
-				// cache both plain and gzip encoded data for possible reuse
+			byte[]	data;
+			byte[]	header_start;
 			
-			byte[] data 		= (byte[])root.get( "_data" );
-					
-			if ( data == null ){
+			if ( xml_output ){
 				
-				data = BEncoder.encode( root );
-				
-				root.put( "_data", data );
-			}
-						
-			if ( gzip_reply ){
+				StringBuffer	xml = new StringBuffer( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" );
 
+				xml.append( "<RESULT>" );
+				
+				if ( specific_torrent != null ){
+					
+					xml.append( "<BTIH>" );
+					xml.append( ByteFormatter.encodeString( specific_torrent.getHash().getBytes()));
+					xml.append( "</BTIH>" );
+					
+					xml.append( BEncoder.encodeToXML( root, true ));
+				}
+				
+				xml.append( "</RESULT>" );
+				
+				data			= xml.toString().getBytes("UTF-8" );
+				
+				header_start = HTTP_RESPONSE_XML_START;
+
+			}else{
+					// cache both plain and gzip encoded data for possible reuse
+				
+				data 		= (byte[])root.get( "_data" );
+						
+				if ( data == null ){
+					
+					data = BEncoder.encode( root );
+					
+					if ( data.length > 1000000 ){
+						
+						File	dump = new File( "bdecoder.dump" );
+						
+						synchronized( TRTrackerServerProcessorTCP.class ){
+							
+							try{
+								Debug.out( "Output is too large, saving diagnostics to " + dump.toString());
+								
+								PrintWriter	pw = new PrintWriter( new FileWriter( dump ));
+								
+								BDecoder.print( pw, root );
+								
+								pw.close();
+								
+							}catch( Throwable e ){
+								
+							}
+						}
+					}
+					
+					root.put( "_data", data );
+				}
+				
+				header_start = HTTP_RESPONSE_START;
+			}	
+			
+			if ( gzip_reply ){
+				
 				byte[]	gzip_data = (byte[])root.get( "_gzipdata");
 				
 				if ( gzip_data == null ){
@@ -635,20 +735,20 @@ TRTrackerServerProcessorTCP
 				
 				data	= gzip_data;
 			}
-						
+			
 				// System.out.println( "TRTrackerServerProcessor::reply: sending " + new String(data));
-						
+							
 				// write the response
 			
 			setTaskState( "writing header" );
 
-			os.write( HTTP_RESPONSE_START );
+			os.write( header_start );
 			
 			byte[]	length_bytes = String.valueOf(data.length).getBytes();
 			
 			os.write( length_bytes );
 
-			int	header_len = HTTP_RESPONSE_START.length + length_bytes.length;
+			int	header_len = header_start.length + length_bytes.length;
 			
 			setTaskState( "writing content" );
 
@@ -666,7 +766,7 @@ TRTrackerServerProcessorTCP
 					
 			os.write( data );
 			
-			server.updateStats( specific_torrent, input_header.length(), header_len+data.length );
+			server.updateStats( request_type, specific_torrent, input_header.length(), header_len+data.length );
 							
 		}finally{
 			
@@ -715,7 +815,7 @@ TRTrackerServerProcessorTCP
 					try{
 						String	resource_str = 
 							( server.isSSL()?"https":"http" ) + "://" +
-								server.getHost() + ":" + server.getPort() + url_path;
+								UrlUtils.convertIPV6Host(server.getHost()) + ":" + server.getPort() + url_path;
 						
 						URL	resource = new URL( resource_str );
 					
@@ -753,7 +853,7 @@ TRTrackerServerProcessorTCP
 					try{
 						String	resource_str = 
 							( server.isSSL()?"https":"http" ) + "://" +
-								server.getHost() + ":" + server.getPort() + url_path;
+							UrlUtils.convertIPV6Host(server.getHost()) + ":" + server.getPort() + url_path;
 						
 						URL	resource = new URL( resource_str );
 					
