@@ -39,43 +39,49 @@ import org.gudy.azureus2.core3.tracker.server.impl.*;
 import org.gudy.azureus2.core3.tracker.protocol.*;
 import org.gudy.azureus2.core3.tracker.protocol.udp.*;
 
+import com.aelitis.net.udp.uc.PRUDPPacket;
+import com.aelitis.net.udp.uc.PRUDPPacketRequest;
+
 public class 
 TRTrackerServerProcessorUDP
 	extends		TRTrackerServerProcessor
 {
+	private static final LogIDs LOGID = LogIDs.TRACKER;
 		// client may connect + then retry announce up to 4 times -> * 6
 	
 	public static final long CONNECTION_ID_LIFETIME	= PRUDPPacket.DEFAULT_UDP_TIMEOUT*6;
 	
-	protected TRTrackerServerUDP		server;
-	protected DatagramSocket			socket;
-	protected DatagramPacket			packet;
+	private TRTrackerServerUDP		server;
+	private DatagramSocket			socket;
+	private DatagramPacket			request_dg;
 	
-	protected static Map				connection_id_map 	= new LinkedHashMap();
-	protected static SecureRandom		random				= new SecureRandom();
-	protected static AEMonitor			random_mon 			= new AEMonitor( "TRTrackerServerUDP:rand" );
+	private static Map				connection_id_map 	= new LinkedHashMap();
+	private static SecureRandom		random				= new SecureRandom();
+	private static AEMonitor		random_mon 			= new AEMonitor( "TRTrackerServerUDP:rand" );
 
+	static{
+	  	PRUDPTrackerCodecs.registerCodecs();
+	}
+	
 	protected
 	TRTrackerServerProcessorUDP(
 		TRTrackerServerUDP		_server,
 		DatagramSocket			_socket,
 		DatagramPacket			_packet )
 	{
-		server	= _server;
-		socket	= _socket;
-		packet	= _packet;
+		server			= _server;
+		socket			= _socket;
+		request_dg		= _packet;
 	}
 	
 	public void
 	runSupport()
-	{		
-		byte[]	_data = packet.getData();
+	{				
+		byte[]	input_buffer = new byte[request_dg.getLength()];
 		
-		byte[]	data = new byte[packet.getLength()];
+		System.arraycopy( request_dg.getData(), 0, input_buffer, 0, input_buffer.length );
 		
-		System.arraycopy( _data, 0, data, 0, data.length );
-		
-		int	packet_data_length = data.length;
+		int	packet_data_length = input_buffer.length;
 		
 		String	auth_user			= null;
 		byte[] 	auth_user_bytes		= null;
@@ -87,9 +93,11 @@ TRTrackerServerProcessorUDP
 				// auth detail should be attached to the packet. Auth details are 16
 				// bytes
 			
-			if ( data.length < 17 ){
+			if ( input_buffer.length < 17 ){
 				
-				LGLogger.log( "TRTrackerServerProcessorUDP: packet received but authorisation missing" ); 
+				Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING,
+						"TRTrackerServerProcessorUDP: "
+								+ "packet received but authorisation missing")); 
 
 				return;
 			}
@@ -100,7 +108,7 @@ TRTrackerServerProcessorUDP
 			
 			auth_hash = new byte[8];
 			
-			System.arraycopy( data, packet_data_length, auth_user_bytes, 0, 8 );
+			System.arraycopy( input_buffer, packet_data_length, auth_user_bytes, 0, 8 );
 			
 			int	user_len = 0;
 			
@@ -111,19 +119,22 @@ TRTrackerServerProcessorUDP
 			
 			auth_user = new String( auth_user_bytes, 0, user_len );
 			
-			System.arraycopy( data, packet_data_length+8, auth_hash, 0, 8 );
+			System.arraycopy( input_buffer, packet_data_length+8, auth_hash, 0, 8 );
 		}
 				
-		DataInputStream is = new DataInputStream(new ByteArrayInputStream(data, 0, packet_data_length ));
+		DataInputStream is = new DataInputStream(new ByteArrayInputStream(input_buffer, 0, packet_data_length ));
 		
 		try{
-			String	client_ip_address = packet.getAddress().getHostAddress();
+			String	client_ip_address = request_dg.getAddress().getHostAddress();
 			
-			PRUDPPacketRequest	request = PRUDPPacketRequest.deserialiseRequest( is );
+			PRUDPPacketRequest	request = PRUDPPacketRequest.deserialiseRequest( null, is );
 			
-			LGLogger.log( "TRTrackerServerProcessorUDP: packet received: " + request.getString()); 
-			
-			PRUDPPacket	reply = null;
+			Logger.log(new LogEvent(LOGID,
+					"TRTrackerServerProcessorUDP: packet received: "
+							+ request.getString())); 
+				
+			PRUDPPacket					reply 	= null;
+			TRTrackerServerTorrentImpl	torrent	= null;
 			
 			if ( auth_user_bytes != null ){
 				
@@ -151,7 +162,9 @@ TRTrackerServerProcessorUDP
 					
 					if ( sha1_pw == null ){
 				
-						LGLogger.log( "TRTrackerServerProcessorUDP: auth fails for user '" + auth_user + "'"); 
+						Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR,
+								"TRTrackerServerProcessorUDP: auth fails for user '"
+										+ auth_user + "'")); 
 
 						reply = new PRUDPPacketReplyError( request.getTransactionId(), "Access Denied" );
 					}
@@ -166,7 +179,7 @@ TRTrackerServerProcessorUDP
 					
 					SHA1Hasher	hasher = new SHA1Hasher();
 					
-					hasher.update( data, 0, packet_data_length);
+					hasher.update( input_buffer, 0, packet_data_length);
 					hasher.update( auth_user_bytes );
 					hasher.update( sha1_pw );
 					
@@ -176,7 +189,9 @@ TRTrackerServerProcessorUDP
 						
 						if ( auth_hash[i] != digest[i] ){
 					
-							LGLogger.log( "TRTrackerServerProcessorUDP: auth fails for user '" + auth_user + "'"); 
+							Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR,
+									"TRTrackerServerProcessorUDP: auth fails for user '"
+											+ auth_user + "'")); 
 	
 							reply = new PRUDPPacketReplyError( request.getTransactionId(), "Access Denied" );
 							
@@ -186,45 +201,68 @@ TRTrackerServerProcessorUDP
 				}
 			}
 			
-			if( reply == null ){
+			int request_type = TRTrackerServerRequest.RT_UNKNOWN;
+			
+			if ( reply == null ){
 				
-				try{
-					int	type = request.getAction();
+				if ( server.isEnabled()){
 					
-					if ( type == PRUDPPacket.ACT_REQUEST_CONNECT ){
+					try{
+						int	type = request.getAction();
 						
-						reply = handleConnect( client_ip_address, request );
+						if ( type == PRUDPPacketTracker.ACT_REQUEST_CONNECT ){
+							
+							reply = handleConnect( client_ip_address, request );
+							
+						}else if (type == PRUDPPacketTracker.ACT_REQUEST_ANNOUNCE ){
+							
+							Object[] x = handleAnnounceAndScrape( client_ip_address, request, TRTrackerServerRequest.RT_ANNOUNCE );
+							
+							reply 	= (PRUDPPacket)x[0];
+							torrent	= (TRTrackerServerTorrentImpl)x[1];
+					
+							request_type = TRTrackerServerRequest.RT_ANNOUNCE;
+							
+						}else if ( type == PRUDPPacketTracker.ACT_REQUEST_SCRAPE ){
+							
+							Object[] x = handleAnnounceAndScrape( client_ip_address, request, TRTrackerServerRequest.RT_SCRAPE );
+							
+							reply 	= (PRUDPPacket)x[0];
+							torrent	= (TRTrackerServerTorrentImpl)x[1];
+		
+							request_type = TRTrackerServerRequest.RT_SCRAPE;
+							
+						}else{
+							
+							reply = new PRUDPPacketReplyError( request.getTransactionId(), "unsupported action");
+						}
+					}catch( Throwable e ){
 						
-					}else if (type == PRUDPPacket.ACT_REQUEST_ANNOUNCE ){
+						// e.printStackTrace();
 						
-						reply = handleAnnounceAndScrape( client_ip_address, request, TRTrackerServerRequest.RT_ANNOUNCE );
+						String	error = e.getMessage();
 						
-					}else if ( type == PRUDPPacket.ACT_REQUEST_SCRAPE ){
+						if ( error == null ){
+							
+							error = e.toString();
+						}
 						
-						reply = handleAnnounceAndScrape( client_ip_address, request, TRTrackerServerRequest.RT_SCRAPE );
-						
-					}else{
-						
-						reply = new PRUDPPacketReplyError( request.getTransactionId(), "unsupported action");
+						reply = new PRUDPPacketReplyError( request.getTransactionId(), error );
 					}
-				}catch( Throwable e ){
+				}else{
+										
+					System.out.println( "UDP Tracker: replying 'disabled' to " + client_ip_address );
 					
-					// e.printStackTrace();
-					
-					String	error = e.getMessage();
-					
-					if ( error == null ){
-						
-						error = e.toString();
-					}
-					
-					reply = new PRUDPPacketReplyError( request.getTransactionId(), error );
+					//if ( client_ip_address.equals( "64.79.127.118" )){
+								
+						reply = new PRUDPPacketReplyError( request.getTransactionId(), "UDP Tracker disabled" );
+					//}
 				}
 			}
 			
 			if ( reply != null ){
 				
-				InetAddress address = packet.getAddress();
+				InetAddress address = request_dg.getAddress();
 				
 				ByteArrayOutputStream	baos = new ByteArrayOutputStream();
 				
@@ -232,16 +270,27 @@ TRTrackerServerProcessorUDP
 										
 				reply.serialise(os);
 				
-				byte[]	buffer = baos.toByteArray();
+				byte[]	output_buffer = baos.toByteArray();
 				
-				DatagramPacket reply_packet = new DatagramPacket(buffer, buffer.length,address,packet.getPort());
+				DatagramPacket reply_packet = new DatagramPacket(output_buffer, output_buffer.length,address,request_dg.getPort());
 							
 				socket.send( reply_packet );
+			
+				server.updateStats( request_type, torrent, input_buffer.length, output_buffer.length );
 			}
 			
 		}catch( Throwable e ){
 			
-			Debug.printStackTrace( e );
+			Logger.log(new LogEvent(LOGID,
+					"TRTrackerServerProcessorUDP: processing fails", e)); 
+		}finally{
+			
+			try{
+				is.close();
+				
+			}catch( Throwable e ){
+				
+			}
 		}
 	}
 	
@@ -345,7 +394,9 @@ TRTrackerServerProcessorUDP
 		return( reply );
 	}
 	
-	protected PRUDPPacket
+		// returns reply packet and associated torrent if exists
+	
+	protected Object[]
 	handleAnnounceAndScrape(
 		String				client_ip_address,
 		PRUDPPacketRequest	request,
@@ -358,7 +409,7 @@ TRTrackerServerProcessorUDP
 			return( null );
 		}
 				
-		byte[]		hash_bytes	= null;
+		List		hashbytes	= new ArrayList();
 		HashWrapper	peer_id		= null;
 		int			port		= 0;
 		String		event		= null;
@@ -372,10 +423,10 @@ TRTrackerServerProcessorUDP
 		
 		if ( request_type == TRTrackerServerRequest.RT_ANNOUNCE ){
 			
-			if ( PRUDPPacket.VERSION == 1 ){
+			if ( PRUDPPacketTracker.VERSION == 1 ){
 				PRUDPPacketRequestAnnounce	announce = (PRUDPPacketRequestAnnounce)request;
 				
-				hash_bytes	= announce.getHash();
+				hashbytes.add(announce.getHash());
 				
 				peer_id		= new HashWrapper( announce.getPeerId());
 				
@@ -419,7 +470,7 @@ TRTrackerServerProcessorUDP
 				
 				PRUDPPacketRequestAnnounce2	announce = (PRUDPPacketRequestAnnounce2)request;
 				
-				hash_bytes	= announce.getHash();
+				hashbytes.add(announce.getHash());
 				
 				peer_id		= new HashWrapper( announce.getPeerId());
 				
@@ -466,7 +517,7 @@ TRTrackerServerProcessorUDP
 			
 			PRUDPPacketRequestScrape	scrape = (PRUDPPacketRequestScrape)request;
 			
-			hash_bytes	= scrape.getHash();
+			hashbytes.addAll(scrape.getHashes());
 		}
 		
 		Map[]						root_out = new Map[1];
@@ -474,21 +525,26 @@ TRTrackerServerProcessorUDP
 		
 		TRTrackerServerTorrentImpl torrent =
 			processTrackerRequest( 
-				server, root_out, peer_out, 
+				server, "", root_out, peer_out, 
 				request_type,
-				hash_bytes,
-				peer_id, false,	false, key, // currently no "no_peer_id" / "compact" in the packet and anyway they aren't returned / key
-				event,
+				(byte[][])hashbytes.toArray(new byte[0][0]), null, null,
+				peer_id, false, TRTrackerServerTorrentImpl.COMPACT_MODE_NONE, key, // currently no "no_peer_id" / "compact" in the packet and anyway they aren't returned / key
+				event, false,
 				port,
-				client_ip_address,
+				0, 0,
+				client_ip_address, client_ip_address,
 				downloaded, uploaded, left,
-				num_want );
+				num_want,
+				TRTrackerServerPeer.CRYPTO_NONE,
+				(byte)1,
+				0,
+				null );
 		
 		Map	root = root_out[0];
 		
 		if ( request_type == TRTrackerServerRequest.RT_ANNOUNCE ){
 
-			if ( PRUDPPacket.VERSION == 1 ){
+			if ( PRUDPPacketTracker.VERSION == 1 ){
 				PRUDPPacketReplyAnnounce reply = new PRUDPPacketReplyAnnounce(request.getTransactionId());
 				
 				reply.setInterval(((Long)root.get("interval")).intValue());
@@ -509,7 +565,8 @@ TRTrackerServerProcessorUDP
 				
 				reply.setPeers( addresses, ports );
 				
-				return( reply );
+				return( new Object[]{ reply, torrent });
+				
 			}else{
 				
 				PRUDPPacketReplyAnnounce2 reply = new PRUDPPacketReplyAnnounce2(request.getTransactionId());
@@ -518,7 +575,7 @@ TRTrackerServerProcessorUDP
 				
 				boolean	local_scrape = client_ip_address.equals( "127.0.0.1" );
 
-				Map scrape_details = torrent.exportScrapeToMap( !local_scrape );
+				Map scrape_details = torrent.exportScrapeToMap( "", client_ip_address,!local_scrape );
 				
 				int	seeders 	= ((Long)scrape_details.get("complete")).intValue();
 				int leechers 	= ((Long)scrape_details.get("incomplete")).intValue();
@@ -541,12 +598,12 @@ TRTrackerServerProcessorUDP
 				
 				reply.setPeers( addresses, ports );
 				
-				return( reply );
+				return( new Object[]{ reply, torrent });
 			}
 			
 		}else{
 			
-			if ( PRUDPPacket.VERSION == 1 ){
+			if ( PRUDPPacketTracker.VERSION == 1 ){
 				
 				PRUDPPacketReplyScrape reply = new PRUDPPacketReplyScrape(request.getTransactionId());
 				
@@ -587,7 +644,7 @@ TRTrackerServerProcessorUDP
 				
 				reply.setDetails( hashes, s_complete, s_downloaded, s_incomplete );
 				
-				return( reply );
+				return( new Object[]{ reply, torrent });
 				
 			}else{
 				
@@ -627,7 +684,7 @@ TRTrackerServerProcessorUDP
 				
 				reply.setDetails( s_complete, s_downloaded, s_incomplete );
 				
-				return( reply );				
+				return( new Object[]{ reply, torrent });				
 			}
 		}
 	}
