@@ -34,9 +34,11 @@ import org.gudy.azureus2.core3.config.*;
 import org.gudy.azureus2.core3.tracker.client.impl.bt.TRTrackerBTAnnouncerImpl;
 import org.gudy.azureus2.core3.tracker.host.TRHost;
 import org.gudy.azureus2.core3.util.AENetworkClassifier;
-import org.gudy.azureus2.core3.util.AEThread;
+import org.gudy.azureus2.core3.util.AEThread2;
+import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.SystemTime;
+import org.gudy.azureus2.core3.util.UrlUtils;
 
 import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
 import com.aelitis.azureus.core.networkmanager.admin.NetworkAdminPropertyChangeListener;
@@ -62,17 +64,19 @@ TRTrackerUtils
 	private static String		bind_ip;
 	
 	private static String		ports_for_url;
+	private static String		ports_for_url_with_crypto;
 	
 	private static CopyOnWriteList		listeners = new CopyOnWriteList();
 	
-	private static Thread		listener_thread;
+	private static AEThread2		listener_thread;
 	
 	static{
 		COConfigurationManager.addAndFireParameterListeners(
 			new String[]
 			{	"Proxy.Data.Enable",
 				"Proxy.Data.SOCKS.inform",
-				"TCP.Announce.Port",
+				"TCP.Listen.Port.Override",
+				"Tracker Client No Port Announce",
 				"network.transport.encrypted.use.crypto.port",
 				"network.transport.encrypted.require",
 				"network.transport.encrypted.fallback.incoming",
@@ -80,89 +84,17 @@ TRTrackerUtils
 				"UDP.Listen.Port",
 				"HTTP.Data.Listen.Port",
 				"HTTP.Data.Listen.Port.Override",
-				"HTTP.Data.Listen.Port.Enable" },
+				"HTTP.Data.Listen.Port.Enable",
+				"Tracker Client Min Announce Interval"
+				},
 			new ParameterListener()
 			{
 				public void 
 				parameterChanged(
 					String parameterName )
 				{
-					boolean socks_peer_inform	= 	
-						COConfigurationManager.getBooleanParameter("Proxy.Data.Enable", false)&&
-						COConfigurationManager.getBooleanParameter("Proxy.Data.SOCKS.inform", true );
-					
-		  				// we currently don't support incoming connections when SOCKs proxying
-			  		
-			 		int	tcp_port_num;
-			 		int	udp_port_num;
-			 		  		
-			  		if ( socks_peer_inform ){
-			  			
-			  			tcp_port_num	= 0;
-			  			udp_port_num	= 0;
-			  		}else{
-			  		
-			 			tcp_port_num	= COConfigurationManager.getIntParameter( "TCP.Listen.Port" );
-			 			udp_port_num	= COConfigurationManager.getIntParameter( "UDP.Listen.Port" );		
-			  		}
-			  		
-			  		String portOverride = COConfigurationManager.getStringParameter("TCP.Announce.Port","");
-			  		
-			  		if(! portOverride.equals("")) {
-			  		  
-			  			try{
-			  				tcp_port_num = Integer.parseInt( portOverride );
-			  				
-			  			}catch( Throwable e ){
-			  				
-			  				Debug.printStackTrace(e);
-			  			}
-			  		}
-			  		  		
-			 		String port = "";
-			 		
-			 		boolean require_crypto = COConfigurationManager.getBooleanParameter( "network.transport.encrypted.require");
-			 		
-			   		if ( require_crypto ){
-			  			
-			  			port += "&requirecrypto=1";
-			  			
-			  		}else{
-			  			
-						port += "&supportcrypto=1"; 
-			  		}
-			 		  
-			 		if ( 	require_crypto &&
-			 				(!COConfigurationManager.getBooleanParameter( "network.transport.encrypted.fallback.incoming") ) &&
-			 				COConfigurationManager.getBooleanParameter( "network.transport.encrypted.use.crypto.port" )){
-			 			
-			 			port += "&port=0&cryptoport=" + tcp_port_num;
-			 			
-			 		}else{
-			 
-			 			port += "&port=" + tcp_port_num;
-			 		}
-						
-					port += "&azudp=" + udp_port_num;
-			 		
-			  		  	//  BitComet extension for no incoming connections
-			  		
-			  		if ( tcp_port_num == 0 ){
-			  			
-			  			port += "&hide=1";
-			  		}	
-			  			  		
-			  		if ( COConfigurationManager.getBooleanParameter( "HTTP.Data.Listen.Port.Enable" )){
-			  			
-			  			int	http_port = COConfigurationManager.getIntParameter( "HTTP.Data.Listen.Port.Override" );
-			  			
-			  			if ( http_port == 0 ){
-			  				
-			  				http_port = COConfigurationManager.getIntParameter( "HTTP.Data.Listen.Port" );
-			  			}
-			  			
-			  			port += "&azhttp=" + http_port;
-			  		}
+					String	port 				= computePortsForURL( false, true );
+			  		String 	port_with_crypto 	= computePortsForURL( true, false );
 			  		
 			  		if ( ports_for_url != null && ( !ports_for_url.equals( port ))){
 			  			
@@ -174,10 +106,10 @@ TRTrackerUtils
 			  				if ( listener_thread == null ){
 			  					
 			  					listener_thread = 
-			  						new AEThread( "TRTrackerUtils:listener", true )
+			  						new AEThread2( "TRTrackerUtils:listener", true )
 			  						{
 			  							public void
-			  							runSupport()
+			  							run()
 			  							{
 			  								try{
 			  									Thread.sleep(30000);
@@ -208,11 +140,115 @@ TRTrackerUtils
 			  			}
 			  		}
 			  		
-			  		ports_for_url 			= port;
+			  		ports_for_url 				= port;
+			  		ports_for_url_with_crypto	= port_with_crypto;
 				}
 			});
 	}
 			
+	private static String
+	computePortsForURL(
+		boolean	force_crypto,
+		boolean	allow_incoming )
+	{
+		boolean socks_peer_inform	= 	
+			COConfigurationManager.getBooleanParameter("Proxy.Data.Enable")&&
+			COConfigurationManager.getBooleanParameter("Proxy.Data.SOCKS.inform");
+		
+				// we currently don't support incoming connections when SOCKs proxying
+		
+		allow_incoming &= !COConfigurationManager.getBooleanParameter("Tracker Client No Port Announce");
+  		
+ 		int	tcp_port_num;
+ 		int	udp_port_num;
+ 		  		
+ 		if ( allow_incoming ){
+ 			
+	  		if ( socks_peer_inform ){
+	  			
+	  			tcp_port_num	= 0;
+	  			udp_port_num	= 0;
+	  		}else{
+	  		
+	 			tcp_port_num	= COConfigurationManager.getIntParameter( "TCP.Listen.Port" );
+	 			udp_port_num	= COConfigurationManager.getIntParameter( "UDP.Listen.Port" );		
+	  		}
+	  		
+	  		String portOverride = COConfigurationManager.getStringParameter("TCP.Listen.Port.Override");
+	  		
+	  		if(! portOverride.equals("")) {
+	  		  
+	  			try{
+	  				tcp_port_num = Integer.parseInt( portOverride );
+	  				
+	  			}catch( Throwable e ){
+	  				
+	  				Debug.printStackTrace(e);
+	  			}
+	  		}
+ 		}else{
+ 			
+  			tcp_port_num	= 0;
+  			udp_port_num	= 0;
+ 		}
+ 		
+ 		String port = "";
+ 		
+ 		if ( force_crypto ){
+ 			
+  			port += "&requirecrypto=1";
+
+ 			port += "&port=0&cryptoport=" + tcp_port_num;
+
+ 		}else{
+ 			
+	 		boolean require_crypto = COConfigurationManager.getBooleanParameter( "network.transport.encrypted.require");
+	 		
+	   		if ( require_crypto ){
+	  			
+	  			port += "&requirecrypto=1";
+	  			
+	  		}else{
+	  			
+				port += "&supportcrypto=1"; 
+	  		}
+	 		  
+	 		if ( 	require_crypto &&
+	 				(!COConfigurationManager.getBooleanParameter( "network.transport.encrypted.fallback.incoming") ) &&
+	 				COConfigurationManager.getBooleanParameter( "network.transport.encrypted.use.crypto.port" )){
+	 			
+	 			port += "&port=0&cryptoport=" + tcp_port_num;
+	 			
+	 		}else{
+	 
+	 			port += "&port=" + tcp_port_num;
+	 		}
+				
+			port += "&azudp=" + udp_port_num;
+	 		
+	  		  	//  BitComet extension for no incoming connections
+	  		
+	  		if ( tcp_port_num == 0 ){
+	  			
+	  			port += "&hide=1";
+	  		}	
+	  			  		
+	  		if ( COConfigurationManager.getBooleanParameter( "HTTP.Data.Listen.Port.Enable" )){
+	  			
+	  			int	http_port = COConfigurationManager.getIntParameter( "HTTP.Data.Listen.Port.Override" );
+	  			
+	  			if ( http_port == 0 ){
+	  				
+	  				http_port = COConfigurationManager.getIntParameter( "HTTP.Data.Listen.Port" );
+	  			}
+	  			
+	  			port += "&azhttp=" + http_port;
+	  		}
+ 		}
+ 		
+ 		return( port );
+	}
+	
 	public static String
 	getPublicIPOverride()
 	{
@@ -243,6 +279,8 @@ TRTrackerUtils
 	
 	private static Map	az_trackers = COConfigurationManager.getMapParameter( "Tracker Client AZ Instances", new HashMap());
 	
+	private static Map	udp_probe_results = COConfigurationManager.getMapParameter( "Tracker Client UDP Probe Results", new HashMap());
+
 
 	static{
 	
@@ -280,6 +318,8 @@ TRTrackerUtils
 	{
 		tracker_ip 		= COConfigurationManager.getStringParameter("Tracker IP", "");
 	
+		tracker_ip = UrlUtils.expandIPV6Host( tracker_ip );
+		
 		String override_ips		= COConfigurationManager.getStringParameter("Override Ip", "");
 		
 		StringTokenizer	tok = new StringTokenizer( override_ips, ";" );
@@ -298,7 +338,7 @@ TRTrackerUtils
 		
 		override_map	= new_override_map;
 		
-		InetAddress bad = NetworkAdmin.getSingleton().getDefaultBindAddress();
+		InetAddress bad = NetworkAdmin.getSingleton().getSingleHomedServiceBindAddress();
 		
 		if ( bad == null ){
 			
@@ -315,7 +355,7 @@ TRTrackerUtils
 		URL		url_in )
 	{
 		return( tracker_ip.length() > 0  &&
-				url_in.getHost().equalsIgnoreCase( tracker_ip ));
+				UrlUtils.expandIPV6Host(url_in.getHost()).equalsIgnoreCase( tracker_ip ));
 	}
 	
 	public static String
@@ -339,20 +379,20 @@ TRTrackerUtils
 				
 		if ( tracker_host.length() > 0 ){
 			
-			if ( COConfigurationManager.getBooleanParameter( "Tracker Port Enable", false )){
+			if ( COConfigurationManager.getBooleanParameter( "Tracker Port Enable")){
 										
 				int port = COConfigurationManager.getIntParameter("Tracker Port", TRHost.DEFAULT_PORT );
 				
 				try{
 					List	l = new ArrayList();
 					
-					l.add( new URL( "http://" + tracker_host + ":" + port + "/announce" ));
+					l.add( new URL( "http://" + UrlUtils.convertIPV6Host( tracker_host ) + ":" + port + "/announce" ));
 					
 					List	ports = stringToPorts( COConfigurationManager.getStringParameter("Tracker Port Backups" ));
 					
 					for (int i=0;i<ports.size();i++){
 						
-						l.add( new URL( "http://" + tracker_host + ":" + ((Integer)ports.get(i)).intValue() + "/announce" ));
+						l.add( new URL( "http://" + UrlUtils.convertIPV6Host( tracker_host ) + ":" + ((Integer)ports.get(i)).intValue() + "/announce" ));
 					}
 
 					urls.add( l );
@@ -363,20 +403,20 @@ TRTrackerUtils
 				}
 			}
 			
-			if ( COConfigurationManager.getBooleanParameter( "Tracker Port SSL Enable", false )){
+			if ( COConfigurationManager.getBooleanParameter( "Tracker Port SSL Enable")){
 				
 				int port = COConfigurationManager.getIntParameter("Tracker Port SSL", TRHost.DEFAULT_PORT_SSL );
 				
 				try{
 					List	l = new ArrayList();
 					
-					l.add( new URL( "https://" + tracker_host + ":" + port + "/announce" ));
+					l.add( new URL( "https://" + UrlUtils.convertIPV6Host( tracker_host ) + ":" + port + "/announce" ));
 					
 					List	ports = stringToPorts( COConfigurationManager.getStringParameter("Tracker Port SSL Backups" ));
 					
 					for (int i=0;i<ports.size();i++){
 						
-						l.add( new URL( "https://" + tracker_host + ":" + ((Integer)ports.get(i)).intValue() + "/announce" ));
+						l.add( new URL( "https://" + UrlUtils.convertIPV6Host( tracker_host ) + ":" + ((Integer)ports.get(i)).intValue() + "/announce" ));
 					}
 
 					urls.add( l );
@@ -397,7 +437,7 @@ TRTrackerUtils
 				try{
 					List	l = new ArrayList();
 					
-					l.add( new URL( "udp://" + tracker_host + ":" + port + "/announce" +
+					l.add( new URL( "udp://" + UrlUtils.convertIPV6Host( tracker_host ) + ":" + port + "/announce" +
 										(auth?"?auth":"" )));
 				
 					urls.add( l );
@@ -566,13 +606,26 @@ TRTrackerUtils
   		return( ports_for_url );
   	}
  	
+ 	public static String
+ 	getPortsForURLFullCrypto()
+ 	{
+ 		return( ports_for_url_with_crypto );
+ 	}
+ 	
  	public static boolean
  	isAZTracker(
  		URL		tracker_url )
  	{
- 	   synchronized( az_trackers ){
+ 		String	host = tracker_url.getHost();
+ 		
+ 		if ( Constants.isAzureusDomain( host )){
+ 			
+ 			return( true );
+ 		}
+ 		
+ 		synchronized( az_trackers ){
  	    	
- 	    	return( az_trackers.containsKey( tracker_url.getHost() + ":" + tracker_url.getPort()));
+ 	    	return( az_trackers.containsKey( host + ":" + tracker_url.getPort()));
  	    }
  	}
  	
@@ -613,6 +666,67 @@ TRTrackerUtils
 		}
 	}
 	
+ 	public static boolean
+ 	isUDPProbeOK(
+ 		URL		tracker_url )
+ 	{
+ 		String	host = tracker_url.getHost();
+ 		
+ 		if ( Constants.isAzureusDomain( host )){
+ 			
+ 			return( false );
+ 		}
+ 		
+ 		synchronized( udp_probe_results ){
+ 	    	
+ 	    	return( udp_probe_results.containsKey( host ));
+ 	    }
+ 	}
+ 	
+	public static void
+ 	setUDPProbeResult(
+ 		URL			tracker_url,
+ 		boolean		probe_ok )
+	{
+		String	key = tracker_url.getHost();
+				
+		synchronized( udp_probe_results ){
+			
+			boolean	changed = false;
+			
+			if ( udp_probe_results.get( key ) == null ){
+			
+				if ( probe_ok ){
+					
+						// arbitrary max size here just in case something weird happens
+					
+					if ( udp_probe_results.size() > 512 ){
+						
+						udp_probe_results.clear();
+					}
+					
+					udp_probe_results.put( key, new Long( SystemTime.getCurrentTime()));
+					
+					changed	= true;
+				}
+			}else{
+				
+				if ( !probe_ok ){
+					
+					if ( udp_probe_results.remove( key ) != null ){
+					
+						changed = true;
+					}
+				}
+			}
+			
+			if ( changed ){
+				
+				COConfigurationManager.setParameter( "Tracker Client UDP Probe Results", udp_probe_results );
+			}
+		}
+	}
+	
 	public static void
 	addListener(
 		TRTrackerUtilsListener	l )
