@@ -23,7 +23,13 @@
 package org.gudy.azureus2.core3.util;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.logging.*;
+import org.gudy.azureus2.platform.PlatformManager;
+import org.gudy.azureus2.platform.PlatformManagerCapabilities;
+import org.gudy.azureus2.platform.PlatformManagerFactory;
+
+import com.aelitis.azureus.core.util.Java15Utils;
 
 /**
  * @author parg
@@ -31,6 +37,7 @@ import org.gudy.azureus2.core3.logging.*;
  */
 
 import java.io.*;
+import java.lang.reflect.Method;
 import java.util.*;
 
 public class 
@@ -109,20 +116,38 @@ AEDiagnostics
 		started_up	= true;
 		
 		try{
+			// Minimize risk of loading to much when in transitory startup mode
+			boolean transitoryStartup = System.getProperty("transitory.startup", "0").equals("1");
+			if (transitoryStartup) {
+				// no vivaldi and Thread monitor for you!
+				load_15_tried = true;
+				// no xxx_?.log logging for you!
+				loggers_enabled = false;
+				// skip tidy check and more!
+				return;
+			}
+
 			debug_dir		= FileUtil.getUserFile( "logs" );
 			
 			debug_save_dir	= new File( debug_dir, "save" );
 			
-			loggers_enabled = COConfigurationManager.getBooleanParameter( "Logger.DebugFiles.Enabled", true );
+			loggers_enabled = COConfigurationManager.getBooleanParameter( "Logger.DebugFiles.Enabled");
 
 			boolean	was_tidy	= COConfigurationManager.getBooleanParameter( CONFIG_KEY );
 			
-			SimpleTimer.addEvent("AEDiagnostics:logCleaner",SystemTime.getCurrentTime() + 60000
-					+ (int) (Math.random() * 15000), new TimerEventPerformer() {
-				public void perform(TimerEvent event) {
-					cleanOldLogs();
+			new AEThread2( "asyncify", true )
+			{
+				public void
+				run()
+				{
+					SimpleTimer.addEvent("AEDiagnostics:logCleaner",SystemTime.getCurrentTime() + 60000
+							+ (int) (Math.random() * 15000), new TimerEventPerformer() {
+						public void perform(TimerEvent event) {
+							cleanOldLogs();
+						}
+					});
 				}
-			});
+			}.start();
 
 			if ( debug_dir.exists()){
 				
@@ -193,14 +218,31 @@ AEDiagnostics
 		// pull in the JDK1.5 monitoring stuff if present
 			
 		try{
-			Class.forName( "com.aelitis.azureus.jdk15.Java15Initialiser" );
+			Class c = Class.forName( "com.aelitis.azureus.jdk15.Java15Initialiser" );
 					
+			if ( c != null ){
+				
+				Method m = c.getDeclaredMethod( "getUtilsProvider", new Class[0] );
+				
+				Java15Utils.Java15UtilsProvider provider = (Java15Utils.Java15UtilsProvider)m.invoke( null, new Object[0] );
+				
+				if ( provider != null ){
+					
+					 Java15Utils.setProvider( provider );
+				}
+			}
 			// System.out.println( "**** AEThread debug on ****" );
 
 		}catch( Throwable e ){
 		}
 	}
 	
+	public static void
+	dumpThreads()
+	{
+		Java15Utils.dumpThreads();
+	}
+	
 	/**
 	 * 
 	 */
@@ -222,7 +264,7 @@ AEDiagnostics
 
 						long last_modified = file.lastModified();
 
-						if (now - last_modified > 30 * 24 * 60 * 60 * 1000L) {
+						if (now - last_modified > 10 * 24 * 60 * 60 * 1000L) {
 
 							file.delete();
 						}
@@ -240,6 +282,14 @@ AEDiagnostics
 		return( startup_complete );
 	}
 	
+	public static File
+	getLogDir()
+	{
+		startup();
+
+		return( debug_dir );
+	}
+	
 	public static synchronized AEDiagnosticsLogger
 	getLogger(
 		String		name )
@@ -275,6 +325,7 @@ AEDiagnostics
 			}
 			
 			loggers.put( name, logger );
+			
 		}
 		
 		return( logger );
@@ -325,7 +376,7 @@ AEDiagnostics
 			}
 			
 			Calendar now = GregorianCalendar.getInstance();
-	        
+
 			String timeStamp =
 				"[" + format(now.get(Calendar.DAY_OF_MONTH))+format(now.get(Calendar.MONTH)+1) + " " + 
 				format(now.get(Calendar.HOUR_OF_DAY))+ ":" + format(now.get(Calendar.MINUTE)) + ":" + format(now.get(Calendar.SECOND)) + "] ";        
@@ -337,6 +388,13 @@ AEDiagnostics
 			try{		
 						
 				pw = new PrintWriter(new FileWriter( log_file, true ));
+				
+				if (!logger.isWrittenToThisSession()) {
+					logger.setWrittenToThisSession(true);
+					pw.println("\n\n[" + now.get(Calendar.YEAR)
+							+ "] Log File Opened for " + Constants.AZUREUS_NAME + " "
+							+ Constants.AZUREUS_VERSION + "\n");
+				}
 			
 				pw.println( str );
 		 							
@@ -427,6 +485,208 @@ AEDiagnostics
 		}
 	}
 	
+	private static final String[][]
+	   	bad_dlls = { 
+			{	"niphk", 			"y", },
+	   		{	"nvappfilter", 		"y", },
+	   		{	"netdog", 			"y", },
+	   		{	"vlsp", 			"y", },
+	   		{	"imon", 			"y", },
+	   		{	"sarah", 			"y", },
+	   		{	"MxAVLsp", 			"y", },
+	   		{	"mclsp", 			"y", },
+	   		{	"radhslib", 		"y", },
+	   		{	"winsflt",			"y", },
+	   		{	"nl_lsp",			"y", },
+	   		{	"AxShlex",			"y", },
+	   		{	"iFW_Xfilter",		"y", },
+	   		{	"gapsp",			"y", },
+	   		{	"WSOCKHK",			"n", },
+	};
+
+	public static void
+	checkDumpsAndNatives()
+	{
+		try{
+			PlatformManager	p_man = PlatformManagerFactory.getPlatformManager();
+			
+			if ( 	p_man.getPlatformType() == PlatformManager.PT_WINDOWS &&
+					p_man.hasCapability( PlatformManagerCapabilities.TestNativeAvailability )){	
+
+				for (int i=0;i<bad_dlls.length;i++){
+					
+					String	dll 	= bad_dlls[i][0];
+					String	load	= bad_dlls[i][1];
+					
+					if ( load.equalsIgnoreCase( "n" )){
+						
+						continue;
+					}
+					
+					if ( !COConfigurationManager.getBooleanParameter( "platform.win32.dll_found." + dll, false )){
+								
+						try{
+							if ( p_man.testNativeAvailability( dll + ".dll" )){
+								
+								COConfigurationManager.setParameter( "platform.win32.dll_found." + dll, true );
+	
+								String	detail = MessageText.getString( "platform.win32.baddll." + dll );
+								
+								Logger.logTextResource(
+										new LogAlert(
+												LogAlert.REPEATABLE, 
+												LogAlert.AT_WARNING,
+												"platform.win32.baddll.info" ),	
+										new String[]{ dll + ".dll", detail });
+							}
+				
+						}catch( Throwable e ){
+							
+							Debug.printStackTrace(e);
+						}
+					}
+				}
+			}
+			
+			File app_dir = new File( SystemProperties.getApplicationPath());
+			
+			if ( app_dir.canRead()){
+				
+				File[]	files = app_dir.listFiles();
+				
+				File	most_recent_dump 	= null;
+				long	most_recent_time	= 0;
+				
+				long	now = SystemTime.getCurrentTime();
+				
+				long	one_week_ago = now - 7*24*60*60*1000;
+				
+				for (int i=0;i<files.length;i++){
+					
+					File	f = files[i];
+					
+					String	name = f.getName();
+					
+					if ( name.startsWith( "hs_err_pid" )){
+						
+						long	last_mod = f.lastModified();
+						
+						if ( last_mod > most_recent_time && last_mod > one_week_ago){
+							
+							most_recent_dump 	= f;
+							most_recent_time	= last_mod;
+						}
+					}
+				}
+				
+				if ( most_recent_dump!= null ){
+					
+					long	last_done = 
+						COConfigurationManager.getLongParameter( "diagnostics.dump.lasttime", 0 ); 
+					
+					if ( last_done < most_recent_time ){
+						
+						COConfigurationManager.setParameter( "diagnostics.dump.lasttime", most_recent_time );
+						
+						analyseDump( most_recent_dump );
+					}
+				}
+			}
+		}catch( Throwable e ){
+			
+			Debug.printStackTrace(e);
+		}
+	}
+	
+	protected static void
+	analyseDump(
+		File	file )
+	{
+		System.out.println( "Analysing " + file );
+		
+		try{
+			LineNumberReader lnr = new LineNumberReader( new FileReader( file ));
+			
+			try{
+				boolean	float_excep	= false;
+				
+				String[]	bad_dlls_uc = new String[bad_dlls.length];
+				
+				for (int i=0;i<bad_dlls.length;i++){
+					
+					String	dll 	= bad_dlls[i][0];
+
+					bad_dlls_uc[i] = (dll + ".dll" ).toUpperCase();
+				}
+				
+				String	alcohol_dll = "AxShlex";
+				
+				List	matches = new ArrayList();
+				
+				while( true ){
+					
+					String	line = lnr.readLine();
+					
+					if ( line == null ){
+						
+						break;
+					}
+					
+					line = line.toUpperCase();
+					
+					if (line.indexOf( "EXCEPTION_FLT") != -1 ){
+						
+						float_excep	= true;
+						
+					}else{
+						
+						for (int i=0;i<bad_dlls_uc.length;i++){
+							
+							String b_uc = bad_dlls_uc[i];
+							
+							if ( line.indexOf( b_uc ) != -1 ){
+								
+								String	dll = bad_dlls[i][0];
+								
+								if ( dll.equals( alcohol_dll )){
+									
+									if ( float_excep ){
+										
+										matches.add( dll );
+									}
+									
+								}else{
+									
+									matches.add( dll );
+								}
+							}
+						}
+					}
+				}
+				
+				for (int i=0;i<matches.size();i++){
+					
+					String	dll = (String)matches.get(i);
+					
+					String	detail = MessageText.getString( "platform.win32.baddll." + dll );
+					
+					Logger.logTextResource(
+							new LogAlert(
+									LogAlert.REPEATABLE, 
+									LogAlert.AT_WARNING,
+									"platform.win32.baddll.info" ),	
+							new String[]{ dll + ".dll", detail });
+				}
+			}finally{
+				
+				lnr.close();
+			}
+		}catch( Throwable e){
+			
+			Debug.printStackTrace( e );
+		}
+	}
+	
 	public static void
 	addEvidenceGenerator(
 		AEDiagnosticsEvidenceGenerator	gen )
