@@ -31,6 +31,7 @@ AEDiagnosticsLogger
 {
 	protected String 		name;
 	protected boolean		first_file	= true;
+	protected boolean		writtenToThisSession = false;
 	
 	protected
 	AEDiagnosticsLogger(
@@ -101,4 +102,17 @@ AEDiagnosticsLogger
 		
 		log( e );
 	}
+
+	public boolean
+	isWrittenToThisSession()
+	{
+		return writtenToThisSession;
+	}
+
+	public void 
+	setWrittenToThisSession(
+			boolean writtenToThisSession ) 
+	{
+		this.writtenToThisSession = writtenToThisSession;
+	}
 }
