@@ -62,7 +62,7 @@ public class Average {
 
     this.nbElements = (_period * 1000) / _refreshRate + 2;
     this.lastUpdate = getEffectiveTime() / _refreshRate;
-    this.values = new long[this.nbElements];
+    //this.values = new long[this.nbElements];
   }
 
   /**
@@ -88,20 +88,24 @@ public class Average {
    * @param timeFactor which is the currentTime divided by the refresh Rate
    */
   private synchronized void update(long timeFactor) {
-    //If we have a really OLD lastUpdate, we could erase the buffer a 
-    //huge number of time, so if it's really old, we change it so we'll only
-    //erase the buffer once.
-    if (lastUpdate < timeFactor - nbElements)
-      lastUpdate = timeFactor - nbElements - 1;
-
-    //For all values between lastUpdate + 1 (next value than last updated)
-    //and timeFactor (which is the new value insertion position) 
-    for (long i = lastUpdate + 1; i <= timeFactor; i++) {
-      //We set the value to 0.
-      values[(int) (i % nbElements)] = 0;
-    }
-    //We also clear the next value to be inserted (so on next time change...)
-    values[(int) ((timeFactor + 1) % nbElements)] = 0;
+	    //If we have a really OLD lastUpdate, we could erase the buffer a 
+	    //huge number of time, so if it's really old, we change it so we'll only
+	    //erase the buffer once.
+	  
+	    if (lastUpdate < timeFactor - nbElements)
+		      lastUpdate = timeFactor - nbElements - 1;
+	    
+	  if(values != null)
+	  {
+		    //For all values between lastUpdate + 1 (next value than last updated)
+		    //and timeFactor (which is the new value insertion position) 
+		    for (long i = lastUpdate + 1; i <= timeFactor; i++) {
+		      //We set the value to 0.
+		      values[(int) (i % nbElements)] = 0;
+		    }
+		    //We also clear the next value to be inserted (so on next time change...)
+		    values[(int) ((timeFactor + 1) % nbElements)] = 0;
+	  }
 
     //And we update lastUpdate.
     lastUpdate = timeFactor;
@@ -113,12 +117,18 @@ public class Average {
    * @param value the value to be added to the Average
    */
   public void addValue(long value) {
-    //We get the current time factor.
-    long timeFactor = getEffectiveTime() / refreshRate;
-    //We first update the buffer.
-    update(timeFactor);
-    //And then we add our value to current element
-    values[(int) (timeFactor % nbElements)] += value;
+    if(values == null && value != 0)
+    	values = new long[nbElements];
+    if(values != null)
+    {
+        //We get the current time factor.
+        long timeFactor = getEffectiveTime() / refreshRate;
+        //We first update the buffer.
+        update(timeFactor);
+        //And then we add our value to current element
+    	values[(int) (timeFactor % nbElements)] += value;
+    }
+    	
   }
 
   /**
@@ -137,29 +147,99 @@ public class Average {
 	  return( DisplayFormatters.formatDecimal( getDoubleAverage(), precision ));
   }
   
-  protected final long getSum() {
-    //We get the current timeFactor
-    long timeFactor = getEffectiveTime() / refreshRate;
-    //We first update the buffer
-    update(timeFactor);
+  public long getAverage(int average_period )
+  {
+	  int	slots = average_period<=0?(nbElements - 2):(average_period / refreshRate);
+	  
+	  if ( slots <= 0 ){
+		  
+		  slots = 1;
+		  
+	  }else if ( slots > nbElements - 2 ){
+		  
+		  slots = nbElements - 2;
+	  }
+	 
+	  if ( slots == 1 ){
+		 
+		  return( getPointValue());
+	  }
+	  
+	  long res = getSum(slots) / ( period * slots / ( nbElements - 2 ));
+	  
+	  return( res );
+  }
+  
+  public long
+  getPointValue()
+  {
+	  long timeFactor = getEffectiveTime() / refreshRate;
+	  //We first update the buffer
+	  update(timeFactor);
 
+	  return(values != null ? values[(int)((timeFactor-1)% nbElements)] : 0);
+  }
+  
+  protected final long getSum() {
     //The sum of all elements used for the average.
     long sum = 0;
-
-    //Starting on oldest one (the one after the next one)
-    //Ending on last one fully updated (the one previous current one)
-    for (long i = timeFactor + 2; i < timeFactor + nbElements; i++) {
-      //Simple addition
-      sum += values[(int) (i % nbElements)];
+    
+    if(values != null)
+    {
+        //We get the current timeFactor
+        long timeFactor = getEffectiveTime() / refreshRate;
+        //We first update the buffer
+        update(timeFactor);
+        //Starting on oldest one (the one after the next one)
+        //Ending on last one fully updated (the one previous current one)
+        for (long i = timeFactor + 2; i < timeFactor + nbElements; i++) {
+          //Simple addition
+          sum += values[(int) (i % nbElements)];
+        }
     }
 
+
     //We return the sum divided by the period
     return(sum);
   }
   
+  protected final long getSum(int slots) {
+	    //We get the current timeFactor
+	    long timeFactor = getEffectiveTime() / refreshRate;
+	    //We first update the buffer
+	    update(timeFactor);
+
+	    //The sum of all elements used for the average.
+	    long sum = 0;
+	    
+	    if ( slots < 1 ){
+	    	
+	    	slots = 1;
+	    	
+	    }else if ( slots > nbElements-2 ){
+	    	
+	    	slots = nbElements-2;
+	    }
+	    
+	    //Starting on oldest one (the one after the next one)
+	    //Ending on last one fully updated (the one previous current one)
+	    
+	    long end_slot 	= timeFactor + nbElements;
+	    long start_slot = end_slot - slots;
+	    
+	    if (values != null)
+			for (long i = start_slot; i < end_slot; i++)
+			{
+				sum += values[(int) (i % nbElements)];
+			}
+
+	    //We return the sum divided by the period
+	    return(sum);
+	  }
+  
   protected long
   getEffectiveTime()
   {
-	  return( SystemTime.getCurrentTime());
+	  return( SystemTime.getMonotonousTime());
   }
 }
