@@ -32,16 +32,27 @@ import java.util.StringTokenizer;
 public class 
 Constants 
 {
-  public static final String SF_WEB_SITE		= "http://azureus.sourceforge.net/";
-  public static final String AELITIS_WEB_SITE   = "http://azureus.aelitis.com/";
-  public static final String GETAZUREUS_WEB_SITE= "http://www.getazureus.com/";
+  public static final String SF_WEB_SITE			= "http://azureus.sourceforge.net/";
+  public static final String AELITIS_WEB_SITE   	= "http://azureus.aelitis.com/";
+  public static final String GETAZUREUS_WEB_SITE	= "http://www.getazureus.com/";
   
-  public static final String AELITIS_TORRENTS	= "http://torrents.aelitis.com:88/torrents/";
-  public static final String AZUREUS_WIKI 		= "http://www.azureuswiki.com/index.php/";
+  public static final String AELITIS_TORRENTS		= "http://torrents.aelitis.com:88/torrents/";
+  public static final String AELITIS_FILES			= "http://torrents.aelitis.com:88/files/";
+  public static final String AZUREUS_WIKI 			= "http://www.azureuswiki.com/index.php/";
+  
+  public static final String  VERSION_SERVER_V4 	= "version.azureusplatform.com";
+  public static final String  VERSION_SERVER_V6 	= "version6.azureusplatform.com";
+
+  public static final String DHT_SEED_ADDRESS_V4	= "dht.aelitis.com";
+  public static final String DHT_SEED_ADDRESS_V6	= "dht6.azureusplatform.com";
   
   public static final String NAT_TEST_SERVER		= "nettest.azureusplatform.com";
   public static final String NAT_TEST_SERVER_HTTP	= "http://nettest.azureusplatform.com/";
    
+  public static final String SPEED_TEST_SERVER		= "speed.azureusplatform.com";
+  
+  public static final String[] AZUREUS_DOMAINS = { "azureusplatform.com", "azureus.com", "aelitis.com" };
+  
   public static final String DEFAULT_ENCODING 	= "UTF8";
   public static final String BYTE_ENCODING 		= "ISO-8859-1";
   public static Charset	BYTE_CHARSET;
@@ -59,16 +70,18 @@ Constants
   }
   
   public static final String INFINITY_STRING	= "\u221E"; // "oo";pa  
-  public static final int    INFINITY_AS_INT = 31536000; // seconds (365days)
+  public static final int    INFINITY_AS_INT = 365*24*3600; // seconds (365days)
+  public static final long   INFINITE_AS_LONG = 10000*365*24*3600; // seconds (10k years)
   
   	// keep the CVS style constant coz version checkers depend on it!
   	// e.g. 2.0.8.3
     //      2.0.8.3_CVS
     //      2.0.8.3_Bnn       // incremental build
   
+  public static String APP_NAME = "Vuze";
   public static final String AZUREUS_NAME	  = "Azureus";
-  public static final String AZUREUS_VERSION  = "2.5.0.4";  //3.0.0.7_CVS
-  public static final byte[] VERSION_ID       = ("-" + "AZ" + "2504" + "-").getBytes();  //MUST be 8 chars long!
+  public static final String AZUREUS_VERSION  = "3.1.1.0";  //3.1.1.1_CVS
+  public static final byte[] VERSION_ID       = ("-" + "AZ" + "3110" + "-").getBytes();  //MUST be 8 chars long!
   
   
   public static final String  OSName = System.getProperty("os.name");
@@ -78,6 +91,7 @@ Constants
   public static final boolean isSolaris			= OSName.equalsIgnoreCase("SunOS");
   public static final boolean isFreeBSD			= OSName.equalsIgnoreCase("FreeBSD");
   public static final boolean isWindowsXP		= OSName.equalsIgnoreCase("Windows XP");
+  public static final boolean isWindowsVista 	= OSName.equalsIgnoreCase("Windows Vista");
   public static final boolean isWindows95		= OSName.equalsIgnoreCase("Windows 95");
   public static final boolean isWindows98		= OSName.equalsIgnoreCase("Windows 98");
   public static final boolean isWindowsME		= OSName.equalsIgnoreCase("Windows ME");
@@ -237,6 +251,30 @@ Constants
 		}
 	}
 	
+	public static boolean
+	isAzureusDomain(
+		String	host )
+	{
+		host = host.toLowerCase();
+
+		for (int i=0; i<AZUREUS_DOMAINS.length; i++) {
+
+			String domain = (String) AZUREUS_DOMAINS[i];
+
+			if ( domain.equals( host )){
+
+				return( true );
+			}
+
+			if ( host.endsWith("." + domain)){
+
+				return( true );
+			}
+		}
+		
+		return( false );
+	}
+	
 	public static void main(String[] args) {
 		System.out.println(compareVersions("3.0.0.1", "3.0.0.0"));
 		System.out.println(compareVersions("3.0.0.0_B1", "3.0.0.0"));
