@@ -164,33 +164,42 @@ public class Debug {
 		killAWTThreads( threadGroup );
 	}
 
-	private static String getCompressedStackTrace(Throwable t,
-			int frames_to_skip) {
+	private static String 
+	getCompressedStackTrace(
+		Throwable t,
+		int frames_to_skip) 
+	{
 		return getCompressedStackTrace(t, frames_to_skip, 200);
 	}
 
-	private static String getCompressedStackTrace(Throwable t,
-			int frames_to_skip, int iMaxLines) {
+	public static String 
+	getCompressedStackTrace(
+		Throwable t,
+		int frames_to_skip, 
+		int iMaxLines) 
+	{
 		String sStackTrace = "";
-  	StackTraceElement[]	st = t.getStackTrace();
-	   
-  	int iMax = Math.min(st.length, iMaxLines + frames_to_skip);
-    for (int i = frames_to_skip; i < iMax; i++) {
-    	
-    	if (i > frames_to_skip)
-    		sStackTrace += ",";
-    	
-    	String cn = st[i].getClassName();
-    	cn = cn.substring( cn.lastIndexOf(".")+1);
-    	
-    	sStackTrace += cn +"::"+st[i].getMethodName()+"::"+st[i].getLineNumber();
-    }
-    
-    if (t.getCause() != null) {
-    	sStackTrace += "\n\tCaused By: " + getCompressedStackTrace(t, 0);
-    }
-    
-    return sStackTrace;
+		StackTraceElement[]	st = t.getStackTrace();
+
+		int iMax = Math.min(st.length, iMaxLines + frames_to_skip);
+		for (int i = frames_to_skip; i < iMax; i++) {
+
+			if (i > frames_to_skip)
+				sStackTrace += ",";
+
+			String cn = st[i].getClassName();
+			cn = cn.substring( cn.lastIndexOf(".")+1);
+
+			sStackTrace += cn +"::"+st[i].getMethodName()+"::"+st[i].getLineNumber();
+		}
+
+		Throwable cause = t.getCause();
+
+		if (cause != null) {
+			sStackTrace += "\n\tCaused By: " + getCompressedStackTrace(cause, 0);
+		}
+
+		return sStackTrace;
 	}
 
 	public static String getStackTrace(boolean bCompressed, boolean bIncludeSelf) {
@@ -345,16 +354,20 @@ public class Debug {
 				this_message = e.getMessage();
 			}
 			
-			if ( this_message == null || this_message.length() == 0 ){
+				// if no exception message then pick up class name. if we have a deliberate
+				// zero length string then we assume that the exception can be ignored for
+				// logging purposes as it is just delegating
+			
+			if ( this_message == null ){
 				
 				this_message = e.getClass().getName();
 				
 				int	pos = this_message.lastIndexOf(".");
 				
-				this_message = this_message.substring( pos+1 );
+				this_message = this_message.substring( pos+1 ).trim();
 			}
-			
-			if ( last_message.indexOf( this_message ) == -1 ){
+						
+			if ( this_message.length() > 0 && last_message.indexOf( this_message ) == -1 ){
 				
 				last_message	= this_message + ( last_message.length()==0?"":(", " + last_message ));
 			}
@@ -369,7 +382,7 @@ public class Debug {
 	getNestedExceptionMessageAndStack(
 		Throwable 		e )
 	{
-		return( getNestedExceptionMessage(e) + ", " + getCompressedStackTrace( e, 2 ));
+		return( getNestedExceptionMessage(e) + ", " + getCompressedStackTrace( e, 0 ));
 	}
 	
 	public static String
@@ -377,7 +390,13 @@ public class Debug {
 	{
 		return( getCompressedStackTrace( new Throwable(), 2 ));
 	}
-	
+
+	public static String
+	getCompressedStackTrace(int iMaxLines)
+	{
+		return( getCompressedStackTrace( new Throwable(), 2, iMaxLines ));
+	}
+
 	public static String
 	getExceptionMessage(
 		Throwable	e )
@@ -403,9 +422,15 @@ public class Debug {
 		return( message );
 	}
 	
+	public static void printStackTrace(Throwable e) {
+		printStackTrace(e, null);
+	}
+
+	
 	public static void
 	printStackTrace(
-		Throwable e )
+		Throwable e,
+		Object context)
 	{
 		String header = "DEBUG::";
 		header = header + new Date(SystemTime.getCurrentTime()).toString() + "::";
@@ -427,7 +452,7 @@ public class Debug {
 		        	// skip stuff generated by the logger
 		        
 		        if ( 	className.indexOf( ".logging." ) != -1 ||
-		        		className.endsWith( ".Debug" )){
+		        		className.endsWith( ".Debug::" )){
 		        	
 		        	continue;
 		        }
@@ -443,13 +468,15 @@ public class Debug {
 			
 			PrintWriter	pw = new PrintWriter( new OutputStreamWriter( baos ));
 			
+			if (context!=null) {pw.print("  "); pw.println(context);}
+			pw.print("  ");
 			e.printStackTrace( pw );
 			
 			pw.close();
 			
 			String	stack = baos.toString();
 					    
-			diagLoggerLogAndOut("  " + stack, true );
+			diagLoggerLogAndOut(stack, true );
 		}catch( Throwable ignore ){
 			
 			e.printStackTrace();
@@ -518,7 +545,7 @@ public class Debug {
 			return "";
 
 		final String sep = File.separator;
-		final String regex = "([\\" + sep + "][^\\" + sep + "]{0,3}+)[^\\" + sep
+		final String regex = "([\\" + sep + "]?[^\\" + sep + "]{0,3}+)[^\\" + sep
 				+ "]*";
 		
 		String secretName = key.replaceAll(regex, "$1");
@@ -527,4 +554,9 @@ public class Debug {
 			secretName += key.substring(iExtensionPos); 
 		return secretName;
 	}
+
+	public static void main(String[] args) {
+		System.out.println(secretFileName("c:\\temp\\hello there.txt"));
+		System.out.println(secretFileName("hello there.txt"));
+	}
 }
