@@ -67,6 +67,10 @@ DirectByteBuffer
     public static final byte        AL_MSG_AZ_PAYLOAD     = 24;
     public static final byte        AL_FILE				  = 25;
     public static final byte        AL_NET_CRYPT		  = 26;
+    public static final byte        AL_MSG_LT_EXT_MESSAGE = 27;
+    public static final byte        AL_MSG_LT_HANDSHAKE   = 28;
+    public static final byte        AL_MSG_UT_PEX         = 29;
+    public static final byte        AL_MSG_BT_DHT_PORT    = 30;
     
 	public static final String[] AL_DESCS =
 	{ "None", "Ext", "Other", "PeerRead", "PeerLen",
@@ -85,7 +89,8 @@ DirectByteBuffer
     "BTPayload",
     "AZPayload",
     "File",
-    "MsgCrypt"};
+    "MsgCrypt",
+    "LTExtMsg","LTExtHandshake","UTPEX", "BTDHTPort"};
 	 
     
     
@@ -133,13 +138,16 @@ DirectByteBuffer
 	public static final byte		OP_READ_SC			= 21;
 	public static final byte		OP_WRITE_SC			= 22;
 	public static final byte		OP_GETBUFFER		= 23;
+	public static final byte		OP_GETSHORT			= 24;
+	public static final byte		OP_PUTSHORT			= 25;
 	
 	public static final String[]	OP_DESCS = 
 		{ 	"limit", 		"limit(int)", 	"position", 	"position(int)", 	"clear", 
 			"flip", 		"remaining", 	"capacity", 	"put(byte[])", 		"put(dbb)",
 			"put(bbb)", 	"putInt", 		"put(byte)",	"get",				"get(int)",
 			"get(byte[])",	"getInt",		"getInt(int",	"hasRemaining",		"read(fc)",		
-			"write(fc)",	"read(sc)",		"write(sc)",	"getBuffer",
+			"write(fc)",	"read(sc)",		"write(sc)",	"getBuffer",		"getShort",
+			"putShort",
 		};
 			
 	protected static final boolean	TRACE				= AEDiagnostics.TRACE_DIRECT_BYTE_BUFFERS;
@@ -149,6 +157,7 @@ DirectByteBuffer
 	private ByteBuffer 				buffer;
 	private DirectByteBufferPool	pool;
 	private byte					allocator;
+	private boolean                 was_returned_to_pool = false;
   
 	
 	//private byte[]					trace_buffer;
@@ -167,6 +176,7 @@ DirectByteBuffer
 		ByteBuffer 				_buffer,
 		DirectByteBufferPool	_pool ) 
 	{
+		if (_buffer == null) {throw new NullPointerException("buffer is null");}
 		allocator	= _allocator;
 		buffer 		= _buffer;
 		pool		= _pool;
@@ -179,7 +189,31 @@ DirectByteBuffer
 			*/
 		}
 	}
-  
+	
+		/**
+		 * constructor for reference counted version
+		 * @param basis
+		 */
+	
+	protected
+	DirectByteBuffer(
+		DirectByteBuffer		basis )
+	{
+		allocator	= basis.allocator;
+		buffer 		= basis.buffer;
+		pool		= null;
+		
+		if (buffer == null) {throw new NullPointerException("basis.buffer is null");}
+	}
+	
+	public ReferenceCountedDirectByteBuffer
+	getReferenceCountedBuffer()
+	{
+		ReferenceCountedDirectByteBuffer res = new ReferenceCountedDirectByteBuffer( this );
+				
+		return( res );
+	}
+	
 	protected void
 	traceUsage(
 		byte		subsystem,
@@ -307,7 +341,9 @@ DirectByteBuffer
 			
 			traceUsage( subsystem, OP_LIMIT_INT );
 		}
-		
+		if (buffer == null) {
+			System.out.println("Trying to limit null buffer - returned? " + this.was_returned_to_pool);
+		}
 		buffer.limit(l);
 	}
   
@@ -436,6 +472,19 @@ DirectByteBuffer
 		buffer.put( data );
 	}
   
+	public void
+	putShort(
+		byte		subsystem,
+		short		x )
+	{
+		if ( TRACE ){
+			
+			traceUsage( subsystem, OP_PUTSHORT);
+		}
+		
+		buffer.putShort( x );
+	}
+	
 	public void
 	putInt(
 		byte		subsystem,
@@ -487,6 +536,18 @@ DirectByteBuffer
 		buffer.get(data);
 	}
   
+	public short
+	getShort(
+		byte		subsystem )
+	{
+		if ( TRACE ){
+			
+			traceUsage( subsystem, OP_GETSHORT);
+		}
+		
+		return( buffer.getShort());
+	}
+	
 	public int
 	getInt(
 		byte		subsystem )
@@ -628,7 +689,6 @@ DirectByteBuffer
 		return( buffer );
 	}
   
-
 	public void 
 	returnToPool() 
 	{				
@@ -649,6 +709,36 @@ DirectByteBuffer
 					pool.returnBuffer( this );
 					
 					buffer	= null;
+					was_returned_to_pool = true;
+				}
+			}
+		}
+	}
+	
+		/**
+		 * Normally you should know when a buffer is/isn't free and NOT CALL THIS METHOD
+		 * However, there are some error situations where the existing code doesn't correctly
+		 * manage things - we know this and don't want spurious logs occuring as per the above
+		 * normal method
+		 */
+	
+	public void 
+	returnToPoolIfNotFree() 
+	{				
+		if ( pool != null ){
+				
+				// we can't afford to return a buffer more than once to the pool as it'll get
+				// handed out twice in parallel and cause weird problems. We haven't been able
+				// to totally eliminiate duplicate returnToPool calls....
+			
+			synchronized( this ){
+				
+				if ( buffer != null ){
+	    	
+					pool.returnBuffer( this );
+					
+					buffer	= null;
+					was_returned_to_pool = true;
 				}
 			}
 		}
