@@ -43,7 +43,7 @@ public class
 DisplayFormatters
 {
 	final private static boolean ROUND_NO = true;
-	final private static boolean ROUND_YES = false;
+	//final private static boolean ROUND_YES = false;
 	final private static boolean TRUNCZEROS_NO = false;
 	final private static boolean TRUNCZEROS_YES = true;
 	
@@ -65,6 +65,7 @@ DisplayFormatters
 	private static NumberFormat	percentage_format;
 
 	private static String[] units;
+	private static String[] units_bits;
 	private static String[] units_rate;
 	private static int unitsStopAt = UNIT_TB;
 
@@ -82,7 +83,7 @@ DisplayFormatters
     private static boolean	data_stats_only;
     
 	static{
-		use_si_units = COConfigurationManager.getBooleanParameter("config.style.useSIUnits", false);
+		use_si_units = COConfigurationManager.getBooleanParameter("config.style.useSIUnits");
 
 		COConfigurationManager.addParameterListener( "config.style.useSIUnits",
 				new ParameterListener()
@@ -91,13 +92,13 @@ DisplayFormatters
 					parameterChanged(
 						String	value )
 					{
-						use_si_units = COConfigurationManager.getBooleanParameter("config.style.useSIUnits", false);
+						use_si_units = COConfigurationManager.getBooleanParameter("config.style.useSIUnits");
 
 						setUnits();
 					}
 				});
 
-		use_units_rate_bits = COConfigurationManager.getBooleanParameter("config.style.useUnitsRateBits", false);
+		use_units_rate_bits = COConfigurationManager.getBooleanParameter("config.style.useUnitsRateBits");
 
 		COConfigurationManager.addParameterListener( "config.style.useUnitsRateBits",
 				new ParameterListener()
@@ -106,13 +107,13 @@ DisplayFormatters
 					parameterChanged(
 						String	value )
 					{
-						use_units_rate_bits = COConfigurationManager.getBooleanParameter("config.style.useUnitsRateBits", false);
+						use_units_rate_bits = COConfigurationManager.getBooleanParameter("config.style.useUnitsRateBits");
 
 						setUnits();
 					}
 				});
 
-	    not_use_GB_TB = COConfigurationManager.getBooleanParameter("config.style.doNotUseGB", false);
+	    not_use_GB_TB = COConfigurationManager.getBooleanParameter("config.style.doNotUseGB");
 	    unitsStopAt = (not_use_GB_TB) ? UNIT_MB : UNIT_TB;
 	
 	    COConfigurationManager.addParameterListener( "config.style.doNotUseGB",
@@ -122,7 +123,7 @@ DisplayFormatters
 	          parameterChanged(
 	            String  value )
 	          {
-	            not_use_GB_TB = COConfigurationManager.getBooleanParameter("config.style.doNotUseGB", false);
+	            not_use_GB_TB = COConfigurationManager.getBooleanParameter("config.style.doNotUseGB");
 	            unitsStopAt = (not_use_GB_TB) ? UNIT_MB : UNIT_TB;
 	
 							setUnits();
@@ -150,8 +151,8 @@ DisplayFormatters
 					parameterChanged(
 						String	x )
 					{
-						separate_prot_data_stats = COConfigurationManager.getBooleanParameter("config.style.separateProtDataStats", false );
-						data_stats_only			 = COConfigurationManager.getBooleanParameter("config.style.dataStatsOnly", false);
+						separate_prot_data_stats = COConfigurationManager.getBooleanParameter("config.style.separateProtDataStats");
+						data_stats_only			 = COConfigurationManager.getBooleanParameter("config.style.dataStatsOnly");
 					}
 				});
 
@@ -166,47 +167,59 @@ DisplayFormatters
       // (1) http://physics.nist.gov/cuu/Units/binary.html
       // (2) http://www.isi.edu/isd/LOOM/documentation/unit-definitions.text
 
-    units = new String[unitsStopAt + 1];
-    units_rate = new String[unitsStopAt + 1];
+	units 		= new String[unitsStopAt + 1];
+	units_bits 	= new String[unitsStopAt + 1];
+    units_rate 	= new String[unitsStopAt + 1];
     
     if ( use_si_units ){
       // fall through intentional
       switch (unitsStopAt) {
         case UNIT_TB:
-          units[UNIT_TB] = getUnit("TiB");
-          units_rate[UNIT_TB] = (use_units_rate_bits) ? getUnit("Tibit")  : getUnit("TiB");
+         units[UNIT_TB] = getUnit("TiB");
+         units_bits[UNIT_TB] = getUnit("Tibit");
+         units_rate[UNIT_TB] = (use_units_rate_bits) ? getUnit("Tibit")  : getUnit("TiB");
         case UNIT_GB:
           units[UNIT_GB]= getUnit("GiB");
+          units_bits[UNIT_GB]= getUnit("Gibit");
           units_rate[UNIT_GB] = (use_units_rate_bits) ? getUnit("Gibit")  : getUnit("GiB");
         case UNIT_MB:
           units[UNIT_MB] = getUnit("MiB");
+          units_bits[UNIT_MB] = getUnit("Mibit");
           units_rate[UNIT_MB] = (use_units_rate_bits) ? getUnit("Mibit")  : getUnit("MiB");
         case UNIT_KB:
           // can be upper or lower case k
           units[UNIT_KB] = getUnit("KiB"); 
+          units_bits[UNIT_KB] = getUnit("Kibit"); 
+
           // can be upper or lower case k, upper more consistent
           units_rate[UNIT_KB] = (use_units_rate_bits) ? getUnit("Kibit")  : getUnit("KiB");
         case UNIT_B:
           units[UNIT_B] = getUnit("B");
+          units_bits[UNIT_B] = getUnit("bit");
           units_rate[UNIT_B] = (use_units_rate_bits)  ?   getUnit("bit")  :   getUnit("B");
       }
     }else{
       switch (unitsStopAt) {
         case UNIT_TB:
           units[UNIT_TB] = getUnit("TB");
+          units_bits[UNIT_TB] = getUnit("Tbit");
           units_rate[UNIT_TB] = (use_units_rate_bits) ? getUnit("Tbit")  : getUnit("TB");
         case UNIT_GB:
           units[UNIT_GB]= getUnit("GB");
+          units_bits[UNIT_GB]= getUnit("Gbit");
           units_rate[UNIT_GB] = (use_units_rate_bits) ? getUnit("Gbit")  : getUnit("GB");
         case UNIT_MB:
           units[UNIT_MB] = getUnit("MB");
+          units_bits[UNIT_MB] = getUnit("Mbit");
           units_rate[UNIT_MB] = (use_units_rate_bits) ? getUnit("Mbit")  : getUnit("MB");
         case UNIT_KB:
           // yes, the k should be lower case
           units[UNIT_KB] = getUnit("kB");
+          units_bits[UNIT_KB] = getUnit("kbit");
           units_rate[UNIT_KB] = (use_units_rate_bits) ? getUnit("kbit")  : getUnit("kB");
         case UNIT_B:
           units[UNIT_B] = getUnit("B");
+          units_bits[UNIT_B] = getUnit("bit");
           units_rate[UNIT_B] = (use_units_rate_bits)  ?  getUnit("bit")  :  getUnit("B");
       }
     }
@@ -356,11 +369,21 @@ DisplayFormatters
 		return( formatByteCountToKiBEtc( n, false, bTruncateZeros ));
 	}
 
-	protected static
+	public static
 	String formatByteCountToKiBEtc(
 		long	n,
 		boolean	rate,
 		boolean bTruncateZeros)
+	{
+		return formatByteCountToKiBEtc(n, rate, bTruncateZeros, -1);
+	}
+
+	public static
+	String formatByteCountToKiBEtc(
+		long	n,
+		boolean	rate,
+		boolean bTruncateZeros,
+		int precision)
 	{
 		double dbl = (rate && use_units_rate_bits) ? n * 8 : n;
 
@@ -371,6 +394,10 @@ DisplayFormatters
 		  dbl /= 1024L;
 		  unitIndex++;
 		}
+	  	
+	  if (precision < 0) {
+	  	precision = UNITS_PRECISION[unitIndex];
+	  }
 			 
 	  // round for rating, because when the user enters something like 7.3kbps
 		// they don't want it truncated and displayed as 7.2  
@@ -380,10 +407,16 @@ DisplayFormatters
 		// "I have a 1.0GB torrent and it says I've downloaded 1.0GB.. why isn't 
 		//  it complete? waaah"
 
-		return formatDecimal(dbl, UNITS_PRECISION[unitIndex], bTruncateZeros, rate)
+		return formatDecimal(dbl, precision, bTruncateZeros, rate)
 				+ (rate ? units_rate[unitIndex] : units[unitIndex]);
 	}
 
+	public static boolean
+	isDataProtSeparate()
+	{
+		return( separate_prot_data_stats );
+	}
+	
 	public static String
 	formatDataProtByteCountToKiBEtc(
 		long	data,
@@ -435,7 +468,8 @@ DisplayFormatters
 		return( formatByteCountToKiBEtc(n,true,TRUNCZEROS_NO));
 	}
 
-	public static String
+
+    public static String
 	formatByteCountToKiBEtcPerSec(
 		long		n,
 		boolean bTruncateZeros)
@@ -489,7 +523,60 @@ DisplayFormatters
 		return( formatByteCountToBase10KBEtc(n) + per_sec );
 	}
 
-   public static String 
+
+    /**
+     * Print the BITS/second in an international format.
+     * @param n -
+     * @return String in an internationalized format.
+     */
+    public static String
+    formatByteCountToBitsPerSec(
+            long n)
+    {
+        return formatBitCountToKiBEtcLocalImpl(n,true,true,-1,true);
+    }
+
+    /**
+     * NOTE: This method is a copy of formatByteCountToKiBEtc. Since the "use_units_rate_bits" member is
+     * static it cannot be used in a local context. Thus this method. More refactoring of this area
+     * should be done. Also need testing of the method to make sure units are accurate.
+     *
+     * Takes a long value that is bytes/bits download and converts it into internationalized units.
+     * @param n - value
+     * @param rate - true if in ? per second. Otherwise false.
+     * @param bTruncateZeros - true if truncating zeros.
+     * @param precision - negative value if same as units.
+     * @param useBits - true if using BITS, otherwise using BYTES.
+     * @return String - with units internationalized properly.
+     */
+    public static
+    String formatBitCountToKiBEtcLocalImpl(
+        long	n,
+        boolean	rate,
+        boolean bTruncateZeros,
+        int precision,
+        boolean useBits)
+    {
+        double dbl = (rate && useBits) ? n * 8 : n;
+
+        int unitIndex = UNIT_B;
+
+        while (dbl >= 1024 && unitIndex < unitsStopAt){
+
+          dbl /= 1024L;
+          unitIndex++;
+        }
+
+      if (precision < 0) {
+          precision = UNITS_PRECISION[unitIndex];
+      }
+
+        return formatDecimal(dbl, precision, bTruncateZeros, rate)
+                + units_bits[unitIndex] + (rate?per_sec:"");
+    }
+
+
+   public static String
    formatETA(long eta) 
    {
      if (eta == 0) return PeerManager_status_finished;
@@ -550,64 +637,81 @@ DisplayFormatters
 		String	tmp = "";
 
 		switch (state) {
-		  case DownloadManager.STATE_WAITING :
-			tmp = ManagerItem_waiting;
-			break;
-		  case DownloadManager.STATE_INITIALIZING :
-			  tmp = ManagerItem_initializing;
-			  break;
-		  case DownloadManager.STATE_INITIALIZED :
-			  tmp = ManagerItem_initializing;
-			  break;
-		  case DownloadManager.STATE_ALLOCATING :
-			tmp = ManagerItem_allocating;
-			break;
-		  case DownloadManager.STATE_CHECKING :
-			tmp = ManagerItem_checking + ": " + formatPercentFromThousands(manager.getStats().getCompleted());
-			break;
-		  case DownloadManager.STATE_FINISHING :
-		    tmp = ManagerItem_finishing;
-			 break;
-		  case DownloadManager.STATE_READY :
-			tmp = ManagerItem_ready;
-			break;
-		  case DownloadManager.STATE_DOWNLOADING :
-			tmp = ManagerItem_downloading;
-			break;
-		  case DownloadManager.STATE_SEEDING :
-         
-			  DiskManager diskManager = manager.getDiskManager();
-			  
-			  if ((diskManager != null) && diskManager.getCompleteRecheckStatus() != -1) {
-				  
-				  int	done = diskManager.getCompleteRecheckStatus();
-				  
-				  if ( done == -1 ){
-					  done = 1000;
-				  }
-				  
-				  tmp = ManagerItem_seeding + " + " + ManagerItem_checking + ": " + formatPercentFromThousands( done );
-				  
-			  }else if(manager.getPeerManager()!= null && manager.getPeerManager().isSuperSeedMode()){
-				  tmp = ManagerItem_superseeding;
-			  }else{
-				  tmp = ManagerItem_seeding;
-			  }
-			  break;
-		case DownloadManager.STATE_STOPPING :
-			tmp = ManagerItem_stopping;
-			break;
-		case DownloadManager.STATE_STOPPED :
-			tmp = manager.isPaused()?ManagerItem_paused:ManagerItem_stopped;
-			break;
-		  case DownloadManager.STATE_QUEUED :
-			tmp = ManagerItem_queued;
-			break;
-		  case DownloadManager.STATE_ERROR :
-			tmp = ManagerItem_error + ": " + manager.getErrorDetails();
-			break;
-			default :
-			tmp = String.valueOf(state);
+			case DownloadManager.STATE_QUEUED:
+				tmp = ManagerItem_queued;
+				break;
+
+			case DownloadManager.STATE_DOWNLOADING:
+				tmp = ManagerItem_downloading;
+				break;
+
+			case DownloadManager.STATE_SEEDING:
+
+				DiskManager diskManager = manager.getDiskManager();
+
+				if ((diskManager != null)
+						&& diskManager.getCompleteRecheckStatus() != -1) {
+
+					int done = diskManager.getCompleteRecheckStatus();
+
+					if (done == -1) {
+						done = 1000;
+					}
+
+					tmp = ManagerItem_seeding + " + " + ManagerItem_checking + ": "
+							+ formatPercentFromThousands(done);
+
+				} else if (manager.getPeerManager() != null
+						&& manager.getPeerManager().isSuperSeedMode()) {
+					tmp = ManagerItem_superseeding;
+				} else {
+					tmp = ManagerItem_seeding;
+				}
+				break;
+
+			case DownloadManager.STATE_STOPPED:
+				tmp = manager.isPaused() ? ManagerItem_paused : ManagerItem_stopped;
+				break;
+
+			case DownloadManager.STATE_ERROR:
+				tmp = ManagerItem_error + ": " + manager.getErrorDetails();
+				break;
+
+			case DownloadManager.STATE_WAITING:
+				tmp = ManagerItem_waiting;
+				break;
+
+			case DownloadManager.STATE_INITIALIZING:
+				tmp = ManagerItem_initializing;
+				break;
+
+			case DownloadManager.STATE_INITIALIZED:
+				tmp = ManagerItem_initializing;
+				break;
+
+			case DownloadManager.STATE_ALLOCATING:
+				tmp = ManagerItem_allocating;
+				break;
+
+			case DownloadManager.STATE_CHECKING:
+				tmp = ManagerItem_checking + ": "
+						+ formatPercentFromThousands(manager.getStats().getCompleted());
+				break;
+
+			case DownloadManager.STATE_FINISHING:
+				tmp = ManagerItem_finishing;
+				break;
+
+			case DownloadManager.STATE_READY:
+				tmp = ManagerItem_ready;
+				break;
+
+			case DownloadManager.STATE_STOPPING:
+				tmp = ManagerItem_stopping;
+				break;
+
+			default:
+				tmp = String.valueOf(state);
 		}
 
 		if (manager.isForceStart() &&
@@ -719,48 +823,52 @@ DisplayFormatters
   public static String formatIntToTwoDigits(int n) {
     return n < 10 ? "0".concat(String.valueOf(n)) : String.valueOf(n);
   }
-
-  public static String
-  formatDate(
-  	long		date )
-  {
-  	if ( date == 0 ){
-  		return( "" );
-  	}
-
-  	SimpleDateFormat temp = new SimpleDateFormat("dd-MMM-yyyy HH:mm:ss");
-
-  	return( temp.format(new Date(date)));
+  
+  private static String formatDate(long date, String format) {
+	  if (date == 0) {return "";}
+	  SimpleDateFormat temp = new SimpleDateFormat(format);
+	  return temp.format(new Date(date));
   }
 
-  public static String
-  formatDateShort(
-    long    date )
-  {
-    if ( date == 0 ){
-      return( "" );
+  public static String formatDate(long date) {
+  	return formatDate(date, "dd-MMM-yyyy HH:mm:ss");
+  }
+
+  public static String formatDateShort(long date) {
+	  return formatDate(date, "MMM dd, HH:mm");
     }
 
-    	// 24 hour clock, no point in including AM/PM
-
-    SimpleDateFormat temp = new SimpleDateFormat("MMM dd, HH:mm");
-
-    return( temp.format(new Date(date)));
+  public static String formatDateNum(long date) {
+	  return formatDate(date, "yyyy-MM-dd HH:mm:ss");
   }
   
-  public static String
-  formatDateNum(
-  	long		date )
-  {
-  	if ( date == 0 ){
-  		return( "" );
-  	}
-
-  	SimpleDateFormat temp = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
-
-  	return( temp.format(new Date(date)));
+  //
+  // These methods will be exposed in the plugin API.
+  //
+  
+  public static String formatCustomDateOnly(long date) {
+	  if (date == 0) {return "";}
+	  return formatDate(date, "dd-MMM-yyyy");
   }
 
+  public static String formatCustomTimeOnly(long date) {
+	  return formatCustomTimeOnly(date, true);
+  }
+	
+  public static String formatCustomTimeOnly(long date, boolean with_secs) {
+	  if (date == 0) {return "";}
+	  return formatDate(date, (with_secs) ? "HH:mm:ss" : "HH:mm");
+  }
+  
+  public static String formatCustomDateTime(long date) {
+	  if (date == 0) {return "";}
+	  return formatDate(date);
+  }
+
+  //
+  // End methods
+  //
+  
   public static String
   formatTime(
     long    time )
