@@ -23,13 +23,18 @@
 package org.gudy.azureus2.core3.util;
 
 import java.io.*;
+import java.lang.reflect.Method;
 import java.net.URI;
+import java.net.URL;
 import java.util.*;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
-import org.gudy.azureus2.core3.logging.*;
+import org.gudy.azureus2.core3.logging.LogAlert;
+import org.gudy.azureus2.core3.logging.LogEvent;
+import org.gudy.azureus2.core3.logging.LogIDs;
+import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.platform.PlatformManager;
 import org.gudy.azureus2.platform.PlatformManagerCapabilities;
 import org.gudy.azureus2.platform.PlatformManagerFactory;
@@ -51,6 +56,19 @@ public class FileUtil {
 	
   private static List		reserved_file_handles 	= new ArrayList();
   private static AEMonitor	class_mon				= new AEMonitor( "FileUtil:class" );
+  
+  private static Method reflectOnUsableSpace;
+  
+  static {
+
+	  try
+	  {
+		  reflectOnUsableSpace = File.class.getMethod("getUsableSpace", (Class[])null);
+	  } catch (NoSuchMethodException e)
+	  {
+		  reflectOnUsableSpace = null;
+	  }
+  }
 
   public static boolean isAncestorOf(File parent, File child) {
 	  parent = canonise(parent);
@@ -103,7 +121,7 @@ public class FileUtil {
   /**
    * Deletes the given dir and all files/dirs underneath
    */
-  public static void recursiveDelete(File f) {
+  public static boolean recursiveDelete(File f) {
     String defSaveDir = COConfigurationManager.getStringParameter("Default save path");
     String moveToDir = COConfigurationManager.getStringParameter("Completed Files Directory", "");
     
@@ -120,24 +138,35 @@ public class FileUtil {
 
       if (f.getCanonicalPath().equals(moveToDir)) {
         System.out.println("FileUtil::recursiveDelete:: not allowed to delete the MoveTo dir !");
-        return;
+        return( false );
       }
       if (f.getCanonicalPath().equals(defSaveDir)) {
         System.out.println("FileUtil::recursiveDelete:: not allowed to delete the default data dir !");
-        return;
+        return( false );
       }
       
       if (f.isDirectory()) {
         File[] files = f.listFiles();
         for (int i = 0; i < files.length; i++) {
-          recursiveDelete(files[i]);
+          if ( !recursiveDelete(files[i])){
+        	  
+        	  return( false );
+          }
+        }
+        if ( !f.delete()){
+        	
+        	return( false );
         }
-        f.delete();
       }
       else {
-        f.delete();
+        if ( !f.delete()){
+        	
+        	return( false );
+        }
       }
     } catch (Exception ignore) {/*ignore*/}
+    
+    return( true );
   }
   
   public static long
@@ -239,7 +268,8 @@ public class FileUtil {
           return;
         }
 
-        if (f.listFiles().length == 0) {
+        File[] files_inside = f.listFiles();
+        if (files_inside.length == 0) {
         	
           if ( !f.delete()){
           	
@@ -249,7 +279,7 @@ public class FileUtil {
           }
         }else{
         	if ( log_warnings ){
-        		Debug.out("Empty folder delete: "+f.listFiles().length+" file(s)/folder(s) still in " + f + ". Not removing.");
+        		Debug.out("Empty folder delete:  " + files_inside.length + " file(s)/folder(s) still in \"" + f + "\" - first listed item is \"" + files_inside[0].getName() + "\". Not removing.");
         	}
         }
       }
@@ -259,7 +289,8 @@ public class FileUtil {
   
   public static String
   convertOSSpecificChars(
-  	String	file_name_in )
+  	String		file_name_in,
+  	boolean		is_folder )
   {
   		// this rule originally from DiskManager
  
@@ -281,12 +312,19 @@ public class FileUtil {
 
 	 		// The definitive list of characters permitted for Windows is defined here:
 	 		// http://support.microsoft.com/kb/q120138/
-  			String not_allowed = "\\/:?*<>"; 
+  			String not_allowed = "\\/:?*<>|"; 
   		 	for (int i=0;i<chars.length;i++){
   		 		if (not_allowed.indexOf(chars[i]) != -1) {
   		  			chars[i] = '_';
   		  		}
   		  	}
+  		 	
+  		 	// windows doesn't like trailing dots and whitespaces in folders, replace them
+   		 	
+  		 	if ( is_folder ){
+  		 	
+  		 		for(int i = chars.length-1;i >= 0 && (chars[i] == '.' || chars[i] == ' ');chars[i] = '_',i--);
+  		 	}
   		}
   		
   			// '/' is valid in mac file names, replace with space
@@ -366,6 +404,17 @@ public class FileUtil {
   	String		file_name,
 	Map			data,
 	boolean		use_backup )
+  {
+	  writeResilientFile( parent_dir, file_name, data, use_backup, true );
+  }
+  
+  public static void
+  writeResilientFile(
+    File		parent_dir,
+  	String		file_name,
+	Map			data,
+	boolean		use_backup,
+	boolean		copy_to_backup )
   {	  
 	  if ( use_backup ){
 		  
@@ -373,7 +422,7 @@ public class FileUtil {
 		  
 		  if ( originator.exists()){
 			  
-			  backupFile( originator, true );
+			  backupFile( originator, copy_to_backup );
 		  }
 	  }
 	  
@@ -462,7 +511,7 @@ public class FileUtil {
  		if ( !use_backups ){
  			
  				// override if a backup file exists. This is needed to cope with backups
- 				// of the main config file itself as when boostrapping we can't get the
+ 				// of the main config file itself as when bootstrapping we can't get the
  				// "use backups" 
  			
  			if ( new File( parent_dir, file_name + ".bak").exists()){
@@ -549,7 +598,7 @@ public class FileUtil {
 	  			Map	res = null;
 	  			
 	  			try{
-	  				res = readResilientFile( parent_dir, file_name, 0, false );
+	  				res = readResilientFile( file_name, parent_dir, file_name, 0, false );
 	  			
 	  			}catch( Throwable e ){
 	  				
@@ -558,7 +607,7 @@ public class FileUtil {
 	  			
 	  			if ( res == null && attempt_recovery ){
 	  				
-	  				res = readResilientFile( parent_dir, file_name, 0, true );
+	  				res = readResilientFile( file_name, parent_dir, file_name, 0, true );
 	  				
 	  				if ( res != null ){
 	  					Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_WARNING,
@@ -587,6 +636,7 @@ public class FileUtil {
   	
 	private static Map
 	readResilientFile(
+		String		original_file_name,
 		File		parent_dir,
 		String		file_name,
 		int			fail_count,
@@ -613,7 +663,7 @@ public class FileUtil {
 	  					// we only alert the user if at least one file was found and failed
 	  					// otherwise it could be start of day when neither file exists yet
 	  					Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_ERROR,
-								"Load of '" + file_name + "' fails, no usable file or backup"));
+								"Load of '" + original_file_name + "' fails, no usable file or backup"));
 	  				}else{
 	  					// drop this log, it doesn't really help to inform about the failure to 
 	  					// find a .saving file
@@ -637,7 +687,7 @@ public class FileUtil {
 //								+ file_name + "' failed, " + "file not found or 0-sized."));
   			}
   			
-  			return( readResilientFile( parent_dir, file_name + ".saving", 0, recovery_mode ));
+  			return( readResilientFile( original_file_name, parent_dir, file_name + ".saving", 0, recovery_mode ));
   		}
 
   		BufferedInputStream bin = null;
@@ -660,8 +710,7 @@ public class FileUtil {
   					}
   	 				
   	 				if (Logger.isEnabled())
-							Logger.log(new LogEvent(LOGID, "Failed to open '" + file.toString()
-								+ "', retrying", e));
+							Logger.log(new LogEvent(LOGID, "Failed to open '" + file.toString()	+ "', retrying", e));
   					
   					Thread.sleep(500);
   				}
@@ -678,9 +727,9 @@ public class FileUtil {
 	    	
 	    	if ( using_backup && !recovery_mode ){
   		
-	    		Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_WARNING,
-						"Load of '" + file_name.substring(0, file_name.length() - 7)
-								+ "' had to revert to backup file")); 
+	    		Logger.log(
+	    				new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_WARNING,
+						"Load of '" + original_file_name + "' had to revert to backup file")); 
 	    	}
 	    	
 	    	return( res );
@@ -728,7 +777,7 @@ public class FileUtil {
 
 		    	if (Logger.isEnabled())
 						Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Read of '"
-								+ file_name + "' failed, decoding error. " + "Renaming to "
+								+ original_file_name + "' failed, decoding error. " + "Renaming to "
 								+ bad.getName()));
 	
 		    		// copy it so its left in place for possible recovery
@@ -740,13 +789,13 @@ public class FileUtil {
 		
 	    		if ( !recovery_mode ){
 	    			Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_ERROR,
-							"Load of '" + file_name + "' fails, no usable file or backup")); 
+							"Load of '" + original_file_name + "' fails, no usable file or backup")); 
 	    		}
 	    			
 	    		return( null );
 	    	}
 	    	
-	    	return( readResilientFile( parent_dir, file_name + ".saving", 1, recovery_mode ));
+	    	return( readResilientFile( original_file_name, parent_dir, file_name + ".saving", 1, recovery_mode ));
  			 
 	    }finally{
 	    	
@@ -918,22 +967,42 @@ public class FileUtil {
         }
       }
     
+    	/**
+    	 * copys the input stream to the file. always closes the input stream
+    	 * @param _source
+    	 * @param _dest
+    	 * @throws IOException
+    	 */
+    
     public static void 
     copyFile( 
-    	final InputStream _source, 
-    	final File _dest )
+    	final InputStream 	_source, 
+    	final File 			_dest )
     
     	throws IOException
     {
     	FileOutputStream	dest = null;
+    
+    	boolean	close_input = true;
     	
     	try{
     		dest = new FileOutputStream(_dest);
    
+    			// copyFile will close from now on, we don't need to
+    		
+    		close_input = false;
+    		
     		copyFile( _source, dest, true );
     		
     	}finally{
     		
+       		try{
+    			if(close_input){
+    				_source.close();
+    			}
+    		}catch( IOException e ){
+     		}
+    		
     		if ( dest != null ){
     			
     			dest.close();
@@ -945,7 +1014,9 @@ public class FileUtil {
     copyFile( 
       InputStream   is,
       OutputStream  os ) 
-    throws IOException {
+    	
+    	throws IOException 
+    {
       copyFile(is,os,true);
     }
     
@@ -1028,7 +1099,12 @@ public class FileUtil {
     		}
     	}else{
     		
-    		copyFile(  from_file_or_dir, new File( to_parent_dir, from_file_or_dir.getName()));
+    		File target = new File( to_parent_dir, from_file_or_dir.getName());
+    		
+    		if ( !copyFile(  from_file_or_dir, target )){
+    			
+    			throw( new IOException( "File copy from " + from_file_or_dir + " to " + target + " failed" ));
+    		}
     	}
     }
     
@@ -1060,6 +1136,38 @@ public class FileUtil {
       }
     }
 
+    public static File
+    getJarFileFromClass(
+    	Class		cla )
+    {
+    	try{
+	    	String str = cla.getName();
+	    	
+	    	str = str.replace( '.', '/' ) + ".class";
+	    	
+	        URL url = cla.getClassLoader().getResource( str );
+	        
+	        if ( url != null ){
+	        	
+	        	String	url_str = url.toExternalForm();
+	
+	        	if ( url_str.startsWith("jar:file:")){
+	
+	        		File jar_file = FileUtil.getJarFileFromURL(url_str);
+	        		
+	        		if ( jar_file.exists()){
+	        			
+	        			return( jar_file );
+	        		}
+	        	}
+	        }
+    	}catch( Throwable e ){
+    		
+    		Debug.printStackTrace(e);
+    	}
+
+        return( null );
+    }
     
     public static File
 	getJarFileFromURL(
@@ -1437,7 +1545,24 @@ public class FileUtil {
 		
 		return fName.substring(fileDotIndex);
 	}
+
+	public static String
+	readFileAsString(
+		File	file,
+		int		size_limit,
+		String charset)
 	
+		throws IOException
+	{
+		FileInputStream fis = new FileInputStream(file);
+		try {
+			return readInputStreamAsString(fis, size_limit, charset);
+		} finally {
+
+			fis.close();
+		}
+	}
+
 	public static String
 	readFileAsString(
 		File	file,
@@ -1459,6 +1584,17 @@ public class FileUtil {
 		InputStream is,
 		int		size_limit )
 	
+		throws IOException
+	{
+		return readInputStreamAsString(is, size_limit, "ISO-8859-1");
+	}
+
+	public static String
+	readInputStreamAsString(
+		InputStream is,
+		int		size_limit,
+		String charSet)
+	
 		throws IOException
 	{
 		StringBuffer result = new StringBuffer(1024);
@@ -1474,7 +1610,7 @@ public class FileUtil {
 				break;
 			}
 
-			result.append(new String(buffer, 0, len, "ISO-8859-1"));
+			result.append(new String(buffer, 0, len, charSet));
 
 			if (size_limit >= 0 && result.length() > size_limit) {
 
@@ -1531,4 +1667,20 @@ public class FileUtil {
 			fis.close();
 		}
 	}
+	
+	public final static boolean getUsableSpaceSupported()
+	{
+		return reflectOnUsableSpace != null;
+	}
+	
+	public final static long getUsableSpace(File f)
+	{
+		try
+		{
+			return ((Long)reflectOnUsableSpace.invoke(f, null)).longValue();
+		} catch (Exception e)
+		{
+			return -1;
+		}		
+	}
 }
