@@ -31,7 +31,7 @@ import java.util.*;
 
 public class Timer
 	extends 	AERunnable
-	implements	SystemTime.consumer
+	implements	SystemTime.ChangeListener
 {
 	private static boolean DEBUG_TIMERS = true;
 	private static ArrayList timers = null;
@@ -119,6 +119,12 @@ public class Timer
 		thread_pool.setWarnWhenFull();
 	}
 	
+	public void
+	setLogCPU()
+	{
+		thread_pool.setLogCPU();
+	}
+	
 	public void
 	runSupport()
 	{
@@ -198,7 +204,8 @@ public class Timer
 	}
 	
 	public void
-	consume(
+	clockChanged(
+		long	current_time,
 		long	offset )
 	{
 		// System.out.println( "Timer '" + thread_pool.getName() +"': clock change by " + offset );
@@ -209,7 +216,7 @@ public class Timer
 			
 			synchronized( this ){
 				
-					// as we're adjusting all events by the same amount the ordering remains valid
+				boolean resort = false;
 				
 				Iterator	it = events.iterator();
 				
@@ -217,12 +224,57 @@ public class Timer
 					
 					TimerEvent	event = (TimerEvent)it.next();
 					
-					long	old_when = event.getWhen();
-					long	new_when = old_when + offset;
+						// absolute events don't have their timings fiddled with
 					
-					// System.out.println( "    adjusted: " + old_when + " -> " + new_when );
+					if ( event.isAbsolute()){
+						
+							// event ordering may change
+						
+						resort = true;
+						
+					}else{
+						
+						long	old_when = event.getWhen();
+						long	new_when = old_when + offset;
+						
+						TimerEventPerformer performer = event.getPerformer();
+						
+							// sanity check for periodic events
+						
+						if ( performer instanceof TimerEventPeriodic ){
+							
+							TimerEventPeriodic	periodic_event = (TimerEventPeriodic)performer;
+							
+							long	freq = periodic_event.getFrequency();
+													
+							if ( new_when > current_time + freq + 5000 ){
+								
+								long	adjusted_when = current_time + freq;
+								
+								Debug.outNoStack( periodic_event.getName() + ": clock change sanity check. Reduced schedule time from " + new_when + " to " +  adjusted_when );
+									
+								new_when = adjusted_when;
+							}
+						}
+						
+						// don't wrap around by accident
+						
+						if ( old_when > 0 && new_when < 0 && offset > 0 ){
+
+							// Debug.out( "Ignoring wrap around for " + event.getName());
+							
+						}else{
+							
+							// System.out.println( "    adjusted: " + old_when + " -> " + new_when );
+						
+							event.setWhen( new_when );
+						}
+					}
+				}
+				
+				if ( resort ){
 					
-					event.setWhen( new_when );
+					events = new TreeSet( events );
 				}
 
 				notify();
@@ -248,10 +300,19 @@ public class Timer
 
 				long old_when = event.getWhen();
 				long new_when = old_when + offset;
+				
+					// don't wrap around by accident
+				
+				if ( old_when > 0 && new_when < 0 && offset > 0 ){
 
-				// System.out.println( "    adjusted: " + old_when + " -> " + new_when );
+					// Debug.out( "Ignoring wrap around for " + event.getName());
+					
+				}else{
+					
+					// System.out.println( "    adjusted: " + old_when + " -> " + new_when );
 
-				event.setWhen(new_when);
+					event.setWhen( new_when );
+				}
 			}
 
 			notify();
@@ -275,6 +336,16 @@ public class Timer
 		return( addEvent( name, SystemTime.getCurrentTime(), when, performer ));
 	}
 	
+	public synchronized TimerEvent
+	addEvent(
+		String				name,
+		long				when,
+		boolean				absolute,
+		TimerEventPerformer	performer )
+	{
+		return( addEvent( name, SystemTime.getCurrentTime(), when, absolute, performer ));
+	}
+	
 	public synchronized TimerEvent
 	addEvent(
 		long				creation_time,
@@ -284,6 +355,16 @@ public class Timer
 		return( addEvent( null, creation_time, when, performer ));
 	}
 	
+	public synchronized TimerEvent
+	addEvent(
+		long				creation_time,
+		long				when,
+		boolean				absolute,
+		TimerEventPerformer	performer )
+	{
+		return( addEvent( null, creation_time, when, absolute, performer ));
+	}
+	
 	public synchronized TimerEvent
 	addEvent(
 		String				name,
@@ -291,7 +372,18 @@ public class Timer
 		long				when,
 		TimerEventPerformer	performer )
 	{
-		TimerEvent	event = new TimerEvent( this, unique_id_next++, creation_time, when, performer );
+		return( addEvent( name, creation_time, when, false, performer ));
+	}
+	
+	public synchronized TimerEvent
+	addEvent(
+		String				name,
+		long				creation_time,
+		long				when,
+		boolean				absolute,
+		TimerEventPerformer	performer )
+	{
+		TimerEvent	event = new TimerEvent( this, unique_id_next++, creation_time, when, absolute, performer );
 		
 		if ( name != null ){
 			
@@ -302,11 +394,6 @@ public class Timer
 		
 		if ( log ){
 			
-			if ( !(performer instanceof TimerEventPerformer )){
-				
-				System.out.println( "Timer '" + thread_pool.getName() + "' - added " + event.getString());
-			}
-					
 			if ( events.size() > max_events_logged ){
 		
 				max_events_logged = events.size();
@@ -336,7 +423,17 @@ public class Timer
 		long				frequency,
 		TimerEventPerformer	performer )
 	{
-		TimerEventPeriodic periodic_performer = new TimerEventPeriodic( this, frequency, performer );
+		return( addPeriodicEvent( name, frequency, false, performer ));
+	}
+	
+	public synchronized TimerEventPeriodic
+	addPeriodicEvent(
+		String				name,
+		long				frequency,
+		boolean				absolute,
+		TimerEventPerformer	performer )
+	{
+		TimerEventPeriodic periodic_performer = new TimerEventPeriodic( this, frequency, absolute, performer );
 		
 		if ( name != null ){
 			
