@@ -46,8 +46,15 @@ import org.gudy.azureus2.plugins.PluginInterface;
 public class 
 TorrentUtils 
 {
+	public static final int TORRENT_FLAG_LOW_NOISE	= 0x00000001;
+	
 	private static final String		TORRENT_AZ_PROP_DHT_BACKUP_ENABLE		= "dht_backup_enable";
 	private static final String		TORRENT_AZ_PROP_DHT_BACKUP_REQUESTED	= "dht_backup_requested";
+	private static final String		TORRENT_AZ_PROP_TORRENT_FLAGS			= "torrent_flags";
+	private static final String		TORRENT_AZ_PROP_PLUGINS					= "plugins";
+	
+	private static final String		MEM_ONLY_TORRENT_PATH		= "?/\\!:mem_only:!\\/?";
+	
 	
 	private static final List	created_torrents;
 	private static final Set	created_torrents_set;
@@ -70,8 +77,7 @@ TorrentUtils
 		COConfigurationManager.addAndFireParameterListener("Save Torrent Backup",
 				new ParameterListener() {
 					public void parameterChanged(String parameterName) {
-						bSaveTorrentBackup = COConfigurationManager.getBooleanParameter(
-								parameterName, false);
+						bSaveTorrentBackup = COConfigurationManager.getBooleanParameter(parameterName);
 					}
 				});
 		
@@ -109,6 +115,16 @@ TorrentUtils
 		 * @throws TOTorrentException
 		 */
 	
+	public static ExtendedTorrent
+	readDelegateFromFile(
+		File		file,
+		boolean		force_initial_discard )
+		
+		throws TOTorrentException
+	{
+		return((ExtendedTorrent)readFromFile( file, true, force_initial_discard ));
+	}
+	
 	public static TOTorrent
 	readFromFile(
 		File		file,
@@ -194,6 +210,26 @@ TorrentUtils
 		return( torrent );
 	}
 	
+	public static void
+	setMemoryOnly(
+		TOTorrent			torrent,
+		boolean				mem_only )
+	{
+		if ( mem_only ){
+			
+			torrent.setAdditionalStringProperty("torrent filename", MEM_ONLY_TORRENT_PATH );
+			
+		}else{
+			
+			String s = torrent.getAdditionalStringProperty("torrent filename");
+			
+			if ( s != null && s.equals( MEM_ONLY_TORRENT_PATH )){
+				
+				torrent.removeAdditionalProperty( "torrent filename" );
+			}
+		}
+	}
+	
 	public static void
 	writeToFile(
 		final TOTorrent		torrent )
@@ -212,30 +248,41 @@ TorrentUtils
 	{
 	   try{
 	   		torrent.getMonitor().enter();
-	    	
-	   			// we've got to re-obtain the pieces here in case they've been thrown
-	   			// away to save memory *before* we rename the torrent file!
-	   		
-	   		torrent.getPieces();
-	   		
+	    		   		
 	    	String str = torrent.getAdditionalStringProperty("torrent filename");
 	    	
-	    	if (str == null){
+	    	if ( str == null ){
 	    		
 	    		throw (new TOTorrentException("TorrentUtils::writeToFile: no 'torrent filename' attribute defined", TOTorrentException.RT_FILE_NOT_FOUND));
 	    	}
-	    		  	    	
+	    	
+	    	if ( str.equals( MEM_ONLY_TORRENT_PATH )){
+	    		
+	    		return;
+	    	}
+	    	
+	    		// save first to temporary file as serialisation may require state to be re-read from
+	    		// the existing file first and if we rename to .bak first then this aint good
+	    		    	
+    		File torrent_file_tmp = new File(str + "._az");
+
+	    	torrent.serialiseToBEncodedFile( torrent_file_tmp );
+
+	    		// now backup if required
+	    	
 	    	File torrent_file = new File(str);
 	    	
-	    	if ( 	( force_backup ||COConfigurationManager.getBooleanParameter("Save Torrent Backup", false)) &&
+	    	if ( 	( force_backup ||COConfigurationManager.getBooleanParameter("Save Torrent Backup")) &&
 	    			torrent_file.exists()) {
 	    		
 	    		File torrent_file_bak = new File(str + ".bak");
 	    		
 	    		try{
 	    			
-	    			// Will return false if it cannot be deleted (including if the file doesn't exist). 
+	    				// Will return false if it cannot be deleted (including if the file doesn't exist).
+	    			
 	    			torrent_file_bak.delete();
+	    			
 	    			torrent_file.renameTo(torrent_file_bak);
 	    			
 	    		}catch( SecurityException e){
@@ -244,7 +291,14 @@ TorrentUtils
 	    		}
 	    	}
 	      
-	    	torrent.serialiseToBEncodedFile(torrent_file);
+	    		// now rename the temp file to required one
+	    	
+	    	if ( torrent_file.exists()){
+	    		
+	    		torrent_file.delete();
+	    	}
+	    	
+	    	torrent_file_tmp.renameTo( torrent_file );
 			
 	   	}finally{
 	   		
@@ -288,6 +342,11 @@ TorrentUtils
     		throw( new TOTorrentException("TorrentUtils::getTorrentFileName: no 'torrent filename' attribute defined", TOTorrentException.RT_FILE_NOT_FOUND));
     	}
 
+    	if ( str.equals( MEM_ONLY_TORRENT_PATH )){
+    		
+    		return( null );
+    	}
+    	
 		return( str );
 	}
 	
@@ -298,20 +357,7 @@ TorrentUtils
 
 		throws TOTorrentException 
 	{
-		try{
-	   		torrent.getMonitor().enter();
-	    	
-	   			// we've got to re-obtain the pieces here in case they've been thrown
-	   			// away to save memory *before* we rename the torrent file!
-	   		
-	   		torrent.getPieces();
-	   			      
-	    	torrent.serialiseToBEncodedFile(file);
-			
-	   	}finally{
-	   		
-	   		torrent.getMonitor().exit();
-	   	}	
+	   	torrent.serialiseToBEncodedFile(file);
 	}
 	
 	public static void
@@ -330,6 +376,11 @@ TorrentUtils
 	    		throw( new TOTorrentException("TorrentUtils::delete: no 'torrent filename' attribute defined", TOTorrentException.RT_FILE_NOT_FOUND));
 	    	}
 	    	
+	    	if ( str.equals( MEM_ONLY_TORRENT_PATH )){
+	    		
+	    		return;
+	    	}
+	    	
 	    	if ( !new File(str).delete()){
 	    		
 	    		throw( new TOTorrentException("TorrentUtils::delete: failed to delete '" + str + "'", TOTorrentException.RT_WRITE_FAILS));
@@ -925,11 +976,9 @@ TorrentUtils
 		return( url.getProtocol().equalsIgnoreCase( "dht" ));
 	}
 	
-	public static void
-	setPluginStringProperty(
-		TOTorrent		torrent,
-		String			name,
-		String			value )
+	private static Map
+	getAzureusProperties(
+		TOTorrent	torrent )
 	{
 		Map	m = torrent.getAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES );
 		
@@ -940,7 +989,53 @@ TorrentUtils
 			torrent.setAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES, m );
 		}
 		
-		Object obj = m.get( "plugins" );
+		return( m );
+	}
+	
+	public static void
+	setFlag(
+		TOTorrent		torrent,
+		int				flag,
+		boolean			value )
+	{
+		Map	m = getAzureusProperties( torrent );
+		
+		Long	flags = (Long)m.get( TORRENT_AZ_PROP_TORRENT_FLAGS );
+		
+		if ( flags == null ){
+			
+			flags = new Long(0);
+		}		
+		
+		m.put( TORRENT_AZ_PROP_TORRENT_FLAGS, new Long(flags.intValue() | flag ));
+	}
+		
+	public static boolean
+	getFlag(
+		TOTorrent		torrent,
+		int				flag )
+	{
+		Map	m = getAzureusProperties( torrent );
+		
+		Long	flags = (Long)m.get( TORRENT_AZ_PROP_TORRENT_FLAGS );
+		
+		if ( flags == null ){
+			
+			return( false );
+		}
+
+		return(( flags.intValue() & flag ) != 0 );
+	}
+	
+	public static void
+	setPluginStringProperty(
+		TOTorrent		torrent,
+		String			name,
+		String			value )
+	{
+		Map	m = getAzureusProperties( torrent );
+		
+		Object obj = m.get( TORRENT_AZ_PROP_PLUGINS );
 		
 		Map	p;
 		
@@ -952,7 +1047,7 @@ TorrentUtils
 			
 			p = new HashMap();
 			
-			m.put( "plugins", p );
+			m.put( TORRENT_AZ_PROP_PLUGINS, p );
 		}
 		
 		if ( value == null ){
@@ -970,14 +1065,9 @@ TorrentUtils
 		TOTorrent		torrent,
 		String			name )
 	{
-		Map	m = torrent.getAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES );
+		Map	m = getAzureusProperties( torrent );
 		
-		if ( m == null ){
-			
-			return( null );
-		}
-		
-		Object	obj = m.get( "plugins" );
+		Object	obj = m.get( TORRENT_AZ_PROP_PLUGINS );
 		
 		if ( obj instanceof Map ){
 		
@@ -1000,16 +1090,9 @@ TorrentUtils
 		String			name,
 		Map				value )
 	{
-		Map	m = torrent.getAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES );
+		Map	m = getAzureusProperties( torrent );
 		
-		if ( m == null ){
-			
-			m = new HashMap();
-			
-			torrent.setAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES, m );
-		}
-		
-		Object obj = m.get( "plugins" );
+		Object obj = m.get( TORRENT_AZ_PROP_PLUGINS );
 		
 		Map	p;
 		
@@ -1021,7 +1104,7 @@ TorrentUtils
 			
 			p = new HashMap();
 			
-			m.put( "plugins", p );
+			m.put( TORRENT_AZ_PROP_PLUGINS, p );
 		}
 		
 		if ( value == null ){
@@ -1039,14 +1122,9 @@ TorrentUtils
 		TOTorrent		torrent,
 		String			name )
 	{
-		Map	m = torrent.getAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES );
+		Map	m = getAzureusProperties( torrent );
 		
-		if ( m == null ){
-			
-			return( null );
-		}
-		
-		Object	obj = m.get( "plugins" );
+		Object	obj = m.get( TORRENT_AZ_PROP_PLUGINS );
 		
 		if ( obj instanceof Map ){
 		
@@ -1068,14 +1146,7 @@ TorrentUtils
 		TOTorrent		torrent,
 		boolean			enabled )
 	{
-		Map	m = torrent.getAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES );
-		
-		if ( m == null ){
-			
-			m = new HashMap();
-			
-			torrent.setAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES, m );
-		}
+		Map	m = getAzureusProperties( torrent );
 		
 		m.put( TORRENT_AZ_PROP_DHT_BACKUP_ENABLE, new Long(enabled?1:0));
 	}
@@ -1086,12 +1157,7 @@ TorrentUtils
 	{
 			// missing -> true
 		
-		Map	m = torrent.getAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES );
-		
-		if ( m == null ){
-			
-			return( true );
-		}
+		Map	m = getAzureusProperties( torrent );
 		
 		Object	obj = m.get( TORRENT_AZ_PROP_DHT_BACKUP_ENABLE );
 		
@@ -1109,12 +1175,7 @@ TorrentUtils
 	{
 			// missing -> false
 		
-		Map	m = torrent.getAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES );
-		
-		if ( m == null ){
-			
-			return( false );
-		}
+		Map	m = getAzureusProperties( torrent );
 		
 		Object obj = m.get( TORRENT_AZ_PROP_DHT_BACKUP_REQUESTED );
 		
@@ -1131,35 +1192,24 @@ TorrentUtils
 		TOTorrent		torrent,
 		boolean			requested )
 	{
-		Map	m = torrent.getAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES );
-		
-		if ( m == null ){
-			
-			m = new HashMap();
-			
-			torrent.setAdditionalMapProperty( TOTorrent.AZUREUS_PROPERTIES, m );
-		}
+		Map	m = getAzureusProperties( torrent );
 		
 		m.put( TORRENT_AZ_PROP_DHT_BACKUP_REQUESTED, new Long(requested?1:0));
 	}
+		
 	
-	public static boolean
-	getDHTTrackerEnabled()
-	{
-		PluginInterface dht_pi = 
-			AzureusCoreFactory.getSingleton().getPluginManager().getPluginInterfaceByClass(
-						DHTPlugin.class );
-				
-		if ( dht_pi == null ){
+	public static boolean isReallyPrivate(TOTorrent torrent) {
+		if ( torrent == null ){
 			
 			return( false );
-			
-		}else{
-			
-			DHTPlugin dht = (DHTPlugin)dht_pi.getPlugin();		
-			
-			return( dht.peekEnabled());
+		}	
+		
+		String announceURL = torrent.getAnnounceURL().toExternalForm();
+		if(announceURL.matches(".*[0-9a-z]{20,40}.*")) {
+			return torrent.getPrivate();
 		}
+		
+		return false;
 	}
 	
 	public static boolean
@@ -1293,16 +1343,49 @@ TorrentUtils
 			});
 	}
 	
-	public static class
+	private static HashSet	torrentFluffKeyset = new HashSet(2);
+	private static Map		fluffThombstone = new HashMap(1);
+	
+	/**
+	 * Register keys that are used for heavyweight maps that should be discarded when the torrent is not in use
+	 * Make sure these keys are only ever used for Map objects!
+	 */
+	public static void
+	registerMapFluff(
+		String[]		fluff )
+	{
+		synchronized (TorrentUtils.class)
+		{
+			for (int i = 0; i < fluff.length; i++)
+				torrentFluffKeyset.add(fluff[i]);
+		}
+	}
+	
+	public interface
+	ExtendedTorrent
+		extends TOTorrent
+	{
+		public byte[][]
+		peekPieces()
+		    		
+			throws TOTorrentException;
+		
+		public void
+		setDiscardFluff(
+			boolean	discard );
+	}
+	
+	private static class
 	torrentDelegate
 		extends LogRelation
-		implements TOTorrent
+		implements ExtendedTorrent
 	{
 		private TOTorrent		delegate;
 		private File			file;
 		
+		private boolean			fluff_dirty;
+		
 		private long			last_pieces_read_time	= SystemTime.getCurrentTime();
-		private byte[][]		pieces;
 		
 		protected
 		torrentDelegate(
@@ -1318,6 +1401,49 @@ TorrentUtils
 			}
 		}
 		
+		public void
+		setDiscardFluff(
+			boolean	discard )
+		{
+			if ( discard && !torrentFluffKeyset.isEmpty() ){
+				
+				//System.out.println( "Discarded fluff for " + new String(getName()));
+				
+				try{
+			   		getMonitor().enter();
+					
+			   		try{
+				   			// if file is out of sync with fluff then force a write
+				   		
+				   		if ( fluff_dirty ){
+				   			
+					   		boolean[]	restored = restoreState( true, true );
+					   		
+					   		delegate.serialiseToBEncodedFile( file );
+					   		
+					   		fluff_dirty = false;
+					   		
+					   		if ( restored[0] ){
+					   			
+					   			discardPieces( SystemTime.getCurrentTime(), true );
+					   		}
+				   		}
+				   		
+				   		for(Iterator it = torrentFluffKeyset.iterator();it.hasNext();){
+				   			
+				   			delegate.setAdditionalMapProperty( (String)it.next(), fluffThombstone );
+				   		}
+			   		}catch( Throwable e ){
+			   		
+			   			Debug.printStackTrace( e );
+			   		}
+				}finally{
+					
+					getMonitor().exit();
+				}
+			}
+		}
+		
 		public byte[]
 		getName()
 		{
@@ -1439,11 +1565,9 @@ TorrentUtils
 				try{
 			   		getMonitor().enter();
 
-			   		TOTorrent	temp = readFromFile( file, false );
-					
-			   		res	= temp.getPieces();
-					
-			   		delegate.setPieces( res );
+			   		restoreState( true, false );
+			   		
+			   		res = delegate.getPieces();
 			   		
 				}finally{
 					
@@ -1454,6 +1578,70 @@ TorrentUtils
 			return( res );
 		}
 
+		/**
+		 * monitor must be held before calling me
+		 * @param do_pieces
+		 * @param do_fluff
+		 * @throws TOTorrentException
+		 */
+		
+		protected boolean[]
+		restoreState(
+			boolean		do_pieces,
+			boolean		do_fluff )
+		
+			throws TOTorrentException
+		{
+	   		boolean	had_pieces = delegate.getPieces() != null;
+	   		
+	   		boolean	had_fluff = true; 
+	   		
+	   		for(Iterator it = torrentFluffKeyset.iterator();it.hasNext();){
+	   			
+	   			had_fluff &= delegate.getAdditionalMapProperty( (String)it.next() ) != fluffThombstone;
+	   		}
+
+	   		if ( had_pieces ){
+	   			
+	   			do_pieces = false;
+	   		}
+	   		
+	   		if ( had_fluff ){
+	   			
+	   			do_fluff = false;
+	   		}
+	   		
+	   		if ( do_pieces || do_fluff ){
+	   		
+		   		TOTorrent	temp = readFromFile( file, false );
+				
+		   		if ( do_pieces ){
+		   		
+		   			byte[][] res	= temp.getPieces();
+				
+		   			delegate.setPieces( res );
+		   		}
+		   		
+		   		if ( do_fluff ){
+		   			
+		   			for (Iterator it = torrentFluffKeyset.iterator(); it.hasNext();){
+					
+						String fluffKey = (String) it.next();
+						
+							// only update the discarded entries as non-discarded may be out of sync
+							// with the file contents
+						
+						if ( delegate.getAdditionalMapProperty( fluffKey ) == fluffThombstone ){
+							
+							delegate.setAdditionalMapProperty(fluffKey, temp.getAdditionalMapProperty(fluffKey));
+						}
+					}
+		   		}
+	   		}
+	   		
+	   		return( new boolean[]{ do_pieces, do_fluff });
+		}
+		
 			/**
 			 * peeks the pieces, will return null if they are discarded
 			 * @return
@@ -1606,21 +1794,93 @@ TorrentUtils
 			String		name,
 			Map			value )
 		{
-			delegate.setAdditionalMapProperty( name, value );
+			if ( torrentFluffKeyset.contains(name)){
+
+				//System.out.println( "Set fluff for " + new String(getName()) + " to " + value );
+
+				try{
+					getMonitor().enter();
+
+					delegate.setAdditionalMapProperty( name, value );
+					
+					fluff_dirty = true;
+					
+				}finally{
+					
+					getMonitor().exit();
+				}
+			}else{
+				
+				delegate.setAdditionalMapProperty( name, value );	
+			}
 		}
 			
 		public Map
 		getAdditionalMapProperty(
 			String		name )
 		{
+			if (torrentFluffKeyset.contains(name)){
+				
+				try{
+					getMonitor().enter();
+
+					Map	result = delegate.getAdditionalMapProperty( name );
+					
+					if ( result == fluffThombstone ){
+					
+						try{
+							restoreState( false, true );
+							
+							Map res = delegate.getAdditionalMapProperty( name );
+
+							//System.out.println( "Restored fluff for " + new String(getName()) + " to " + res );
+
+							return( res );
+							
+						}catch( Throwable e ){
+							
+							Debug.out( "Property '" + name + " lost due to torrent read error", e );
+						}
+					}
+				}finally{
+					
+					getMonitor().exit();
+				}
+			}
+			
 			return( delegate.getAdditionalMapProperty( name ));
 		}
 		
-		public Object
-		getAdditionalProperty(
-			String		name )
-		{
-			return( delegate.getAdditionalProperty( name ));
+		public Object getAdditionalProperty(String name) {
+			if (torrentFluffKeyset.contains(name))
+			{
+				try
+				{
+					getMonitor().enter();
+					
+					Object result = delegate.getAdditionalProperty(name);
+					if (result == fluffThombstone)
+					{
+						try
+						{
+							restoreState(false, true);
+							Object res = delegate.getAdditionalProperty(name);
+							//System.out.println( "Restored fluff for " + new String(getName()) + " to " + res );
+							
+							return (res);
+							
+						} catch (Throwable e)
+						{
+							Debug.out("Property '" + name + " lost due to torrent read error", e);
+						}
+					}
+				} finally
+				{
+					getMonitor().exit();
+				}
+			}
+			
+			return delegate.getAdditionalProperty(name);
 		}
 
 		public void
@@ -1628,23 +1888,70 @@ TorrentUtils
 			String		name,
 			Object		value )
 		{
-			delegate.setAdditionalProperty( name, value );
+			if ( torrentFluffKeyset.contains(name)){
+
+				//System.out.println( "Set fluff for " + new String(getName()) + " to " + value );
+
+				try{
+					getMonitor().enter();
+
+					delegate.setAdditionalProperty( name, value );
+					
+					fluff_dirty = true;
+					
+				}finally{
+					
+					getMonitor().exit();
+				}
+			}else{
+			
+				delegate.setAdditionalProperty( name, value );
+			}
 		}
 		
 		public void
 		removeAdditionalProperty(
 			String name )
 		{
-			delegate.removeAdditionalProperty( name );
-		}
-		
+			if(delegate.getAdditionalProperty(name) == null)
+				return;
+			
+			if ( torrentFluffKeyset.contains(name)){
+
+				//System.out.println( "Set fluff for " + new String(getName()) + " to " + value );
+
+				try{
+					getMonitor().enter();
+					
+					delegate.removeAdditionalProperty( name );
+					
+					fluff_dirty = true;
+					
+				}finally{
+					
+					getMonitor().exit();
+				}
+			}else{
+				
+				delegate.removeAdditionalProperty( name );
+			}
+		}	
 		
 		public void
 		removeAdditionalProperties()
 		{
-			delegate.removeAdditionalProperties();
-		}
-		
+			try{
+				getMonitor().enter();
+
+				delegate.removeAdditionalProperties();
+				
+				fluff_dirty = true;
+				
+			}finally{
+				
+				getMonitor().exit();
+			}
+		}		
 
 		public void
 		serialiseToBEncodedFile(
@@ -1656,17 +1963,28 @@ TorrentUtils
 			
 			try{
 		   		getMonitor().enter();
+		   				   		
+		   		boolean[]	restored = restoreState( true, true );
 		   		
-		   		boolean	had_pieces = delegate.getPieces() != null;
-		   		
-		   		getPieces();
-			
 		   		delegate.serialiseToBEncodedFile( target_file );
 		   		
-		   		if ( !had_pieces ){
+		   		if ( target_file.equals( file )){
+		   			
+		   			fluff_dirty = false;
+		   		}
+		   		
+		   		if ( restored[0] ){
 		   			
 		   			discardPieces( SystemTime.getCurrentTime(), true );
 		   		}
+		   		
+		   		if ( restored[1] ){
+		   			
+		   			for (Iterator it = torrentFluffKeyset.iterator(); it.hasNext();){
+		   				
+		   				delegate.setAdditionalMapProperty( (String)it.next(), fluffThombstone );
+		   			}
+		   		}
 			}finally{
 				
 				getMonitor().exit();
@@ -1684,17 +2002,23 @@ TorrentUtils
 			try{
 		   		getMonitor().enter();
 		   		
-		   		boolean	had_pieces = delegate.getPieces() != null;
-
-		   		getPieces();
+		   		boolean[]	restored = restoreState( true, true );
 			
 		   		Map	result = delegate.serialiseToMap();
 		   		
-		   		if ( !had_pieces ){
+		   		if ( restored[0] ){
 		   			
 		   			discardPieces( SystemTime.getCurrentTime(), true );
 		   		}
 		   		
+		   		if ( restored[1]){
+				
+					for (Iterator it = torrentFluffKeyset.iterator(); it.hasNext();){
+					
+						delegate.setAdditionalMapProperty((String) it.next(), fluffThombstone);
+					}
+				}
+		   		
 		   		return( result );
 		   		
 			}finally{
@@ -1706,7 +2030,7 @@ TorrentUtils
 
 		public void
 		serialiseToXMLFile(
-				File		target_file )
+			File		target_file )
 			  
 		   throws TOTorrentException
 		{
@@ -1715,16 +2039,22 @@ TorrentUtils
 			try{
 		   		getMonitor().enter();
 		   		
-		   		boolean	had_pieces = delegate.getPieces() != null;
-
-		   		getPieces();
+		   		boolean[]	restored = restoreState( true, true );
 			
 		   		delegate.serialiseToXMLFile( target_file );
 			
-		   		if ( !had_pieces ){
+		   		if ( restored[0] ){
 		   			
 		   			discardPieces( SystemTime.getCurrentTime(), true );
 		   		}
+		   		
+		   		if ( restored[1]){
+				
+					for (Iterator it = torrentFluffKeyset.iterator(); it.hasNext();){
+					
+						delegate.setAdditionalMapProperty((String) it.next(), fluffThombstone);
+					}
+				}
 			}finally{
 				
 				getMonitor().exit();
@@ -1772,8 +2102,7 @@ TorrentUtils
 			throws IOException {
 		File torrentDir;
 		boolean saveTorrents = persistent
-				&& COConfigurationManager.getBooleanParameter("Save Torrent Files",
-						true);
+				&& COConfigurationManager.getBooleanParameter("Save Torrent Files");
 		if (saveTorrents)
 			torrentDir = new File(COConfigurationManager
 					.getDirectoryParameter("General_sDefaultTorrent_Directory"));
@@ -1782,8 +2111,7 @@ TorrentUtils
 
 		//if the torrent is already in the completed files dir, use this
 		//torrent instead of creating a new one in the default dir
-		boolean moveWhenDone = COConfigurationManager.getBooleanParameter(
-				"Move Completed When Done", false);
+		boolean moveWhenDone = COConfigurationManager.getBooleanParameter("Move Completed When Done");
 		String completedDir = COConfigurationManager.getStringParameter(
 				"Completed Files Directory", "");
 		if (moveWhenDone && completedDir.length() > 0) {
