@@ -27,27 +27,21 @@ package org.gudy.azureus2.platform.win32;
  *
  */
 
+import java.io.File;
+import java.net.InetAddress;
+import java.util.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
-import org.gudy.azureus2.core3.logging.*;
-import org.gudy.azureus2.core3.util.AEMonitor;
-import org.gudy.azureus2.core3.util.Constants;
-import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.core3.util.SystemProperties;
-import org.gudy.azureus2.platform.PlatformManager;
-import org.gudy.azureus2.platform.PlatformManagerCapabilities;
-import org.gudy.azureus2.platform.PlatformManagerListener;
-import org.gudy.azureus2.platform.PlatformManagerPingCallback;
+import org.gudy.azureus2.core3.logging.LogAlert;
+import org.gudy.azureus2.core3.logging.Logger;
+import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.platform.*;
 import org.gudy.azureus2.platform.win32.access.AEWin32Access;
 import org.gudy.azureus2.platform.win32.access.AEWin32AccessListener;
 import org.gudy.azureus2.platform.win32.access.AEWin32Manager;
 
 import org.gudy.azureus2.plugins.platform.PlatformManagerException;
 
-import java.io.File;
-import java.io.IOException;
-import java.net.InetAddress;
-import java.util.*;
-
 
 public class 
 PlatformManagerImpl
@@ -59,6 +53,7 @@ PlatformManagerImpl
 	
 	public static final String					DLL_NAME = "aereg";
 	
+	public static final String				VUZE_ASSOC		= "Vuze";
 	public static final String				NEW_MAIN_ASSOC	= "Azureus";
 	public static final String				OLD_MAIN_ASS0C	= "BitTorrent";
 	
@@ -164,6 +159,7 @@ PlatformManagerImpl
 	        capabilitySet.add(PlatformManagerCapabilities.ShowFileInBrowser);
 	        capabilitySet.add(PlatformManagerCapabilities.GetVersion);
 	        capabilitySet.add(PlatformManagerCapabilities.SetTCPTOSEnabled);
+	        capabilitySet.add(PlatformManagerCapabilities.ComputerIDAvailability);
 	        
 	        
 	        if ( Constants.compareVersions( access.getVersion(), "1.11" ) >= 0 &&
@@ -178,9 +174,13 @@ PlatformManagerImpl
 	            capabilitySet.add(PlatformManagerCapabilities.TestNativeAvailability);
 	        }
 	        
-	        if ( Constants.compareVersions( access.getVersion(), "1.2" ) >= 0 ){
+	        if ( Constants.compareVersions( access.getVersion(), "1.14" ) >= 0 ){
 	        	
 	            capabilitySet.add(PlatformManagerCapabilities.TraceRouteAvailability);
+	        }
+
+	        if ( Constants.compareVersions( access.getVersion(), "1.15" ) >= 0 ){
+	        	
 	            capabilitySet.add(PlatformManagerCapabilities.PingAvailability);
 	        }
 
@@ -219,8 +219,7 @@ PlatformManagerImpl
 			
 			if ( current.indexOf( app_exe_name ) != -1 && !current.equals(target)){
 				
-				access.writeStringValue( 	
-						AEWin32Access.HKEY_CLASSES_ROOT,
+				writeStringToHKCRandHKCU( 	
 						NEW_MAIN_ASSOC + "\\DefaultIcon",
 						"",
 						target );
@@ -316,24 +315,26 @@ PlatformManagerImpl
 			
 				String az_home;
 				
-				try{
-					az_home = access.getApplicationInstallDir( SystemProperties.getApplicationName());
-					
-					az_exe = new File( az_home + File.separator + app_exe_name ).getAbsoluteFile();
-	
-					if ( !az_exe.exists()){
-						
-						throw( new PlatformManagerException( app_exe_name + " not found in " + az_home + ", please re-install"));
-					}
-				}catch( Throwable e ){
-					
-						//hmmm, well let's try the app dir
-					
-					az_home = SystemProperties.getApplicationPath();		
-					
-					az_exe = new File( az_home + File.separator + app_exe_name ).getAbsoluteFile();
-				}
+				// Try the app dir first, because we may not be using the one in the registry
+				az_home = SystemProperties.getApplicationPath();		
 				
+				az_exe = new File(az_home + File.separator + app_exe_name).getAbsoluteFile();
+
+				if (!az_exe.exists()) {
+					try {
+						az_home = access.getApplicationInstallDir(SystemProperties.getApplicationName());
+
+						az_exe = new File(az_home + File.separator + app_exe_name).getAbsoluteFile();
+
+						if (!az_exe.exists()) {
+
+							throw (new PlatformManagerException(app_exe_name
+									+ " not found in " + az_home + ", please re-install"));
+						}
+					} catch (Throwable e) {
+					}
+				}
+								
 				if ( !az_exe.exists()){
 					
 					String	msg = app_exe_name + " not found in " + az_home + " - can't check file associations. Please re-install " + SystemProperties.getApplicationName();
@@ -459,7 +460,7 @@ PlatformManagerImpl
 			
 			unregisterAdditionalFileType( OLD_MAIN_ASS0C, ".torrent" );
 			
-			registerAdditionalFileType( NEW_MAIN_ASSOC, "Azureus Download", ".torrent", "application/x-bittorrent" );
+			registerAdditionalFileType( NEW_MAIN_ASSOC, Constants.APP_NAME + " Download", ".torrent", "application/x-bittorrent" );
 		}
 		
 		boolean	reg = isAdditionalFileTypeRegistered( NEW_MAIN_ASSOC, ".torrent" );
@@ -468,13 +469,22 @@ PlatformManagerImpl
 		
 		if ( !reg && !COConfigurationManager.getBooleanParameter( "platform.win32.autoregdone", false )){
 			
-			registerAdditionalFileType( NEW_MAIN_ASSOC, "Azureus Download", ".torrent", "application/x-bittorrent" );
+			registerAdditionalFileType( NEW_MAIN_ASSOC, Constants.APP_NAME + " Download", ".torrent", "application/x-bittorrent" );
 
 			COConfigurationManager.setParameter( "platform.win32.autoregdone", true );
 			
 			reg	= true;
 		}
 		
+			// always register .vuze association
+		
+		boolean	vuze_reg = isAdditionalFileTypeRegistered( VUZE_ASSOC, ".vuze" );
+
+		if ( !vuze_reg ){
+			
+			registerAdditionalFileType( VUZE_ASSOC, "Vuze File", ".vuze", "application/x-vuze" );
+		}
+		
 		return( reg );
 	}
 	
@@ -518,6 +528,15 @@ PlatformManagerImpl
 				return( test1.length() ==0?RT_NONE:RT_OTHER );
 			}
 			
+			String test2 =
+				access.readStringValue(
+						AEWin32Access.HKEY_CLASSES_ROOT,
+						type,
+						"");
+			if ( !test2.equals( NEW_MAIN_ASSOC )) {
+				return test2.length() == 0 ? RT_NONE : RT_OTHER;
+			}
+
 				// MRU list is just that, to remove the "always open with" we need to kill
 				// the "application" entry, if it exists
 			
@@ -594,7 +613,9 @@ PlatformManagerImpl
 	{
 		registerMagnet();
 		
-		registerAdditionalFileType( NEW_MAIN_ASSOC, "Azureus Download", ".torrent", "application/x-bittorrent" );
+		registerAdditionalFileType( NEW_MAIN_ASSOC, Constants.APP_NAME + " Download", ".torrent", "application/x-bittorrent" );
+		
+		registerAdditionalFileType( VUZE_ASSOC, "Vuze File", ".vuze", "application/x-vuze" );
 	}
 	
 	protected void
@@ -636,7 +657,7 @@ PlatformManagerImpl
 		throws PlatformManagerException
 	{
 		// 	WriteRegStr HKCR ".torrent" "" "Azureus"
-		// 	WriteRegStr HKCR "Azureus" "" "Azureus Torrent"
+		// 	WriteRegStr HKCR "Azureus" "" "Vuze Torrent"
 		// 	WriteRegStr HKCR "Azureus\shell" "" "open"
 		// 	WriteRegStr HKCR "Azureus\DefaultIcon" "" $INSTDIR\Azureus.exe,1
 		// 	WriteRegStr HKCR "Azureus\shell\open\command" "" '"$INSTDIR\Azureus.exe" "%1"'
@@ -648,46 +669,39 @@ PlatformManagerImpl
 			
 			unregisterAdditionalFileType( name, type );
 
-			access.writeStringValue( 	
-					AEWin32Access.HKEY_CLASSES_ROOT,
+			writeStringToHKCRandHKCU(
 					type,
 					"",
 					name );
 		
-			access.writeStringValue( 	
-					AEWin32Access.HKEY_CLASSES_ROOT,
+			writeStringToHKCRandHKCU(
 					name,
 					"",
 					description );
 			
-			access.writeStringValue( 	
-					AEWin32Access.HKEY_CLASSES_ROOT,
+			writeStringToHKCRandHKCU(
 					name + "\\shell",
 					"",
 					"open" );
 			
-			access.writeStringValue( 	
-					AEWin32Access.HKEY_CLASSES_ROOT,
+			writeStringToHKCRandHKCU(
 					name + "\\DefaultIcon",
 					"",
 					az_exe_string + "," + getIconIndex());
 			
-			access.writeStringValue( 	
-					AEWin32Access.HKEY_CLASSES_ROOT,
+			writeStringToHKCRandHKCU(
 					name + "\\shell\\open\\command",
 					"",
 					"\"" + az_exe_string + "\" \"%1\"" );
 					
-			access.writeStringValue( 	
-					AEWin32Access.HKEY_CLASSES_ROOT,
+			writeStringToHKCRandHKCU(
 					name + "\\Content Type" ,
 					"",
 					content_type );
 			
 			if ( url_protocol ){
 				
-				access.writeStringValue( 	
-						AEWin32Access.HKEY_CLASSES_ROOT,
+				writeStringToHKCRandHKCU(
 						name,
 						"URL Protocol",
 						"" );
@@ -703,6 +717,24 @@ PlatformManagerImpl
 		}
 	}
 	
+	private void writeStringToHKCRandHKCU(String subkey, String name, String value) {
+		// HKCU will most likely fail on Vista due to permissions
+		try {
+			access.writeStringValue(AEWin32Access.HKEY_CLASSES_ROOT, subkey, name, value);
+		} catch (Throwable e) {
+			if (!Constants.isWindowsVista) {
+				Debug.out(e);
+			}
+		}
+
+		try {
+			access.writeStringValue(AEWin32Access.HKEY_CURRENT_USER,
+					"Software\\Classes\\" + subkey, name, value);
+		} catch (Throwable e) {
+			Debug.out(e);
+		}
+	}
+	
 	public void
 	unregisterAdditionalFileType(
 		String		name,				// e.g. "Azureus"
@@ -743,7 +775,28 @@ PlatformManagerImpl
 				
 				// Debug.printStackTrace( e );
 			}
+
+			try{
+				access.deleteKey( 	
+					AEWin32Access.HKEY_CURRENT_USER,
+					"Software\\Classes\\" + type );
+				
+			}catch( Throwable e ){
+				
+				// Debug.printStackTrace( e );
+			}
 			
+			try{
+				access.deleteKey( 	
+					AEWin32Access.HKEY_CURRENT_USER,
+					"Software\\Classes\\" + name,
+					true );
+				
+			}catch( Throwable e ){
+				
+				// Debug.printStackTrace( e );
+			}
+
 		}catch( Throwable e ){
 			
 			throw( new PlatformManagerException( "Failed to delete registry details", e ));
@@ -826,12 +879,17 @@ PlatformManagerImpl
         try
         {
         	File file = new File(file_name);
+        	
+        	access.createProcess( "explorer.exe " + ( file.isDirectory() ? "/e," : "/e,/select," ) + "\"" + file_name + "\"", false );
+        	
+        	/*
         	Runtime.getRuntime().exec(
         			new String[] { "explorer.exe",
         					file.isDirectory() ? "/e," : "/e,/select,",
         							"\"" + file_name + "\"" });
+        							*/
         }
-        catch (IOException e)
+        catch (Throwable e)
         {
             throw new PlatformManagerException("Failed to show file " + file_name, e);
         }
@@ -861,7 +919,7 @@ PlatformManagerImpl
 	traceRoute(
 		InetAddress							interface_address,
 		InetAddress							target,
-		PlatformManagerPingCallback	callback )
+		PlatformManagerPingCallback			callback )
 	
 		throws PlatformManagerException
 	{
@@ -901,6 +959,16 @@ PlatformManagerImpl
 		}	
 	}
 	
+	public int shellExecute(String operation, String file, String parameters,
+			String directory, int SW_const) throws PlatformManagerException {
+		try {
+			return access.shellExecute(operation, file, parameters, directory, SW_const);
+		} catch (Throwable e) {
+			throw( new PlatformManagerException( "Failed to shellExecute", e ));
+		}
+	}
+
+	
     /**
      * {@inheritDoc}
      */
@@ -928,6 +996,14 @@ PlatformManagerImpl
 			
 			t_type = PlatformManagerListener.ET_SHUTDOWN;
 			
+		}else if ( type == AEWin32AccessListener.ET_SUSPEND ){
+			
+			t_type = PlatformManagerListener.ET_SUSPEND;
+				
+		}else if ( type == AEWin32AccessListener.ET_RESUME ){
+			
+			t_type = PlatformManagerListener.ET_RESUME;
+				
 		}else{
 			
 			return;
@@ -961,4 +1037,94 @@ PlatformManagerImpl
     {
     	listeners.remove( listener );
     }
+
+  /**
+   * Gets an ID to identify this computer to azureus.  Used when the computer
+   * has muliple user accounts and we need a way to not duplicate efforts
+   * (An example would be to skip downloading something another user on the
+   * computer has already downloaded)
+   * <p>
+   * The default for the ID is the AZID of the first user profile.
+	 * 
+	 * @return ID
+	 */
+	public String getAzComputerID() {
+		boolean needWrite = false;
+		String cid = null;
+		try {
+			cid = access.readStringValue(AEWin32Access.HKEY_LOCAL_MACHINE,
+					"SOFTWARE\\Azureus", "CID");
+		} catch (Exception e) {
+		}
+
+		if (cid == null || cid.length() == 0) {
+			needWrite = true;
+			try {
+				File commonPath = new File(access.getCommonAppData(),
+						SystemProperties.APPLICATION_NAME);
+				if (commonPath.isDirectory()) {
+					File fCID = new File(commonPath, "azCID.txt");
+					if (fCID.exists()) {
+						cid = FileUtil.readFileAsString(fCID, 255, "utf8");
+					}
+				}
+			} catch (Exception e) {
+			}
+		}
+
+		if (cid == null || cid.length() == 0) {
+			needWrite = true;
+			cid = COConfigurationManager.getStringParameter("ID");
+		}
+
+		if (cid == null || cid.length() == 0) {
+			needWrite = true;
+			cid = RandomUtils.generateRandomAlphanumerics(20);
+		}
+
+		if (needWrite) {
+			setAzComputerID(cid);
+		}
+		return cid;
+	}
+
+	/**
+	 * @param cid
+	 */
+	private void setAzComputerID(String cid) {
+		try {
+			access.writeStringValue(AEWin32Access.HKEY_LOCAL_MACHINE,
+					"SOFTWARE\\Azureus", "CID", cid);
+		} catch (Exception e) {
+			Debug.out("Could not write CID: " + e.getMessage());
+		}
+
+		try {
+			String sCommonAppData = access.getCommonAppData();
+			if (sCommonAppData != null && sCommonAppData.length() > 0) {
+				File commonPath = new File(sCommonAppData);
+				if (commonPath.isDirectory()) {
+					commonPath = new File(commonPath, SystemProperties.APPLICATION_NAME);
+					FileUtil.mkdirs(commonPath);
+
+					File fCID = new File(commonPath, "azCID.txt");
+					FileUtil.writeBytesAsFile(fCID.getAbsolutePath(),
+							cid.getBytes("utf8"));
+				}
+			}
+		} catch (Exception e) {
+			e.printStackTrace();
+		}
+	}
+	
+	public static void main(String[] args) {
+		try {
+			PlatformManagerImpl impl = new PlatformManagerImpl();
+			System.out.println(impl.getAzComputerID());
+		} catch (PlatformManagerException e) {
+			// TODO Auto-generated catch block
+			e.printStackTrace();
+		}
+		
+	}
 }
