@@ -34,12 +34,15 @@
 #include "org_gudy_azureus2_platform_win32_access_impl_AEWin32AccessInterface.h"
 
 
-#define VERSION "1.2"
+#define VERSION "1.16"
 
  
 HMODULE	application_module;
 bool	non_unicode			= false;
 
+jclass AEWin32AccessInterface_class;
+jclass AEWin32AccessExceptionImpl_class;
+
 UINT		uThreadId;
 HINSTANCE	hInstance;
 LRESULT CALLBACK WndProcW(HWND, UINT, WPARAM, LPARAM);
@@ -106,19 +109,17 @@ throwException(
 	char*			operation,
 	char*			message )
 {
-	jclass except = env->FindClass( "org/gudy/azureus2/platform/win32/access/impl/AEWin32AccessExceptionImpl" );
-	
 	bool	ok = false;
 
-	if ( except != NULL ){
+	if ( AEWin32AccessExceptionImpl_class != NULL ){
 
-		jmethodID method = env->GetMethodID( except, "<init>", "(Ljava/lang/String;Ljava/lang/String;)V" );
+		jmethodID method = env->GetMethodID( AEWin32AccessExceptionImpl_class, "<init>", "(Ljava/lang/String;Ljava/lang/String;)V" );
 	
 		if ( method != NULL ){
 
 	
 			jobject new_object =
-					env->NewObject( except, 
+					env->NewObject( AEWin32AccessExceptionImpl_class, 
 									method, 
 									env->NewStringUTF((const char*)operation), 
 									env->NewStringUTF((const char*)message ));
@@ -129,6 +130,9 @@ throwException(
 
 				ok = true;
 			}
+		}else{
+
+			fprintf( stderr, "AEWin32AccessInterface: failed to resolve constructor" );
 		}
 	}
 
@@ -377,30 +381,32 @@ callback(
 		return( -1 );
 	}
 
-	jclass callback_class = env->FindClass("org/gudy/azureus2/platform/win32/access/impl/AEWin32AccessInterface" );
 
 	jlong result = -1;
 
-	if ( callback_class != NULL ){
+	if ( AEWin32AccessInterface_class != NULL ){
 
 		jint	j_msg		= Msg;
 		jint	j_param1	= wParam;
 		jlong	j_param2	= lParam;
 
 
-		jmethodID method = env->GetStaticMethodID(callback_class, "callback", "(IIJ)J");
+		jmethodID method = env->GetStaticMethodID( AEWin32AccessInterface_class, "callback", "(IIJ)J");
 
-		result = env->CallStaticLongMethod(callback_class, method, j_msg, j_param1, j_param2 );
+		if ( method != NULL ){
+
+			result = env->CallStaticLongMethod( AEWin32AccessInterface_class, method, j_msg, j_param1, j_param2 );
+
+		}else{
+					
+			fprintf( stderr, "failed to resolve callback method\n" );
+		}
 	}
 
-	if ( callback_class != NULL ){
-
-		env->DeleteLocalRef( callback_class );
-	}
 
 	jvm->DetachCurrentThread();
 
-	return((HRESULT)result );
+	return((HRESULT)result ); 
 }
 
 
@@ -413,10 +419,38 @@ Java_org_gudy_azureus2_platform_win32_access_impl_AEWin32AccessInterface_initial
 
 	env->GetJavaVM(&jvm);
 
+	jclass local_ref = env->FindClass("org/gudy/azureus2/platform/win32/access/impl/AEWin32AccessInterface" );
+
+	if ( local_ref == NULL ){
+
+		fprintf( stderr, "failed to find AEWin32AccessInterface class\n" );
+
+	}else{
+
+		AEWin32AccessInterface_class = (jclass)env->NewGlobalRef( local_ref );
+
+		env->DeleteLocalRef( local_ref );
+	}
+
+	local_ref = env->FindClass( "org/gudy/azureus2/platform/win32/access/impl/AEWin32AccessExceptionImpl" );
+	
+	if ( local_ref == NULL ){
+
+		fprintf( stderr, "failed to find AEWin32AccessExceptionImpl class\n" );
+
+	}else{
+
+		AEWin32AccessExceptionImpl_class = (jclass)env->NewGlobalRef( local_ref );
+
+		env->DeleteLocalRef( local_ref );
+	}
+
 	if ( non_unicode ){
+
 		hThread = (HANDLE)_beginthreadex(NULL, 0, &CreateWndThreadA, NULL, 0, &uThreadId);
 
 	}else{
+
 		hThread = (HANDLE)_beginthreadex(NULL, 0, &CreateWndThreadW, NULL, 0, &uThreadId);
 	}
 
@@ -1030,6 +1064,48 @@ Java_org_gudy_azureus2_platform_win32_access_impl_AEWin32AccessInterface_testNat
 	}
 }
 
+// 1.3
+JNIEXPORT jint JNICALL
+Java_org_gudy_azureus2_platform_win32_access_impl_AEWin32AccessInterface_shellExecuteW(
+	JNIEnv	*env,
+	jclass	cla,
+	jstring _operation,
+	jstring	_file,
+	jstring _parameters,
+	jstring _directory,
+	jint _showCmd )
+{
+	WCHAR		operation[20];
+	WCHAR		file[5192];
+	WCHAR		parameters[16000];
+	WCHAR		directory[5192];
+	INT			showCmd = _showCmd;
+	
+	if ( !jstringToCharsW( env, _operation, operation, sizeof( operation ))){
+		return -1;
+	}
+
+	if ( !jstringToCharsW( env, _file, file, sizeof( file ))){
+		return -1;
+	}
+
+	if ( !jstringToCharsW( env, _parameters, parameters, sizeof( parameters ))){
+		return -1;
+	}
+
+	if ( !jstringToCharsW( env, _directory, directory, sizeof( directory ))){
+		return -1;
+	}
+	
+	// Not sure if ShellExecute treats "\0" as NULL, so do explicit check 
+	return(jint) ShellExecuteW(NULL,
+			_operation == NULL ? NULL : operation,
+			_file == NULL ? NULL : file,
+			_parameters == NULL ? NULL : parameters,
+			_directory == NULL ? NULL : directory,
+			showCmd);
+}
+
 
 // NON-UNICODE VARIANT FOR WIN95,98,ME
 
@@ -1658,6 +1734,48 @@ Java_org_gudy_azureus2_platform_win32_access_impl_AEWin32AccessInterface_testNat
 	}
 }
 
+// 1.3
+JNIEXPORT jint JNICALL
+Java_org_gudy_azureus2_platform_win32_access_impl_AEWin32AccessInterface_shellExecuteA(
+	JNIEnv	*env,
+	jclass	cla,
+	jstring _operation,
+	jstring	_file,
+	jstring _parameters,
+	jstring _directory,
+	jint _showCmd )
+{
+	char	operation[20];
+	char	file[5192];
+	char	parameters[16000];
+	char	directory[5192];
+	INT		showCmd = _showCmd;
+	
+	if ( !jstringToCharsA( env, _operation, operation, sizeof( operation ))){
+		return -1;
+	}
+
+	if ( !jstringToCharsA( env, _file, file, sizeof( file ))){
+		return -1;
+	}
+
+	if ( !jstringToCharsA( env, _parameters, parameters, sizeof( parameters ))){
+		return -1;
+	}
+
+	if ( !jstringToCharsA( env, _directory, directory, sizeof( directory ))){
+		return -1;
+	}
+	
+	// Not sure if ShellExecute treats "\0" as NULL, so do explicit check 
+	return (jint)ShellExecuteA(NULL,
+			_operation == NULL ? NULL : operation,
+			_file == NULL ? NULL : file,
+			_parameters == NULL ? NULL : parameters,
+			_directory == NULL ? NULL : directory,
+			showCmd);
+}
+
 
 // BLAH
 
@@ -1832,3 +1950,24 @@ Java_org_gudy_azureus2_platform_win32_access_impl_AEWin32AccessInterface_testNat
 		return( Java_org_gudy_azureus2_platform_win32_access_impl_AEWin32AccessInterface_testNativeAvailabilityW( env, cla, name ));
 	}
 }
+
+JNIEXPORT jint JNICALL
+Java_org_gudy_azureus2_platform_win32_access_impl_AEWin32AccessInterface_shellExecute(
+	JNIEnv	*env,
+	jclass	cla,
+	jstring operation,
+	jstring	file,
+	jstring parameters,
+	jstring directory,
+	jint showCmd )
+{
+	if ( non_unicode ){
+		return
+			Java_org_gudy_azureus2_platform_win32_access_impl_AEWin32AccessInterface_shellExecuteA(
+				env, cla, operation, file, parameters, directory, showCmd );
+	}else{
+		return
+			Java_org_gudy_azureus2_platform_win32_access_impl_AEWin32AccessInterface_shellExecuteW(
+				env, cla, operation, file, parameters, directory, showCmd );
+	}
+}
