@@ -24,6 +24,9 @@ package org.gudy.azureus2.plugins.download;
 import java.io.File;
 import java.util.Map;
 
+import org.gudy.azureus2.plugins.download.savelocation.DefaultSaveLocationManager;
+import org.gudy.azureus2.plugins.download.savelocation.SaveLocationChange;
+import org.gudy.azureus2.plugins.download.savelocation.SaveLocationManager;
 import org.gudy.azureus2.plugins.torrent.Torrent;
 import org.gudy.azureus2.plugins.torrent.TorrentAttribute;
 import org.gudy.azureus2.plugins.disk.DiskManager;
@@ -31,7 +34,11 @@ import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.plugins.download.session.SessionAuthenticator;
 import org.gudy.azureus2.plugins.peers.PeerManager;
 
-/** Management of a Torrent's activity.
+/**
+ * Management of a Torrent's activity.
+ * 
+ * <b>Note:</b> All listener based methods are now located in {@link DownloadEventNotifier}.
+ * 
  * <PRE>
  * A download's lifecycle:
  * torrent gets added
@@ -59,7 +66,7 @@ import org.gudy.azureus2.plugins.peers.PeerManager;
  */
 
 public interface 
-Download 
+Download extends DownloadEventNotifier
 {
   /** waiting to be told to start preparing */  
 	public static final int ST_WAITING     = 1;
@@ -116,7 +123,7 @@ Download
 	 * 
 	 * @since 2.5.0.1
 	 */
-	public static final long FLAG_DISABLE_AUTO_FILE_MOVE   = 0x00000004;
+	public static final long FLAG_DISABLE_AUTO_FILE_MOVE 	= 0x00000004;
 
     /**
      * Flag value - if set, then it means this download has been considered
@@ -125,8 +132,31 @@ Download
      * 
      * @since 2.5.0.1
      */
-    public static final long FLAG_MOVE_ON_COMPLETION_DONE  = 0x00000008;
+    public static final long FLAG_MOVE_ON_COMPLETION_DONE	= 0x00000008;
 	
+    /**
+     * Flag value - if set the user won't be bothered with popups/completion events during
+     * the download's life. This is used, for example, for downloads used to run speed-tests
+     * @since 3.0.1.3
+     */
+    
+    public static final long FLAG_LOW_NOISE					= 0x00000010;
+
+    	/**
+    	 * Flag value - normally the permitted peer sources for a download are fixed and can't be changed
+    	 * this flag allows the permitted peer source set to be increased/decreased (but not beyond the enforced
+    	 * values required to honour a torrent's 'private' flag
+    	 */
+    public static final long FLAG_ALLOW_PERMITTED_PEER_SOURCE_CHANGES = 0x00000020;
+    
+    
+    /**
+     * Flag value - if set the data will not be delete when the download is "deleted" from
+     * the v3 interface.
+     * @since 3.1.0.0
+     */
+    public static final long FLAG_DO_NOT_DELETE_DATA_ON_REMOVE = 0x00000040;
+    
 	/** get state from above ST_ set
    * @return ST_ constant
    *
@@ -636,18 +666,36 @@ Download
 	getMaximumDownloadKBPerSecond();
     
     
-    /**
-     * Get the max upload rate allowed for this download.
-     * @return upload rate in bytes per second, 0 for unlimited, -1 for upload disabled
-     */
+	    /**
+	     * Get the max upload rate allowed for this download.
+	     * @return upload rate in bytes per second, 0 for unlimited, -1 for upload disabled
+	     */
+  	
     public int getUploadRateLimitBytesPerSecond();
     
-    /**
-     * Set the max upload rate allowed for this download.
-     * @param max_rate_bps limit in bytes per second, 0 for unlimited, -1 for upload disabled
-     */
+	    /**
+	     * Set the max upload rate allowed for this download.
+	     * @param max_rate_bps limit in bytes per second, 0 for unlimited, -1 for upload disabled
+	     */
+    
     public void setUploadRateLimitBytesPerSecond( int max_rate_bps );
     
+	    /**
+	     * Get the max download rate allowed for this download.
+	     * @return upload rate in bytes per second, 0 for unlimited, -1 for download disabled
+	     * @since 3013
+	     */
+    
+    public int getDownloadRateLimitBytesPerSecond();
+    
+	    /**
+	     * Set the max download rate allowed for this download.
+	     * @param max_rate_bps limit in bytes per second, 0 for unlimited, -1 for dowmload disabled
+	     * @since 3013
+	     */
+    
+    public void setDownloadRateLimitBytesPerSecond( int max_rate_bps );
+  
 
 	/**
 	 * Indicates if the download has completed or not, exluding any files marked
@@ -692,7 +740,13 @@ Download
   	
   		/**
   		 * Move a download's data files to a new location. Download must be stopped and persistent
-  		 * @since 2.3.0.5
+  		 * 
+  		 * <p>
+  		 * 
+  		 * If a download is running, it will be automatically paused and resumed afterwards - be
+  		 * aware that this behaviour may generate <tt>stateChanged</tt> events being fired.
+		 *
+		 * @since 2.3.0.5
   		 * @param new_parent_dir
   		 * @throws DownloadException
   		 */
@@ -703,6 +757,26 @@ Download
   	
   		throws DownloadException;
   	
+  	/**
+  	 * Move a download's data files to a new location, and rename the download at the same time.
+  	 * Download must be stopped and persistent. This is equivalent to calling <tt>moveDataFiles[File]</tt>
+  	 * and then <tt>renameDownload[String]</tt>.
+  	 * 
+  	 * For convenience, either argument can be <tt>null</tt>, but not both.
+  	 * 
+  	 * <p>
+  	 * 
+  	 * If a download is running, it will be automatically paused and resumed afterwards - be
+  	 * aware that this behaviour may generate <tt>stateChanged</tt> events being fired.
+  	 * 
+  	 * @since 3.0.2
+  	 * @throws DownloadException
+  	 * @see {@link #moveDataFiles(File)}
+  	 * @see {@link #renameDownload(String)}
+  	 */
+  	public void moveDataFiles(File new_parent_dir, String new_name) throws DownloadException;
+  	
+  	
   		/**
 		 * Move a download's torrent file to a new location. Download must be stopped and persistent
 		 * @since 2.3.0.5
@@ -725,6 +799,11 @@ Download
   	 * This will not rename the displayed name for the torrent - if you wish to do that, you must do it via
   	 * the {@link org.gudy.azureus2.plugins.torrent.TorrentAttribute TorrentAttribute} class.
   	 * 
+  	 * <p>
+  	 * 
+  	 * If a download is running, it will be automatically paused and resumed afterwards - be
+  	 * aware that this behaviour may generate <tt>stateChanged</tt> events being fired.
+  	 * 
   	 * @param name New name for the download.
   	 * @see #moveDataFiles(File)
   	 */
@@ -783,120 +862,10 @@ Download
 	requestTrackerScrape(
 		boolean		immediate );
 	
-	/**
-	 * Adds a listener to the download that will be informed of changes in the download's state
-	 * @param l
-   *
-   * @since 2.0.7.0
-	 */
-	public void
-	addListener(
-		DownloadListener	l );
-	
-	/**
-	 * Removes listeners added above
-	 * @param l
-   *
-   * @since 2.0.7.0
-	 */
-	public void
-	removeListener(
-		DownloadListener	l );
 
-	/**
-	 * Adds a listener that will be informed when the latest announce/scrape results change.
-	 * <p>
-	 * listener events will be immediately triggered after listener is added
-	 * 
-	 * @param l
-   *
-   * @since 2.0.7.0
-	 */
-	public void
-	addTrackerListener(
-		DownloadTrackerListener	l );
 	
-	/**
-	 * Adds a listener that will be informed when the latest announce/scrape results change
-	 *  
-	 * @param l Listener to add
-	 * @param immediateTrigger Whether to immediately trigger listener's events
-   *
-   * @since 2.4.0.3
-	 */
-	void addTrackerListener(DownloadTrackerListener l, boolean immediateTrigger);
-  
-	/**
-	 * Removes listeners added above
-	 * @param l
-   *
-   * @since 2.0.7.0
-	 */
-	public void
-	removeTrackerListener(
-		DownloadTrackerListener	l );
-	
-	/**
-	 * Adds a listener that will be informed when a download is about to be removed. This gives
-	 * the implementor the opportunity to veto the removal
-	 * @param l
-   *
-   * @since 2.0.7.0
-	 */
-	public void
-	addDownloadWillBeRemovedListener(
-		DownloadWillBeRemovedListener	l );
-	
-	/**
-	 * Removes the listener added above
-	 * @param l
-   *
-   * @since 2.0.7.0
-	 */
-	public void
-	removeDownloadWillBeRemovedListener(
-		DownloadWillBeRemovedListener	l );
-	
-		/**
-		 * Add a listener that will be invoked when a request is made to activate a torrent
-		 * @param l
-		 * @since 2403
-		 */
-	
-	public void
-	addActivationListener(
-		DownloadActivationListener		l );
-	
-		/**
-		 * Remove a listener that will be invoked when a request is made to activate a torrent
-		 * @param l
-		 * @since 2403
-		 */
-	
-	public void
-	removeActivationListener(
-		DownloadActivationListener		l );
-	
-	/**
-	 * Adds a listener to the download that will be informed of 
-	 * @param l
-   *
-   * @since 2.1.0.0
-	 */
-	public void
-	addPeerListener(
-		DownloadPeerListener	l );
-	
-	/**
-	 * Removes listeners added above
-	 * @param l
-   *
-   * @since 2.1.0.0
-	 */
-	public void
-	removePeerListener(
-		DownloadPeerListener	l );
 
+	
 	/**
 	 * The torrents with the highest rankings will be seeded first.
 	 * 
@@ -911,14 +880,6 @@ Download
 	 */
 	public void setSeedingRank(int rank);
   
-	public void
-	addPropertyListener(
-		DownloadPropertyListener	l );
-	
-	public void
-	removePropertyListener(
-		DownloadPropertyListener	l );
-	
   /**
    * Get the local peerID advertised to the download swarm.
    * @return self peer id
@@ -986,20 +947,89 @@ Download
    * @param for_moving Indicates whether you want this information for the purposes
    *     of moving the download or not.
    * @author amc1
+   * @deprecated Use {@link #calculateDefaultDownloadLocation()} instead.
    * @return An array of type <tt>File</tt> of size 2, first element containing the
    *     calculated location for the download's data files, and the second element
    *     containing the location for the download's torrent file.
    */
   public File[] calculateDefaultPaths(boolean for_moving);
-  
+
   /**
    * Returns <tt>true</tt> if the download is being saved to one of the default
    * save directories.
    * 
    * @since 2.5.0.2
+   * @deprecated Use {@link DefaultSaveLocationManager#isInDefaultSaveDir(Download)} instead.
    * @author amc1
    */
   public boolean isInDefaultSaveDir();
+  
+  /**
+   * @since 3.0.4.3
+   * @return
+   */
+  
+  public boolean isRemoved();
+  
+  /**
+   * Returns <tt>true</tt> if Azureus will allow the data files for the torrent
+   * to be moved.
+   * 
+   * @since 3.0.5.1
+   */
+  public boolean canMoveDataFiles();
+  
+  /**
+   * Returns a {@link SaveLocationChange} object describing the appropriate location
+   * for the download (and torrent file) to exist in, based on the download's completion
+   * state, the <tt>for-completion</tt> rules in place, and the {@link SaveLocationManager}
+   * object in use.
+   * 
+   * @since 3.0.5.3 
+   */
+  public SaveLocationChange calculateDefaultDownloadLocation();
+  
+  /**
+   * Apply the changes in the given {@link SaveLocationChange} object - this includes
+   * moving torrent and data file data.
+   * 
+   * @param slc The change to apply.
+   * @since 3.1.0.1
+   * @throws DownloadException If there is a problem moving the data.
+   */
+  public void changeLocation(SaveLocationChange slc) throws DownloadException;
 
-
+  	/**
+  	 * get user-defined key/value
+  	 * @param key
+  	 * @return
+  	 * @since 3.0.5.3
+  	 */
+  
+  public Object getUserData( Object key );
+  
+  	/**
+  	 * set user defined value. this is TRANSIENT and not persisted over Azureus stop/start
+  	 * @param key
+  	 * @param data
+  	 */
+  public void setUserData( Object key, Object data );
+  
+  /**
+   * Simple method to start the download. Will not raise an error if it
+   * didn't work, or if the download is already running.
+   * 
+   * @since 3.0.5.3
+   * @param force <tt>true</tt> to force the download to be started.
+   */
+  public void startDownload(boolean force);
+  
+  /**
+   * Simple method to stop the download. Will not raise an error if it
+   * didn't work, or if the download is already stopped.
+   * 
+   * @since 3.0.5.3
+   */
+  public void stopDownload();
+  
 }
