@@ -121,6 +121,15 @@ DownloadStats
 	public int
 	getDownloadCompleted(boolean bLive);
 
+	/**
+	 * if isChecking then returns completeness, 1000 -> 100%
+	 * @return
+	 */
+
+	public int
+	getCheckingDoneInThousandNotation();
+
+
 	/**
 	 * Gives the number of bytes downloaded
 	 * @return
@@ -130,6 +139,14 @@ DownloadStats
 	public long
 	getDownloaded();
 	
+	/**
+	 * Gives number of bytes remaining
+	 * @return
+	 */
+	
+	public long
+	getRemaining();
+	
 	/**
 	 * Gives the number of bytes uploaded
 	 * @return
@@ -260,6 +277,24 @@ DownloadStats
 	public long 
 	getSecondsOnlySeeding();
 	
+	/**
+	 * Returns the number of seconds running time since data was downloaded, -1 if never 
+	 * @since 2501
+	 * @return
+	 */
+	
+	public long
+	getSecondsSinceLastDownload();
+	
+	/**
+	 * Returns the number of seconds running time since data was uploaded, -1 if never 
+	 * @since 2501
+	 * @return
+	 */
+	
+	public long
+	getSecondsSinceLastUpload();
+	
 	/**
 	 * returns an indication of the health of the torrent 
 	 * @return	see above HEALTH constants
