@@ -45,6 +45,9 @@ InstallablePlugin
 	public String
 	getDescription();
 	
+	public String
+	getRelativeURLBase();
+	
 		/**
 		 * Returns the plugin's interface if already installed, null if it isn't
 		 * @return
@@ -53,6 +56,9 @@ InstallablePlugin
 	public PluginInterface
 	getAlreadyInstalledPlugin();
 	
+	public boolean
+	isAlreadyInstalled();
+	
 	public void
 	install(
 		boolean		shared )
