@@ -22,15 +22,18 @@
 
 package org.gudy.azureus2.plugins.ui;
 
+import java.io.File;
 import java.net.URL;
 
 import org.gudy.azureus2.plugins.PluginView;
+import org.gudy.azureus2.plugins.torrent.Torrent;
 import org.gudy.azureus2.plugins.ui.model.*;
 import org.gudy.azureus2.plugins.ui.SWT.SWTManager;
+import org.gudy.azureus2.plugins.ui.menus.MenuManager;
 import org.gudy.azureus2.plugins.ui.tables.TableManager;
 
 /**
- * Management tools for the User Inferface
+ * Management tools for the user interface.
  * 
  * @author parg
  */
@@ -96,7 +99,8 @@ UIManager
 	 * Creates a basic plugin view model and adds it to the plugin in one step.
 	 * view is placed inside the plugins section of the configuration page.
 	 * 
-	 * @param name
+	 * @param name The name for the view (this should be localised text, rather
+	 *     than a message string).
 	 * @return BasicPluginViewModel
 	 * @since 2.1.0.2
 	 */
@@ -137,6 +141,21 @@ UIManager
 		String		message_resource,
 		String		contents );
 	
+	/**
+	 * 
+	 * @param title_resource
+	 * @param message
+	 * @param message_map - see UIManagerEvent.MT_x
+	 * @return selected value
+	 * @since 3.0.5.3
+	 */
+	
+	public long
+	showMessageBox(
+		String					title_resource,
+		String					message_resource,
+		long					message_map );
+	
 		/**
 		 * @since 2.3.0.6
 		 * @param url
@@ -148,6 +167,16 @@ UIManager
 	
 		throws UIException;
 	
+		/**
+		 * 
+		 * @param torrent
+		 * @since 3.0.5.3
+		 */
+	
+	public void
+	openTorrent(
+		Torrent		torrent );
+	
 	/** Retrieve a class of SWT specific functions 
 	 * 
 	 * @deprecated 
@@ -170,16 +199,13 @@ UIManager
 	 */
 	public boolean showConfigSection(String sectionID);
 
-	  /* Future
-	  public MenuManager getMenuManager();
-	  In MenuManager..
-	  public Menu addMenu(String resourceKey);
-	  public Menu addMenu(String resourceKey, String parentKey);
-	  public Menu addMenu(String resourceKey, Menu parent);
-	  public MenuItem addMenuItem(String resourceKey);
-	  public MenuItem addMenuItem(String resourceKey, String parentKey);
-	  public MenuItem addMenuItem(String resourceKey, Menu parent);
-	  */
+	/**
+	 * Retrieve the menu manager.
+	 * 
+	 * @return Menu management functions
+	 * @since 3.0.0.7
+	 */
+    public MenuManager getMenuManager();
   
 		/**
 		 * UIs should support generic UI-agnostic views such as the basic config model by default. The can also
@@ -256,4 +282,72 @@ UIManager
  	public void
   	removeUIEventListener(
   		UIManagerEventListener listener );
+ 	
+ 	/**
+ 	 * Returns <tt>true</tt> if there any user interfaces currently attached to
+ 	 * Azureus.
+ 	 * 
+ 	 * Note that this value may change over time, and that at the point when a
+ 	 * plugin is being initialised, there is no guarantee over whether if the
+ 	 * user interfaces will have been created yet or not.
+ 	 * 
+ 	 * If you want to monitor what user interfaces are being attached / detached
+ 	 * from Azureus, you should add a <tt>UIListener</tt> to this object.
+ 	 * 
+ 	 * @since 3.0.0.7
+ 	 */
+ 	public boolean hasUIInstances();
+ 	
+ 	/**
+ 	 * Returns an array of all currently attached user interfaces.
+ 	 * 
+ 	 * Note that this value may change over time, and that at the point when a
+ 	 * plugin is being initialised, there is no guarantee over whether if the
+ 	 * user interfaces will have been created yet or not.
+ 	 * 
+ 	 * If you want to monitor what user interfaces are being attached / detached
+ 	 * from Azureus, you should add a <tt>UIListener</tt> to this object.
+ 	 * 
+ 	 * @since 3.0.0.7
+ 	 */
+ 	public UIInstance[] getUIInstances();
+ 	
+ 	/**
+ 	 * Retrieves a {@link UIInputReceiver} from any interface available, or
+ 	 * returns <tt>null</tt> if one is not available. This is a convenience
+ 	 * method to allow you to grab an instance without having to iterate over
+ 	 * any attached interfaces.
+ 	 * 
+ 	 * @see UIInstance#getInputReceiver()
+ 	 * @since 3.0.5.3
+ 	 */
+ 	public UIInputReceiver getInputReceiver();
+
+ 	/**
+ 	 * Retrieves a {@link UIMessage} from any interface available, or
+ 	 * returns <tt>null</tt> if one is not available. This is a convenience
+ 	 * method to allow you to grab an instance without having to iterate over
+ 	 * any attached interfaces.
+ 	 * 
+ 	 * @see UIInstance#createMessage()
+ 	 * @since 3.0.5.3
+ 	 */
+ 	public UIMessage createMessage();
+ 	
+ 	/**
+ 	 * Opens up the file using the associated application. 
+ 	 * 
+ 	 * @param file The file to open.
+ 	 * @since 3.0.5.3
+ 	 */
+ 	public void openFile(File file);
+
+ 	/**
+ 	 * Shows the file in a file explorer application in its parent folder. 
+ 	 * 
+ 	 * @param file The file to show.
+ 	 * @since 3.0.5.3
+ 	 */
+ 	public void showFile(File file);
+ 	
 }
