@@ -102,6 +102,8 @@ public interface TableCell {
   String getText();
 
   /** Change the cell's foreground color.
+   * <p>
+   * pass -1 to return color back to default
    *
    * @param red red value (0 - 255)
    * @param green green value (0 - 255)
@@ -110,6 +112,26 @@ public interface TableCell {
    *         False - Color was already set.
    */
   boolean setForeground(int red, int green, int blue);
+
+  /**
+   * Change the cell's foreground color
+   * 
+   * @param rgb int array containing red, green, and blue values, respectively.
+   *            null to return color back to default
+   * @return
+   *
+   * @since 3.0.4.3
+   */
+  boolean setForeground(int[] rgb);
+  
+  /**
+   * Change the cell's foreground color to the user's defined "error" color.
+   * 
+   * @since 3.0.3.5
+   * @return True - Color changed. <br>
+   *         False - Color was already set.
+   */
+  boolean setForegroundToErrorColor();
   
   /**
    * Get the foreground color of the cell
@@ -120,6 +142,15 @@ public interface TableCell {
    */
   int[] getForeground();
 
+	/**
+	 * Get the background color of the cell
+	 * 
+	 * @return array containing red, green, and blue color.  Might be null
+	 *
+	 * @since 3.0.3.5
+	 */
+	int[] getBackground();
+	  
   /** Sets a Comparable object that column sorting will act on.  If you never 
    * call setSortValue, your column will be sorted by the cell's text.
    *
@@ -203,6 +234,15 @@ public interface TableCell {
    * @since 2.3.0.7
    */
   public boolean isDisposed();
+  
+  /**
+   * Retrieves the number of lines available for setting text
+   * 
+   * @return # of lines available, -1 if unknown
+   *
+   * @since 3.0.1.1
+   */
+  public int getMaxLines();
 
   //////////////////////////////////
   // Start TYPE_GRAPHIC functions //
@@ -223,6 +263,13 @@ public interface TableCell {
   public int getHeight();
   
   /** Sets the image to be drawn.
+   * <p>
+   * In our SWT implementation, setting the graphic, even if it's to the same 
+   * object, will result in a repaint.  We have no idea if you modified the
+   * image itself while keeping the object the same (for example, painting
+   * on the existing image).  So, if you know that you did not change the image
+   * you should check getGraphic() first to see if you really need to set
+   * the image again.
    *
    * @param img image to be stored & drawn
    * @return true - image was changed.<br>
@@ -257,6 +304,13 @@ public interface TableCell {
    */
   public void setFillCell(boolean bFillCell);
 
+	/**
+	 * @return
+	 *
+	 * @since 3.0.5.3
+	 */
+	int getMarginHeight();
+
   /**
    * Specifies the number of pixels of vertical margin that will
    * be placed along the top and bottom edges of the layout for
@@ -268,6 +322,13 @@ public interface TableCell {
    */
   public void setMarginHeight(int height);
 
+	/**
+	 * @return
+	 *
+	 * @since 3.0.5.3
+	 */
+	int getMarginWidth();
+
   /**
    * Specifies the number of pixels of horizontal margin that will
    * be placed along the left and right edges of the layout for
@@ -337,4 +398,23 @@ public interface TableCell {
    * @param listenerObject Object implementing some cell listeneters
    */
   public void addListeners(Object listenerObject);
+
+  /**
+   * Returns a Graphic of what's behind the cell  
+   * 
+   * @return
+   *
+   * @since 3.0.3.5
+   */
+  public Graphic getBackgroundGraphic();
+
+	/**
+	 * Return the position of the mouse relative to the cell.
+	 *  
+	 * @return array of 2 containing x and y position position relative to cell. 
+	 * 				null if cell doesn't have mouse.
+	 *
+	 * @since 3.0.4.3
+	 */
+	public int[] getMouseOffset();
 }
