@@ -32,6 +32,9 @@ public interface
 ResourceDownloader 
 {
 	public static final String	PR_STRING_CONTENT_TYPE		= "ContentType";
+
+		// Properties prefixed with URL_ will be passed directly to URL connections. For example
+		// URL_User-Agent will be passed as User-Agent
 	
 		/**
 		 * Get a sensible name for the download based on its details (e.g. URL)
@@ -61,8 +64,12 @@ ResourceDownloader
 	asyncDownload();
 	
 		/**
-		 * attempts to get the size of the download. Returns -1 if the size can't be determined
-		 * @return
+		 * Attempts to get the size of the download. Returns -1 if the size can't be determined.
+		 * 
+		 * <p>
+		 * 
+		 * <b>Note:</b> You must not call this method from the <tt>reportActivity</tt> callback method. 
+		 * 
 		 * @throws ResourceDownloaderException
 		 */
 	
@@ -71,6 +78,13 @@ ResourceDownloader
 	
 		throws ResourceDownloaderException;
 
+	public void
+	setProperty(
+		String		name,
+		Object		value )
+	
+		throws ResourceDownloaderException;
+	
 	public Object
 	getProperty(
 		String		name )
