@@ -22,6 +22,8 @@
  
 package org.gudy.azureus2.plugins.utils.xml.simpleparser;
 
+import java.io.PrintWriter;
+
 public interface 
 SimpleXMLParserDocumentNode
 {
@@ -44,4 +46,11 @@ SimpleXMLParserDocumentNode
 	public SimpleXMLParserDocumentNode
 	getChild(
 		String		name );
+	
+	public void
+	print();
+	
+	public void
+	print(
+		PrintWriter		pw );
 }
