@@ -132,7 +132,7 @@ PluginInitializer
    					"CorePatcher",
 					"true" },
 	   		{	 PluginManagerDefaults.PID_PLATFORM_CHECKER, 
-   					"org.gudy.azureus2.platform.win32.PlatformManagerUpdateChecker", 
+   					"org.gudy.azureus2.platform.PlatformManagerPluginDelegate", 
    					"azplatform2", 
    					"azplatform2",
 					"true" },
@@ -146,11 +146,28 @@ PluginInitializer
 					"azextseed", 
 					"azextseed",
 	   				"true" },
-	   		{	 PluginManagerDefaults.PID_EXTERNAL_SEED, 
+	   		{	 PluginManagerDefaults.PID_LOCAL_TRACKER, 
 	   				"com.aelitis.azureus.plugins.tracker.local.LocalTrackerPlugin", 
 	   				"azlocaltracker", 
 	   				"azlocaltracker",
 					"true" },
+			{	 PluginManagerDefaults.PID_NET_STATUS, 
+		   			"com.aelitis.azureus.plugins.net.netstatus.NetStatusPlugin", 
+		   			"aznetstat", 
+		   			"aznetstat",
+					"true" },
+			{	 PluginManagerDefaults.PID_BUDDY, 
+					"com.aelitis.azureus.plugins.net.buddy.BuddyPlugin", 
+					"azbuddy", 
+					"azbuddy",
+					"true" },
+			/* disable until we can get some tracker admins to work on this
+	   		{	 PluginManagerDefaults.PID_TRACKER_PEER_AUTH, 
+					"com.aelitis.azureus.plugins.tracker.peerauth.TrackerPeerAuthPlugin", 
+					"aztrackerpeerauth", 
+					"aztrackerpeerauth",
+					"true" },
+			*/
         };
  
   
@@ -164,8 +181,6 @@ PluginInitializer
   				"safepeer", 		"SafePeer",				"1.2.4" },
   		{ "org.gudy.azureus2.countrylocator.Plugin", 
   				"CountryLocator", 	"Country Locator",		"1.0" },
-		{ "org.gudy.azureus2.ui.webplugin.remoteui.xml.server.XMLHTTPServerPlugin", 
-  				"xml_http_if",		"XML over HTTP",		"1.0" },
 		{ "org.cneclipse.bdcc.BDCCPlugin", 
   				"bdcc", 			"BitTorrent IRC Bot",	"2.1" },
 		{ "org.cneclipse.multiport.MultiPortPlugin", 
@@ -192,11 +207,16 @@ PluginInitializer
   
   private List		loaded_pi_list		= new ArrayList();
   
-  private List		plugins				= new ArrayList();
-  private List		plugin_interfaces	= new ArrayList();
+  private static boolean	loading_builtin;
   
+  private List		s_plugins				= new ArrayList();
+  private List		s_plugin_interfaces		= new ArrayList();
+  
+  private List		initThreads;
   private boolean	initialisation_complete;
   
+  private volatile boolean	plugins_initialised;
+  
   
   public static PluginInitializer
   getSingleton(
@@ -272,7 +292,85 @@ PluginInitializer
 	}  	
   }
   
- 
+  protected static boolean
+  isLoadingBuiltin()
+  {
+	  return( loading_builtin );
+  }
+  
+  public static void
+  checkJDKVersion(
+	String		name,
+	Properties	props,
+	boolean		alert_on_fail )
+  
+  	throws PluginException
+  {
+      String	required_jdk = (String)props.get( "plugin.jdk.min_version" );
+
+      if ( required_jdk != null ){
+    	  
+    	  String	actual_jdk = System.getProperty( "java.version" );
+    	  
+    	  required_jdk 	= normaliseJDK( required_jdk );
+    	  actual_jdk	= normaliseJDK( actual_jdk );
+    	  
+    	  if ( required_jdk.length() == 0 || actual_jdk.length() == 0 ){
+    		  
+    		  return;
+    	  }
+    	  
+    	  if ( Constants.compareVersions( actual_jdk, required_jdk ) < 0 ){
+    		  
+    	    	String	msg =  "Plugin " + (name.length()>0?(name+" "):"" ) + "requires Java version " + required_jdk + " or higher";
+    	      	
+    	    	if ( alert_on_fail ){
+    	      	
+    	    		Logger.log(new LogAlert(LogAlert.REPEATABLE, LogAlert.AT_ERROR, msg));
+    	    	}
+    	    	
+    	        throw( new PluginException( msg ));
+    	  }
+      }
+  }
+  
+  protected static String
+  normaliseJDK(
+	String	jdk )
+  {
+	  try{
+		  String	str = "";
+
+		  // take leading digit+. portion only
+
+		  for (int i=0;i<jdk.length();i++){
+
+			  char c = jdk.charAt( i );
+
+			  if ( c == '.' || Character.isDigit( c )){
+
+				  str += c;
+
+			  }else{
+
+				  break;
+			  }
+		  }
+
+		  	// convert 5|6|... to 1.5|1.6 etc
+
+		  if ( Integer.parseInt( "" + str.charAt(0)) > 1 ){
+			  
+			  str = "1." + str;
+		  }
+		  
+		  return( str );
+		  
+	  }catch( Throwable e ){
+		  
+		  return( "" );
+	  }
+  }
   
   protected 
   PluginInitializer(
@@ -306,14 +404,52 @@ PluginInitializer
        
     plugin_manager = PluginManagerImpl.getSingleton( this );
     
-    UpdaterUtils.checkPlugin();
+    UpdaterUtils.checkBootstrapPlugins();
+  }
+  
+  protected void
+  fireCreated(
+	PluginInterfaceImpl pi )
+  {
+	  azureus_core.triggerLifeCycleComponentCreated( pi );
+  }
+  
+  protected boolean
+  isInitialisationThread()
+  {
+	  return initThreads != null && initThreads.contains(Thread.currentThread());
   }
   
   	public List 
 	loadPlugins(
-			AzureusCore		core,
-			boolean bSkipAlreadyLoaded) 
+		AzureusCore		core,
+		boolean bSkipAlreadyLoaded) 
   	{
+  		if ( bSkipAlreadyLoaded ){
+  		
+  				// discard any failed ones
+  			
+  			List pis;
+  			
+  			synchronized( s_plugin_interfaces ){
+  				
+  				pis = new ArrayList( s_plugin_interfaces );
+  			}
+  			
+  			for (int i=0;i<pis.size();i++){
+  		  		
+  				PluginInterfaceImpl pi = (PluginInterfaceImpl)pis.get(i);
+  				
+  		  		Plugin p = pi.getPlugin();
+  		  		
+  		  		if ( p instanceof FailedPlugin ){
+  		  			
+  		  			unloadPlugin( pi );
+  		  		}
+  		  	} 
+  			
+  		}
+  		
   		List pluginLoaded = new ArrayList();
   		
   		PluginManagerImpl.setStartDetails( core );
@@ -367,6 +503,8 @@ PluginInitializer
   			if ( def.isDefaultPluginEnabled( builtin_plugins[i][0])){
     		
   				try{
+  					loading_builtin	= true;
+  					
   						// lazyness here, for builtin we use static load method with default plugin interface
   						// if we need to improve on this then we'll have to move to a system more akin to
   						// the dir-loaded plugins
@@ -390,6 +528,9 @@ PluginInitializer
 						Logger.log(new LogAlert(LogAlert.UNREPEATABLE,
 								"Load of built in plugin '" + builtin_plugins[i][2] + "' fails", e));
 					}
+  				}finally{
+  					
+  					loading_builtin = false;
   				}
   			}else{
   				if (Logger.isEnabled())
@@ -710,11 +851,20 @@ PluginInitializer
         throw( new PluginException( msg, e ));
       }
 
+      checkJDKVersion( pluginName, props, true );
+      
       plugin_class_string = (String)props.get( "plugin.class");
       
       if ( plugin_class_string == null ){
       	
       	plugin_class_string = (String)props.get( "plugin.classes");
+      	
+      	if ( plugin_class_string == null ){
+      		
+      			// set so we don't bork later will npe
+      		
+      		plugin_class_string = "";
+      	}
       }
       
       String	plugin_name_string = (String)props.get( "plugin.name");
@@ -762,8 +912,9 @@ PluginInitializer
   					existing_parent.equals( FileUtil.getUserFile( "plugins" ))){
   				
   					// skip this overridden plugin
-  				
-  				Logger.log(new LogEvent(LOGID, "Plugin '" + plugin_name_string
+  	
+  				if (Logger.isEnabled())
+  					Logger.log(new LogEvent(LOGID, "Plugin '" + plugin_name_string
 								+ "/" + plugin_class
 								+ ": shared version overridden by user-specific one"));
   				
@@ -858,7 +1009,15 @@ PluginInitializer
 	    		  
 	    	  }catch( Throwable e ){
 	      	
-	    		  load_failure	= e;
+	    		  if ( 	e instanceof ClassNotFoundException &&
+	    				props.getProperty( "plugin.install_if_missing", "no" ).equalsIgnoreCase( "yes" )){
+	    			  
+	    			  // don't report the failure
+	    			  
+	    		  }else{
+	    			  
+	    			  load_failure	= e;
+	    		  }
 	      	
 	    		  plugin = new FailedPlugin(plugin_name,directory.getAbsolutePath());
 	    	  }
@@ -909,11 +1068,17 @@ PluginInitializer
 	    	  
 	    	  if ( !pid.equals(UpdaterUpdateChecker.getPluginID())){
 	    		  
-		      	String	msg = "Error loading plugin '" + pluginName + "' / '" + plugin_class_string + "'";
-		   	 
-		      	Logger.log(new LogAlert(LogAlert.UNREPEATABLE, msg, load_failure));
+		      	String msg = "Error loading plugin '" + pluginName + "' / '" + plugin_class_string + "'";
+		      	LogAlert la;
+		      	if (load_failure instanceof UnsupportedClassVersionError) {
+		      		la = new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_ERROR, msg + ". " + MessageText.getString("plugin.install.class_version_error"));
+		      	}
+		      	else {
+		      		la = new LogAlert(LogAlert.UNREPEATABLE, msg, load_failure);
+		      	}
+		      	Logger.log(la);
 	
-		      	System.out.println( msg + " : " + load_failure);
+		      	System.out.println( msg + ": " + load_failure);
 		      }
 	      }
   		}
@@ -939,127 +1104,223 @@ PluginInitializer
  	 
     	Logger.log(new LogAlert(LogAlert.UNREPEATABLE, msg, e));
 
-    	System.out.println( msg + " : " + e);
+    	System.out.println( msg + ": " + e);
     	
     	throw( new PluginException( msg, e ));
     }
   }
   
-  public void initialisePlugins() {
-		for (int i = 0; i < loaded_pi_list.size(); i++) {
-			try {
-				List l = (List) loaded_pi_list.get(i);
-
-				if (l.size() > 0) {
-					PluginInterfaceImpl plugin_interface = (PluginInterfaceImpl) l.get(0);
-
-					if (Logger.isEnabled())
-						Logger.log(new LogEvent(LOGID, "Initializing plugin '"
-								+ plugin_interface.getPluginName() + "'"));
-
-					if (core_operation != null) {
-						core_operation.reportCurrentTask(MessageText
-								.getString("splash.plugin.init")
-								+ plugin_interface.getPluginName());
+  public void 
+  initialisePlugins() 
+  {
+	  try{
+		  if(initThreads == null)
+			  initThreads = new ArrayList();
+		  initThreads.add(Thread.currentThread());
+		  
+		  final LinkedList initQueue = new LinkedList();
+		  
+			for (int i = 0; i < loaded_pi_list.size(); i++) {
+				final int idx = i;
+				initQueue.add(new Runnable() {
+					public void run() {
+						try {
+							List l = (List) loaded_pi_list.get(idx);
+							
+							if (l.size() > 0) {
+								PluginInterfaceImpl plugin_interface = (PluginInterfaceImpl) l.get(0);
+			
+								if (Logger.isEnabled())
+									Logger.log(new LogEvent(LOGID, "Initializing plugin '"
+											+ plugin_interface.getPluginName() + "'"));
+			
+								if (core_operation != null) {
+									core_operation.reportCurrentTask(MessageText
+											.getString("splash.plugin.init")
+											+ " " + plugin_interface.getPluginName());
+								}
+			
+								initialisePlugin(l);
+								
+								if (Logger.isEnabled())
+									Logger.log(new LogEvent(LOGID, "Initialization of plugin '"
+											+ plugin_interface.getPluginName() + "' complete"));
+			
+							}
+			
+						} catch (PluginException e) {
+							// already handled
+						} finally {
+							if (core_operation != null) {
+								core_operation.reportPercent((100 * (idx + 1)) / loaded_pi_list.size());
+							}
+						}
+						
+						// some plugins try and steal the logger stdout redirects. 
+						// re-establish them if needed
+						Logger.doRedirects();
 					}
+				});
+			}
+	
 
-					initialisePlugin(l);
+	
+			// now do built in ones
+			
+			initQueue.add(new Runnable() {
+				public void run() {
+					if (Logger.isEnabled())
+						Logger.log(new LogEvent(LOGID, "Initializing built-in plugins"));
+				}
+			});
+	
+
+	
+			final PluginManagerDefaults def = PluginManager.getDefaults();
+	
+			for (int i = 0; i < builtin_plugins.length; i++) {
+				
+				final int idx = i;
+				
+				initQueue.add(new Runnable() {
+					public void run() {
+						if (def.isDefaultPluginEnabled(builtin_plugins[idx][0])) {
+							String id = builtin_plugins[idx][2];
+							String key = builtin_plugins[idx][3];
+			
+							try {
+								Class cla = root_class_loader.loadClass(
+										builtin_plugins[idx][1]);
+			
+								if (Logger.isEnabled())
+									Logger.log(new LogEvent(LOGID, "Initializing built-in plugin '"
+											+ builtin_plugins[idx][2] + "'" ));
+			
+								initializePluginFromClass(cla, id, key);
+			
+								if (Logger.isEnabled())
+								Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING,
+										"Initialization of built in plugin '" + builtin_plugins[idx][2] + "' complete"));
+							} catch (Throwable e) {
+								try {
+									// replace it with a "broken" plugin instance
+									initializePluginFromClass(FailedPlugin.class, id, key);
+			
+								} catch (Throwable f) {
+								}
+			
+								if (builtin_plugins[idx][4].equalsIgnoreCase("true")) {
+									Debug.printStackTrace(e);
+									Logger.log(new LogAlert(LogAlert.UNREPEATABLE,
+											"Initialization of built in plugin '" + builtin_plugins[idx][2]
+													+ "' fails", e));
+								}
+							}
+						} else {
+							if (Logger.isEnabled())
+								Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING,
+										"Built-in plugin '" + builtin_plugins[idx][2] + "' is disabled"));
+						}
+				
+					}
+				});
+
+			}
+	
+			initQueue.add(new Runnable() {
+				public void run() {
+					if (Logger.isEnabled())
+						Logger.log(new LogEvent(LOGID,
+								"Initializing dynamically registered plugins"));
+				}
+			});
+
+			for (int i = 0; i < registration_queue.size(); i++) {
+				
+				final int idx = i;
+				
+				initQueue.add(new Runnable() {
+					public void run() {
+						try {
+							Object entry = registration_queue.get(idx);
+			
+							if (entry instanceof Class) {
+			
+								Class cla = (Class) entry;
+			
+								singleton.initializePluginFromClass(cla, INTERNAL_PLUGIN_ID, cla
+										.getName());
+			
+							} else {
+								Object[] x = (Object[]) entry;
+			
+								Plugin plugin = (Plugin) x[0];
+			
+								singleton.initializePluginFromInstance(plugin, (String) x[1], plugin
+										.getClass().getName());
+							}
+						} catch (PluginException e) {
+						}
+					}
+				});
+			}
+			
+			AEThread2 secondaryInitializer = new AEThread2("2nd PluginInitializer Thread",true)
+			{
+				public void run() {
+					initThreads.add(Thread.currentThread());
 					
-					if (Logger.isEnabled())
-						Logger.log(new LogEvent(LOGID, "Initialization of plugin '"
-								+ plugin_interface.getPluginName() + "' complete"));
-
-				}
-
-			} catch (PluginException e) {
-				// already handled
-			} finally {
-				if (core_operation != null) {
-					core_operation.reportPercent((100 * (i + 1)) / loaded_pi_list.size());
-				}
-			}
-		}
-
-		// some plugins try and steal the logger stdout redirects. 
-		// re-establish them if needed
-		Logger.doRedirects();
-
-		// now do built in ones
-
-		if (Logger.isEnabled())
-			Logger.log(new LogEvent(LOGID, "Initializing built-in plugins"));
-
-		PluginManagerDefaults def = PluginManager.getDefaults();
-
-		for (int i = 0; i < builtin_plugins.length; i++) {
-			if (def.isDefaultPluginEnabled(builtin_plugins[i][0])) {
-				String id = builtin_plugins[i][2];
-				String key = builtin_plugins[i][3];
-
-				try {
-					Class cla = root_class_loader.loadClass(
-							builtin_plugins[i][1]);
-
-					if (Logger.isEnabled())
-						Logger.log(new LogEvent(LOGID, "Initializing built-in plugin '"
-								+ builtin_plugins[i][2] + "'" ));
-
-					initializePluginFromClass(cla, id, key);
-
-					Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING,
-							"Initialization of built in plugin '" + builtin_plugins[i][2] + "' complete"));
-				} catch (Throwable e) {
-					try {
-						// replace it with a "broken" plugin instance
-						initializePluginFromClass(FailedPlugin.class, id, key);
-
-					} catch (Throwable f) {
-					}
-
-					if (builtin_plugins[i][4].equalsIgnoreCase("true")) {
-						Debug.printStackTrace(e);
-						Logger.log(new LogAlert(LogAlert.UNREPEATABLE,
-								"Initialization of built in plugin '" + builtin_plugins[i][2]
-										+ "' fails", e));
+					while(true)
+					{
+						Runnable toRun;
+						synchronized (initQueue)
+						{
+							if(initQueue.isEmpty())
+								break;
+							toRun = (Runnable)initQueue.remove(0);
+						}
+						toRun.run();
 					}
+					
+					initThreads.remove(Thread.currentThread());
 				}
-			} else {
-				if (Logger.isEnabled())
-					Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING,
-							"Built-in plugin '" + builtin_plugins[i][2] + "' is disabled"));
-			}
-		}
-
-		if (Logger.isEnabled())
-			Logger.log(new LogEvent(LOGID,
-					"Initializing dynamically registered plugins"));
-
-		for (int i = 0; i < registration_queue.size(); i++) {
-			try {
-				Object entry = registration_queue.get(i);
-
-				if (entry instanceof Class) {
-
-					Class cla = (Class) entry;
-
-					singleton.initializePluginFromClass(cla, INTERNAL_PLUGIN_ID, cla
-							.getName());
-
-				} else {
-					Object[] x = (Object[]) entry;
-
-					Plugin plugin = (Plugin) x[0];
-
-					singleton.initializePluginFromInstance(plugin, (String) x[1], plugin
-							.getClass().getName());
+			};
+			secondaryInitializer.start();
+			
+			while(true)
+			{
+				Runnable toRun;
+				synchronized (initQueue)
+				{
+					if(initQueue.isEmpty())
+						break;
+					toRun = (Runnable)initQueue.remove(0);
 				}
-			} catch (PluginException e) {
+				toRun.run();
 			}
-		}
-
-		registration_queue.clear();
+			
+			secondaryInitializer.join();
+			
+			registration_queue.clear();
+			
+			plugins_initialised = true;
+			
+			fireEvent( PluginEvent.PEV_ALL_PLUGINS_INITIALISED );
+	  }finally{
+		  
+		  initThreads = null;
+	  }
 	}
   
+  	protected void
+  	checkPluginsInitialised()
+  	{
+  		if ( !plugins_initialised ){
+  			
+  			Debug.out( "Wait until plugin initialisation is complete until doing this!" );
+  		}
+  	}
+  
   	private void
 	initialisePlugin(
 		List	l )
@@ -1073,7 +1334,12 @@ PluginInitializer
   			PluginInterfaceImpl	plugin_interface = (PluginInterfaceImpl)l.get(i);
   			
   			if (plugin_interface.isDisabled()) {
-  				plugin_interfaces.add( plugin_interface );
+  				
+  				synchronized( s_plugin_interfaces ){
+  				
+  					s_plugin_interfaces.add( plugin_interface );
+  				}
+  				
   				continue;
   			}
   	
@@ -1088,8 +1354,10 @@ PluginInitializer
       	
 				UtilitiesImpl.setPluginThreadContext( plugin_interface );
 				
+				fireCreated( plugin_interface );
+				
   				plugin.initialize(plugin_interface);
-      	
+      	  				
   				if (!(plugin instanceof FailedPlugin)){
   					
   					plugin_interface.setOperational( true );
@@ -1099,10 +1367,13 @@ PluginInitializer
   				load_failure	= e;
   			}
      
-  			plugins.add( plugin );
-	      
-  			plugin_interfaces.add( plugin_interface );
+  			synchronized( s_plugin_interfaces ){
+  				
+  				s_plugins.add( plugin );
 	      
+  				s_plugin_interfaces.add( plugin_interface );
+  			}
+  			
   			if ( load_failure != null ){
 	      	
   				Debug.printStackTrace( load_failure );
@@ -1141,34 +1412,35 @@ PluginInitializer
   		
   		return;
   	}
-  	
-    if( core_operation != null ){
-	      	
-    	String	plugin_name;
-		
-		if ( plugin_config_key.length() == 0 ){
-			
-			plugin_name = plugin_class.getName();
-    	
-	    	int	pos = plugin_name.lastIndexOf(".");
-	    	
-	    	if ( pos != -1 ){
-	    		
-	    		plugin_name = plugin_name.substring( pos+1 );
-	    		
-	    	}
-		}else{
-		
-			plugin_name = plugin_config_key;
-		}
-    	
-		core_operation.reportCurrentTask(MessageText.getString("splash.plugin.init") + plugin_name );
-    }
-    
+  	    
   	try{
   	
   		Plugin plugin = (Plugin) plugin_class.newInstance();
   		
+  		String	plugin_name;
+
+  		if ( plugin_config_key.length() == 0 ){
+
+  			plugin_name = plugin_class.getName();
+
+  			int	pos = plugin_name.lastIndexOf(".");
+
+  			if ( pos != -1 ){
+
+  				plugin_name = plugin_name.substring( pos+1 );
+
+  			}
+  		}else{
+
+  			plugin_name = plugin_config_key;
+  		}
+
+  		Properties properties = new Properties();
+  		
+  			// default plugin name
+  		
+  		properties.put( "plugin.name", plugin_name );
+  		
   		PluginInterfaceImpl plugin_interface = 
   			new PluginInterfaceImpl(
   						plugin, 
@@ -1176,10 +1448,17 @@ PluginInitializer
 						plugin_class,
 						plugin_class.getClassLoader(),
 						plugin_config_key,
-						new Properties(),
+						properties,
 						"",
 						plugin_id,
 						null );
+
+	      // Must use getPluginID() instead of pid, because they may differ
+	      boolean bEnabled = COConfigurationManager
+							.getBooleanParameter("PluginInfo."
+									+ plugin_interface.getPluginID() + ".enabled", true);
+	      
+	      plugin_interface.setDisabled(!bEnabled);
   		
 		UtilitiesImpl.setPluginThreadContext( plugin_interface );
 
@@ -1199,17 +1478,29 @@ PluginInitializer
 						"Load of built in plugin '" + plugin_id + "' fails", e));
 		}
 		 
-  		plugin.initialize(plugin_interface);
+		 if (bEnabled) {
+			 
+			 if ( core_operation != null ){
+				 
+				 core_operation.reportCurrentTask(MessageText.getString("splash.plugin.init") + " " + plugin_interface.getPluginName());
+			 }
+	
+			 fireCreated( plugin_interface );
+			 
+	  		plugin.initialize(plugin_interface);
+	  		
+	  		if (!(plugin instanceof FailedPlugin)){
+	  			
+	  			plugin_interface.setOperational( true );
+	  		}
+		 }
   		
-  		if (!(plugin instanceof FailedPlugin)){
-  			
-  			plugin_interface.setOperational( true );
-  		}
-  		
-   		plugins.add( plugin );
+		 synchronized( s_plugin_interfaces ){
    		
-   		plugin_interfaces.add( plugin_interface );
+			s_plugins.add( plugin );
    		
+			s_plugin_interfaces.add( plugin_interface );
+		 }
   	}catch(Throwable e){
   		
   		Debug.printStackTrace( e );
@@ -1247,6 +1538,8 @@ PluginInitializer
   		
 		UtilitiesImpl.setPluginThreadContext( plugin_interface );
 
+		fireCreated( plugin_interface );
+		
   		plugin.initialize(plugin_interface);
   		
   		if (!(plugin instanceof FailedPlugin)){
@@ -1254,10 +1547,12 @@ PluginInitializer
   			plugin_interface.setOperational( true );
   		}
   		
-   		plugins.add( plugin );
+  		synchronized( s_plugin_interfaces ){
    		
-   		plugin_interfaces.add( plugin_interface );
+  			s_plugins.add( plugin );
    		
+  			s_plugin_interfaces.add( plugin_interface );
+  		}
   	}catch(Throwable e){
   		
   		Debug.printStackTrace( e );
@@ -1276,9 +1571,14 @@ PluginInitializer
   unloadPlugin(
   	PluginInterfaceImpl		pi )
   {
-  	plugins.remove( pi.getPlugin());
+	synchronized( s_plugin_interfaces ){
+	
+		s_plugins.remove( pi.getPlugin());
   	
-  	plugin_interfaces.remove( pi );
+		s_plugin_interfaces.remove( pi );
+	}
+	
+  	pi.unloadSupport();
   	
   	for (int i=0;i<loaded_pi_list.size();i++){
   		
@@ -1381,33 +1681,47 @@ PluginInitializer
   public void
   destroyInitiated()
   {	
-  	for (int i=0;i<plugin_interfaces.size();i++){
-  		
-  		((PluginInterfaceImpl)plugin_interfaces.get(i)).closedownInitiated();
-  	} 
-  	
-  	if ( default_plugin != null ){
-  		
-  		default_plugin.closedownInitiated();
-  	}
+	  List plugin_interfaces;
+
+	  synchronized( s_plugin_interfaces ){
+
+		  plugin_interfaces = new ArrayList( s_plugin_interfaces );
+	  }
+		
+	  for (int i=0;i<plugin_interfaces.size();i++){
+
+		  ((PluginInterfaceImpl)plugin_interfaces.get(i)).closedownInitiated();
+	  } 
+
+	  if ( default_plugin != null ){
+
+		  default_plugin.closedownInitiated();
+	  }
   }
   
   public void
   destroyed()
   {
-  	for (int i=0;i<plugin_interfaces.size();i++){
-  		
-  		((PluginInterfaceImpl)plugin_interfaces.get(i)).closedownComplete();
-  	}  
-  	
- 	if ( default_plugin != null ){
-  		
-  		default_plugin.closedownComplete();
-  	}
+	  List plugin_interfaces;
+
+	  synchronized( s_plugin_interfaces ){
+
+		  plugin_interfaces = new ArrayList( s_plugin_interfaces );
+	  }
+
+	  for (int i=0;i<plugin_interfaces.size();i++){
+
+		  ((PluginInterfaceImpl)plugin_interfaces.get(i)).closedownComplete();
+	  }  
+
+	  if ( default_plugin != null ){
+
+		  default_plugin.closedownComplete();
+	  }
   }
   
   
-  public void seedingStatusChanged( boolean seeding_only_mode ){
+  public void seedingStatusChanged( boolean seeding_only_mode, boolean b ){
     /*nothing*/
   }
   
@@ -1433,6 +1747,13 @@ PluginInitializer
   			}
   		};
   	
+  	List plugin_interfaces;
+  		  	
+  	synchronized( s_plugin_interfaces ){
+  			
+  		plugin_interfaces = new ArrayList( s_plugin_interfaces );
+  	}
+  	
   	for (int i=0;i<plugin_interfaces.size();i++){
   		
   		try{
@@ -1472,6 +1793,13 @@ PluginInitializer
   	
   	UIManagerImpl.initialisationComplete();
   	
+	List plugin_interfaces;
+	
+	synchronized( s_plugin_interfaces ){
+		
+		plugin_interfaces = new ArrayList( s_plugin_interfaces );
+	}
+	
   	for (int i=0;i<plugin_interfaces.size();i++){
   		
   		((PluginInterfaceImpl)plugin_interfaces.get(i)).initialisationComplete();
@@ -1495,10 +1823,16 @@ PluginInitializer
   }
 
   protected List getPluginInterfacesSupport() {
-  	return plugin_interfaces;
+	  
+	checkPluginsInitialised();
+		
+	synchronized( s_plugin_interfaces ){
+		
+		return( new ArrayList( s_plugin_interfaces ));
+	}
   }
   
-  protected PluginInterface[]
+  public PluginInterface[]
   getPlugins()
   {
   	List	pis = getPluginInterfacesSupport();
@@ -1526,7 +1860,14 @@ PluginInitializer
   protected PluginInterfaceImpl
   getPluginFromClass(
   	String	class_name )
-  {  	
+  {  
+	List plugin_interfaces;
+	
+	synchronized( s_plugin_interfaces ){
+		
+		plugin_interfaces = new ArrayList( s_plugin_interfaces );
+	}
+	
   	for (int i=0;i<plugin_interfaces.size();i++){
   		
   		PluginInterfaceImpl	pi = (PluginInterfaceImpl)plugin_interfaces.get(i);
@@ -1567,6 +1908,13 @@ PluginInitializer
 		try{
 			writer.indent();
 
+			List plugin_interfaces;
+			
+			synchronized( s_plugin_interfaces ){
+				
+				plugin_interfaces = new ArrayList( s_plugin_interfaces );
+			}
+			
 		 	for (int i=0;i<plugin_interfaces.size();i++){
 		  		
 		  		PluginInterfaceImpl	pi = (PluginInterfaceImpl)plugin_interfaces.get(i);
