@@ -23,13 +23,24 @@ package org.gudy.azureus2.pluginsimpl.local;
 
 import java.util.*;
 import java.io.File;
+import java.net.URL; 
 
+import org.gudy.azureus2.platform.PlatformManagerFactory;
 import org.gudy.azureus2.plugins.*;
+import org.gudy.azureus2.plugins.dht.mainline.*;
 import org.gudy.azureus2.plugins.logging.Logger;
+import org.gudy.azureus2.plugins.messaging.MessageManager;
+import org.gudy.azureus2.plugins.network.ConnectionManager;
+import org.gudy.azureus2.pluginsimpl.local.dht.mainline.*;
+import org.gudy.azureus2.pluginsimpl.local.clientid.ClientIDManagerImpl;
+import org.gudy.azureus2.pluginsimpl.local.ddb.DDBaseImpl;
 import org.gudy.azureus2.pluginsimpl.local.download.DownloadManagerImpl;
 import org.gudy.azureus2.pluginsimpl.local.installer.PluginInstallerImpl;
+import org.gudy.azureus2.pluginsimpl.local.ipc.IPCInterfaceImpl;
 import org.gudy.azureus2.pluginsimpl.local.ipfilter.IPFilterImpl;
 import org.gudy.azureus2.pluginsimpl.local.logging.LoggerImpl;
+import org.gudy.azureus2.pluginsimpl.local.messaging.MessageManagerImpl;
+import org.gudy.azureus2.pluginsimpl.local.network.ConnectionManagerImpl;
 import org.gudy.azureus2.pluginsimpl.local.peers.protocol.*;
 import org.gudy.azureus2.pluginsimpl.local.sharing.ShareManagerImpl;
 import org.gudy.azureus2.pluginsimpl.local.torrent.TorrentManagerImpl;
@@ -40,6 +51,7 @@ import org.gudy.azureus2.pluginsimpl.local.ui.config.ParameterRepository;
 import org.gudy.azureus2.pluginsimpl.local.ui.config.PluginConfigUIFactoryImpl;
 import org.gudy.azureus2.pluginsimpl.local.utils.*;
 import org.gudy.azureus2.pluginsimpl.local.update.*;
+import org.gudy.azureus2.plugins.ipc.IPCInterface;
 import org.gudy.azureus2.plugins.ipfilter.IPFilter;
 import org.gudy.azureus2.plugins.tracker.Tracker;
 import org.gudy.azureus2.plugins.ui.config.Parameter;
@@ -47,7 +59,10 @@ import org.gudy.azureus2.plugins.ui.config.PluginConfigUIFactory;
 import org.gudy.azureus2.plugins.ui.tables.peers.PluginPeerItemFactory;
 import org.gudy.azureus2.plugins.ui.tables.mytorrents.PluginMyTorrentsItemFactory;
 import org.gudy.azureus2.plugins.peers.protocol.*;
+import org.gudy.azureus2.plugins.platform.PlatformManager;
 import org.gudy.azureus2.plugins.sharing.*;
+import org.gudy.azureus2.plugins.clientid.ClientIDManager;
+import org.gudy.azureus2.plugins.ddb.DistributedDatabase;
 import org.gudy.azureus2.plugins.download.*;
 import org.gudy.azureus2.plugins.torrent.*;
 import org.gudy.azureus2.plugins.ui.*;
@@ -55,37 +70,57 @@ import org.gudy.azureus2.plugins.ui.config.ConfigSection;
 import org.gudy.azureus2.plugins.utils.*;
 import org.gudy.azureus2.plugins.update.*;
 
-import org.gudy.azureus2.ui.swt.FileDownloadWindow;
-import org.gudy.azureus2.ui.swt.mainwindow.MainWindow;
-import org.gudy.azureus2.ui.swt.mainwindow.TorrentOpener;
-import org.gudy.azureus2.ui.swt.views.table.utils.TableColumnManager;
-
 import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.core3.logging.*;
 
+import com.aelitis.azureus.core.AzureusCoreComponent;
+import com.aelitis.azureus.core.util.CopyOnWriteList;
+
+
+
 /**
  * @author Olivier
  *
  */
 public class 
 PluginInterfaceImpl 
-	implements PluginInterface 
+	implements PluginInterface, AzureusCoreComponent
 {
-  protected Plugin				plugin;
-  protected PluginInitializer	initialiser;
-  protected Object				initialiser_key;
-  protected ClassLoader			class_loader;
-  protected List				listeners 		= new ArrayList();
-  protected List				event_listeners	= new ArrayList();
-  protected String 				pluginConfigKey;
-  protected Properties 			props;
-  protected String 				pluginDir;
-  protected PluginConfig 		config;
-  protected String				plugin_id;
-  protected String				plugin_version;
-  protected boolean				operational;
-  protected Logger				logger;
-   
+	private static final LogIDs LOGID = org.gudy.azureus2.core3.logging.LogIDs.PLUGIN;
+
+  private Plugin				plugin;
+  private PluginInitializer		initialiser;
+  private Object				initialiser_key;
+  private ClassLoader			class_loader;
+  private CopyOnWriteList		listeners 		= new CopyOnWriteList();
+  private CopyOnWriteList		event_listeners	= new CopyOnWriteList();
+  private String				key;
+  private String 				pluginConfigKey;
+  private Properties 			props;
+  private String 				pluginDir;
+  private PluginConfigImpl		config;
+  private String				plugin_version;
+  private boolean				operational;
+  private boolean				disabled;
+  private Logger				logger;
+  private IPCInterfaceImpl		ipc_interface;
+  private List					children		= new ArrayList();
+  private List configSections = new ArrayList();
+  
+  /**
+   * This is the plugin ID value we were given when we were created.
+   * 
+   * We might use it, but it depends what value is the plugins properties
+   * (which will override this value).
+   */ 
+  private String				given_plugin_id;
+  
+  /**
+   * We store this value as soon as someone calls getPluginID(), meaning
+   * we will return a consistent value for the plugin's lifetime.
+   */
+  private String                plugin_id_to_use;
+  
   public 
   PluginInterfaceImpl(
   		Plugin				_plugin,
@@ -102,12 +137,14 @@ PluginInterfaceImpl
   	initialiser			= _initialiser;
   	initialiser_key		= _initialiser_key;
   	class_loader		= _class_loader;
+  	key					= _key;
   	pluginConfigKey 	= "Plugin." + _key;
     props 				= new propertyWrapper(_props );
     pluginDir 			= _pluginDir;
-    config 				= new PluginConfigImpl(pluginConfigKey);
-    plugin_id			= _plugin_id;
+    config 				= new PluginConfigImpl(this,pluginConfigKey);
+    given_plugin_id     = _plugin_id;
     plugin_version		= _plugin_version;
+    ipc_interface		= new IPCInterfaceImpl( initialiser, plugin );
   }
   
   	public Plugin
@@ -141,6 +178,10 @@ PluginInterfaceImpl
   		return( initialiser.getPluginManager());
   	}
   	
+  	public String getApplicationName() {
+  		return Constants.APP_NAME;
+  	}
+  	
 	public String
 	getAzureusName()
 	{
@@ -154,6 +195,9 @@ PluginInterfaceImpl
 	}
 	
 
+	/**
+	 * @deprecated
+	 */
   public void addView(PluginView view)
   {
     getUIManager().getSWTManager().addView(view);
@@ -162,20 +206,39 @@ PluginInterfaceImpl
   public void addConfigSection(ConfigSection section)
   {
   	ConfigSectionRepository.getInstance().addConfigSection(section);
+  	configSections.add(section);
   }
 
+  public void removeConfigSection(ConfigSection section)
+  {
+  	ConfigSectionRepository.getInstance().removeConfigSection(section);
+  	configSections.remove(section);
+  }
+  
+  public ConfigSection[] getConfigSections() {
+  	return (ConfigSection[]) configSections.toArray(new ConfigSection[0]);
+  }
+  
   /**
    * @deprecated
    */
   public void openTorrentFile(String fileName) {
-    TorrentOpener.openTorrent(initialiser.getAzureusCore(),fileName);
+	  try{
+		  getDownloadManager().addDownload( new File(fileName));
+	  }catch( DownloadException e ){
+		  throw( new RuntimeException(e));
+	  }
   }
 
   /**
    * @deprecated
    */
   public void openTorrentURL(String url) {
-    new FileDownloadWindow(initialiser.getAzureusCore(),MainWindow.getWindow().getDisplay(),url, null);
+	  try{
+		  getDownloadManager().addDownload( new URL( url ));
+	  }catch( Throwable e ){
+		  throw( new RuntimeException(e));
+	  } 
   }
       
   public void
@@ -236,14 +299,26 @@ PluginInterfaceImpl
   public String
   getPluginID()
   {
-  	String	id = (String)props.get("plugin.id");
+	  String id = (String)props.get("plugin.id");
+	  
+	  // hack alert - azupdater needs to change its plugin id due to general hackage
+	  
+	  if ( id != null && id.equals( "azupdater" )){
+		  
+		  plugin_id_to_use = id;
+	  }
+	  
+	  if (plugin_id_to_use != null) {return plugin_id_to_use;}
+	  
+	// Calculate what plugin ID value to use - look at the properties file
+	// first, and if that isn't correct, base it on the given plugin ID
+	// value we were given.
   	
-  	if ( id == null ){
-  		
-  		id = plugin_id;
-  	}
+  	if (id == null) {id = given_plugin_id;}
+  	if (id == null) {id = "<none>";}
   	
-  	return( id==null?"<none>":id );
+  	plugin_id_to_use = id;
+  	return plugin_id_to_use;
   }
 
   public boolean
@@ -257,9 +332,29 @@ PluginInterfaceImpl
   public boolean
   isBuiltIn()
   {
-  	return( getPluginDirectoryName().length() == 0 );
+	  String	dir = getPluginDirectoryName();
+	  
+	  if ( dir == null ){
+		  
+		  return( PluginInitializer.isLoadingBuiltin());
+	  }
+	  
+  		return( dir.length() == 0 || getPluginID().equals( "azupdater" ));
   }
   
+	public void
+	setDisabled(
+		boolean	_disabled )
+	{
+		disabled	= _disabled;
+	}
+	  
+	public boolean
+	isDisabled()
+	{
+		return( disabled );
+	}
+	
   public Properties getPluginProperties() 
   {
     return(props);
@@ -300,13 +395,13 @@ PluginInterfaceImpl
   
   /** @deprecated Use getUIManager().getTableManager().createColumn */
   public void addColumnToPeersTable(String columnName, PluginPeerItemFactory item) {
-    TableColumnManager.getInstance().addExtension(columnName,item);
+    Debug.out( "Method PluginInterface::addColumnToPeersTable deprecated. Use getUIManager().getTableManager().createColumn" );
   }
   
   /** @deprecated Use getUIManager().getTableManager().createColumn */
   public void addColumnToMyTorrentsTable(String columnName, PluginMyTorrentsItemFactory item) {
-    TableColumnManager.getInstance().addExtension(columnName,item);
-  }
+	   Debug.out( "Method PluginInterface::addColumnToMyTorrentsTable deprecated. Use getUIManager().getTableManager().createColumn" );
+ }
 
   public Tracker getTracker() {
   	return( TrackerImpl.getSingleton());
@@ -326,17 +421,21 @@ PluginInterfaceImpl
   	return( DownloadManagerImpl.getSingleton(initialiser.getAzureusCore()));
   }
   
+  public MainlineDHTManager getMainlineDHTManager() {
+	  return new MainlineDHTManagerImpl(initialiser.getAzureusCore());
+  }
+  
   public TorrentManager
   getTorrentManager()
   {
-  	return( TorrentManagerImpl.getSingleton());
+  	return( TorrentManagerImpl.getSingleton().specialise( this ));
   }
   
   public Logger getLogger() 
   {
   	if ( logger == null ){
   		
-  		logger = new LoggerImpl();
+  		logger = new LoggerImpl( this );
   	}
   	
   	return( logger );
@@ -351,7 +450,7 @@ PluginInterfaceImpl
   public Utilities
   getUtilities()
   {
-  	return( new UtilitiesImpl( this ));
+  	return( new UtilitiesImpl( initialiser.getAzureusCore(), this ));
   }
   
   public ShortCuts
@@ -383,7 +482,16 @@ PluginInterfaceImpl
   {
   	String dir = getPluginDirectoryName();
   	
-  		// if not dir based then just test this one
+  		// mechanism to override unloadability 
+  	
+   	boolean	disable_unload = getPluginProperties().getProperty( "plugin.unload.disabled", "" ).equalsIgnoreCase( "true" );
+  	
+  	if ( disable_unload ){
+  		
+  		return( false );
+  	}
+  	
+		// if not dir based then just test this one
   	
   	if ( dir == null || dir.length() == 0 ){
   		
@@ -412,6 +520,14 @@ PluginInterfaceImpl
   		}
   	}
   	
+  	for (int i=0;i<children.size();i++){
+  		
+  		if ( !((PluginInterface)children.get(i)).isUnloadable()){
+  			
+  			return( false );
+  		}
+  	}
+  	
   	return( true );
   }
   
@@ -461,6 +577,21 @@ PluginInterfaceImpl
 	  		}
 		}
   	}
+  	
+  	for (int i=0;i<children.size();i++){
+  		
+  		((PluginInterface)children.get(i)).unload();
+  	}
+  	
+  	setOperational(false);
+
+  	class_loader = null;
+  }
+  
+  protected void
+  unloadSupport()
+  {
+	  ipc_interface.unload();
   }
   
   public void
@@ -468,8 +599,14 @@ PluginInterfaceImpl
   
   	throws PluginException
   {
-  	unload();
-  		
+	  	// we use the "reload" method to load disabled plugins regardless of whether they are
+	  	// unloadable. If currently disabled then no unloading to do anyway
+	  
+	if ( isUnloadable() || isOperational()){
+		
+	  unload();
+	}
+	  
   	initialiser.reloadPlugin( this );
   }
   
@@ -481,65 +618,127 @@ PluginInterfaceImpl
 		PluginInstallerImpl.getSingleton(getPluginManager()).uninstall( this );
 	}
 	
+	public boolean
+	isInitialisationThread()
+	{
+		return( initialiser.isInitialisationThread());
+	}
+	
+	 public ClientIDManager
+	 getClientIDManager()
+	 {
+	 	return( ClientIDManagerImpl.getSingleton());
+	 }
+	 
+   
+   public ConnectionManager getConnectionManager() {
+     return ConnectionManagerImpl.getSingleton( initialiser.getAzureusCore());
+   }
+   
+   public MessageManager getMessageManager() {
+     return MessageManagerImpl.getSingleton( initialiser.getAzureusCore() );
+   }
+   
+   
+   public DistributedDatabase
+   getDistributedDatabase()
+   {
+   	return( DDBaseImpl.getSingleton(initialiser.getAzureusCore()));
+   }
+   
+   public PlatformManager
+   getPlatformManager()
+   {
+	   return( PlatformManagerFactory.getPlatformManager());
+   }
+   
   protected void
   initialisationComplete()
   {
-  	for (int i=0;i<listeners.size();i++){
-  		
-  		try{
-  			((PluginListener)listeners.get(i)).initializationComplete();
-  			
-  		}catch( Throwable e ){
-  			
-  			Debug.printStackTrace( e );
-  		}
-  	}
+	  Iterator it = listeners.iterator();
+	  
+	  while( it.hasNext()){
+
+		  try{
+			  ((PluginListener)it.next()).initializationComplete();
+
+		  }catch( Throwable e ){
+
+			  Debug.printStackTrace( e );
+		  }
+	  }
+
+	  for (int i=0;i<children.size();i++){
+
+		  ((PluginInterfaceImpl)children.get(i)).initialisationComplete();
+	  }
   }
   
   protected void
   closedownInitiated()
   {
-  	for (int i=0;i<listeners.size();i++){
-  		
-  		try{
-  			((PluginListener)listeners.get(i)).closedownInitiated();
-  			
-  		}catch( Throwable e ){
-  			
-  			Debug.printStackTrace( e );
-  		}
-  	}
+	  Iterator it = listeners.iterator();
+	  
+	  while( it.hasNext()){
+
+		  try{
+			  ((PluginListener)it.next()).closedownInitiated();
+
+		  }catch( Throwable e ){
+
+			  Debug.printStackTrace( e );
+		  }
+	  }
+
+	  for (int i=0;i<children.size();i++){
+
+		  ((PluginInterfaceImpl)children.get(i)).closedownInitiated();
+	  }
   }
   
   protected void
   closedownComplete()
   {
-  	for (int i=0;i<listeners.size();i++){
-  		
-  		try{
-  			((PluginListener)listeners.get(i)).closedownComplete();
-  			
-  		}catch( Throwable e ){
-  			
-  			Debug.printStackTrace( e );
-  		}
-  	}
+	  Iterator it = listeners.iterator();
+	  
+	  while( it.hasNext()){
+
+		  try{
+			  ((PluginListener)it.next()).closedownComplete();
+
+		  }catch( Throwable e ){
+
+			  Debug.printStackTrace( e );
+		  }
+	  }
+
+	  for (int i=0;i<children.size();i++){
+
+		  ((PluginInterfaceImpl)children.get(i)).closedownComplete();
+	  }
   }
   
-  protected void
-  fireEvent(
-  	PluginEvent		event )
+  public void
+  firePluginEvent(
+	PluginEvent		event )
   {
-  	for (int i=0;i<event_listeners.size();i++){
-  		
-  		try{
-  			((PluginEventListener)event_listeners.get(i)).handleEvent( event );
-  			
-  		}catch( Throwable e ){
-  			
-  			Debug.printStackTrace( e );
-  		}
-  	} 	
+	  Iterator it = event_listeners.iterator();
+	  
+	  while( it.hasNext()){
+
+		  try{
+			  ((PluginEventListener)it.next()).handleEvent( event );
+
+		  }catch( Throwable e ){
+
+			  Debug.printStackTrace( e );
+		  }
+	  } 
+
+	  for (int i=0;i<children.size();i++){
+
+		  ((PluginInterfaceImpl)children.get(i)).firePluginEvent(event);
+	  }
   }
   
   public ClassLoader
@@ -548,6 +747,69 @@ PluginInterfaceImpl
   	return( class_loader );
   }
   
+	public PluginInterface
+	getLocalPluginInterface(
+		Class		plugin_class,
+		String		id )
+	
+		throws PluginException
+	{
+		try{
+			Plugin	p = (Plugin)plugin_class.newInstance();
+			
+			// Discard plugin.id from the properties, we want the
+			// plugin ID we create to take priority - not a value
+			// from the original plugin ID properties file.
+			Properties local_props = new Properties(props);
+			local_props.remove("plugin.id");
+	
+			PluginInterfaceImpl pi =
+				new PluginInterfaceImpl(
+			  		p,
+			  		initialiser,
+					initialiser_key,
+					class_loader,
+					key + "." + id,
+					local_props,
+					pluginDir,
+					getPluginID() + "." + id,
+					plugin_version ); 
+			
+			initialiser.fireCreated( pi );
+			
+			p.initialize( pi );
+			
+			children.add( pi );
+			
+			return( pi );
+			
+		}catch( Throwable e ){
+			
+			if ( e instanceof PluginException ){
+				
+				throw((PluginException)e);
+			}
+			
+			throw( new PluginException( "Local initialisation fails", e ));
+		}
+	}
+	
+	 public IPCInterface 
+	 getIPC() 
+	 {
+		 return( ipc_interface );
+	 }
+	 
+	public boolean
+	isShared()
+	{
+		String shared_dir 	= FileUtil.getApplicationFile( "plugins" ).toString(); 
+		   
+		String	plugin_dir = getPluginDirectoryName();
+				
+		return( plugin_dir.startsWith( shared_dir ));
+	}
+	
   public void
   addListener(
   	PluginListener	l )
@@ -581,6 +843,47 @@ PluginInterfaceImpl
   	event_listeners.remove(l);
   }
   
+	protected void
+	generateEvidence(
+		IndentWriter		writer )
+	{
+		writer.println( getPluginName());
+
+		try{
+			writer.indent();
+			
+			writer.println( "id:" + getPluginID() + ",version:" + getPluginVersion());
+			
+			String user_dir 	= FileUtil.getUserFile( "plugins" ).toString(); 
+			String shared_dir 	= FileUtil.getApplicationFile( "plugins" ).toString(); 
+			   
+			String	plugin_dir = getPluginDirectoryName();
+			
+			String	type;
+			
+			if ( plugin_dir.startsWith( shared_dir )){
+				
+				type = "shared";
+			
+			}else	if ( plugin_dir.startsWith( user_dir )){
+					
+				type = "per-user";	
+
+			}else{
+				
+				type = "built-in";
+			}
+			
+			writer.println( "type:" + type + ",enabled:" + !isDisabled() + ",operational:" + isOperational());
+			
+		}finally{
+			
+			writer.exdent();
+		}
+	}
+  
+  
+  
   	// unfortunately we need to protect ourselves against the plugin itself trying to set
   	// plugin.version and plugin.id as this screws things up if they get it "wrong".
   	// They should be setting these things in the plugin.properties file
@@ -615,11 +918,15 @@ PluginInterfaceImpl
 	{
   			// if its us then we probably know what we're doing :P
   		
-  		if ( !plugin.getClass().getName().startsWith( "org.gudy")){
+  		if ( ! ( plugin.getClass().getName().startsWith( "org.gudy") || plugin.getClass().getName().startsWith( "com.aelitis."))){
   			
 	  		if ( str.equalsIgnoreCase( "plugin.id" ) || str.equalsIgnoreCase("plugin.version" )){
 	  		 			
-	  			LGLogger.log(LGLogger.AT_COMMENT, "Plugin '" + getPluginName() + "' tried to set property '" + str + "' - action ignored" );
+	  			if (org.gudy.azureus2.core3.logging.Logger.isEnabled())
+						org.gudy.azureus2.core3.logging.Logger
+								.log(new LogEvent(LOGID, LogEvent.LT_WARNING, "Plugin '"
+										+ getPluginName() + "' tried to set property '" + str
+										+ "' - action ignored"));
 	  			
 	  			return( null );
 	  		}
@@ -635,7 +942,7 @@ PluginInterfaceImpl
 	{
 			// if its us then we probably know what we're doing :P
   		
-  		if ( !plugin.getClass().getName().startsWith( "org.gudy")){
+  		if ( ! ( plugin.getClass().getName().startsWith( "org.gudy") || plugin.getClass().getName().startsWith( "com.aelitis."))){
   			
 	 		if ((!initialising ) && key instanceof String ){
 	  			
@@ -643,7 +950,11 @@ PluginInterfaceImpl
 	  			
 	  	 		if ( k_str.equalsIgnoreCase( "plugin.id" ) || k_str.equalsIgnoreCase("plugin.version" )){
 	  	 	  		
-	  	 			LGLogger.log(LGLogger.AT_COMMENT, "Plugin '" + getPluginName() + "' tried to set property '" + k_str + "' - action ignored" );
+	  	 			if (org.gudy.azureus2.core3.logging.Logger.isEnabled())
+							org.gudy.azureus2.core3.logging.Logger.log(new LogEvent(LOGID,
+									LogEvent.LT_WARNING, "Plugin '" + getPluginName()
+											+ "' tried to set property '" + k_str
+											+ "' - action ignored"));
 	  	 		 
 	  	 			return( null );
 	  	 	  	}
