@@ -36,9 +36,11 @@ import org.gudy.azureus2.plugins.ddb.*;
 
 
 import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.core.util.CopyOnWriteList;
 import com.aelitis.azureus.plugins.dht.DHTPlugin;
 import com.aelitis.azureus.plugins.dht.DHTPluginContact;
 import com.aelitis.azureus.plugins.dht.DHTPluginKeyStats;
+import com.aelitis.azureus.plugins.dht.DHTPluginListener;
 import com.aelitis.azureus.plugins.dht.DHTPluginOperationListener;
 import com.aelitis.azureus.plugins.dht.DHTPluginProgressListener;
 import com.aelitis.azureus.plugins.dht.DHTPluginTransferHandler;
@@ -84,6 +86,8 @@ DDBaseImpl
 	private AzureusCore		azureus_core;
 	private DHTPlugin		dht_use_accessor;
 		
+	private CopyOnWriteList	listeners = new CopyOnWriteList();
+	
 	protected
 	DDBaseImpl(
 		final AzureusCore	_azureus_core )
@@ -118,6 +122,24 @@ DDBaseImpl
 					
 					if ( dht_use_accessor.isEnabled()){
 						
+						dht_use_accessor.addListener(
+							new DHTPluginListener()
+							{
+								public void
+								localAddressChanged(
+									DHTPluginContact	local_contact )
+								{
+									List l = listeners.getList();
+									
+									dbEvent ev = new dbEvent( DistributedDatabaseEvent.ET_LOCAL_CONTACT_CHANGED );
+									
+									for (int i=0;i<l.size();i++){
+										
+										((DistributedDatabaseListener)l.get(i)).event( ev );
+									}
+								}
+							});
+						
 						try{
 							addTransferHandler(	torrent_transfer, torrent_transfer );
 		
@@ -162,6 +184,19 @@ DDBaseImpl
 		return( dht.isExtendedUseAllowed());	
 	}
 	
+	public DistributedDatabaseContact
+	getLocalContact()
+	{
+		DHTPlugin	dht = grabDHT();
+		
+		if ( dht == null ){
+			
+			return( null );
+		}
+		
+		return( new DDBaseContactImpl( this, dht.getLocalAddress()));	
+	}
+	
 	protected void
 	throwIfNotAvailable()
 	
@@ -452,7 +487,32 @@ DDBaseImpl
 	{
 		throwIfNotAvailable();
 		
-		getDHT().remove( ((DDBaseKeyImpl)key).getBytes(),
+		getDHT().remove( 
+					((DDBaseKeyImpl)key).getBytes(),
+					key.getDescription(),
+					new listenerMapper( listener, DistributedDatabaseEvent.ET_VALUE_DELETED, key, 0, false, false ));
+	}
+	
+	public void
+	delete(
+		DistributedDatabaseListener		listener,
+		DistributedDatabaseKey			key,
+		DistributedDatabaseContact[]	targets )
+	
+		throws DistributedDatabaseException
+	{
+		throwIfNotAvailable();
+		
+		DHTPluginContact[]	plugin_targets = new DHTPluginContact[ targets.length ];
+		
+		for (int i=0;i<targets.length;i++){
+			
+			plugin_targets[i] = ((DDBaseContactImpl)targets[i]).getContact();
+		}
+		
+		getDHT().remove( 
+					plugin_targets,
+					((DDBaseKeyImpl)key).getBytes(),
 					key.getDescription(),
 					new listenerMapper( listener, DistributedDatabaseEvent.ET_VALUE_DELETED, key, 0, false, false ));
 	}
@@ -567,7 +627,9 @@ DDBaseImpl
 			
 		}else{
 			
-			byte[]	data = getDHT().read( 
+			DHTPluginContact plugin_contact = contact.getContact();
+			
+			byte[]	data = plugin_contact.read( 
 								new DHTPluginProgressListener()
 								{
 									public void
@@ -591,7 +653,6 @@ DDBaseImpl
 										listener.reportCompleteness( percent );
 									}
 								},
-								contact.getContact(),
 								DDBaseHelpers.getKey(type.getClass()).getHash(),
 								((DDBaseKeyImpl)key).getBytes(),
 								timeout );
@@ -605,6 +666,20 @@ DDBaseImpl
 		}
 	}
 	
+	public void 
+	addListener(
+		DistributedDatabaseListener l ) 
+	{
+		listeners.add( l );
+	}
+	
+	public void 
+	removeListener(
+		DistributedDatabaseListener l )
+	{
+		listeners.remove( l );
+	}
+	
 	protected class
 	listenerMapper
 		implements DHTPluginOperationListener
@@ -777,6 +852,7 @@ DDBaseImpl
 
 		public void
 		complete(
+			byte[]	timeout_key,
 			boolean	timeout_occurred )
 		{
 			if ( !complete_disabled ){
@@ -799,6 +875,13 @@ DDBaseImpl
 		private DistributedDatabaseValue	value;
 		private DDBaseContactImpl			contact;
 		
+		protected 
+		dbEvent(
+			int						_type )		
+		{
+			type	= _type;
+		}
+		
 		protected 
 		dbEvent(
 			int						_type,
