@@ -24,11 +24,12 @@ package org.gudy.azureus2.pluginsimpl.local.disk;
 
 import java.util.*;
 
+import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.disk.DiskManagerFileInfoListener;
 import org.gudy.azureus2.core3.download.DownloadManagerPeerListener;
 import org.gudy.azureus2.core3.peer.PEPeer;
 import org.gudy.azureus2.core3.peer.PEPeerManager;
-import org.gudy.azureus2.core3.peer.PEPiece;
 import org.gudy.azureus2.core3.torrent.TOTorrent;
 import org.gudy.azureus2.core3.torrent.TOTorrentFile;
 import org.gudy.azureus2.core3.util.AESemaphore;
@@ -38,6 +39,7 @@ import org.gudy.azureus2.core3.util.DirectByteBuffer;
 import org.gudy.azureus2.core3.util.SystemTime;
 import org.gudy.azureus2.plugins.disk.DiskManagerChannel;
 import org.gudy.azureus2.plugins.disk.DiskManagerEvent;
+import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.plugins.disk.DiskManagerListener;
 import org.gudy.azureus2.plugins.disk.DiskManagerRequest;
 import org.gudy.azureus2.plugins.download.Download;
@@ -47,13 +49,33 @@ import org.gudy.azureus2.pluginsimpl.local.utils.PooledByteBufferImpl;
 
 import com.aelitis.azureus.core.peermanager.piecepicker.PiecePicker;
 import com.aelitis.azureus.core.peermanager.piecepicker.PieceRTAProvider;
+import com.aelitis.azureus.core.util.CopyOnWriteList;
 
 public class 
 DiskManagerChannelImpl 
 	implements DiskManagerChannel, DiskManagerFileInfoListener, DownloadManagerPeerListener, PieceRTAProvider
 {
-	private static final int		BUFFER_SECS 			= 30;
-	private static final int		MIN_PIECES_TO_BUFFER	= 3;
+	private static int		BUFFER_MILLIS;
+	private static int		MIN_PIECES_TO_BUFFER;
+	
+	static{
+		COConfigurationManager.addAndFireParameterListeners(
+			new String[]{
+				"filechannel.rt.buffer.millis",
+				"filechannel.rt.buffer.pieces",
+			},
+			new ParameterListener()
+			{
+				public void 
+				parameterChanged(
+					String parameterName )
+				{
+					BUFFER_MILLIS			= COConfigurationManager.getIntParameter( "filechannel.rt.buffer.millis" );
+					MIN_PIECES_TO_BUFFER 	= COConfigurationManager.getIntParameter( "filechannel.rt.buffer.pieces" );
+				}
+			});
+	}
+	
 	
 	private static final boolean	TRACE = false;
 	
@@ -103,6 +125,50 @@ DiskManagerChannelImpl
 	private static final String	channel_key = "DiskManagerChannel";
 	private static int	channel_id_next;
 	
+		// hack to allow other components to be informed when channels are created
+	
+	private static CopyOnWriteList	listeners = new CopyOnWriteList();
+	
+	public static void 
+	addListener(
+		channelCreateListener		l )
+	{
+		listeners.add( l );
+	}
+	
+	public static void 
+	removeListener(
+		channelCreateListener		l )
+	{
+		listeners.remove( l );
+	}
+	
+	protected static void
+	reportCreated(
+		DiskManagerChannel	channel )
+	{
+		Iterator it = listeners.iterator();
+		
+		while( it.hasNext()){
+			
+			try{
+				((channelCreateListener)it.next()).channelCreated( channel );
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
+			}
+		}
+	}
+	
+	public static interface
+	channelCreateListener
+	{
+		public void
+		channelCreated(
+			DiskManagerChannel	channel );
+	}
+	
 	private DownloadImpl	download;
 	
 	private org.gudy.azureus2.pluginsimpl.local.disk.DiskManagerFileInfoImpl	plugin_file;
@@ -119,9 +185,14 @@ DiskManagerChannelImpl
 		
 	private Average	byte_rate = Average.getInstance( 1000, 20 );
 	
+	private long	start_position;
+	private long	start_time;
 	private long	current_position;
+	
 	private request	current_request;
 	
+	private long	buffer_millis;
+	
 	private PEPeerManager	peer_manager;
 	
 	private long[]	rtas;
@@ -161,6 +232,14 @@ DiskManagerChannelImpl
 		piece_size	= tf.getTorrent().getPieceLength();
 		
 		core_file.addListener( this );
+		
+		reportCreated( this );
+	}
+	
+	public DiskManagerFileInfo 
+	getFile() 
+	{
+		return( plugin_file );
 	}
 	
 	public DiskManagerRequest
@@ -277,6 +356,12 @@ DiskManagerChannelImpl
 		// System.out.println( "data checked:" + offset + "/" + length );
 	}
 	
+	public void
+	peerManagerWillBeAdded(
+		PEPeerManager	manager )
+	{
+	}
+	
 	public void
 	peerManagerAdded(
 		PEPeerManager	manager )
@@ -306,19 +391,7 @@ DiskManagerChannelImpl
 		PEPeer	peer )
 	{
 	}
-		
-	public void
-	pieceAdded(
-		PEPiece 	piece )
-	{
-	}
-		
-	public void
-	pieceRemoved(
-		PEPiece		piece )
-	{
-	}
-	       	
+    	
    	public long[]
    	updateRTAs(
    		PiecePicker		picker )
@@ -329,7 +402,7 @@ DiskManagerChannelImpl
    		
    		long	rate = byte_rate.getAverage();
    		
-   		long	buffer_bytes = BUFFER_SECS * rate;
+   		long	buffer_bytes = ( BUFFER_MILLIS * rate ) / 1000;
    		
    		int	pieces_to_buffer = (int)( buffer_bytes / piece_size );
    		
@@ -338,7 +411,7 @@ DiskManagerChannelImpl
    			pieces_to_buffer	= 1;
    		}
    		
-   		int	millis_per_piece = BUFFER_SECS*1000/pieces_to_buffer; 
+   		int	millis_per_piece = BUFFER_MILLIS/pieces_to_buffer; 
 
    		if ( pieces_to_buffer < MIN_PIECES_TO_BUFFER ){
    			
@@ -351,6 +424,8 @@ DiskManagerChannelImpl
    		 
    		long	now = SystemTime.getCurrentTime();
    		
+   		now += buffer_millis;
+   		
    		for (int i=first_piece;i<first_piece+pieces_to_buffer&&i<rtas.length;i++){
    			
    			rtas[i]	= now + (( i - first_piece ) * millis_per_piece );
@@ -358,7 +433,19 @@ DiskManagerChannelImpl
    		
    		return( rtas );
    	}
+   
+   	public long
+   	getStartTime()
+   	{
+   		return( start_time );
+   	}
    	
+   	public long
+   	getStartPosition()
+   	{
+   		return( start_position );
+   	}
+   
 	public long
 	getCurrentPosition()
 	{
@@ -378,6 +465,25 @@ DiskManagerChannelImpl
 		return( current_position + r.getAvailableBytes());
 	}
 	
+	public void
+	setBufferMillis(
+		long	millis )
+	{
+		buffer_millis = millis;
+	}
+	public String
+	getUserAgent()
+	{
+		request r = current_request;
+		
+		if ( r == null ){
+			
+			return( null );
+		}
+		
+		return( r.getUserAgent());
+	}
+	
 	public void
 	destroy()
 	{
@@ -422,10 +528,18 @@ DiskManagerChannelImpl
 		private long	request_length;
 		private List	listeners	= new ArrayList();
 		
+		private String	user_agent;
+		
 		private volatile boolean	cancelled;
 		
 		AESemaphore	wait_sem = new AESemaphore( "DiskManagerChannelImpl:wait" );
 		
+		protected
+		request()
+		{
+			start_time	= SystemTime.getCurrentTime();
+		}
+		
 		public void
 		setType(
 			int			_type )
@@ -438,6 +552,7 @@ DiskManagerChannelImpl
 			long		_offset )
 		{
 			request_offset	= _offset;
+			start_position	= request_offset;
 		}
 		
 		public void
@@ -456,6 +571,19 @@ DiskManagerChannelImpl
 			}
 		}
 		
+		public void
+		setUserAgent(
+			String	str )
+		{
+			user_agent	= str;
+		}
+		
+		protected String
+		getUserAgent()
+		{
+			return( user_agent );
+		}
+		
 		public long
 		getAvailableBytes()
 		{
