@@ -33,6 +33,7 @@ import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.core3.category.*;
 import org.gudy.azureus2.core3.global.*;
 import org.gudy.azureus2.core3.download.*;
+import org.gudy.azureus2.core3.download.impl.DownloadManagerMoveHandler;
 import org.gudy.azureus2.core3.peer.*;
 import org.gudy.azureus2.core3.torrent.*;
 import org.gudy.azureus2.core3.tracker.client.*;
@@ -50,6 +51,8 @@ import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.plugins.download.Download;
 import org.gudy.azureus2.plugins.download.DownloadActivationEvent;
 import org.gudy.azureus2.plugins.download.DownloadActivationListener;
+import org.gudy.azureus2.plugins.download.DownloadAttributeListener;
+import org.gudy.azureus2.plugins.download.DownloadCompletionListener;
 import org.gudy.azureus2.plugins.download.DownloadListener;
 import org.gudy.azureus2.plugins.download.DownloadPeerListener;
 import org.gudy.azureus2.plugins.download.DownloadPropertyListener;
@@ -62,18 +65,21 @@ import org.gudy.azureus2.plugins.download.DownloadException;
 import org.gudy.azureus2.plugins.download.DownloadRemovalVetoException;
 import org.gudy.azureus2.plugins.download.DownloadWillBeRemovedListener;
 import org.gudy.azureus2.plugins.download.session.SessionAuthenticator;
+import org.gudy.azureus2.plugins.download.savelocation.SaveLocationChange;
 
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.logging.LogRelation;
 
 import com.aelitis.azureus.core.util.CopyOnWriteList;
+import com.aelitis.azureus.core.util.CopyOnWriteMap;
 
 public class 
 DownloadImpl
 	extends LogRelation
 	implements 	Download, DownloadManagerListener, 
-				DownloadManagerTrackerListener, DownloadManagerPeerListener,
-				DownloadManagerStateListener, DownloadManagerActivationListener
+				DownloadManagerTrackerListener,
+				DownloadManagerStateListener, DownloadManagerActivationListener,
+				DownloadManagerStateAttributeListener
 {
 	private DownloadManager		download_manager;
 	private DownloadStatsImpl		download_stats;
@@ -92,10 +98,15 @@ DownloadImpl
 	private AEMonitor	tracker_listeners_mon	= new AEMonitor( "Download:TL");
 	private List		removal_listeners 		= new ArrayList();
 	private AEMonitor	removal_listeners_mon	= new AEMonitor( "Download:RL");
-	private List		peer_listeners			= new ArrayList();
+	private Map			peer_listeners			= new HashMap();
 	private AEMonitor	peer_listeners_mon		= new AEMonitor( "Download:PL");
 	
-	private CopyOnWriteList	activation_listeners	= new CopyOnWriteList();
+	private CopyOnWriteList completion_listeners     = new CopyOnWriteList();
+	
+	private CopyOnWriteMap read_attribute_listeners_map_cow  = new CopyOnWriteMap();
+	private CopyOnWriteMap write_attribute_listeners_map_cow = new CopyOnWriteMap();
+	
+	private CopyOnWriteList	activation_listeners = new CopyOnWriteList();
 	private DownloadActivationEvent	activation_state;
 	
 	protected
@@ -372,7 +383,7 @@ DownloadImpl
 	{
 		if ( !download_manager.canForceRecheck()){
 			
-			throw( new DownloadException( "Download::recheckData: download must be stopped, quued or in error state" ));
+			throw( new DownloadException( "Download::recheckData: download must be stopped, queued or in error state" ));
 		}
 		
 		download_manager.forceRecheck();
@@ -500,8 +511,13 @@ DownloadImpl
 	  	return( null );
   }
   
-  public void setListAttribute(TorrentAttribute attribute, String[] value) {
+  public void 
+  setListAttribute(
+	TorrentAttribute attribute, 
+	String[] value) 
+  {
 	  String name = convertAttribute(attribute);
+	  
 	  if (name != null) {
 		  download_manager.getDownloadState().setListAttribute(name, value);
 	  }
@@ -516,7 +532,10 @@ DownloadImpl
 	  	
 	  	if ( name != null ){
 			
-			download_manager.getDownloadState().setMapAttribute( name, value );
+	  			// gotta clone before updating in case user has read values and then just
+	  			// updated them - setter code optimises out sets of the same values...
+	  		
+			download_manager.getDownloadState().setMapAttribute( name, BEncoder.cloneMap( value ));
 	  	}
   }
   
@@ -624,10 +643,6 @@ DownloadImpl
 		
 		return( DownloadManagerState.AT_RELATIVE_SAVE_PATH );	
 		
-	}else if ( attribute.getName() == TorrentAttribute.TA_CONTENT_MAP ){
-		
-		return( DownloadManagerState.AT_CONTENT_MAP );
-		
 	}else if ( attribute.getName() == TorrentAttribute.TA_SHARE_PROPERTIES ){
   		
 			// this is a share-level attribute only, not propagated to individual downloads
@@ -678,10 +693,6 @@ DownloadImpl
 		
 		return ( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_RELATIVE_SAVE_PATH ));
 		
-	}else if ( name.equals ( DownloadManagerState.AT_CONTENT_MAP)){
-		
-		return ( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_CONTENT_MAP ));
-
 	}else if ( name.startsWith( "Plugin." )){
   		
 		return( TorrentManagerImpl.getSingleton().getAttribute( name ));
@@ -727,14 +738,8 @@ DownloadImpl
 			GlobalManager globalManager = download_manager.getGlobalManager();
 			
 			try{
-				globalManager.canDownloadManagerBeRemoved(download_manager );
 				
-				if ( delete_torrent || delete_data ){
-					
-					download_manager.stopIt( dl_state, delete_torrent, delete_data );
-				}
-				
-				globalManager.removeDownloadManager(download_manager);
+				globalManager.removeDownloadManager(download_manager, delete_torrent, delete_data);
 				
 			}catch( GlobalManagerDownloadRemovalVetoException e ){
 				
@@ -761,7 +766,7 @@ DownloadImpl
 			GlobalManager globalManager = download_manager.getGlobalManager();
 			
 			try{
-				globalManager.canDownloadManagerBeRemoved(download_manager);
+				globalManager.canDownloadManagerBeRemoved(download_manager, false, false);
 				
 			}catch( GlobalManagerDownloadRemovalVetoException e ){
 				
@@ -838,18 +843,31 @@ DownloadImpl
 		
 		latest_state	= convertState(state);
 		
-		// System.out.println("Plug:prev = " + prev_state + ", curr = " + latest_state );
+		// System.out.println("Plug: dl = " + getName() + ", prev = " + prev_state + ", curr = " + latest_state + ", signalled state = " + state);
 		
 		boolean curr_forcedStart = isForceStart();
-	
+		
+		// Copy reference in case any attempts to remove or add listeners are tried.
+		List listeners_to_use = listeners;
+		
 		if ( prev_state != latest_state || latest_forcedStart != curr_forcedStart ){
 			
 			latest_forcedStart = curr_forcedStart;
 			
-			for (int i=0;i<listeners.size();i++){
+			for (int i=0;i<listeners_to_use.size();i++){
 				
 				try{
-					((DownloadListener)listeners.get(i)).stateChanged( this, prev_state, latest_state );
+					long startTime = SystemTime.getCurrentTime();
+					DownloadListener listener = (DownloadListener)listeners_to_use.get(i);
+
+					listener.stateChanged( this, prev_state, latest_state );
+					
+					long diff = SystemTime.getCurrentTime() - startTime;
+					if (diff > 1000) {
+						System.out.println("Plugin should move long processes (" + diff
+								+ "ms) off of Download's stateChanged listener trigger. "
+								+ listener);
+					}
 				
 				}catch( Throwable e ){
 					
@@ -862,6 +880,19 @@ DownloadImpl
 	public void
 	downloadComplete(DownloadManager manager)
 	{	
+		if (this.completion_listeners.isEmpty()) {return;}
+		Iterator itr = this.completion_listeners.iterator();
+		DownloadCompletionListener dcl;
+		while (itr.hasNext()) {
+			dcl = (DownloadCompletionListener)itr.next();
+			long startTime = SystemTime.getCurrentTime();
+			try {dcl.onCompletion(this);}
+			catch (Throwable t) {Debug.printStackTrace(t);}
+			long diff = SystemTime.getCurrentTime() - startTime;
+			if (diff > 1000) {
+				System.out.println("Plugin should move long processes (" + diff + "ms) off of Download's onCompletion listener trigger. " + dcl);
+			}
+		}
 	}
 
 	public void 
@@ -884,7 +915,17 @@ DownloadImpl
   {	
 	for (int i = 0; i < listeners.size(); i++) {
 		try {
-			((DownloadListener)listeners.get(i)).positionChanged(this, oldPosition, newPosition);
+			long startTime = SystemTime.getCurrentTime();
+			DownloadListener listener = (DownloadListener)listeners.get(i);
+
+			listener.positionChanged(this, oldPosition, newPosition);
+
+			long diff = SystemTime.getCurrentTime() - startTime;
+			if (diff > 1000) {
+				System.out.println("Plugin should move long processes (" + diff
+						+ "ms) off of Download's positionChanged listener trigger. "
+						+ listener);
+			}
 		} catch (Throwable e) {
 			Debug.printStackTrace( e );
 		}
@@ -909,6 +950,7 @@ DownloadImpl
 		}
 	}
 	
+
 	public void
 	removeListener(
 		DownloadListener	l )
@@ -927,6 +969,45 @@ DownloadImpl
 		}
 	}
 	
+	public void addAttributeListener(DownloadAttributeListener listener, TorrentAttribute attr, int event_type) {
+		String attribute = convertAttribute(attr);
+		if (attribute == null) {return;}
+		
+		CopyOnWriteMap attr_map = this.getAttributeMapForType(event_type);
+		CopyOnWriteList listener_list = (CopyOnWriteList)attr_map.get(attribute);
+		boolean add_self = false;
+		
+		if (listener_list == null) {
+			listener_list = new CopyOnWriteList();
+			attr_map.put(attribute, listener_list);
+		}
+		add_self = listener_list.isEmpty();
+		
+		listener_list.add(listener);
+		if (add_self) {
+			download_manager.getDownloadState().addListener(this, attribute, event_type);
+		}
+	}
+
+	public void removeAttributeListener(DownloadAttributeListener listener, TorrentAttribute attr, int event_type) {
+		String attribute = convertAttribute(attr);
+		if (attribute == null) {return;}
+		
+		CopyOnWriteMap attr_map = this.getAttributeMapForType(event_type);
+		CopyOnWriteList listener_list = (CopyOnWriteList)attr_map.get(attribute);
+		boolean remove_self = false;
+			
+		if (listener_list != null) {
+			listener_list.remove(listener);
+			remove_self = listener_list.isEmpty();
+		}
+		
+		if (remove_self) {
+			download_manager.getDownloadState().removeListener(this, attribute, event_type);
+		}
+		
+	}
+	
 	public DownloadAnnounceResult
 	getLastAnnounceResult()
 	{
@@ -969,6 +1050,12 @@ DownloadImpl
 		}
 	}
 	
+	// Used by DownloadEventNotifierImpl.
+	void announceTrackerResultsToListener(DownloadTrackerListener l) {
+		l.announceResult(last_announce_result);
+		l.scrapeResult(last_scrape_result);
+	}
+	
 	public void
 	announceResult(
 		TRTrackerAnnouncerResponse			response )
@@ -1139,11 +1226,7 @@ DownloadImpl
 			tracker_listeners_mon.exit();
 		}
 		
-		if (immediateTrigger) {
-			l.announceResult( last_announce_result );
-			
-			l.scrapeResult( last_scrape_result );
-		}
+		if (immediateTrigger) {this.announceTrackerResultsToListener(l);}
 	}
 	
 	public void
@@ -1209,49 +1292,88 @@ DownloadImpl
 	
 	public void
 	addPeerListener(
-		DownloadPeerListener	l )
+		final DownloadPeerListener	listener )
 	{
+		DownloadManagerPeerListener delegate =
+			new DownloadManagerPeerListener()
+			{
+				
+				public void
+				peerManagerAdded(
+					PEPeerManager	manager )
+				{
+					PeerManager pm = PeerManagerImpl.getPeerManager( manager);
+					
+					listener.peerManagerAdded( DownloadImpl.this, pm );
+				}
+				
+				public void
+				peerManagerRemoved(
+					PEPeerManager	manager )
+				{
+					PeerManager pm = PeerManagerImpl.getPeerManager( manager);
+					
+					listener.peerManagerRemoved( DownloadImpl.this, pm );
+
+				}
+				
+				public void
+				peerManagerWillBeAdded(
+					PEPeerManager	manager )
+				{
+				}
+
+				public void
+				peerAdded(
+					PEPeer 	peer )
+				{				
+				}
+					
+				public void
+				peerRemoved(
+					PEPeer	peer )
+				{	
+				}
+			};
+		
 		try{
 			peer_listeners_mon.enter();
+							
+			peer_listeners.put( listener, delegate );
 		
-			List	new_peer_listeners	= new ArrayList( peer_listeners );
-			
-			new_peer_listeners.add( l );
-			
-			peer_listeners	= new_peer_listeners;
-			
-			if ( peer_listeners.size() == 1 ){
-				
-				download_manager.addPeerListener( this );
-			}
 		}finally{
 			
 			peer_listeners_mon.exit();
 		}
+		
+		download_manager.addPeerListener( delegate );
 	}
 	
 	
 	public void
 	removePeerListener(
-		DownloadPeerListener	l )
+		DownloadPeerListener	listener )
 	{
+		DownloadManagerPeerListener delegate;
+		
 		try{
 			peer_listeners_mon.enter();
+			
+			delegate = (DownloadManagerPeerListener)peer_listeners.remove( listener );
 
-			List	new_peer_listeners	= new ArrayList( peer_listeners );
-			
-			new_peer_listeners.remove( l );
-			
-			peer_listeners	= new_peer_listeners;
-			
-			if ( peer_listeners.size() == 0 ){
-				
-				download_manager.removePeerListener( this );
-			}
 		}finally{
 			
 			peer_listeners_mon.exit();
 		}
+		
+		if ( delegate == null ){
+			
+			Debug.out( "Listener not found for removal" );
+			
+		}else{
+			
+			download_manager.removePeerListener( delegate );
+		}
 	}
 	
 	public boolean
@@ -1335,37 +1457,25 @@ DownloadImpl
 			peer_listeners_mon.exit();
 		}
 	}
-	
-	
-	
-	public void
-	peerManagerAdded(
-		PEPeerManager	manager )
-	{
-		if ( peer_listeners.size() > 0 ){
-			
-			PeerManager pm = PeerManagerImpl.getPeerManager( manager);
-		
-			for (int i=0;i<peer_listeners.size();i++){
-		
-				((DownloadPeerListener)peer_listeners.get(i)).peerManagerAdded( this, pm );
-			}
+
+	public void	addCompletionListener(DownloadCompletionListener l) {
+		try {
+			listeners_mon.enter();
+			this.completion_listeners.add(l);
+		}
+		finally{
+			listeners_mon.exit();
 		}
 	}
 	
-	public void
-	peerManagerRemoved(
-		PEPeerManager	manager )
-	{
-		if ( peer_listeners.size() > 0 ){
-			
-			PeerManager pm = PeerManagerImpl.getPeerManager( manager);
-		
-			for (int i=0;i<peer_listeners.size();i++){
-		
-				((DownloadPeerListener)peer_listeners.get(i)).peerManagerRemoved( this, pm );
-			}
-		}	
+	public void	removeCompletionListener(DownloadCompletionListener l) {
+		try {
+			listeners_mon.enter();
+			this.completion_listeners.remove(l);
+		}
+		finally{
+			listeners_mon.exit();
+		}
 	}
 	
  	public PeerManager
@@ -1416,39 +1526,16 @@ DownloadImpl
 		return( res );
 	}
 	
-	public void
-	peerAdded(
-		PEPeer 	peer )
-	{
-		
-	}
-		
-	public void
-	peerRemoved(
-		PEPeer	peer )
-	{
-		
-	}
-		
-	public void
-	pieceAdded(
-		PEPiece 	piece )
-	{
-		
-	}
-		
-	public void
-	pieceRemoved(
-		PEPiece		piece )
-	{
-		
-	}
-	
  	public void
 	setMaximumDownloadKBPerSecond(
 		int		kb )
  	{
- 		download_manager.getStats().setDownloadRateLimitBytesPerSecond( kb < 0 ? 0 : kb*1024 );
+         if(kb==-1){
+            Debug.out("setMaximiumDownloadKBPerSecond got value (-1) ZERO_DOWNLOAD. (-1)"+
+                "does not work through this method, use getDownloadRateLimitBytesPerSecond() instead.");
+         }//if
+
+         download_manager.getStats().setDownloadRateLimitBytesPerSecond( kb < 0 ? 0 : kb*1024 );
  	}
   	
   	public int
@@ -1465,6 +1552,14 @@ DownloadImpl
       download_manager.getStats().setUploadRateLimitBytesPerSecond( max_rate_bps );
   	}
 
+  	public int getDownloadRateLimitBytesPerSecond() {
+  		return download_manager.getStats().getDownloadRateLimitBytesPerSecond();
+  	}
+
+  	public void setDownloadRateLimitBytesPerSecond( int max_rate_bps ) {
+  		download_manager.getStats().setDownloadRateLimitBytesPerSecond( max_rate_bps );
+  	}
+
   public int getSeedingRank() {
     return download_manager.getSeedingRank();
   }
@@ -1493,6 +1588,19 @@ DownloadImpl
  			throw( new DownloadException("move operation failed", e ));
  		}
   	}
+
+	public void moveDataFiles(File new_parent_dir, String new_name)
+  	
+  		throws DownloadException
+  	{
+ 		try{
+ 			download_manager.moveDataFiles( new_parent_dir, new_name );
+ 			
+ 		}catch( DownloadManagerException e ){
+ 			
+ 			throw( new DownloadException("move / rename operation failed", e ));
+ 		}
+  	}
 	
 	public void renameDownload(String new_name) throws DownloadException {
 		try {download_manager.renameDownload(new_name);}
@@ -1516,8 +1624,13 @@ DownloadImpl
  		}  	
  	}
   	
+  	/**
+  	 * @deprecated
+  	 */
   	public File[] calculateDefaultPaths(boolean for_moving) {
-  		return download_manager.calculateDefaultPaths(for_moving);
+  	  SaveLocationChange slc = this.calculateDefaultDownloadLocation(); 
+	  if (slc == null) {return null;}
+	  return new File[] {slc.download_location, slc.torrent_location};
   	}
   	
   	public boolean isInDefaultSaveDir() {
@@ -1551,10 +1664,11 @@ DownloadImpl
   }
   
   
-  public boolean isMessagingEnabled() {  return download_manager.isAZMessagingEnabled();  }
+  public boolean isMessagingEnabled() {  return download_manager.isExtendedMessagingEnabled();  }
 
   public void setMessagingEnabled( boolean enabled ) {
-    download_manager.setAZMessagingEnabled( enabled );
+	  throw new RuntimeException("setMessagingEnabled is in the process of being removed - if you are seeing this error, let the Azureus developers know that you need this method to stay!");
+    //download_manager.setAZMessagingEnabled( enabled );
   }
   
   
@@ -1576,6 +1690,9 @@ DownloadImpl
   public void setPriority(int priority) {
   }
 
+  public boolean isRemoved() {
+	return( download_manager.isDestroyed());
+  }
   // Pass LogRelation off to core objects
 
 	/* (non-Javadoc)
@@ -1591,4 +1708,92 @@ DownloadImpl
 	public Object[] getQueryableInterfaces() {
 		return new Object[] { download_manager };
 	}
-}
+	
+	private CopyOnWriteMap getAttributeMapForType(int event_type) {
+		return event_type == DownloadPropertyEvent.PT_TORRENT_ATTRIBUTE_WILL_BE_READ ? read_attribute_listeners_map_cow : write_attribute_listeners_map_cow;
+	}
+	
+	public boolean canMoveDataFiles() {
+		return download_manager.canMoveDataFiles();
+	}
+	
+	public void attributeEventOccurred(DownloadManager download, String attribute, int event_type) {
+		CopyOnWriteMap attr_listener_map = getAttributeMapForType(event_type);
+
+		TorrentAttribute attr = convertAttribute(attribute);
+		if (attr == null) {return;}
+		
+		List listeners = null;
+		listeners = ((CopyOnWriteList)attr_listener_map.get(attribute)).getList();
+
+		if (listeners == null) {return;}
+	
+		for (int i=0; i<listeners.size(); i++) {
+			DownloadAttributeListener dal = (DownloadAttributeListener)listeners.get(i);
+			try {dal.attributeEventOccurred(this, attr, event_type);}
+			catch (Throwable t) {Debug.printStackTrace(t);}
+		}
+	}
+	
+	public SaveLocationChange calculateDefaultDownloadLocation() {
+		return DownloadManagerMoveHandler.recalculatePath(this.download_manager);
+	}
+	
+	 public Object getUserData( Object key ){
+		 return( download_manager.getUserData(key));
+	 }
+
+	 public void setUserData( Object key, Object data ){
+		 download_manager.setUserData(key, data);
+	 }
+	 
+	 public void startDownload(boolean force) {
+		if (force) {
+			this.setForceStart(true);
+			return;
+		}
+		this.setForceStart(false);
+		
+		int state = this.getState();
+		if (state == DownloadManager.STATE_STOPPED ||	state == DownloadManager.STATE_QUEUED) {
+			download_manager.setStateWaiting();
+		}
+		
+	 }
+	 
+	 public void stopDownload() {
+		 if (download_manager.getState() == DownloadManager.STATE_STOPPED) {return;}
+		 download_manager.stopIt(DownloadManager.STATE_STOPPED, false, false);
+	 }
+	 
+	 public void changeLocation(SaveLocationChange slc) throws DownloadException {
+		 
+		 // No change in the file.
+		 boolean has_change = slc.hasDownloadChange() || slc.hasTorrentChange();
+		 if (!has_change) {return;}
+		 
+		 // Test that one of the locations is actually different.
+		 has_change = slc.isDifferentDownloadLocation(new File(this.getSavePath()));
+		 if (!has_change) {
+			 has_change = slc.isDifferentTorrentLocation(new File(this.getTorrentFileName()));
+		 }
+		 
+		 if (!has_change) {return;}
+
+		 boolean try_to_resume = !this.isPaused();
+		 try {
+			 try {
+				 if (slc.hasDownloadChange()) {download_manager.moveDataFiles(slc.download_location, slc.download_name);}
+				 if (slc.hasTorrentChange()) {download_manager.moveTorrentFile(slc.torrent_location, slc.torrent_name);}
+			 }
+			 catch (DownloadManagerException e) {
+				 throw new DownloadException(e.getMessage(), e);
+			 }
+		 }
+		 finally {
+			 if (try_to_resume) {this.resume();}
+		 }
+			 
+	 }
+	 
+}
\ No newline at end of file
