@@ -39,16 +39,21 @@ import org.gudy.azureus2.pluginsimpl.local.torrent.*;
 import org.gudy.azureus2.pluginsimpl.local.ui.UIManagerImpl;
 import org.gudy.azureus2.plugins.download.DownloadException;
 import org.gudy.azureus2.plugins.download.Download;
+import org.gudy.azureus2.plugins.download.DownloadEventNotifier;
 import org.gudy.azureus2.plugins.download.DownloadManagerListener;
 import org.gudy.azureus2.plugins.download.DownloadManagerStats;
 import org.gudy.azureus2.plugins.download.DownloadRemovalVetoException;
 import org.gudy.azureus2.plugins.download.DownloadWillBeAddedListener;
+import org.gudy.azureus2.plugins.download.savelocation.DefaultSaveLocationManager;
+import org.gudy.azureus2.plugins.download.savelocation.SaveLocationManager;
 
 import org.gudy.azureus2.core3.torrent.*;
 import org.gudy.azureus2.core3.config.*;
 import org.gudy.azureus2.core3.global.*;
 import org.gudy.azureus2.core3.disk.DiskManager;
 import org.gudy.azureus2.core3.download.*;
+import org.gudy.azureus2.core3.download.impl.DownloadManagerMoveHandler;
+import org.gudy.azureus2.core3.download.impl.DownloadManagerDefaultPaths;
 import org.gudy.azureus2.core3.util.*;
 
 
@@ -79,9 +84,10 @@ DownloadManagerImpl
 		}
 	}
 	
-	private AzureusCore				azureus_core;
+	//private AzureusCore				azureus_core;
 	private GlobalManager			global_manager;
 	private DownloadManagerStats	stats;
+	private DownloadEventNotifierImpl global_dl_notifier;
 	
 	private List			listeners		= new ArrayList();
 	private CopyOnWriteList	dwba_listeners	= new CopyOnWriteList();
@@ -95,10 +101,11 @@ DownloadManagerImpl
 	DownloadManagerImpl(
 		AzureusCore	_azureus_core )
 	{
-		azureus_core	= _azureus_core;
+		//azureus_core	= _azureus_core;
 		global_manager	= _azureus_core.getGlobalManager();
 		
 		stats = new DownloadManagerStatsImpl( global_manager );
+		global_dl_notifier = new DownloadEventNotifierImpl(this);
 		
 		global_manager.addListener(
 			new GlobalManagerListener()
@@ -164,7 +171,7 @@ DownloadManagerImpl
 				}
                 
                 
-                public void seedingStatusChanged( boolean seeding_only_mode ){
+                public void seedingStatusChanged( boolean seeding_only_mode, boolean b ){
                   //TODO
                 }           
 			});
@@ -174,7 +181,9 @@ DownloadManagerImpl
 			{
 				public void
 				downloadWillBeRemoved(
-					DownloadManager	dm )
+					DownloadManager	dm,
+					boolean remove_torrent,
+					boolean remove_data )
 				
 					throws GlobalManagerDownloadRemovalVetoException
 				{					
@@ -205,7 +214,7 @@ DownloadManagerImpl
 	addDownload(
 		final URL	url) 
 	{
-		addDownload(url,null,true);
+		addDownload(url,null,true,null);
 	}
 	
 	public void 
@@ -215,7 +224,7 @@ DownloadManagerImpl
 	
 		throws DownloadException
 	{
-		addDownload(url,null,auto_download);	
+		addDownload(url,null,auto_download,null);	
 	}
 	
 	public void 
@@ -223,18 +232,28 @@ DownloadManagerImpl
 		final URL	url,
 		final URL 	referrer) 
 	{
-		addDownload(url,referrer,true);
+		addDownload(url,referrer,true,null);
+	}
+	
+	public void 
+	addDownload(
+		URL 		url, 
+		Map 		request_properties ) 
+	{
+		addDownload(url,null,true,request_properties);
 	}
 	
 	public void 
 	addDownload(
 		final URL	url,
 		final URL 	referrer,
-		boolean		auto_download )
+		boolean		auto_download,
+		Map			request_properties )
 	{
-		UIManagerImpl.fireEvent( UIManagerEvent.ET_OPEN_TORRENT_VIA_URL, new Object[]{ url, referrer, new Boolean( auto_download )});
+		UIManagerImpl.fireEvent( UIManagerEvent.ET_OPEN_TORRENT_VIA_URL, new Object[]{ url, referrer, new Boolean( auto_download ), request_properties });
 	}
 	
+
 	protected void
 	addDownloadManager(
 		DownloadManager	dm )
@@ -295,6 +314,18 @@ DownloadManagerImpl
 		File		torrent_file,
 		File		data_location )
 	
+		throws DownloadException
+	{
+		return( addDownload( torrent, torrent_file, data_location, getInitialState()));
+	}
+	
+	public Download
+	addDownload(
+		Torrent		torrent,
+		File		torrent_file,
+		File		data_location,
+		int			initial_state )
+	
 		throws DownloadException
 	{
 		if ( torrent_file == null ){
@@ -337,14 +368,7 @@ DownloadManagerImpl
 		
 		if ( data_location == null ){
 			
-		    boolean useDefDataDir = COConfigurationManager.getBooleanParameter("Use default data dir");
-		    
-		    String data_dir = null;
-		    
-		    if (useDefDataDir){
-		    	
-		    	data_dir = COConfigurationManager.getStringParameter("Default save path");
-		    }
+		    String data_dir = COConfigurationManager.getStringParameter("Default save path");
 		    
 		    if ( data_dir == null || data_dir.length() == 0 ){
 		    	
@@ -361,13 +385,15 @@ DownloadManagerImpl
 			hash = torrent.getHash();
 		} catch (Exception e) { }
 		
+		boolean	for_seeding = torrent.isComplete();
+		
 		DownloadManager dm = global_manager.addDownloadManager(
 				torrent_file.toString(), hash, data_location.toString(),
-				getInitialState(), true);
+				initial_state, true, for_seeding, null );
 		
 		if ( dm == null ){
 			
-			throw( new DownloadException( "DownloadManager::addDownload - failed"));
+			throw( new DownloadException( "DownloadManager::addDownload - failed, download may already in the process of being added"));
 		}
 		
 		addDownloadManager( dm );
@@ -375,6 +401,17 @@ DownloadManagerImpl
 		return( getDownload( dm ));
 	}
 
+	public Download
+	addDownloadStopped(
+		Torrent		torrent,
+		File		torrent_location,
+		File		data_location )
+	
+		throws DownloadException
+	{
+		return( addDownload( torrent, torrent_location, data_location, DownloadManager.STATE_STOPPED ));
+	}
+	
 	public Download
 	addNonPersistentDownload(
 		Torrent		torrent,
@@ -426,9 +463,18 @@ DownloadManagerImpl
 		
 		return( dl );
 	}
+	
+	public static DownloadImpl[] getDownloadStatic(DownloadManager[] dm) {
+		ArrayList res = new ArrayList(dm.length);
+		for (int i=0; i<dm.length; i++) {
+			try {res.add(getDownloadStatic(dm[i]));}
+			catch (DownloadException de) {}
+		}
+		return (DownloadImpl[])res.toArray(new DownloadImpl[res.size()]);
+	}
 
 	/**
-	 * Retrieve the plugin Downlaod object related to the DowloadManager
+	 * Retrieve the plugin Downlaod object related to the DownloadManager
 	 * 
 	 * @param dm DownloadManager to find
 	 * @return plugin object
@@ -489,22 +535,25 @@ DownloadManagerImpl
 	
 		throws DownloadException
 	{
-		for (int i=0;i<downloads.size();i++){
+		if ( torrent != null ){
 			
-			Download	dl = (Download)downloads.get(i);
-			
-			TorrentImpl	t = (TorrentImpl)dl.getTorrent();
-			
-				// can be null if broken torrent
-			
-			if ( t == null ){
+			for (int i=0;i<downloads.size();i++){
 				
-				continue;
-			}
-			
-			if ( t.getTorrent().hasSameHashAs( torrent )){
+				Download	dl = (Download)downloads.get(i);
 				
-				return( dl );
+				TorrentImpl	t = (TorrentImpl)dl.getTorrent();
+				
+					// can be null if broken torrent
+				
+				if ( t == null ){
+					
+					continue;
+				}
+				
+				if ( t.getTorrent().hasSameHashAs( torrent )){
+					
+					return( dl );
+				}
 			}
 		}
 		
@@ -695,57 +744,57 @@ DownloadManagerImpl
 		return( global_manager.isSeedingOnly());
 	}
 	
-	public void
-	addListener(
-		DownloadManagerListener	l )
-	{
-		List	downloads_copy;
-		
-		try{
-			listeners_mon.enter();
-			
-			List	new_listeners = new ArrayList( listeners );
-			
-			new_listeners.add( l );
-		
-			listeners	= new_listeners;
-			
-			downloads_copy = new ArrayList( downloads );
+	public void addListener(DownloadManagerListener l) {addListener(l, true);}
 	
-		}finally{
-			
+	public void addListener(DownloadManagerListener l, boolean notify_of_current_downloads) {
+		List downloads_copy = null;
+
+		try {
+			listeners_mon.enter();
+			List new_listeners = new ArrayList(listeners);
+			new_listeners.add(l);
+			listeners = new_listeners;
+			if (notify_of_current_downloads) {
+				downloads_copy = new ArrayList(downloads);
+			}
+		}
+		finally {
 			listeners_mon.exit();
-		}	
-		
-		for (int i=0;i<downloads_copy.size();i++){
-			
-			try{
-					
-				l.downloadAdded((Download)downloads_copy.get(i));
-				
-			}catch( Throwable e ){
-				
-				Debug.printStackTrace( e );
+		}
+
+		if (downloads_copy != null) {
+			for (int i = 0; i < downloads_copy.size(); i++) {
+				try {l.downloadAdded((Download) downloads_copy.get(i));}
+				catch (Throwable e) {Debug.printStackTrace(e);}
 			}
 		}
 	}
 	
-	public void
-	removeListener(
-		DownloadManagerListener	l )
-	{
-		try{
+	public void removeListener(DownloadManagerListener l) {removeListener(l, false);}
+
+	public void removeListener(DownloadManagerListener l, boolean notify_of_current_downloads) {
+		List downloads_copy = null;
+		
+		try {
 			listeners_mon.enter();
-			
-			List	new_listeners	= new ArrayList( listeners );
-			
+			List new_listeners = new ArrayList(listeners);
 			new_listeners.remove(l);
-			
-			listeners	= new_listeners;
-			
-		}finally{
+			listeners = new_listeners;
+			if (notify_of_current_downloads) {
+				downloads_copy = new ArrayList(downloads);
+			}
+		}
+		finally {
 			listeners_mon.exit();
-		}	
+		}
+
+		if (downloads_copy != null) {
+			for (int i = 0; i < downloads_copy.size(); i++) {
+				try {l.downloadRemoved((Download) downloads_copy.get(i));}
+				catch (Throwable e) {Debug.printStackTrace(e);}
+			}
+		}
+	
 	}
 	
 	public void
@@ -888,10 +937,21 @@ DownloadManagerImpl
 		}
 	}
 	
-	public boolean
-	isExteralDownload(
-		Download	download )
-	{
-		return( !( download instanceof DownloadImpl ));
+	public DownloadEventNotifier getGlobalDownloadEventNotifier() {
+		return this.global_dl_notifier;
 	}
+	
+	public void setSaveLocationManager(SaveLocationManager manager) {
+		if (manager == null) {manager = getDefaultSaveLocationManager();}
+		DownloadManagerMoveHandler.CURRENT_HANDLER = manager;				
+	}
+	
+	public SaveLocationManager getSaveLocationManager() {
+		return DownloadManagerMoveHandler.CURRENT_HANDLER;
+	}	
+
+	public DefaultSaveLocationManager getDefaultSaveLocationManager() {
+		return DownloadManagerDefaultPaths.DEFAULT_HANDLER;
+	}	
+	
 }
