@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- * 
+ *
  * AELITIS, SAS au capital de 46,603.30 euros
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  *
@@ -28,25 +28,29 @@ package org.gudy.azureus2.pluginsimpl.local.tracker;
  *
  */
 
-import java.util.*;
-import java.io.*;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
 import java.net.InetSocketAddress;
 import java.net.URL;
+import java.util.ArrayList;
+import java.util.List;
 
-import org.gudy.azureus2.plugins.PluginInterface;
-import org.gudy.azureus2.plugins.tracker.*;
-import org.gudy.azureus2.plugins.tracker.web.*;
-import org.gudy.azureus2.pluginsimpl.local.utils.UtilitiesImpl;
-import org.gudy.azureus2.core3.tracker.host.*;
-import org.gudy.azureus2.core3.tracker.server.*;
+import org.gudy.azureus2.core3.tracker.host.TRHostAuthenticationListener;
+import org.gudy.azureus2.core3.tracker.server.TRTrackerServerAuthenticationListener;
 import org.gudy.azureus2.core3.util.AEMonitor;
+import org.gudy.azureus2.plugins.PluginInterface;
+import org.gudy.azureus2.plugins.tracker.Tracker;
+import org.gudy.azureus2.plugins.tracker.web.TrackerWebContext;
+import org.gudy.azureus2.plugins.tracker.web.TrackerWebPageGenerator;
+import org.gudy.azureus2.pluginsimpl.local.utils.UtilitiesImpl;
 
-public abstract class 
-TrackerWCHelper 
+public abstract class
+TrackerWCHelper
 	implements TrackerWebContext, TRHostAuthenticationListener, TRTrackerServerAuthenticationListener
 {
 	private PluginInterface		plugin_interface;
-	
+
 	private Tracker		tracker;
 	private List		generators 	= new ArrayList();
 
@@ -57,14 +61,14 @@ TrackerWCHelper
 	{
 		plugin_interface = UtilitiesImpl.getPluginThreadContext();
 	}
-	
+
 	protected void
 	setTracker(
 		Tracker		_tracker )
 	{
 		tracker	= _tracker;
 	}
-	
+
 	public boolean
 	handleExternalRequest(
 		InetSocketAddress	_client_address,
@@ -74,82 +78,90 @@ TrackerWCHelper
 		String				_header,
 		InputStream			_is,
 		OutputStream		_os )
-	
+
 		throws IOException
-	{	
+	{
 		UtilitiesImpl.setPluginThreadContext( plugin_interface );
-		
+
 		TrackerWebPageRequestImpl	request = new TrackerWebPageRequestImpl( tracker, this, _client_address, _user, _url, _absolute_url, _header, _is );
-		TrackerWebPageResponseImpl	reply 	= new TrackerWebPageResponseImpl( _os );
-		
+		TrackerWebPageResponseImpl	reply 	= new TrackerWebPageResponseImpl( _os , request);
+
 		for (int i=0;i<generators.size();i++){
 
 			TrackerWebPageGenerator	generator;
-			
+
 			try{
 				this_mon.enter();
-				
+
 				if ( i >= generators.size()){
-					
+
 					break;
 				}
-				
+
 				generator = (TrackerWebPageGenerator)generators.get(i);
-				
+
 			}finally{
-				
+
 				this_mon.exit();
 			}
-			
+
 			if ( generator.generate( request, reply )){
-					
+
 				reply.complete();
-					
+
 				return( true );
 			}
 		}
-		
+
 		return( false );
-	}	
-	
-	
+	}
+
+
 	public TrackerWebPageGenerator[]
 	getPageGenerators()
 	{
 		TrackerWebPageGenerator[]	res = new TrackerWebPageGenerator[generators.size()];
-		
+
 		generators.toArray( res );
-		
+
 		return( res );
 	}
-	
+
 	public void
 	addPageGenerator(
 		TrackerWebPageGenerator	generator )
-	{		
+	{
 		try{
 			this_mon.enter();
-		
+
 			generators.add( generator );
-			
+
 		}finally{
-			
+
 			this_mon.exit();
 		}
 	}
-	
+
 	public void
 	removePageGenerator(
 		TrackerWebPageGenerator	generator )
 	{
 		try{
 			this_mon.enter();
-		
+
 			generators.remove( generator );
-			
+
 		}finally{
-			
+
 			this_mon.exit();
 		}
-	}	
+
+	}
+
+	public void
+	destroy()
+	{
+		generators.clear();
+	}
+
 }
