@@ -58,6 +58,7 @@ UpdateCheckInstanceImpl
 	private boolean		cancelled;
 	
 	private boolean		automatic	= true;
+	private boolean		low_noise	= false;
 	
 	protected AEMonitor this_mon 	= new AEMonitor( "UpdateCheckInstance" );
 	
@@ -138,6 +139,19 @@ UpdateCheckInstanceImpl
 		return( automatic );
 	}
 	
+	public void
+	setLowNoise(
+		boolean	a )
+	{
+		low_noise = a;
+	}
+	
+	public boolean
+	isLowNoise()
+	{
+		return( low_noise );
+	}
+	
 	public void
 	start()
 	{
@@ -156,6 +170,8 @@ UpdateCheckInstanceImpl
 							
 						}catch( Throwable e ){
 							
+							checker.reportProgress( "Update check failed: " + Debug.getNestedExceptionMessage( e ));
+							
 							checker.failed();
 						}
 					}
