@@ -35,6 +35,8 @@ import org.gudy.azureus2.plugins.update.*;
 
 import org.gudy.azureus2.plugins.utils.resourcedownloader.*;
 
+import com.aelitis.azureus.core.util.CopyOnWriteList;
+
 public class 
 UpdateImpl 
 	implements Update
@@ -42,14 +44,18 @@ UpdateImpl
 	private UpdateCheckInstanceImpl	instance;
 	private String					name;
 	private String[]				description;
+	private String					relative_url_base	= "";
 	private String					new_version;
 	private ResourceDownloader[]	downloaders;
 	private boolean					mandatory;
 	private int						restart_required;
+	private String 					description_url;
 
 	private Object					user_object;
 	
-	private List					listeners = new ArrayList();
+	private CopyOnWriteList			listeners = new CopyOnWriteList();
+	private volatile boolean		cancelled;
+	private volatile boolean		complete;
 	
 	protected
 	UpdateImpl(
@@ -107,6 +113,33 @@ UpdateImpl
 		return( description );
 	}
 	
+	public String
+	getRelativeURLBase()
+	{
+		return( relative_url_base );
+	}
+	
+	public void
+	setRelativeURLBase(
+		String	base )
+	{
+		relative_url_base = base;
+	}
+	
+	// @see org.gudy.azureus2.plugins.update.Update#getDesciptionURL()
+	public String
+	getDesciptionURL()
+	{
+		return description_url;
+	}
+	
+	// @see org.gudy.azureus2.plugins.update.Update#setDescriptionURL(java.lang.String)
+	public void
+	setDescriptionURL(String url)
+	{
+		description_url = url;
+	}
+	
 	public String
 	getNewVersion()
 	{
@@ -154,6 +187,8 @@ UpdateImpl
 	public void
 	cancel()
 	{
+		cancelled = true;
+		
 		for (int i=0;i<downloaders.length;i++){
 			
 			try{
@@ -164,15 +199,13 @@ UpdateImpl
 				Debug.printStackTrace( e );
 			}
 		}
-	}
-	
-	public void
-	complete()
-	{
-		for (int i=0;i<listeners.size();i++){
+		
+		Iterator it = listeners.iterator();
+		
+		while( it.hasNext()){
 			
 			try{
-				((UpdateListener)listeners.get(i)).complete( this );
+				((UpdateListener)it.next()).cancelled( this );
 				
 			}catch( Throwable e ){
 				
@@ -181,6 +214,37 @@ UpdateImpl
 		}
 	}
 	
+	public void
+	complete()
+	{
+		complete	= true;
+		
+		Iterator it = listeners.iterator();
+		
+		for (int i=0;i<listeners.size();i++){
+			
+			try{
+				((UpdateListener)it.next()).complete( this );
+				
+			}catch( Throwable e ){
+				
+				Debug.printStackTrace(e);
+			}
+		}
+	}
+	
+	public boolean
+	isCancelled()
+	{
+		return( cancelled );
+	}
+	
+	public boolean
+	isComplete()
+	{
+		return( complete );
+	}
+	
 	public Object
 	getDecision(
 		int			decision_type,
@@ -207,6 +271,15 @@ UpdateImpl
 		UpdateListener	l )
 	{
 		listeners.add( l );
+		
+		if ( cancelled ){
+			
+			l.cancelled( this );
+			
+		}else if ( complete ){
+			
+			l.complete( this );
+		}
 	}
 	
 	public void
