@@ -35,6 +35,7 @@ import org.gudy.azureus2.platform.PlatformManagerFactory;
 import org.gudy.azureus2.plugins.update.*;
 
 import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.logging.*;
 
 public class 
@@ -64,6 +65,8 @@ UpdateInstallerImpl
 				
 				boolean	found_failure = false;
 				
+				String	files = "";
+				
 				for (int i=0;i<dirs.length;i++){
 					
 					File	dir = dirs[i];
@@ -74,14 +77,23 @@ UpdateInstallerImpl
 						
 						found_failure	= true;
 						
+						File[] x = dir.listFiles();
+						
+						if ( x != null ){
+							
+							for (int j=0;j<x.length;j++){
+								
+								files += (files.length()==0?"":",") + x[j].getName();
+							}
+						}
+						
 						FileUtil.recursiveDelete( dir );
 					}
 				}
 				
 				if ( found_failure ){
 					Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_ERROR,
-							"Installation of at least one component failed"
-									+ " - see 'update.log' for details"));
+							MessageText.getString("Alert.failed.update", new String[]{ files })));
 				}
 			}
 		}catch( Throwable e ){
@@ -195,15 +207,28 @@ UpdateInstallerImpl
 			
 			File	parent = to_f.getParentFile();
 			
+			if ( parent != null && !parent.exists()){
+				
+				parent.mkdirs();
+			}
+			
 			if ( parent != null ){
 				
 				// we're going to need write access to the parent, let's try
 				
 				if ( !parent.canWrite()){
-					Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_WARNING,
+					
+						// Vista install process goes through permissions elevation process
+						// so don't warn about lack of write permissions
+					
+					if ( !Constants.isWindowsVista ){
+						
+						Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_WARNING,
 							"The location '" + parent.toString()
 									+ "' isn't writable, this update will probably fail."
-									+ " Check permissions and rety the update"));
+									+ " Check permissions and retry the update"));
+						
+					}
 				}
 			}
 							
