@@ -201,7 +201,18 @@ SimpleXMLParserDocumentImpl
 	public void
 	print()
 	{
-		root_node.print( "" );
+		PrintWriter	pw = new PrintWriter( System.out );
+		
+		print( pw );
+		
+		pw.flush();
+	}
+	
+	public void
+	print(
+		PrintWriter	pw )
+	{
+		root_node.print( pw, "" );
 	}
 	
 		// idea is to flatten out any unwanted structure. We just want the resultant
@@ -241,10 +252,10 @@ SimpleXMLParserDocumentImpl
 	
     private static class MyErrorHandler implements ErrorHandler {
         /** Error handler output goes here */
-        private PrintWriter out;
+        //private PrintWriter out;
 
         MyErrorHandler(PrintWriter out) {
-            this.out = out;
+            //this.out = out;
         }
 
         /**
