@@ -32,6 +32,7 @@ import java.util.Map;
 import org.gudy.azureus2.plugins.disk.DiskManager;
 import org.gudy.azureus2.plugins.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.plugins.download.*;
+import org.gudy.azureus2.plugins.download.savelocation.*;
 import org.gudy.azureus2.plugins.download.session.SessionAuthenticator;
 import org.gudy.azureus2.plugins.peers.PeerManager;
 import org.gudy.azureus2.plugins.torrent.*;
@@ -773,6 +774,14 @@ RPDownload
 
   public void setUploadRateLimitBytesPerSecond( int max_rate_bps ) {  notSupported();  }
   
+	public int getDownloadRateLimitBytesPerSecond() {
+	   notSupported();
+	    return 0;
+  	}
+
+  	public void setDownloadRateLimitBytesPerSecond( int max_rate_bps ) {
+		notSupported();
+  	}
   	
   	public int
 	getMaximumDownloadKBPerSecond()
@@ -870,6 +879,10 @@ RPDownload
   	{
  		notSupported();
   	}
+ 	
+ 	public void moveDataFiles(File new_parent_dir, String new_name) throws DownloadException {
+ 		notSupported();
+ 	}
   	
   	public void
   	moveTorrentFile(
@@ -970,4 +983,27 @@ RPDownload
 	public void setListAttribute(TorrentAttribute ta, String[] value) {notSupported();}
 	public void setLongAttribute(TorrentAttribute ta, long value) {notSupported();}
 	public void setFlag(long flag, boolean set) {notSupported();}
+	
+	public void addAttributeListener(DownloadAttributeListener l, TorrentAttribute a, int e) {notSupported();}
+	public void removeAttributeListener(DownloadAttributeListener l, TorrentAttribute a, int e) {notSupported();}
+	
+	public void addCompletionListener(DownloadCompletionListener l) {notSupported();}
+	public void removeCompletionListener(DownloadCompletionListener l) {notSupported();}
+	
+	public boolean isRemoved() {notSupported();	return false; }
+	public boolean canMoveDataFiles() {notSupported(); return false;}
+	public SaveLocationChange calculateDefaultDownloadLocation() {notSupported(); return null;}
+	
+	public Object getUserData(Object key) {
+		notSupported();
+		return null;
+	}
+	
+	public void setUserData(Object key, Object data) {
+		notSupported();
+	}
+	
+	public void startDownload(boolean force) {notSupported();}
+	public void stopDownload() {notSupported();}
+	public void changeLocation(SaveLocationChange slc) {notSupported();}
 }
\ No newline at end of file
