@@ -163,6 +163,15 @@ RPDownloadStats
 		return( bLive ? downloadCompletedLive : downloadCompletedStored );
 	}
 
+	
+	public int
+	getCheckingDoneInThousandNotation()
+	{
+		notSupported();
+		
+		return( 0 );
+	}
+
 	public long
 	getDownloaded()
 	{
@@ -175,6 +184,14 @@ RPDownloadStats
 		return( uploaded );
 	}
 	
+	public long
+	getRemaining()
+	{
+		notSupported();
+		
+		return( 0 );
+	}
+	
 	public long
 	getDiscarded()
 	{
@@ -259,6 +276,20 @@ RPDownloadStats
 		 return ( 0 );
   }
   
+	public long
+	getSecondsSinceLastDownload()
+	{
+		notSupported();
+		return ( 0 );	
+	}
+	
+	public long
+	getSecondsSinceLastUpload()
+	{
+		notSupported();
+		return ( 0 );	
+	}
+	
 	public int
 	getHealth()
 	{
