@@ -39,6 +39,7 @@ import org.gudy.azureus2.plugins.utils.resourcedownloader.*;
 import org.gudy.azureus2.pluginsimpl.update.sf.*;
 import org.gudy.azureus2.pluginsimpl.local.utils.resourcedownloader.*;
 
+import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.html.*;
 import org.gudy.azureus2.core3.util.AEMonitor;
 import org.gudy.azureus2.core3.util.Constants;
@@ -47,14 +48,37 @@ import org.gudy.azureus2.core3.util.SystemProperties;
 import org.gudy.azureus2.core3.util.SystemTime;
 import org.gudy.azureus2.core3.logging.*;
 
+import com.aelitis.azureus.core.versioncheck.VersionCheckClient;
+
 public class 
 SFPluginDetailsLoaderImpl 
 	implements SFPluginDetailsLoader, ResourceDownloaderListener
 {
 	private static final LogIDs LOGID = LogIDs.CORE;
 
-	private static final String	site_prefix = "http://azureus.sourceforge.net/";
+	private static final String	site_prefix_default = Constants.SF_WEB_SITE;
 	
+	private static String site_prefix;
+	
+	static{
+		try{
+			Map data = VersionCheckClient.getSingleton().getVersionCheckInfo( VersionCheckClient.REASON_PLUGIN_UPDATE );
+			
+			byte[] b_sp = (byte[])data.get("plugin_update_url");
+			
+			if ( b_sp == null ){
+				
+				site_prefix = site_prefix_default;
+				
+			}else{
+				
+				site_prefix = new String( b_sp );
+			}
+		}catch( Throwable e ){
+			
+			site_prefix = site_prefix_default;
+		}
+	}
 	private static String	base_url_params;
 	
 	static{
@@ -66,6 +90,10 @@ SFPluginDetailsLoaderImpl
 			
 			base_url_params += "&arch=" + URLEncoder.encode(System.getProperty( "os.arch"),"UTF-8" );
 			
+			base_url_params += "&ui=" + URLEncoder.encode(COConfigurationManager.getStringParameter("ui"),"UTF-8" );
+			
+			base_url_params += "&java=" + URLEncoder.encode(System.getProperty( "java.version" ),"UTF-8" );
+			
 		}catch( Throwable e ){
 			
 			Debug.printStackTrace(e);
@@ -130,6 +158,12 @@ SFPluginDetailsLoaderImpl
 		reset();
 	}
 	
+	protected String
+	getRelativeURLBase()
+	{
+		return( site_prefix );
+	}
+	
 	protected void
 	loadPluginList()
 	
@@ -144,7 +178,11 @@ SFPluginDetailsLoaderImpl
 			
 			Properties	details = new Properties();
 			
-			details.load( dl.download());
+			InputStream is = dl.download();
+			
+			details.load( is );
+			
+			is.close();
 			
 			Iterator it = details.keySet().iterator();
 			
@@ -175,20 +213,25 @@ SFPluginDetailsLoaderImpl
 					}
 				}
 				
-				String	version 	= (String)bits.get(0);
-				String	cvs_version = (String)bits.get(1);
-				String	name		= (String)bits.get(2);
-				String	category	= "";
-				
-				if ( bits.size() > 3 ){
-					
-					category = (String)bits.get(3);
+				if (bits.size() < 3) {
+					Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR,
+							"SF loadPluginList failed for plugin '" + plugin_id
+									+ "'.  Details array is " + bits.size() + " (3 min)"));
+				} else {
+					String version = (String) bits.get(0);
+					String cvs_version = (String) bits.get(1);
+					String name = (String) bits.get(2);
+					String category = "";
+
+					if (bits.size() > 3) {
+						category = (String) bits.get(3);
+					}
+
+					plugin_ids.add(plugin_id);
+
+					plugin_map.put(plugin_id.toLowerCase(), new SFPluginDetailsImpl(this,
+							plugin_id, version, cvs_version, name, category));
 				}
-				
-				plugin_ids.add( plugin_id );
-				
-				plugin_map.put(plugin_id.toLowerCase(), 
-				               new SFPluginDetailsImpl( this, plugin_id, version, cvs_version, name, category ));
 			}
 			
 			plugin_ids_loaded	= true;
@@ -304,6 +347,12 @@ SFPluginDetailsLoaderImpl
 						
 						plugin_cvs_download = site_prefix + cvs_dl_links[0];
 					}
+					
+					String info_url = null;
+					if (rows[9].getCells().length > 1) {
+						info_url = rows[9].getCells()[1].getContent();
+					}
+
 
 					// System.out.println( "got plugin:" + plugin_name + "/" + plugin_version + "/" + plugin_download + "/" + plugin_auth );
 					
@@ -312,7 +361,8 @@ SFPluginDetailsLoaderImpl
 									plugin_auth,
 									plugin_cvs_download,
 									rows[6].getCells()[0].getContent(),
-									rows[9].getCells()[0].getContent());
+									rows[9].getCells()[0].getContent(),
+									info_url);
 					
 					return( true );
 				}
@@ -439,6 +489,13 @@ SFPluginDetailsLoaderImpl
 	{
 	}
 	
+	public void
+	reportAmountComplete(
+		ResourceDownloader	downloader,
+		long				amount )
+	{
+	}
+	
 	public void
 	reportActivity(
 		ResourceDownloader	downloader,
