@@ -20,23 +20,33 @@
  */
 package org.gudy.azureus2.ui.swt;
 
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.Map;
+import java.util.*;
+import java.util.List;
 
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.events.SelectionEvent;
+import org.eclipse.swt.events.SelectionListener;
 import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.layout.*;
 import org.eclipse.swt.widgets.*;
 
+import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.Constants;
-import org.gudy.azureus2.ui.swt.components.*;
+import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.ui.swt.components.BufferedToolItem;
+import org.gudy.azureus2.ui.swt.shells.MessageBoxShell;
+
+import com.aelitis.azureus.ui.UIFunctions;
+import com.aelitis.azureus.ui.UIFunctionsManager;
 
 /**
  * @author Olivier
  *
  */
 public class IconBar {
+	private final boolean OVERRIDE_SHOW_UISWITCHER = System.getProperty(
+			"ui.toolbar.uiswitcher", "0").equals("1");
   
   CoolBar coolBar;
   Composite parent;    
@@ -44,12 +54,14 @@ public class IconBar {
   
   IconBarEnabler currentEnabler;
 	private Composite cIconBar;
+	
+	private static List listeners = new ArrayList(0);
   
   public IconBar(Composite parent) {
     this.parent = parent;
     cIconBar = new Composite(parent, SWT.NONE);
     
-    GridLayout layout = new GridLayout();
+    GridLayout layout = new GridLayout(2, false);
     layout.marginHeight = 0;
     layout.marginWidth = 0;
     layout.horizontalSpacing = 0;
@@ -64,11 +76,45 @@ public class IconBar {
     this.coolBar.setLocked(true);
     
     coolBar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
-    
 
-    Label separator = new Label(cIconBar, SWT.SEPARATOR | SWT.HORIZONTAL);
-    separator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
+    // We could setup a listener on the parameter and dynamically remove/add
+    // ui switcher button, but it's not worth the effort
+		boolean enableUISwitcher = OVERRIDE_SHOW_UISWITCHER
+				|| COConfigurationManager.getBooleanParameter("ui.toolbar.uiswitcher")
+				|| COConfigurationManager.getBooleanParameter("ui.asked", false);
     
+		if (enableUISwitcher) {
+			ToolBar tbSwitch = new ToolBar(cIconBar, SWT.FLAT);
+			GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
+			tbSwitch.setLayoutData(gridData);
+			ToolItem tiSwitch = new ToolItem(tbSwitch, SWT.PUSH);
+			tiSwitch.setImage(ImageRepository.getImage("cb_switch"));
+			Messages.setLanguageText(tiSwitch, "iconBar.switch.tooltip", true);
+			tiSwitch.addSelectionListener(new SelectionListener() {
+				public void widgetSelected(SelectionEvent e) {
+					String uiOld = COConfigurationManager.getStringParameter("ui");
+					String uiNew = UISwitcherUtil.openSwitcherWindow(true);
+					if (!uiOld.equals(uiNew)) {
+  					int result = MessageBoxShell.open(IconBar.this.parent.getShell(),
+  							MessageText.getString("dialog.uiswitcher.restart.title"),
+  							MessageText.getString("dialog.uiswitcher.restart.text"),
+  							new String[] {
+  								MessageText.getString("UpdateWindow.restart"),
+  								MessageText.getString("UpdateWindow.restartLater"),
+  							}, 0);
+  					if (result == 0) {
+    					UIFunctions uif = UIFunctionsManager.getUIFunctions();
+    					if (uif != null) {
+    						uif.dispose(true, false);
+    					}
+  					}
+					}
+				}
+
+				public void widgetDefaultSelected(SelectionEvent e) {
+				}
+			});
+		}
 	}
 
 	public void setEnabled(String itemKey,boolean enabled) {
@@ -88,6 +134,10 @@ public class IconBar {
     refreshEnableItems();
   }
   
+  public IconBarEnabler getCurrentEnabler() {
+  	return this.currentEnabler;
+  }
+  
   private void refreshEnableItems() {
     Iterator iter = itemKeyToControl.keySet().iterator();
     while(iter.hasNext()) {
@@ -123,6 +173,10 @@ public class IconBar {
     return bufferedToolItem;
   }  
   
+  public void addITemKeyToControl(String key, BufferedToolItem item) {
+  	itemKeyToControl.put(key, item);
+  }
+  
   private void initBar() {
     //The File Menu
     CoolItem coolItem = new CoolItem(coolBar,SWT.NULL);
@@ -138,7 +192,7 @@ public class IconBar {
     coolItem.setMinimumSize(p.x,p.y);
     
     
-    coolItem = new CoolItem(coolBar,SWT.NULL); 
+    coolItem = new CoolItem(coolBar,SWT.NULL);
     toolBar = new ToolBar(coolBar,SWT.FLAT);    
     createBufferedToolItem(toolBar,SWT.PUSH,"top","cb_top","iconBar.top.tooltip");
     createBufferedToolItem(toolBar,SWT.PUSH,"up","cb_up","iconBar.up.tooltip");
@@ -151,12 +205,25 @@ public class IconBar {
     createBufferedToolItem(toolBar,SWT.PUSH,"start","cb_start","iconBar.start.tooltip");
     createBufferedToolItem(toolBar,SWT.PUSH,"stop","cb_stop","iconBar.stop.tooltip");
     createBufferedToolItem(toolBar,SWT.PUSH,"remove","cb_remove","iconBar.remove.tooltip");
+    
 
+    for (Iterator iter = listeners.iterator(); iter.hasNext();) {
+    	try {
+    		IconBarListener l = (IconBarListener) iter.next();
+    		l.iconBarInitialized(coolBar, this);
+    	} catch (Exception e) {
+    		Debug.out(e);
+    	}
+		}
+    
     toolBar.pack();
     p = toolBar.getSize();
     coolItem.setControl(toolBar);
+    if (Constants.isOSX) {
+    	p.x += 12;
+    }
     coolItem.setSize(p.x,p.y);
-    coolItem.setMinimumSize(p.x,p.y);    
+  	coolItem.setMinimumSize(p.x,p.y);
   }
   
   public void setLayoutData(Object layoutData) {
@@ -196,4 +263,13 @@ public class IconBar {
 		currentEnabler = null;
 	}
 
+	public static void addListener(IconBarListener l) {
+		if (!listeners.contains(l)) {
+			listeners.add(l);
+		}
+	}
+	
+	public static interface IconBarListener {
+		public void iconBarInitialized(CoolBar coolBar, IconBar ib);
+	}
 }
