@@ -23,6 +23,8 @@ package org.gudy.azureus2.ui.swt.components.graphics;
 import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.graphics.Rectangle;
+
+import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.ui.swt.mainwindow.Colors;
 
 /**
@@ -37,17 +39,25 @@ public class ScaledGraphic extends BackGroundGraphic {
   protected Image bufferScale;
   protected int lastMax;
   
+  private int update_divider_width = 0;
   
   public ScaledGraphic(Scale scale,ValueFormater formater) {
     this.scale = scale;
     this.formater = formater;
   }
   
+  public void setUpdateDividerWidth(int width) {
+	  this.update_divider_width = width;
+  }
+  
   protected void drawScale(boolean sizeChanged) {
-    if(drawCanvas == null || drawCanvas.isDisposed())
+    if(drawCanvas == null || drawCanvas.isDisposed() || !drawCanvas.isVisible())
       return;
    
     drawBackGround(sizeChanged);
+  	if (bufferBackground == null || bufferBackground.isDisposed()) {
+  		return;
+  	}
    
     boolean scaleChanged = lastMax != scale.getMax();
     
@@ -62,6 +72,7 @@ public class ScaledGraphic extends BackGroundGraphic {
       bufferScale = new Image(drawCanvas.getDisplay(),bounds);
       
       GC gcBuffer = new GC(bufferScale);
+      try {
       gcBuffer.drawImage(bufferBackground,0,0);
       gcBuffer.setForeground(Colors.black);
       //gcImage.setBackground(null);
@@ -72,7 +83,17 @@ public class ScaledGraphic extends BackGroundGraphic {
         gcBuffer.drawLine(1,height,bounds.width - 70 ,height);
         gcBuffer.drawText(formater.format(levels[i]),bounds.width - 65,height - 12,true);
       }
-      gcBuffer.dispose();
+      if (this.update_divider_width > 0) {
+    	  for (int i=bounds.width - 70; i > 0; i-=this.update_divider_width) {
+    		  gcBuffer.setForeground(Colors.grey);
+    		  gcBuffer.drawLine(i, 0, i, bounds.height);
+    	  }
+      }
+      } catch (Exception e) {
+      	Debug.out(e);
+      } finally {
+      	gcBuffer.dispose();
+      }
     }
   }   
   
