@@ -31,6 +31,7 @@ import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.DirectoryDialog;
 import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Listener;
+import org.eclipse.swt.widgets.Shell;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.ui.swt.ImageRepository;
 
@@ -44,6 +45,8 @@ DirectoryParameter
 	extends Parameter
 {
 	Control[] controls;
+	
+	StringParameter sp;
 	  
 	public 
 	DirectoryParameter(
@@ -51,32 +54,27 @@ DirectoryParameter
 		String			name,
 		String			defaultValue )
 	{  
+  	super(name);
 	  	controls = new Control[2];
 	           	    
-	    final org.gudy.azureus2.ui.swt.config.StringParameter sp =
-	    	new org.gudy.azureus2.ui.swt.config.StringParameter(
-	    	    pluginGroup,
-	    	    name,
-				defaultValue );
+	    sp = new StringParameter(pluginGroup, name, defaultValue);
 	    
 	    controls[0] = sp.getControl();
 	    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
 	    controls[0].setLayoutData(gridData);
 	    
 	    Button browse = new Button(pluginGroup, SWT.PUSH);
-	    Image imgOpenFolder = ImageRepository.getImage("openFolderButton");
+	    Image imgOpenFolder = ImageRepository.getImage(getBrowseImageResource());
 	    browse.setImage(imgOpenFolder);
 	    imgOpenFolder.setBackground(browse.getBackground());
 	    browse.setToolTipText(MessageText.getString("ConfigView.button.browse"));
 
 	    browse.addListener(SWT.Selection, new Listener() {
 	      public void handleEvent(Event event) {
-	        DirectoryDialog dialog = new DirectoryDialog(pluginGroup.getShell(), SWT.APPLICATION_MODAL);
-	        dialog.setFilterPath(sp.getValue());        
-	        String path = dialog.open();
-	        if (path != null) {
-	          sp.setValue(path);
-	        }
+	    	  String path = DirectoryParameter.this.openDialog(pluginGroup.getShell(), sp.getValue());
+	    	  if (path != null) {
+	    		  sp.setValue(path);
+	    	  }
 	      }
 	    });
 	    controls[1] = browse;
@@ -99,4 +97,20 @@ DirectoryParameter
 	{
 	    return controls;
 	}
+	
+	protected String getBrowseImageResource() {
+		return "openFolderButton";
+	}
+	
+	protected String openDialog(Shell shell, String old_value) {
+        DirectoryDialog dialog = new DirectoryDialog(shell, SWT.APPLICATION_MODAL);
+        dialog.setFilterPath(old_value);        
+        return dialog.open();
+	}
+
+  public void setValue(Object value) {
+  	if (value instanceof String) {
+  		sp.setValue((String)value);
+  	}
+  }
 }
