@@ -21,30 +21,41 @@
  */
 package org.gudy.azureus2.ui.swt.mainwindow;
 
-import com.aelitis.azureus.core.*;
-import com.aelitis.azureus.ui.IUIIntializer;
+import java.lang.reflect.Constructor;
+import java.util.ArrayList;
+import java.util.Iterator;
 
 import org.eclipse.swt.widgets.Display;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.global.GlobalManager;
 import org.gudy.azureus2.core3.internat.MessageText;
+import org.gudy.azureus2.core3.ipfilter.IpFilterManager;
 import org.gudy.azureus2.core3.logging.*;
-import org.gudy.azureus2.core3.util.AEMonitor;
-import org.gudy.azureus2.core3.util.AERunnable;
-import org.gudy.azureus2.core3.util.AESemaphore;
-import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.plugins.PluginEvent;
+import org.gudy.azureus2.plugins.utils.DelayedTask;
+import org.gudy.azureus2.pluginsimpl.local.utils.UtilitiesImpl;
 import org.gudy.azureus2.ui.common.util.UserAlerts;
 import org.gudy.azureus2.ui.swt.*;
 import org.gudy.azureus2.ui.swt.auth.AuthenticatorWindow;
 import org.gudy.azureus2.ui.swt.auth.CertificateTrustWindow;
+import org.gudy.azureus2.ui.swt.auth.CryptoWindow;
 import org.gudy.azureus2.ui.swt.networks.SWTNetworkSelection;
 import org.gudy.azureus2.ui.swt.pluginsinstaller.InstallPluginWizard;
 import org.gudy.azureus2.ui.swt.progress.ProgressWindow;
 import org.gudy.azureus2.ui.swt.update.UpdateMonitor;
+import org.gudy.azureus2.ui.swt.updater2.PreUpdateChecker;
 import org.gudy.azureus2.ui.swt.updater2.SWTUpdateChecker;
 
-import java.util.ArrayList;
-import java.util.Iterator;
+import com.aelitis.azureus.core.*;
+import com.aelitis.azureus.core.util.CopyOnWriteList;
+import com.aelitis.azureus.launcher.Launcher;
+import com.aelitis.azureus.ui.IUIIntializer;
+import com.aelitis.azureus.ui.InitializerListener;
+import com.aelitis.azureus.ui.swt.UIFunctionsManagerSWT;
+import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
+import com.aelitis.azureus.ui.swt.utils.UIMagnetHandler;
 
 /**
  * this class initiatize all GUI and Core components which are :
@@ -63,45 +74,38 @@ Initializer
   private GlobalManager 	gm;
   private StartServer 		startServer;
   
-  private ArrayList listeners;
+  private CopyOnWriteList listeners = new CopyOnWriteList();
+  
   private AEMonitor	listeners_mon	= new AEMonitor( "Initializer:l" );
 
   private String[] args;
   
+  private AESemaphore semFilterLoader = new AESemaphore("filter loader");
+  
+  private AESemaphore init_task = new AESemaphore("delayed init");
+  
   public 
   Initializer(
   		final AzureusCore		_azureus_core,
   		StartServer 			_server,
 		String[] 				_args ) 
-  {
-    listeners = new ArrayList();
-    
+  {   
     azureus_core	= _azureus_core;
     startServer 	= _server;
     args 			= _args;
     
-    	// these lifecycle actions are initially to handle plugin-initiated stops and restarts
-    
-    azureus_core.addLifecycleListener(
-			new AzureusCoreLifecycleAdapter()
-			{
-				public boolean
-				stopRequested(
-					AzureusCore		_core )
-				
-					throws AzureusCoreException
-				{
-					return( handleStopRestart(false));
+    Thread filterLoaderThread = new AEThread("filter loader", true) {
+			public void runSupport() {
+				try {
+					azureus_core.getIpFilterManager().getIPFilter();
+				} finally {
+					semFilterLoader.releaseForever();
 				}
-				
-				public boolean
-				restartRequested(
-					final AzureusCore		core )
-				{
-					return( handleStopRestart(true));
-				}
-			});
-    
+			}
+		};
+		filterLoaderThread.setPriority(Thread.MIN_PRIORITY);
+		filterLoaderThread.start();
+
     try {
       SWTThread.createInstance(this);
     } catch(SWTThreadAlreadyInstanciatedException e) {
@@ -109,60 +113,73 @@ Initializer
     }
   }  
 
-  public boolean
+  public static boolean
   handleStopRestart(
   	final boolean	restart )
   {
-  	if (Utils.isThisThreadSWT())
-  		return( MainWindow.getWindow().dispose(restart,true));
-		
-		final AESemaphore			sem 	= new AESemaphore("SWTInit::stopRestartRequest");
-		final boolean[]				ok	 	= {false};
-		
-		try{
-			Utils.execSWTThread(
-					new AERunnable()
-					{
-						public void
-						runSupport()
-						{
-							try{
-								ok[0] = MainWindow.getWindow().dispose(restart,true);
-								
-							}catch( Throwable e ){
-								
-								Debug.printStackTrace(e);
-							
-							}finally{
-								
-								sem.release();
-							}
-						}
-					});
-		
-		}catch( Throwable e ){
-			
-			Debug.printStackTrace(e);
-			
-			sem.release();
+		UIFunctionsSWT functionsSWT = UIFunctionsManagerSWT.getUIFunctionsSWT();
+		if (functionsSWT != null) {
+			return functionsSWT.dispose(restart, true);
 		}
-	
-		sem.reserve();
-	
-		return( ok[0] );
-  }
+
+		return false;
+	}
 	
 
   public void 
   run() 
   {
+
   	try{
+  		String uiMode = UISwitcherUtil.openSwitcherWindow(false);
+  		
+  		if (uiMode.equals("az3")) {
+  			try {
+  				final Class az3Class = Class.forName("com.aelitis.azureus.ui.swt.Initializer");
+
+  				final Constructor constructor = az3Class.getConstructor(new Class[] {
+  						AzureusCore.class,
+  						Boolean.TYPE,
+  						String[].class
+  				});
+
+  				IUIIntializer initializer = (IUIIntializer) constructor.newInstance(new Object[] {
+  						azureus_core,
+  						new Boolean(false),
+  						args
+  				});
+
+  				initializer.run();
+  				return;
+  			} catch (Throwable t) {
+  				// use print stack trace because we don't want to introduce logger
+  				t.printStackTrace();
+  				// Ignore and use AZ2
+  			}
+  		}
+  		// else AZ2UI
+  		
+		DelayedTask delayed_task = UtilitiesImpl.addDelayedTask("SWT Initialisation", new Runnable()
+				{
+					public void
+					run()
+					{
+						init_task.reserve();
+					}
+				});
+
+		delayed_task.queue();
+			  
+
+		
   		// initialise the SWT locale util
 	  	
 	    new LocaleUtilSWT( azureus_core );
 
 			Display display = SWTThread.getInstance().getDisplay();
 
+			new UIMagnetHandler(azureus_core);
+
 			UIConfigDefaultsSWT.initialize();
 			
 	    //The splash window, if displayed will need some images. 
@@ -173,7 +190,7 @@ Initializer
 	      SplashWindow.create(display,this);
 	    }
 	    	    
-	    setNbTasks(6);
+	    setNbTasks(7);
 	    
 	    nextTask(); 
 	    reportCurrentTaskByKey("splash.firstMessageNoI18N");
@@ -197,6 +214,7 @@ Initializer
 	    new SWTNetworkSelection();
 	    
 	    new AuthenticatorWindow();
+	    new CryptoWindow();
 	    
 	    new CertificateTrustWindow();
 
@@ -212,99 +230,119 @@ Initializer
 	    
 	    azureus_core.addListener( this );
 	    
-	    azureus_core.addLifecycleListener(
-	    	new AzureusCoreLifecycleAdapter()
-			{
-	    		public void
-				componentCreated(
-					AzureusCore					core,
-					AzureusCoreComponent		comp )
-	    		{
-	    			if ( comp instanceof GlobalManager ){
-	    				
-	    				gm	= (GlobalManager)comp;
+	    azureus_core.addLifecycleListener(new AzureusCoreLifecycleAdapter() {
+				public void componentCreated(AzureusCore core, AzureusCoreComponent comp) {
+					if (comp instanceof GlobalManager) {
 
-	    				nextTask();	    
-	    				reportCurrentTask(MessageText.getString("splash.initializePlugins"));
-	    			}
-	    		}
-	    		
-	    		public void
+						gm = (GlobalManager) comp;
+
+						nextTask();
+						reportCurrentTask(MessageText.getString("splash.initializePlugins"));
+					}
+				}
+
+				public void 
 				started(
-					AzureusCore		core )
-	    		{	    		      	
-	    			if (gm == null)
-	    				return;
+					final AzureusCore core) 
+				{
+					boolean	main_window_will_report_complete = false;
+					
+					try{
+						if (gm == null)
+							return;
+	
+						new UserAlerts(gm);
+	
+						nextTask();
+						IpFilterManager ipFilterManager = azureus_core.getIpFilterManager();
+						if (ipFilterManager != null) {
+							String s = MessageText.getString("splash.loadIpFilters");
+	  					do {
+	  						reportCurrentTask(s);
+	  						s += ".";
+	  					} while (!semFilterLoader.reserve(3000));
+						}
+						
+						nextTask();
+						reportCurrentTaskByKey("splash.initializeGui");
+	
+						Colors.getInstance();
+	
+						Cursors.init();
+	
+						main_window_will_report_complete = true;
+						
+						// main window controls further progress now
+						new MainWindow(core, Initializer.this,
+								logEvents);
+	
+						if (finalLogListener != null)
+							Logger.removeListener(finalLogListener);
+	
+						SWTUpdateChecker.initialize();
+	
+						PreUpdateChecker.initialize( core, COConfigurationManager.getStringParameter("ui"));
+	
+						UpdateMonitor.getSingleton(core); // setup the update monitor
+	
+						//Tell listeners that all is initialized :
+						Alerts.initComplete();
+	
+	
+						
+	
+						//Finally, open torrents if any.
+						for (int i = 0; i < args.length; i++) {
+	
+							try {
+								TorrentOpener.openTorrent(args[i]);
+	
+							} catch (Throwable e) {
+	
+								Debug.printStackTrace(e);
+							}
+						}
+					}finally{
+					
+						if ( !main_window_will_report_complete ){
+							init_task.release();
+						}
+					}
+				}
 
-    		    new UserAlerts(gm);
-    		    
-    		    nextTask();	    
-    		    reportCurrentTaskByKey("splash.initializeGui");
-    		    
-    		    new Colors();
-    		    
-    		    Cursors.init();
-    		    
-    		    MainWindow main_window = new MainWindow(core, Initializer.this,
-							logEvents);
-    		    
-    		    if (finalLogListener != null)
-    		    	Logger.removeListener(finalLogListener);
+				public void stopping(AzureusCore core) {
+					Alerts.stopInitiated();
+				}
 
-    		    nextTask();
-    		    
-	    		    reportCurrentTaskByKey( "splash.openViews");
+				public void stopped(AzureusCore core) {
+				}
 
-	    		    SWTUpdateChecker.initialize();
-	    		    
-	    		    UpdateMonitor.getSingleton( core );	// setup the update monitor
-	    			
-	    		    //Tell listeners that all is initialized :
-	    		    
-	    		    Alerts.initComplete();
-	    		    
-	    		    // 7th task: just for triggering a > 100% completion
-	    		    nextTask();
-	    		    
-	    		    
-	    		    //Finally, open torrents if any.
-	    		    
-	    		    for (int i=0;i<args.length;i++){
-	    		    	
-	    		    	try{
-	    		    		TorrentOpener.openTorrent(args[i]);
-	    		    		
-	    	        	}catch( Throwable e ){
-	    	        		
-	    	        		Debug.printStackTrace(e);
-	    	        	}	
-	    		    }
-	    		}
-	    		
-	    		public void
-	    		stopping(
-	    			AzureusCore		core )
-	    		{
-	    			Alerts.stopInitiated();
-	    		}
-	    		
-	    		public void
-	    		stopped(
-	    			AzureusCore		core )
-	    		{
-	    		}
-	    		
-	    		public boolean
-	    		syncInvokeRequired()
-	    		{
-	    			return( true );
-	    		}
+				public boolean syncInvokeRequired() {
+					return (true);
+				}
+
+				public boolean
+				requiresPluginInitCompleteBeforeStartedEvent()
+				{
+					return( false );
+				}
+				
+				public boolean stopRequested(AzureusCore _core)
+
+				throws AzureusCoreException {
+					return (handleStopRestart(false));
+				}
+
+				public boolean restartRequested(final AzureusCore core) {
+					return (handleStopRestart(true));
+				}
 			});
 	    
 	    azureus_core.start();
 
   	}catch( Throwable e ){
   		Logger.log(new LogEvent(LOGID, "Initialization fails:", e));
+  		init_task.release();
   	} 
   }
   
@@ -356,8 +394,15 @@ Initializer
      
 	    Iterator iter = listeners.iterator();
 	    while(iter.hasNext()) {
-	    	InitializerListener listener = (InitializerListener) iter.next();
-	      listener.reportCurrentTask(currentTaskString);
+	    	try{
+	    		InitializerListener listener = (InitializerListener) iter.next();
+	      
+	    		listener.reportCurrentTask(currentTaskString);
+	    		
+	    	}catch( Throwable e ){
+	    		
+	    		Debug.printStackTrace( e );
+	    	}
 	    }
     }finally{
     	
@@ -365,6 +410,24 @@ Initializer
     }
   }
   
+  public void
+  initializationComplete()
+  {
+	  azureus_core.getPluginManager().firePluginEvent( PluginEvent.PEV_INITIALISATION_UI_COMPLETES );
+	  
+	  new DelayedEvent( 
+			  "SWTInitComplete:delay",
+			  8500,
+			  new AERunnable()
+			  {
+				  public void
+				  runSupport()
+				  {
+					  init_task.release();
+				  }
+			  });
+  }
+  
   // AzureusCoreListener
   public void reportPercent(int percent) {
   	int overallPercent = overallPercent(percent);
@@ -373,8 +436,15 @@ Initializer
     
 	    Iterator iter = listeners.iterator();
 	    while(iter.hasNext()) {
-	    	InitializerListener listener = (InitializerListener) iter.next();
-	      listener.reportPercent(overallPercent);
+	    	try{
+	    		InitializerListener listener = (InitializerListener) iter.next();
+	      
+	    		listener.reportPercent(overallPercent);
+	    		
+	    	}catch( Throwable e ){
+	    		
+	    		Debug.printStackTrace( e );
+	    	}
 	    }
 
 	    if (overallPercent > 100) {
@@ -402,17 +472,13 @@ Initializer
     }
     
   	try{
-	    if ( startServer != null ){
-	    
-	    	startServer.stopIt();
-	    }
-	    
 	    Cursors.dispose();
 	    
 	    SWTThread.getInstance().terminate();  
 	    
 	}finally{
 	  	
+		try{
 	    if ( azureus_core != null && !close_already_in_progress ){
 
 	    	try{
@@ -431,19 +497,31 @@ Initializer
 	    		Debug.out( e );
 	    	}
 	    }
+		}finally{
+			
+				// do this after closing core to minimise window when the we aren't 
+				// listening and therefore another Azureus start can potentially get
+				// in and screw things up
+			
+		    if ( startServer != null ){
+			    
+		    	startServer.stopIt();
+		    }
+		}
 	}
   }
   
   private int nbTasks = 1;
   private int currentTask = 0;
   private int currentPercent = 0;
+  private int lastTaskPercent = 0;
   
   private void setNbTasks(int _nbTasks) {
     currentTask = 0;
     nbTasks = _nbTasks;
   }
   
-  private void nextTask() {
+  public void nextTask() {
     currentTask++;
     currentPercent = 100 * currentTask / (nbTasks) ;
     //0% done of current task
@@ -451,6 +529,7 @@ Initializer
   }
   
   private int overallPercent(int taskPercent) {
+  	lastTaskPercent = taskPercent;
     //System.out.println("ST percent " + currentPercent + " / " + taskPercent + " : " + (currentPercent + (taskPercent / nbTasks)));
     return currentPercent + taskPercent / nbTasks;
   }
@@ -458,10 +537,26 @@ Initializer
   private void reportCurrentTaskByKey(String key) {
     reportCurrentTask(MessageText.getString(key));
   }
+ 
+  // @see com.aelitis.azureus.ui.IUIIntializer#increaseProgresss()
+  public void increaseProgress() {
+  	if (lastTaskPercent < 100) {
+  		reportPercent(lastTaskPercent + 1);
+  	}
+  }
   
+	public void abortProgress() {
+		currentTask = nbTasks;
+		nextTask();
+		reportPercent(101);
+	}
   
   public static void main(String args[]) 
   {
+	  if(Launcher.checkAndLaunch(Initializer.class, args))
+		  return;
+	  
+  	System.err.println("Shouldn't you be starting with org.gudy.azureus2.ui.swt.Main?");
  	AzureusCore		core = AzureusCoreFactory.create();
 
  	new Initializer( core, null,args);
