@@ -20,22 +20,14 @@
 package org.gudy.azureus2.ui.swt.mainwindow;
 
 import java.text.NumberFormat;
-import java.util.ArrayList;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.CLabel;
-import org.eclipse.swt.custom.StackLayout;
 import org.eclipse.swt.events.MouseAdapter;
 import org.eclipse.swt.events.MouseEvent;
-import org.eclipse.swt.graphics.GC;
-import org.eclipse.swt.graphics.Image;
-import org.eclipse.swt.graphics.Point;
-import org.eclipse.swt.layout.FormAttachment;
-import org.eclipse.swt.layout.FormData;
-import org.eclipse.swt.layout.GridData;
-import org.eclipse.swt.layout.GridLayout;
+import org.eclipse.swt.graphics.*;
+import org.eclipse.swt.layout.*;
 import org.eclipse.swt.widgets.*;
-
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.config.impl.TransferSpeedValidator;
@@ -45,15 +37,13 @@ import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.ipfilter.IpFilter;
 import org.gudy.azureus2.core3.stats.transfer.OverallStats;
 import org.gudy.azureus2.core3.stats.transfer.StatsFactory;
-import org.gudy.azureus2.core3.util.AEMonitor;
-import org.gudy.azureus2.core3.util.AERunnable;
-import org.gudy.azureus2.core3.util.Constants;
-import org.gudy.azureus2.core3.util.DisplayFormatters;
-import org.gudy.azureus2.ui.swt.BlockedIpsWindow;
-import org.gudy.azureus2.ui.swt.ImageRepository;
-import org.gudy.azureus2.ui.swt.Messages;
-import org.gudy.azureus2.ui.swt.Utils;
-import org.gudy.azureus2.ui.swt.update.UpdateProgressWindow;
+import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.plugins.PluginInterface;
+import org.gudy.azureus2.plugins.PluginManager;
+import org.gudy.azureus2.plugins.network.ConnectionManager;
+import org.gudy.azureus2.plugins.ui.config.ConfigSection;
+import org.gudy.azureus2.ui.swt.*;
+import org.gudy.azureus2.ui.swt.progress.*;
 import org.gudy.azureus2.ui.swt.update.UpdateWindow;
 
 import com.aelitis.azureus.core.AzureusCore;
@@ -63,17 +53,13 @@ import com.aelitis.azureus.core.networkmanager.NetworkManager;
 import com.aelitis.azureus.plugins.dht.DHTPlugin;
 import com.aelitis.azureus.ui.UIFunctions;
 import com.aelitis.azureus.ui.UIFunctionsManager;
-
-import org.gudy.azureus2.plugins.PluginInterface;
-import org.gudy.azureus2.plugins.PluginManager;
-import org.gudy.azureus2.plugins.network.ConnectionManager;
-import org.gudy.azureus2.plugins.ui.config.ConfigSection;
-import org.gudy.azureus2.plugins.update.*;
+import com.aelitis.azureus.ui.UIStatusTextClickListener;
 
 /**
  * Moved from MainWindow and GUIUpdater
  */
-public class MainStatusBar {
+public class MainStatusBar
+{
 	/**
 	 * Warning status icon identifier
 	 */
@@ -81,27 +67,17 @@ public class MainStatusBar {
 
 	private AEMonitor this_mon = new AEMonitor("MainStatusBar");
 
-	private ArrayList update_stack = new ArrayList();
-
 	private UpdateWindow updateWindow;
 
 	private Composite statusBar;
 
-	private Composite statusArea;
-
-	private StackLayout layoutStatusArea;
-
 	private CLabel statusText;
 
 	private String statusTextKey = "";
 
 	private String statusImageKey = null;
 
-	private Composite statusUpdate;
-
-	private Label statusUpdateLabel;
-
-	private ProgressBar statusUpdateProgressBar;
+	private AZProgressBar progressBar;
 
 	private CLabel ipBlocked;
 
@@ -115,6 +91,8 @@ public class MainStatusBar {
 
 	private CLabel statusUp;
 
+	private Composite plugin_label_composite;
+
 	private Display display;
 
 	// For Refresh..
@@ -142,6 +120,49 @@ public class MainStatusBar {
 
 	private UIFunctions uiFunctions;
 
+	private UIStatusTextClickListener clickListener;
+
+	//	 final int borderFlag = (Constants.isOSX) ? SWT.SHADOW_NONE : SWT.SHADOW_IN;
+	private static final int borderFlag = SWT.SHADOW_NONE;
+
+	/**
+	 * Just a flag to differentiate az3 from other versions; default status bar text is handled differently between versions.
+	 * Specifically speaking the Vuze UI status text is just empty whereas the Classic UI status text has an icon
+	 * and the application version number.
+	 */
+	private boolean isAZ3 = false;
+
+	/**
+	 * Just a reference to the static <code>ProgressReportingManager</code> to make the code look cleaner instead of
+	 * using <code>ProgressReportingManager.getInstance().xxx()</code> everywhere.
+	 */
+	private ProgressReportingManager PRManager = ProgressReportingManager.getInstance();
+
+	/**
+	 * A <code>GridData</code> for the progress bar; used to dynamically provide .widthHint to the layout manager
+	 */
+	private GridData progressGridData = new GridData(SWT.RIGHT, SWT.CENTER,
+			false, false);
+
+	/**
+	 * A clickable image label that brings up the Progress viewer 
+	 */
+	private CLabelPadding progressViewerImageLabel;
+
+	private Image progress_error_img = null;
+
+	private Image progress_info_img = null;
+
+	private Image progress_viewer_img = null;
+
+	private Image currentProgressImage = null;
+
+	private boolean updateProgressBarDisplayQueued = false;
+
+	protected IProgressReport latestReport = null;
+
+	protected AEMonitor latestReport_mon = new AEMonitor("latestReport");
+
 	/**
 	 * 
 	 */
@@ -158,11 +179,10 @@ public class MainStatusBar {
 
 	/**
 	 * 
-	 * @return composite holiding the statusbar
+	 * @return composite holding the statusbar
 	 */
-	public Composite initStatusBar(final AzureusCore core, final GlobalManager globalManager,
-			Display display, final Composite parent)
-	{
+	public Composite initStatusBar(final AzureusCore core,
+			final GlobalManager globalManager, Display display, final Composite parent) {
 		this.display = display;
 		this.globalManager = globalManager;
 		this.azureusCore = core;
@@ -170,13 +190,14 @@ public class MainStatusBar {
 
 		FormData formData;
 
-		//final int borderFlag = (Constants.isOSX) ? SWT.SHADOW_NONE : SWT.SHADOW_IN;
-		final int borderFlag = SWT.SHADOW_NONE;
+		Color fgColor = parent.getForeground();
 
 		statusBar = new Composite(parent, SWT.NONE);
+		statusBar.setForeground(fgColor);
+		isAZ3 = "az3".equalsIgnoreCase(COConfigurationManager.getStringParameter("ui"));
 
 		GridLayout layout_status = new GridLayout();
-		layout_status.numColumns = 7;
+		layout_status.numColumns = 20;
 		layout_status.horizontalSpacing = 0;
 		layout_status.verticalSpacing = 0;
 		layout_status.marginHeight = 0;
@@ -193,21 +214,11 @@ public class MainStatusBar {
 		}
 		statusBar.setLayout(layout_status);
 
-		GridData gridData;
-
-		//Composite with StackLayout
-		statusArea = new Composite(statusBar, SWT.NONE);
-		gridData = new GridData(GridData.FILL_BOTH);
-		statusArea.setLayoutData(gridData);
-
-		layoutStatusArea = new StackLayout();
-		statusArea.setLayout(layoutStatusArea);
-
 		//Either the Status Text
-		statusText = new CLabel(statusArea, borderFlag);
-		gridData = new GridData(GridData.FILL_HORIZONTAL
-				| GridData.VERTICAL_ALIGN_FILL);
-		statusText.setLayoutData(gridData);
+		statusText = new CLabel(statusBar, borderFlag);
+		statusText.setForeground(fgColor);
+		statusText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL
+				| GridData.VERTICAL_ALIGN_FILL));
 
 		// This is the highest image displayed on the statusbar
 		Image image = ImageRepository.getImage(STATUS_ICON_WARN);
@@ -227,8 +238,12 @@ public class MainStatusBar {
 
 		Listener listener = new Listener() {
 			public void handleEvent(Event e) {
-				if (updateWindow != null) {
-					updateWindow.show();
+				if (clickListener == null) {
+					if (updateWindow != null) {
+						updateWindow.show();
+					}
+				} else {
+					clickListener.UIStatusTextClicked();
 				}
 			}
 		};
@@ -236,53 +251,89 @@ public class MainStatusBar {
 		statusText.addListener(SWT.MouseUp, listener);
 		statusText.addListener(SWT.MouseDoubleClick, listener);
 
-		//Or a composite with a label, a progressBar and a button
-		statusUpdate = new Composite(statusArea, SWT.NULL);
-		gridData = new GridData(GridData.FILL_HORIZONTAL
-				| GridData.VERTICAL_ALIGN_FILL);
-		statusUpdate.setLayoutData(gridData);
-		GridLayout layoutStatusUpdate = new GridLayout(2, false);
-		layoutStatusUpdate.marginHeight = 0;
-		layoutStatusUpdate.marginWidth = 0;
-		statusUpdate.setLayout(layoutStatusUpdate);
+		// final int progressFlag = (Constants.isOSX) ? SWT.INDETERMINATE	: SWT.HORIZONTAL;
+		// KN: Don't know why OSX is treated differently but this check was already here from the previous code
+		if (true == Constants.isOSX) {
+			progressBar = new AZProgressBar(statusBar, true);
+		} else {
+			progressBar = new AZProgressBar(statusBar, false);
+		}
 
-		statusUpdateLabel = new Label(statusUpdate, SWT.NULL);
-		gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
-		statusUpdateLabel.setLayoutData(gridData);
-		Messages.setLanguageText(statusUpdateLabel,
-				"MainWindow.statusText.checking");
-		Messages.setLanguageText(statusUpdateLabel,
-				"MainWindow.status.update.tooltip");
-		statusUpdateLabel.addMouseListener(new MouseAdapter() {
-			public void mouseDoubleClick(MouseEvent arg0) {
-				showUpdateProgressWindow();
+		progressBar.setVisible(false);
+		progressGridData = new GridData(SWT.CENTER, SWT.CENTER, false, false);
+		progressGridData.widthHint = 5;
+		progressBar.setLayoutData(progressGridData);
+
+		/*
+		 * Progress reporting window image label
+		 */
+		progress_error_img = ImageRepository.getImage("progress_error");
+		progress_info_img = ImageRepository.getImage("progress_info");
+		progress_viewer_img = ImageRepository.getImage("progress_viewer");
+
+		progressViewerImageLabel = new CLabelPadding(statusBar, SWT.NONE);
+		// image set below after adding listener
+		progressViewerImageLabel.setToolTipText(MessageText.getString("Progress.reporting.statusbar.button.tooltip"));
+		progressViewerImageLabel.addMouseListener(new MouseAdapter() {
+			public void mouseDown(MouseEvent e) {
+				/*
+				 * Opens the progress viewer if any of the reporters in the array is NOT already opened
+				 * KN: TODO -- This is only a partial solution to minimize the occurrence of the main progress window
+				 * opening more than once.  The one remaining case where multiple windows will still open is
+				 * when you have one opened already... then run another process such as a torrent file download...
+				 * at this point this new process is not in the already opened window so the check would
+				 * allow the second window to open.
+				 */
+				IProgressReporter[] reporters = PRManager.getReportersArray(false);
+				if (reporters.length == 0) {
+					/*
+					 * If there's nothing to see then open the window; the default widow will say there's nothing to see
+					 * KN: calling isShowingEmpty return true is there is already a window opened showing the empty panel
+					 */
+					if (false == ProgressReporterWindow.isShowingEmpty()) {
+						ProgressReporterWindow.open(reporters,
+								ProgressReporterWindow.SHOW_TOOLBAR);
+					}
+				} else {
+
+					for (int i = 0; i < reporters.length; i++) {
+						if (false == ProgressReporterWindow.isOpened(reporters[i])) {
+							ProgressReporterWindow.open(reporters,
+									ProgressReporterWindow.SHOW_TOOLBAR);
+							break;
+						}
+					}
+				}
 			}
 		});
 
-		final int progressFlag = (Constants.isOSX) ? SWT.INDETERMINATE
-				: SWT.HORIZONTAL;
-		statusUpdateProgressBar = new ProgressBar(statusUpdate, progressFlag);
-		gridData = new GridData(GridData.FILL_BOTH);
-		statusUpdateProgressBar.setLayoutData(gridData);
-		Messages.setLanguageText(statusUpdateProgressBar,
-				"MainWindow.status.update.tooltip");
-		statusUpdateProgressBar.addMouseListener(new MouseAdapter() {
-			public void mouseDoubleClick(MouseEvent arg0) {
-				showUpdateProgressWindow();
-			}
-		});
-
-		layoutStatusArea.topControl = statusText;
 		statusBar.layout();
 
+		this.plugin_label_composite = new Composite(statusBar, SWT.NONE);
+		this.plugin_label_composite.setForeground(fgColor);
+		GridLayout gridLayout = new GridLayout();
+		gridLayout.horizontalSpacing = 0;
+		gridLayout.verticalSpacing = 0;
+		gridLayout.marginHeight = 0;
+		gridLayout.marginBottom = 0;
+		gridLayout.marginTop = 0;
+		gridLayout.marginLeft = 0;
+		gridLayout.marginRight = 0;
+		gridLayout.numColumns = 20; // Something nice and big. :)
+
+		GridData gridData = new GridData(GridData.FILL_VERTICAL);
+		gridData.heightHint = height;
+		gridData.minimumHeight = height;
+		plugin_label_composite.setLayout(gridLayout);
+		plugin_label_composite.setLayoutData(gridData);
+
 		srStatus = new CLabelPadding(statusBar, borderFlag);
 		srStatus.setText(MessageText.getString("SpeedView.stats.ratio"));
 
 		COConfigurationManager.addAndFireParameterListener("Status Area Show SR",
 				new ParameterListener() {
 					public void parameterChanged(String parameterName) {
-						srStatus.setVisible(COConfigurationManager.getBooleanParameter(
-								parameterName));
+						srStatus.setVisible(COConfigurationManager.getBooleanParameter(parameterName));
 						statusBar.layout();
 					}
 				});
@@ -293,23 +344,19 @@ public class MainStatusBar {
 		COConfigurationManager.addAndFireParameterListener("Status Area Show NAT",
 				new ParameterListener() {
 					public void parameterChanged(String parameterName) {
-						natStatus.setVisible(COConfigurationManager.getBooleanParameter(
-								parameterName));
+						natStatus.setVisible(COConfigurationManager.getBooleanParameter(parameterName));
 						statusBar.layout();
 					}
 				});
 
 		dhtStatus = new CLabelPadding(statusBar, borderFlag);
 		dhtStatus.setText("");
-		dhtStatus.setToolTipText(MessageText
-				.getString("MainWindow.dht.status.tooltip"));
-	
+		dhtStatus.setToolTipText(MessageText.getString("MainWindow.dht.status.tooltip"));
 
 		COConfigurationManager.addAndFireParameterListener("Status Area Show DDB",
 				new ParameterListener() {
 					public void parameterChanged(String parameterName) {
-						dhtStatus.setVisible(COConfigurationManager.getBooleanParameter(
-								parameterName));
+						dhtStatus.setVisible(COConfigurationManager.getBooleanParameter(parameterName));
 						statusBar.layout();
 					}
 				});
@@ -321,23 +368,18 @@ public class MainStatusBar {
 				BlockedIpsWindow.showBlockedIps(azureusCore, parent.getShell());
 			}
 		});
-		
-		
+
 		COConfigurationManager.addAndFireParameterListener("Status Area Show IPF",
 				new ParameterListener() {
 					public void parameterChanged(String parameterName) {
-						ipBlocked.setVisible(COConfigurationManager.getBooleanParameter( parameterName));
+						ipBlocked.setVisible(COConfigurationManager.getBooleanParameter(parameterName));
 						statusBar.layout();
 					}
 				});
-		
-		
-		
 
 		statusDown = new CLabelPadding(statusBar, borderFlag);
 		statusDown.setImage(ImageRepository.getImage("down"));
-		statusDown
-				.setText(/*MessageText.getString("ConfigView.download.abbreviated") +*/"n/a");
+		statusDown.setText(/*MessageText.getString("ConfigView.download.abbreviated") +*/"n/a");
 		Messages.setLanguageText(statusDown,
 				"MainWindow.status.updowndetails.tooltip");
 
@@ -349,8 +391,7 @@ public class MainStatusBar {
 
 		statusUp = new CLabelPadding(statusBar, borderFlag);
 		statusUp.setImage(ImageRepository.getImage("up"));
-		statusUp
-				.setText(/*MessageText.getString("ConfigView.upload.abbreviated") +*/"n/a");
+		statusUp.setText(/*MessageText.getString("ConfigView.upload.abbreviated") +*/"n/a");
 		Messages.setLanguageText(statusUp,
 				"MainWindow.status.updowndetails.tooltip");
 
@@ -372,13 +413,11 @@ public class MainStatusBar {
 
 				OverallStats stats = StatsFactory.getStats();
 
-				long ratio = (1000 * stats.getUploadedBytes() / (stats
-						.getDownloadedBytes() + 1));
+				long ratio = (1000 * stats.getUploadedBytes() / (stats.getDownloadedBytes() + 1));
 
 				if (ratio < 900) {
 
-					Utils
-							.launch(Constants.AZUREUS_WIKI + "Share_Ratio");
+					Utils.launch(Constants.AZUREUS_WIKI + "Share_Ratio");
 				}
 			}
 		};
@@ -389,37 +428,82 @@ public class MainStatusBar {
 			public void handleEvent(Event e) {
 				uiFunctions.showConfig(ConfigSection.SECTION_CONNECTION);
 
-				if (azureusCore.getPluginManager().getDefaultPluginInterface()
-						.getConnectionManager().getNATStatus() != ConnectionManager.NAT_OK) {
-					Utils
-							.launch(Constants.AZUREUS_WIKI + "NAT_problem");
+				if (azureusCore.getPluginManager().getDefaultPluginInterface().getConnectionManager().getNATStatus() != ConnectionManager.NAT_OK) {
+					Utils.launch(Constants.AZUREUS_WIKI + "NAT_problem");
 				}
 			}
 		};
 
 		natStatus.addListener(SWT.MouseDoubleClick, lNAT);
 
-		// Status Bar Menu construction
-		final Menu menuUpSpeed = new Menu(statusBar.getShell(), SWT.POP_UP);
-		menuUpSpeed.addListener(SWT.Show, new Listener() {
-			public void handleEvent(Event e) {
-				SelectableSpeedMenu.generateMenuItems(menuUpSpeed, core, globalManager,
-						true);
-			}
-		});
-		statusUp.setMenu(menuUpSpeed);
+		boolean bSpeedMenu = COConfigurationManager.getBooleanParameter("GUI_SWT_bOldSpeedMenu");
 
-		final Menu menuDownSpeed = new Menu(statusBar.getShell(), SWT.POP_UP);
-		menuDownSpeed.addListener(SWT.Show, new Listener() {
-			public void handleEvent(Event e) {
-				SelectableSpeedMenu.generateMenuItems(menuDownSpeed, core,
-						globalManager, false);
-			}
-		});
-		statusDown.setMenu(menuDownSpeed);
+		if (bSpeedMenu) {
+			// Status Bar Menu construction
+			final Menu menuUpSpeed = new Menu(statusBar.getShell(), SWT.POP_UP);
+			menuUpSpeed.addListener(SWT.Show, new Listener() {
+				public void handleEvent(Event e) {
+					SelectableSpeedMenu.generateMenuItems(menuUpSpeed, core,
+							globalManager, true);
+				}
+			});
+			statusUp.setMenu(menuUpSpeed);
+		} else {
 
-		addUpdateListener();
+			statusUp.addMouseListener(new MouseAdapter() {
+				public void mouseDown(MouseEvent e) {
+					if (!(e.button == 3 || (e.button == 1 && e.stateMask == SWT.CONTROL))) {
+						return;
+					}
+					Event event = new Event();
+					event.type = SWT.MouseUp;
+					event.widget = e.widget;
+					event.stateMask = e.stateMask;
+					event.button = e.button;
+					e.widget.getDisplay().post(event);
 
+					Utils.execSWTThread(new AERunnable() {
+						public void runSupport() {
+							SelectableSpeedMenu.invokeSlider(true);
+						}
+					});
+				}
+			});
+		}
+
+		if (bSpeedMenu) {
+			final Menu menuDownSpeed = new Menu(statusBar.getShell(), SWT.POP_UP);
+			menuDownSpeed.addListener(SWT.Show, new Listener() {
+				public void handleEvent(Event e) {
+					SelectableSpeedMenu.generateMenuItems(menuDownSpeed, core,
+							globalManager, false);
+				}
+			});
+			statusDown.setMenu(menuDownSpeed);
+		} else {
+			statusDown.addMouseListener(new MouseAdapter() {
+				public void mouseDown(MouseEvent e) {
+					if (!(e.button == 3 || (e.button == 1 && e.stateMask == SWT.CONTROL))) {
+						return;
+					}
+					Event event = new Event();
+					event.type = SWT.MouseUp;
+					event.widget = e.widget;
+					event.stateMask = e.stateMask;
+					event.button = e.button;
+					e.widget.getDisplay().post(event);
+
+					Utils.execSWTThread(new AERunnable() {
+						public void runSupport() {
+							SelectableSpeedMenu.invokeSlider(false);
+						}
+					});
+				}
+			});
+		}
+
+		PRManager.addListener(new ProgressListener());
+		setProgressImage();
 		return statusBar;
 	}
 
@@ -430,15 +514,47 @@ public class MainStatusBar {
 	public void setStatusText(String keyedSentence) {
 		this.statusTextKey = keyedSentence == null ? "" : keyedSentence;
 		statusImageKey = null;
+		this.clickListener = null;
 		if (statusTextKey.length() == 0) { // reset
-			if (Constants.isCVSVersion()) {
-				statusTextKey = "MainWindow.status.unofficialversion ("
-						+ Constants.AZUREUS_VERSION + ")";
-				statusImageKey = STATUS_ICON_WARN;
-			} else if (!Constants.isOSX) { //don't show official version numbers for OSX L&F
-				statusTextKey = Constants.AZUREUS_NAME + " "
-						+ Constants.AZUREUS_VERSION;
-			}
+			resetStatus();
+		}
+
+		updateStatusText();
+	}
+
+	private void resetStatus() {
+		if (Constants.isCVSVersion()) {
+			statusTextKey = "MainWindow.status.unofficialversion ("
+					+ Constants.AZUREUS_VERSION + ")";
+			statusImageKey = STATUS_ICON_WARN;
+		} else if (!Constants.isOSX) { //don't show official version numbers for OSX L&F
+			statusTextKey = Constants.APP_NAME + " " + Constants.AZUREUS_VERSION;
+			statusImageKey = null;
+		}
+
+	}
+
+	/**
+	 * @param statustype
+	 * @param string
+	 * @param l
+	 */
+	public void setStatusText(int statustype, String string,
+			UIStatusTextClickListener l) {
+		this.statusTextKey = string == null ? "" : string;
+
+		if (statusTextKey.length() == 0) { // reset
+			resetStatus();
+		}
+
+		this.clickListener = l;
+		if (statustype == UIFunctions.STATUSICON_WARNING) {
+			statusImageKey = STATUS_ICON_WARN;
+		}
+		if (statustype == UIFunctions.STATUSICON_WARNING) {
+			statusImageKey = STATUS_ICON_WARN;
+		} else {
+			statusImageKey = null;
 		}
 
 		updateStatusText();
@@ -461,8 +577,8 @@ public class MainStatusBar {
 			public void runSupport() {
 				if (statusText != null && !statusText.isDisposed()) {
 					statusText.setText(MessageText.getStringForSentence(text));
-					statusText.setImage((statusImageKey == null) ? null : ImageRepository
-							.getImage(statusImageKey));
+					statusText.setImage((statusImageKey == null) ? null
+							: ImageRepository.getImage(statusImageKey));
 				}
 			}
 		});
@@ -494,217 +610,21 @@ public class MainStatusBar {
 		}
 	}
 
-	private void addUpdateListener() {
-		azureusCore.getPluginManager().getDefaultPluginInterface()
-				.getUpdateManager().addListener(new UpdateManagerListener() {
-					public void checkInstanceCreated(UpdateCheckInstance instance) {
-
-						new updateStatusChanger(instance);
-					}
-				});
-	}
-
-	protected class updateStatusChanger {
-		UpdateCheckInstance instance;
-
-		int check_num = 0;
-
-		boolean active;
-
-		protected updateStatusChanger(UpdateCheckInstance _instance) {
-			instance = _instance;
-
-			try {
-				this_mon.enter();
-
-				update_stack.add(this);
-
-				instance.addListener(new UpdateCheckInstanceListener() {
-					public void cancelled(UpdateCheckInstance instance) {
-						deactivate();
-					}
-
-					public void complete(UpdateCheckInstance instance) {
-						deactivate();
-					}
-				});
-
-				UpdateChecker[] checkers = instance.getCheckers();
-
-				UpdateCheckerListener listener = new UpdateCheckerListener() {
-					public void cancelled(UpdateChecker checker) {
-						// we don't count a cancellation as progress step
-					}
-
-					public void completed(UpdateChecker checker) {
-						setNextCheck();
-					}
-
-					public void failed(UpdateChecker checker) {
-						setNextCheck();
-					}
-
-				};
-				for (int i = 0; i < checkers.length; i++) {
-					checkers[i].addListener(listener);
-				}
-
-				activate();
-
-			} finally {
-
-				this_mon.exit();
-			}
-
-		}
-
-		protected UpdateCheckInstance getInstance() {
-			return (instance);
-		}
-
-		private void activate() {
-			try {
-				this_mon.enter();
-
-				active = true;
-
-				switchStatusToUpdate();
-
-				setNbChecks(instance.getCheckers().length);
-
-			} finally {
-
-				this_mon.exit();
-			}
-		}
-
-		private void deactivate() {
-			try {
-				this_mon.enter();
-
-				active = false;
-
-				for (int i = 0; i < update_stack.size(); i++) {
-
-					if (update_stack.get(i) == this) {
-
-						update_stack.remove(i);
-
-						break;
-					}
-				}
-				if (update_stack.size() == 0) {
-
-					switchStatusToText();
-
-				} else {
-
-					((updateStatusChanger) update_stack.get(update_stack.size() - 1))
-							.activate();
-				}
-
-			} finally {
-
-				this_mon.exit();
-			}
-		}
-
-		private void setNbChecks(final int nbChecks) {
-			if (display != null && !display.isDisposed())
-				Utils.execSWTThread(new AERunnable() {
-					public void runSupport() {
-						if (statusUpdateProgressBar == null
-								|| statusUpdateProgressBar.isDisposed())
-							return;
-						statusUpdateProgressBar.setMinimum(0);
-						statusUpdateProgressBar.setMaximum(nbChecks);
-						statusUpdateProgressBar.setSelection(check_num);
-					}
-				});
-		}
-
-		private void setNextCheck() {
-			if (display != null && !display.isDisposed())
-				Utils.execSWTThread(new AERunnable() {
-					public void runSupport() {
-						if (statusUpdateProgressBar == null
-								|| statusUpdateProgressBar.isDisposed())
-							return;
-
-						check_num++;
-
-						if (active) {
-							statusUpdateProgressBar.setSelection(check_num);
-						}
-					}
-				});
-		}
-
-		private void switchStatusToUpdate() {
-			if (display != null && !display.isDisposed())
-				Utils.execSWTThread(new AERunnable() {
-					public void runSupport() {
-						if (statusArea == null || statusArea.isDisposed()) {
-							return;
-						}
-
-						String name = instance.getName();
-
-						if (MessageText.keyExists(name)) {
-
-							name = MessageText.getString(name);
-						}
-
-						statusUpdateLabel.setText(name);
-
-						layoutStatusArea.topControl = statusUpdate;
-						statusArea.layout();
-					}
-				});
-		}
-
-		private void switchStatusToText() {
-			if (display != null && !display.isDisposed())
-				Utils.execSWTThread(new AERunnable() {
-					public void runSupport() {
-						if (statusArea == null || statusArea.isDisposed()) {
-							return;
-						}
-						layoutStatusArea.topControl = statusText;
-						statusArea.layout();
-					}
-				});
-		}
-	}
-
-	protected void showUpdateProgressWindow() {
-		try {
-			this_mon.enter();
-
-			UpdateCheckInstance[] instances = new UpdateCheckInstance[update_stack
-					.size()];
-
-			for (int i = 0; i < instances.length; i++) {
-
-				instances[i] = ((updateStatusChanger) update_stack.get(i))
-						.getInstance();
-			}
-
-			UpdateProgressWindow.show(instances, statusBar.getShell());
-
-		} finally {
-
-			this_mon.exit();
-		}
-	}
-
 	/**
 	 */
 	public void refreshStatusBar() {
 		if (ipBlocked.isDisposed()) {
 			return;
 		}
-		
+
+		// Plugins.
+		Control[] plugin_elements = this.plugin_label_composite.getChildren();
+		for (int i = 0; i < plugin_elements.length; i++) {
+			if (plugin_elements[i] instanceof UpdateableCLabel) {
+				((UpdateableCLabel) plugin_elements[i]).checkForRefresh();
+			}
+		}
+
 		// IP Filter Status Section
 		IpFilter ip_filter = azureusCore.getIpFilterManager().getIPFilter();
 
@@ -715,17 +635,15 @@ public class MainStatusBar {
 				+ "/"
 				+ numberFormat.format(ip_filter.getNbBannedIps())
 				+ "/"
-				+ numberFormat.format(azureusCore.getIpFilterManager().getBadIps()
-						.getNbBadIps()));
-		ipBlocked.setToolTipText(MessageText.getString(
-				"MainWindow.IPs.tooltip",
-				new String[] { DisplayFormatters.formatDateShort(ip_filter.getLastUpdateTime())
+				+ numberFormat.format(azureusCore.getIpFilterManager().getBadIps().getNbBadIps()));
+		ipBlocked.setToolTipText(MessageText.getString("MainWindow.IPs.tooltip",
+				new String[] {
+					DisplayFormatters.formatDateShort(ip_filter.getLastUpdateTime())
 				}));
 
 		// SR status section
 
-		long ratio = (1000 * overall_stats.getUploadedBytes() / (overall_stats
-				.getDownloadedBytes() + 1));
+		long ratio = (1000 * overall_stats.getUploadedBytes() / (overall_stats.getDownloadedBytes() + 1));
 
 		int sr_status;
 
@@ -794,8 +712,9 @@ public class MainStatusBar {
 
 			ratio_str = (ratio / 1000) + "." + partial;
 
-			srStatus.setToolTipText(MessageText.getString(tooltipID,
-					new String[] { ratio_str }));
+			srStatus.setToolTipText(MessageText.getString(tooltipID, new String[] {
+				ratio_str
+			}));
 
 			last_sr_ratio = ratio;
 		}
@@ -842,16 +761,17 @@ public class MainStatusBar {
 		}
 
 		// DHT Status Section
-		int dht_status = (dhtPlugin == null) ? DHTPlugin.STATUS_DISABLED 	: dhtPlugin.getStatus();
+		int dht_status = (dhtPlugin == null) ? DHTPlugin.STATUS_DISABLED
+				: dhtPlugin.getStatus();
 		long dht_count = -1;
 		//boolean	reachable = false;
 		if (dht_status == DHTPlugin.STATUS_RUNNING) {
 			DHT[] dhts = dhtPlugin.getDHTs();
 
 			//reachable = dhts.length > 0 && dhts[0].getTransport().isReachable();
-			
+
 			//if ( reachable ){
-				dht_count = dhts[0].getControl().getStats().getEstimatedDHTSize();
+			dht_count = dhts[0].getControl().getStats().getEstimatedDHTSize();
 			//}
 		}
 
@@ -859,10 +779,11 @@ public class MainStatusBar {
 			Image img = ImageRepository.getImage("sb_count");
 			switch (dht_status) {
 				case DHTPlugin.STATUS_RUNNING:
-					
+
 					dhtStatus.setToolTipText(MessageText.getString("MainWindow.dht.status.tooltip"));
-					dhtStatus.setText(MessageText.getString("MainWindow.dht.status.users").replaceAll("%1", numberFormat.format(dht_count)));
-					
+					dhtStatus.setText(MessageText.getString("MainWindow.dht.status.users").replaceAll(
+							"%1", numberFormat.format(dht_count)));
+
 					/*
 					if ( reachable ){
 						dhtStatus.setImage(ImageRepository.getImage("greenled"));
@@ -911,12 +832,10 @@ public class MainStatusBar {
 		GlobalManagerStats stats = globalManager.getStats();
 
 		statusDown.setText((dl_limit == 0 ? "" : "[" + dl_limit + "K] ")
-				+ DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(stats
-						.getDataReceiveRate(), stats.getProtocolReceiveRate()));
+				+ DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(
+						stats.getDataReceiveRate(), stats.getProtocolReceiveRate()));
 
-		boolean auto_up = COConfigurationManager
-				.getBooleanParameter(TransferSpeedValidator
-						.getActiveAutoUploadParameter(globalManager))
+		boolean auto_up = COConfigurationManager.getBooleanParameter(TransferSpeedValidator.getActiveAutoUploadParameter(globalManager))
 				&& TransferSpeedValidator.isAutoUploadAvailable(azureusCore);
 
 		int ul_limit_norm = NetworkManager.getMaxUploadRateBPSNormal() / 1024;
@@ -937,8 +856,8 @@ public class MainStatusBar {
 		statusUp.setText((ul_limit_norm == 0 ? "" : "[" + ul_limit_norm + "K"
 				+ seeding_only + "]")
 				+ (auto_up ? "* " : " ")
-				+ DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(stats
-						.getDataSendRate(), stats.getProtocolSendRate()));
+				+ DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(
+						stats.getDataSendRate(), stats.getProtocolSendRate()));
 
 		// End of Status Sections
 		statusBar.layout();
@@ -952,6 +871,11 @@ public class MainStatusBar {
 			statusText.setToolTipText(string);
 	}
 
+	public static interface CLabelUpdater
+	{
+		public void update(CLabel label);
+	}
+
 	/**
 	 * CLabel that shrinks to fit text after a specific period of time.
 	 * Makes textual changes less jumpy
@@ -960,7 +884,9 @@ public class MainStatusBar {
 	 * @created Mar 21, 2006
 	 *
 	 */
-	private class CLabelPadding extends CLabel {
+	private class CLabelPadding
+		extends CLabel
+	{
 		private int lastWidth = 0;
 
 		private long widthSetOn = 0;
@@ -979,6 +905,7 @@ public class MainStatusBar {
 			GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_CENTER
 					| GridData.VERTICAL_ALIGN_FILL);
 			setLayoutData(gridData);
+			setForeground(parent.getForeground());
 		}
 
 		/* (non-Javadoc)
@@ -1004,4 +931,270 @@ public class MainStatusBar {
 		}
 	}
 
+	private class UpdateableCLabel
+		extends CLabelPadding
+	{
+
+		private CLabelUpdater updater;
+
+		public UpdateableCLabel(Composite parent, int style, CLabelUpdater updater) {
+			super(parent, style);
+			this.updater = updater;
+		}
+
+		private void checkForRefresh() {
+			updater.update(this);
+		}
+	}
+
+	public CLabel createStatusEntry(final CLabelUpdater updater) {
+		final CLabel[] result = new CLabel[1];
+		Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+				try {
+					this_mon.enter();
+					result[0] = new UpdateableCLabel(plugin_label_composite, borderFlag,
+							updater);
+					result[0].setLayoutData(new GridData(GridData.FILL_BOTH));
+				} finally {
+					this_mon.exit();
+				}
+			}
+		}, false);
+		return result[0];
+	}
+
+	// =============================================================
+	// Below code are ProgressBar/Status text specific
+	// =============================================================	
+	/**
+	 * Show or hide the Progress Bar
+	 * @param state
+	 */
+	private void showProgressBar(boolean state) {
+		/*
+		 * We show/hide the progress bar simply by setting the .widthHint and letting the statusBar handle the layout
+		 */
+		if (true == state && false == progressBar.isVisible()) {
+			progressGridData.widthHint = 100;
+			progressBar.setVisible(true);
+			statusBar.layout();
+		} else if (false == state && true == progressBar.isVisible()) {
+			progressBar.setVisible(false);
+			progressGridData.widthHint = 0;
+			statusBar.layout();
+		}
+	}
+
+	/**
+	 * Updates the display of the ProgressBar and/or the status text
+	 * @param pReport the <code>ProgressReport</code> containing the information
+	 * to display; can be <code>null</code> in which case the status text and progress bar will be reset to default states
+	 */
+	private void updateProgressBarDisplay(IProgressReport pReport) {
+		latestReport_mon.enter();
+		try {
+			latestReport = pReport;
+		} finally {
+			latestReport_mon.exit();
+		}
+		if (null == progressBar || progressBar.isDisposed()
+				|| updateProgressBarDisplayQueued) {
+			return;
+		}
+		updateProgressBarDisplayQueued = true;
+
+		Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+				latestReport_mon.enter();
+				try {
+					updateProgressBarDisplayQueued = false;
+
+					if ((null == progressBar || true == progressBar.isDisposed())) {
+						return;
+					}
+
+					if (null != latestReport) {
+						/*
+						 * Pass the values through to the progressbar
+						 */
+						progressBar.setMinimum(latestReport.getMinimum());
+						progressBar.setMaximum(latestReport.getMaximum());
+						progressBar.setIndeterminate(latestReport.isIndeterminate());
+						progressBar.setPercentage(latestReport.getPercentage());
+						showProgressBar(true);
+
+						/*
+						 * Update status text
+						 */
+						if (true == isAZ3) {
+							statusText.setText(latestReport.getName());
+						} else {
+							setStatusText(latestReport.getName());
+						}
+					}
+
+					else {
+						/*
+						 * Since the pReport is null then reset progress display appropriately
+						 */
+						showProgressBar(false);
+
+						if (true == isAZ3) {
+							statusText.setText("");
+						} else {
+							setStatusText(null);
+						}
+					}
+				} finally {
+					latestReport_mon.exit();
+				}
+
+			}
+
+		}, true);
+
+	}
+
+	private void setProgressImage() {
+		Image newProgressImage;
+		if (PRManager.getReporterCount(ProgressReportingManager.COUNT_ERROR) > 0) {
+			newProgressImage = progress_error_img;
+		} else if (PRManager.getReporterCount(ProgressReportingManager.COUNT_ALL) > 0) {
+			newProgressImage = progress_info_img;
+		} else {
+			newProgressImage = progress_viewer_img;
+		}
+		if (currentProgressImage != newProgressImage) {
+			currentProgressImage = newProgressImage;
+			Utils.execSWTThread(new AERunnable() {
+				public void runSupport() {
+					progressViewerImageLabel.setImage(currentProgressImage);
+				}
+			});
+		}
+	}
+
+	/**
+	 * A listener that listens to any changes notified from the <code>ProgressReportingManager</code> and
+	 * accordingly update the progress bar and/or the status text area.
+	 * @author knguyen
+	 *
+	 */
+	private class ProgressListener
+		implements IProgressReportingListener, IProgressReportConstants
+	{
+
+		public int reporting(int eventType, IProgressReporter reporter) {
+
+			/*
+			 * Show the appropriate image based on the content of the reporting manager
+			 */
+			setProgressImage();
+
+			if (null == reporter) {
+				return RETVAL_OK;
+			}
+
+			if (MANAGER_EVENT_REMOVED == eventType) {
+				updateFromPrevious();
+			} else if (MANAGER_EVENT_ADDED == eventType
+					|| MANAGER_EVENT_UPDATED == eventType) {
+				/*
+				 * Get a ProgressReport to ensure all data is consistent
+				 */
+				IProgressReport pReport = reporter.getProgressReport();
+
+				/*
+				 * Pops up the ProgressReportingWindow to show this report if it is an error report;
+				 * this is to help catch the users attention
+				 */
+				if (true == pReport.isInErrorState()) {
+					
+					if(true == "reporterType_updater".equals(pReport.getReporterType())){
+						/*
+						 * Suppressing the pop-up for update-related errors
+						 */
+						return RETVAL_OK;
+					}
+					
+					final IProgressReporter final_reporter = reporter;
+					
+
+					/*
+					 * The new window is opened only if there is not one already showing the same reporter
+					 */
+					if (false == ProgressReporterWindow.isOpened(final_reporter)) {
+						Utils.execSWTThread(new AERunnable() {
+							public void runSupport() {
+								if ( !ProgressReporterWindow.isOpened(final_reporter)){
+									ProgressReporterWindow.open(final_reporter,
+											ProgressReporterWindow.NONE);
+								}
+							}
+						}, true);
+					}
+				}
+
+				/*
+				 * If this reporter is not active then get the previous reporter that is still active and display info from that
+				 */
+				if (false == pReport.isActive()) {
+					updateFromPrevious();
+				} else {
+					update(pReport);
+				}
+			}
+
+			return RETVAL_OK;
+		}
+
+		private void update(final IProgressReport pReport) {
+
+			if (null == pReport) {
+				updateProgressBarDisplay(null);
+				return;
+			}
+
+			/*
+			 * If there is at least 2 reporters still active then show the progress bar as indeterminate
+			 * and display the text from the current reporter
+			 */
+			if (true == PRManager.hasMultipleActive()) {
+				Utils.execSWTThread(new AERunnable() {
+					public void runSupport() {
+						setStatusText(pReport.getName());
+						progressBar.setIndeterminate(true);
+						showProgressBar(true);
+					}
+				}, true);
+			} else {
+				updateProgressBarDisplay(pReport);
+			}
+		}
+
+		private void updateFromPrevious() {
+			/*
+			 * Get the previous reporter that is still active
+			 */
+			IProgressReporter previousReporter = PRManager.getNextActiveReporter();
+
+			/*
+			 * If null then we reset the status text and the progress bar
+			 */
+			if (null != previousReporter) {
+				update(previousReporter.getProgressReport());
+			} else {
+				update(null);
+			}
+		}
+	}
+
+	public Rectangle getBounds() {
+		if (null != statusBar) {
+			return statusBar.getBounds();
+		}
+		return null;
+	}
+
 }
\ No newline at end of file
