@@ -24,25 +24,37 @@ package org.gudy.azureus2.ui.swt.mainwindow;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
-import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.Listener;
-import org.eclipse.swt.widgets.Menu;
-import org.eclipse.swt.widgets.MenuItem;
-import org.eclipse.swt.widgets.MessageBox;
+import org.eclipse.swt.widgets.*;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.impl.TransferSpeedValidator;
 import org.gudy.azureus2.core3.global.GlobalManager;
 import org.gudy.azureus2.core3.global.GlobalManagerStats;
 import org.gudy.azureus2.core3.internat.MessageText;
+import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.DisplayFormatters;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.shells.InputShell;
+import org.gudy.azureus2.ui.swt.shells.SpeedScaleShell;
 
 import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.core.AzureusCoreFactory;
 
 public class SelectableSpeedMenu {
 
+	private final static int[] increases = {
+		1,
+		1,
+		3,
+		5, // totals 10
+		10,
+		40, // totals 50
+		50, // 100
+		50,
+		50,
+		100
+	};
+	
 	public static void generateMenuItems(final Menu parent,
 			final AzureusCore core, final GlobalManager globalManager, final boolean up_menu)
 	{
@@ -61,16 +73,7 @@ public class SelectableSpeedMenu {
 
         int maxBandwidth = COConfigurationManager.getIntParameter(configKey);
         final boolean unlim = (maxBandwidth == 0);
-        if(maxBandwidth == 0 && !up_menu )
-        {
-      		GlobalManagerStats stats = globalManager.getStats();
-      		int dataReceive = stats.getDataReceiveRate();
-      		if (dataReceive < 1024) {
-            maxBandwidth = 275;
-      		} else {
-      			maxBandwidth = dataReceive / 1024;      			
-      		}
-        }
+        maxBandwidth = adjustMaxBandWidth(maxBandwidth, globalManager, up_menu);
         
         boolean	auto = false;
         
@@ -110,25 +113,9 @@ public class SelectableSpeedMenu {
         	speed_limits = parseSpeedPartitionString(COConfigurationManager.getStringParameter(config_prefix + "values", ""));
         }
 
-        if (speed_limits == null) {
-        	java.util.List l = new java.util.ArrayList(); 
-	        int delta = 0;
-	        for (int i = 0; i < speedPartitions; i++) {
-	            final int[] valuePair;
-	              if (delta == 0)
-	                valuePair = new int[] { maxBandwidth };
-	              else
-	                valuePair = new int[] { maxBandwidth - delta, maxBandwidth + delta };
-	
-	              for (int j = 0; j < valuePair.length; j++) {
-	            	  if (j==0) {l.add(0, new Integer(valuePair[j]));}
-	            	  else {l.add(new Integer(valuePair[j]));}
-	              }
-	
-	              delta += (delta >= 50) ? 50 : (delta >= 10) ? 10 : (delta >= 5) ? 5 : (delta >= 2) ? 3 : 1;
-	        }
-	        speed_limits = (Integer[])l.toArray(new Integer[l.size()]);
-        }
+		if (speed_limits == null) {
+			speed_limits = getGenericSpeedList(speedPartitions, maxBandwidth);
+		}
         
         for (int i=0; i<speed_limits.length; i++) {
         	Integer i_value = speed_limits[i]; 
@@ -197,7 +184,28 @@ public class SelectableSpeedMenu {
 		});
     }
 
-	  private static java.util.Map parseSpeedPartitionStringCache = new java.util.HashMap();
+	  /**
+	 * @param configKey
+	 * @return
+	 *
+	 * @since 3.0.1.7
+	 */
+	private static int adjustMaxBandWidth(int maxBandwidth,
+			GlobalManager globalManager, boolean up_menu) {
+    if(maxBandwidth == 0 && !up_menu )
+    {
+  		GlobalManagerStats stats = globalManager.getStats();
+  		int dataReceive = stats.getDataReceiveRate();
+  		if (dataReceive < 1024) {
+        maxBandwidth = 275;
+  		} else {
+  			maxBandwidth = dataReceive / 1024;      			
+  		}
+    }
+    return maxBandwidth;
+	}
+
+		private static java.util.Map parseSpeedPartitionStringCache = new java.util.HashMap();
 	  private synchronized static Integer[] parseSpeedPartitionString(String s) {
 		  Integer[] result = (Integer[])parseSpeedPartitionStringCache.get(s);
 		  if (result == null) {
@@ -260,6 +268,151 @@ public class SelectableSpeedMenu {
 	           }
 	       };
 	   }
+
+	   
+	public static Integer[] getGenericSpeedList(int speedPartitions,
+			int maxBandwidth) {
+		java.util.List l = new java.util.ArrayList();
+		int delta = 0;
+		int increaseLevel = 0;
+		for (int i = 0; i < speedPartitions; i++) {
+			final int[] valuePair;
+			if (delta == 0) {
+				valuePair = new int[] {
+					maxBandwidth
+				};
+			} else {
+				valuePair = new int[] {
+					maxBandwidth - delta * (maxBandwidth <= 1024 ? 1 : 1024),
+					maxBandwidth + delta * (maxBandwidth < 1024 ? 1 : 1024)
+				};
+			}
+
+			for (int j = 0; j < valuePair.length; j++) {
+				if (j == 0) {
+					l.add(0, new Integer(valuePair[j]));
+				} else {
+					l.add(new Integer(valuePair[j]));
+				}
+			}
+
+			delta += increases[increaseLevel];
+			if (increaseLevel < increases.length - 1) {
+				increaseLevel++;
+			}
+		}
+		return (Integer[]) l.toArray(new Integer[l.size()]);
+	}
+	   
+	/**
+	 * @since 3.0.1.7
+	 */
+	public static void invokeSlider(boolean isUpSpeed) {
+		final String prefix = MessageText.getString(isUpSpeed
+				? "GeneralView.label.maxuploadspeed"
+				: "GeneralView.label.maxdownloadspeed");
+
+		GlobalManager gm = AzureusCoreFactory.getSingleton().getGlobalManager();
+
+		final String configAutoKey = TransferSpeedValidator.getActiveAutoUploadParameter(gm);
+		boolean auto = COConfigurationManager.getBooleanParameter(configAutoKey);
+
+		final String configKey = isUpSpeed
+				? TransferSpeedValidator.getActiveUploadParameter(gm)
+				: "Max Download Speed KBs";
+		int maxBandwidth = COConfigurationManager.getIntParameter(configKey);
+		final boolean unlim = (maxBandwidth == 0);
+		if (unlim && !isUpSpeed) {
+			GlobalManagerStats stats = gm.getStats();
+			int dataReceive = stats.getDataReceiveRate();
+			if (dataReceive >= 1024) {
+				maxBandwidth = dataReceive / 1024;
+			}
+		}
+
+		SpeedScaleShell speedScale = new SpeedScaleShell() {
+			public String getStringValue(int value, String sValue) {
+				if (sValue != null) {
+					return prefix + ": " + sValue;
+				}
+				if (value == 0) {
+					return MessageText.getString("MyTorrentsView.menu.setSpeed.unlimited");
+				}
+				if (value == -1) {
+					return MessageText.getString("ConfigView.auto");
+				}
+				return prefix
+						+ ": "
+						+ (value == 0 ? MessageText.getString("ConfigView.unlimited")
+								: DisplayFormatters.formatByteCountToKiBEtcPerSec(
+										getValue() * 1024, true));
+			}
+		};
+		int max = unlim ? (isUpSpeed ? 100 : 800) : maxBandwidth * 5;
+		if (max < 50) {
+			max = 50;
+		}
+		speedScale.setMaxValue(max);
+		speedScale.setMaxTextValue(9999999);
+
+		final String config_prefix = "config.ui.speed.partitions.manual."
+				+ (isUpSpeed ? "upload" : "download") + ".";
+		int lastValue = COConfigurationManager.getIntParameter(config_prefix
+				+ "last", -10);
+
+		Integer[] speed_limits;
+		if (COConfigurationManager.getBooleanParameter(config_prefix + "enabled",
+				false)) {
+			speed_limits = parseSpeedPartitionString(COConfigurationManager.getStringParameter(
+					config_prefix + "values", ""));
+		} else {
+			speed_limits = getGenericSpeedList(6, maxBandwidth);
+		}
+		if (speed_limits != null) {
+			for (int i = 0; i < speed_limits.length; i++) {
+				int value = speed_limits[i].intValue();
+				if (value > 0) {
+					speedScale.addOption(DisplayFormatters.formatByteCountToKiBEtcPerSec(
+							value * 1024, true), value);
+					if (value == lastValue) {
+						lastValue = -10;
+					}
+				}
+			}
+		}
+		speedScale.addOption(
+				MessageText.getString("MyTorrentsView.menu.setSpeed.unlimited"), 0);
+		speedScale.addOption(MessageText.getString("ConfigView.auto"), -1);
+
+		if (lastValue > 0) {
+			speedScale.addOption(DisplayFormatters.formatByteCountToKiBEtcPerSec(
+					lastValue * 1024, true), lastValue);
+		}
+
+		// SWT BUG: on windows/linux, if mouse is down on shell open, all mouse events
+		// will not reflect this
+		if (speedScale.open(auto ? -1 : maxBandwidth, Constants.isWindows
+				|| Constants.isLinux)) {
+			int value = speedScale.getValue();
+
+			if (!speedScale.wasMenuChosen() || lastValue == value) {
+				COConfigurationManager.setParameter(config_prefix + "last",
+						maxBandwidth);
+			}
+
+			if (value >= 0) {
+				if (auto) {
+					COConfigurationManager.setParameter(configAutoKey, false);
+				}
+				COConfigurationManager.setParameter(configKey, value);
+				COConfigurationManager.save();
+			} else {
+				// autospeed
+				COConfigurationManager.setParameter(configAutoKey, true);
+				COConfigurationManager.save();
+			}
+		}
+	}
 	
 }
  
\ No newline at end of file
