@@ -21,7 +21,9 @@ package org.gudy.azureus2.ui.swt.maketorrent;
 
 import java.io.File;
 import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.dnd.DND;
@@ -85,6 +87,7 @@ NewTorrentWizard
   long	  manual_piece_size;
   
   boolean 			useMultiTracker = false;
+  boolean 			useWebSeed = false;
   
   private boolean 	addOtherHashes	= 	COConfigurationManager.getBooleanParameter( "CreateTorrent.default.addhashes", false );
   
@@ -92,6 +95,9 @@ NewTorrentWizard
   String multiTrackerConfig = "";
   List trackers = new ArrayList();
   
+  String webSeedConfig = "";
+  Map	webseeds = new HashMap();
+  
   boolean autoOpen 			= false;
   boolean autoHost 			= false;
   boolean permitDHT			= true;
@@ -104,7 +110,7 @@ NewTorrentWizard
   	AzureusCore		azureus_core,
 	Display 		display) 
   {
-    super(azureus_core, display, "wizard.title");
+    super(azureus_core, "wizard.title");
     
     cancel.addListener(SWT.Selection, new Listener() {
       public void handleEvent(Event arg0) {
