@@ -10,17 +10,21 @@
 package org.gudy.azureus2.ui.swt.osx;
 
 import java.io.IOException;
+import java.lang.reflect.Method;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.internal.Callback;
 import org.eclipse.swt.internal.carbon.*;
 import org.eclipse.swt.widgets.*;
 
+import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.platform.macosx.access.jnilib.OSXAccess;
+import org.gudy.azureus2.ui.swt.UIExitUtilsSWT;
 import org.gudy.azureus2.ui.swt.Utils;
+import org.gudy.azureus2.ui.swt.speedtest.SpeedTestWizard;
 import org.gudy.azureus2.ui.swt.config.wizard.ConfigureWizard;
 import org.gudy.azureus2.ui.swt.help.AboutWindow;
 import org.gudy.azureus2.ui.swt.mainwindow.TorrentOpener;
@@ -38,33 +42,59 @@ public class CarbonUIEnhancer {
    private static final int kHICommandServices= ('s'<<24) + ('e'<<16) + ('r'<<8) + 'v';
    private static final int kHICommandWizard = ('a'<<24) + ('z'<<16) + ('c' << 8) + 'n';
    private static final int kHICommandNatTest = ('a'<<24) + ('z'<<16) + ('n' << 8) + 't';
+   private static final int kHICommandSpeedTest = ('a'<<24) + ('z'<<16) + ('s'<<8) + 't';
    private static final int kHICommandRestart = ('a'<<24) + ('z'<<16) + ('r'<<8) + 's';
 
    private static final int typeAEList = ('l'<<24) + ('i'<<16) + ('s'<<8) + 't';
    private static final int kCoreEventClass = ('a'<<24) + ('e'<<16) + ('v'<<8) + 't';
    private static final int kAEOpenDocuments = ('o'<<24) + ('d'<<16) + ('o'<<8) + 'c';
+   private static final int kAEReopenApplication = ('r'<<24) + ('a'<<16) + ('p'<<8) + 'p';
+   private static final int kAEOpenContents = ('o'<<24) + ('c'<<16) + ('o'<<8) + 'n';
    private static final int kURLEventClass = ('G'<<24) + ('U'<<16) + ('R'<<8) + 'L';
 
    private static final int typeText = ('T'<<24) + ('E'<<16) + ('X'<<8) + 'T';
 
-   private static final String RESOURCE_BUNDLE= "org.eclipse.ui.carbon.Messages"; //$NON-NLS-1$
-   private static String fgAboutActionName;
-   private static String fgWizardActionName;
-   private static String fgNatTestActionName;
-   private static String fgRestartActionName;
+   private static final String RESOURCE_BUNDLE = "org.eclipse.ui.carbon.Messages"; //$NON-NLS-1$
 
+	private static String fgAboutActionName;
+
+	private static String fgWizardActionName;
+
+	private static String fgNatTestActionName;
+
+	private static String fgRestartActionName;
+
+	private static String fgSpeedTestActionName;
+
+	private static int memmove_type = 0;
+
+	/**
+	 * KN: Some of the menu items have been removed for the Vuze and Vuze Advanced UI's;
+	 * the classic UI still retains all its menu items as before.  Follow this flag in the code
+	 * to see which menu items are effected.
+	 */
+	private boolean isAZ3 = "az3".equalsIgnoreCase(COConfigurationManager.getStringParameter("ui"));
+    
    public CarbonUIEnhancer() {
       if (fgAboutActionName == null) {
          fgAboutActionName = MessageText.getString("MainWindow.menu.help.about").replaceAll("&", "");
       }
-      if(fgWizardActionName == null) {
-          fgWizardActionName = MessageText.getString("MainWindow.menu.file.configure").replaceAll("&", "");
-      }
-      if(fgNatTestActionName == null) {
-      	fgNatTestActionName = MessageText.getString("MainWindow.menu.tools.nattest").replaceAll("&", "");
+      
+      if(false == isAZ3){
+        if(fgWizardActionName == null) {
+            fgWizardActionName = MessageText.getString("MainWindow.menu.file.configure").replaceAll("&", "");
+        }
+        if(fgNatTestActionName == null) {
+        	fgNatTestActionName = MessageText.getString("MainWindow.menu.tools.nattest").replaceAll("&", "");
+        }
+      
+        if(fgSpeedTestActionName == null){
+            fgSpeedTestActionName = MessageText.getString("MainWindow.menu.tools.speedtest").replaceAll("&", "");
+        }
       }
+
       if(fgRestartActionName == null) {
-          fgRestartActionName = MessageText.getString("MainWindow.menu.file.restart").replaceAll("&", "");
+        fgRestartActionName = MessageText.getString("MainWindow.menu.file.restart").replaceAll("&", "");
       }
       earlyStartup();
       registerTorrentFile();
@@ -99,6 +129,34 @@ public class CarbonUIEnhancer {
 	}
 
    private void registerTorrentFile() {
+ 		int result;
+
+ 		Callback clickDockIconCallback = new Callback(target, "clickDockIcon", 3);
+ 		int clickDocIcon = clickDockIconCallback.getAddress();
+ 		if (clickDocIcon == 0) {
+ 			clickDockIconCallback.dispose();
+ 		} else {
+ 			result = OS.AEInstallEventHandler(kCoreEventClass, kAEReopenApplication,
+ 					clickDocIcon, 0, false);
+
+ 			if (result != OS.noErr) {
+ 				Debug.out("OSX: Could Install ReopenApplication Event Handler. Error: " + result);
+ 			}
+ 		}
+
+ 		Callback openContentsCallback = new Callback(target, "openContents", 3);
+ 		int openContents = openContentsCallback.getAddress();
+ 		if (openContents == 0) {
+ 			openContentsCallback.dispose();
+ 		} else {
+ 			result = OS.AEInstallEventHandler(kCoreEventClass, kAEOpenContents,
+ 					openContents, 0, false);
+
+ 			if (result != OS.noErr) {
+ 				Debug.out("OSX: Could Install OpenContents Event Handler. Error: " + result);
+ 			}
+ 		}
+
 		Callback openDocCallback = new Callback(target, "openDocProc", 3);
 		int openDocProc = openDocCallback.getAddress();
 		if (openDocProc == 0) {
@@ -107,27 +165,48 @@ public class CarbonUIEnhancer {
 			return;
 		}
 
-		int result;
 		result = OS.AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments,
 				openDocProc, 0, false);
 
 		if (result != OS.noErr) {
-			Debug.out("OSX: Could Install OpenDocs Event Handler. Error: " + result);
+			Debug.out("OSX: Could not Install OpenDocs Event Handler. Error: " + result);
 			return;
 		}
 
 		result = OS.AEInstallEventHandler(kURLEventClass, kURLEventClass,
 				openDocProc, 0, false);
-
 		if (result != OS.noErr) {
-			Debug.out("OSX: Could Install OpenDocs Event Handler. Error: " + result);
+			Debug.out("OSX: Could not Install URLEventClass Event Handler. Error: " + result);
 			return;
 		}
+		
+		///
+		
+		Callback quitAppCallback = new Callback(target, "quitAppProc", 3);
+		int quitAppProc = quitAppCallback.getAddress();
+		if (quitAppProc == 0) {
+			Debug.out("OSX: Could not find Callback 'quitApp'");
+			quitAppCallback.dispose();
+		} else {
+			result = OS.AEInstallEventHandler(kCoreEventClass, OS.kAEQuitApplication,
+					quitAppProc, 0, false);
+			if (result != OS.noErr) {
+				Debug.out("OSX: Could not install QuitApplication Event Handler. Error: "
+						+ result);
+			}
+		}
+
+		///
 
 		int appTarget = OS.GetApplicationEventTarget();
 		Callback appleEventCallback = new Callback(this, "appleEventProc", 3);
 		int appleEventProc = appleEventCallback.getAddress();
-		int[] mask3 = new int[] { OS.kEventClassAppleEvent, OS.kEventAppleEvent, kURLEventClass, };
+		int[] mask3 = new int[] {
+				OS.kEventClassAppleEvent,
+				OS.kEventAppleEvent,
+				kURLEventClass,
+				kAEReopenApplication,
+				kAEOpenContents,};
 		result = OS.InstallEventHandler(appTarget, appleEventProc,
 				mask3.length / 2, mask3, 0, null);
 		if (result != OS.noErr) {
@@ -179,11 +258,23 @@ public class CarbonUIEnhancer {
                   return OS.noErr;
                }
                case kHICommandWizard:
-                  new ConfigureWizard(AzureusCoreFactory.getSingleton(), display);
+                  new ConfigureWizard(AzureusCoreFactory.getSingleton(), false);
                   return OS.noErr;
                case kHICommandNatTest:
                  new NatTestWindow();
                  return OS.noErr;
+               case kHICommandSpeedTest:
+                 new SpeedTestWizard(AzureusCoreFactory.getSingleton(), display);  
+                 return OS.noErr;
+                 
+               case OS.kAEQuitApplication:
+           			UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+          			if (uiFunctions != null) {
+          				uiFunctions.dispose(false, false);
+            			return OS.noErr;
+          			} else {
+          				UIExitUtilsSWT.setSkipCloseCheck(true);
+          			}
                default:
                   break;
                }
@@ -224,35 +315,45 @@ public class CarbonUIEnhancer {
                // disable services menu
          OS.DisableMenuCommand(menu, kHICommandServices);
 
-          // wizard menu
-         l= fgWizardActionName.length();
-         buffer= new char[l];
-         fgWizardActionName.getChars(0, l, buffer, 0);
-         str= OS.CFStringCreateWithCharacters(OS.kCFAllocatorDefault, buffer, l);
-         OS.InsertMenuItemTextWithCFString(menu, str, (short) 3, 0, kHICommandWizard);
-         OS.CFRelease(str);
+         if(false == isAZ3){
+            // wizard menu
+           l= fgWizardActionName.length();
+           buffer= new char[l];
+           fgWizardActionName.getChars(0, l, buffer, 0);
+           str= OS.CFStringCreateWithCharacters(OS.kCFAllocatorDefault, buffer, l);
+           OS.InsertMenuItemTextWithCFString(menu, str, (short) 3, 0, kHICommandWizard);
+           OS.CFRelease(str);
+           
+           
+           // NAT test menu
+           l= fgNatTestActionName.length();
+           buffer= new char[l];
+           fgNatTestActionName.getChars(0, l, buffer, 0);
+           str= OS.CFStringCreateWithCharacters(OS.kCFAllocatorDefault, buffer, l);
+           OS.InsertMenuItemTextWithCFString(menu, str, (short) 4, 0, kHICommandNatTest);
+           OS.CFRelease(str);
+           
+  
+            //SpeedTest
+            l = fgSpeedTestActionName.length();
+            buffer = new char[l];
+            fgSpeedTestActionName.getChars(0,l,buffer,0);
+            str= OS.CFStringCreateWithCharacters(OS.kCFAllocatorDefault, buffer, l);
+            OS.InsertMenuItemTextWithCFString(menu, str, (short) 5, 0, kHICommandSpeedTest);
+            OS.CFRelease(str);
+         }
          
-         
-         // NAT test menu
-         l= fgNatTestActionName.length();
-         buffer= new char[l];
-         fgNatTestActionName.getChars(0, l, buffer, 0);
-         str= OS.CFStringCreateWithCharacters(OS.kCFAllocatorDefault, buffer, l);
-         OS.InsertMenuItemTextWithCFString(menu, str, (short) 4, 0, kHICommandNatTest);
-         OS.CFRelease(str);
-         
-
-          OS.InsertMenuItemTextWithCFString(menu, 0, (short) 5, OS.kMenuItemAttrSeparator, 0);
+          OS.InsertMenuItemTextWithCFString(menu, 0, (short) 6, OS.kMenuItemAttrSeparator, 0);
 
           // restart menu
          l= fgRestartActionName.length();
          buffer= new char[l];
          fgRestartActionName.getChars(0, l, buffer, 0);
          str= OS.CFStringCreateWithCharacters(OS.kCFAllocatorDefault, buffer, l);
-         OS.InsertMenuItemTextWithCFString(menu, str, (short) 6, 0, kHICommandRestart);
+         OS.InsertMenuItemTextWithCFString(menu, str, (short) 7, 0, kHICommandRestart);
          OS.CFRelease(str);
 
-          OS.InsertMenuItemTextWithCFString(menu, 0, (short) 7, OS.kMenuItemAttrSeparator, 0);
+          OS.InsertMenuItemTextWithCFString(menu, 0, (short) 8, OS.kMenuItemAttrSeparator, 0);
       }
 
       // schedule disposal of callback object
@@ -281,11 +382,11 @@ public class CarbonUIEnhancer {
 
    int appleEventProc(int nextHandler, int theEvent, int userData) {
 		int eventClass = OS.GetEventClass(theEvent);
-//		int eventKind = OS.GetEventKind(theEvent);
+		//int eventKind = OS.GetEventKind(theEvent);
 
-//		System.out.println("appleEventProc " + OSXtoString(eventClass) + ";"
-//				+ OS.GetEventKind(theEvent) + ";" + OSXtoString(theEvent) + ";"
-//				+ OSXtoString(userData));
+		//System.out.println("appleEventProc " + OSXtoString(eventClass) + ";"
+		//		+ OS.GetEventKind(theEvent) + ";" + OSXtoString(theEvent) + ";"
+		//		+ OSXtoString(userData));
 
 		// Process teh odoc event
 		if (eventClass == OS.kEventClassAppleEvent) {
@@ -294,17 +395,22 @@ public class CarbonUIEnhancer {
 					null, 4, null, aeEventID) != OS.noErr) {
 				return OS.eventNotHandledErr;
 			}
-//			System.out.println("EventID = " + OSXtoString(aeEventID[0]));
-			if (aeEventID[0] != kAEOpenDocuments && aeEventID[0] != kURLEventClass) {
+			//System.out.println("EventID = " + OSXtoString(aeEventID[0]));
+			if (aeEventID[0] != kAEOpenDocuments 
+					&& aeEventID[0] != kURLEventClass
+					&& aeEventID[0] != kAEReopenApplication 
+					&& aeEventID[0] != kAEOpenContents
+					&& aeEventID[0] != OS.kAEQuitApplication) {
 				return OS.eventNotHandledErr;
 			}
 
+			// Handle Event
 			EventRecord eventRecord = new EventRecord();
 			OS.ConvertEventRefToEventRecord(theEvent, eventRecord);
 			OS.AEProcessAppleEvent(eventRecord);
 
-			if (true)
-				return OS.noErr;
+			// Tell Mac we are handling this event
+			return OS.noErr;
 		}
 
 		return OS.eventNotHandledErr;
@@ -318,8 +424,81 @@ public class CarbonUIEnhancer {
 		c[3] = (char)(i & 0xff);
 		return new String(c);
 	}
+ 	
+ 	private static void memmove(byte[] dest, int src, int size) {
+		switch (memmove_type) {
+			case 0:
+				try {
+					OSXAccess.memmove(dest, src, size);
+					memmove_type = 0;
+					return;
+				} catch (Throwable e) {
+				}
+				// FALL THROUGH
+
+			case 1:
+				try {
+					Class cMemMove = Class.forName("org.eclipse.swt.internal.carbon.OS");
+
+					Method method = cMemMove.getMethod("memmove", new Class[] {
+						byte[].class,
+						Integer.TYPE,
+						Integer.TYPE
+					});
+
+					method.invoke(null, new Object[] {
+						dest,
+						new Integer(src),
+						new Integer(size)
+					});
+					memmove_type = 1;
+					return;
+				} catch (Throwable e) {
+				}
+
+				// FALL THROUGH
+			case 2:
+				try {
+					Class cMemMove = Class.forName("org.eclipse.swt.internal.carbon.OS");
+
+					Method method = cMemMove.getMethod("memcpy", new Class[] {
+						byte[].class,
+						Integer.TYPE,
+						Integer.TYPE
+					});
+
+					method.invoke(null, new Object[] {
+						dest,
+						new Integer(src),
+						new Integer(size)
+					});
+
+					memmove_type = 2;
+					return;
+				} catch (Throwable e) {
+				}
+
+				// FALL THROUGH
+
+			default:
+				break;
+		}
+
+		memmove_type = 3;
+	}
 
   final static Object target = new Object() {
+		int quitAppProc(int theAppleEvent, int reply, int handlerRefcon) {
+			UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+			if (uiFunctions != null) {
+				uiFunctions.dispose(false, false);
+			} else {
+				UIExitUtilsSWT.setSkipCloseCheck(true);
+				Display.getDefault().dispose();
+			}
+			return OS.noErr;
+		}
+  	
 		int openDocProc(int theAppleEvent, int reply, int handlerRefcon) {
 			AEDesc aeDesc = new AEDesc();
 			EventRecord eventRecord = new EventRecord();
@@ -350,7 +529,7 @@ public class CarbonUIEnhancer {
 					if (OS.AEGetNthPtr(aeDesc, i + 1, OS.typeFSRef, aeKeyword, typeCode,
 							dataPtr, maximumSize, actualSize) == OS.noErr) {
 						byte[] fsRef = new byte[actualSize[0]];
-						OS.memcpy(fsRef, dataPtr, actualSize[0]);
+						memmove(fsRef, dataPtr, actualSize[0]);
 						int dirUrl = OS.CFURLCreateFromFSRef(OS.kCFAllocatorDefault, fsRef);
 						int dirString = OS.CFURLCopyFileSystemPath(dirUrl,
 								OS.kCFURLPOSIXPathStyle);
@@ -367,7 +546,7 @@ public class CarbonUIEnhancer {
 					if (OS.AEGetNthPtr(aeDesc, i + 1, typeText, aeKeyword, typeCode,
 							dataPtr, maximumSize, actualSize) == OS.noErr) {
 						byte[] urlRef = new byte[actualSize[0]];
-						OS.memcpy(urlRef, dataPtr, actualSize[0]);
+						memmove(urlRef, dataPtr, actualSize[0]);
 						fileNames[i] = new String(urlRef);
 					}
 
@@ -380,6 +559,20 @@ public class CarbonUIEnhancer {
 			return OS.noErr;
 		}
 
+		int clickDockIcon(int nextHandler, int theEvent, int userData) {
+			UIFunctions uiFunctions = UIFunctionsManager.getUIFunctions();
+			if (uiFunctions != null) {
+				uiFunctions.bringToFront();
+				return OS.noErr;
+			}
+			return OS.eventNotHandledErr;
+		}
+
+		int openContents(int nextHandler, int theEvent, int userData) {
+			Debug.out("openDocContents");
+			return OS.noErr;
+		}
+
 		int toolbarToggle(int nextHandler, int theEvent, int userData) {
 			int eventKind = OS.GetEventKind(theEvent);
 			if (eventKind != OS.kEventWindowToolbarSwitchMode) {
