@@ -26,6 +26,8 @@ import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.widgets.Display;
 import org.gudy.azureus2.plugins.download.Download;
 import org.gudy.azureus2.plugins.ui.UIInstance;
+import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;
+import org.gudy.azureus2.plugins.ui.model.BasicPluginViewModel;
 import org.gudy.azureus2.plugins.ui.tables.TableManager;
 
 /**
@@ -65,6 +67,13 @@ public interface UISWTInstance extends UIInstance {
 	 * @since 2.3.0.7
 	 */
 	public static final String VIEW_TORRENT_FILES = TableManager.TABLE_TORRENT_FILES;
+	
+	/**
+	 * ID of the top bar of az3ui
+	 * 
+	 * @since 3.0.1.3
+	 */
+	public static final String VIEW_TOPBAR = "TopBar";
 
 	/** Retrieve the SWT Display object that Azureus uses (when in SWT mode).
 	 * If you have a thread that does some periodic/asynchronous stuff, Azureus 
@@ -119,6 +128,20 @@ public interface UISWTInstance extends UIInstance {
 	 * @since 2.5.0.1
 	 */
 	public boolean openView(String sParentID, String sViewID, Object dataSource);
+
+	/**
+	 * Open a previously added view
+	 * 
+	 * @param sParentID ParentID of the view to be shown
+	 * @param sViewID id of the view to be shown
+	 * @param dataSource any data you need to pass the view
+	 * @param setfocus <tt>true</tt> if you want to display the view immediately,
+	 *   <tt>false</tt> if you want to display it in the background.
+	 * @return success level
+	 * @since 3.0.5.3
+	 */
+	public boolean openView(String sParentID, String sViewID, Object dataSource, boolean setfocus);
+
 	
 	/** 
 	 * Create and open a view in the main window immediately.  If you are calling
@@ -136,6 +159,25 @@ public interface UISWTInstance extends UIInstance {
 	 */
 	public void openMainView(String sViewID, UISWTViewEventListener l,
 			Object dataSource);
+	
+	/** 
+	 * Create and open a view in the main window immediately.  If you are calling
+	 * this from {@link org.gudy.azureus2.plugins.ui.UIManagerListener#UIAttached(UIInstance)},
+	 * the view will not gain focus.
+	 * <p>
+	 * Tip: You can add a menu item to a table view, and when triggered, have
+	 *      it open a new window, passing the datasources that were selected
+	 * 
+	 * @param sViewID ID to give your view
+	 * @param l Listener to be triggered when View Events occur
+	 * @param dataSource objects to set {@link UISWTView#getDataSource()} with
+	 * @param setfocus <tt>true</tt> if you want to display the view immediately,
+	 *   <tt>false</tt> if you want to display it in the background.
+	 *
+	 * @since 3.0.5.3
+	 */
+	public void openMainView(String sViewID, UISWTViewEventListener l,
+			Object dataSource, boolean setfocus);
 
 	/**
 	 * Remove all views that belong to a specific parent and of a specific View 
@@ -208,4 +250,37 @@ public interface UISWTInstance extends UIInstance {
 	 * @param display <tt>true</tt> to show a download bar, <tt>false</tt> to hide it.
 	 */
 	public void showDownloadBar(Download download, boolean display);
+	
+	/**
+	 * Shows or hides the transfers bar.
+	 * 
+	 * @since 3.0.1.3
+	 * @param display <tt>true</tt> to show the bar, <tt>false</tt> to hide it.
+	 */
+	public void showTransfersBar(boolean display);
+	
+	/**
+	 * Creates an entry in the status bar to display custom status information.
+	 * 
+	 * @since 3.0.0.7
+	 * @see UISWTStatusEntry
+	 */
+	public UISWTStatusEntry createStatusEntry();
+	
+	/**
+	 * Opens the window linked to a given BasicPluginViewModel object.
+	 * 
+	 * @return <tt>true</tt> if the view was opened successfully.
+	 * @since 3.0.5.3
+	 */
+	public boolean openView(BasicPluginViewModel model);
+
+	/**
+	 * Opens the window linked to a given BasicPluginViewModel object.
+	 * 
+	 * @return <tt>true</tt> if the view was opened successfully.
+	 * @since 3.0.5.3
+	 */
+	public void openConfig(BasicPluginConfigModel model);
+
 }
