@@ -32,45 +32,52 @@ import java.util.WeakHashMap;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.awt.SWT_AWT;
+import org.eclipse.swt.custom.CLabel;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.graphics.ImageData;
 import org.eclipse.swt.layout.GridData;
+import org.eclipse.swt.widgets.*;
 import org.eclipse.swt.widgets.Composite;
-import org.eclipse.swt.widgets.Display;
-import org.eclipse.swt.widgets.Shell;
+import org.eclipse.swt.widgets.Event;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.download.DownloadManager;
+import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.AERunnable;
+import org.gudy.azureus2.core3.util.AETemporaryFileHandler;
 import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.plugins.download.Download;
-import org.gudy.azureus2.pluginsimpl.local.download.DownloadImpl;
-import org.gudy.azureus2.ui.swt.FileDownloadWindow;
-import org.gudy.azureus2.ui.swt.MinimizedWindow;
-import org.gudy.azureus2.ui.swt.SimpleTextEntryWindow;
-import org.gudy.azureus2.ui.swt.TextViewerWindow;
-import org.gudy.azureus2.ui.swt.Utils;
-import org.gudy.azureus2.ui.swt.mainwindow.ClipboardCopy;
-import org.gudy.azureus2.ui.swt.mainwindow.SWTThread;
-import org.gudy.azureus2.ui.swt.mainwindow.TorrentOpener;
+import org.gudy.azureus2.plugins.ui.model.BasicPluginViewModel;
+import org.gudy.azureus2.ui.common.util.MenuItemManager;
+import org.gudy.azureus2.ui.swt.*;
+import org.gudy.azureus2.ui.swt.mainwindow.*;
+import org.gudy.azureus2.ui.swt.minibar.AllTransfersBar;
+import org.gudy.azureus2.ui.swt.minibar.DownloadBar;
 import org.gudy.azureus2.ui.swt.plugins.*;
+import org.gudy.azureus2.ui.swt.pluginsimpl.UIMessageImpl;
 import org.gudy.azureus2.ui.swt.shells.MessageBoxShell;
-import org.gudy.azureus2.ui.swt.views.table.impl.TableColumnImpl;
 import org.gudy.azureus2.ui.swt.views.table.utils.TableColumnManager;
 import org.gudy.azureus2.ui.swt.views.table.utils.TableContextMenuManager;
+import org.gudy.azureus2.ui.swt.views.utils.ManagerUtils;
 
 import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.ui.IUIIntializer;
 import com.aelitis.azureus.ui.UIFunctionsManager;
+import com.aelitis.azureus.ui.common.table.impl.TableColumnImpl;
 import com.aelitis.azureus.ui.swt.UIFunctionsManagerSWT;
 import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
 
 import org.gudy.azureus2.plugins.PluginInterface;
+import org.gudy.azureus2.plugins.download.Download;
+import org.gudy.azureus2.plugins.torrent.Torrent;
 import org.gudy.azureus2.plugins.ui.*;
+import org.gudy.azureus2.plugins.ui.menus.MenuItem;
 import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;
-import org.gudy.azureus2.plugins.ui.model.BasicPluginViewModel;
 import org.gudy.azureus2.plugins.ui.tables.TableColumn;
 import org.gudy.azureus2.plugins.ui.tables.TableContextMenuItem;
 
+import org.gudy.azureus2.pluginsimpl.local.download.DownloadImpl;
+import org.gudy.azureus2.pluginsimpl.local.ui.UIManagerImpl;
+
 public class 
 UISWTInstanceImpl
 	implements UIInstanceFactory, UISWTInstance, UIManagerEventListener
@@ -97,22 +104,15 @@ UISWTInstanceImpl
 		uiFunctions = (UIFunctionsSWT) UIFunctionsManager.getUIFunctions();
 	}
 	
-	public void init() {
-		try{
-			UIManager	ui_manager = core.getPluginManager().getDefaultPluginInterface().getUIManager();
-			
-			ui_manager.addUIEventListener( this );
-			
-			bUIAttaching = true;
-			
-			ui_manager.attachUI( this );
-			
-			bUIAttaching = false;
-			
-		}catch( UIException e ){
-			
-			Debug.printStackTrace(e);
-		}
+	public void init(IUIIntializer init) {
+		UIManager ui_manager = core.getPluginManager().getDefaultPluginInterface().getUIManager();
+		ui_manager.addUIEventListener(this);
+		
+		bUIAttaching = true;
+		
+		((UIManagerImpl) ui_manager).attachUI(this, init);
+		
+		bUIAttaching = false;
 	}
   
 	public UIInstance
@@ -157,13 +157,102 @@ UISWTInstanceImpl
 				
 				break;
 			}
-			
+			case UIManagerEvent.ET_SHOW_MSG_BOX:
+			{
+				final int[] result = { UIManagerEvent.MT_NONE };
+				
+					Utils.execSWTThread(
+					new Runnable()
+					{
+						public void 
+						run()
+						{
+							UIFunctionsManagerSWT.getUIFunctionsSWT().bringToFront();
+							
+							Object[]	params = (Object[])data;
+							
+							long	_styles = ((Long)(params[2])).longValue();
+							
+							int		styles	= 0;
+							
+							if (( _styles & UIManagerEvent.MT_YES ) != 0 ){
+								
+								styles |= SWT.YES;
+							}
+							if (( _styles & UIManagerEvent.MT_NO ) != 0 ){
+								
+								styles |= SWT.NO;
+							}
+							if (( _styles & UIManagerEvent.MT_OK ) != 0 ){
+								
+								styles |= SWT.OK;
+							}
+							if (( _styles & UIManagerEvent.MT_CANCEL ) != 0 ){
+								
+								styles |= SWT.CANCEL;
+							}
+							
+
+							Shell shell = uiFunctions.getMainShell();
+							
+							if ( shell != null ){
+								
+								int _r = Utils.openMessageBox(shell, styles, 
+										MessageText.getString((String)params[0]), 
+										MessageText.getString((String)params[1]));
+								
+								int	r = 0;
+								
+								if (( _r & SWT.YES ) != 0 ){
+									
+									r |= UIManagerEvent.MT_YES;
+								}
+								if (( _r & SWT.NO ) != 0 ){
+									
+									r |= UIManagerEvent.MT_NO;
+								}
+								if (( _r & SWT.OK ) != 0 ){
+									
+									r |= UIManagerEvent.MT_OK;
+								}
+								if (( _r & SWT.CANCEL ) != 0 ){
+									
+									r |= UIManagerEvent.MT_CANCEL;
+								}
+								
+								result[0] = r;
+							}
+						}
+					}, false );
+				
+				event.setResult( new Long( result[0] ));
+				
+				break;
+			}
 			case UIManagerEvent.ET_OPEN_TORRENT_VIA_FILE:
 			{	
 				TorrentOpener.openTorrent(((File)data).toString());
 
 				break;
 			}
+			case UIManagerEvent.ET_OPEN_TORRENT_VIA_TORRENT:
+			{	
+				Torrent t = (Torrent)data;
+				
+				try{
+					File f = AETemporaryFileHandler.createTempFile();
+					
+					t.writeToFile( f );
+					
+					TorrentOpener.openTorrent( f.toString());
+
+				}catch( Throwable e ){
+					
+					Debug.printStackTrace( e );
+				}
+				
+				break;
+			}
 			case UIManagerEvent.ET_OPEN_TORRENT_VIA_URL:
 			{
 				Display display = SWTThread.getInstance().getDisplay();
@@ -172,10 +261,11 @@ UISWTInstanceImpl
 					public void runSupport() {
 						Object[] params = (Object[]) data;
 
-						URL target = (URL) params[0];
-						URL referrer = (URL) params[1];
-						boolean auto_download = ((Boolean) params[2]).booleanValue();
-
+						URL 		target 				= (URL) params[0];
+						URL 		referrer 			= (URL) params[1];
+						boolean 	auto_download 		= ((Boolean) params[2]).booleanValue();
+						Map			request_properties	= (Map)params[3];
+						
 						// programmatic request to add a torrent, make sure az is visible
 
 						if (!COConfigurationManager.getBooleanParameter("add_torrents_silently")) {
@@ -186,7 +276,8 @@ UISWTInstanceImpl
 							Shell shell = uiFunctions.getMainShell();
 							if (shell != null) {
 								new FileDownloadWindow(core, shell, target.toString(),
-										referrer == null ? null : referrer.toString());
+										referrer == null ? null : referrer.toString(),
+										request_properties );
 							}
 						} else {
 
@@ -205,6 +296,9 @@ UISWTInstanceImpl
 					BasicPluginViewModel model = (BasicPluginViewModel)data;
 					
 					// property bundles can't handle spaces in keys
+					//
+					// If this behaviour changes, change the openView(model)
+					// method lower down.
 					String sViewID = model.getName().replaceAll(" ", ".");
 					BasicPluginViewImpl view = new BasicPluginViewImpl(model);
 					addView(UISWTInstance.VIEW_MAIN, sViewID, view);
@@ -217,6 +311,9 @@ UISWTInstanceImpl
 				if ( data instanceof BasicPluginViewModel ){
 					BasicPluginViewModel model = (BasicPluginViewModel)data;
 					// property bundles can't handle spaces in keys
+					//
+					// If this behaviour changes, change the openView(model)
+					// method lower down.
 					String sViewID = model.getName().replaceAll(" ", ".");
 					removeViews(UISWTInstance.VIEW_MAIN, sViewID);
 				}
@@ -290,14 +387,25 @@ UISWTInstanceImpl
 				break;
 			} 
 			case UIManagerEvent.ET_ADD_TABLE_CONTEXT_MENU_ITEM:{
-				
 				TableContextMenuItem	item = (TableContextMenuItem)data;
-				
 				TableContextMenuManager.getInstance().addContextMenuItem(item);
-				
 				break;
 			}
-			
+			case UIManagerEvent.ET_ADD_MENU_ITEM: {
+				MenuItem item = (MenuItem)data;
+				MenuItemManager.getInstance().addMenuItem(item);
+				break;
+			}
+			case UIManagerEvent.ET_REMOVE_TABLE_CONTEXT_MENU_ITEM:{
+				TableContextMenuItem item = (TableContextMenuItem)data;
+				TableContextMenuManager.getInstance().removeContextMenuItem(item);
+				break;
+			}
+			case UIManagerEvent.ET_REMOVE_MENU_ITEM: {
+				MenuItem item = (MenuItem)data;
+				MenuItemManager.getInstance().removeMenuItem(item);
+				break;
+			}			
 			case UIManagerEvent.ET_SHOW_CONFIG_SECTION: {
 				event.setResult(new Boolean(false));
 
@@ -308,6 +416,18 @@ UISWTInstanceImpl
 
 				break;
 			}
+			case UIManagerEvent.ET_FILE_OPEN: {
+				File file_to_use = (File)data;
+				Utils.launch(file_to_use.getAbsolutePath());
+				break;
+			}
+			case UIManagerEvent.ET_FILE_SHOW: {
+				File file_to_use = (File)data;
+				final boolean use_open_containing_folder = COConfigurationManager.getBooleanParameter("MyTorrentsView.menu.show_parent_folder_enabled");
+				ManagerUtils.open(file_to_use, use_open_containing_folder);
+				break;
+			}			
+			
 			default:
 			{
 				done	= false;
@@ -357,6 +477,7 @@ UISWTInstanceImpl
 	}
   
 
+	/** @deprecated */
 	public void addView(final UISWTPluginView view, boolean bAutoOpen) {
 		try {
 			uiFunctions.addPluginView(view);
@@ -377,6 +498,7 @@ UISWTInstanceImpl
 		}
 	}
 
+	/** @deprecated */
 	public void
 	addView(
 		final UISWTAWTPluginView	view,
@@ -521,7 +643,6 @@ UISWTInstanceImpl
 			Utils.execSWTThread(new AERunnable() {
 				public void runSupport() {
 					try {
-						UIFunctionsSWT uiFunctions = UIFunctionsManagerSWT.getUIFunctionsSWT();
 						if (uiFunctions != null) {
 							uiFunctions.removePluginView(sViewID);
 						}
@@ -533,9 +654,14 @@ UISWTInstanceImpl
 		}
 		subViews.remove(sViewID);
 	}
-	
+
 	public boolean openView(final String sParentID, final String sViewID,
 			final Object dataSource) {
+		return openView(sParentID, sViewID, dataSource, true);
+	}
+	
+	public boolean openView(final String sParentID, final String sViewID,
+			final Object dataSource, final boolean setfocus) {
 		Map subViews = (Map) views.get(sParentID);
 		if (subViews == null) {
 			return false;
@@ -548,10 +674,9 @@ UISWTInstanceImpl
 
 		Utils.execSWTThread(new AERunnable() {
 			public void runSupport() {
-				UIFunctionsSWT uiFunctions = UIFunctionsManagerSWT.getUIFunctionsSWT();
 				if (uiFunctions != null) {
 					uiFunctions.openPluginView(sParentID, sViewID, l, dataSource,
-							!bUIAttaching);
+							setfocus && !bUIAttaching);
 				}
 			}
 		});
@@ -561,11 +686,16 @@ UISWTInstanceImpl
 
 	public void openMainView(final String sViewID,
 			final UISWTViewEventListener l, final Object dataSource) {
+		openMainView(sViewID, l, dataSource, true);
+	}
+	
+	public void openMainView(final String sViewID,
+			final UISWTViewEventListener l, final Object dataSource,
+			final boolean setfocus) {
 		Utils.execSWTThread(new AERunnable() {
 			public void runSupport() {
-				UIFunctionsSWT uiFunctions = UIFunctionsManagerSWT.getUIFunctionsSWT();
 				if (uiFunctions != null) {
-					uiFunctions.openPluginView(UISWTInstance.VIEW_MAIN, sViewID, l, dataSource, !bUIAttaching);
+					uiFunctions.openPluginView(UISWTInstance.VIEW_MAIN, sViewID, l, dataSource, setfocus && !bUIAttaching);
 				}
 			}
 		});
@@ -574,7 +704,6 @@ UISWTInstanceImpl
 	public UISWTView[] getOpenViews(String sParentID) {
 		if (sParentID.equals(UISWTInstance.VIEW_MAIN)) {
 			try {
-				UIFunctionsSWT uiFunctions = UIFunctionsManagerSWT.getUIFunctionsSWT();
 				if (uiFunctions != null) {
 					return uiFunctions.getPluginViews();
 				}
@@ -585,6 +714,7 @@ UISWTInstanceImpl
 		return new UISWTView[0];
 	}
 	
+	// @see org.gudy.azureus2.plugins.ui.UIInstance#promptUser(java.lang.String, java.lang.String, java.lang.String[], int)
 	public int promptUser(String title, String text, String[] options,
 			int defaultOption) {
 		return MessageBoxShell.open(uiFunctions.getMainShell(), title, text,
@@ -597,13 +727,24 @@ UISWTInstanceImpl
 		if (dm == null) {return;} // Not expecting this, but just in case...
 		Utils.execSWTThread(new AERunnable() {
 			public void runSupport() {
-				boolean displayed = MinimizedWindow.isOpen(dm);
-				if (display == displayed) {return;}
 				if (display) {
-					new MinimizedWindow(dm, getDisplay().getActiveShell());
+					DownloadBar.open(dm, getDisplay().getActiveShell());
 				}
 				else {
-					MinimizedWindow.close(dm);
+					DownloadBar.close(dm);
+				}
+			}
+		}, false);
+	}
+	
+	public void showTransfersBar(final boolean display) {
+		Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+				if (display) {
+					AllTransfersBar.open(core.getGlobalManager(), getDisplay().getActiveShell());
+				}
+				else {
+					AllTransfersBar.close(core.getGlobalManager());
 				}
 			}
 		}, false);
@@ -620,6 +761,48 @@ UISWTInstanceImpl
 		return new SimpleTextEntryWindow(getDisplay());
 	}
 	
+	public UIMessage createMessage() {
+		return new UIMessageImpl();
+	}
+	
+	public UISWTStatusEntry createStatusEntry() {
+		final UISWTStatusEntryImpl entry = new UISWTStatusEntryImpl();
+		UIFunctionsSWT functionsSWT = UIFunctionsManagerSWT.getUIFunctionsSWT();
+		if (functionsSWT == null) {
+			return null;
+		}
+		
+		MainStatusBar mainStatusBar = functionsSWT.getMainStatusBar();
+		if (mainStatusBar == null) {
+			return null;
+		}
+		final CLabel label = mainStatusBar.createStatusEntry(entry);
+		final Listener click_listener = new Listener() {
+			public void handleEvent(Event e) {
+				entry.onClick();
+			}
+		};
+
+		Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+				label.addListener(SWT.MouseDoubleClick, click_listener);
+			}
+		}, true);
+		
+		return entry;
+	}
+	
+	public boolean openView(BasicPluginViewModel model) {
+		return openView(VIEW_MAIN, model.getName().replaceAll(" ", "."), null);
+	}
+	
+	public void openConfig(final BasicPluginConfigModel model) {
+		Utils.execSWTThread(new Runnable() {
+			public void run() {
+				uiFunctions.showConfig(model.getSection());
+			}
+		});
+	}
 	
 	protected static class
 	instanceWrapper
@@ -677,6 +860,13 @@ UISWTInstanceImpl
 			delegate.openMainView( sViewID, l, dataSource );
 		}
 
+		public void 
+		openMainView(String sViewID, UISWTViewEventListener l,Object dataSource, boolean setfocus)
+		{
+			delegate.openMainView( sViewID, l, dataSource, setfocus );
+		}
+
+		
 		public void 
 		removeViews(String sParentID, String sViewID)
 		{
@@ -726,14 +916,40 @@ UISWTInstanceImpl
 			return delegate.openView(sParentID, sViewID, dataSource);
 		}
 		
+		public boolean openView(String sParentID, String sViewID, Object dataSource, boolean setfocus) {
+			return delegate.openView(sParentID, sViewID, dataSource, setfocus);
+		}
+		
 		public UIInputReceiver getInputReceiver() {
 			return delegate.getInputReceiver();
 		}
 		
+		public UIMessage createMessage() {
+			return delegate.createMessage();
+		}
+		
 		public void showDownloadBar(Download download, boolean display) {
 			delegate.showDownloadBar(download, display);
 		}
 		
+		public void showTransfersBar(boolean display) {
+			delegate.showTransfersBar(display);
+		}
+		
+		public UISWTStatusEntry createStatusEntry() {
+			return delegate.createStatusEntry();
+		}
+		
+		public boolean openView(BasicPluginViewModel model) {
+			return delegate.openView(model);
+		}
+
+		public void openConfig(BasicPluginConfigModel model) {
+			delegate.openConfig(model);
+		}
 		
 	}
+	
+	
+	
 }
