@@ -105,9 +105,10 @@ public class IPWFilePanel extends AbstractWizardPanel {
   
   private void checkValidFile() {
     String fileName = txtFile.getText();
+    String error_message = null;
     try {
       File f = new File(fileName);
-      if(f.exists() && (f.getName().endsWith(".jar") || f.getName().endsWith(".zip")) ) {
+      if(f.isFile() && (f.getName().endsWith(".jar") || f.getName().endsWith(".zip")) ) {
         wizard.setErrorMessage("");
         wizard.setNextEnabled(true);
         List list = new ArrayList();
@@ -116,14 +117,27 @@ public class IPWFilePanel extends AbstractWizardPanel {
         ((InstallPluginWizard)wizard).plugins = list;
         valid = true;
         return;
-      }           
+      }
+    } catch (org.gudy.azureus2.plugins.PluginException e) {
+    	error_message = e.getMessage();
+    	Debug.printStackTrace(e);
     } catch(Exception e) {    
-    	
+    	error_message = null;
     	Debug.printStackTrace(e);
     }
     valid = false;
-    wizard.setErrorMessage(MessageText.getString("installPluginsWizard.file.invalidfile"));
-    wizard.setNextEnabled(false);
+    if (!fileName.equals("")) {
+    	String error_message_full;
+    	if (new File(fileName).isFile()) {
+    		error_message_full = MessageText.getString("installPluginsWizard.file.invalidfile");
+    	}
+    	else {
+    		error_message_full = MessageText.getString("installPluginsWizard.file.no_such_file");
+    	}
+    	if (error_message != null) {error_message_full += " (" + error_message + ")";}
+    	wizard.setErrorMessage(error_message_full);
+    	wizard.setNextEnabled(false);
+    }
   }
   
 	public boolean 
