@@ -20,7 +20,6 @@
 package org.gudy.azureus2.ui.swt.shells;
 
 import java.util.ArrayList;
-import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import org.eclipse.swt.SWT;
@@ -39,10 +38,11 @@ import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.ui.swt.ImageRepository;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.Utils;
-import org.gudy.azureus2.ui.swt.mainwindow.TorrentOpener;
+import org.gudy.azureus2.ui.swt.mainwindow.ClipboardCopy;
+import org.gudy.azureus2.ui.swt.shells.GCStringPrinter.URLInfo;
 
-import com.aelitis.azureus.ui.swt.UIFunctionsManagerSWT;
-import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
+import com.aelitis.azureus.ui.swt.*;
+import com.aelitis.azureus.ui.swt.utils.ColorCache;
 
 /**
  * 
@@ -62,7 +62,8 @@ import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
  * @created Mar 7, 2006
  *
  */
-public class MessageSlideShell {
+public class MessageSlideShell
+{
 	private static boolean USE_SWT32_BG_SET = true;
 
 	private static final boolean DEBUG = false;
@@ -98,6 +99,9 @@ public class MessageSlideShell {
 	/** Current popup being displayed */
 	private static int currentPopupIndex = -1;
 
+	/** Index of first message which the user has not seen (index) - set to -1 if we don't care. :) **/
+	private static int firstUnreadMessage = -1;
+
 	/** Shell for popup */
 	private Shell shell;
 
@@ -119,7 +123,7 @@ public class MessageSlideShell {
 
 	/** paused state of auto-close delay */
 	private boolean bDelayPaused = false;
-	
+
 	/** List of SWT objects needing disposal */
 	private ArrayList disposeList = new ArrayList();
 
@@ -129,6 +133,17 @@ public class MessageSlideShell {
 	/** Position this popup is in the history list */
 	private int idxHistory;
 
+	private Image imgPopup;
+
+	/** Forces the timer feature to be active; overriding the default behavior */
+	private boolean forceTimer = true;
+
+	protected Color colorURL;
+	
+	private Color colorFG;
+
+	private int shellWidth;
+
 	/** Open a popup using resource keys for title/text
 	 * 
 	 * @param display Display to create the shell on
@@ -147,6 +162,36 @@ public class MessageSlideShell {
 				MessageText.getString(keyPrefix + ".text", textParams), details);
 	}
 
+	public MessageSlideShell(Display display, int iconID, String keyPrefix,
+			String details, String[] textParams, Object[] relatedObjects) {
+		this(display, iconID, MessageText.getString(keyPrefix + ".title"),
+				MessageText.getString(keyPrefix + ".text", textParams), details,
+				relatedObjects, false);
+	}
+
+	/**
+	 * 
+	 * @param display
+	 * @param iconID
+	 * @param keyPrefix
+	 * @param details
+	 * @param textParams
+	 * @param relatedObjects
+	 * @param forceTimer Forces the timer feature to be active; overriding the default logic
+	 */
+	public MessageSlideShell(Display display, int iconID, String keyPrefix,
+			String details, String[] textParams, Object[] relatedObjects,
+			boolean forceTimer) {
+		this(display, iconID, MessageText.getString(keyPrefix + ".title"),
+				MessageText.getString(keyPrefix + ".text", textParams), details,
+				relatedObjects, forceTimer);
+	}
+
+	public MessageSlideShell(Display display, int iconID, String title,
+			String text, String details) {
+		this(display, iconID, title, text, details, null, false);
+	}
+
 	/**
 	 * Open Mr Slidey
 	 * 
@@ -156,13 +201,17 @@ public class MessageSlideShell {
 	 * @param text Text to put in the body
 	 * @param details Text displayed when the Details button is pressed.  Null
 	 *                 for disabled Details button.
+	 * @param forceTimer Forces the timer feature to be active; overriding the default logic
 	 */
 	public MessageSlideShell(Display display, int iconID, String title,
-			String text, String details) {
+			String text, String details, Object[] relatedObjects, boolean forceTimer) {
 		try {
 			monitor.enter();
 
-			PopupParams popupParams = new PopupParams(iconID, title, text, details);
+			this.forceTimer = forceTimer;
+
+			PopupParams popupParams = new PopupParams(iconID, title, text, details,
+					relatedObjects);
 			historyList.add(popupParams);
 			if (currentPopupIndex < 0) {
 				create(display, popupParams, true);
@@ -181,10 +230,49 @@ public class MessageSlideShell {
 		create(display, popupParams, bSlide);
 	}
 
+	public static void displayLastMessage(final Display display,
+			final boolean last_unread) {
+		display.asyncExec(new AERunnable() {
+			public void runSupport() {
+				if (historyList.isEmpty()) {
+					return;
+				}
+				if (currentPopupIndex >= 0) {
+					return;
+				} // Already being displayed.
+				int msg_index = firstUnreadMessage;
+				if (!last_unread || msg_index == -1) {
+					msg_index = historyList.size() - 1;
+				}
+				new MessageSlideShell(display,
+						(PopupParams) historyList.get(msg_index), true);
+			}
+		});
+	}
+
+	/**
+	 * Adds this message to the slide shell without forcing it to be displayed.
+	 * @param relatedTo 
+	 */
+	public static void recordMessage(int iconID, String title, String text,
+			String details, Object[] relatedTo) {
+		try {
+			monitor.enter();
+			historyList.add(new PopupParams(iconID, title, text, details, relatedTo));
+			if (firstUnreadMessage == -1) {
+				firstUnreadMessage = historyList.size() - 1;
+			}
+		} finally {
+			monitor.exit();
+		}
+	}
+
 	private void create(final Display display, final PopupParams popupParams,
 			boolean bSlide) {
+
+		firstUnreadMessage = -1; // Reset the last read message counter.
+
 		GridData gridData;
-		int shellWidth;
 		int style = SWT.ON_TOP;
 
 		boolean bDisableSliding = COConfigurationManager.getBooleanParameter("GUI_SWT_DisableAlertSliding");
@@ -224,12 +312,13 @@ public class MessageSlideShell {
 
 		// Load Images
 		// Disable BG Image on OSX
-		Image imgPopup;
-		if (Constants.isOSX && (SWT.getVersion() < 3221 || !USE_SWT32_BG_SET)) {
-			USE_SWT32_BG_SET = false;
-			imgPopup = null;
-		} else {
-			imgPopup = ImageRepository.getImage("popup");
+		if (imgPopup == null) {
+			if (Constants.isOSX && (SWT.getVersion() < 3221 || !USE_SWT32_BG_SET)) {
+				USE_SWT32_BG_SET = false;
+				imgPopup = null;
+			} else {
+				imgPopup = ImageRepository.getImage("popup");
+			}
 		}
 		Rectangle imgPopupBounds;
 		if (imgPopup != null) {
@@ -239,29 +328,20 @@ public class MessageSlideShell {
 			shellWidth = SHELL_DEF_WIDTH;
 			imgPopupBounds = null;
 		}
-		Image imgIcon = null;
-		switch (popupParams.iconID) {
-			case SWT.ICON_ERROR:
-				imgIcon = ImageRepository.getImage("error");
-				break;
+		Image imgIcon = popupParams.iconID <= 0 ? null : display.getSystemImage(popupParams.iconID);
 
-			case SWT.ICON_WARNING:
-				imgIcon = ImageRepository.getImage("warning");
-				break;
-
-			case SWT.ICON_INFORMATION:
-				imgIcon = ImageRepository.getImage("info");
-				break;
-
-			default:
-				imgIcon = null;
-				break;
+		/*
+		 * If forceTimer is true then we always show the counter for auto-closing the shell;
+		 * otherwise proceed to the more fine-grained logic
+		 */
+		if (true == forceTimer) {
+			bDelayPaused = false;
+		} else {
+			// if there's a link, or the info is non-information,
+			// disable timer and mouse watching
+			bDelayPaused = UrlUtils.parseHTMLforURL(popupParams.text) != null
+					|| popupParams.iconID != SWT.ICON_INFORMATION || !bSlide;
 		}
-		
-		// if there's a link, or the info is non-information,
-		// disable timer and mouse watching
-		bDelayPaused = UrlUtils.parseHTMLforURL(popupParams.text) != null
-				|| popupParams.iconID != SWT.ICON_INFORMATION || !bSlide;
 		// Pause the auto-close delay when mouse is over slidey
 		// This will be applies to every control
 		final MouseTrackAdapter mouseAdapter = bDelayPaused ? null
@@ -300,6 +380,20 @@ public class MessageSlideShell {
 		Utils.setShellIcon(shell);
 		shell.setText(popupParams.title);
 
+		UISkinnableSWTListener[] listeners = UISkinnableManagerSWT.getInstance().getSkinnableListeners(
+				MessageSlideShell.class.toString());
+		for (int i = 0; i < listeners.length; i++) {
+			try {
+				listeners[i].skinBeforeComponents(shell, this, popupParams.relatedTo);
+			} catch (Exception e) {
+				Debug.out(e);
+			}
+		}
+		
+		if (colorFG == null) {
+			colorFG = display.getSystemColor(SWT.COLOR_BLACK);
+		}
+
 		FormLayout shellLayout = new FormLayout();
 		shell.setLayout(shellLayout);
 
@@ -322,7 +416,7 @@ public class MessageSlideShell {
 		if (SWT.getVersion() < 3100)
 			gridData.widthHint = 140;
 		lblTitle.setLayoutData(gridData);
-		lblTitle.setForeground(display.getSystemColor(SWT.COLOR_BLACK));
+		lblTitle.setForeground(colorFG);
 		lblTitle.setText(popupParams.title);
 		FontData[] fontData = lblTitle.getFont().getFontData();
 		fontData[0].setStyle(SWT.BOLD);
@@ -332,7 +426,7 @@ public class MessageSlideShell {
 		lblTitle.setFont(boldFont);
 
 		final Button btnDetails = new Button(cShell, SWT.TOGGLE);
-		btnDetails.setForeground(display.getSystemColor(SWT.COLOR_BLACK));
+		btnDetails.setForeground(colorFG);
 		Messages.setLanguageText(btnDetails, "popup.error.details");
 		gridData = new GridData();
 		btnDetails.setLayoutData(gridData);
@@ -381,59 +475,10 @@ public class MessageSlideShell {
 			}
 		});
 
-		try {
-			Link linkLabel = new Link(cShell, SWT.WRAP);
-			gridData = new GridData(GridData.FILL_BOTH);
-			gridData.horizontalSpan = 3;
-			linkLabel.setLayoutData(gridData);
-			linkLabel.setForeground(display.getSystemColor(SWT.COLOR_BLACK));
-			linkLabel.setText(popupParams.text);
-			linkLabel.addSelectionListener(new SelectionAdapter() {
-				public void widgetSelected(SelectionEvent e) {
-					if (e.text.endsWith(".torrent"))
-						TorrentOpener.openTorrent(e.text);
-					else
-						Utils.launch(e.text);
-				}
-			});
-
-			Matcher matcher = Pattern
-					.compile(REGEX_URLHTML, Pattern.CASE_INSENSITIVE).matcher(
-							popupParams.text);
-			String tooltip = null;
-			while (matcher.find()) {
-				if (tooltip == null)
-					tooltip = "";
-				else
-					tooltip += "\n";
-				tooltip += matcher.group(2) + ": " + matcher.group(1);
-			}
-			linkLabel.setToolTipText(tooltip);
-		} catch (Throwable t) {
-			// 3.0
-			Label linkLabel = new Label(cShell, SWT.WRAP);
-			gridData = new GridData(GridData.FILL_BOTH);
-			gridData.horizontalSpan = 3;
-			linkLabel.setLayoutData(gridData);
-
-			//<a href="http://atorre.s">test</A> and <a href="http://atorre.s">test2</A>
-
-			popupParams.text = Pattern.compile(REGEX_URLHTML,
-					Pattern.CASE_INSENSITIVE).matcher(popupParams.text).replaceAll(
-					"$2 ($1)");
-
-			if (sDetails == null) {
-				sDetails = popupParams.text;
-			} else {
-				sDetails = popupParams.text + "\n---------\n" + sDetails;
-			}
-
-			linkLabel.setForeground(display.getSystemColor(SWT.COLOR_BLACK));
-			linkLabel.setText(popupParams.text);
-		}
+		createLinkLabel(cShell, popupParams);
 
 		lblCloseIn = new Label(cShell, SWT.TRAIL);
-		lblCloseIn.setForeground(display.getSystemColor(SWT.COLOR_BLACK));
+		lblCloseIn.setForeground(colorFG);
 		// Ensure computeSize computes for 2 lined label
 		lblCloseIn.setText("\n");
 		gridData = new GridData(SWT.FILL, SWT.TOP, true, false);
@@ -470,8 +515,9 @@ public class MessageSlideShell {
 		if (idxHistory > 0) {
 			final Button btnPrev = new Button(cButtons, SWT.PUSH);
 			btnPrev.setForeground(display.getSystemColor(SWT.COLOR_BLACK));
-			btnPrev.setText(MessageText.getString("popup.previous", new String[] { ""
-					+ idxHistory }));
+			btnPrev.setText(MessageText.getString("popup.previous", new String[] {
+				"" + idxHistory
+			}));
 			btnPrev.addListener(SWT.MouseUp, new Listener() {
 				public void handleEvent(Event arg0) {
 					disposeShell(shell);
@@ -600,7 +646,7 @@ public class MessageSlideShell {
 
 							e.gc.drawImage(imgTransparent, 0, 0, e.width, e.height, e.x, e.y,
 									e.width, e.height);
-							
+
 							img.dispose();
 							imgTransparent.dispose();
 						} finally {
@@ -617,27 +663,27 @@ public class MessageSlideShell {
 
 		Rectangle bounds = null;
 		try {
-    	UIFunctionsSWT uiFunctions = UIFunctionsManagerSWT.getUIFunctionsSWT();
-    	if (uiFunctions != null) {
+			UIFunctionsSWT uiFunctions = UIFunctionsManagerSWT.getUIFunctionsSWT();
+			if (uiFunctions != null) {
 				Shell mainShell = uiFunctions.getMainShell();
 				if (mainShell != null) {
 					bounds = mainShell.getMonitor().getClientArea();
 				}
-    	} else {
-    		Shell shell = display.getActiveShell();
+			} else {
+				Shell shell = display.getActiveShell();
 				if (shell != null) {
 					bounds = shell.getMonitor().getClientArea();
 				}
-    	}
-    	if (bounds == null) {
+			}
+			if (bounds == null) {
 				bounds = shell.getMonitor().getClientArea();
-    	}
+			}
 		} catch (Exception e) {
 		}
 		if (bounds == null) {
 			bounds = display.getClientArea();
 		}
-		
+
 		Rectangle endBounds;
 		if (bDisableSliding) {
 			endBounds = new Rectangle(((bounds.x + bounds.width) / 2)
@@ -679,8 +725,7 @@ public class MessageSlideShell {
 
 				if (currentPopupIndex == idxHistory) {
 					if (DEBUG)
-						System.out
-								.println("Clear #" + currentPopupIndex + "/" + idxHistory);
+						System.out.println("Clear #" + currentPopupIndex + "/" + idxHistory);
 					try {
 						monitor.enter();
 						currentPopupIndex = -1;
@@ -703,9 +748,93 @@ public class MessageSlideShell {
 		if (mouseAdapter != null)
 			addMouseTrackListener(shell, mouseAdapter);
 
+		for (int i = 0; i < listeners.length; i++) {
+			try {
+				listeners[i].skinAfterComponents(shell, this, popupParams.relatedTo);
+			} catch (Exception e) {
+				Debug.out(e);
+			}
+		}
+
 		runPopup(endBounds, idxHistory, bSlide);
 	}
 
+	/**
+	 * @param shell2
+	 * @param b
+	 *
+	 * @since 3.0.0.9
+	 */
+	private void createLinkLabel(Composite shell, final PopupParams popupParams) {
+
+		final Canvas canvas = new Canvas(shell, SWT.None) {
+			public Point computeSize(int wHint, int hHint, boolean changed) {
+				Rectangle area = new Rectangle(0, 0, shellWidth, 5000);
+				GC gc = new GC(this);
+				GCStringPrinter sp = new GCStringPrinter(gc, popupParams.text, area, true,
+						false, SWT.WRAP | SWT.TOP);
+				sp.calculateMetrics();
+				gc.dispose();
+				Point size = sp.getCalculatedSize();
+				return size;
+			}
+		};
+
+		Listener l = new Listener() {
+			GCStringPrinter sp;
+
+			public void handleEvent(Event e) {
+				if (e.type == SWT.Paint) {
+					Rectangle area = canvas.getClientArea();
+					sp = new GCStringPrinter(e.gc, popupParams.text, area, true, false,
+							SWT.WRAP | SWT.TOP);
+					sp.setUrlColor(ColorCache.getColor(e.gc.getDevice(), "#0000ff"));
+					if (colorURL != null) {
+						sp.setUrlColor(colorURL);
+					}
+					if (colorFG != null) {
+						e.gc.setForeground(colorFG);
+					}
+					sp.printString();
+				} else if (e.type == SWT.MouseMove) {
+					if (sp != null) {
+						URLInfo hitUrl = sp.getHitUrl(e.x, e.y);
+						if (hitUrl != null) {
+							canvas.setCursor(canvas.getDisplay().getSystemCursor(
+									SWT.CURSOR_HAND));
+							canvas.setToolTipText(hitUrl.url);
+						} else {
+							canvas.setCursor(canvas.getDisplay().getSystemCursor(
+									SWT.CURSOR_ARROW));
+							canvas.setToolTipText(null);
+						}
+					}
+				} else if (e.type == SWT.MouseUp) {
+					if (sp != null) {
+						URLInfo hitUrl = sp.getHitUrl(e.x, e.y);
+						if (hitUrl != null) {
+							Utils.launch(hitUrl.url);
+						}
+					}
+				}
+			}
+		};
+		canvas.addListener(SWT.Paint, l);
+		canvas.addListener(SWT.MouseMove, l);
+		canvas.addListener(SWT.MouseUp, l);
+
+		ClipboardCopy.addCopyToClipMenu(canvas,
+				new ClipboardCopy.copyToClipProvider() {
+					public String getText() {
+						return (popupParams.title + "\n\n" + popupParams.text);
+					}
+				});
+
+		GridData gridData = new GridData(GridData.FILL_BOTH);
+		gridData.horizontalSpan = 3;
+		canvas.setLayoutData(gridData);
+	}
+
 	/**
 	 * @param numAfter
 	 */
@@ -713,8 +842,9 @@ public class MessageSlideShell {
 		if (numAfter <= 0)
 			Messages.setLanguageText(btnNext, "popup.error.hide");
 		else
-			Messages.setLanguageText(btnNext, "popup.next", new String[] { ""
-					+ numAfter });
+			Messages.setLanguageText(btnNext, "popup.next", new String[] {
+				"" + numAfter
+			});
 		cShell.layout(true);
 	}
 
@@ -815,8 +945,7 @@ public class MessageSlideShell {
 		final Display display = shell.getDisplay();
 
 		if (DEBUG)
-			System.out
-					.println("runPopup " + idx + ((bSlide) ? " Slide" : " Instant"));
+			System.out.println("runPopup " + idx + ((bSlide) ? " Slide" : " Instant"));
 
 		AEThread thread = new AEThread("Slidey", true) {
 			private final static int PAUSE = 500;
@@ -826,7 +955,7 @@ public class MessageSlideShell {
 					return;
 
 				if (bSlide) {
-					new SlideShell(shell, SWT.UP, endBounds).run();
+					new ShellSlider(shell, SWT.UP, endBounds).run();
 				} else {
 					Utils.execSWTThread(new AERunnable() {
 
@@ -837,13 +966,13 @@ public class MessageSlideShell {
 					});
 				}
 
-				int delayLeft = COConfigurationManager
-						.getIntParameter("Message Popup Autoclose in Seconds") * 1000;
+				int delayLeft = COConfigurationManager.getIntParameter("Message Popup Autoclose in Seconds") * 1000;
 				final boolean autohide = (delayLeft != 0);
 
 				long lastDelaySecs = 0;
 				int lastNumPopups = -1;
-				while ((!autohide || bDelayPaused || delayLeft > 0) && !shell.isDisposed()) {
+				while ((!autohide || bDelayPaused || delayLeft > 0)
+						&& !shell.isDisposed()) {
 					int delayPausedOfs = (bDelayPaused ? 1 : 0);
 					final long delaySecs = Math.round(delayLeft / 1000.0)
 							+ delayPausedOfs;
@@ -861,14 +990,18 @@ public class MessageSlideShell {
 								lblCloseIn.setRedraw(false);
 								if (!bDelayPaused && autohide)
 									sText += MessageText.getString("popup.closing.in",
-											new String[] { String.valueOf(delaySecs) });
+											new String[] {
+												String.valueOf(delaySecs)
+											});
 
 								int numPopupsAfterUs = numPopups - idx - 1;
 								boolean bHasMany = numPopupsAfterUs > 0;
 								if (bHasMany) {
 									sText += "\n";
 									sText += MessageText.getString("popup.more.waiting",
-											new String[] { String.valueOf(numPopupsAfterUs) });
+											new String[] {
+												String.valueOf(numPopupsAfterUs)
+											});
 								}
 
 								lblCloseIn.setText(sText);
@@ -912,7 +1045,7 @@ public class MessageSlideShell {
 
 					// slide out current popup
 					if (bSlide)
-						new SlideShell(shell, SWT.RIGHT).run();
+						new ShellSlider(shell, SWT.RIGHT).run();
 
 					disposeShell(shell);
 				}
@@ -932,6 +1065,11 @@ public class MessageSlideShell {
 		});
 	}
 
+	public static String stripOutHyperlinks(String message) {
+		return Pattern.compile(REGEX_URLHTML, Pattern.CASE_INSENSITIVE).matcher(
+				message).replaceAll("$2");
+	}
+
 	/**
 	 * Waits until all slideys are closed before returning to caller.
 	 */
@@ -946,194 +1084,8 @@ public class MessageSlideShell {
 		}
 	}
 
-	/**
-	 * XXX This could/should be its own class 
-	 */
-	private class SlideShell {
-		private int STEP = 8;
-
-		private int PAUSE = 30;
-
-		private Shell shell;
-
-		private Rectangle shellBounds = null;
-
-		private Rectangle endBounds;
-
-		private final int direction;
-
-		private final boolean slideIn;
-
-		/**
-		 * Slide In
-		 * 
-		 * @param shell
-		 * @param direction 
-		 * @param endBounds 
-		 */
-		public SlideShell(final Shell shell, int direction,
-				final Rectangle endBounds) {
-			this.shell = shell;
-			this.endBounds = endBounds;
-			this.slideIn = true;
-			this.direction = direction;
-
-			if (shell == null || shell.isDisposed())
-				return;
-
-			Display display = shell.getDisplay();
-			display.syncExec(new Runnable() {
-				public void run() {
-					if (shell == null || shell.isDisposed())
-						return;
-
-					switch (SlideShell.this.direction) {
-						case SWT.UP:
-						default:
-							shell.setLocation(endBounds.x, endBounds.y);
-							Rectangle displayBounds = null;
-							try {
-								boolean ok = false;
-								Monitor[] monitors = shell.getDisplay().getMonitors();
-								for (int i = 0; i < monitors.length; i++) {
-									Monitor monitor = monitors[i];
-									displayBounds = monitor.getBounds();
-									int x = endBounds.x;
-									int y = endBounds.y;
-									if (displayBounds.contains(endBounds.x, endBounds.y)) {
-										ok = true;
-										break;
-									}
-								}
-								if (!ok) {
-									displayBounds = shell.getMonitor().getBounds();
-								}
-							} catch (Throwable t) {
-								displayBounds = shell.getDisplay().getBounds();
-							}
-
-							shellBounds = new Rectangle(endBounds.x, displayBounds.y
-									+ displayBounds.height, endBounds.width, 0);
-							break;
-					}
-					shell.setBounds(shellBounds);
-					shell.setVisible(true);
-
-					if (DEBUG)
-						System.out.println("Slide In: " + shell.getText());
-				}
-			});
-		}
-
-		/**
-		 * Slide Out
-		 * 
-		 * @param shell
-		 * @param direction
-		 */
-		public SlideShell(final Shell shell, int direction) {
-			this.shell = shell;
-			this.slideIn = false;
-			this.direction = direction;
-			if (DEBUG && canContinue())
-				shell.getDisplay().syncExec(new Runnable() {
-					public void run() {
-						System.out.println("Slide Out: " + shell.getText());
-					}
-				});
-		}
-
-		private boolean canContinue() {
-			if (shell == null || shell.isDisposed())
-				return false;
-
-			if (shellBounds == null)
-				return true;
-
-			//System.out.println((slideIn ? "In" : "Out") + ";" + direction + ";S:" + shellBounds + ";" + endBounds);
-			if (slideIn) {
-				if (direction == SWT.UP) {
-					return shellBounds.y > endBounds.y;
-				}
-				// TODO: Other directions
-			} else {
-				if (direction == SWT.RIGHT) {
-					// stop early, because some OSes have trim, and won't allow the window
-					// to go smaller than it.
-					return shellBounds.width > 10;
-				}
-			}
-			return false;
-		}
-
-		public void run() {
-
-			while (canContinue()) {
-				long lStartedAt = System.currentTimeMillis();
-
-				shell.getDisplay().syncExec(new AERunnable() {
-					public void runSupport() {
-						if (shell == null || shell.isDisposed()) {
-							return;
-						}
-
-						if (shellBounds == null) {
-							shellBounds = shell.getBounds();
-						}
-
-						int delta;
-						if (slideIn) {
-							switch (direction) {
-								case SWT.UP:
-									delta = Math.min(endBounds.height - shellBounds.height, STEP);
-									shellBounds.height += delta;
-									delta = Math.min(shellBounds.y - endBounds.y, STEP);
-									shellBounds.y -= delta;
-									break;
-
-								default:
-									break;
-							}
-						} else {
-							switch (direction) {
-								case SWT.RIGHT:
-									delta = Math.min(shellBounds.width, STEP);
-									shellBounds.width -= delta;
-									shellBounds.x += delta;
-
-									if (shellBounds.width == 0) {
-										shell.dispose();
-										return;
-									}
-									break;
-
-								default:
-									break;
-							}
-						}
-
-						shell.setBounds(shellBounds);
-						shell.update();
-					}
-				});
-
-				try {
-					long lDrawTime = System.currentTimeMillis() - lStartedAt;
-					long lSleepTime = PAUSE - lDrawTime;
-					if (lSleepTime < 15) {
-						double d = (lDrawTime + 15.0) / PAUSE;
-						PAUSE *= d;
-						STEP *= d;
-						lSleepTime = 15;
-					}
-					Thread.sleep(lSleepTime);
-				} catch (Exception e) {
-				}
-			}
-		}
-	}
-
-	private class PopupParams {
+	private static class PopupParams
+	{
 		int iconID;
 
 		String title;
@@ -1144,6 +1096,8 @@ public class MessageSlideShell {
 
 		long addedOn;
 
+		Object[] relatedTo;
+
 		/**
 		 * @param iconID
 		 * @param title
@@ -1157,6 +1111,19 @@ public class MessageSlideShell {
 			this.details = details;
 			addedOn = System.currentTimeMillis();
 		}
+
+		/**
+		 * @param iconID2
+		 * @param title2
+		 * @param text2
+		 * @param details2
+		 * @param relatedTo
+		 */
+		public PopupParams(int iconID, String title, String text, String details,
+				Object[] relatedTo) {
+			this(iconID, title, text, details);
+			this.relatedTo = relatedTo;
+		}
 	}
 
 	/**
@@ -1166,7 +1133,7 @@ public class MessageSlideShell {
 	 */
 	public static void main(String[] args) {
 		final Display display = Display.getDefault();
-		
+
 		Shell shell = new Shell(display, SWT.DIALOG_TRIM);
 		shell.setLayout(new FillLayout());
 		Button btn = new Button(shell, SWT.PUSH);
@@ -1176,14 +1143,14 @@ public class MessageSlideShell {
 			}
 		});
 		shell.open();
-		
+
 		while (!shell.isDisposed()) {
 			if (!display.readAndDispatch()) {
 				display.sleep();
 			}
 		}
 	}
-	
+
 	public static void test(Display display) {
 
 		ImageRepository.loadImages(display);
@@ -1231,4 +1198,34 @@ public class MessageSlideShell {
 
 		MessageSlideShell.waitUntilClosed();
 	}
+
+	/**
+	 * @return the imgPopup
+	 */
+	public Image getImgPopup() {
+		return imgPopup;
+	}
+
+	/**
+	 * @param imgPopup the imgPopup to set
+	 */
+	public void setImgPopup(Image imgPopup) {
+		this.imgPopup = imgPopup;
+	}
+
+	public Color getUrlColor() {
+		return colorURL;
+	}
+
+	public void setUrlColor(Color urlColor) {
+		this.colorURL = urlColor;
+	}
+
+	public Color getColorFG() {
+		return colorFG;
+	}
+
+	public void setColorFG(Color colorFG) {
+		this.colorFG = colorFG;
+	}
 }
