@@ -39,6 +39,7 @@ import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.core3.util.FileUtil;
 import org.gudy.azureus2.core3.util.SystemProperties;
+import org.gudy.azureus2.core3.util.SystemTime;
 import org.gudy.azureus2.plugins.update.UpdatableComponent;
 import org.gudy.azureus2.plugins.update.Update;
 import org.gudy.azureus2.plugins.update.UpdateChecker;
@@ -125,22 +126,38 @@ public class SWTUpdateChecker implements UpdatableComponent
 		        	    		}
 		        	    	}else{
 		        	    		
-		        		    	if ( update_prevented_version != versionGetter.getCurrentVersion()){
-			        		    		
-			        	    		String	alert = 
-			        	    			MessageText.getString( 
-			        	    					"swt.alert.cant.update", 
-			        	    					new String[]{
-				        	    					String.valueOf( versionGetter.getCurrentVersion()),
-				        	    					String.valueOf( versionGetter.getLatestVersion()),
-			        	    						jar_file_dir.toString(), 
-			        	    						expected_dir.toString()});
-			        	    		
-			        	     		Logger.log(	new LogAlert(LogAlert.UNREPEATABLE, LogEvent.LT_ERROR, alert ));
+		        	    			// we need to periodically remind the user there's a problem as they need to realise that
+		        	    			// it is causing ALL updates (core/plugin) to fail
+		        	    		
+		        	    		String	alert = 
+		        	    			MessageText.getString( 
+		        	    					"swt.alert.cant.update", 
+		        	    					new String[]{
+			        	    					String.valueOf( versionGetter.getCurrentVersion()),
+			        	    					String.valueOf( versionGetter.getLatestVersion()),
+		        	    						jar_file_dir.toString(), 
+		        	    						expected_dir.toString()});
+		        	    		
+		        	    		checker.reportProgress( alert );
+		        	    		
+		        	    		long	last_prompt = COConfigurationManager.getLongParameter( "swt.update.prevented.version.time", 0 );
+		        	    		long	now			= SystemTime.getCurrentTime();
+		        	    		
+		        	    		boolean force = now < last_prompt || now - last_prompt > 7*24*60*60*1000;
+		        	    		
+		        	    		if ( !checker.getCheckInstance().isAutomatic()){
+		        	    			
+		        	    			force = true;
+		        	    		}
+		        	    		
+		        		    	if ( force || update_prevented_version != versionGetter.getCurrentVersion()){
+			        		    				        	    		
+			        	     		Logger.log(	new LogAlert(LogAlert.REPEATABLE, LogEvent.LT_ERROR, alert ));
 			        						
 			        	     		update_prevented_version = versionGetter.getCurrentVersion();
 			        	     		
 			        	    		COConfigurationManager.setParameter( "swt.update.prevented.version", update_prevented_version );
+			        	    		COConfigurationManager.setParameter( "swt.update.prevented.version.time", now );
 		        		    	}
 		        	    	}
 		        	    }
@@ -196,11 +213,13 @@ public class SWTUpdateChecker implements UpdatableComponent
 	      
 	      final Update update = 
 	    	  checker.addUpdate("SWT Library for " + versionGetter.getPlatform(),
-		          new String[] {"SWT is the graphical library used by Azureus"},
+		          new String[] {"SWT is the graphical library used by " + Constants.APP_NAME},
 		          "" + versionGetter.getLatestVersion(),
 		          swtDownloader,
 		          Update.RESTART_REQUIRED_YES
-	          );      
+	          );
+	      
+	      update.setDescriptionURL(versionGetter.getInfoURL());
 	      
 	      swtDownloader.addListener(new ResourceDownloaderAdapter() {
 		        
