@@ -27,6 +27,7 @@ import java.util.Map;
 import org.eclipse.swt.SWT;
 
 import org.gudy.azureus2.core3.logging.*;
+import org.gudy.azureus2.core3.util.AEVerifier;
 import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.platform.PlatformManager;
 import org.gudy.azureus2.platform.PlatformManagerCapabilities;
@@ -50,6 +51,8 @@ public class SWTVersionGetter {
   private UpdateChecker	checker;
   
   private String[] mirrors;
+
+	private String infoURL;
   
   public 
   SWTVersionGetter(
@@ -130,6 +133,37 @@ public class SWTVersionGetter {
       msg += " url=" + mirrors[0];
     }
     
+    byte[] info_bytes = (byte[])reply.get( "swt_info_url" );
+    
+    if ( info_bytes != null ){
+    	
+    	byte[] sig = (byte[])reply.get( "swt_info_sig" );
+
+		if ( sig == null ){
+			
+			Logger.log( new LogEvent( LogIDs.LOGGER, "swt info signature check failed - missing signature" ));
+
+		}else{
+			
+		  	try{
+	    		infoURL = new String( info_bytes );
+	    		
+	    		try{
+					AEVerifier.verifyData( infoURL, sig );
+					
+				}catch( Throwable e ){
+
+					Logger.log( new LogEvent( LogIDs.LOGGER, "swt info signature check failed", e  ));
+					
+					infoURL = null;
+				}
+	    	}catch ( Exception e ){
+	    		
+	    		Logger.log(new LogEvent(LOGID, "swt info_url", e));
+	    	}
+		}
+    }
+
     if (Logger.isEnabled())
 			Logger.log(new LogEvent(LOGID, msg));
   }
@@ -160,4 +194,8 @@ public class SWTVersionGetter {
   public String[] getMirrors() {
     return mirrors;
   }
+
+	public String getInfoURL() {
+		return infoURL;
+	}
 }
