@@ -21,6 +21,7 @@
 package org.gudy.azureus2.ui.swt.views;
 
 import java.net.URL;
+import java.text.DecimalFormat;
 import java.util.Iterator;
 import java.util.List;
 
@@ -28,6 +29,8 @@ import org.eclipse.swt.SWT;
 import org.eclipse.swt.dnd.Clipboard;
 import org.eclipse.swt.dnd.TextTransfer;
 import org.eclipse.swt.dnd.Transfer;
+import org.eclipse.swt.events.DisposeEvent;
+import org.eclipse.swt.events.DisposeListener;
 import org.eclipse.swt.events.MouseAdapter;
 import org.eclipse.swt.events.MouseEvent;
 import org.eclipse.swt.events.SelectionAdapter;
@@ -54,6 +57,7 @@ import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncer;
 import org.gudy.azureus2.core3.tracker.client.TRTrackerScraperResponse;
 import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.ui.swt.Messages;
+import org.gudy.azureus2.ui.swt.TorrentUtil;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.components.BufferedLabel;
 import org.gudy.azureus2.ui.swt.components.BufferedTruncatedLabel;
@@ -99,10 +103,13 @@ public class GeneralView extends AbstractIView implements ParameterListener,
   //Text maxUploads;
   BufferedLabel totalSpeed;
   BufferedLabel ave_completion;
+  BufferedLabel distributedCopies;
   BufferedLabel seeds;
   BufferedLabel peers;
+  BufferedLabel completedLbl;
   Group gInfo;
   BufferedLabel fileName;
+  BufferedLabel torrentStatus;
   BufferedLabel fileSize;
   BufferedLabel saveIn;
   BufferedLabel hash;
@@ -117,13 +124,16 @@ public class GeneralView extends AbstractIView implements ParameterListener,
   BufferedLabel pieceSize;
   Control lblComment;
   BufferedLabel creation_date;
-  BufferedLabel user_comment;
+  BufferedLabel privateStatus;
+  Control user_comment;
   BufferedLabel hashFails;
   BufferedLabel shareRatio;
   Button		updateButton;
   
   private int graphicsUpdate = COConfigurationManager.getIntParameter("Graphics Update");
 
+  private Composite parent;
+
   /**
    * Initialize GeneralView
    */
@@ -137,20 +147,19 @@ public class GeneralView extends AbstractIView implements ParameterListener,
 			manager = (DownloadManager)((Object[])newDataSource)[0];
 		else
 			manager = (DownloadManager)newDataSource;
+
+		refreshInfo();
 	}
 
   /* (non-Javadoc)
    * @see org.gudy.azureus2.ui.swt.IView#initialize(org.eclipse.swt.widgets.Composite)
    */
   public void initialize(Composite composite) {
-  	if (manager == null)
-  		return;
+  	parent = composite;
 
-    pieces = new boolean[manager.getNbPieces()];
+    genComposite = new Canvas(parent, SWT.NULL);
 
-    this.display = composite.getDisplay();
 
-    genComposite = new Canvas(composite, SWT.NULL);
     GridLayout genLayout = new GridLayout();
     genLayout.marginHeight = 0;
     try {
@@ -162,6 +171,20 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     genLayout.numColumns = 1;
     genComposite.setLayout(genLayout);
 
+    refreshInfo();
+    COConfigurationManager.addParameterListener("Graphics Update", this);
+  }
+  
+  private void refreshInfo() {
+  	if (manager == null || parent == null)
+  		return;
+  	
+  	Utils.disposeComposite(genComposite, false);
+  	
+    pieces = new boolean[manager.getNbPieces()];
+
+    this.display = parent.getDisplay();
+
     gFile = new Composite(genComposite, SWT.SHADOW_OUT);
     GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
     gFile.setLayoutData(gridData);
@@ -270,8 +293,14 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     Messages.setLanguageText(label, "GeneralView.label.peers"); 
     peers = new BufferedLabel(gTransfer, SWT.LEFT);
     gridData = new GridData(GridData.FILL_HORIZONTAL);
-    gridData.horizontalSpan = 3;
     peers.setLayoutData(gridData);
+    
+    label = new Label(gTransfer, SWT.LEFT);
+    Messages.setLanguageText(label, "GeneralView.label.completed"); 
+    completedLbl = new BufferedLabel(gTransfer, SWT.LEFT);
+    gridData = new GridData(GridData.FILL_HORIZONTAL);
+    completedLbl.setLayoutData(gridData);
+
 
     
     label = new Label(gTransfer, SWT.LEFT);
@@ -285,10 +314,15 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     Messages.setLanguageText(label, "GeneralView.label.swarm_average_completion"); 
     ave_completion = new BufferedLabel(gTransfer, SWT.LEFT);
     gridData = new GridData(GridData.FILL_HORIZONTAL);
-    gridData.horizontalSpan = 3;
     ave_completion.setLayoutData(gridData);
     
+    label = new Label(gTransfer, SWT.LEFT);
+    Messages.setLanguageText(label, "GeneralView.label.distributedCopies"); 
+    distributedCopies = new BufferedLabel(gTransfer, SWT.LEFT);
+    gridData = new GridData(GridData.FILL_HORIZONTAL);
+    distributedCopies.setLayoutData(gridData);
     
+
     ////////////////////////
 
     gInfo = new Group(genComposite, SWT.SHADOW_OUT);
@@ -307,10 +341,10 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     fileName.setLayoutData(gridData);
 
     label = new Label(gInfo, SWT.LEFT);
-    Messages.setLanguageText(label, "GeneralView.label.totalsize"); //$NON-NLS-1$
-    fileSize = new BufferedLabel(gInfo, SWT.LEFT);
+    Messages.setLanguageText(label, "GeneralView.label.status"); //$NON-NLS-1$
+    torrentStatus = new BufferedLabel(gInfo, SWT.LEFT);
     gridData = new GridData(GridData.FILL_HORIZONTAL);
-    fileSize.setLayoutData(gridData);
+    torrentStatus.setLayoutData(gridData);
 
     label = new Label(gInfo, SWT.LEFT);
     Messages.setLanguageText(label, "GeneralView.label.savein"); //$NON-NLS-1$
@@ -318,6 +352,18 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     gridData = new GridData(GridData.FILL_HORIZONTAL);
     saveIn.setLayoutData(gridData);
 
+    label = new Label(gInfo, SWT.LEFT);
+    Messages.setLanguageText(label, "GeneralView.label.totalsize"); //$NON-NLS-1$
+    fileSize = new BufferedLabel(gInfo, SWT.LEFT);
+    gridData = new GridData(GridData.FILL_HORIZONTAL);
+    fileSize.setLayoutData(gridData);
+
+    label = new Label(gInfo, SWT.LEFT);
+    Messages.setLanguageText(label, "GeneralView.label.numberofpieces"); //$NON-NLS-1$
+    pieceNumber = new BufferedLabel(gInfo, SWT.LEFT);
+    gridData = new GridData(GridData.FILL_HORIZONTAL);
+    pieceNumber.setLayoutData(gridData);
+
     label = new Label(gInfo, SWT.LEFT);
     Messages.setLanguageText(label, "GeneralView.label.hash"); //$NON-NLS-1$
     hash = new BufferedLabel(gInfo, SWT.LEFT);
@@ -353,18 +399,6 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     	}
     });
     
-    label = new Label(gInfo, SWT.LEFT);
-    Messages.setLanguageText(label, "GeneralView.label.numberofpieces"); //$NON-NLS-1$
-    pieceNumber = new BufferedLabel(gInfo, SWT.LEFT);
-    gridData = new GridData(GridData.FILL_HORIZONTAL);
-    pieceNumber.setLayoutData(gridData);
-
-    label = new Label(gInfo, SWT.LEFT);
-    Messages.setLanguageText(label, "GeneralView.label.size"); //$NON-NLS-1$
-    pieceSize = new BufferedLabel(gInfo, SWT.LEFT);
-    gridData = new GridData(GridData.FILL_HORIZONTAL);
-    pieceSize.setLayoutData(gridData);
-
     label = new Label(gInfo, SWT.LEFT);
     Messages.setLanguageText(label, "GeneralView.label.trackerurl"); //$NON-NLS-1$
     label.setCursor(Cursors.handCursor);
@@ -514,24 +548,54 @@ public class GeneralView extends AbstractIView implements ParameterListener,
 		});
     
     
+    gridData = new GridData(GridData.FILL_HORIZONTAL);
+    gridData.horizontalSpan = 1;
+    trackerUrlValue.setLayoutData(gridData);
+    
+    label = new Label(gInfo, SWT.LEFT);
+    Messages.setLanguageText(label, "GeneralView.label.size"); //$NON-NLS-1$
+    pieceSize = new BufferedLabel(gInfo, SWT.LEFT);
+    gridData = new GridData(GridData.FILL_HORIZONTAL);
+    pieceSize.setLayoutData(gridData);
+    
     label = new Label(gInfo, SWT.LEFT);
     Messages.setLanguageText(label, "GeneralView.label.creationdate"); //$NON-NLS-1$
     creation_date = new BufferedLabel(gInfo, SWT.LEFT);
     gridData = new GridData(GridData.FILL_HORIZONTAL);
-    gridData.horizontalSpan = 1;
     creation_date.setLayoutData(gridData);
     
+    label = new Label(gInfo, SWT.LEFT);
+    Messages.setLanguageText(label, "GeneralView.label.private"); //$NON-NLS-1$
+    privateStatus = new BufferedLabel(gInfo, SWT.LEFT);
     gridData = new GridData(GridData.FILL_HORIZONTAL);
-    gridData.horizontalSpan = 1;
-    trackerUrlValue.setLayoutData(gridData);
+    privateStatus.setLayoutData(gridData);    
 
+	// empty row
+    label = new Label(gInfo, SWT.LEFT);
+    gridData = new GridData(GridData.FILL_HORIZONTAL);
+    gridData.horizontalSpan = 4;
+    label.setLayoutData(gridData);
+    
+    
     
     label = new Label(gInfo, SWT.LEFT);
     Messages.setLanguageText(label, "GeneralView.label.tracker"); //$NON-NLS-1$
     tracker_status = new BufferedTruncatedLabel(gInfo, SWT.LEFT,150);
     gridData = new GridData(GridData.FILL_HORIZONTAL);
-    gridData.horizontalSpan = 3;
-    tracker_status.setLayoutData(gridData);    
+    tracker_status.setLayoutData(gridData);
+    
+    updateButton = new Button(gInfo, SWT.PUSH);
+    Messages.setLanguageText(updateButton, "GeneralView.label.trackerurlupdate"); //$NON-NLS-1$
+    gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
+    gridData.verticalSpan = 2;
+    gridData.horizontalSpan = 2;
+    updateButton.setLayoutData(gridData);
+    updateButton.addSelectionListener(new SelectionAdapter()
+    {
+    	public void widgetSelected(SelectionEvent event) {
+    		manager.requestTrackerAnnounce(false);
+    	}
+    });
         
     label = new Label(gInfo, SWT.LEFT);
     Messages.setLanguageText(label, "GeneralView.label.updatein"); //$NON-NLS-1$
@@ -540,33 +604,50 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     trackerUpdateIn.setLayoutData(gridData);
     
 
-    updateButton = new Button(gInfo, SWT.PUSH);
-    Messages.setLanguageText(updateButton, "GeneralView.label.trackerurlupdate"); //$NON-NLS-1$
-    gridData = new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
-		updateButton.setLayoutData(gridData);
-	    
-		updateButton.addSelectionListener(new SelectionAdapter() {
-	      public void widgetSelected(SelectionEvent event) {
-	        manager.requestTrackerAnnounce(false);
-	      }
-	    });
+	// empty row
     
     label = new Label(gInfo, SWT.LEFT);
+    gridData = new GridData(GridData.FILL_HORIZONTAL);
+    gridData.horizontalSpan = 4;
+    label.setLayoutData(gridData);
+    
     
-    	// row
 
+    
     label = new Label(gInfo, SWT.LEFT);
+    label.setCursor(Cursors.handCursor);
+    label.setForeground(Colors.blue);
     Messages.setLanguageText(label, "GeneralView.label.user_comment"); //$NON-NLS-1$
-    user_comment = new BufferedLabel(gInfo, SWT.LEFT);
+
+    try {
+    	user_comment = new Link(gInfo, SWT.LEFT | SWT.WRAP);
+    	((Link)user_comment).addSelectionListener(new SelectionAdapter() {
+				public void widgetSelected(SelectionEvent e) {
+					Utils.launch(e.text);
+				}
+			});
+    } catch (Throwable e) {
+    	user_comment = new Label(gInfo, SWT.LEFT | SWT.WRAP);
+    }
+    
     gridData = new GridData(GridData.FILL_HORIZONTAL);
     gridData.horizontalSpan = 3;
     user_comment.setLayoutData(gridData);
+    
+    label.addMouseListener(new MouseAdapter() {
+    	private void editComment() {
+    		TorrentUtil.promptUserForComment(new DownloadManager[] {manager});
+    	}
+
+        public void mouseDoubleClick(MouseEvent arg0) {editComment();}
+        public void mouseDown(MouseEvent arg0) {editComment();}
+      });
 
     label = new Label(gInfo, SWT.LEFT);
     gridData = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
     label.setLayoutData(gridData);
-    
     Messages.setLanguageText(label, "GeneralView.label.comment"); //$NON-NLS-1$
+    
     try {
     	lblComment = new Link(gInfo, SWT.LEFT | SWT.WRAP);
     	((Link)lblComment).addSelectionListener(new SelectionAdapter() {
@@ -604,10 +685,14 @@ public class GeneralView extends AbstractIView implements ParameterListener,
       shell.setSize(size);
     }
     
+    genComposite.addDisposeListener(new DisposeListener() {
+    	public void widgetDisposed(DisposeEvent e) {
+    		menuTracker.dispose();
+    	}
+    });
+    
     genComposite.layout();
     //Utils.changeBackgroundComposite(genComposite,MainWindow.getWindow().getBackground());
-
-    COConfigurationManager.addParameterListener("Graphics Update", this);
   }
 
   /* (non-Javadoc)
@@ -674,13 +759,16 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     TRTrackerScraperResponse hd = manager.getTrackerScrapeResponse();
     String seeds_str = manager.getNbSeeds() +" "+ MessageText.getString("GeneralView.label.connected");
     String peers_str = manager.getNbPeers() +" "+ MessageText.getString("GeneralView.label.connected");
+    String completed;
     if(hd != null && hd.isValid()) {
       seeds_str += " ( " + hd.getSeeds() +" "+ MessageText.getString("GeneralView.label.in_swarm") + " )";
       peers_str += " ( " + hd.getPeers() +" "+ MessageText.getString("GeneralView.label.in_swarm") + " )";
+      completed = hd.getCompleted() > -1 ? Integer.toString(hd.getCompleted()) : "?";
+
     } else {
-      //seeds += " (?)";
-      //peers += " (?)";
+      completed = "?";
     }
+    
     String _shareRatio = "";
     int sr = manager.getStats().getShareRatio();
     
@@ -697,6 +785,8 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     String swarm_speed = DisplayFormatters.formatByteCountToKiBEtcPerSec( stats.getTotalAverage() ) + " ( " +DisplayFormatters.formatByteCountToKiBEtcPerSec( stats.getTotalAveragePerPeer())+ " " +MessageText.getString("GeneralView.label.averagespeed") + " )";    
     
     String swarm_completion = "";
+    String distributedCopies = "0.000";
+    String piecesDoneAndSum = ""+manager.getNbPieces();
     
     PEPeerManager pm = manager.getPeerManager();
     if( pm != null ) {
@@ -704,7 +794,13 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     	if( comp >= 0 ) {
     		swarm_completion = DisplayFormatters.formatPercentFromThousands( comp );
     	}
+    	
+    	piecesDoneAndSum = pm.getPiecePicker().getNbPiecesDone() + "/" + piecesDoneAndSum;
+    	
+    	distributedCopies = new DecimalFormat("0.000").format(pm.getPiecePicker().getMinAvailability()-pm.getNbSeeds()-(pm.isSeeding()&&stats.getDownloadCompleted(false)==1000?1:0));
     }
+    
+    
 
     setStats(
     		DisplayFormatters.formatDownloaded(stats),
@@ -716,9 +812,11 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     		""+(manager.getStats().getUploadRateLimitBytesPerSecond() /1024),
       	seeds_str,
       	peers_str,
+      	completed,
       	DisplayFormatters.formatHashFails(manager),
       	_shareRatio,
-      	swarm_completion
+      	swarm_completion,
+      	distributedCopies
     );
       
     setTracker(manager);
@@ -728,13 +826,16 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     setInfos(
       manager.getDisplayName(),
 	  DisplayFormatters.formatByteCountToKiBEtc(manager.getSize()),
+	  DisplayFormatters.formatDownloadStatus(manager),
       manager.getSaveLocation().toString(),
       TorrentUtils.nicePrintTorrentHash(torrent),
-      manager.getNbPieces(),
+      piecesDoneAndSum,
       manager.getPieceLength(),
       manager.getTorrentComment(),
       DisplayFormatters.formatDate(manager.getTorrentCreationDate()*1000),
-      manager.getDownloadState().getUserComment());
+      manager.getDownloadState().getUserComment(),
+      MessageText.getString("GeneralView."+(manager.getTorrent().getPrivate()?"yes":"no"))
+      );
     
     
     //A special layout, for OS X and Linux, on which for some unknown reason
@@ -1048,9 +1149,12 @@ public class GeneralView extends AbstractIView implements ParameterListener,
 	String dl_speed, String ul_speed,
 	String s, 
 	String p,
+	String completed,
 	String hash_fails,
 	String share_ratio,
-	String ave_comp ) 
+	String ave_comp,
+	String distr_copies
+	) 
   {
     if (display == null || display.isDisposed())
       return;
@@ -1061,6 +1165,7 @@ public class GeneralView extends AbstractIView implements ParameterListener,
 	uploadSpeed.setText( uls );
 	totalSpeed.setText( ts );
 	ave_completion.setText( ave_comp );
+	distributedCopies.setText(distr_copies);
 	
 	/*
 	if ( !maxDLSpeed.getText().equals( dl_speed )){
@@ -1075,7 +1180,8 @@ public class GeneralView extends AbstractIView implements ParameterListener,
 	*/
 	
 	seeds.setText( s);
-	peers.setText( p); 
+	peers.setText( p);
+	completedLbl.setText(completed);
 	hashFails.setText( hash_fails);
 	shareRatio.setText( share_ratio);     
   }
@@ -1157,55 +1263,65 @@ public class GeneralView extends AbstractIView implements ParameterListener,
   private void setInfos(
     final String _fileName,
     final String _fileSize,
+    final String _torrentStatus,
     final String _path,
     final String _hash,
-    final int _pieceNumber,
+    final String _pieceData,
     final String _pieceLength,
     final String _comment,
 	final String _creation_date,
-	final String _user_comment) {
-		if (display == null || display.isDisposed())
+	final String _user_comment,
+	final String isPrivate) {
+    if (display == null || display.isDisposed())
 			return;
-		Utils.execSWTThread(new AERunnable() {
+		Utils.execSWTThread(new AERunnable()
+		{
 			public void runSupport() {
-				fileName.setText(_fileName );
+				fileName.setText(_fileName);
 				fileSize.setText(_fileSize);
+				torrentStatus.setText(_torrentStatus);
 				saveIn.setText(_path);
 				hash.setText(_hash);
-				pieceNumber.setText("" + _pieceNumber); //$NON-NLS-1$
+				pieceNumber.setText(_pieceData); //$NON-NLS-1$
 				pieceSize.setText(_pieceLength);
-
-				String sOldComment = (String) lblComment.getData("comment");
-
-				if (!_comment.equals(sOldComment)) {
-					if (lblComment instanceof Label) {
-						((Label) lblComment).setText(_comment);
-					} else if (lblComment instanceof Link) {
+				creation_date.setText(_creation_date);
+				privateStatus.setText(isPrivate);
+				boolean do_relayout = false;
+				do_relayout = setCommentAndFormatLinks(lblComment, _comment.length() > 5000 && Constants.isWindowsXP ? _comment.substring(0, 5000) : _comment ) | do_relayout;
+				do_relayout = setCommentAndFormatLinks(user_comment, _user_comment) | do_relayout;
+				if (do_relayout)
+				{
+					gInfo.layout();
+				}
+			}
+		});
+	}
+ 
+  private static boolean setCommentAndFormatLinks(Control c, String new_comment) {
+	  String old_comment = (String)c.getData("comment");
+	  if (new_comment == null) {new_comment = "";}
+	  if (new_comment.equals(old_comment)) {return false;}
+	
+	  c.setData("comment", new_comment);
+	  if (c instanceof Label) {
+		  ((Label) c).setText(new_comment);
+	  } else if (c instanceof Link) {
 						String sNewComment;
-
-						sNewComment = _comment.replaceAll(
+		  sNewComment = new_comment.replaceAll(
 								"([^=\">][\\s]+|^)(http://[\\S]+)", "$1<A HREF=\"$2\">$2</A>");
 						// need quotes around url
-						sNewComment = sNewComment.replaceAll("(href=)(htt[^\\s>]+)",
-								"$1\"$2\"");
+		  sNewComment = sNewComment.replaceAll("(href=)(htt[^\\s>]+)", "$1\"$2\"");
 
 						// Examples:
 						// http://cowbow.com/fsdjl&sdfkj=34.sk9391 moo
 						// <A HREF=http://cowbow.com/fsdjl&sdfkj=34.sk9391>moo</a>
 						// <A HREF="http://cowbow.com/fsdjl&sdfkj=34.sk9391">moo</a>
 						// <A HREF="http://cowbow.com/fsdjl&sdfkj=34.sk9391">http://moo.com</a>
-
-						((Link) lblComment).setText(sNewComment);
-					}
-
-				}
-
-				creation_date.setText(_creation_date);
-				user_comment.setText(_user_comment);
-			}
-		});
-	}
- 
+		  ((Link)c).setText(sNewComment);
+			  	        	}
+	  return true;
+			  	    
+			          }
 
   public void parameterChanged(String parameterName) {
     graphicsUpdate = COConfigurationManager.getIntParameter("Graphics Update");
@@ -1217,5 +1333,5 @@ public class GeneralView extends AbstractIView implements ParameterListener,
 		UIDebugGenerator.obfusticateArea(image, (Control) saveIn.getWidget(),
 				shellOffset, Debug.secretFileName(saveIn.getText()));
 		return image;
-	}
+}
 }
