@@ -30,26 +30,22 @@ import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.widgets.*;
 
-import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.download.DownloadManagerListener;
-import org.gudy.azureus2.core3.global.GlobalManagerDownloadRemovalVetoException;
 import org.gudy.azureus2.core3.internat.MessageText;
-import org.gudy.azureus2.core3.torrent.TOTorrent;
+import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.pluginsimpl.local.download.DownloadImpl;
 import org.gudy.azureus2.pluginsimpl.local.download.DownloadManagerImpl;
-import org.gudy.azureus2.ui.swt.Alerts;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.debug.ObfusticateImage;
 import org.gudy.azureus2.ui.swt.debug.ObfusticateTab;
-import org.gudy.azureus2.ui.swt.mainwindow.Colors;
 import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;
 import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;
 import org.gudy.azureus2.ui.swt.pluginsimpl.UISWTInstanceImpl;
 import org.gudy.azureus2.ui.swt.pluginsimpl.UISWTViewImpl;
 import org.gudy.azureus2.ui.swt.views.utils.ManagerUtils;
-import org.gudy.azureus2.ui.swt.wizards.sendtorrent.SendTorrentWizard;
 
 import com.aelitis.azureus.core.AzureusCore;
 import com.aelitis.azureus.ui.UIFunctions;
@@ -79,11 +75,44 @@ public class ManagerView extends AbstractIView implements
 	DownloadManager manager) 
   {
   	azureus_core	= _azureus_core;
-    this.manager 	= manager;
+  	dataSourceChanged(manager);
     
-    dataSourceChanged(manager);
+  }
+  
+  public void dataSourceChanged(Object newDataSource) {
+  	super.dataSourceChanged(newDataSource);
 
-    manager.addListener(this);
+    if (manager != null) {
+    	manager.removeListener(this);
+    }
+
+  	DownloadImpl dataSourcePlugin = null;
+  	if (newDataSource instanceof DownloadImpl) {
+  		dataSourcePlugin = (DownloadImpl) newDataSource;
+  		manager = dataSourcePlugin.getDownload();
+  	} else if (newDataSource instanceof DownloadManager) {
+    	manager = (DownloadManager) newDataSource;
+      try {
+      	dataSourcePlugin = DownloadManagerImpl.getDownloadStatic(manager);
+      } catch (DownloadException e) { /* Ignore */ }
+  	} else {
+  		manager = null;
+  	}
+  	
+    if (manager != null) {
+    	manager.addListener(this);
+    }
+
+		for (int i = 0; i < tabViews.size(); i++) {
+			IView view = (IView) tabViews.get(i);
+			if (view != null) {
+				if (view instanceof UISWTViewImpl) {
+					((UISWTViewImpl) view).dataSourceChanged(dataSourcePlugin);
+				} else {
+					view.dataSourceChanged(newDataSource);
+				}
+			}
+		}
   }
 
   /* (non-Javadoc)
@@ -94,7 +123,9 @@ public class ManagerView extends AbstractIView implements
   	if (uiFunctions != null) {
   		uiFunctions.removeManagerView(manager);
   	}
-    manager.removeListener(this);
+  	if (manager != null) {
+  		manager.removeListener(this);
+  	}
     
     if ( !folder.isDisposed()){
     	
@@ -145,26 +176,36 @@ public class ManagerView extends AbstractIView implements
    * @see org.gudy.azureus2.ui.swt.IView#initialize(org.eclipse.swt.widgets.Composite)
    */
   public void initialize(Composite composite) {
-    
-	  	if (folder == null) {
-	    folder = new TabFolder(composite, SWT.LEFT);
-	    folder.setBackground(Colors.background);
-	  	} else {
-	  	  System.out.println("ManagerView::initialize : folder isn't null !!!");
-	  	}
 
-	  IView views[] = { new GeneralView(), new PeersView(),
-				new PeersGraphicView(), new PiecesView(), new FilesView(), new TorrentInfoView( manager ),
-				new TorrentOptionsView( manager ), new LoggerView() };
+  	if (folder == null) {
+			folder = new TabFolder(composite, SWT.LEFT);
+		} else {
+			System.out.println("ManagerView::initialize : folder isn't null !!!");
+		}
+  	
+  	ArrayList iviews_to_use = new ArrayList();
+  	iviews_to_use.add(new GeneralView());
+  	iviews_to_use.add(new PeersView());
+  	iviews_to_use.add(new PeersGraphicView());
+  	iviews_to_use.add(new PiecesView());
+  	iviews_to_use.add(new FilesView());
+  	iviews_to_use.add(new TorrentInfoView(manager));
+  	iviews_to_use.add(new TorrentOptionsView(manager));
+  	if (Logger.isEnabled()) {
+  		iviews_to_use.add(new LoggerView());
+  	}
+  	
+  	IView[] views = (IView[])iviews_to_use.toArray(new IView[iviews_to_use.size()]);
 
-		for (int i = 0; i < views.length; i++)
-			addSection(views[i], manager);
+  	for (int i = 0; i < views.length; i++)
+		addSection(views[i], manager);
 
     // Call plugin listeners
 		UIFunctionsSWT uiFunctions = UIFunctionsManagerSWT.getUIFunctionsSWT();
 		if (uiFunctions != null) {
 			UISWTInstanceImpl pluginUI = uiFunctions.getSWTPluginInstanceImpl();
-			Map pluginViews = pluginUI.getViewListeners(UISWTInstance.VIEW_MYTORRENTS);
+			Map pluginViews = pluginUI == null ? null
+					: pluginUI.getViewListeners(UISWTInstance.VIEW_MYTORRENTS);
 			if (pluginViews != null) {
 				String[] sNames = (String[]) pluginViews.keySet().toArray(new String[0]);
 				for (int i = 0; i < sNames.length; i++) {
@@ -290,9 +331,6 @@ public class ManagerView extends AbstractIView implements
 		if (itemKey.equals("remove"))
 			return true;
 		
-		if (itemKey.equals("send"))
-			return true;
-		
 		return false;
 	}
 
@@ -303,12 +341,12 @@ public class ManagerView extends AbstractIView implements
 		}
 		
 		if (itemKey.equals("start")) {
-			ManagerUtils.queue(manager, folder);
+			ManagerUtils.queue(manager, folder.getShell());
 			return;
 		}
 		
 		if (itemKey.equals("stop")) {
-			ManagerUtils.stop(manager, folder);
+			ManagerUtils.stop(manager, folder.getShell());
 			return;
 		}
 		
@@ -331,41 +369,7 @@ public class ManagerView extends AbstractIView implements
 		}
 		
 		if (itemKey.equals("remove")) {
-
-			if (COConfigurationManager.getBooleanParameter("confirm_torrent_removal")) {
-				MessageBox mb = new MessageBox(folder.getShell(), SWT.ICON_WARNING
-						| SWT.YES | SWT.NO);
-				mb.setText(MessageText.getString("deletedata.title"));
-				mb.setMessage(MessageText
-						.getString("MyTorrentsView.confirm_torrent_removal")
-						+ manager.getDisplayName());
-				if (mb.open() == SWT.NO) {
-					return;
-				}
-			}
-
-			new AEThread("asyncStop", true) {
-				public void runSupport() {
-					try {
-
-						manager.stopIt(DownloadManager.STATE_STOPPED, false, false);
-						manager.getGlobalManager().removeDownloadManager(manager);
-
-					} catch (GlobalManagerDownloadRemovalVetoException e) {
-
-						if(!e.isSilent()) {
-							Alerts.showErrorMessageBoxUsingResourceString(
-									"globalmanager.download.remove.veto", e);	
-						}
-						
-					}
-				}
-			}.start();
-		}
-
-		if (itemKey.equals("send")) {
-			new SendTorrentWizard(azureus_core, folder.getDisplay(),
-					new TOTorrent[] { manager.getTorrent() });
+			ManagerUtils.remove(manager, null, false, false);
 		}
 	}
   
@@ -425,7 +429,11 @@ public class ManagerView extends AbstractIView implements
 	public Image obfusticatedImage(Image image, Point shellOffset) {
 		IView view = getActiveView();
 		if (view instanceof ObfusticateImage) {
-			((ObfusticateImage)view).obfusticatedImage(image, shellOffset);
+			try {
+				((ObfusticateImage)view).obfusticatedImage(image, shellOffset);
+			} catch (Exception e) {
+				Debug.out("Obfusticating " + view, e);
+			}
 		}
 		return image;
 	}
@@ -435,4 +443,6 @@ public class ManagerView extends AbstractIView implements
     return DisplayFormatters.formatPercentFromThousands(completed) + " : " + manager;
 	}
 	
+	public DownloadManager getDownload() {return manager;}
+	
 }
