@@ -29,7 +29,6 @@ import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.events.MouseAdapter;
 import org.eclipse.swt.events.MouseEvent;
 import org.eclipse.swt.layout.*;
-import org.eclipse.swt.program.Program;
 import org.eclipse.swt.widgets.Group;
 import org.eclipse.swt.widgets.Label;
 
@@ -38,18 +37,18 @@ import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.peer.PEPeerSource;
 import org.gudy.azureus2.core3.util.AENetworkClassifier;
 import org.gudy.azureus2.plugins.ui.config.ConfigSection;
+import org.gudy.azureus2.ui.swt.components.LinkLabel;
 import org.gudy.azureus2.ui.swt.config.*;
 import org.gudy.azureus2.ui.swt.mainwindow.Colors;
 import org.gudy.azureus2.ui.swt.mainwindow.Cursors;
 import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
 import org.gudy.azureus2.ui.swt.Messages;
+import org.gudy.azureus2.ui.swt.Utils;
 
 public class ConfigSectionConnection implements UISWTConfigSection {
 	
 	private static final String CFG_PREFIX = "ConfigView.section.connection.";
-	
-	private static final boolean ENABLE_TCP_UDP_EXPERIMENTAL	= false;
-	
+		
 	public String configSectionGetParentSection() {
 		return ConfigSection.SECTION_ROOT;
 	}
@@ -63,6 +62,11 @@ public class ConfigSectionConnection implements UISWTConfigSection {
 
 	public void configSectionDelete() {
 	}
+	
+	public int maxUserMode() {
+		return 2;
+	}
+
 
 	public Composite configSectionCreate(final Composite parent) {
 		Label label;
@@ -98,23 +102,22 @@ public class ConfigSectionConnection implements UISWTConfigSection {
 		label.setLayoutData(gridData);
 
 		final IntParameter tcplisten = new IntParameter(cMiniArea,
-				"TCP.Listen.Port", 1, 65535, false, false);
+				"TCP.Listen.Port", 1, 65535);
 		gridData = new GridData();
 		gridData.widthHint = 40;
 		tcplisten.setLayoutData(gridData);
 
-		tcplisten.addChangeListener(new ParameterChangeListener() {
-			public void parameterChanged(Parameter p, boolean caused_internally) {
-				int val = tcplisten.getValue();
-
-				if (val == 6880 || val == 6881) {
-					val = 6881;
-					tcplisten.setValue(val);
+		tcplisten.addChangeListener(new ParameterChangeAdapter() {
+			public void intParameterChanging(Parameter p, int toValue) {
+				if (toValue == 6880) {
+					toValue = 6881;
+					tcplisten.setValue(toValue);
 				}
-				
-				if ( !separate_ports ){
-					
-					COConfigurationManager.setParameter( "UDP.Listen.Port", val );
+
+				if (!separate_ports) {
+					COConfigurationManager.setParameter("UDP.Listen.Port", toValue);
+					COConfigurationManager.setParameter("UDP.NonData.Listen.Port",
+							toValue);
 				}
 			}
 		});
@@ -127,23 +130,28 @@ public class ConfigSectionConnection implements UISWTConfigSection {
 			label.setLayoutData(gridData);
 
 			final IntParameter udp_listen = new IntParameter(cMiniArea,
-					"UDP.Listen.Port", 1, 65535, false, false);
+					"UDP.Listen.Port", 1, 65535);
 			gridData = new GridData();
 			gridData.widthHint = 40;
 			udp_listen.setLayoutData(gridData);
 
-			udp_listen.addChangeListener(new ParameterChangeListener() {
-				public void parameterChanged(Parameter p, boolean caused_internally) {
-					int val = udp_listen.getValue();
+			final boolean MULTI_UDP = COConfigurationManager.ENABLE_MULTIPLE_UDP_PORTS && userMode > 1;
+			
+			udp_listen.addChangeListener(new ParameterChangeAdapter() {
+				public void intParameterChanging(Parameter p, int toValue) {
+					if (toValue == 6880) {
+						toValue = 6881;
+						udp_listen.setValue(toValue);
+					}
 
-					if (val == 6880 || val == 6881) {
-						val = 6881;
-						udp_listen.setValue(val);
+					if (!MULTI_UDP) {
+						COConfigurationManager.setParameter("UDP.NonData.Listen.Port",
+								toValue);
 					}
 				}
 			});
 		
-			if ( ENABLE_TCP_UDP_EXPERIMENTAL && userMode > 1 ){
+			if ( MULTI_UDP ){
 			
 				Composite cNonDataUDPArea = new Composite(cSection, SWT.NULL);
 				layout = new GridLayout();
@@ -160,23 +168,22 @@ public class ConfigSectionConnection implements UISWTConfigSection {
 				gridData.horizontalIndent = 16;
 				commonUDP.setLayoutData( gridData );
 				
-				final IntParameter non_data_udp_listen = new IntParameter(cNonDataUDPArea, "UDP.NonData.Listen.Port", false );
+				final IntParameter non_data_udp_listen = new IntParameter(
+						cNonDataUDPArea, "UDP.NonData.Listen.Port");
 	
 				non_data_udp_listen.addChangeListener(
-					new ParameterChangeListener() 
+					new ParameterChangeAdapter() 
 					{
-						public void parameterChanged(Parameter p, boolean caused_internally) {
-							int val = non_data_udp_listen.getValue();
-	
-							if (val == 6880 || val == 6881) {
-								val = 6881;
-								non_data_udp_listen.setValue(val);
+						public void intParameterChanging(Parameter p, int toValue) {
+							if (toValue == 6880) {
+								toValue = 6881;
+								non_data_udp_listen.setValue(toValue);
 							}
 						}
 					});
 				
 				udp_listen.addChangeListener(
-						new ParameterChangeListener() 
+						new ParameterChangeAdapter() 
 						{
 							public void parameterChanged(Parameter p, boolean caused_internally)
 							{
@@ -201,7 +208,7 @@ public class ConfigSectionConnection implements UISWTConfigSection {
 				commonUDP.setAdditionalActionPerformer(new ChangeSelectionActionPerformer( non_data_udp_listen.getControls(), true ));
 				
 				commonUDP.addChangeListener(
-					new ParameterChangeListener() 
+					new ParameterChangeAdapter() 
 					{
 						public void 
 						parameterChanged(
@@ -260,15 +267,67 @@ public class ConfigSectionConnection implements UISWTConfigSection {
 			linkLabel.setLayoutData(gridData);
 			linkLabel.addMouseListener(new MouseAdapter() {
 				public void mouseDoubleClick(MouseEvent arg0) {
-					Program.launch((String) ((Label) arg0.widget).getData());
+					Utils.launch((String) ((Label) arg0.widget).getData());
 				}
 
 				public void mouseDown(MouseEvent arg0) {
-					Program.launch((String) ((Label) arg0.widget).getData());
+					Utils.launch((String) ((Label) arg0.widget).getData());
 				}
 			});
 		}
 
+		if (userMode > 0) {
+			/////////////////////// HTTP ///////////////////
+
+			Group http_group = new Group(cSection, SWT.NULL);
+			
+			Messages.setLanguageText(http_group,CFG_PREFIX + "group.http");
+			
+			GridLayout http_layout = new GridLayout();
+			
+			http_layout.numColumns = 2;
+
+			http_group.setLayout(http_layout);
+
+			gridData = new GridData(GridData.FILL_HORIZONTAL);
+			http_group.setLayoutData(gridData);
+
+			label = new Label(http_group, SWT.WRAP);
+			Messages.setLanguageText(label, CFG_PREFIX + "group.http.info");
+
+			new LinkLabel(
+					http_group, 
+					"ConfigView.label.please.visit.here",
+					"http://azureus.aelitis.com/wiki/index.php?title=HTTP_Seeding");
+
+			final BooleanParameter enable_http = 
+				new BooleanParameter(http_group, "HTTP.Data.Listen.Port.Enable", CFG_PREFIX + "http.enable");
+			gridData = new GridData();
+			gridData.horizontalSpan = 2;
+			enable_http.setLayoutData( gridData );
+
+			label = new Label(http_group, SWT.NULL);
+			Messages.setLanguageText(label, CFG_PREFIX + "http.port" );
+
+			IntParameter http_port = new IntParameter(http_group, "HTTP.Data.Listen.Port");
+
+			gridData = new GridData();
+			gridData.widthHint = 40;
+			http_port.setLayoutData( gridData );
+
+			label = new Label(http_group, SWT.NULL);
+			Messages.setLanguageText(label, CFG_PREFIX + "http.portoverride" );
+
+			IntParameter http_port_override = new IntParameter(http_group, "HTTP.Data.Listen.Port.Override");
+
+			gridData = new GridData();
+			gridData.widthHint = 40;
+			http_port_override.setLayoutData( gridData );
+
+			enable_http.setAdditionalActionPerformer( new ChangeSelectionActionPerformer( http_port ));
+			enable_http.setAdditionalActionPerformer( new ChangeSelectionActionPerformer( http_port_override ));
+		}
+		
 		if (userMode > 0) {
 			/////////////////////// PEER SOURCES GROUP ///////////////////
 
@@ -278,7 +337,7 @@ public class ConfigSectionConnection implements UISWTConfigSection {
 			GridLayout peer_sources_layout = new GridLayout();
 			peer_sources_group.setLayout(peer_sources_layout);
 
-			gridData = new GridData();
+			gridData = new GridData(GridData.FILL_HORIZONTAL);
 			peer_sources_group.setLayoutData(gridData);
 
 			label = new Label(peer_sources_group, SWT.WRAP);
@@ -314,7 +373,7 @@ public class ConfigSectionConnection implements UISWTConfigSection {
 				GridLayout networks_layout = new GridLayout();
 				networks_group.setLayout(networks_layout);
 
-				gridData = new GridData();
+				gridData = new GridData(GridData.FILL_HORIZONTAL);
 				networks_group.setLayoutData(gridData);
 
 				label = new Label(networks_group, SWT.NULL);
