@@ -27,23 +27,16 @@ import java.util.*;
 import java.util.List;
 
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.events.MouseAdapter;
-import org.eclipse.swt.events.MouseEvent;
-import org.eclipse.swt.events.SelectionAdapter;
-import org.eclipse.swt.events.SelectionEvent;
+import org.eclipse.swt.events.*;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.internat.MessageText;
+import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.FileUtil;
-import org.gudy.azureus2.plugins.PluginException;
-import org.gudy.azureus2.plugins.PluginInterface;
-import org.gudy.azureus2.plugins.ui.config.ConfigSection;
-import org.gudy.azureus2.plugins.ui.config.Parameter;
-import org.gudy.azureus2.pluginsimpl.local.PluginInterfaceImpl;
-import org.gudy.azureus2.pluginsimpl.local.ui.config.BooleanParameterImpl;
-import org.gudy.azureus2.pluginsimpl.local.ui.config.ParameterRepository;
 import org.gudy.azureus2.ui.swt.ImageRepository;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.Utils;
@@ -57,6 +50,15 @@ import org.gudy.azureus2.ui.swt.views.ConfigView;
 
 import com.aelitis.azureus.core.AzureusCore;
 
+import org.gudy.azureus2.plugins.PluginException;
+import org.gudy.azureus2.plugins.PluginInterface;
+import org.gudy.azureus2.plugins.ui.config.ConfigSection;
+import org.gudy.azureus2.plugins.ui.config.Parameter;
+
+import org.gudy.azureus2.pluginsimpl.local.PluginInterfaceImpl;
+import org.gudy.azureus2.pluginsimpl.local.ui.config.BooleanParameterImpl;
+import org.gudy.azureus2.pluginsimpl.local.ui.config.ParameterRepository;
+
 /**
  * Configuration Section that lists all the plugins and sets up
  * subsections for plugins that used the PluginConfigModel object.
@@ -66,7 +68,7 @@ import com.aelitis.azureus.core.AzureusCore;
  * @author TuxPaper
  *
  */
-public class ConfigSectionPlugins implements UISWTConfigSection {
+public class ConfigSectionPlugins implements UISWTConfigSection, ParameterListener {
 	private final static String HEADER_PREFIX = "ConfigView.pluginlist.column.";
 
 	private final static String[] COLUMN_HEADERS = { "loadAtStartup", "type",
@@ -84,6 +86,8 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 	FilterComparator comparator;
 	
 	List pluginIFs;
+
+	private Table table;
 	
 	class FilterComparator implements Comparator {
 		boolean ascending = true;
@@ -229,7 +233,7 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 				
 				case FIELD_UNLOADABLE: {
 					return MessageText.getString("Button."
-							+ (pluginIF.isUnloadable() ? "yes" : "no"));
+							+ (pluginIF.isUnloadable() ? "yes" : "no")).replaceAll("&", "");
 				}
 			} // switch
 
@@ -260,6 +264,11 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 
 	public void configSectionSave() {
 	}
+	
+	public int maxUserMode() {
+		return 0;
+	}
+
 
 	public void configSectionDelete() {
 	}
@@ -379,7 +388,7 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 			}
 		});
 
-		pluginIFs = Arrays.asList(azureusCore.getPluginManager().getPlugins());
+		pluginIFs = rebuildPluginIFs();
 
 		Collections.sort(pluginIFs, new Comparator() {
 			public int compare(Object o1, Object o2) {
@@ -392,8 +401,8 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 		labelInfo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
 		Messages.setLanguageText(labelInfo, "ConfigView.pluginlist.info");
 
-		final Table table = new Table(infoGroup, SWT.BORDER | SWT.SINGLE
-				| SWT.CHECK | SWT.VIRTUAL | SWT.FULL_SELECTION);
+		table = new Table(infoGroup, SWT.BORDER | SWT.SINGLE | SWT.CHECK
+				| SWT.VIRTUAL | SWT.FULL_SELECTION);
 		gridData = new GridData(GridData.FILL_BOTH);
 		gridData.heightHint = 200;
 		gridData.widthHint = 200;
@@ -422,7 +431,7 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 		layout = new GridLayout();
 		layout.marginHeight = 0;
 		layout.marginWidth = 0;
-		layout.numColumns = 2;
+		layout.numColumns = 3;
 		cButtons.setLayout(layout);
 		cButtons.setLayoutData(new GridData());
 		
@@ -446,7 +455,7 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 								}
 							}
 						}
-						pluginIFs = Arrays.asList(azureusCore.getPluginManager().getPlugins());
+						pluginIFs = rebuildPluginIFs();
 						table.setItemCount(pluginIFs.size());
 						Collections.sort(pluginIFs, comparator);
 						table.clearAll();
@@ -454,6 +463,39 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 				}
 			}
 		});
+		btnUnload.setEnabled( false );
+		
+		final Button btnLoad = new Button(cButtons, SWT.PUSH);
+		btnUnload.setLayoutData(new GridData());
+		Messages.setLanguageText(btnLoad, "ConfigView.pluginlist.loadSelected");
+		btnLoad.addSelectionListener(new SelectionAdapter() {
+			public void widgetSelected(SelectionEvent e) {
+				int[] items = table.getSelectionIndices();
+				for (int i = 0; i < items.length; i++) {
+					int index = items[i];
+					if (index >= 0 && index < pluginIFs.size()) {
+						PluginInterface pluginIF = (PluginInterface) pluginIFs.get(index);
+
+						if (pluginIF.isDisabled()) {
+							try {
+								COConfigurationManager.setParameter("PluginInfo."
+										+ pluginIF.getPluginID() + ".enabled", true );
+								pluginIF.setDisabled( false );
+								pluginIF.reload();
+							} catch (PluginException e1) {
+								// TODO Auto-generated catch block
+								e1.printStackTrace();
+							}
+						}
+						pluginIFs = rebuildPluginIFs();
+						table.setItemCount(pluginIFs.size());
+						Collections.sort(pluginIFs, comparator);
+						table.clearAll();
+					}
+				}
+			}
+		});
+		btnLoad.setEnabled( false );
 		
 		
 		final Button btnScan = new Button(cButtons, SWT.PUSH);
@@ -462,7 +504,7 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 		btnScan.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
 				azureusCore.getPluginManager().refreshPluginList();
-				pluginIFs = Arrays.asList(azureusCore.getPluginManager().getPlugins());
+				pluginIFs = rebuildPluginIFs();
 				table.setItemCount(pluginIFs.size());
 				Collections.sort(pluginIFs, comparator);
 				table.clearAll();
@@ -498,16 +540,28 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 
 		table.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
-				if (e.detail != SWT.CHECK)
-					return;
 				TableItem item = (TableItem) e.item;
-				if (item.getGrayed()) {
-					if (!item.getChecked())
-						item.setChecked(true);
-					return;
+				int index = table.indexOf(item);
+				PluginInterface pluginIF = (PluginInterface) pluginIFs.get(index);
+
+				if (e.detail == SWT.CHECK){
+						
+					if (item.getGrayed()) {
+						if (!item.getChecked())
+							item.setChecked(true);
+						return;
+					}
+
+					pluginIF.setDisabled(!item.getChecked());
+					COConfigurationManager.setParameter("PluginInfo."
+							+ item.getData("PluginID") + ".enabled", item.getChecked());
 				}
-				COConfigurationManager.setParameter("PluginInfo."
-						+ item.getData("PluginID") + ".enabled", item.getChecked());
+				
+				btnUnload.setEnabled( pluginIF.isUnloadable());
+				boolean bEnabled = COConfigurationManager.getBooleanParameter("PluginInfo."
+						+ pluginIF.getPluginID() + ".enabled", true);
+				btnLoad.setEnabled(pluginIF.isDisabled() && pluginIF.isOperational()
+						&& !bEnabled);
 			}
 		});
 
@@ -517,6 +571,35 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 		return infoGroup;
 	}
 
+	/**
+	 * @return
+	 *
+	 * @since 3.0.5.3
+	 */
+	private List rebuildPluginIFs() {
+		List pluginIFs = Arrays.asList(azureusCore.getPluginManager().getPlugins());
+		for (Iterator iter = pluginIFs.iterator(); iter.hasNext();) {
+			PluginInterface pi = (PluginInterface) iter.next();
+			// COConfigurationManager will not add the same listener twice
+			COConfigurationManager.addParameterListener("PluginInfo."
+					+ pi.getPluginID() + ".enabled", this);
+		}
+		return pluginIFs;
+	}
+	
+	// @see org.gudy.azureus2.core3.config.ParameterListener#parameterChanged(java.lang.String)
+	public void parameterChanged(String parameterName) {
+		if (table != null) {
+			Utils.execSWTThread(new AERunnable() {
+				public void runSupport() {
+					if (table != null && !table.isDisposed()) {
+						table.clearAll();
+					}
+				}
+			});
+		}
+	}
+
 	public void initPluginSubSections() {
 		// Create subsections for plugins that used the PluginConfigModel object
 		// =====================================================================
