@@ -75,6 +75,11 @@ public class ConfigSectionStats implements UISWTConfigSection {
   public void configSectionDelete() {
   }
   
+	public int maxUserMode() {
+		return 0;
+	}
+
+  
 
   public Composite configSectionCreate(final Composite parent) {
     Image imgOpenFolder = ImageRepository.getImage("openFolderButton");
@@ -97,7 +102,7 @@ public class ConfigSectionStats implements UISWTConfigSection {
                              "ConfigView.section.stats.enable");
     enableStats.setLayoutData(gridData);
 
-    Control[] controls = new Control[12];
+    Control[] controls = new Control[14];
 
     // row
 
@@ -207,14 +212,35 @@ public class ConfigSectionStats implements UISWTConfigSection {
     BooleanParameter exportPeers = 
         new BooleanParameter(gStats, "Stats Export Peer Details",
                              "ConfigView.section.stats.exportpeers");
-    enableStats.setLayoutData(gridData);
+    exportPeers.setLayoutData(gridData);
 
     controls[11] = exportPeers.getControl();
     
+ 	// ROW
+    
+    gridData = new GridData();
+    gridData.horizontalSpan = 3;
+    BooleanParameter exportFiles = 
+        new BooleanParameter(gStats, "Stats Export File Details",
+                             "ConfigView.section.stats.exportfiles");
+    exportFiles.setLayoutData(gridData);
+
+    controls[12] = exportFiles.getControl();
+
+    // ROW
+    
+    gridData = new GridData();
+    gridData.horizontalSpan = 3;
+    BooleanParameter graph_dividers = new BooleanParameter(gStats, "Stats Graph Dividers", "ConfigView.section.stats.graph_update_dividers");
+    graph_dividers.setLayoutData(gridData);
+
+    controls[13] = graph_dividers.getControl();
+    
     	// control stuff
     
     enableStats.setAdditionalActionPerformer(new ChangeSelectionActionPerformer(controls));
 
+ 
     return gStats;
   }
 }
