@@ -62,6 +62,11 @@ ConfigSectionTracker
   public void configSectionDelete() {
   }
   
+	public int maxUserMode() {
+		return 0;
+	}
+
+  
 
   public Composite configSectionCreate(final Composite parent) {
     GridData gridData;
