@@ -29,7 +29,9 @@ import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.Composite;
+import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Label;
+import org.eclipse.swt.widgets.Listener;
 
 import org.gudy.azureus2.plugins.ui.config.ConfigSection;
 import org.gudy.azureus2.ui.swt.components.LinkLabel;
@@ -38,9 +40,22 @@ import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
 import org.gudy.azureus2.ui.swt.ImageRepository;
 import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.core3.config.impl.TransferSpeedValidator;
 import org.gudy.azureus2.core3.internat.MessageText;
 
+import com.aelitis.azureus.core.AzureusCore;
+
 public class ConfigSectionTransfer implements UISWTConfigSection {
+	
+	private AzureusCore	core;
+	
+	public 
+	ConfigSectionTransfer(
+		AzureusCore	_core )
+	{
+		core	= _core;
+	}
+	
 	public String configSectionGetParentSection() {
 		return ConfigSection.SECTION_ROOT;
 	}
@@ -54,6 +69,11 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 
 	public void configSectionDelete() {
 	}
+	
+	public int maxUserMode() {
+		return 2;
+	}
+
 
 	public Composite configSectionCreate(final Composite parent) {
 		GridData gridData;
@@ -84,7 +104,7 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 		gridData = new GridData();
 		gridData.widthHint = 35;
 		final IntParameter paramMaxUploadSpeed = new IntParameter(cSection,
-				"Max Upload Speed KBs", 1, -1, true, true);
+				"Max Upload Speed KBs", 0, -1);
 		paramMaxUploadSpeed.setLayoutData(gridData);
 
 		//  max upload speed when seeding
@@ -107,16 +127,15 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 		label.setImage(img);
 
 		gridData = new GridData();
-		BooleanParameter enable_seeding_rate = new BooleanParameter(
-				cMaxUploadSpeedOptionsArea, "enable.seedingonly.upload.rate", false,
+		final BooleanParameter enable_seeding_rate = new BooleanParameter(
+				cMaxUploadSpeedOptionsArea, "enable.seedingonly.upload.rate",
 				"ConfigView.label.maxuploadspeedseeding");
 		enable_seeding_rate.setLayoutData(gridData);
 
 		gridData = new GridData();
 		gridData.widthHint = 35;
-		IntParameter paramMaxUploadSpeedSeeding = new IntParameter(
-				cMaxUploadSpeedOptionsArea, "Max Upload Speed Seeding KBs", 1, -1, true,
-				false);
+		final IntParameter paramMaxUploadSpeedSeeding = new IntParameter(
+				cMaxUploadSpeedOptionsArea, "Max Upload Speed Seeding KBs", 0, -1);
 		paramMaxUploadSpeedSeeding.setLayoutData(gridData);
 		enable_seeding_rate
 				.setAdditionalActionPerformer(new ChangeSelectionActionPerformer(
@@ -158,9 +177,7 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 
 			gridData = new GridData();
 			gridData.widthHint = 35;
-			new IntParameter(
-					 cSection,
-					 "max.uploads.when.busy.inc.min.secs", 0, -1, true, false).setLayoutData(gridData);
+			new IntParameter(cSection, "max.uploads.when.busy.inc.min.secs", 0, -1).setLayoutData(gridData);
 		}
 		
 		// max download speed
@@ -168,16 +185,60 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 		label = new Label(cSection, SWT.NULL);
 		label.setLayoutData(gridData);
 		Messages.setLanguageText(label, "ConfigView.label.maxdownloadspeed");
-
+		
 		gridData = new GridData();
 		gridData.widthHint = 35;
 		final IntParameter paramMaxDownSpeed = new IntParameter(cSection,
-				"Max Download Speed KBs", 0, -1, true, true);
+				"Max Download Speed KBs", 0, -1);
 		paramMaxDownSpeed.setLayoutData(gridData);
-
+		
 		// max upload/download limit dependencies
-		paramMaxUploadSpeed.addChangeListener(new ParameterChangeListener() {
+		
+		Listener l = new Listener() {
+	
+			public void handleEvent(Event event) {
+				boolean disableAuto = false;
+				boolean disableAutoSeeding = false;
+				
+				if(enable_seeding_rate.isSelected())
+				{
+					disableAutoSeeding = event.widget == paramMaxUploadSpeedSeeding.getControl();
+					disableAuto = event.widget == paramMaxDownSpeed.getControl() || event.widget == paramMaxUploadSpeed.getControl();
+				} else
+				{
+					disableAuto = true;
+					disableAutoSeeding = true;
+				}
+					
+					
+				if(disableAuto)
+					COConfigurationManager.setParameter(TransferSpeedValidator.AUTO_UPLOAD_ENABLED_CONFIGKEY, false);
+				if(disableAutoSeeding)
+					COConfigurationManager.setParameter(TransferSpeedValidator.AUTO_UPLOAD_SEEDING_ENABLED_CONFIGKEY, false);
+			}
+		};
+		
+		paramMaxDownSpeed.getControl().addListener(SWT.Selection, l);
+		paramMaxUploadSpeed.getControl().addListener(SWT.Selection, l);
+		paramMaxUploadSpeedSeeding.getControl().addListener(SWT.Selection, l);
+		
+		
+		paramMaxUploadSpeed.addChangeListener(new ParameterChangeAdapter() {
+			
 			public void parameterChanged(Parameter p, boolean internal) {
+				if ( paramMaxUploadSpeed.isDisposed()){
+					paramMaxUploadSpeed.removeChangeListener( this );
+					return;
+				}
+				
+					// we don't want to police these limits when auto-speed is running as
+					// they screw things up bigtime
+				
+				if ( TransferSpeedValidator.isAutoSpeedActive( core.getGlobalManager())){
+					
+					return;
+				}
+				
 				int up_val = paramMaxUploadSpeed.getValue();
 				int down_val = paramMaxDownSpeed.getValue();
 
@@ -198,15 +259,26 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 			}
 		});
 
-		paramMaxDownSpeed.addChangeListener(new ParameterChangeListener() {
+		paramMaxDownSpeed.addChangeListener(new ParameterChangeAdapter() {
 			public void parameterChanged(Parameter p, boolean internal) {
+				
+				if ( paramMaxDownSpeed.isDisposed()){
+					paramMaxDownSpeed.removeChangeListener( this );
+					return;
+				}
+				
+					// we don't want to police these limits when auto-speed is running as
+					// they screw things up bigtime
+				
+				if ( TransferSpeedValidator.isAutoSpeedActive( core.getGlobalManager())){
+					
+					return;
+				}
+				
 				int up_val = paramMaxUploadSpeed.getValue();
 				int down_val = paramMaxDownSpeed.getValue();
 
-				if (!internal) {
-
-					manual_max_download_speed[0] = down_val;
-				}
+				manual_max_download_speed[0] = down_val;
 
 				if (up_val < COConfigurationManager.CONFIG_DEFAULT_MIN_MAX_UPLOAD_SPEED) {
 
@@ -223,6 +295,7 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 		});
 
 		if (userMode > 0) {
+			
 
 			// max uploads
 			gridData = new GridData();
@@ -233,7 +306,7 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 			gridData = new GridData();
 			gridData.widthHint = 35;
 			IntParameter paramMaxUploads = new IntParameter(cSection, "Max Uploads",
-					2, -1, false, false);
+					2, -1);
 			paramMaxUploads.setLayoutData(gridData);
 
 				// max uploads when seeding
@@ -257,34 +330,20 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 			
 			gridData = new GridData();
 			BooleanParameter enable_seeding_uploads = new BooleanParameter(
-					cMaxUploadsOptionsArea, "enable.seedingonly.maxuploads", false,
+					cMaxUploadsOptionsArea, "enable.seedingonly.maxuploads",
 					"ConfigView.label.maxuploadsseeding");
 			enable_seeding_uploads.setLayoutData(gridData);
 
 			gridData = new GridData();
 			gridData.widthHint = 35;
 			IntParameter paramMaxUploadsSeeding = new IntParameter(
-					cMaxUploadsOptionsArea, "Max Uploads Seeding", 2, -1, false,
-					false);
+					cMaxUploadsOptionsArea, "Max Uploads Seeding", 2, -1);
 			paramMaxUploadsSeeding.setLayoutData(gridData);
 			enable_seeding_uploads
 					.setAdditionalActionPerformer(new ChangeSelectionActionPerformer(
 							paramMaxUploadsSeeding.getControl()));
 			
 			
-			
-			
-			
-			
-			
-			
-			
-			
-			
-			
-			
-			
-			
 			////
 
 			gridData = new GridData();
@@ -298,6 +357,44 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 					"Max.Peer.Connections.Per.Torrent");
 			paramMaxClients.setLayoutData(gridData);
 
+			
+			/////
+			
+				// max peers when seeding
+			
+			final Composite cMaxPeersOptionsArea = new Composite(cSection, SWT.NULL);
+			layout = new GridLayout();
+			layout.numColumns = 3;
+			layout.marginWidth = 0;
+			layout.marginHeight = 0;
+			cMaxPeersOptionsArea.setLayout(layout);
+			gridData = new GridData();
+			gridData.horizontalIndent = 15;
+			gridData.horizontalSpan = 2;
+			cMaxPeersOptionsArea.setLayoutData(gridData);
+			label = new Label(cMaxPeersOptionsArea, SWT.NULL);
+			img = ImageRepository.getImage("subitem");
+			img.setBackground(label.getBackground());
+			gridData = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
+			label.setLayoutData(gridData);
+			label.setImage(img);
+			
+			gridData = new GridData();
+			BooleanParameter enable_max_peers_seeding = new BooleanParameter(
+					cMaxPeersOptionsArea, "Max.Peer.Connections.Per.Torrent.When.Seeding.Enable",
+					"ConfigView.label.maxuploadsseeding");
+			enable_max_peers_seeding.setLayoutData(gridData);
+
+			gridData = new GridData();
+			gridData.widthHint = 35;
+			IntParameter paramMaxPeersSeeding = new IntParameter(
+					cMaxPeersOptionsArea, "Max.Peer.Connections.Per.Torrent.When.Seeding", 0, -1);
+			paramMaxPeersSeeding.setLayoutData(gridData);
+			enable_max_peers_seeding
+					.setAdditionalActionPerformer(new ChangeSelectionActionPerformer(
+							paramMaxPeersSeeding.getControl()));
+	
+			
 			/////
 
 			gridData = new GridData();
@@ -310,25 +407,52 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 			IntParameter paramMaxClientsTotal = new IntParameter(cSection,
 					"Max.Peer.Connections.Total");
 			paramMaxClientsTotal.setLayoutData(gridData);
+			
+			gridData = new GridData();
+			label = new Label(cSection, SWT.NULL);
+			label.setLayoutData(gridData);
+			Messages.setLanguageText(label, "ConfigView.label.maxseedspertorrent");
+
+			gridData = new GridData();
+			gridData.widthHint = 35;
+			new IntParameter(cSection,"Max Seeds Per Torrent").setLayoutData(gridData);
+
+			
+			gridData = new GridData();
+			gridData.horizontalSpan = 2;
+			BooleanParameter useReqLimiting = new BooleanParameter(cSection, "Use Request Limiting",
+				"ConfigView.label.userequestlimiting");
+			useReqLimiting.setLayoutData(gridData);
+
+			gridData = new GridData();
+			gridData.horizontalSpan = 2;
+			BooleanParameter useReqLimitingPrios = new BooleanParameter(cSection, "Use Request Limiting Priorities",
+				"ConfigView.label.userequestlimitingpriorities");
+			useReqLimitingPrios.setLayoutData(gridData);
+			useReqLimiting
+			.setAdditionalActionPerformer(new ChangeSelectionActionPerformer(
+					useReqLimitingPrios.getControl()));
+			
+			
 
 			gridData = new GridData();
 			gridData.horizontalSpan = 2;
 			BooleanParameter allowSameIP = new BooleanParameter(cSection,
-					"Allow Same IP Peers", false, "ConfigView.label.allowsameip");
+					"Allow Same IP Peers", "ConfigView.label.allowsameip");
 			allowSameIP.setLayoutData(gridData);
 
 			// lazy bit field
 			gridData = new GridData();
 			gridData.horizontalSpan = 2;
 			BooleanParameter lazybf = new BooleanParameter(cSection,
-					"Use Lazy Bitfield", false, "ConfigView.label.lazybitfield");
+					"Use Lazy Bitfield", "ConfigView.label.lazybitfield");
 			lazybf.setLayoutData(gridData);
 
 			// prioritise first/last pieces
 			gridData = new GridData();
 			gridData.horizontalSpan = 2;
 			BooleanParameter firstPiece = new BooleanParameter(cSection,
-					"Prioritize First Piece", false,
+					"Prioritize First Piece",
 					"ConfigView.label.prioritizefirstpiece");
 			firstPiece.setLayoutData(gridData);
 
@@ -336,7 +460,7 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 			gridData = new GridData();
 			gridData.horizontalSpan = 2;
 			BooleanParameter mostCompletedFiles = new BooleanParameter(cSection,
-					"Prioritize Most Completed Files", false,
+					"Prioritize Most Completed Files",
 					"ConfigView.label.prioritizemostcompletedfiles");
 			mostCompletedFiles.setLayoutData(gridData);
 
@@ -363,9 +487,20 @@ public class ConfigSectionTransfer implements UISWTConfigSection {
 			StringParameter ignore_ports = new StringParameter(cMiniArea,
 					"Ignore.peer.ports", "0");
 			ignore_ports.setLayoutData(gridData);
-
-		} //end usermode>0
-
+	
+			gridData = new GridData();
+			gridData.horizontalSpan = 2;
+			BooleanParameter pauseOnExit = new BooleanParameter(cSection,
+					"Pause Downloads On Exit", "ConfigView.label.pause.downloads.on.exit");
+			pauseOnExit.setLayoutData(gridData);
+	
+			gridData = new GridData();
+			gridData.horizontalSpan = 2;
+			BooleanParameter resumeOnStart = new BooleanParameter(cSection,
+					"Resume Downloads On Start", "ConfigView.label.resume.downloads.on.start");
+			resumeOnStart.setLayoutData(gridData);
+		}
+		
 		return cSection;
 	}
 }
