@@ -51,7 +51,9 @@ import com.aelitis.azureus.core.dht.control.DHTControlActivity;
 import com.aelitis.azureus.core.dht.control.DHTControlListener;
 import com.aelitis.azureus.core.dht.control.DHTControlStats;
 import com.aelitis.azureus.core.dht.db.DHTDBStats;
+import com.aelitis.azureus.core.dht.nat.DHTNATPuncher;
 import com.aelitis.azureus.core.dht.router.DHTRouterStats;
+import com.aelitis.azureus.core.dht.transport.DHTTransport;
 import com.aelitis.azureus.core.dht.transport.DHTTransportFullStats;
 import com.aelitis.azureus.core.dht.transport.DHTTransportStats;
 import com.aelitis.azureus.plugins.dht.DHTPlugin;
@@ -61,9 +63,10 @@ import com.aelitis.azureus.plugins.dht.DHTPlugin;
  */
 public class DHTView extends AbstractIView {
   
-  public static final int DHT_TYPE_MAIN = 0;
-  public static final int DHT_TYPE_CVS  = 1;
-  
+  public static final int DHT_TYPE_MAIN 	= DHT.NW_MAIN;
+  public static final int DHT_TYPE_CVS  	= DHT.NW_CVS;
+  public static final int DHT_TYPE_MAIN_V6 	= DHT.NW_MAIN_V6;
+
   DHT dht;
   
   Composite panel;
@@ -74,7 +77,7 @@ public class DHTView extends AbstractIView {
   Label lblUpTime,lblNumberOfUsers;
   Label lblNodes,lblLeaves;
   Label lblContacts,lblReplacements,lblLive,lblUnknown,lblDying;
-  Label lblRendezvous, lblReachable;
+  Label lblSkew, lblRendezvous, lblReachable;
   Label lblKeys,lblValues;
   Label lblLocal,lblDirect,lblIndirect;
   Label lblDivFreq,lblDivSize;
@@ -116,12 +119,18 @@ public class DHTView extends AbstractIView {
          
       DHT[] dhts = ((DHTPlugin)dht_pi.getPlugin()).getDHTs();
       
-      if( dhts.length <= dht_type ){
-        return;
+      for (int i=0;i<dhts.length;i++){
+    	  if ( dhts[i].getTransport().getNetwork() == dht_type ){
+    		  dht = dhts[i];
+    		  break;
+    	  }
       }
 	  
-	  dht	= dhts[ dht_type ];
-	  
+      if ( dht == null ){
+    	  
+    	  return;
+      }
+      
       controlListener = new DHTControlListener() {
         public void activityChanged(DHTControlActivity activity,int type) {
           activityChanged = true;
@@ -140,8 +149,8 @@ public class DHTView extends AbstractIView {
     layout.numColumns = 2;
     panel.setLayout(layout);
     
-    yes_str = MessageText.getString( "Button.yes");
-    no_str 	= MessageText.getString( "Button.no");
+    yes_str = MessageText.getString( "Button.yes").replaceAll("&", "");
+    no_str 	= MessageText.getString( "Button.no").replaceAll("&", "");
     
     initialiseGeneralGroup();
     initialiseDBGroup();
@@ -165,6 +174,7 @@ public class DHTView extends AbstractIView {
     layout.numColumns = 6;
     gGeneral.setLayout(layout);
     
+    	// row1 
     
     Label label = new Label(gGeneral,SWT.NONE);
     Messages.setLanguageText(label,"DHTView.general.uptime");    
@@ -184,6 +194,9 @@ public class DHTView extends AbstractIView {
     lblReachable = new Label(gGeneral,SWT.NONE);
     lblReachable.setLayoutData(new GridData(SWT.FILL,SWT.TOP,true,false));
     
+    
+    	// row 2
+    
     label = new Label(gGeneral,SWT.NONE);
     Messages.setLanguageText(label,"DHTView.general.nodes");    
     
@@ -202,6 +215,7 @@ public class DHTView extends AbstractIView {
     lblRendezvous = new Label(gGeneral,SWT.NONE);
     lblRendezvous.setLayoutData(new GridData(SWT.FILL,SWT.TOP,true,false));
     
+    	// row 3
     
     label = new Label(gGeneral,SWT.NONE);
     Messages.setLanguageText(label,"DHTView.general.contacts");    
@@ -221,9 +235,14 @@ public class DHTView extends AbstractIView {
     lblLive= new Label(gGeneral,SWT.NONE);
     lblLive.setLayoutData(new GridData(SWT.FILL,SWT.TOP,true,false));
     
-    label = new Label(gGeneral,SWT.NONE);
-    label = new Label(gGeneral,SWT.NONE);
+    	// row 4
     
+    label = new Label(gGeneral,SWT.NONE);
+    Messages.setLanguageText(label,"DHTView.general.skew");    
+    
+    lblSkew= new Label(gGeneral,SWT.NONE);
+    lblSkew.setLayoutData(new GridData(SWT.FILL,SWT.TOP,true,false));
+
     label = new Label(gGeneral,SWT.NONE);
     Messages.setLanguageText(label,"DHTView.general.unknown");    
     
@@ -348,7 +367,7 @@ public class DHTView extends AbstractIView {
     label.setLayoutData(data);
     
     
-    in = new Canvas(gTransport,SWT.NONE);
+    in = new Canvas(gTransport,SWT.NO_BACKGROUND);
     data = new GridData(GridData.FILL_BOTH);
     data.horizontalSpan = 3;
     in.setLayoutData(data);
@@ -361,7 +380,7 @@ public class DHTView extends AbstractIView {
     data.horizontalSpan = 3;
     label.setLayoutData(data);
     
-    out = new Canvas(gTransport,SWT.NONE);
+    out = new Canvas(gTransport,SWT.NO_BACKGROUND);
     data = new GridData(GridData.FILL_BOTH);
     data.horizontalSpan = 3;
     out.setLayoutData(data);
@@ -472,15 +491,28 @@ public class DHTView extends AbstractIView {
     
     activityTable.setHeaderVisible(true);
     Listener computeLastRowWidthListener = new Listener() {
+    	// inUse flag to prevent a SWT stack overflow.  For some reason
+    	// the setWidth call was triggering a resize.
+    	boolean inUse = false;
       public void handleEvent(Event event) {
-        if(activityTable == null || activityTable.isDisposed()) return;
-        int totalWidth = activityTable.getClientArea().width;
-        int remainingWidth = totalWidth 
-                               - colStatus.getWidth()
-                               - colType.getWidth()
-                               - colName.getWidth();
-        if(remainingWidth > 0)
-          colDetails.setWidth(remainingWidth);
+      	if (inUse) {
+      		return;
+      	}
+
+      	inUse = true;
+       	try {
+          if(activityTable == null || activityTable.isDisposed()) return;
+          int totalWidth = activityTable.getClientArea().width;
+          int remainingWidth = totalWidth 
+                                 - colStatus.getWidth()
+                                 - colType.getWidth()
+                                 - colName.getWidth();
+          if(remainingWidth > 0)
+            colDetails.setWidth(remainingWidth);
+
+       	} finally {
+      		inUse = false;
+      	}
       }
     };
     activityTable.addListener(SWT.Resize, computeLastRowWidthListener);    
@@ -512,7 +544,17 @@ public class DHTView extends AbstractIView {
   }
 
   public String getFullTitle() {
-    return dht_type == DHT_TYPE_MAIN ? MessageText.getString("DHTView.title.full") : MessageText.getString("DHTView.title.fullcvs");
+	  if ( dht_type == DHT_TYPE_MAIN ){
+
+		  return( "DHTView.title.full" );
+
+	  }else if ( dht_type == DHT_TYPE_CVS ){
+
+		  return( "DHTView.title.fullcvs" );
+	  }else{
+
+		  return( "DHTView.title.full_v6" );
+	  }
   }
   
   public Composite getComposite() {
@@ -538,10 +580,23 @@ public class DHTView extends AbstractIView {
   private void refreshGeneral() {
     DHTControlStats controlStats = dht.getControl().getStats();
     DHTRouterStats routerStats = dht.getRouter().getStats();
+    DHTTransport transport = dht.getTransport();
+    DHTTransportStats transportStats = transport.getStats();
     lblUpTime.setText(TimeFormatter.format(controlStats.getRouterUptime() / 1000));
     lblNumberOfUsers.setText("" + controlStats.getEstimatedDHTSize());
-    lblReachable.setText(dht.getTransport().isReachable()?yes_str:no_str);
-    lblRendezvous.setText(dht.getTransport().isReachable()?"":(dht.getNATPuncher().operational()?yes_str:no_str));
+    lblReachable.setText(transport.isReachable()?yes_str:no_str);
+    
+    DHTNATPuncher puncher = dht.getNATPuncher();
+    
+    String	puncher_str;
+    
+    if ( puncher == null ){
+    	puncher_str = "";
+    }else{
+    	puncher_str = puncher.operational()?yes_str:no_str;
+    }
+    
+    lblRendezvous.setText(transport.isReachable()?"":puncher_str);
     long[] stats = routerStats.getStats();
     lblNodes.setText("" + stats[DHTRouterStats.ST_NODES]);
     lblLeaves.setText("" + stats[DHTRouterStats.ST_LEAVES]);
@@ -550,6 +605,10 @@ public class DHTView extends AbstractIView {
     lblLive.setText("" + stats[DHTRouterStats.ST_CONTACTS_LIVE]);
     lblUnknown.setText("" + stats[DHTRouterStats.ST_CONTACTS_UNKNOWN]);
     lblDying.setText("" + stats[DHTRouterStats.ST_CONTACTS_DEAD]);
+    
+    long skew_average = transportStats.getSkewAverage();
+    
+    lblSkew.setText( skew_average==0?"":(skew_average<0?"-":"") + TimeFormatter.format100ths( Math.abs(skew_average )));
   }
   
   private int refreshIter = 0;
@@ -627,9 +686,8 @@ public class DHTView extends AbstractIView {
   }
   
   public String getData() {
-    return dht_type == DHT_TYPE_MAIN ? "DHTView.title.full" :"DHTView.title.fullcvs";
+	  return( getFullTitle());
   }
-    
 }
 
 
