@@ -21,6 +21,9 @@
  */
 package org.gudy.azureus2.ui.swt.views.stats;
 
+import java.util.ArrayList;
+import java.util.List;
+
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.events.SelectionListener;
@@ -39,6 +42,7 @@ import org.gudy.azureus2.ui.swt.views.AbstractIView;
 import org.gudy.azureus2.ui.swt.views.IView;
 
 import com.aelitis.azureus.core.AzureusCore;
+import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
 
 /**
  * 
@@ -53,16 +57,14 @@ public class StatsView extends AbstractIView {
   TabItem itemActivity;
   TabItem itemStats;
   TabItem itemCache;
-  TabItem itemDHT;
-  TabItem itemDHTcvs;
-  TabItem itemVivaldi;
+  TabItem[] itemDHTs;
+  TabItem[] itemVivaldis;
   
-  IView viewActivity;
-  IView viewStats;
-  IView viewCache;
-  IView viewDHT;
-  IView viewDHTcvs;
-  IView viewVivaldi;
+  ActivityView viewActivity;
+  TransferStatsView viewStats;
+  CacheView viewCache;
+  DHTView[] viewDHTs;
+  IView[] viewVivaldis;
   UpdateThread updateThread;
   
   public StatsView(GlobalManager manager,AzureusCore core) {
@@ -77,25 +79,30 @@ public class StatsView extends AbstractIView {
 			super("StatsView Update Thread");
 		}
     
-    public void run() {
-      try {
-        bContinue = true;
-        while(bContinue) {   
-          ((ActivityView)viewActivity).periodicUpdate();
-          ((CacheView)viewCache).periodicUpdate(); 
-          ((DHTView)viewDHT).periodicUpdate();
-          ((TransferStatsView)viewStats).periodicUpdate();
-          
-          if( viewDHTcvs != null ) {
-            ((DHTView)viewDHTcvs).periodicUpdate();
-          }
-          
-          Thread.sleep(1000);
-        }
-      } catch(Exception e) {
-      	Debug.printStackTrace( e );  
-      }
-    }
+		public void run() {
+			bContinue = true;
+			while(bContinue) {  
+				try {
+
+					viewActivity.periodicUpdate();
+					viewCache.periodicUpdate(); 
+					viewStats.periodicUpdate();
+
+					for (int i=0;i<viewDHTs.length;i++){
+						viewDHTs[i].periodicUpdate();
+					}
+				} catch(Exception e) {
+					Debug.printStackTrace( e );  
+				}
+				try{
+					Thread.sleep(1000);
+				}catch( Throwable e ){
+
+					Debug.out( e );
+					break;
+				}
+			}
+		}
     
     public void stopIt() {
       bContinue = false;
@@ -106,31 +113,62 @@ public class StatsView extends AbstractIView {
     folder = new TabFolder(composite, SWT.LEFT);
     folder.setBackground(Colors.background);
     
+    List	dhts = new ArrayList();
+    List  vivaldis = new ArrayList();
+
+    dhts.add( new DHTView( DHTView.DHT_TYPE_MAIN ));  
+    vivaldis.add( new VivaldiView( VivaldiView.DHT_TYPE_MAIN ));  
+
+    if ( NetworkAdmin.getSingleton().hasIPV6Potential()){
+  
+    	dhts.add(  new DHTView( DHTView.DHT_TYPE_MAIN_V6 ));
+    	vivaldis.add(  new VivaldiView( VivaldiView.DHT_TYPE_MAIN_V6 ));
+    }
+    
+    if( Constants.isCVSVersion()){
+    	
+       	dhts.add(  new DHTView( DHTView.DHT_TYPE_CVS ));
+       	vivaldis.add(  new VivaldiView( VivaldiView.DHT_TYPE_CVS ));
+    }
+    
+    viewDHTs = new DHTView[dhts.size()];
+    viewVivaldis = new IView[vivaldis.size()];
+    
+    dhts.toArray( viewDHTs );
+    vivaldis.toArray( viewVivaldis );
+    
     itemActivity = new TabItem(folder, SWT.NULL);
     itemStats = new TabItem(folder, SWT.NULL);
     itemCache  = new TabItem(folder, SWT.NULL);
-    itemDHT  = new TabItem(folder, SWT.NULL);
-    if( Constants.isCVSVersion() )  itemDHTcvs  = new TabItem(folder, SWT.NULL);
-    itemVivaldi = new TabItem(folder,SWT.NULL);
+    
+    itemDHTs = new TabItem[viewDHTs.length];
+    
+    for (int i=0;i<itemDHTs.length;i++){
+    	itemDHTs[i] = new TabItem(folder, SWT.NULL);
+    }
+
+    itemVivaldis = new TabItem[viewVivaldis.length];
+
+    for (int i=0;i<itemVivaldis.length;i++){
+      itemVivaldis[i] = new TabItem(folder,SWT.NULL);
+    }
 
     viewActivity = new ActivityView(manager);
     viewStats = new TransferStatsView(manager,core);
     viewCache = new CacheView();
-    viewDHT = new DHTView( DHTView.DHT_TYPE_MAIN );  
-    if( Constants.isCVSVersion() )  viewDHTcvs = new DHTView( DHTView.DHT_TYPE_CVS );
-    viewVivaldi = new VivaldiView();
+    
+ 
+    
     
     Messages.setLanguageText(itemActivity, viewActivity.getData());
     Messages.setLanguageText(itemStats, viewStats.getData());
     Messages.setLanguageText(itemCache, viewCache.getData());
-    Messages.setLanguageText(itemDHT, viewDHT.getData());
     
-    if( viewDHTcvs != null ) {
-      Messages.setLanguageText(itemDHTcvs, viewDHTcvs.getData());
+    for (int i=0;i<viewDHTs.length;i++){
+    	Messages.setLanguageText(itemDHTs[i], viewDHTs[i].getData());
+    	Messages.setLanguageText(itemVivaldis[i], viewVivaldis[i].getData());
     }
     
-    Messages.setLanguageText(itemVivaldi, viewVivaldi.getData());
-    
     TabItem items[] = {itemActivity};
     folder.setSelection(items);
     
@@ -143,16 +181,15 @@ public class StatsView extends AbstractIView {
     viewCache.initialize(folder);
     itemCache.setControl(viewCache.getComposite());
     
-    viewDHT.initialize(folder);
-    itemDHT.setControl(viewDHT.getComposite());
-
-    if( viewDHTcvs != null ) {
-      viewDHTcvs.initialize(folder);
-      itemDHTcvs.setControl(viewDHTcvs.getComposite());
+    for (int i=0;i<viewDHTs.length;i++){
+    	viewDHTs[i].initialize(folder);
+    	itemDHTs[i].setControl(viewDHTs[i].getComposite());
     }
     
-    viewVivaldi.initialize(folder);
-    itemVivaldi.setControl(viewVivaldi.getComposite());
+    for (int i=0;i<viewVivaldis.length;i++){
+    	viewVivaldis[i].initialize(folder);
+    	itemVivaldis[i].setControl(viewVivaldis[i].getComposite());
+    }
     
     folder.addSelectionListener(new SelectionListener() {
       public void widgetSelected(SelectionEvent e) {
@@ -165,7 +202,9 @@ public class StatsView extends AbstractIView {
     
     refresh();
     viewActivity.getComposite().layout(true);
-    viewVivaldi.getComposite().layout(true);
+    for (int i=0;i<viewVivaldis.length;i++){
+      viewVivaldis[i].getComposite().layout(true);
+    }
     
     updateThread = new UpdateThread(); 
     updateThread.setDaemon(true);
@@ -194,28 +233,20 @@ public class StatsView extends AbstractIView {
         return;
       }
       
-      if( index == 3 ) {
-        if (viewDHT != null && !itemDHT.isDisposed())  viewDHT.refresh();
-        return;
+      if ( index-3 < viewDHTs.length ){
+    	  if ( !itemDHTs[index-3].isDisposed()){
+    		  viewDHTs[index-3].refresh();
+    	  }
+    	  return;
       }
       
-      if( Constants.isCVSVersion() ) {
-        if( index == 4 ) {
-          if (viewDHTcvs != null && !itemDHTcvs.isDisposed())  viewDHTcvs.refresh();
-          return;
+      if ( index-3-viewDHTs.length < viewVivaldis.length ){
+        if ( !itemVivaldis[index-3-viewDHTs.length].isDisposed()){
+          viewVivaldis[index-3-viewDHTs.length].refresh();
         }
-        
-        if( index == 5 ) {
-          if (viewVivaldi != null && !itemVivaldi.isDisposed())  viewVivaldi.refresh();
-          return;
-        }
-      }
-      
-      if( index == 4 ) {
-        if (viewVivaldi != null && !itemVivaldi.isDisposed())  viewVivaldi.refresh();
         return;
       }
-
+  
     } catch (Exception e) {
     	Debug.printStackTrace( e );
     }
@@ -230,7 +261,7 @@ public class StatsView extends AbstractIView {
   public void
   showDHT()
   {
-	  folder.setSelection( new TabItem[]{ itemDHT });
+	  folder.setSelection( new TabItem[]{ itemDHTs[0] });
   }
   
   public Composite getComposite() {
@@ -260,8 +291,11 @@ public class StatsView extends AbstractIView {
     viewActivity.delete();
     viewStats.delete();
     viewCache.delete();
-    viewDHT.delete();
-    if( viewDHTcvs != null )  viewDHTcvs.delete();
+    
+    for (int i=0;i<viewDHTs.length;i++){
+    	viewDHTs[i].delete();
+    }
+
     if(! folder.isDisposed()) {
       Utils.disposeComposite(folder);
     }
