@@ -41,11 +41,15 @@ import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.components.*;
 import org.gudy.azureus2.ui.swt.debug.ObfusticateCellText;
+import org.gudy.azureus2.ui.swt.mainwindow.Colors;
 import org.gudy.azureus2.ui.swt.plugins.UISWTGraphic;
 import org.gudy.azureus2.ui.swt.pluginsimpl.UISWTGraphicImpl;
-import org.gudy.azureus2.ui.swt.views.table.TableCellCore;
-import org.gudy.azureus2.ui.swt.views.table.TableColumnCore;
-import org.gudy.azureus2.ui.swt.views.table.TableRowCore;
+import org.gudy.azureus2.ui.swt.views.table.TableCellSWT;
+import org.gudy.azureus2.ui.swt.views.table.TableRowSWT;
+import org.gudy.azureus2.ui.swt.views.table.TableViewSWT;
+import org.gudy.azureus2.ui.swt.views.table.utils.TableColumnSWTUtils;
+
+import com.aelitis.azureus.ui.common.table.*;
 
 import org.gudy.azureus2.plugins.ui.Graphic;
 import org.gudy.azureus2.plugins.ui.UIRuntimeException;
@@ -63,27 +67,13 @@ import org.gudy.azureus2.plugins.ui.tables.*;
  * This object is needed to split core code from plugin code.
  */
 public class TableCellImpl 
-       implements TableCellCore
+       implements TableCellSWT
 {
 	private static final LogIDs LOGID = LogIDs.GUI;
-  private TableRowCore tableRow;
-  private Comparable sortValue;
-  private boolean bSortValueIsText = true;
-  private BufferedTableItem bufferedTableItem;
-  private ArrayList refreshListeners;
-  private ArrayList disposeListeners;
-  private ArrayList tooltipListeners;
-	private ArrayList cellMouseListeners;
-	private ArrayList cellVisibilityListeners;
-  private TableColumnCore tableColumn;
-  private boolean valid;
-  private int refreshErrLoopCount;
-  private int tooltipErrLoopCount;
-  private int loopFactor;
-  private Object oToolTip;
-	private int iCursorID = -1;
-	private Graphic graphic = null;
-  
+	
+	private static final byte FLAG_VALID = 1;
+	private static final byte FLAG_SORTVALUEISTEXT = 2;
+	private static final byte FLAG_TOOLTIPISAUTO = 4;
   /**
    * For refreshing, this flag manages whether the row is actually up to date.
    * 
@@ -96,20 +86,49 @@ public class TableCellImpl
    * that the row will set its visuals again (this time, actually
    * updating a viewable object).
    */
-	private boolean bIsUpToDate = true;
+	private static final byte FLAG_UPTODATE = 8;
+	private static final byte FLAG_DISPOSED = 16;
+	private static final byte FLAG_MUSTREFRESH = 32;
+	private static final byte FLAG_VISUALLY_CHANGED_SINCE_REFRESH = 64;
 	
-	private boolean bDisposed = false;
-	
-	private boolean bMustRefresh = false;
+	private byte flags;
+
+  private TableRowCore tableRow;
+  private Comparable sortValue;
+  private BufferedTableItem bufferedTableItem;
+  private ArrayList refreshListeners;
+  private ArrayList disposeListeners;
+  private ArrayList tooltipListeners;
+	private ArrayList cellMouseListeners;
+	private ArrayList cellMouseMoveListeners;
+	private ArrayList cellVisibilityListeners;
+  private TableColumnCore tableColumn;
+  private byte refreshErrLoopCount;
+  private byte tooltipErrLoopCount;
+  private byte loopFactor;
+  private Object oToolTip;
+	private int iCursorID = -1;
+	private Graphic graphic = null;
 	
+	private ArrayList childCells;
+  
 	public boolean bDebug = false;
   
-  private AEMonitor 	this_mon 	= new AEMonitor( "TableCell" );
-	private boolean bCellVisuallyChangedSinceRefresh;
+  private static AEMonitor 	this_mon 	= new AEMonitor( "TableCell" );
 
   private static final String CFG_PAINT = "GUI_SWT_bAlternateTablePainting";
   private static boolean bAlternateTablePainting;
 
+	private static int MAX_REFRESHES = 10;
+	private static int MAX_REFRESHES_WITHIN_MS = 100;
+
+	private boolean bInRefresh = false;
+	private long lastRefresh;
+	private int numFastRefreshes;
+
+	private byte restartRefresh = 0;
+	private boolean bInRefreshAsync = false;
+	
   static {
   	COConfigurationManager.addAndFireParameterListener(CFG_PAINT,
 				new ParameterListener() {
@@ -124,18 +143,19 @@ public class TableCellImpl
       int position, BufferedTableItem item) {
     this.tableColumn = _tableColumn;
     this.tableRow = _tableRow;
-    valid = false;
+    flags = FLAG_SORTVALUEISTEXT | FLAG_UPTODATE;
     refreshErrLoopCount = 0;
     tooltipErrLoopCount = 0;
     loopFactor = 0;
 
-    bufferedTableItem = item;
+    if (item != null) {
+    	bufferedTableItem = item;
+    } else {
+    	createBufferedTableItem(position);
+    }
 
-    Utils.execSWTThread(new AERunnable() {
-    	public void runSupport() {
-        tableColumn.invokeCellAddedListeners(TableCellImpl.this);
-    	}
-    });
+    tableColumn.invokeCellAddedListeners(TableCellImpl.this);
+    //bDebug = (position == 1) && tableColumn.getTableID().equalsIgnoreCase("Peers");
   }
 
   /**
@@ -145,23 +165,9 @@ public class TableCellImpl
    * @param _tableColumn
    * @param position
    */
-  public TableCellImpl(TableRowCore _tableRow, TableColumnCore _tableColumn,
+  public TableCellImpl(TableRowSWT _tableRow, TableColumnCore _tableColumn,
                        int position) {
-    this.tableColumn = _tableColumn;
-    this.tableRow = _tableRow;
-    valid = false;
-    refreshErrLoopCount = 0;
-    tooltipErrLoopCount = 0;
-    loopFactor = 0;
-
-    createBufferedTableItem(position);
-    
-    Utils.execSWTThread(new AERunnable() {
-    	public void runSupport() {
-        tableColumn.invokeCellAddedListeners(TableCellImpl.this);
-    	}
-    });
-    //bDebug = (position == 1) && tableColumn.getTableID().equalsIgnoreCase("Peers");
+  	this(_tableRow, _tableColumn, position, null);
   }
   
   private void createBufferedTableItem(int position) {
@@ -173,7 +179,7 @@ public class TableCellImpl
 	          TableCellImpl.this.refresh();
 	        }
 	        public void invalidate() {
-	          TableCellImpl.this.valid = false;
+	        	clearFlag(FLAG_VALID);
 	        }
 	      };
 	    } else {
@@ -182,7 +188,7 @@ public class TableCellImpl
 	          TableCellImpl.this.refresh();
 	        }
 	        public void invalidate() {
-	          TableCellImpl.this.valid = false;
+	        	clearFlag(FLAG_VALID);
 	        }
 	      };
 	    }
@@ -193,7 +199,7 @@ public class TableCellImpl
           TableCellImpl.this.refresh();
         }
         public void invalidate() {
-          TableCellImpl.this.valid = false;
+        	clearFlag(FLAG_VALID);
         }
       };
     }
@@ -209,6 +215,19 @@ public class TableCellImpl
     Logger.log(new LogEvent(LOGID, "Table Cell Plugin for Column #" + sPosition
 				+ " generated an exception ", e));
   }
+
+  private void pluginError(String s) {
+    String sTitleLanguageKey = tableColumn.getTitleLanguageKey();
+
+		String sPosition = "r"
+				+ tableRow.getIndex()
+				+ (bufferedTableItem == null ? "null" : "c"
+						+ bufferedTableItem.getPosition() + " ("
+						+ MessageText.getString(sTitleLanguageKey) + ")");
+		Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR,
+				"Table Cell Plugin for Column #" + sPosition + ":" + s + "\n  "
+						+ Debug.getStackTrace(true, true)));
+  }
   
   private void checkCellForSetting() {
   	if (isDisposed())
@@ -244,7 +263,9 @@ public class TableCellImpl
   }
   
   public boolean isValid() {
-    return valid;
+  	// Called often.. inline faster
+  	return (flags & FLAG_VALID) != 0;
+    //return hasFlag(FLAG_VALID);
   }
   
   public Color getForegroundSWT() {
@@ -253,8 +274,34 @@ public class TableCellImpl
     return bufferedTableItem.getForeground();
   }
   
+  public Color getBackgroundSWT() {
+		checkCellForSetting();
+
+		return bufferedTableItem.getBackground();
+	}
+
+  public int[] getBackground() {
+		if (bufferedTableItem == null) {
+			return null;
+		}
+		Color color = bufferedTableItem.getBackground();
+
+		if (color == null) {
+			return null;
+		}
+
+		return new int[] {
+			color.getRed(),
+			color.getGreen(),
+			color.getBlue()
+		};
+	}
+
   // @see org.gudy.azureus2.plugins.ui.tables.TableCell#getForeground()
   public int[] getForeground() {
+  	if (bufferedTableItem == null) {
+  		return new int[] { 0, 0, 0 };
+  	}
 		Color color = bufferedTableItem.getForeground();
 
 		if (color == null) {
@@ -272,19 +319,42 @@ public class TableCellImpl
   	if (isInvisibleAndCanRefresh())
   		return false;
 
-  	bCellVisuallyChangedSinceRefresh = true;
-    return bufferedTableItem.setForeground(color);
+    boolean set = bufferedTableItem.setForeground(color);
+    if (set) {
+    	setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
+    }
+    return set;
   }
-  
-  public boolean setForeground(int red, int green, int blue) {
-  	checkCellForSetting();
 
-  	// Don't need to set when not visible
-  	if (isInvisibleAndCanRefresh())
-  		return false;
+	public boolean setForeground(int red, int green, int blue) {
+		checkCellForSetting();
 
-  	bCellVisuallyChangedSinceRefresh = true;
-    return bufferedTableItem.setForeground(red, green, blue);
+		// Don't need to set when not visible
+		if (isInvisibleAndCanRefresh())
+			return false;
+
+		boolean set;
+		if (red < 0 || green < 0 || blue < 0) {
+			set = bufferedTableItem.setForeground(null);
+		} else {
+			set = bufferedTableItem.setForeground(red, green, blue);
+		}
+		if (set) {
+			setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
+		}
+		return set;
+	}
+
+	// @see org.gudy.azureus2.plugins.ui.tables.TableCell#setForeground(int[])
+	public boolean setForeground(int[] rgb) {
+		if (rgb == null || rgb.length < 3) {
+			return setForeground((Color) null);
+		}
+		return setForeground(rgb[0], rgb[1], rgb[2]);
+	}
+
+  public boolean setForegroundToErrorColor() {
+	  return setForeground(Colors.colorError);
   }
 
   public boolean setText(String text) {
@@ -293,7 +363,7 @@ public class TableCellImpl
   		text = "";
   	boolean bChanged = false;
 
-  	if (bSortValueIsText && !text.equals(sortValue)) {
+  	if (hasFlag(FLAG_SORTVALUEISTEXT) && !text.equals(sortValue)) {
   		bChanged = true;
   		sortValue = text;
     	tableColumn.setLastSortValueChange(SystemTime.getCurrentTime());
@@ -310,7 +380,7 @@ public class TableCellImpl
 //  		return false;
 //  	}
 
-    if (bufferedTableItem.setText(text) && !bSortValueIsText)
+    if (bufferedTableItem.setText(text) && !hasFlag(FLAG_SORTVALUEISTEXT))
     	bChanged = true;
 
 		if (bDebug) {
@@ -318,8 +388,25 @@ public class TableCellImpl
 		}
 
 		if (bChanged) {
-			bCellVisuallyChangedSinceRefresh = true;
+			setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
 		}
+
+		boolean do_auto = this.tableColumn.doesAutoTooltip();
+		
+		// If we were using auto tooltips (and we aren't any more), then
+		// clear up previously set tooltips.
+		if (!do_auto) {
+			if (hasFlag(FLAG_TOOLTIPISAUTO)) {
+				this.oToolTip = null;
+				clearFlag(FLAG_TOOLTIPISAUTO);
+			}
+		}
+		
+		else {
+			this.oToolTip = text;
+			setFlag(FLAG_TOOLTIPISAUTO);
+		}
+		
   	return bChanged;
   }
   
@@ -329,19 +416,28 @@ public class TableCellImpl
   }
   
   public String getText() {
-  	if (bSortValueIsText && sortValue instanceof String)
+  	if (hasFlag(FLAG_SORTVALUEISTEXT) && sortValue instanceof String)
   		return (String)sortValue;
+  	if (bufferedTableItem == null) {
+  		return null;
+  	}
     return bufferedTableItem.getText();
   }
 
   public boolean isShown() {
-    return bufferedTableItem.isShown();
+  	if (bufferedTableItem == null) {
+  		return false;
+  	}
+
+    return bufferedTableItem.isShown()
+				&& tableRow.getView().isColumnVisible(tableColumn);
   }
   
   public boolean setSortValue(Comparable valueToSort) {
 		if (!tableColumn.isSortValueLive()) {
 			// objects that can't change aren't live
-			if (!(valueToSort instanceof Number) && !(valueToSort instanceof String)) {
+			if (!(valueToSort instanceof Number) && !(valueToSort instanceof String)
+					&& !(valueToSort instanceof TableColumnSortObject)) {
 				tableColumn.setSortValueLive(true);
 			}
 		}
@@ -354,8 +450,8 @@ public class TableCellImpl
     if (sortValue == valueToSort)
       return false;
 
-    if (bSortValueIsText) {
-      bSortValueIsText = false;
+    if (hasFlag(FLAG_SORTVALUEISTEXT)) {
+    	clearFlag(FLAG_SORTVALUEISTEXT);
     	if (sortValue instanceof String)
 	    	// Make sure text is actually in the cell (it may not have been if
 	      // cell wasn't created at the time of setting)
@@ -367,6 +463,11 @@ public class TableCellImpl
 			return false;
 		}
 
+    if ((valueToSort instanceof Number) && (sortValue instanceof Number)
+				&& sortValue.equals(valueToSort)) {
+			return false;
+		}
+
   	if (bDebug)
   		debug("Setting SortValue to "
 					+ ((valueToSort == null) ? "null" : valueToSort.getClass().getName()));
@@ -413,6 +514,7 @@ public class TableCellImpl
   
   public void setToolTip(Object tooltip) {
     oToolTip = tooltip;
+    clearFlag(FLAG_TOOLTIPISAUTO);
   }
 
   public Object getToolTip() {
@@ -420,7 +522,16 @@ public class TableCellImpl
   }
 
 	public boolean isDisposed() {
-		return bDisposed;
+		return hasFlag(FLAG_DISPOSED);
+	}
+	
+	// @see org.gudy.azureus2.plugins.ui.tables.TableCell#getMaxLines()
+	public int getMaxLines() {
+		if (bufferedTableItem == null) {
+			// use 1 in case some plugin borks on div by zero
+			return 1;
+		}
+		return bufferedTableItem.getMaxLines();
 	}
   
   /* Start TYPE_GRAPHIC Functions */
@@ -472,7 +583,7 @@ public class TableCellImpl
     graphic = null;
     boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(img);
     if (b) {
-      bCellVisuallyChangedSinceRefresh = true;
+    	setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
     }
     return b;
   }
@@ -485,12 +596,17 @@ public class TableCellImpl
     if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
       return false;
 
+    if (img == graphic && numFastRefreshes >= MAX_REFRESHES) {
+    	pluginError("TableCellImpl::setGraphic to same Graphic object. "
+					+ "Forcing refresh.");
+    }
+    
     graphic = img;
 
     if (img == null) {
       boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(null);
       if (b) {
-        bCellVisuallyChangedSinceRefresh = true;
+      	setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
       }
     }
 
@@ -498,7 +614,7 @@ public class TableCellImpl
     	Image imgSWT = ((GraphicSWT)img).getImage();
     	boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(imgSWT);
       if (b) {
-        bCellVisuallyChangedSinceRefresh = true;
+      	setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
       }
     }
     
@@ -506,7 +622,7 @@ public class TableCellImpl
     	Image imgSWT = ((UISWTGraphic)img).getImage();
     	boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(imgSWT);
       if (b) {
-        bCellVisuallyChangedSinceRefresh = true;
+      	setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
       }
     }
     
@@ -540,7 +656,7 @@ public class TableCellImpl
     	((BufferedGraphicTableItem)bufferedTableItem).setOrientation(SWT.FILL);
     else
     	setOrientationViaColumn();
-    bCellVisuallyChangedSinceRefresh = true;
+    setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
   }
 
 	public void setMarginHeight(int height) {
@@ -549,7 +665,7 @@ public class TableCellImpl
     if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
       return;
     ((BufferedGraphicTableItem)bufferedTableItem).setMargin(-1, height);
-    bCellVisuallyChangedSinceRefresh = true;
+    setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
   }
 
   public void setMarginWidth(int width) {
@@ -558,7 +674,19 @@ public class TableCellImpl
     if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
       return;
     ((BufferedGraphicTableItem)bufferedTableItem).setMargin(width, -1);
-    bCellVisuallyChangedSinceRefresh = true;
+    setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
+  }
+
+	public int getMarginHeight() {
+    if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
+      return 0;
+    return ((BufferedGraphicTableItem)bufferedTableItem).getMarginHeight();
+  }
+
+  public int getMarginWidth() {
+    if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
+      return 0;
+    return ((BufferedGraphicTableItem)bufferedTableItem).getMarginWidth();
   }
 
   /* End TYPE_GRAPHIC Functions */
@@ -680,6 +808,33 @@ public class TableCellImpl
 		}
 	}
 
+	public void addMouseMoveListener(TableCellMouseMoveListener listener) {
+		try {
+			this_mon.enter();
+
+			if (cellMouseMoveListeners == null)
+				cellMouseMoveListeners = new ArrayList(1);
+
+			cellMouseMoveListeners.add(listener);
+			
+		} finally {
+			this_mon.exit();
+		}
+	}
+
+	public void removeMouseMoveListener(TableCellMouseMoveListener listener) {
+		try {
+			this_mon.enter();
+
+			if (cellMouseMoveListeners == null)
+				return;
+
+			cellMouseMoveListeners.remove(listener);
+
+		} finally {
+			this_mon.exit();
+		}
+	}
 
 	public void addVisibilityListener(TableCellVisibilityListener listener) {
 		try {
@@ -710,8 +865,9 @@ public class TableCellImpl
 	}
 
 	public void addListeners(Object listenerObject) {
-		if (listenerObject instanceof TableCellDisposeListener)
+		if (listenerObject instanceof TableCellDisposeListener) {
 			addDisposeListener((TableCellDisposeListener)listenerObject);
+		}
 
 		if (listenerObject instanceof TableCellRefreshListener)
 			addRefreshListener((TableCellRefreshListener)listenerObject);
@@ -719,8 +875,13 @@ public class TableCellImpl
 		if (listenerObject instanceof TableCellToolTipListener)
 			addToolTipListener((TableCellToolTipListener)listenerObject);
 
-		if (listenerObject instanceof TableCellMouseListener)
-			addMouseListener((TableCellMouseListener)listenerObject);
+		if (listenerObject instanceof TableCellMouseMoveListener) {
+			addMouseMoveListener((TableCellMouseMoveListener) listenerObject);
+		}
+
+		if (listenerObject instanceof TableCellMouseListener) {
+			addMouseListener((TableCellMouseListener) listenerObject);
+		}
 
 		if (listenerObject instanceof TableCellVisibilityListener)
 			addVisibilityListener((TableCellVisibilityListener)listenerObject);
@@ -740,133 +901,221 @@ public class TableCellImpl
   //////////////////////////////////
 	
   public void invalidate(final boolean bMustRefresh) {
-  	valid = false;
+  	if (!hasFlag(FLAG_VALID)) {
+  		if (bMustRefresh) {
+  			if (hasFlag(FLAG_MUSTREFRESH)) {
+  				return;
+  			}
+  		} else {
+  			return;
+  		}
+  	}
+  	clearFlag(FLAG_VALID);
+  	
+  	setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
 
   	if (bDebug)
   		debug("Invalidate Cell;" + bMustRefresh);
 
-  	if (bMustRefresh)
-  		this.bMustRefresh = true;
+  	if (bMustRefresh) {
+  		setFlag(FLAG_MUSTREFRESH);
+  		if (bufferedTableItem != null) {
+  			bufferedTableItem.invalidate();
+  		}
+  	}
   }
-
+  
+  // @see com.aelitis.azureus.ui.common.table.TableCellCore#refresh()
   public boolean refresh() {
     return refresh(true);
   }
   
-  public boolean refresh(boolean bDoGraphics) {
-  	return refresh(bDoGraphics, isShown());
+  // @see com.aelitis.azureus.ui.common.table.TableCellCore#refreshAsync()
+  public void refreshAsync() {
+  	if (bInRefreshAsync) {
+  		//System.out.println(System.currentTimeMillis() + "] SKIP " + restartRefresh);
+  		if (restartRefresh < Byte.MAX_VALUE) {
+  			restartRefresh++;
+  		}
+  		return;
+  	}
+  	bInRefreshAsync = true;
+
+		AERunnable runnable = new AERunnable() {
+			public void runSupport() {
+				//System.out.println(System.currentTimeMillis() + "] REFRESH!");
+				restartRefresh = 0;
+				refresh(true);
+				bInRefreshAsync = false;
+				//System.out.println(System.currentTimeMillis() + "] REFRESH OUT!");
+				if (restartRefresh > 0) {
+					refreshAsync();
+				}
+			}
+		};
+		Utils.execSWTThreadLater(25, runnable);
   }
-
-  public boolean refresh(boolean bDoGraphics, boolean bRowVisible) {
-  	return refresh(bDoGraphics, bRowVisible, isShown());
-  }
-
-  private boolean bInRefresh = false;
-  public boolean refresh(boolean bDoGraphics, boolean bRowVisible,
-			boolean bCellVisible)
-	{
-  	boolean ret = bCellVisuallyChangedSinceRefresh;
-
-  	int iErrCount = 0;
-  	try {
-      if (refreshErrLoopCount > 2) {
-        return bCellVisuallyChangedSinceRefresh;
-      }
-
-      iErrCount = tableColumn.getConsecutiveErrCount();
-      if (iErrCount > 10) {
-      	refreshErrLoopCount = 3;
-        return bCellVisuallyChangedSinceRefresh;
-      }
-      
-      if (bInRefresh) {
-      	// Skip a Refresh call when being called from within refresh.
-      	// This could happen on virtual tables where SetData calls us again, or
-      	// if we ever introduce plugins to refresh.
-      	if (bDebug)
-      		debug("Calling Refresh from Refresh :) Skipping.");
-      	return bCellVisuallyChangedSinceRefresh;
-      }
-    	bInRefresh = true;
   
-      // See bIsUpToDate variable comments
-      if (bCellVisible && !bIsUpToDate) {
-      	if (bDebug)
-      		debug("Setting Invalid because visible & not up to date");
-      	valid = false;
-      	bIsUpToDate = true;
-      } else if (!bCellVisible && bIsUpToDate) {
-      	bIsUpToDate = false;
-      }
+  // @see com.aelitis.azureus.ui.common.table.TableCellCore#refresh(boolean)
+  public boolean refresh(boolean bDoGraphics) {
+  	boolean isRowShown;
+  	if (tableRow != null) {
+  		TableView view = tableRow.getView();
+  		isRowShown = view.isRowVisible(tableRow);
+  	} else {
+  		isRowShown = true;
+  	}
+		boolean isCellShown = isRowShown && isShown();
+		return refresh(bDoGraphics, isRowShown, isCellShown);
+  }
 
-    	if (bDebug) {
-    		debug("Cell Valid?" + valid + "; Visible?" + tableRow.isVisible() + "/" + bufferedTableItem.isShown());
-    	}
-      int iInterval = tableColumn.getRefreshInterval();
-    	if (iInterval == TableColumnCore.INTERVAL_INVALID_ONLY && !valid
-    			&& !bMustRefresh && bSortValueIsText && sortValue != null
-					&& tableColumn.getType() == TableColumnCore.TYPE_TEXT_ONLY) {
-    		if (bCellVisible) {
-	      	if (bDebug)
-	      		debug("fast refresh: setText");
-	    		setText((String)sortValue);
-	    		valid = true;
-    		}
-    	} else if ((iInterval == TableColumnCore.INTERVAL_LIVE ||
-          (iInterval == TableColumnCore.INTERVAL_GRAPHIC && bDoGraphics) ||
-          (iInterval > 0 && (loopFactor % iInterval) == 0) ||
-          !valid || bMustRefresh)) 
-      {
-      	boolean bWasValid = isValid();
+  // @see com.aelitis.azureus.ui.common.table.TableCellCore#refresh(boolean, boolean)
+  public boolean refresh(boolean bDoGraphics, boolean bRowVisible) {
+		boolean isCellShown = bRowVisible && isShown();
+		return refresh(bDoGraphics, bRowVisible, isCellShown);
+  }
 
-      	if (bDebug)
-      		debug("invoke refresh");
+  // @see com.aelitis.azureus.ui.common.table.TableCellCore#refresh(boolean, boolean, boolean)
+  public boolean refresh(boolean bDoGraphics, boolean bRowVisible,  boolean bCellVisible)
+  {
+	  boolean ret = getVisuallyChangedSinceRefresh();
+  	clearFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
 
-      	long lTimeStart = SystemTime.getCurrentTime();
-        tableColumn.invokeCellRefreshListeners(this);
-        if (refreshListeners != null) {
-          for (int i = 0; i < refreshListeners.size(); i++) {
-            ((TableCellRefreshListener)(refreshListeners.get(i))).refresh(this);
-          }
-        }
-      	long lTimeEnd = SystemTime.getCurrentTime();
-      	tableColumn.addRefreshTime(lTimeEnd - lTimeStart);
+	  int iErrCount = 0;
+	  if (refreshErrLoopCount > 2) {
+		  return ret;
+	  }
 
-        // Change to valid only if we weren't valid before the listener calls
-        // This is in case the listeners set valid to false when it was true
-        if (!bWasValid) 
-        	valid = true;
-        
-        if (bMustRefresh)
-        	bMustRefresh = false;
-      }
-      loopFactor++;
-      refreshErrLoopCount = 0;
-      if (iErrCount > 0)
-        tableColumn.setConsecutiveErrCount(0);
-      
-    	ret = bCellVisuallyChangedSinceRefresh;
-    } catch (Throwable e) {
-      refreshErrLoopCount++;
-      tableColumn.setConsecutiveErrCount(++iErrCount);
-      pluginError(e);
-      if (refreshErrLoopCount > 2)
-      	Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR,
-						"TableCell will not be refreshed anymore this session."));
-    } finally {
-    	bCellVisuallyChangedSinceRefresh = false;
-    	bInRefresh = false;
+	  iErrCount = tableColumn.getConsecutiveErrCount();
+	  if (iErrCount > 10) {
+		  refreshErrLoopCount = 3;
+		  return ret;
+	  }
+
+	  if (bInRefresh) {
+		  // Skip a Refresh call when being called from within refresh.
+		  // This could happen on virtual tables where SetData calls us again, or
+		  // if we ever introduce plugins to refresh.
+		  if (bDebug)
+			  debug("Calling Refresh from Refresh :) Skipping.");
+		  return ret;
+	  }
+	  try {
+		  bInRefresh = true;
+		  if (ret) {
+  		  long now = SystemTime.getCurrentTime();
+  		  if (now - lastRefresh < MAX_REFRESHES_WITHIN_MS) {
+  		  	numFastRefreshes++;
+  		  	if (numFastRefreshes >= MAX_REFRESHES) {
+  		  		if ((numFastRefreshes % MAX_REFRESHES) == 0) {
+    			  	pluginError("this plugin is crazy. tried to refresh "
+									+ numFastRefreshes + " times in " + (now - lastRefresh)
+									+ "ms");
+  		  		}
+  		  		return ret;
+  		  	}
+  		  } else {
+  		  	numFastRefreshes = 0;
+  			  lastRefresh = now;
+  		  }
+		  }
+
+		  // See bIsUpToDate variable comments
+		  if (bCellVisible && !isUpToDate()) {
+			  if (bDebug)
+				  debug("Setting Invalid because visible & not up to date");
+			  clearFlag(FLAG_VALID);
+			  setFlag(FLAG_UPTODATE);
+		  } else if (!bCellVisible && isUpToDate()) {
+			  if (bDebug)
+				  debug("Setting not up to date because cell not visible " + Debug.getCompressedStackTrace());
+		  	clearFlag(FLAG_UPTODATE);
+		  }
+		  
+		  if (bDebug) {
+			  debug("Cell Valid?" + hasFlag(FLAG_VALID) + "; Visible?" + tableRow.isVisible() + "/" + bufferedTableItem.isShown());
+		  }
+		  int iInterval = tableColumn.getRefreshInterval();
+		  if (iInterval == TableColumnCore.INTERVAL_INVALID_ONLY 
+		  	&& !hasFlag(FLAG_MUSTREFRESH | FLAG_VALID) && hasFlag(FLAG_SORTVALUEISTEXT) && sortValue != null
+			  && tableColumn.getType() == TableColumnCore.TYPE_TEXT_ONLY) {
+			  if (bCellVisible) {
+				  if (bDebug)
+					  debug("fast refresh: setText");
+				  ret = setText((String)sortValue);
+				  setFlag(FLAG_VALID);
+			  }
+		  } else if ((iInterval == TableColumnCore.INTERVAL_LIVE ||
+			  (iInterval == TableColumnCore.INTERVAL_GRAPHIC && bDoGraphics) ||
+			  (iInterval > 0 && (loopFactor % iInterval) == 0) ||
+			  !hasFlag(FLAG_VALID) || hasFlag(FLAG_MUSTREFRESH))) 
+		  {
+			  boolean bWasValid = isValid();
+
+			  if (bDebug)
+				  debug("invoke refresh; wasValid? " + bWasValid);
+
+			  long lTimeStart = SystemTime.getMonotonousTime();
+			  tableColumn.invokeCellRefreshListeners(this, !bCellVisible);
+			  if (refreshListeners != null) {
+				  for (int i = 0; i < refreshListeners.size(); i++) {
+					  TableCellRefreshListener l = (TableCellRefreshListener)refreshListeners.get(i);
+					  if(l instanceof TableCellLightRefreshListener)
+						  ((TableCellLightRefreshListener)l).refresh(this,!bCellVisible);
+					  else
+						  l.refresh(this);
+				  }
+			  }
+			  long lTimeEnd = SystemTime.getMonotonousTime();
+			  tableColumn.addRefreshTime(lTimeEnd - lTimeStart);
+
+			  // Change to valid only if we weren't valid before the listener calls
+			  // This is in case the listeners set valid to false when it was true
+			  if (!bWasValid) 
+			  	setFlag(FLAG_VALID);
+
+			  if (hasFlag(FLAG_MUSTREFRESH)) {
+			  	clearFlag(FLAG_MUSTREFRESH);
+			  }
+		  }
+		  loopFactor++;
+		  refreshErrLoopCount = 0;
+		  if (iErrCount > 0)
+			  tableColumn.setConsecutiveErrCount(0);
+
+		  ret = getVisuallyChangedSinceRefresh();
+		  if (bDebug)
+			  debug("refresh done; visual change? " + ret + ";" + Debug.getCompressedStackTrace());
+	  } catch (Throwable e) {
+		  refreshErrLoopCount++;
+		  if (tableColumn != null) {
+			  tableColumn.setConsecutiveErrCount(++iErrCount);
+		  }
+		  pluginError(e);
+		  if (refreshErrLoopCount > 2)
+			  Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR,
+				  "TableCell will not be refreshed anymore this session."));
+	  } finally {
+		  bInRefresh = false;
+	  }
+
+    if (childCells != null) {
+    	Object[] childCellsArray = childCells.toArray();
+    	for (int i = 0; i < childCellsArray.length; i++) {
+				TableCellImpl childCell = (TableCellImpl) childCellsArray[i];
+				childCell.refresh(bDoGraphics, bRowVisible, bCellVisible);
+			}
     }
-  	return ret;
+	  return ret;
   }
   
   public boolean getVisuallyChangedSinceRefresh() {
-  	return bCellVisuallyChangedSinceRefresh;
+  	return hasFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
   }
 
 
   public void dispose() {
-  	bDisposed = true;
+  	setFlag(FLAG_DISPOSED);
 
     tableColumn.invokeCellDisposeListeners(this);
 
@@ -881,9 +1130,11 @@ public class TableCellImpl
         pluginError(e);
       }
     }
-
-    if (bufferedTableItem != null)
-      bufferedTableItem.dispose();
+    
+    if (bufferedTableItem != null) {
+			bufferedTableItem.setForeground(null);
+			bufferedTableItem.dispose();
+		}
     
     refreshListeners = null;
     bufferedTableItem = null;
@@ -892,46 +1143,73 @@ public class TableCellImpl
     sortValue = null;
   }
   
-  public void setIcon(Image img) {
+  public boolean setIcon(Image img) {
   	if (isInvisibleAndCanRefresh())
-  		return;
+  		return false;
 
     bufferedTableItem.setIcon(img);
     graphic = null;
-    bCellVisuallyChangedSinceRefresh = true;
+    setFlag(FLAG_VISUALLY_CHANGED_SINCE_REFRESH);
+    return true;
   }
   
   public Image getIcon() {
+  	if (bufferedTableItem == null) {
+  		return null;
+  	}
   	return bufferedTableItem.getIcon();
   }
 
   public boolean needsPainting() {
+  	if (bufferedTableItem == null) {
+  		return false;
+  	}
     return bufferedTableItem.needsPainting();
   }
   
   public void doPaint(GC gc) {
-  	if ((!bIsUpToDate || !valid)
-				&& (refreshListeners != null || tableColumn.hasCellRefreshListener())) {
-  		if (bDebug) {
-  			debug("doPaint: invoke refresh");
-  		}
-  		refresh();
-  	}
+  	//This sometimes causes a infinite loop if the listener invalidates
+  	//the drawing area
+  	//if ((!hasFlag(FLAG_UPTODATE) || !hasFlag(FLAG_VALID)) && !bInRefresh && !bInRefreshAsync
+		//		&& (refreshListeners != null || tableColumn.hasCellRefreshListener())) {
+  	//	if (bDebug) {
+  	//		debug("doPaint: invoke refresh");
+  	//	}
+  	//	refresh(true);
+  	//}
 
 		if (bDebug) {
-			debug("doPaint " + bIsUpToDate + ";" + valid + ";" + refreshListeners);
+			debug("doPaint up2date:" + hasFlag(FLAG_UPTODATE) + ";v:" + hasFlag(FLAG_VALID) + ";rl=" + refreshListeners);
 		}
     bufferedTableItem.doPaint(gc);
+    
+    if (childCells != null) {
+    	Object[] childCellsArray = childCells.toArray();
+    	for (int i = 0; i < childCellsArray.length; i++) {
+				TableCellImpl childCell = (TableCellImpl) childCellsArray[i];
+				childCell.doPaint(gc);
+			}
+    }
   }
 
   public void locationChanged() {
-    bufferedTableItem.locationChanged();
+  	if (bufferedTableItem != null) {
+  		bufferedTableItem.locationChanged();
+  	}
   }
 
   public TableRowCore getTableRowCore() {
     return tableRow;
   }
-  
+
+	// @see org.gudy.azureus2.ui.swt.views.table.TableCellSWT#getTableRowSWT()
+	public TableRowSWT getTableRowSWT() {
+		if (tableRow instanceof TableRowSWT) {
+			return (TableRowSWT)tableRow;
+		}
+		return null;
+	}
+
 	/* (non-Javadoc)
 	 * @see java.lang.Object#toString()
 	 */
@@ -1009,13 +1287,18 @@ public class TableCellImpl
   }
 
   public void invokeMouseListeners(TableCellMouseEvent event) {
-		if (cellMouseListeners == null)
+		ArrayList listeners = event.eventType == TableCellMouseEvent.EVENT_MOUSEMOVE
+				? cellMouseMoveListeners : cellMouseListeners;
+		if (listeners == null)
 			return;
-
-		for (int i = 0; i < cellMouseListeners.size(); i++) {
+		
+		if (event.cell != null && event.row == null) {
+			event.row = event.cell.getTableRow();
+		}
+		
+		for (int i = 0; i < listeners.size(); i++) {
 			try {
-				TableCellMouseListener l = (TableCellMouseListener) (cellMouseListeners
-						.get(i));
+				TableCellMouseListener l = (TableCellMouseListener) (listeners.get(i));
 
 				l.cellMouseTrigger(event);
 
@@ -1025,7 +1308,12 @@ public class TableCellImpl
 		}
 	}
 
-  public void invokeVisibilityListeners(int visibility) {
+	public void invokeVisibilityListeners(int visibility,
+			boolean invokeColumnListeners) {
+		if (invokeColumnListeners) {
+			tableColumn.invokeCellVisibilityListeners(this, visibility);
+		}
+
 		if (cellVisibilityListeners == null)
 			return;
 
@@ -1050,28 +1338,36 @@ public class TableCellImpl
   }
   
 	public void setUpToDate(boolean upToDate) {
-		bIsUpToDate = upToDate;
+	  if (bDebug)
+		  debug("set up to date to " + upToDate);
+		if (upToDate) {
+			setFlag(FLAG_UPTODATE);
+		} else {
+			clearFlag(FLAG_UPTODATE);
+		}
 	}
 	
 	public boolean isUpToDate() {
-		return bIsUpToDate;
+		return hasFlag(FLAG_UPTODATE);
 	}
 	
-	private void debug(final String s) {
+	public void debug(final String s) {
 		Utils.execSWTThread(new AERunnable() {
 			public void runSupport() {
 				System.out.println(SystemTime.getCurrentTime() + ": r"
-						+ tableRow.getIndex() + "c" + tableColumn.getPosition()
-						+ "r.v?" + ((tableRow.isVisible() ? "Y":"N"))
-						+ ";" + s);
+						+ tableRow.getIndex() + "c" + tableColumn.getPosition() + "r.v?"
+						+ ((tableRow.isVisible() ? "Y" : "N")) + ";" + s);
 			}
 		}, true);
 	}
 
 	public Rectangle getBounds() {
+		if (isDisposed()) {
+			return new Rectangle(0, 0, 0, 0);
+		}
 		Rectangle bounds = bufferedTableItem.getBounds();
 		if (bounds == null) {
-      return new Rectangle(0,0,0,0);
+			return new Rectangle(0, 0, 0, 0);
 		}
     return bounds;
 	}
@@ -1080,15 +1376,9 @@ public class TableCellImpl
 		if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
 			return;
 		
-		BufferedGraphicTableItem ti = (BufferedGraphicTableItem) bufferedTableItem;
-
 		int align = tableColumn.getAlignment();
-    if (align == TableColumn.ALIGN_CENTER)
-    	ti.setOrientation(SWT.CENTER); 
-    else if (align == TableColumn.ALIGN_LEAD)
-    	ti.setOrientation(SWT.LEFT); 
-    else if (align == TableColumn.ALIGN_TRAIL)
-    	ti.setOrientation(SWT.RIGHT); 
+		BufferedGraphicTableItem ti = (BufferedGraphicTableItem) bufferedTableItem;
+		ti.setOrientation(TableColumnSWTUtils.convertColumnAlignmentToSWT(align));
 	}
 
 	public String getObfusticatedText() {
@@ -1102,7 +1392,17 @@ public class TableCellImpl
 		return null;
 	}
 
+	public Graphic getBackgroundGraphic() {
+		if (bufferedTableItem == null) {
+			return null;
+		}
+  	return new UISWTGraphicImpl(bufferedTableItem.getBackgroundImage());
+	}
+
 	public Image getBackgroundImage() {
+		if (bufferedTableItem == null) {
+			return null;
+		}
   	return bufferedTableItem.getBackgroundImage();
 	}
 	
@@ -1116,5 +1416,49 @@ public class TableCellImpl
 	
 	public void setCursorID(int cursorID) {
 		iCursorID = cursorID;
+		if (isMouseOver()) {
+			Utils.execSWTThread(new AERunnable() {
+				public void runSupport() {
+					bufferedTableItem.setCursor(iCursorID);
+				}
+			});
+		}
+	}
+	
+	public boolean isMouseOver() {
+		if (bufferedTableItem == null) {
+			return false;
+		}
+		return bufferedTableItem.isMouseOver();
+	}
+	
+	public int[] getMouseOffset() {
+		Point ofs = ((TableViewSWT) tableRow.getView()).getTableCellMouseOffset(this);
+		return ofs == null ? null : new int[] { ofs.x, ofs.y };
+	}
+	
+	private boolean hasFlag(int flag) {
+		return (flags & flag) != 0;
+	}
+	
+	private void setFlag(int flag) {
+		flags |= flag;
+	}
+	
+	private void clearFlag(int flag) {
+		flags &= ~flag;
+	}
+
+	/**
+	 * @param childCell
+	 *
+	 * @since 3.0.5.3
+	 */
+	public void addChildCell(TableCellImpl childCell) {
+		if (childCells == null) {
+			childCells = new ArrayList(1);
+		}
+		//TODO: childCell.setParentCell(this);
+		childCells.add(childCell);
 	}
 }
