@@ -22,34 +22,35 @@
  */
 package org.gudy.azureus2.ui.swt.views.table.impl;
 
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.Map;
+import java.util.*;
 
 import org.eclipse.swt.SWTException;
 import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.widgets.Table;
+
 import org.gudy.azureus2.core3.disk.DiskManagerFileInfo;
 import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.peer.PEPeer;
 import org.gudy.azureus2.core3.peer.PEPiece;
 import org.gudy.azureus2.core3.tracker.host.TRHostTorrent;
-import org.gudy.azureus2.core3.util.AEMonitor;
-import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.ui.swt.Utils;
+import org.gudy.azureus2.ui.swt.components.BufferedTableRow;
+import org.gudy.azureus2.ui.swt.mainwindow.Colors;
+import org.gudy.azureus2.ui.swt.views.table.TableCellSWT;
+import org.gudy.azureus2.ui.swt.views.table.TableRowSWT;
+import org.gudy.azureus2.ui.swt.views.table.TableViewSWT;
+
+import com.aelitis.azureus.ui.common.table.*;
 
 import org.gudy.azureus2.plugins.download.DownloadException;
-import org.gudy.azureus2.plugins.ui.tables.TableCell;
-import org.gudy.azureus2.plugins.ui.tables.TableManager;
+import org.gudy.azureus2.plugins.ui.tables.*;
+
 import org.gudy.azureus2.pluginsimpl.local.disk.DiskManagerFileInfoImpl;
 import org.gudy.azureus2.pluginsimpl.local.download.DownloadManagerImpl;
 import org.gudy.azureus2.pluginsimpl.local.peers.PeerManagerImpl;
 import org.gudy.azureus2.pluginsimpl.local.tracker.TrackerTorrentImpl;
-import org.gudy.azureus2.ui.swt.components.BufferedTableRow;
-import org.gudy.azureus2.ui.swt.views.TableView;
-import org.gudy.azureus2.ui.swt.views.table.TableCellCore;
-import org.gudy.azureus2.ui.swt.views.table.TableColumnCore;
-import org.gudy.azureus2.ui.swt.views.table.TableRowCore;
 
 
 
@@ -65,7 +66,7 @@ import org.gudy.azureus2.ui.swt.views.table.TableRowCore;
  */
 public class TableRowImpl 
        extends BufferedTableRow 
-       implements TableRowCore
+       implements TableRowSWT
 {
   /** List of cells in this column.  They are not stored in display order */
   private Map mTableCells;
@@ -75,9 +76,15 @@ public class TableRowImpl
   private Object pluginDataSource;
   private boolean bDisposed;
   private boolean bSetNotUpToDateLastRefresh = false;
+	private TableView tableView;
+
+  private static AEMonitor this_mon = new AEMonitor( "TableRowImpl" );
+	private ArrayList mouseListeners;
+	private boolean wasShown = false;
+	private Map dataList;
+	
+	private int lastIndex = -1;
 
-  private static AEMonitor sortedDisposal_mon = new AEMonitor( "TableRowImpl" );
-  
   // XXX add rowVisuallyupdated bool like in ListRow
 
   /**
@@ -89,13 +96,14 @@ public class TableRowImpl
    * @param dataSource
    * @param bSkipFirstColumn
    */
-  public TableRowImpl(Table table, String sTableID,
+  public TableRowImpl(TableView tv, Table table, String sTableID,
 			TableColumnCore[] columnsSorted, Object dataSource,
 			boolean bSkipFirstColumn) {
 		super(table);
+		this.tableView = tv;
     this.sTableID = sTableID;
     coreDataSource = dataSource;
-    mTableCells = new HashMap();
+    mTableCells = new LightHashMap();
     bDisposed = false;
 
     // create all the cells for the column
@@ -117,7 +125,7 @@ public class TableRowImpl
     boolean valid = true;
     Iterator iter = mTableCells.values().iterator();
     while (iter.hasNext()) {
-      TableCellCore cell = (TableCellCore)iter.next();
+    	TableCellSWT cell = (TableCellSWT)iter.next();
       if (cell != null)
         valid &= cell.isValid();
     }
@@ -144,74 +152,123 @@ public class TableRowImpl
   		return null;
     return (TableCell)mTableCells.get(field);
   }
-  
+
+	public void addMouseListener(TableRowMouseListener listener) {
+		try {
+			this_mon.enter();
+
+			if (mouseListeners == null)
+				mouseListeners = new ArrayList(1);
+
+			mouseListeners.add(listener);
+
+		} finally {
+			this_mon.exit();
+		}
+	}
+
+	public void removeMouseListener(TableRowMouseListener listener) {
+		try {
+			this_mon.enter();
+
+			if (mouseListeners == null)
+				return;
+
+			mouseListeners.remove(listener);
+
+		} finally {
+			this_mon.exit();
+		}
+	}
+
+  public void invokeMouseListeners(TableRowMouseEvent event) {
+		ArrayList listeners = mouseListeners;
+		if (listeners == null)
+			return;
+		
+		for (int i = 0; i < listeners.size(); i++) {
+			try {
+				TableRowMouseListener l = (TableRowMouseListener) (listeners.get(i));
+
+				l.rowMouseTrigger(event);
+
+			} catch (Throwable e) {
+				Debug.printStackTrace(e);
+			}
+		}
+	}
+
   /* Start Core-Only functions */
   ///////////////////////////////
 
-  public void delete(boolean bDeleteSWTObject) {
-		sortedDisposal_mon.enter();
+  public void delete() {
+		this_mon.enter();
 
 		try {
 			if (bDisposed)
 				return;
 
-			if (TableView.DEBUGADDREMOVE)
+			if (TableViewSWT.DEBUGADDREMOVE)
 				System.out.println((table.isDisposed() ? "" : table.getData("Name"))
 						+ " row delete; index=" + getIndex());
 
-			try {
-				// Manually dispose of TableCellImpl objects, since row does
-				// not contain a list of them.
-				Iterator iter = mTableCells.values().iterator();
-				while (iter.hasNext()) {
-					TableCellCore item = (TableCellCore) iter.next();
-					item.dispose();
-				}
-			} finally {
-				// Dispose of row
-				if (bDeleteSWTObject) {
-					dispose();
-				}
+			Iterator iter = mTableCells.values().iterator();
+			while (iter.hasNext()) {
+				TableCellSWT item = (TableCellSWT) iter.next();
+				item.dispose();
 			}
+			
+			setForeground((Color) null);
 
 			bDisposed = true;
 		} finally {
-			sortedDisposal_mon.exit();
+			this_mon.exit();
 		}
 	}
   
-  public boolean refresh(boolean bDoGraphics) {
-    if (bDisposed)
-      return false;
+  public List refresh(boolean bDoGraphics) {
+    if (bDisposed) {
+      return Collections.EMPTY_LIST;
+    }
     
     boolean bVisible = isVisible();
 
     return refresh(bDoGraphics, bVisible);
   }
 
-  public boolean refresh(boolean bDoGraphics, boolean bVisible) {
+  public List refresh(boolean bDoGraphics, boolean bVisible) {
     // If this were called from a plugin, we'd have to refresh the sorted column
     // even if we weren't visible
-    
+    ArrayList list = new ArrayList();
+
+  	if (bDisposed) {
+  		return list;
+  	}
+
     if (!bVisible) {
     	if (!bSetNotUpToDateLastRefresh) {
     		setUpToDate(false);
     		bSetNotUpToDateLastRefresh = true;
     	}
-  		return false;
+  		return list;
   	}
     
 		bSetNotUpToDateLastRefresh = false;
 		
 		//System.out.println(SystemTime.getCurrentTime() + "refresh " + getIndex());
 
-		boolean changed = false;
     Iterator iter = mTableCells.values().iterator();
-    while(iter.hasNext()) {
-      TableCellCore item = (TableCellCore)iter.next();
-      changed |= item.refresh(bDoGraphics, bVisible);
-    }
-    return changed;
+    while (iter.hasNext())
+		{
+			TableCellSWT item = (TableCellSWT) iter.next();
+			TableColumn column = item.getTableColumn();
+			if (column != tableView.getSortColumn() && !tableView.isColumnVisible(column))
+				continue;
+			boolean changed = item.refresh(bDoGraphics, bVisible);
+			if (changed)
+				list.add(item);
+		}
+    return list;
   }
 
   public void setAlternatingBGColor(boolean bEvenIfNotVisible) {
@@ -224,7 +281,7 @@ public class TableRowImpl
 
   	Iterator iter = mTableCells.values().iterator();
   	while(iter.hasNext()) {
-  		TableCellCore item = (TableCellCore)iter.next();
+  		TableCellSWT item = (TableCellSWT)iter.next();
   		if (item.getTableColumn().getPosition() > iStartColumn)
   		  item.locationChanged();
   	}
@@ -234,15 +291,22 @@ public class TableRowImpl
   	doPaint(gc, isVisible());
   }
 
+  // @see org.gudy.azureus2.ui.swt.views.table.TableRowSWT#doPaint(org.eclipse.swt.graphics.GC, boolean, boolean)
   public void doPaint(GC gc, boolean bVisible) {
     if (bDisposed || !bVisible)
       return;
 
     Iterator iter = mTableCells.values().iterator();
     while(iter.hasNext()) {
-      TableCellCore item = (TableCellCore) iter.next();
-  		if (item.needsPainting()) {
-  			item.doPaint(gc);
+    	TableCellSWT cell = (TableCellSWT) iter.next();
+    	if (cell == null) {
+    		continue;
+    	}
+//    	if (bOnlyIfChanged && !cell.getVisuallyChangedSinceRefresh()) {
+//    		continue;
+//    	}
+  		if (cell.needsPainting()) {
+  			cell.doPaint(gc);
   		}
     }
   }
@@ -254,62 +318,79 @@ public class TableRowImpl
     return (TableCellCore)mTableCells.get(field);
   }
 
-  public Object getDataSource(boolean bCoreObject) {
+	/**
+	 * @param name
+	 * @return
+	 */
+	public TableCellSWT getTableCellSWT(String name) {
   	if (bDisposed)
   		return null;
 
-    if (bCoreObject)
-      return coreDataSource;
-      
-    if (pluginDataSource != null)
-      return pluginDataSource;
+    return (TableCellSWT)mTableCells.get(name);
+	}
+  
+  public Object getDataSource(boolean bCoreObject) {
+		if (bDisposed)
+			return null;
 
-    if (sTableID.equals(TableManager.TABLE_MYTORRENTS_COMPLETE) ||
-        sTableID.equals(TableManager.TABLE_MYTORRENTS_INCOMPLETE)) {
-      DownloadManager dm = (DownloadManager)coreDataSource;
-      if (dm != null) {
-        try {
-          pluginDataSource = DownloadManagerImpl.getDownloadStatic(dm);
-        } catch (DownloadException e) { /* Ignore */ }
-      }
-    }
-    if (sTableID.equals(TableManager.TABLE_TORRENT_PEERS)) {
-      PEPeer peer = (PEPeer)coreDataSource;
-      if (peer != null)
-        pluginDataSource = PeerManagerImpl.getPeerForPEPeer( peer );
-    }
+		if (bCoreObject)
+			return coreDataSource;
 
-    if (sTableID.equals(TableManager.TABLE_TORRENT_PIECES)) {
-      // XXX There is no Piece object for plugins yet
-      PEPiece piece = (PEPiece)coreDataSource;
-      if (piece != null)
-        pluginDataSource = null;
-    }
+		if (pluginDataSource != null)
+			return pluginDataSource;
 
-    if (sTableID.equals(TableManager.TABLE_TORRENT_FILES)) {
-      DiskManagerFileInfo fileInfo = (DiskManagerFileInfo)coreDataSource;
-      if (fileInfo != null){
-    	  try {
-    		  pluginDataSource = 
-    			  new DiskManagerFileInfoImpl(
-    					  DownloadManagerImpl.getDownloadStatic(fileInfo.getDownloadManager()),
-    					  fileInfo);
-          } catch (DownloadException e) { /* Ignore */ }
-      }
-    }
+		if ((coreDataSource instanceof DownloadManager)
+				|| sTableID.equals(TableManager.TABLE_MYTORRENTS_COMPLETE)
+				|| sTableID.equals(TableManager.TABLE_MYTORRENTS_INCOMPLETE)) {
+			DownloadManager dm = (DownloadManager) coreDataSource;
+			if (dm != null) {
+				try {
+					pluginDataSource = DownloadManagerImpl.getDownloadStatic(dm);
+				} catch (DownloadException e) { /* Ignore */
+				}
+			}
+		}
+		if ((coreDataSource instanceof PEPeer)
+				|| sTableID.equals(TableManager.TABLE_TORRENT_PEERS)) {
+			PEPeer peer = (PEPeer) coreDataSource;
+			if (peer != null)
+				pluginDataSource = PeerManagerImpl.getPeerForPEPeer(peer);
+		}
 
-    if (sTableID.equals(TableManager.TABLE_MYSHARES)) {
-      pluginDataSource = coreDataSource;
-    }
+		if ((coreDataSource instanceof PEPiece)
+				|| sTableID.equals(TableManager.TABLE_TORRENT_PIECES)) {
+			// XXX There is no Piece object for plugins yet
+			PEPiece piece = (PEPiece) coreDataSource;
+			if (piece != null)
+				pluginDataSource = null;
+		}
 
-    if (sTableID.equals(TableManager.TABLE_MYTRACKER)) {
-      TRHostTorrent item = (TRHostTorrent)coreDataSource;
-      if (item != null)
-        pluginDataSource = new TrackerTorrentImpl(item);
-    }
-    
-    return pluginDataSource;
-  }
+		if ((coreDataSource instanceof DiskManagerFileInfo)
+				|| sTableID.equals(TableManager.TABLE_TORRENT_FILES)) {
+			DiskManagerFileInfo fileInfo = (DiskManagerFileInfo) coreDataSource;
+			if (fileInfo != null) {
+				try {
+					pluginDataSource = new DiskManagerFileInfoImpl(
+							DownloadManagerImpl.getDownloadStatic(fileInfo.getDownloadManager()),
+							fileInfo);
+				} catch (DownloadException e) { /* Ignore */
+				}
+			}
+		}
+
+		if (sTableID.equals(TableManager.TABLE_MYSHARES)) {
+			pluginDataSource = coreDataSource;
+		}
+
+		if ((coreDataSource instanceof TRHostTorrent)
+				|| sTableID.equals(TableManager.TABLE_MYTRACKER)) {
+			TRHostTorrent item = (TRHostTorrent) coreDataSource;
+			if (item != null)
+				pluginDataSource = new TrackerTorrentImpl(item);
+		}
+
+		return pluginDataSource;
+	}
   
 	public boolean isRowDisposed() {
 		return bDisposed;
@@ -322,24 +403,84 @@ public class TableRowImpl
 		if (bDisposed)
 			return -1;
 
+		if (lastIndex >= 0) {
+			TableRowCore row = ((TableViewSWTImpl) tableView).getRowQuick(lastIndex);
+			if (row == this) {
+				return lastIndex;
+			}
+		}
+
+		lastIndex = ((TableViewSWTImpl) tableView).indexOf(this);
+		return lastIndex;
+
+		//return super.getIndex();
+	}
+	
+	public int getRealIndex() {
 		return super.getIndex();
 	}
-
-	public boolean setTableItem(int newIndex) {
+	
+	public boolean setTableItem(int newIndex, boolean isVisible)
+	{
 		if (bDisposed) {
 			System.out.println("XXX setTI: bDisposed from " + Debug.getCompressedStackTrace());
 			return false;
 		}
+		
+		//if (getRealIndex() != newIndex) {
+		//	((TableViewSWTImpl)tableView).debug("sTI " + newIndex + "; via " + Debug.getCompressedStackTrace(4));
+		//}
+		boolean changed = super.setTableItem(newIndex, false, isVisible);
+		setShown(tableView.isRowVisible(this), changed);
+		return changed; 
+	}
 
-		return setTableItem(newIndex, false);
+	public boolean setTableItem(int newIndex) {
+		return setTableItem(newIndex,true);
 	}
 	
-	public void setForeground(Color c) {
-  	// Don't need to set when not visible
-  	if (!isVisible())
-  		return;
-  	
-  	super.setForeground(c);
+	// @see org.gudy.azureus2.ui.swt.components.BufferedTableRow#setForeground(int, int, int)
+	public void setForeground(int r, int g, int b) {
+		// Don't need to set when not visible
+		if (!isVisible()) {
+			return;
+		}
+
+		super.setForeground(r, g, b);
+	}
+
+	// @see org.gudy.azureus2.ui.swt.components.BufferedTableRow#setForeground(org.eclipse.swt.graphics.Color)
+	public void setForeground(final Color c) {
+		// Don't need to set when not visible
+		if (!isVisible())
+			return;
+
+		Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+				TableRowImpl.this.setForgroundInSWTThread(c);
+			}
+		});
+	}
+	
+	private void setForgroundInSWTThread(Color c) {
+		if (!isVisible())
+			return;
+
+		super.setForeground(c);
+	}
+
+
+	// @see org.gudy.azureus2.plugins.ui.tables.TableRow#setForeground(int[])
+	public void setForeground(int[] rgb) {
+		if (rgb == null || rgb.length < 3) {
+			setForeground((Color) null);
+			return;
+		}
+		setForeground(rgb[0], rgb[1], rgb[2]);
+	}
+
+	public void setForegroundToErrorColor() {
+		this.setForeground(Colors.colorError);
 	}
 
 	/* (non-Javadoc)
@@ -353,27 +494,28 @@ public class TableRowImpl
 
     Iterator iter = mTableCells.values().iterator();
     while (iter.hasNext()) {
-      TableCellCore cell = (TableCellCore)iter.next();
+    	TableCellSWT cell = (TableCellSWT)iter.next();
       if (cell != null)
         cell.invalidate(true);
     }
 	}
 	
-	public void repaint() {
-		super.invalidate();
-	}
-
 	public void setUpToDate(boolean upToDate) {
   	if (bDisposed)
   		return;
 
     Iterator iter = mTableCells.values().iterator();
     while (iter.hasNext()) {
-      TableCellCore cell = (TableCellCore)iter.next();
+    	TableCellSWT cell = (TableCellSWT)iter.next();
       if (cell != null)
         cell.setUpToDate(upToDate);
     }
 	}
+	
+	// @see com.aelitis.azureus.ui.common.table.TableRowCore#redraw()
+	public void redraw() {
+		refresh(true);
+	}
 
 	public String toString() {
 		String result = "TableRowImpl@" + Integer.toHexString(hashCode());
@@ -387,4 +529,56 @@ public class TableRowImpl
 		return result + ((index == null) ? "" : ("/#" + index)); 		
 	}
 
+	// @see com.aelitis.azureus.ui.common.table.TableRowCore#getView()
+	public TableView getView() {
+		return tableView;
+	}
+
+	/**
+	 * @param b
+	 *
+	 * @since 3.0.4.3
+	 */
+	public void setShown(boolean b, boolean force) {
+  	if (bDisposed)
+  		return;
+  	
+  	if (b == wasShown && !force) {
+  		return;
+  	}
+  	wasShown  = b;
+
+    Iterator iter = mTableCells.values().iterator();
+    while (iter.hasNext()) {
+    	TableCellSWT cell = (TableCellSWT)iter.next();
+      if (cell != null) {
+        cell.invokeVisibilityListeners(b
+						? TableCellVisibilityListener.VISIBILITY_SHOWN
+						: TableCellVisibilityListener.VISIBILITY_HIDDEN, true);
+      }
+    }
+	}
+	public boolean isMouseOver() {
+		return tableView.getTableRowWithCursor() == this;
+	}
+
+	public void setData(String id, Object data) {
+		if (dataList == null) {
+			dataList = new HashMap(1);
+		}
+		if (data == null) {
+			dataList.remove("id");
+		} else {
+			dataList.put(id, data);
+		}
+	}
+	
+	public Object getData(String id) {
+		return dataList == null ? null : dataList.get(id);
+	}
+
+	// @see com.aelitis.azureus.ui.common.table.TableRowCore#setDrawableHeight(int)
+	public boolean setDrawableHeight(int height) {
+		return setHeight(height);
+	}
 }
