@@ -24,29 +24,39 @@
 
 package org.gudy.azureus2.ui.swt.views.tableitems.files;
 
-import org.eclipse.swt.graphics.GC;
+import java.io.File;
+import java.io.IOException;
+
+import org.eclipse.swt.SWT;
 import org.eclipse.swt.graphics.Image;
-import org.eclipse.swt.widgets.Display;
+import org.eclipse.swt.widgets.MessageBox;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.disk.DiskManagerFileInfo;
-import org.gudy.azureus2.core3.util.Constants;
+import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.Debug;
+import org.gudy.azureus2.core3.util.FileUtil;
 import org.gudy.azureus2.ui.swt.ImageRepository;
+import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.debug.ObfusticateCellText;
-import org.gudy.azureus2.ui.swt.views.table.TableCellCore;
+import org.gudy.azureus2.ui.swt.views.table.TableCellSWT;
 import org.gudy.azureus2.ui.swt.views.table.utils.CoreTableColumn;
 
+import org.gudy.azureus2.plugins.ui.menus.MenuItem;
+import org.gudy.azureus2.plugins.ui.menus.MenuItemListener;
 import org.gudy.azureus2.plugins.ui.tables.*;
 
+import com.aelitis.azureus.core.AzureusCoreOperation;
+import com.aelitis.azureus.core.AzureusCoreOperationTask;
+
 /** Torrent name cell for My Torrents.
  *
  * @author Olivier
  * @author TuxPaper (2004/Apr/17: modified to TableCellAdapter)
  */
 public class NameItem extends CoreTableColumn implements
-		TableCellRefreshListener, ObfusticateCellText, TableCellDisposeListener
+		TableCellLightRefreshListener, ObfusticateCellText, TableCellDisposeListener
 {
 	private static boolean bShowIcon;
 
@@ -58,22 +68,50 @@ public class NameItem extends CoreTableColumn implements
 					}
 				});
 	}
+	
+	final TableContextMenuItem menuItem;
 
 	/** Default Constructor */
 	public NameItem() {
 		super("name", ALIGN_LEAD, POSITION_LAST, 300,
 				TableManager.TABLE_TORRENT_FILES);
+		setInplaceEdit(true);
 		setType(TableColumn.TYPE_TEXT);
-	}
+		menuItem = addContextMenuItem("FilesView.name.fastRename");
+		
+		menuItem.setStyle(MenuItem.STYLE_CHECK);
+		//menuItem.setText(MessageText.getString("FilesView.name.fastRename")); TODO make this work
+		menuItem.setData(Boolean.valueOf(isInplaceEdit()));
 
-	public void refresh(TableCell cell) {
+		menuItem.addMultiListener(new MenuItemListener() {
+			public void selected(MenuItem menu, Object target) {
+				menu.setData(Boolean.valueOf(!isInplaceEdit())); // XXX broken, should toggle checkmarks
+				setInplaceEdit(!isInplaceEdit());
+			}
+		});
+	}
+	
+	public void postConfigLoad() {
+		setInplaceEdit(getUserData("noInplaceEdit") == null);
+		menuItem.setData(Boolean.valueOf(isInplaceEdit()));
+	}
+	
+	public void preConfigSave() {
+		if(isInplaceEdit())
+			removeUserData("noInplaceEdit");
+		else
+			setUserData("noInplaceEdit", new Integer(1));
+	}
+	
+	public void refresh(TableCell cell, boolean sortOnlyRefresh)
+	{
 		DiskManagerFileInfo fileInfo = (DiskManagerFileInfo) cell.getDataSource();
 		String name = (fileInfo == null) ? "" : fileInfo.getFile(true).getName();
 		if (name == null)
 			name = "";
 		//setText returns true only if the text is updated
 		if (cell.setText(name) || !cell.isValid()) {
-			if (bShowIcon) {
+			if (bShowIcon && !sortOnlyRefresh) {
 				Image icon;
 				if (fileInfo == null) {
 					icon = null;
@@ -81,26 +119,20 @@ public class NameItem extends CoreTableColumn implements
 					// Don't ever dispose of PathIcon, it's cached and may be used elsewhere
 					icon = ImageRepository.getPathIcon(fileInfo.getFile(true).getPath());
 
-					if (Constants.isWindows) {
-						disposeCellIcon(cell);
-
-						// recomposite to avoid artifacts - transparency mask does not work
-						final Image dstImage = new Image(Display.getCurrent(),
-								icon.getBounds().width, icon.getBounds().height);
-						GC gc = new GC(dstImage);
-						gc.drawImage(icon, 0, 0);
-						gc.dispose();
-						icon = dstImage;
-					}
 				}
 
 				// cheat for core, since we really know it's a TabeCellImpl and want to use
 				// those special functions not available to Plugins
-				((TableCellCore) cell).setIcon(icon);
+				((TableCellSWT) cell).setIcon(icon);
 			}
 		}
 	}
 
+	public void refresh(TableCell cell)
+	{
+		refresh(cell, false);
+	}
+
 	public String getObfusticatedText(TableCell cell) {
 		DiskManagerFileInfo fileInfo = (DiskManagerFileInfo) cell.getDataSource();
 		String name = (fileInfo == null) ? ""
@@ -111,18 +143,60 @@ public class NameItem extends CoreTableColumn implements
 	}
 
 	public void dispose(TableCell cell) {
-		if (bShowIcon && Constants.isWindows) {
-			disposeCellIcon(cell);
-		}
 	}
 
 	private void disposeCellIcon(TableCell cell) {
-		final Image img = ((TableCellCore) cell).getIcon();
+		final Image img = ((TableCellSWT) cell).getIcon();
 		if (img != null) {
-			((TableCellCore) cell).setIcon(null);
+			((TableCellSWT) cell).setIcon(null);
 			if (!img.isDisposed()) {
 				img.dispose();
 			}
 		}
 	}
+	
+	public boolean inplaceValueSet(TableCell cell, String value, boolean finalEdit) {
+		if (value.equalsIgnoreCase(cell.getText()) || "".equals(value) || "".equals(cell.getText()))
+			return true;
+		final DiskManagerFileInfo fileInfo = (DiskManagerFileInfo) cell.getDataSource();
+		final File target;
+		
+		try
+		{
+			target = new File(fileInfo.getFile(true).getParentFile(), value).getCanonicalFile();
+		} catch (IOException e)
+		{
+			return false;
+		}
+			
+		if(!finalEdit)
+			return !target.exists();
+
+		
+		if(target.exists())
+			return false;
+		
+
+		// code stolen from FilesView
+		final boolean[] result = { false };
+		boolean paused = fileInfo.getDownloadManager().pause();
+		FileUtil.runAsTask(new AzureusCoreOperationTask()
+		{
+			public void run(AzureusCoreOperation operation) {
+				result[0] = fileInfo.setLink(target);
+			}
+		});
+		if(paused)
+			fileInfo.getDownloadManager().resume();
+		
+		if (!result[0])
+		{
+			MessageBox mb = new MessageBox(Utils.findAnyShell(), SWT.ICON_ERROR | SWT.OK);
+			mb.setText(MessageText.getString("FilesView.rename.failed.title"));
+			mb.setMessage(MessageText.getString("FilesView.rename.failed.text"));
+			mb.open();
+		}
+		
+		return true;
+	}
 }
