@@ -44,6 +44,7 @@ implements TableCellRefreshListener
 	public DownSpeedLimitItem(String sTableID) {
 	  super("maxdownspeed", ALIGN_TRAIL, POSITION_INVISIBLE, 35, sTableID);
 	  setRefreshInterval(INTERVAL_LIVE);
+    setMinWidthAuto(true);
 	}
 
 	public void refresh(TableCell cell) {
@@ -53,7 +54,7 @@ implements TableCellRefreshListener
 	    return;
 	  
 	  if(value == -1) {
-	    cell.setText(MessageText.getString("MyTorrents.items.UpSpeedLimit.disabled"));
+	    cell.setText(MessageText.getString("MyTorrents.items.DownSpeedLimit.disabled"));
 	  } else if(value  == 0) {
 	    cell.setText(Constants.INFINITY_STRING);
 	  } else {
