@@ -21,16 +21,20 @@
  * AELITIS, SAS au capital de 46,603.30 euros,
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  */
- 
+
 package org.gudy.azureus2.ui.swt.views.tableitems.mytorrents;
 
-import org.gudy.azureus2.core3.util.DisplayFormatters;
-import org.gudy.azureus2.core3.download.DownloadManager;
-import org.gudy.azureus2.plugins.ui.tables.*;
-import org.gudy.azureus2.ui.swt.views.table.TableCellCore;
-import org.gudy.azureus2.ui.swt.views.table.utils.CoreTableColumn;
-import org.gudy.azureus2.ui.swt.mainwindow.Colors;
+import org.eclipse.swt.graphics.Color;
 
+import org.gudy.azureus2.core3.download.DownloadManager;
+import org.gudy.azureus2.core3.util.DisplayFormatters;
+import org.gudy.azureus2.ui.swt.Utils;
+import org.gudy.azureus2.ui.swt.mainwindow.Colors;
+import org.gudy.azureus2.ui.swt.views.table.utils.CoreTableColumn;
+
+import org.gudy.azureus2.plugins.ui.tables.TableCell;
+import org.gudy.azureus2.plugins.ui.tables.TableCellRefreshListener;
+import org.gudy.azureus2.plugins.ui.tables.TableRow;
 
 /**
  *
@@ -38,27 +42,35 @@ import org.gudy.azureus2.ui.swt.mainwindow.Colors;
  * @author TuxPaper (2004/Apr/17: modified to TableCellAdapter)
  */
 public class StatusItem
-       extends CoreTableColumn 
-       implements TableCellRefreshListener
+	extends CoreTableColumn
+	implements TableCellRefreshListener
 {
-  /** Default Constructor */
-  public StatusItem(String sTableID) {
-    super("status", POSITION_LAST, 80, sTableID);
-    setRefreshInterval(INTERVAL_LIVE);
-  }
+	/** Default Constructor */
+	public StatusItem(String sTableID) {
+		super("status", POSITION_LAST, 80, sTableID);
+		setRefreshInterval(INTERVAL_LIVE);
+	}
 
-  public void refresh(TableCell cell) {
-    DownloadManager dm = (DownloadManager)cell.getDataSource();
-    
-    if( cell.setText( dm == null ? "" : DisplayFormatters.formatDownloadStatus(dm) ) || !cell.isValid() ) {
-      int state = dm.getState();
-      if (state == DownloadManager.STATE_SEEDING)
-        ((TableCellCore)cell).getTableRowCore().setForeground(Colors.blues[Colors.BLUES_MIDDARK]);
-      else if (state == DownloadManager.STATE_ERROR)
-        ((TableCellCore)cell).getTableRowCore().setForeground(Colors.colorError);
-      else
-        ((TableCellCore)cell).getTableRowCore().setForeground(null);
-    }
- 
-  }
+	public void refresh(TableCell cell) {
+		DownloadManager dm = (DownloadManager) cell.getDataSource();
+
+		if (cell.setText(dm == null ? ""
+				: DisplayFormatters.formatDownloadStatus(dm))
+				|| !cell.isValid()) {
+			TableRow row = cell.getTableRow();
+			if (row != null && dm != null) {
+				int state = dm.getState();
+				Color color = null;
+				if (state == DownloadManager.STATE_SEEDING) {
+					color = Colors.blues[Colors.BLUES_MIDDARK];
+				} else if (state == DownloadManager.STATE_ERROR) {
+					color = Colors.colorError;
+				} else {
+					color = null;
+				}
+				row.setForeground(Utils.colorToIntArray(color));
+			}
+		}
+
+	}
 }
