@@ -29,14 +29,16 @@ import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.plugins.peers.*;
 import org.gudy.azureus2.plugins.ui.tables.peers.*;
 import org.gudy.azureus2.plugins.ui.tables.*;
-import org.gudy.azureus2.ui.swt.views.table.impl.TableColumnImpl;
+
+import com.aelitis.azureus.ui.common.table.impl.TableColumnImpl;
 
 /** Cell/Factory to support old style Plugin columns
  *
  * @author Olivier
  * @author TuxPaper (2004/Apr/17: modified to TableCellAdapter)
+ * @deprecated
  */
-/** Link the old PluginMyTorrentsItemFactory to the new generic stuff */
+// Link the old PluginMyTorrentsItemFactory to the new generic stuff
 public class OldPeerPluginItem
        extends TableColumnImpl
        implements TableCellAddedListener
@@ -58,6 +60,9 @@ public class OldPeerPluginItem
     new Cell(cell);
   }
 
+  /**
+   * @deprecated
+   */
   private class Cell
           implements TableCellRefreshListener, PeerTableItem
   {
