@@ -21,40 +21,44 @@
  * AELITIS, SAS au capital de 46,603.30 euros,
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  */
- 
+
 package org.gudy.azureus2.ui.swt.views.tableitems.peers;
 
-import org.gudy.azureus2.ui.swt.mainwindow.Colors;
-
 import org.gudy.azureus2.core3.peer.PEPeer;
-import org.gudy.azureus2.ui.swt.views.table.TableCellCore;
-import org.gudy.azureus2.plugins.ui.tables.*;
+import org.gudy.azureus2.ui.swt.Utils;
+import org.gudy.azureus2.ui.swt.mainwindow.Colors;
 import org.gudy.azureus2.ui.swt.views.table.utils.CoreTableColumn;
 
+import org.gudy.azureus2.plugins.ui.tables.TableCell;
+import org.gudy.azureus2.plugins.ui.tables.TableCellRefreshListener;
+import org.gudy.azureus2.plugins.ui.tables.TableRow;
+
 /**
  *
  * @author Olivier
  * @author TuxPaper (2004/Apr/19: modified to TableCellAdapter)
  */
 public class SnubbedItem
-       extends CoreTableColumn 
-       implements TableCellRefreshListener
+	extends CoreTableColumn
+	implements TableCellRefreshListener
 {
-  /** Default Constructor */
-  public SnubbedItem() {
-    super("S", ALIGN_CENTER, POSITION_INVISIBLE, 20, TableManager.TABLE_TORRENT_PEERS);
-    setRefreshInterval(INTERVAL_LIVE);
-  }
+	public SnubbedItem(String table_id) {
+		super("S", ALIGN_CENTER, POSITION_INVISIBLE, 20, table_id);
+		setRefreshInterval(INTERVAL_LIVE);
+	}
 
-  public void refresh(TableCell cell) {
-    PEPeer peer = (PEPeer)cell.getDataSource();
-    boolean bSnubbed = (peer == null) ? false : peer.isSnubbed();
+	public void refresh(TableCell cell) {
+		PEPeer peer = (PEPeer) cell.getDataSource();
+		boolean bSnubbed = (peer == null) ? false : peer.isSnubbed();
 
-    if (!cell.setSortValue(bSnubbed ? 1 : 0) && cell.isValid())
-      return;
+		if (!cell.setSortValue(bSnubbed ? 1 : 0) && cell.isValid())
+			return;
 
-    cell.setText(bSnubbed ? "*" : "");
-    ((TableCellCore)cell).getTableRowCore().setForeground(bSnubbed ? Colors.grey 
-    		: null);
-  }
+		cell.setText(bSnubbed ? "*" : "");
+
+		TableRow row = cell.getTableRow();
+		if (row != null) {
+			row.setForeground(Utils.colorToIntArray(bSnubbed ? Colors.grey : null));
+		}
+	}
 }
