@@ -40,19 +40,28 @@ public class TimeToSendPieceItem
        implements TableCellRefreshListener
 {
   /** Default Constructor */
-  public TimeToSendPieceItem() {
-    super("timetosend", ALIGN_TRAIL, POSITION_INVISIBLE,
-          70, TableManager.TABLE_TORRENT_PEERS);
+  public TimeToSendPieceItem(String table_id) {
+    super("timetosend", ALIGN_TRAIL, POSITION_INVISIBLE, 70, table_id);
     setRefreshInterval(INTERVAL_LIVE);
   }
 
   public void refresh(TableCell cell) {
     PEPeer peer = (PEPeer)cell.getDataSource();
     long value = (peer == null) ? 0 : peer.getUploadHint();
+    
+    Comparable sortValue = cell.getSortValue();
+    long oldValue = 0;
+    if (sortValue instanceof Number) {
+    	oldValue = ((Number)sortValue).longValue();
+    }
 
     if (!cell.setSortValue(value) && cell.isValid())
       return;
 
-    cell.setText(TimeFormatter.format(value / 1000));
+    String text = TimeFormatter.format(value / 1000);
+    if (oldValue > 0) {
+    	text += ", " + TimeFormatter.format(oldValue / 1000);
+    }
+    cell.setText(text);
   }
 }
