@@ -26,10 +26,11 @@ import org.eclipse.swt.events.MenuListener;
 import org.eclipse.swt.widgets.*;
 import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.global.GlobalManager;
+import org.gudy.azureus2.core3.global.GlobalManagerStats;
 import org.gudy.azureus2.core3.internat.MessageText;
-import org.gudy.azureus2.core3.util.DisplayFormatters;
-import org.gudy.azureus2.ui.swt.ImageRepository;
-import org.gudy.azureus2.ui.swt.Messages;
+import org.gudy.azureus2.core3.util.*;
+import org.gudy.azureus2.ui.common.util.MenuItemManager;
+import org.gudy.azureus2.ui.swt.*;
 import org.gudy.azureus2.ui.swt.mainwindow.SWTThread;
 import org.gudy.azureus2.ui.swt.mainwindow.SelectableSpeedMenu;
 import org.gudy.azureus2.ui.swt.views.utils.ManagerUtils;
@@ -65,11 +66,71 @@ public class SystemTraySWT
 		tray = display.getSystemTray();
 		trayItem = new TrayItem(tray, SWT.NULL);
 
-		trayItem.setImage(ImageRepository.getImage("azureus"));
+		if(!Constants.isOSX) {
+			trayItem.setImage(ImageRepository.getImage("azureus"));
+		} else {
+			trayItem.setImage(ImageRepository.getImage("azureus_grey"));
+		}
 		trayItem.setVisible(true);
 
 		menu = new Menu(uiFunctions.getMainShell(), SWT.POP_UP);
+		menu.addMenuListener(new MenuListener() {
+			public void menuShown(MenuEvent _menu) {}
 
+			public void menuHidden(MenuEvent _menu) {
+				if(Constants.isOSX) {
+					trayItem.setImage(ImageRepository.getImage("azureus_grey"));
+				}
+			}
+		});
+		
+		MenuBuildUtils.addMaintenanceListenerForMenu(menu, new MenuBuildUtils.MenuBuilder() {
+			public void buildMenu(Menu menu) {
+				fillMenu(menu);
+			}
+		});
+
+		trayItem.addListener(SWT.DefaultSelection, new Listener() {
+			public void handleEvent(Event arg0) {
+				showMainWindow();
+			}
+		});
+
+		
+		trayItem.addListener(SWT.Selection, new Listener() {
+			long lastTime = 0;
+
+			public void handleEvent(Event arg0) {
+				// Bug in Windows (seems to have started around SWT 3.3 Release 
+				// Candidates) where double click isn't interpreted as DefaultSelection
+				// Since we "know" SWT.Selection is actually a mouse down, check
+				// if two mouse downs happen in a short timespan and fake a 
+				// DefaultSelection
+				if (Constants.isWindows) {
+					long now = SystemTime.getCurrentTime();
+					if (now - lastTime < 200) {
+						showMainWindow();
+					} else {
+						lastTime = now;
+					}
+				} else if (Constants.isOSX) {
+					trayItem.setImage(ImageRepository.getImage("azureus_white"));
+					menu.setVisible(true);
+				}
+			}
+		});
+		
+		trayItem.addListener(SWT.MenuDetect, new Listener() {
+			public void handleEvent(Event arg0) {
+				menu.setVisible(true);
+			}
+		});
+
+		
+	}
+	
+	public void fillMenu(final Menu menu) {
+		
 		final MenuItem itemShow = new MenuItem(menu, SWT.NULL);
 		Messages.setLanguageText(itemShow, "SystemTray.menu.show");
 
@@ -78,9 +139,20 @@ public class SystemTraySWT
 		final MenuItem itemCloseAll = new MenuItem(menu, SWT.NULL);
 		Messages.setLanguageText(itemCloseAll,
 				"SystemTray.menu.closealldownloadbars");
+		
+		final MenuItem itemShowGlobalTransferBar = new MenuItem(menu, SWT.CHECK);
+		Messages.setLanguageText(itemShowGlobalTransferBar,
+			"SystemTray.menu.open_global_transfer_bar");
 
 		new MenuItem(menu, SWT.SEPARATOR);
 
+		org.gudy.azureus2.plugins.ui.menus.MenuItem[] menu_items;
+		menu_items = MenuItemManager.getInstance().getAllAsArray("systray");
+		if (menu_items.length > 0) {
+			MenuBuildUtils.addPluginMenuItems(uiFunctions.getMainShell(), menu_items, menu, true, true, MenuBuildUtils.BASIC_MENU_ITEM_CONTROLLER);
+			new MenuItem(menu, SWT.SEPARATOR);
+		}
+		
 		createUploadLimitMenu(menu);
 		createDownloadLimitMenu(menu);
 
@@ -133,42 +205,33 @@ public class SystemTraySWT
 			}
 		});
 
-		menu.addMenuListener(new MenuListener() {
-			public void menuShown(MenuEvent _menu) {
-				GlobalManager gm = AzureusCoreFactory.getSingleton().getGlobalManager();
-				itemPause.setEnabled(gm.canPauseDownloads());
-
-				itemResume.setEnabled(gm.canResumeDownloads());
-			}
-
-			public void menuHidden(MenuEvent _menu) {
-
-			}
-		});
+		GlobalManager gm = AzureusCoreFactory.getSingleton().getGlobalManager();
+		itemPause.setEnabled(gm.canPauseDownloads());
+		itemResume.setEnabled(gm.canResumeDownloads());
 
 		itemCloseAll.addListener(SWT.Selection, new Listener() {
 			public void handleEvent(Event arg0) {
 				uiFunctions.closeDownloadBars();
 			}
 		});
-
+		
+		itemShowGlobalTransferBar.setSelection(uiFunctions.isGlobalTransferBarShown());
+		itemShowGlobalTransferBar.addListener(SWT.Selection, new Listener() {
+			public void handleEvent(Event arg0) {
+				if (uiFunctions.isGlobalTransferBarShown()) {
+					uiFunctions.closeGlobalTransferBar();
+				}
+				else {
+					uiFunctions.showGlobalTransferBar();
+				}
+			}
+		});
+		
 		itemExit.addListener(SWT.Selection, new Listener() {
 			public void handleEvent(Event arg0) {
 				uiFunctions.requestShutdown();
 			}
 		});
-
-		trayItem.addListener(SWT.DefaultSelection, new Listener() {
-			public void handleEvent(Event arg0) {
-				showMainWindow();
-			}
-		});
-
-		trayItem.addListener(SWT.MenuDetect, new Listener() {
-			public void handleEvent(Event arg0) {
-				menu.setVisible(true);
-			}
-		});
 	}
 
 	/**
@@ -216,9 +279,13 @@ public class SystemTraySWT
 	}
 
 	public void dispose() {
-		if (trayItem != null && !trayItem.isDisposed()) {
-			trayItem.dispose();
-		}
+		Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+				if (trayItem != null && !trayItem.isDisposed()) {
+					trayItem.dispose();
+				}
+			}
+		});
 	}
 
 	public void update() {
@@ -254,17 +321,21 @@ public class SystemTraySWT
 		}
 
 		GlobalManager gm = AzureusCoreFactory.getSingleton().getGlobalManager();
+		GlobalManagerStats stats = gm.getStats();
 
 		toolTip.append(seeding_text).append(downloading_text).append("\n");
 		toolTip.append(MessageText.getString("ConfigView.download.abbreviated")).append(
 				" ");
-		toolTip.append(DisplayFormatters.formatByteCountToKiBEtcPerSec(gm.getStats().getDataReceiveRate()
-				+ gm.getStats().getProtocolReceiveRate()));
+
+		toolTip.append(DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(
+				stats.getDataReceiveRate(), stats.getProtocolReceiveRate()));
+		
 		toolTip.append(", ").append(
 				MessageText.getString("ConfigView.upload.abbreviated")).append(" ");
-		toolTip.append(DisplayFormatters.formatByteCountToKiBEtcPerSec(gm.getStats().getDataSendRate()
-				+ gm.getStats().getProtocolSendRate()));
-
+		toolTip.append(DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(
+				stats.getDataSendRate(), stats.getProtocolSendRate()));
+		
+		
 		trayItem.setToolTipText(toolTip.toString());
 
 		//Why should we refresh the image? it never changes ...
@@ -274,7 +345,7 @@ public class SystemTraySWT
 	}
 
 	private void showMainWindow() {
-		uiFunctions.bringToFront();
+		uiFunctions.bringToFront(false);
 	}
 
 	public void updateLanguage() {
